@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,7 +55,8 @@
 #include "hoverpoints.h"
 #include "composition.h"
 
-class QDesignerFormEditorInterface;
+QT_FORWARD_DECLARE_CLASS(QDesignerFormEditorInterface)
+
 class PathDeformRendererEx : public PathDeformRenderer
 {
     Q_OBJECT
@@ -200,21 +221,25 @@ class ArthurPlugins : public QObject, public QDesignerCustomWidgetCollectionInte
 {
     Q_OBJECT
     Q_INTERFACES(QDesignerCustomWidgetCollectionInterface)
-        public:
-    QList<QDesignerCustomWidgetInterface*> customWidgets() const
-    {
-        QList<QDesignerCustomWidgetInterface *> plugins;
-        plugins
-            << new DeformPlugin
-            << new XFormPlugin
-            << new GradientEditorPlugin
-            << new GradientRendererPlugin
-            << new StrokeRenderPlugin
-            << new CompositionModePlugin;
-        return plugins;
-    }
+
+public:
+    ArthurPlugins(QObject *parent = 0);
+    QList<QDesignerCustomWidgetInterface*> customWidgets() const { return m_plugins; }
+
+private:
+    QList<QDesignerCustomWidgetInterface *> m_plugins;
 };
 
+ArthurPlugins::ArthurPlugins(QObject *parent) :
+    QObject(parent)
+{
+    m_plugins << new DeformPlugin(this)
+              << new XFormPlugin(this)
+              << new GradientEditorPlugin(this)
+              << new GradientRendererPlugin(this)
+              << new StrokeRenderPlugin(this)
+              << new CompositionModePlugin(this);
+}
 
 #include "plugin.moc"
 
