@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -186,7 +206,7 @@ inline static bool x_less_than(const QPointF &p1, const QPointF &p2)
 
 void GradientEditor::pointsUpdated()
 {
-    double w = m_alpha_shade->width();
+    qreal w = m_alpha_shade->width();
 
     QGradientStops stops;
 
@@ -200,7 +220,7 @@ void GradientEditor::pointsUpdated()
     qSort(points.begin(), points.end(), x_less_than);
 
     for (int i=0; i<points.size(); ++i) {
-        double x = int(points.at(i).x());
+        qreal x = int(points.at(i).x());
         if (i < points.size() - 1 && x == points.at(i+1).x())
             continue;
         QColor color((0x00ff0000 & m_red_shade->colorAt(int(x))) >> 16,
@@ -232,13 +252,13 @@ void GradientEditor::setGradientStops(const QGradientStops &stops)
 {
     QPolygonF pts_red, pts_green, pts_blue, pts_alpha;
 
-    double h_red = m_red_shade->height();
-    double h_green = m_green_shade->height();
-    double h_blue = m_blue_shade->height();
-    double h_alpha = m_alpha_shade->height();
+    qreal h_red = m_red_shade->height();
+    qreal h_green = m_green_shade->height();
+    qreal h_blue = m_blue_shade->height();
+    qreal h_alpha = m_alpha_shade->height();
 
     for (int i=0; i<stops.size(); ++i) {
-        double pos = stops.at(i).first;
+        qreal pos = stops.at(i).first;
         QRgb color = stops.at(i).second.rgba();
         pts_red << QPointF(pos * m_red_shade->width(), h_red - qRed(color) * h_red / 255);
         pts_green << QPointF(pos * m_green_shade->width(), h_green - qGreen(color) * h_green / 255);
@@ -253,7 +273,6 @@ void GradientEditor::setGradientStops(const QGradientStops &stops)
 
 }
 
-
 GradientWidget::GradientWidget(QWidget *parent)
     : QWidget(parent)
 {
@@ -293,7 +312,14 @@ GradientWidget::GradientWidget(QWidget *parent)
 
     QPushButton *showSourceButton = new QPushButton(mainGroup);
     showSourceButton->setText("Show Source");
-
+#ifdef QT_OPENGL_SUPPORT
+    QPushButton *enableOpenGLButton = new QPushButton(mainGroup);
+    enableOpenGLButton->setText("Use OpenGL");
+    enableOpenGLButton->setCheckable(true);
+    enableOpenGLButton->setChecked(m_renderer->usesOpenGL());
+    if (!QGLFormat::hasOpenGL())
+        enableOpenGLButton->hide();
+#endif
     QPushButton *whatsThisButton = new QPushButton(mainGroup);
     whatsThisButton->setText("What's This?");
     whatsThisButton->setCheckable(true);
@@ -311,6 +337,9 @@ GradientWidget::GradientWidget(QWidget *parent)
     mainGroupLayout->addWidget(defaultsGroup);
     mainGroupLayout->addStretch(1);
     mainGroupLayout->addWidget(showSourceButton);
+#ifdef QT_OPENGL_SUPPORT
+    mainGroupLayout->addWidget(enableOpenGLButton);
+#endif
     mainGroupLayout->addWidget(whatsThisButton);
 
     QVBoxLayout *editorGroupLayout = new QVBoxLayout(editorGroup);
@@ -349,7 +378,9 @@ GradientWidget::GradientWidget(QWidget *parent)
     connect(default4Button, SIGNAL(clicked()), this, SLOT(setDefault4()));
 
     connect(showSourceButton, SIGNAL(clicked()), m_renderer, SLOT(showSource()));
-
+#ifdef QT_OPENGL_SUPPORT
+    connect(enableOpenGLButton, SIGNAL(clicked(bool)), m_renderer, SLOT(enableOpenGL(bool)));
+#endif
     connect(whatsThisButton, SIGNAL(clicked(bool)), m_renderer, SLOT(setDescriptionEnabled(bool)));
     connect(whatsThisButton, SIGNAL(clicked(bool)),
             m_renderer->hoverPoints(), SLOT(setDisabled(bool)));
@@ -468,11 +499,13 @@ void GradientRenderer::paint(QPainter *p)
         g = QLinearGradient(pts.at(0), pts.at(1));
 
     } else if (m_gradientType == Qt::RadialGradientPattern) {
-        g = QRadialGradient(pts.at(0), qMin(width(), height()) / 3.0, pts.at(1));
-
+        QLineF line(pts.at(0), pts.at(1));
+        if (line.length() > 132)
+            line.setLength(132);
+        g = QRadialGradient(line.p1(), qMin(width(), height()) / 3.0, line.p2());
     } else {
         QLineF l(pts.at(0), pts.at(1));
-        double angle = l.angle(QLineF(0, 0, 1, 0));
+        qreal angle = l.angle(QLineF(0, 0, 1, 0));
         if (l.dy() > 0)
             angle = 360 - angle;
         g = QConicalGradient(pts.at(0), angle);
