@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +46,8 @@
 #include <QApplication>
 #include <QPainterPath>
 #include <QPainter>
+#include <QMap>
+#include <qdebug.h>
 
 void render_qt_text(QPainter *painter, int w, int h, const QColor &color) {
     QPainterPath path;
@@ -96,10 +118,49 @@ void render_qt_text(QPainter *painter, int w, int h, const QColor &color) {
     painter->drawPath(path);
 }
 
+void usage()
+{
+    qWarning() << "Usage: mainwindow [-SizeHint<color> <width>x<height>] ...";
+    exit(1);
+}
+
+QMap<QString, QSize> parseCustomSizeHints(int argc, char **argv)
+{
+    QMap<QString, QSize> result;
+
+    for (int i = 1; i < argc; ++i) {
+        QString arg = QString::fromLocal8Bit(argv[i]);
+
+        if (arg.startsWith(QLatin1String("-SizeHint"))) {
+            QString name = arg.mid(9);
+            if (name.isEmpty())
+                usage();
+            if (++i == argc)
+                usage();
+            QString sizeStr = QString::fromLocal8Bit(argv[i]);
+            int idx = sizeStr.indexOf(QLatin1Char('x'));
+            if (idx == -1)
+                usage();
+            bool ok;
+            int w = sizeStr.left(idx).toInt(&ok);
+            if (!ok)
+                usage();
+            int h = sizeStr.mid(idx + 1).toInt(&ok);
+            if (!ok)
+                usage();
+            result[name] = QSize(w, h);
+        }
+    }
+
+    return result;
+}
+
 int main(int argc, char **argv)
 {
     QApplication app(argc, argv);
-    MainWindow mainWin;
+    QMap<QString, QSize> customSizeHints = parseCustomSizeHints(argc, argv);
+    MainWindow mainWin(customSizeHints);
+    mainWin.resize(800, 600);
     mainWin.show();
     return app.exec();
 }
