@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -72,9 +72,9 @@ void MenuContentItem::prepare()
 
 void MenuContentItem::animationStopped(int id)
 {
-    if (this->name == QLatin1String("Qt Examples and Demos"))
+    if (this->name == Colors::rootMenuName)
         return; // Optimization hack.
-        
+
     if (id == DemoItemAnimation::ANIM_OUT){
         // Free up some memory:
         delete this->heading;
