@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -76,8 +76,8 @@ ArthurStyle::ArthurStyle()
 
 void ArthurStyle::drawHoverRect(QPainter *painter, const QRect &r) const
 {
-    double h = r.height();
-    double h2 = r.height() / 2.0;
+    qreal h = r.height();
+    qreal h2 = r.height() / qreal(2);
     QPainterPath path;
     path.addRect(r.x() + h2, r.y() + 0, r.width() - h2 * 2, r.height());
     path.addEllipse(r.x(), r.y(), h, h);
