@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,7 @@
 
 #include "window.h"
 
+//! [0]
 Window::Window()
 {
     createIconGroupBox();
@@ -75,15 +76,19 @@ Window::Window()
     setWindowTitle(tr("Systray"));
     resize(400, 300);
 }
+//! [0]
 
+//! [1]
 void Window::setVisible(bool visible)
 {
     minimizeAction->setEnabled(visible);
     maximizeAction->setEnabled(!isMaximized());
     restoreAction->setEnabled(isMaximized() || !visible);
-    QWidget::setVisible(visible);
+    QDialog::setVisible(visible);
 }
+//! [1]
 
+//! [2]
 void Window::closeEvent(QCloseEvent *event)
 {
     if (trayIcon->isVisible()) {
@@ -96,7 +101,9 @@ void Window::closeEvent(QCloseEvent *event)
         event->ignore();
     }
 }
+//! [2]
 
+//! [3]
 void Window::setIcon(int index)
 {
     QIcon icon = iconComboBox->itemIcon(index);
@@ -105,7 +112,9 @@ void Window::setIcon(int index)
 
     trayIcon->setToolTip(iconComboBox->itemText(index));
 }
+//! [3]
 
+//! [4]
 void Window::iconActivated(QSystemTrayIcon::ActivationReason reason)
 {
     switch (reason) {
@@ -121,7 +130,9 @@ void Window::iconActivated(QSystemTrayIcon::ActivationReason reason)
         ;
     }
 }
+//! [4]
 
+//! [5]
 void Window::showMessage()
 {
     QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::MessageIcon(
@@ -129,13 +140,16 @@ void Window::showMessage()
     trayIcon->showMessage(titleEdit->text(), bodyEdit->toPlainText(), icon,
                           durationSpinBox->value() * 1000);
 }
+//! [5]
 
+//! [6]
 void Window::messageClicked()
 {
     QMessageBox::information(0, tr("Systray"),
                              tr("Sorry, I already gave what help I could.\n"
                                 "Maybe you should try asking a human?"));
 }
+//! [6]
 
 void Window::createIconGroupBox()
 {
