@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,8 +47,9 @@
 
 #define MESSAGE \
     Dialog::tr("<p>Message boxes have a caption, a text, " \
-               "and up to three buttons, each with standard or custom texts." \
-               "<p>Click a button or press Esc.")
+               "and any number of buttons, each with standard or custom texts." \
+               "<p>Click a button to close the message box. Pressing the Esc button " \
+               "will activate the detected escape button (if any).")
 
 Dialog::Dialog(QWidget *parent)
     : QDialog(parent)
@@ -128,6 +149,14 @@ Dialog::Dialog(QWidget *parent)
     connect(warningButton, SIGNAL(clicked()), this, SLOT(warningMessage()));
     connect(errorButton, SIGNAL(clicked()), this, SLOT(errorMessage()));
 
+    native = new QCheckBox(this);
+    native->setText("Use native file dialog.");
+    native->setChecked(true);
+#ifndef Q_WS_WIN
+#ifndef Q_OS_MAC
+    native->hide();
+#endif
+#endif
     QGridLayout *layout = new QGridLayout;
     layout->setColumnStretch(1, 1);
     layout->setColumnMinimumWidth(1, 250);
@@ -161,6 +190,7 @@ Dialog::Dialog(QWidget *parent)
     layout->addWidget(warningLabel, 13, 1);
     layout->addWidget(errorButton, 14, 0);
     layout->addWidget(errorLabel, 14, 1);
+    layout->addWidget(native, 15, 0);
     setLayout(layout);
 
     setWindowTitle(tr("Standard Dialogs"));
@@ -168,24 +198,29 @@ Dialog::Dialog(QWidget *parent)
 
 void Dialog::setInteger()
 {
+//! [0]
     bool ok;
     int i = QInputDialog::getInteger(this, tr("QInputDialog::getInteger()"),
                                      tr("Percentage:"), 25, 0, 100, 1, &ok);
     if (ok)
         integerLabel->setText(tr("%1%").arg(i));
+//! [0]
 }
 
 void Dialog::setDouble()
 {
+//! [1]
     bool ok;
     double d = QInputDialog::getDouble(this, tr("QInputDialog::getDouble()"),
                                        tr("Amount:"), 37.56, -10000, 10000, 2, &ok);
     if (ok)
         doubleLabel->setText(QString("$%1").arg(d));
+//! [1]
 }
 
 void Dialog::setItem()
 {
+//! [2]
     QStringList items;
     items << tr("Spring") << tr("Summer") << tr("Fall") << tr("Winter");
 
@@ -194,16 +229,19 @@ void Dialog::setItem()
                                          tr("Season:"), items, 0, false, &ok);
     if (ok && !item.isEmpty())
         itemLabel->setText(item);
+//! [2]
 }
 
 void Dialog::setText()
 {
+//! [3]
     bool ok;
     QString text = QInputDialog::getText(this, tr("QInputDialog::getText()"),
                                          tr("User name:"), QLineEdit::Normal,
                                          QDir::home().dirName(), &ok);
     if (ok && !text.isEmpty())
         textLabel->setText(text);
+//! [3]
 }
 
 void Dialog::setColor()
@@ -228,31 +266,45 @@ void Dialog::setFont()
 
 void Dialog::setExistingDirectory()
 {
+    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
+    if (!native->isChecked())
+        options |= QFileDialog::DontUseNativeDialog;
     QString directory = QFileDialog::getExistingDirectory(this,
                                 tr("QFileDialog::getExistingDirectory()"),
                                 directoryLabel->text(),
-                                QFileDialog::DontResolveSymlinks
-                                | QFileDialog::ShowDirsOnly);
+                                options);
     if (!directory.isEmpty())
         directoryLabel->setText(directory);
 }
 
 void Dialog::setOpenFileName()
 {
+    QFileDialog::Options options;
+    if (!native->isChecked())
+        options |= QFileDialog::DontUseNativeDialog;
+    QString selectedFilter;
     QString fileName = QFileDialog::getOpenFileName(this,
                                 tr("QFileDialog::getOpenFileName()"),
                                 openFileNameLabel->text(),
-                                tr("All Files (*);;Text Files (*.txt)"));
+                                tr("All Files (*);;Text Files (*.txt)"),
+                                &selectedFilter,
+                                options);
     if (!fileName.isEmpty())
         openFileNameLabel->setText(fileName);
 }
 
 void Dialog::setOpenFileNames()
 {
+    QFileDialog::Options options;
+    if (!native->isChecked())
+        options |= QFileDialog::DontUseNativeDialog;
+    QString selectedFilter;
     QStringList files = QFileDialog::getOpenFileNames(
                                 this, tr("QFileDialog::getOpenFileNames()"),
                                 openFilesPath,
-                                tr("All Files (*);;Text Files (*.txt)"));
+                                tr("All Files (*);;Text Files (*.txt)"),
+                                &selectedFilter,
+                                options);
     if (files.count()) {
         openFilesPath = files[0];
         openFileNamesLabel->setText(QString("[%1]").arg(files.join(", ")));
@@ -261,10 +313,16 @@ void Dialog::setOpenFileNames()
 
 void Dialog::setSaveFileName()
 {
+    QFileDialog::Options options;
+    if (!native->isChecked())
+        options |= QFileDialog::DontUseNativeDialog;
+    QString selectedFilter;
     QString fileName = QFileDialog::getSaveFileName(this,
                                 tr("QFileDialog::getSaveFileName()"),
                                 saveFileNameLabel->text(),
-                                tr("All Files (*);;Text Files (*.txt)"));
+                                tr("All Files (*);;Text Files (*.txt)"),
+                                &selectedFilter,
+                                options);
     if (!fileName.isEmpty())
         saveFileNameLabel->setText(fileName);
 }
