@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +46,7 @@
 #include "droparea.h"
 #include "dropsitewindow.h"
 
+//! [constructor part1]
 DropSiteWindow::DropSiteWindow()
 {
     abstractLabel = new QLabel(tr("This example accepts drags from other "
@@ -33,11 +54,15 @@ DropSiteWindow::DropSiteWindow()
                                   "provided by the drag object."));
     abstractLabel->setWordWrap(true);
     abstractLabel->adjustSize();
+//! [constructor part1]
 
+//! [constructor part2]
     dropArea = new DropArea;
     connect(dropArea, SIGNAL(changed(const QMimeData *)),
             this, SLOT(updateFormatsTable(const QMimeData *)));
+//! [constructor part2]
 
+//! [constructor part3]
     QStringList labels;
     labels << tr("Format") << tr("Content");
 
@@ -46,7 +71,9 @@ DropSiteWindow::DropSiteWindow()
     formatsTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
     formatsTable->setHorizontalHeaderLabels(labels);
     formatsTable->horizontalHeader()->setStretchLastSection(true);
+//! [constructor part3]
 
+//! [constructor part4]
     clearButton = new QPushButton(tr("Clear"));
     quitButton = new QPushButton(tr("Quit"));
 
@@ -56,7 +83,9 @@ DropSiteWindow::DropSiteWindow()
 
     connect(quitButton, SIGNAL(pressed()), this, SLOT(close()));
     connect(clearButton, SIGNAL(pressed()), dropArea, SLOT(clear()));
+//! [constructor part4]
 
+//! [constructor part5]
     QVBoxLayout *mainLayout = new QVBoxLayout;
     mainLayout->addWidget(abstractLabel);
     mainLayout->addWidget(dropArea);
@@ -67,23 +96,35 @@ DropSiteWindow::DropSiteWindow()
     setWindowTitle(tr("Drop Site"));
     setMinimumSize(350, 500);
 }
+//! [constructor part5]
 
+//! [updateFormatsTable() part1]
 void DropSiteWindow::updateFormatsTable(const QMimeData *mimeData)
 {
     formatsTable->setRowCount(0);
     if (!mimeData)
         return;
+//! [updateFormatsTable() part1]
 
+//! [updateFormatsTable() part2]        
     foreach (QString format, mimeData->formats()) {
         QTableWidgetItem *formatItem = new QTableWidgetItem(format);
         formatItem->setFlags(Qt::ItemIsEnabled);
         formatItem->setTextAlignment(Qt::AlignTop | Qt::AlignLeft);
+//! [updateFormatsTable() part2]
 
+//! [updateFormatsTable() part3]
         QString text;
         if (format == "text/plain") {
             text = mimeData->text().simplified();
         } else if (format == "text/html") {
             text = mimeData->html().simplified();
+        } else if (format == "text/uri-list") {
+            QList<QUrl> urlList = mimeData->urls();
+            for (int i = 0; i < urlList.size() && i < 32; ++i) {
+                QString url = urlList.at(i).path();
+                text.append(url + " ");
+            }
         } else {
             QByteArray data = mimeData->data(format);
             for (int i = 0; i < data.size() && i < 32; ++i) {
@@ -93,12 +134,15 @@ void DropSiteWindow::updateFormatsTable(const QMimeData *mimeData)
                 text.append(hex + " ");
             }
         }
+//! [updateFormatsTable() part3]   
 
+//! [updateFormatsTable() part4]
         int row = formatsTable->rowCount();
         formatsTable->insertRow(row);
         formatsTable->setItem(row, 0, new QTableWidgetItem(format));
         formatsTable->setItem(row, 1, new QTableWidgetItem(text));
     }
-
+    
     formatsTable->resizeColumnToContents(0);
 }
+//! [updateFormatsTable() part4] 
