@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,26 +55,33 @@ FtpWindow::FtpWindow(QWidget *parent)
 
     statusLabel = new QLabel(tr("Please enter the name of an FTP server."));
 
-    fileList = new QListWidget;
+    fileList = new QTreeWidget;
     fileList->setEnabled(false);
+    fileList->setRootIsDecorated(false);
+    fileList->setHeaderLabels(QStringList() << tr("Name") << tr("Size") << tr("Owner") << tr("Group") << tr("Time"));
+    fileList->header()->setStretchLastSection(false);
 
     connectButton = new QPushButton(tr("Connect"));
     connectButton->setDefault(true);
     
-    downloadButton = new QPushButton(tr("Download"));
-    downloadButton->setEnabled(false);
-
     cdToParentButton = new QPushButton;
     cdToParentButton->setIcon(QPixmap(":/images/cdtoparent.png"));
     cdToParentButton->setEnabled(false);
 
+    downloadButton = new QPushButton(tr("Download"));
+    downloadButton->setEnabled(false);
+
     quitButton = new QPushButton(tr("Quit"));
 
+    buttonBox = new QDialogButtonBox;
+    buttonBox->addButton(downloadButton, QDialogButtonBox::ActionRole);
+    buttonBox->addButton(quitButton, QDialogButtonBox::RejectRole);
+
     progressDialog = new QProgressDialog(this);
 
-    connect(fileList, SIGNAL(itemDoubleClicked(QListWidgetItem *)),
-            this, SLOT(processItem(QListWidgetItem *)));
-    connect(fileList, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)),
+    connect(fileList, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
+            this, SLOT(processItem(QTreeWidgetItem *, int)));
+    connect(fileList, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)),
             this, SLOT(enableDownloadButton()));
     connect(progressDialog, SIGNAL(canceled()), this, SLOT(cancelDownload()));
     connect(connectButton, SIGNAL(clicked()), this, SLOT(connectOrDisconnect()));
@@ -68,21 +95,21 @@ FtpWindow::FtpWindow(QWidget *parent)
     topLayout->addWidget(cdToParentButton);
     topLayout->addWidget(connectButton);
     
-    QHBoxLayout *buttonLayout = new QHBoxLayout;
-    buttonLayout->addStretch(1);
-    buttonLayout->addWidget(downloadButton);
-    buttonLayout->addWidget(quitButton);
-
     QVBoxLayout *mainLayout = new QVBoxLayout;
     mainLayout->addLayout(topLayout);
     mainLayout->addWidget(fileList);
     mainLayout->addWidget(statusLabel);
-    mainLayout->addLayout(buttonLayout);
+    mainLayout->addWidget(buttonBox);
     setLayout(mainLayout);
 
     setWindowTitle(tr("FTP"));
 }
 
+QSize FtpWindow::sizeHint() const
+{
+    return QSize(500, 300);
+}
+
 void FtpWindow::connectOrDisconnect()
 {
     if (ftp) {
@@ -90,12 +117,19 @@ void FtpWindow::connectOrDisconnect()
         ftp->deleteLater();
         ftp = 0;
         fileList->setEnabled(false);
+        cdToParentButton->setEnabled(false);
         downloadButton->setEnabled(false);
+        connectButton->setEnabled(true);
         connectButton->setText(tr("Connect"));
+#ifndef QT_NO_CURSOR
+        setCursor(Qt::ArrowCursor);
+#endif
         return;
     }
-    
-    QApplication::setOverrideCursor(Qt::WaitCursor);
+
+#ifndef QT_NO_CURSOR
+    setCursor(Qt::WaitCursor);
+#endif
     
     ftp = new QFtp(this);
     connect(ftp, SIGNAL(commandFinished(int, bool)),
@@ -105,11 +139,27 @@ void FtpWindow::connectOrDisconnect()
     connect(ftp, SIGNAL(dataTransferProgress(qint64, qint64)),
             this, SLOT(updateDataTransferProgress(qint64, qint64)));
 
-    ftp->connectToHost(ftpServerLineEdit->text());
-    ftp->login();
-    ftp->list();
+    fileList->clear();
+    currentPath.clear();
+    isDirectory.clear();
+
+    QUrl url(ftpServerLineEdit->text());
+    if (!url.isValid() || url.scheme().toLower() != QLatin1String("ftp")) {
+        ftp->connectToHost(ftpServerLineEdit->text(), 21);
+        ftp->login();
+    } else {
+        ftp->connectToHost(url.host(), url.port(21));
+
+        if (!url.userName().isEmpty())
+            ftp->login(QUrl::fromPercentEncoding(url.userName().toLatin1()), url.password());
+        else
+            ftp->login();
+        if (!url.path().isEmpty())
+            ftp->cd(url.path());
+    }
 
     fileList->setEnabled(true);
+    connectButton->setEnabled(false);
     connectButton->setText(tr("Disconnect"));
     statusLabel->setText(tr("Connecting to FTP server %1...")
                          .arg(ftpServerLineEdit->text()));
@@ -117,7 +167,7 @@ void FtpWindow::connectOrDisconnect()
 
 void FtpWindow::downloadFile()
 {
-    QString fileName = fileList->currentItem()->text();
+    QString fileName = fileList->currentItem()->text(0);
 
     if (QFile::exists(fileName)) {
         QMessageBox::information(this, tr("FTP"),
@@ -136,11 +186,11 @@ void FtpWindow::downloadFile()
         return;
     }
 
-    ftp->get(fileList->currentItem()->text(), file);
+    ftp->get(fileList->currentItem()->text(0), file);
 
     progressDialog->setLabelText(tr("Downloading %1...").arg(fileName));
-    progressDialog->show();
     downloadButton->setEnabled(false);
+    progressDialog->exec();
 }
 
 void FtpWindow::cancelDownload()
@@ -150,14 +200,18 @@ void FtpWindow::cancelDownload()
 
 void FtpWindow::ftpCommandFinished(int, bool error)
 {
+#ifndef QT_NO_CURSOR
+    setCursor(Qt::ArrowCursor);
+#endif
+
     if (ftp->currentCommand() == QFtp::ConnectToHost) {
         if (error) {
-            QApplication::restoreOverrideCursor();
             QMessageBox::information(this, tr("FTP"),
                                      tr("Unable to connect to the FTP server "
                                         "at %1. Please check that the host "
                                         "name is correct.")
                                      .arg(ftpServerLineEdit->text()));
+            connectOrDisconnect();
             return;
         }
 
@@ -165,11 +219,14 @@ void FtpWindow::ftpCommandFinished(int, bool error)
                              .arg(ftpServerLineEdit->text()));
         fileList->setFocus();
         downloadButton->setDefault(true);
+        connectButton->setEnabled(true);
         return;
     }
 
+    if (ftp->currentCommand() == QFtp::Login)
+        ftp->list();
+
     if (ftp->currentCommand() == QFtp::Get) {
-        QApplication::restoreOverrideCursor();
         if (error) {
             statusLabel->setText(tr("Canceled download of %1.")
                                  .arg(file->fileName()));
@@ -182,10 +239,10 @@ void FtpWindow::ftpCommandFinished(int, bool error)
         }
         delete file;
         enableDownloadButton();
+        progressDialog->hide();
     } else if (ftp->currentCommand() == QFtp::List) {
-        QApplication::restoreOverrideCursor();
         if (isDirectory.isEmpty()) {
-            fileList->addItem(tr("<empty>"));
+            fileList->addTopLevelItem(new QTreeWidgetItem(QStringList() << tr("<empty>")));
             fileList->setEnabled(false);
         }
     }
@@ -193,22 +250,27 @@ void FtpWindow::ftpCommandFinished(int, bool error)
 
 void FtpWindow::addToList(const QUrlInfo &urlInfo)
 {
-    QListWidgetItem *item = new QListWidgetItem;
-    item->setText(urlInfo.name());
+    QTreeWidgetItem *item = new QTreeWidgetItem;
+    item->setText(0, urlInfo.name());
+    item->setText(1, QString::number(urlInfo.size()));
+    item->setText(2, urlInfo.owner());
+    item->setText(3, urlInfo.group());
+    item->setText(4, urlInfo.lastModified().toString("MMM dd yyyy"));
+
     QPixmap pixmap(urlInfo.isDir() ? ":/images/dir.png" : ":/images/file.png");
-    item->setIcon(pixmap);
+    item->setIcon(0, pixmap);
 
     isDirectory[urlInfo.name()] = urlInfo.isDir();
-    fileList->addItem(item);
+    fileList->addTopLevelItem(item);
     if (!fileList->currentItem()) {
-        fileList->setCurrentItem(fileList->item(0));
+        fileList->setCurrentItem(fileList->topLevelItem(0));
         fileList->setEnabled(true);
     }
 }
 
-void FtpWindow::processItem(QListWidgetItem *item)
+void FtpWindow::processItem(QTreeWidgetItem *item, int /*column*/)
 {
-    QString name = item->text();
+    QString name = item->text(0);
     if (isDirectory.value(name)) {
         fileList->clear();
         isDirectory.clear();
@@ -216,14 +278,18 @@ void FtpWindow::processItem(QListWidgetItem *item)
         ftp->cd(name);
         ftp->list();
         cdToParentButton->setEnabled(true);
-        QApplication::setOverrideCursor(Qt::WaitCursor);
+#ifndef QT_NO_CURSOR
+        setCursor(Qt::WaitCursor);
+#endif
         return;
     }
 }
 
 void FtpWindow::cdToParent()
 {
-    QApplication::setOverrideCursor(Qt::WaitCursor);
+#ifndef QT_NO_CURSOR
+    setCursor(Qt::WaitCursor);
+#endif
     fileList->clear();
     isDirectory.clear();
     currentPath = currentPath.left(currentPath.lastIndexOf('/'));
@@ -245,9 +311,9 @@ void FtpWindow::updateDataTransferProgress(qint64 readBytes,
 
 void FtpWindow::enableDownloadButton()
 {
-    QListWidgetItem *current = fileList->currentItem();
+    QTreeWidgetItem *current = fileList->currentItem();
     if (current) {
-        QString currentFile = current->text();
+        QString currentFile = current->text(0);
         downloadButton->setEnabled(!isDirectory.value(currentFile));
     } else {
         downloadButton->setEnabled(false);
