@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,7 @@
 
 #include <QtPlugin>
 
+QT_BEGIN_NAMESPACE
 class QImage;
 class QPainter;
 class QWidget;
@@ -54,7 +55,9 @@ class QPoint;
 class QRect;
 class QString;
 class QStringList;
+QT_END_NAMESPACE
 
+//! [0]
 class BrushInterface
 {
 public:
@@ -68,7 +71,9 @@ public:
     virtual QRect mouseRelease(const QString &brush, QPainter &painter,
                                const QPoint &pos) = 0;
 };
+//! [0]
 
+//! [1]
 class ShapeInterface
 {
 public:
@@ -78,7 +83,9 @@ public:
     virtual QPainterPath generateShape(const QString &shape,
                                        QWidget *parent) = 0;
 };
+//! [1]
 
+//! [2]
 class FilterInterface
 {
 public:
@@ -88,12 +95,19 @@ public:
     virtual QImage filterImage(const QString &filter, const QImage &image,
                                QWidget *parent) = 0;
 };
+//! [2]
 
+QT_BEGIN_NAMESPACE
+//! [3] //! [4]
 Q_DECLARE_INTERFACE(BrushInterface,
                     "com.trolltech.PlugAndPaint.BrushInterface/1.0")
+//! [3]
 Q_DECLARE_INTERFACE(ShapeInterface,
                     "com.trolltech.PlugAndPaint.ShapeInterface/1.0")
+//! [5]
 Q_DECLARE_INTERFACE(FilterInterface,
                     "com.trolltech.PlugAndPaint.FilterInterface/1.0")
+//! [4] //! [5]
+QT_END_NAMESPACE
 
 #endif
