@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -30,18 +50,23 @@
 
 const float Pi = 3.14159f;
 
+//! [0]
 QStringList BasicToolsPlugin::brushes() const
 {
     return QStringList() << tr("Pencil") << tr("Air Brush")
                          << tr("Random Letters");
 }
+//! [0]
 
+//! [1]
 QRect BasicToolsPlugin::mousePress(const QString &brush, QPainter &painter,
                                    const QPoint &pos)
 {
     return mouseMove(brush, painter, pos, pos);
 }
+//! [1]
 
+//! [2]
 QRect BasicToolsPlugin::mouseMove(const QString &brush, QPainter &painter,
                                   const QPoint &oldPos, const QPoint &newPos)
 {
@@ -53,6 +78,7 @@ QRect BasicToolsPlugin::mouseMove(const QString &brush, QPainter &painter,
     QColor color = painter.pen().color();
     int thickness = painter.pen().width();
     QColor transparentColor(color.red(), color.green(), color.blue(), 0);
+//! [2] //! [3]
 
     if (brush == tr("Pencil")) {
         painter.drawLine(oldPos, newPos);
@@ -87,19 +113,25 @@ QRect BasicToolsPlugin::mouseMove(const QString &brush, QPainter &painter,
     painter.restore();
     return boundingRect;
 }
+//! [3]
 
+//! [4]
 QRect BasicToolsPlugin::mouseRelease(const QString & /* brush */,
                                      QPainter & /* painter */,
                                      const QPoint & /* pos */)
 {
     return QRect(0, 0, 0, 0);
 }
+//! [4]
 
+//! [5]
 QStringList BasicToolsPlugin::shapes() const
 {
     return QStringList() << tr("Circle") << tr("Star") << tr("Text...");
 }
+//! [5]
 
+//! [6]
 QPainterPath BasicToolsPlugin::generateShape(const QString &shape,
                                              QWidget *parent)
 {
@@ -127,13 +159,17 @@ QPainterPath BasicToolsPlugin::generateShape(const QString &shape,
 
     return path;
 }
+//! [6]
 
+//! [7]
 QStringList BasicToolsPlugin::filters() const
 {
     return QStringList() << tr("Invert Pixels") << tr("Swap RGB")
                          << tr("Grayscale");
 }
+//! [7]
 
+//! [8]
 QImage BasicToolsPlugin::filterImage(const QString &filter, const QImage &image,
                                      QWidget * /* parent */)
 {
@@ -155,5 +191,10 @@ QImage BasicToolsPlugin::filterImage(const QString &filter, const QImage &image,
     }
     return result;
 }
+//! [8]
 
+QT_BEGIN_NAMESPACE
+//! [9]
 Q_EXPORT_PLUGIN2(pnp_basictools, BasicToolsPlugin)
+//! [9]
+QT_END_NAMESPACE
