@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2005-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,14 +47,19 @@
 
 #include "sortingbox.h"
 
+//! [0]
 SortingBox::SortingBox()
 {
     setAttribute(Qt::WA_StaticContents);
+//! [0] //! [1]
     setMouseTracking(true);
+//! [1] //! [2]
     setBackgroundRole(QPalette::Base);
+//! [2]
 
     itemInMotion = 0;
 
+//! [3]
     newCircleButton = createToolButton(tr("New Circle"),
                                        QIcon(":/images/circle.png"),
                                        SLOT(createNewCircle()));
@@ -57,7 +82,8 @@ SortingBox::SortingBox()
     trianglePath.lineTo(120, 100);
     trianglePath.lineTo(x + 120 / 2, y);
 
-    setWindowTitle(tr("Tooltips"));
+//! [3] //! [4]
+    setWindowTitle(tr("Tool Tips"));
     resize(500, 300);
 
     createShapeItem(circlePath, tr("Circle"), initialItemPosition(circlePath),
@@ -67,20 +93,25 @@ SortingBox::SortingBox()
     createShapeItem(trianglePath, tr("Triangle"),
                     initialItemPosition(trianglePath), initialItemColor());
 }
+//! [4]
 
+//! [5]
 bool SortingBox::event(QEvent *event)
 {
+//! [5] //! [6]
     if (event->type() == QEvent::ToolTip) {
         QHelpEvent *helpEvent = static_cast<QHelpEvent *>(event);
         int index = itemAt(helpEvent->pos());
         if (index != -1)
             QToolTip::showText(helpEvent->globalPos(), shapeItems[index].toolTip());
         else
-            QToolTip::showText(helpEvent->globalPos(), "");
+            QToolTip::hideText();
     }
     return QWidget::event(event);
 }
+//! [6]
 
+//! [7]
 void SortingBox::resizeEvent(QResizeEvent * /* event */)
 {
     int margin = style()->pixelMetric(QStyle::PM_DefaultTopLevelMargin);
@@ -91,18 +122,25 @@ void SortingBox::resizeEvent(QResizeEvent * /* event */)
     y = updateButtonGeometry(newSquareButton, x, y);
     updateButtonGeometry(newTriangleButton, x, y);
 }
+//! [7]
 
+//! [8]
 void SortingBox::paintEvent(QPaintEvent * /* event */)
 {
     QPainter painter(this);
+    painter.setRenderHint(QPainter::Antialiasing);
     foreach (ShapeItem shapeItem, shapeItems) {
+//! [8] //! [9]
         painter.translate(shapeItem.position());
+//! [9] //! [10]
         painter.setBrush(shapeItem.color());
         painter.drawPath(shapeItem.path());
         painter.translate(-shapeItem.position());
     }
 }
+//! [10]
 
+//! [11]
 void SortingBox::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton) {
@@ -115,13 +153,17 @@ void SortingBox::mousePressEvent(QMouseEvent *event)
         }
     }
 }
+//! [11]
 
+//! [12]
 void SortingBox::mouseMoveEvent(QMouseEvent *event)
 {
     if ((event->buttons() & Qt::LeftButton) && itemInMotion)
         moveItemTo(event->pos());
 }
+//! [12]
 
+//! [13]
 void SortingBox::mouseReleaseEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton && itemInMotion) {
@@ -129,28 +171,36 @@ void SortingBox::mouseReleaseEvent(QMouseEvent *event)
         itemInMotion = 0;
     }
 }
+//! [13]
 
+//! [14]
 void SortingBox::createNewCircle()
 {
     static int count = 1;
     createShapeItem(circlePath, tr("Circle <%1>").arg(++count),
                     randomItemPosition(), randomItemColor());
 }
+//! [14]
 
+//! [15]
 void SortingBox::createNewSquare()
 {
     static int count = 1;
     createShapeItem(squarePath, tr("Square <%1>").arg(++count),
                     randomItemPosition(), randomItemColor());
 }
+//! [15]
 
+//! [16]
 void SortingBox::createNewTriangle()
 {
     static int count = 1;
     createShapeItem(trianglePath, tr("Triangle <%1>").arg(++count),
                     randomItemPosition(), randomItemColor());
 }
+//! [16]
 
+//! [17]
 int SortingBox::itemAt(const QPoint &pos)
 {
     for (int i = shapeItems.size() - 1; i >= 0; --i) {
@@ -160,15 +210,20 @@ int SortingBox::itemAt(const QPoint &pos)
     }
     return -1;
 }
+//! [17]
 
+//! [18]
 void SortingBox::moveItemTo(const QPoint &pos)
 {
     QPoint offset = pos - previousPosition;
     itemInMotion->setPosition(itemInMotion->position() + offset);
+//! [18] //! [19]
     previousPosition = pos;
     update();
 }
+//! [19]
 
+//! [20]
 int SortingBox::updateButtonGeometry(QToolButton *button, int x, int y)
 {
     QSize size = button->sizeHint();
@@ -178,7 +233,9 @@ int SortingBox::updateButtonGeometry(QToolButton *button, int x, int y)
     return y - size.rheight()
            - style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
 }
+//! [20]
 
+//! [21]
 void SortingBox::createShapeItem(const QPainterPath &path,
                                  const QString &toolTip, const QPoint &pos,
                                  const QColor &color)
@@ -191,7 +248,9 @@ void SortingBox::createShapeItem(const QPainterPath &path,
     shapeItems.append(shapeItem);
     update();
 }
+//! [21]
 
+//! [22]
 QToolButton *SortingBox::createToolButton(const QString &toolTip,
                                           const QIcon &icon, const char *member)
 {
@@ -203,7 +262,9 @@ QToolButton *SortingBox::createToolButton(const QString &toolTip,
 
     return button;
 }
+//! [22]
 
+//! [23]
 QPoint SortingBox::initialItemPosition(const QPainterPath &path)
 {
     int x;
@@ -216,18 +277,25 @@ QPoint SortingBox::initialItemPosition(const QPainterPath &path)
 
     return QPoint(x, y);
 }
+//! [23]
 
+//! [24]
 QPoint SortingBox::randomItemPosition()
 {
-    return QPoint(rand() % (width() - 120), rand() % (height() - 120));
+    return QPoint(qrand() % (width() - 120), qrand() % (height() - 120));
 }
+//! [24]
 
+//! [25]
 QColor SortingBox::initialItemColor()
 {
     return QColor::fromHsv(((shapeItems.size() + 1) * 85) % 256, 255, 190);
 }
+//! [25]
 
+//! [26]
 QColor SortingBox::randomItemColor()
 {
-    return QColor::fromHsv(rand() % 256, 255, 190);
+    return QColor::fromHsv(qrand() % 256, 255, 190);
 }
+//! [26]
