@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,18 +44,18 @@
 #ifndef QPLATFORMDEFS_H
 #define QPLATFORMDEFS_H
 
+// Set any POSIX/XOPEN defines at the top of this file to turn on specific APIs
+#define _XOPEN_SOURCE 500
+#define __EXTENSIONS__
+
 // Get Qt defines/settings
 
 #include "qglobal.h"
 
-// Set any POSIX/XOPEN defines at the top of this file to turn on specific APIs
-
 #include <unistd.h>
 
-
 // We are hot - unistd.h should have turned on the specific APIs we requested
 
-
 #include <pthread.h>
 #include <dirent.h>
 #include <fcntl.h>
