@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,6 +52,8 @@
 #include <qhash.h>
 #include <qfileinfo.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifdef Q_OS_WIN32
 #define QT_POPEN _popen
 #else
@@ -59,7 +81,6 @@ class MakefileGenerator : protected QMakeSourceFileInfo
     QString spec;
     bool init_opath_already, init_already, no_io;
     QHash<QString, bool> init_compiler_already;
-    QStringList createObjectList(const QStringList &sources);
     QString build_args(const QString &outdir=QString());
     void checkMultipleDefinition(const QString &, const QString &);
 
@@ -69,10 +90,10 @@ class MakefileGenerator : protected QMakeSourceFileInfo
     mutable QHash<ReplaceExtraCompilerCacheKey, QString> extraCompilerVariablesCache;
 
 protected:
+    QStringList createObjectList(const QStringList &sources);
+
     //makefile style generator functions
     void writeObj(QTextStream &, const QString &src);
-    void writeLexSrc(QTextStream &, const QString &lex);
-    void writeYaccSrc(QTextStream &, const QString &yac);
     void writeInstalls(QTextStream &t, const QString &installs, bool noBuild=false);
     void writeHeader(QTextStream &t);
     void writeSubDirs(QTextStream &t);
@@ -81,18 +102,22 @@ protected:
     void writeExtraTargets(QTextStream &t);
     void writeExtraCompilerTargets(QTextStream &t);
     void writeExtraCompilerVariables(QTextStream &t);
-    virtual bool writeMakefile(QTextStream &);
+    virtual bool writeStubMakefile(QTextStream &t);
+    virtual bool writeMakefile(QTextStream &t);
 
     //generating subtarget makefiles
     struct SubTarget
     {
-        QString directory, profile, target, makefile;
+        QString name;
+	QString in_directory, out_directory;
+        QString profile, target, makefile;
         QStringList depends;
     };
     enum SubTargetFlags {
-        SubTargetsNoFlags=0x00,
         SubTargetInstalls=0x01,
-        SubTargetOrdered=0x02
+        SubTargetOrdered=0x02,
+
+        SubTargetsNoFlags=0x00
     };
     void writeSubTargets(QTextStream &t, QList<SubTarget*> subtargets, int flags);
 
@@ -112,7 +137,9 @@ protected:
     virtual QString unescapeFilePath(const QString &path) const;
     virtual QStringList unescapeFilePaths(const QStringList &path) const;
     virtual QString escapeFilePath(const QString &path) const { return path; }
+    virtual QString escapeDependencyPath(const QString &path) const { return escapeFilePath(path); }
     QStringList escapeFilePaths(const QStringList &paths) const;
+    QStringList escapeDependencyPaths(const QStringList &paths) const;
 
     //initialization
     void verifyCompilers();
@@ -138,6 +165,13 @@ protected:
     };
     QStringList findFilesInVPATH(QStringList l, uchar flags, const QString &var="");
 
+    inline int findExecutable(const QStringList &cmdline)
+    { int ret; canExecute(cmdline, &ret); return ret; }
+    bool canExecute(const QStringList &cmdline, int *argv0) const;
+    inline bool canExecute(const QString &cmdline) const
+    { return canExecute(cmdline.split(' '), 0); }
+
+    bool mkdir(const QString &dir) const;
     QString mkdir_p_asstring(const QString &dir, bool escape=true) const;
 
     //subclasses can use these to query information about how the generator was "run"
@@ -148,7 +182,10 @@ protected:
     virtual bool doDepends() const { return Option::mkfile::do_deps; }
 
     void filterIncludedFiles(const QString &);
-    virtual void processSources() { filterIncludedFiles("SOURCES"); filterIncludedFiles("GENERATED_SOURCES"); }
+    virtual void processSources() {
+	filterIncludedFiles("SOURCES");
+	filterIncludedFiles("GENERATED_SOURCES");
+    }
 
     //for cross-platform dependent directories
     virtual void usePlatformDir();
@@ -157,6 +194,7 @@ protected:
     virtual QString defaultInstall(const QString &);
 
     //for prl
+    QString prlFileName(bool fixify=true);
     void writePrlFile();
     bool processPrlFile(QString &);
     virtual void processPrlVariable(const QString &, const QStringList &);
@@ -206,6 +244,7 @@ public:
     virtual bool supportsMergedBuilds() { return false; }
     virtual bool mergeBuildProject(MakefileGenerator * /*other*/) { return false; }
     virtual bool openOutput(QFile &, const QString &build) const;
+    virtual bool isWindowsShell() const { return Option::target_mode == Option::TARG_WIN_MODE; }
 };
 
 inline void MakefileGenerator::setNoIO(bool o)
@@ -223,4 +262,6 @@ inline bool MakefileGenerator::findLibraries()
 inline MakefileGenerator::~MakefileGenerator()
 { }
 
+QT_END_NAMESPACE
+
 #endif // MAKEFILE_H
