@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,7 @@
 #else
 # include <io.h>
 #endif
+#include <qdebug.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
@@ -43,6 +64,8 @@
 #include <share.h>
 #endif
 
+QT_BEGIN_NAMESPACE
+
 #if 1
 #define qmake_endOfLine(c) (c == '\r' || c == '\n')
 #else
@@ -190,7 +213,13 @@ void QMakeSourceFileInfo::dependTreeWalker(SourceFile *node, SourceDependChildre
 
 void QMakeSourceFileInfo::setDependencyPaths(const QList<QMakeLocalFileName> &l)
 {
-    depdirs = l;
+    // Ensure that depdirs does not contain the same paths several times, to minimize the stats
+    QList<QMakeLocalFileName> ll;
+    for (int i = 0; i < l.count(); ++i) {
+        if (!ll.contains(l.at(i)))
+            ll.append(l.at(i));
+    }
+    depdirs = ll;
 }
 
 QStringList QMakeSourceFileInfo::dependencies(const QString &file)
@@ -324,7 +353,7 @@ char *QMakeSourceFileInfo::getBuffer(int s) {
     return spare_buffer;
 }
 
-#ifdef Q_WS_WIN
+#ifndef S_ISDIR
 #define S_ISDIR(x) (x & _S_IFDIR)
 #endif
 
@@ -576,6 +605,8 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                         ++x;
                     }
                 }
+            } else {
+                --x;
             }
         }
 
@@ -630,7 +661,7 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                     dep->exists = exists;
                 }
             }
-            if(dep) {
+            if(dep && dep->file != file->file) {
                 dep->included_count++;
                 if(dep->exists) {
                     debug_msg(5, "%s:%d Found dependency to %s", file->file.real().toLatin1().constData(),
@@ -716,6 +747,19 @@ bool QMakeSourceFileInfo::findMocs(SourceFile *file)
                     }
                 }
             }
+        } else if(*(buffer+x) == '\'' || *(buffer+x) == '"') {
+            const char term = *(buffer+(x++));
+            while(x < buffer_len) {
+                if(*(buffer+x) == term)
+                    break;
+                if(*(buffer+x) == '\\') {
+                    x+=2;
+                } else {
+                    if(qmake_endOfLine(*(buffer+x)))
+                        ++line_count;
+                    ++x;
+                }
+            }
         }
         if(((buffer_len > x+2 &&  *(buffer+x+1) == 'Q' && *(buffer+x+2) == '_')
                    ||
@@ -912,3 +956,5 @@ bool QMakeSourceFileInfo::verifyCache(const QMap<QString, QStringList> &v)
 {
     return v == getCacheVerification();
 }
+
+QT_END_NAMESPACE
