@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,11 +49,13 @@
 #include <qfileinfo.h>
 #include <qregexp.h>
 
+QT_BEGIN_NAMESPACE
+
 QString project_builtin_regx() //calculate the builtin regular expression..
 {
     QString ret;
     QStringList builtin_exts;
-    builtin_exts << Option::c_ext << Option::ui_ext << Option::yacc_ext << Option::lex_ext << ".ts" << ".qrc";
+    builtin_exts << Option::c_ext << Option::ui_ext << Option::yacc_ext << Option::lex_ext << ".ts" << ".xlf" << ".qrc";
     builtin_exts += Option::h_ext + Option::cpp_ext;
     for(int i = 0; i < builtin_exts.size(); ++i) {
         if(!ret.isEmpty())
@@ -432,7 +434,7 @@ ProjectGenerator::addFile(QString file)
             where = "LEXSOURCES";
         else if(file.endsWith(Option::yacc_ext))
             where = "YACCSOURCES";
-        else if(file.endsWith(".ts"))
+        else if(file.endsWith(".ts") || file.endsWith(".xlf"))
             where = "TRANSLATIONS";
         else if(file.endsWith(".qrc"))
             where = "RESOURCES";
@@ -455,6 +457,12 @@ ProjectGenerator::getWritableVar(const QString &v, bool)
     if(vals.isEmpty())
         return "";
 
+    // If values contain spaces, ensure that they are quoted
+    for(QStringList::iterator it = vals.begin(); it != vals.end(); ++it) {
+        if ((*it).contains(' ') && !(*it).startsWith(' '))
+            *it = '\"' + *it + '\"';
+    }
+
     QString ret;
     if(v.endsWith("_REMOVE"))
         ret = v.left(v.length() - 7) + " -= ";
@@ -500,3 +508,5 @@ ProjectGenerator::fixPathToQmake(const QString &file)
         ret = ret.replace(Option::dir_sep, QLatin1String("/"));
     return ret;
 }
+
+QT_END_NAMESPACE
