@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -44,6 +49,8 @@
 
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 DspMakefileGenerator::DspMakefileGenerator() : Win32MakefileGenerator(), init_flag(false)
 {
 }
@@ -182,7 +189,7 @@ bool DspMakefileGenerator::writeDspParts(QTextStream &t)
     writeFileGroup(t, listNames, "Resources", "rc;qrc");
     listNames = QStringList("TRANSLATIONS");
     allListNames += listNames;
-    writeFileGroup(t, listNames, "Translations", "ts");
+    writeFileGroup(t, listNames, "Translations", "ts;xlf");
     listNames = QStringList("LEXSOURCES");
     allListNames += listNames;
     writeFileGroup(t, listNames, "Lexables", "l");
@@ -581,7 +588,7 @@ bool DspMakefileGenerator::writeProjectMakefile()
         writeFileGroup(t, listNames, "Resources", "rc;qrc");
         listNames = QStringList("TRANSLATIONS");
         allListNames += listNames;
-        writeFileGroup(t, listNames, "Translations", "ts");
+        writeFileGroup(t, listNames, "Translations", "ts;xlf");
         listNames = QStringList("LEXSOURCES");
         allListNames += listNames;
         writeFileGroup(t, listNames, "Lexables", "l");
@@ -1133,6 +1140,7 @@ QString DspMakefileGenerator::writeBuildstepForFileForConfig(const QString &file
                 }
             }
 
+
             QString mappedFile;
             if (hasBuiltin) {
                 mappedFile = fileOut;
@@ -1197,3 +1205,5 @@ QString DspMakefileGenerator::writeBuildstepForFileForConfig(const QString &file
 
     return ret;
 }
+
+QT_END_NAMESPACE
