@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 
 #include "winmakefile.h"
 
+QT_BEGIN_NAMESPACE
+
 class NmakeMakefileGenerator : public Win32MakefileGenerator
 {
     bool init_flag;
@@ -71,4 +73,6 @@ public:
 inline NmakeMakefileGenerator::~NmakeMakefileGenerator()
 { }
 
+QT_END_NAMESPACE
+
 #endif // MSVC_NMAKE_H
