@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,12 +46,17 @@
 #include <qdir.h>
 #include <qregexp.h>
 #include <qhash.h>
+#include <qdebug.h>
 #include <qsettings.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
+QT_BEGIN_NAMESPACE
+
 //convenience
+const char *Option::application_argv0 = 0;
 QString Option::prf_ext;
+QString Option::js_ext;
 QString Option::prl_ext;
 QString Option::libtool_ext;
 QString Option::pkgcfg_ext;
@@ -40,6 +65,7 @@ QStringList Option::h_ext;
 QString Option::cpp_moc_ext;
 QString Option::h_moc_ext;
 QStringList Option::cpp_ext;
+QStringList Option::c_ext;
 QString Option::obj_ext;
 QString Option::lex_ext;
 QString Option::yacc_ext;
@@ -70,6 +96,7 @@ QStringList Option::user_configs;
 QStringList Option::after_user_configs;
 QString Option::user_template;
 QString Option::user_template_prefix;
+QStringList Option::shellPath;
 #if defined(Q_OS_WIN32)
 Option::TARG_MODE Option::target_mode = Option::TARG_WIN_MODE;
 #elif defined(Q_OS_MAC)
@@ -94,6 +121,7 @@ bool Option::mkfile::do_deps = true;
 bool Option::mkfile::do_mocs = true;
 bool Option::mkfile::do_dep_heuristics = true;
 bool Option::mkfile::do_preprocess = false;
+bool Option::mkfile::do_stub_makefile = false;
 bool Option::mkfile::do_cache = true;
 QString Option::mkfile::cachefile;
 QStringList Option::mkfile::project_files;
@@ -122,44 +150,44 @@ bool usage(const char *a0)
             "mode for qmake, but you may use this to test qmake on an existing project\n"
             "\n"
             "Mode:\n"
-            "\t-project       Put qmake into project file generation mode%s\n"
-            "\t               In this mode qmake interprets files as files to\n"
-            "\t               be built,\n"
-            "\t               defaults to %s\n"
-            "\t-makefile      Put qmake into makefile generation mode%s\n"
-            "\t               In this mode qmake interprets files as project files to\n"
-            "\t               be processed, if skipped qmake will try to find a project\n"
-            "\t               file in your current working directory\n"
+            "  -project       Put qmake into project file generation mode%s\n"
+            "                 In this mode qmake interprets files as files to\n"
+            "                 be built,\n"
+            "                 defaults to %s\n"
+            "  -makefile      Put qmake into makefile generation mode%s\n"
+            "                 In this mode qmake interprets files as project files to\n"
+            "                 be processed, if skipped qmake will try to find a project\n"
+            "                 file in your current working directory\n"
             "\n"
             "Warnings Options:\n"
-            "\t-Wnone         Turn off all warnings\n"
-            "\t-Wall          Turn on all warnings\n"
-            "\t-Wparser       Turn on parser warnings\n"
-            "\t-Wlogic        Turn on logic warnings\n"
+            "  -Wnone         Turn off all warnings\n"
+            "  -Wall          Turn on all warnings\n"
+            "  -Wparser       Turn on parser warnings\n"
+            "  -Wlogic        Turn on logic warnings\n"
             "\n"
             "Options:\n"
-            "\t * You can place any variable assignment in options and it will be     *\n"
-            "\t * processed as if it was in [files]. These assignments will be parsed *\n"
-            "\t * before [files].                                                     *\n"
-            "\t-o file        Write output to file\n"
-            "\t-unix          Run in unix mode\n"
-            "\t-win32         Run in win32 mode\n"
-            "\t-macx          Run in Mac OS X mode\n"
-            "\t-d             Increase debug level\n"
-            "\t-t templ       Overrides TEMPLATE as templ\n"
-            "\t-tp prefix     Overrides TEMPLATE so that prefix is prefixed into the value\n"
-            "\t-help          This help\n"
-            "\t-v             Version information\n"
-            "\t-after         All variable assignments after this will be\n"
-            "\t               parsed after [files]\n"
-            "\t-norecursive   Don't do a recursive search\n"
-            "\t-recursive     Do a recursive search\n"
-            "\t-cache file    Use file as cache           [makefile mode only]\n"
-            "\t-spec spec     Use spec as QMAKESPEC       [makefile mode only]\n"
-            "\t-nocache       Don't use a cache file      [makefile mode only]\n"
-            "\t-nodepend      Don't generate dependencies [makefile mode only]\n"
-            "\t-nomoc         Don't generate moc targets  [makefile mode only]\n"
-            "\t-nopwd         Don't look for files in pwd [project mode only]\n"
+            "   * You can place any variable assignment in options and it will be     *\n"
+            "   * processed as if it was in [files]. These assignments will be parsed *\n"
+            "   * before [files].                                                     *\n"
+            "  -o file        Write output to file\n"
+            "  -unix          Run in unix mode\n"
+            "  -win32         Run in win32 mode\n"
+            "  -macx          Run in Mac OS X mode\n"
+            "  -d             Increase debug level\n"
+            "  -t templ       Overrides TEMPLATE as templ\n"
+            "  -tp prefix     Overrides TEMPLATE so that prefix is prefixed into the value\n"
+            "  -help          This help\n"
+            "  -v             Version information\n"
+            "  -after         All variable assignments after this will be\n"
+            "                 parsed after [files]\n"
+            "  -norecursive   Don't do a recursive search\n"
+            "  -recursive     Do a recursive search\n"
+            "  -cache file    Use file as cache           [makefile mode only]\n"
+            "  -spec spec     Use spec as QMAKESPEC       [makefile mode only]\n"
+            "  -nocache       Don't use a cache file      [makefile mode only]\n"
+            "  -nodepend      Don't generate dependencies [makefile mode only]\n"
+            "  -nomoc         Don't generate moc targets  [makefile mode only]\n"
+            "  -nopwd         Don't look for files in pwd [project mode only]\n"
             ,a0,
             default_mode(a0) == Option::QMAKE_GENERATE_PROJECT  ? " (default)" : "", project_builtin_regx().toLatin1().constData(),
             default_mode(a0) == Option::QMAKE_GENERATE_MAKEFILE ? " (default)" : ""
@@ -218,12 +246,12 @@ Option::parseCommandLine(int argc, char **argv, int skip)
                 Option::debug_level++;
             } else if(opt == "version" || opt == "v" || opt == "-version") {
                 fprintf(stdout,
-                        "QMake version: %s\n"
+                        "QMake version %s\n"
                         "Using Qt version %s in %s\n",
                         qmake_version(), QT_VERSION_STR,
                         QLibraryInfo::location(QLibraryInfo::LibrariesPath).toLatin1().constData());
 #ifdef QMAKE_OPENSOURCE_VERSION
-                fprintf(stdout, "QMake is Open Source software from Trolltech AS.\n");
+                fprintf(stdout, "QMake is Open Source software from Trolltech ASA.\n");
 #endif
                 return Option::QMAKE_CMDLINE_BAIL;
             } else if(opt == "h" || opt == "help") {
@@ -251,6 +279,8 @@ Option::parseCommandLine(int argc, char **argv, int skip)
                         Option::mkfile::do_mocs = false;
                     } else if(opt == "nocache") {
                         Option::mkfile::do_cache = false;
+                    } else if(opt == "createstub") {
+                        Option::mkfile::do_stub_makefile = true;
                     } else if(opt == "nodependheuristics") {
                         Option::mkfile::do_dep_heuristics = false;
                     } else if(opt == "E") {
@@ -307,9 +337,26 @@ Option::parseCommandLine(int argc, char **argv, int skip)
     return Option::QMAKE_CMDLINE_SUCCESS;
 }
 
+#ifdef Q_OS_WIN
+static QStringList detectShellPath()
+{
+    QStringList paths;
+    QString path = qgetenv("PATH");
+    QStringList pathlist = path.toLower().split(";");
+    for (int i = 0; i < pathlist.count(); i++) {
+        QString maybeSh = pathlist.at(i) + "/sh.exe";
+        if (QFile::exists(maybeSh)) {
+            paths.append(maybeSh);
+        }
+    }
+    return paths;
+}
+#endif
+
 int
 Option::init(int argc, char **argv)
 {
+    Option::application_argv0 = 0;
     Option::cpp_moc_mod = "";
     Option::h_moc_mod = "moc_";
     Option::lex_mod = "_lex";
@@ -318,8 +365,10 @@ Option::init(int argc, char **argv)
     Option::libtool_ext = ".la";
     Option::pkgcfg_ext = ".pc";
     Option::prf_ext = ".prf";
+    Option::js_ext = ".js";
     Option::ui_ext = ".ui";
     Option::h_ext << ".h" << ".hpp" << ".hh" << ".hxx";
+    Option::c_ext << ".c";
 #ifndef Q_OS_WIN
     Option::h_ext << ".H";
 #endif
@@ -334,6 +383,7 @@ Option::init(int argc, char **argv)
     Option::pro_ext = ".pro";
 #ifdef Q_OS_WIN
     Option::dirlist_sep = ";";
+    Option::shellPath = detectShellPath();
 #else
     Option::dirlist_sep = ":";
 #endif
@@ -341,12 +391,17 @@ Option::init(int argc, char **argv)
     Option::field_sep = ' ';
 
     if(argc && argv) {
+        Option::application_argv0 = argv[0];
         QString argv0 = argv[0];
         if(Option::qmake_mode == Option::QMAKE_GENERATE_NOTHING)
             Option::qmake_mode = default_mode(argv0);
-        if(!argv0.isEmpty() && argv0.at(0) == QLatin1Char('/')) {
+        if(!argv0.isEmpty() && !QFileInfo(argv0).isRelative()) {
             Option::qmake_abslocation = argv0;
-        } else if (argv0.contains(QLatin1Char('/'))) { //relative PWD
+        } else if (argv0.contains(QLatin1Char('/'))
+#ifdef Q_OS_WIN
+		   || argv0.contains(QLatin1Char('\\'))
+#endif
+	    ) { //relative PWD
             Option::qmake_abslocation = QDir::current().absoluteFilePath(argv0);
         } else { //in the PATH
             QByteArray pEnv = qgetenv("PATH");
@@ -360,6 +415,9 @@ Option::init(int argc, char **argv)
                 if ((*p).isEmpty())
                     continue;
                 QString candidate = currentDir.absoluteFilePath(*p + QLatin1Char('/') + argv0);
+#ifdef Q_OS_WIN
+                candidate += ".exe";
+#endif
                 if (QFile::exists(candidate)) {
                     Option::qmake_abslocation = candidate;
                     break;
@@ -463,6 +521,7 @@ Option::init(int argc, char **argv)
             Option::dir_sep = "/";
         Option::obj_ext = ".o";
     }
+    Option::qmake_abslocation = Option::fixPathToTargetOS(Option::qmake_abslocation);
     return QMAKE_CMDLINE_SUCCESS;
 }
 
@@ -474,6 +533,9 @@ bool Option::postProcessProject(QMakeProject *project)
     Option::h_ext = project->variables()["QMAKE_EXT_H"];
     if(h_ext.isEmpty())
         h_ext << ".h";
+    Option::c_ext = project->variables()["QMAKE_EXT_C"];
+    if(c_ext.isEmpty())
+        c_ext << ".c"; //something must be there
 
     if(!project->isEmpty("QMAKE_EXT_RES"))
         Option::res_ext = project->first("QMAKE_EXT_RES");
@@ -485,6 +547,8 @@ bool Option::postProcessProject(QMakeProject *project)
         Option::prl_ext = project->first("QMAKE_EXT_PRL");
     if(!project->isEmpty("QMAKE_EXT_PRF"))
         Option::prf_ext = project->first("QMAKE_EXT_PRF");
+    if(!project->isEmpty("QMAKE_EXT_JS"))
+        Option::prf_ext = project->first("QMAKE_EXT_JS");
     if(!project->isEmpty("QMAKE_EXT_UI"))
         Option::ui_ext = project->first("QMAKE_EXT_UI");
     if(!project->isEmpty("QMAKE_EXT_CPP_MOC"))
@@ -517,14 +581,18 @@ bool Option::postProcessProject(QMakeProject *project)
 QString
 Option::fixString(QString string, uchar flags)
 {
+    const QString orig_string = string;
     static QHash<FixStringCacheKey, QString> *cache = 0;
     if(!cache) {
         cache = new QHash<FixStringCacheKey, QString>;
         qmakeAddCacheClear(qmakeDeleteCacheClear_QHashFixStringCacheKeyQString, (void**)&cache);
     }
     FixStringCacheKey cacheKey(string, flags);
-    if(cache->contains(cacheKey))
-        return cache->value(cacheKey);
+    if(cache->contains(cacheKey)) {
+	const QString ret = cache->value(cacheKey);
+	//qDebug() << "Fix (cached) " << orig_string << "->" << ret;
+        return ret;
+    }
 
     //fix the environment variables
     if(flags & Option::FixEnvVars) {
@@ -564,6 +632,7 @@ Option::fixString(QString string, uchar flags)
         string = string.mid(1, string.length()-2);
 
     //cache
+    //qDebug() << "Fix" << orig_string << "->" << string;
     cache->insert(cacheKey, string);
     return string;
 }
@@ -611,86 +680,6 @@ void warn_msg(QMakeWarn type, const char *fmt, ...)
     fprintf(stderr, "\n");
 }
 
-#include "../src/corelib/global/qconfig.cpp"
-QString QLibraryInfo::location(QLibraryInfo::LibraryLocation loc)
-{
-    QString ret;
-    const char *path = 0;
-    switch (loc) {
-#ifdef QT_CONFIGURE_PREFIX_PATH
-    case PrefixPath:
-        path = QT_CONFIGURE_PREFIX_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_DOCUMENTATION_PATH
-    case DocumentationPath:
-        path = QT_CONFIGURE_DOCUMENTATION_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_HEADERS_PATH
-    case HeadersPath:
-        path = QT_CONFIGURE_HEADERS_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_LIBRARIES_PATH
-    case LibrariesPath:
-        path = QT_CONFIGURE_LIBRARIES_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_BINARIES_PATH
-    case BinariesPath:
-        path = QT_CONFIGURE_BINARIES_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_PLUGINS_PATH
-    case PluginsPath:
-        path = QT_CONFIGURE_PLUGINS_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_DATA_PATH
-    case DataPath:
-        path = QT_CONFIGURE_DATA_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_TRANSLATIONS_PATH
-    case TranslationsPath:
-        path = QT_CONFIGURE_TRANSLATIONS_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_SETTINGS_PATH
-    case SettingsPath:
-        path = QT_CONFIGURE_SETTINGS_PATH;
-        break;
-#endif
-#ifdef QT_CONFIGURE_EXAMPLES_PATH
-        case ExamplesPath:
-            path = QT_CONFIGURE_EXAMPLES_PATH;
-            break;
-#endif
-#ifdef QT_CONFIGURE_DEMOS_PATH
-        case DemosPath:
-            path = QT_CONFIGURE_DEMOS_PATH;
-            break;
-#endif
-    default:
-        break;
-    }
-
-    if (path)
-        ret = QString::fromLocal8Bit(path);
-
-    if (QDir::isRelativePath(ret)) {
-        if (loc == PrefixPath) {
-            // we make the prefix path absolute to the current directory
-            return QDir::current().absoluteFilePath(ret);
-        } else {
-            // we make any other path absolute to the prefix directory
-            return QDir(location(PrefixPath)).absoluteFilePath(ret);
-        }
-    }
-    return ret;
-}
-
 class QMakeCacheClearItem {
 private:
     qmakeCacheClearFunc func;
@@ -716,3 +705,72 @@ qmakeAddCacheClear(qmakeCacheClearFunc func, void **data)
 {
     cache_items.append(new QMakeCacheClearItem(func, data));
 }
+
+#ifdef Q_OS_WIN
+# include <windows.h>
+
+QT_USE_NAMESPACE
+#endif
+
+QString qmake_libraryInfoFile()
+{
+    QString ret;
+#if defined( Q_OS_WIN )
+    QFileInfo filePath;
+    QT_WA({
+        unsigned short module_name[256];
+        GetModuleFileNameW(0, reinterpret_cast<wchar_t *>(module_name), sizeof(module_name));
+        filePath = QString::fromUtf16(module_name);
+    }, {
+        char module_name[256];
+        GetModuleFileNameA(0, module_name, sizeof(module_name));
+        filePath = QString::fromLocal8Bit(module_name);
+    });
+    ret = filePath.filePath();
+#else
+    QString argv0 = QFile::decodeName(QByteArray(Option::application_argv0));
+    QString absPath;
+
+    if (!argv0.isEmpty() && argv0.at(0) == QLatin1Char('/')) {
+        /*
+          If argv0 starts with a slash, it is already an absolute
+          file path.
+        */
+        absPath = argv0;
+    } else if (argv0.contains(QLatin1Char('/'))) {
+        /*
+          If argv0 contains one or more slashes, it is a file path
+          relative to the current directory.
+        */
+        absPath = QDir::current().absoluteFilePath(argv0);
+    } else {
+        /*
+          Otherwise, the file path has to be determined using the
+          PATH environment variable.
+        */
+        QByteArray pEnv = qgetenv("PATH");
+        QDir currentDir = QDir::current();
+        QStringList paths = QString::fromLocal8Bit(pEnv.constData()).split(QLatin1String(":"));
+        for (QStringList::const_iterator p = paths.constBegin(); p != paths.constEnd(); ++p) {
+            if ((*p).isEmpty())
+                continue;
+            QString candidate = currentDir.absoluteFilePath(*p + QLatin1Char('/') + argv0);
+            QFileInfo candidate_fi(candidate);
+            if (candidate_fi.exists() && !candidate_fi.isDir()) {
+                absPath = candidate;
+                break;
+            }
+        }
+    }
+
+    absPath = QDir::cleanPath(absPath);
+
+    QFileInfo fi(absPath);
+    ret = fi.exists() ? fi.canonicalFilePath() : QString();
+#endif
+    if(!ret.isEmpty())
+        ret = QDir(QFileInfo(ret).absolutePath()).filePath("qt.conf");
+    return ret;
+}
+
+QT_END_NAMESPACE
