@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@
 #include <qstringlist.h>
 #include <qfile.h>
 
+QT_BEGIN_NAMESPACE
+
 #define QMAKE_VERSION_MAJOR 2
 #define QMAKE_VERSION_MINOR 1
 #define QMAKE_VERSION_PATCH 0
@@ -209,4 +211,6 @@ public:
     static QString location(LibraryLocation);
 };
 
+QT_END_NAMESPACE
+
 #endif // OPTION_H
