@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 
 #ifndef QT_NO_CODECS
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QIsciiCodec
     \brief The QIsciiCodec class provides conversion to and from the ISCII encoding.
@@ -283,4 +285,6 @@ QString QIsciiCodec::convertToUnicode(const char* chars, int len, ConverterState
     return result;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_CODECS
