@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -58,6 +58,8 @@
 #include "QtCore/qabstractfileengine.h"
 #include "QtCore/qfile.h"
 
+QT_BEGIN_NAMESPACE
+
 class QAbstractFileEnginePrivate
 {
 public:
@@ -74,4 +76,6 @@ public:
     Q_DECLARE_PUBLIC(QAbstractFileEngine)
 };
 
+QT_END_NAMESPACE
+
 #endif // QABSTRACTFILEENGINE_P_H
