@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -30,6 +50,8 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QDataStream
     \reentrant
@@ -59,24 +81,11 @@
 
     Example (write binary data to a stream):
 
-    \code
-        QFile file("file.dat");
-        file.open(QIODevice::WriteOnly);
-        QDataStream out(&file);   // we will serialize the data into the file
-        out << "the answer is";   // serialize a string
-        out << (qint32)42;        // serialize an integer
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 0
 
     Example (read binary data from a stream):
 
-    \code
-        QFile file("file.dat");
-        file.open(QIODevice::ReadOnly);
-        QDataStream in(&file);    // read the data serialized from the file
-        QString str;
-        qint32 a;
-        in >> str >> a;           // extract "the answer is" and 42
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 1
 
     Each item written to the stream is written in a predefined binary
     format that varies depending on the item's type. Supported Qt
@@ -111,9 +120,7 @@
     compatibility, you can hardcode the version number in the
     application:
 
-    \code
-        stream.setVersion(QDataStream::Qt_4_0);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 2
 
     If you are producing a new binary data format, such as a file
     format for documents created by your application, you could use a
@@ -121,53 +128,11 @@
     would write a brief header containing a magic string and a version
     number to give yourself room for future expansion. For example:
 
-    \code
-        QFile file("file.xxx");
-        file.open(QIODevice::WriteOnly);
-        QDataStream out(&file);
-
-        // Write a header with a "magic number" and a version
-        out << (quint32)0xA0B0C0D0;
-        out << (qint32)123;
-
-        out.setVersion(QDataStream::Qt_4_0);
-
-        // Write the data
-        out << lots_of_interesting_data;
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 3
 
     Then read it in with:
 
-    \code
-        QFile file("file.xxx");
-        file.open(QIODevice::ReadOnly);
-        QDataStream in(&file);
-
-        // Read and check the header
-        quint32 magic;
-        in >> magic;
-        if (magic != 0xA0B0C0D0)
-            return XXX_BAD_FILE_FORMAT;
-
-        // Read the version
-        qint32 version;
-        in >> version;
-        if (version < 100)
-            return XXX_BAD_FILE_TOO_OLD;
-        if (version > 123)
-            return XXX_BAD_FILE_TOO_NEW;
-
-        if (version <= 110)
-            in.setVersion(QDataStream::Qt_3_2);
-        else
-            in.setVersion(QDataStream::Qt_4_0);
-
-        // Read the data
-        in >> lots_of_interesting_data;
-        if (version >= 120)
-            in >> data_new_in_XXX_version_1_2;
-        in >> other_interesting_data;
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 4
 
     You can select which byte order to use when serializing data. The
     default setting is big endian (MSB first). Changing it to little
@@ -201,7 +166,7 @@
     The byte order used for reading/writing the data.
 
     \value BigEndian Most significant byte first (the default)
-    \value LittleEndian Less significant byte first
+    \value LittleEndian Least significant byte first
 */
 
 /*!
@@ -219,19 +184,22 @@
   QDataStream member functions
  *****************************************************************************/
 
-#ifndef QT_NO_DEBUG
 #undef  CHECK_STREAM_PRECOND
+#ifndef QT_NO_DEBUG
 #define CHECK_STREAM_PRECOND(retVal) \
     if (!dev) { \
         qWarning("QDataStream: No device"); \
         return retVal; \
     }
 #else
-#define CHECK_STREAM_PRECOND(retVal)
+#define CHECK_STREAM_PRECOND(retVal) \
+    if (!dev) { \
+        return retVal; \
+    }
 #endif
 
 enum {
-    DefaultStreamVersion = QDataStream::Qt_4_2
+    DefaultStreamVersion = QDataStream::Qt_4_4
 };
 
 // ### 4.0: when streaming invalid QVariants, just the type should
@@ -385,9 +353,9 @@ void QDataStream::setDevice(QIODevice *d)
 }
 
 /*!
-    Unsets the I/O device. This is the same as calling setDevice(0).
-
-    \sa device(), setDevice()
+    \obsolete
+    Unsets the I/O device.
+    Use setDevice(0) instead.
 */
 
 void QDataStream::unsetDevice()
@@ -512,6 +480,8 @@ void QDataStream::setByteOrder(ByteOrder bo)
     \value Qt_4_0 Version 7 (Qt 4.0, Qt 4.1)
     \value Qt_4_1 Version 7 (Qt 4.0, Qt 4.1)
     \value Qt_4_2 Version 8 (Qt 4.2)
+    \value Qt_4_3 Version 9 (Qt 4.3)
+    \value Qt_4_4 Version 10 (Qt 4.4)
 
     \sa setVersion(), version()
 */
@@ -543,23 +513,21 @@ void QDataStream::setByteOrder(ByteOrder bo)
 
     \table
     \header \i Qt Version       \i QDataStream Version
-    \row \i Qt 4.2              \i 8
-    \row \i Qt 4.0              \i 7
-    \row \i Qt 3.3              \i 6
-    \row \i Qt 3.1, 3.2         \i 5
-    \row \i Qt 3.0              \i 4
-    \row \i Qt 2.1, 2.2, 2.3    \i 3
-    \row \i Qt 2.0              \i 2
-    \row \i Qt 1.x              \i 1
+    \row \i Qt 4.3                  \i 9
+    \row \i Qt 4.2                  \i 8
+    \row \i Qt 4.0, 4.1            \i 7
+    \row \i Qt 3.3                  \i 6
+    \row \i Qt 3.1, 3.2             \i 5
+    \row \i Qt 3.0                  \i 4
+    \row \i Qt 2.1, 2.2, 2.3      \i 3
+    \row \i Qt 2.0                  \i 2
+    \row \i Qt 1.x                  \i 1
     \endtable
 
     The \l Version enum provides symbolic constants for the different
     versions of Qt. For example:
 
-    \code
-        QDataStream out(file);
-        out.setVersion(QDataStream::Qt_4_0);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdatastream.cpp 5
 
     \sa version(), Version
 */
@@ -762,6 +730,9 @@ QDataStream &QDataStream::operator>>(float &f)
     return *this;
 }
 
+#if defined(Q_DOUBLE_FORMAT)
+#define Q_DF(x) Q_DOUBLE_FORMAT[(x)] - '0'
+#endif
 
 /*!
     \overload
@@ -775,6 +746,7 @@ QDataStream &QDataStream::operator>>(double &f)
 {
     f = 0.0;
     CHECK_STREAM_PRECOND(*this)
+#ifndef Q_DOUBLE_FORMAT
     if (noswap) {
         if (dev->read((char *)&f, 8) != 8) {
             f = 0.0;
@@ -796,6 +768,34 @@ QDataStream &QDataStream::operator>>(double &f)
             setStatus(ReadPastEnd);
         }
     }
+#else
+    //non-standard floating point format
+    register uchar *p = (uchar *)(&f);
+    char b[8];
+    if (dev->read(b, 8) == 8) {
+        if (noswap) {
+            *p++ = b[Q_DF(0)];
+            *p++ = b[Q_DF(1)];
+            *p++ = b[Q_DF(2)];
+            *p++ = b[Q_DF(3)];
+            *p++ = b[Q_DF(4)];
+            *p++ = b[Q_DF(5)];
+            *p++ = b[Q_DF(6)];
+            *p = b[Q_DF(7)];
+        } else {
+            *p++ = b[Q_DF(7)];
+            *p++ = b[Q_DF(6)];
+            *p++ = b[Q_DF(5)];
+            *p++ = b[Q_DF(4)];
+            *p++ = b[Q_DF(3)];
+            *p++ = b[Q_DF(2)];
+            *p++ = b[Q_DF(1)];
+            *p = b[Q_DF(0)];
+        }
+    } else {
+        setStatus(ReadPastEnd);
+    }
+#endif
     return *this;
 }
 
@@ -873,8 +873,8 @@ QDataStream &QDataStream::readBytes(char *&s, uint &l)
 }
 
 /*!
-    Reads \a len bytes from the stream into \a s and returns a
-    reference to the stream.
+    Reads at most \a len bytes from the stream into \a s and returns the number of
+    bytes read. If an error occurs, this function returns -1.
 
     The buffer \a s must be preallocated. The data is \e not encoded.
 
@@ -1064,6 +1064,7 @@ QDataStream &QDataStream::operator<<(float f)
 QDataStream &QDataStream::operator<<(double f)
 {
     CHECK_STREAM_PRECOND(*this)
+#ifndef Q_DOUBLE_FORMAT
     if (noswap) {
         dev->write((char *)&f, sizeof(double));
     } else {
@@ -1079,6 +1080,30 @@ QDataStream &QDataStream::operator<<(double f)
         b[0] = *p;
         dev->write(b, 8);
     }
+#else
+    register uchar *p = (uchar *)(&f);
+    char b[8];
+    if (noswap) {
+        b[Q_DF(0)] = *p++;
+        b[Q_DF(1)] = *p++;
+        b[Q_DF(2)] = *p++;
+        b[Q_DF(3)] = *p++;
+        b[Q_DF(4)] = *p++;
+        b[Q_DF(5)] = *p++;
+        b[Q_DF(6)] = *p++;
+        b[Q_DF(7)] = *p;
+    } else {
+        b[Q_DF(7)] = *p++;
+        b[Q_DF(6)] = *p++;
+        b[Q_DF(5)] = *p++;
+        b[Q_DF(4)] = *p++;
+        b[Q_DF(3)] = *p++;
+        b[Q_DF(2)] = *p++;
+        b[Q_DF(1)] = *p++;
+        b[Q_DF(0)] = *p;
+    }
+    dev->write(b, 8);
+#endif
     return *this;
 }
 
@@ -1144,7 +1169,7 @@ int QDataStream::writeRawData(const char *s, int len)
 
     Skips \a len bytes from the device. Returns the number of bytes
     actually skipped, or -1 on error.
-    
+
     This is equivalent to calling readRawData() on a buffer of length
     \a len and ignoring the buffer.
 
@@ -1193,4 +1218,6 @@ int QDataStream::skipRawData(int len)
 */
 #endif
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_DATASTREAM
