@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,7 +44,7 @@
 #include "qplatformdefs.h"
 #include "qdir.h"
 #include "qabstractfileengine.h"
-#ifndef QT_NO_DEBUG
+#ifndef QT_NO_DEBUG_STREAM
 #include "qdebug.h"
 #endif
 #include "qfsfileengine.h"
@@ -36,8 +56,11 @@
 # include "qresource.h"
 #endif
 
+#include "../kernel/qcoreglobaldata_p.h"
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 static QString driveSpec(const QString &path)
 {
 #ifdef Q_OS_WIN
@@ -65,7 +88,7 @@ protected:
     QDirPrivate(QDir*, const QDir *copy=0);
     ~QDirPrivate();
 
-    void initFileEngine(const QString &file);
+    QString initFileEngine(const QString &file);
 
     void updateFileLists() const;
     void sortFileList(QDir::SortFlags, QStringList &, QStringList *, QFileInfoList *) const;
@@ -106,7 +129,7 @@ private:
         inline void clear() {
             listsDirty = 1;
         }
-        mutable QAtomic ref;
+        mutable QAtomicInt ref;
 
         QString path;
         QStringList nameFilters;
@@ -126,12 +149,12 @@ private:
         if ((path.endsWith(QLatin1Char('/')) || path.endsWith(QLatin1Char('\\')))
                 && path.length() > 1) {
 #ifdef Q_OS_WIN
-            if (!(path.length() == 3 && path.at(1) == ':'))
+            if (!(path.length() == 3 && path.at(1) == QLatin1Char(':')))
 #endif
                 path.truncate(path.length() - 1);
         }
         if(!data->fileEngine || !QDir::isRelativePath(path))
-            initFileEngine(path);
+            path = initFileEngine(path);
         data->fileEngine->setFileName(path);
         // set the path to be the qt friendly version so then we can operate on it using just /
         data->path = data->fileEngine->fileName(QAbstractFileEngine::DefaultName);
@@ -179,7 +202,7 @@ static int qt_cmp_si_sort_flags;
 extern "C" {
 #endif
 
-#ifdef Q_OS_TEMP
+#ifdef Q_OS_WINCE
 static int __cdecl qt_cmp_si(const void *n1, const void *n2)
 #else
 static int qt_cmp_si(const void *n1, const void *n2)
@@ -205,7 +228,7 @@ static int qt_cmp_si(const void *n1, const void *n2)
         r = f1->item.lastModified().secsTo(f2->item.lastModified());
         break;
       case QDir::Size:
-        r = f2->item.size() - f1->item.size();
+          r = int(qBound<qint64>(-1, f2->item.size() - f1->item.size(), 1));
         break;
       case QDir::Type:
       {
@@ -272,7 +295,8 @@ inline void QDirPrivate::sortFileList(QDir::SortFlags sort, QStringList &l,
             si[i].item = QFileInfo(path + l.at(i));
         }
         qt_cmp_si_sort_flags = sort;
-        qsort(si, i, sizeof(si[0]), qt_cmp_si);
+        if ((sort & QDir::SortByMask) != QDir::Unsorted)
+            qsort(si, i, sizeof(si[0]), qt_cmp_si);
         // put them back in the list(s)
         for (int j = 0; j<i; j++) {
             if(infos)
@@ -293,13 +317,14 @@ inline void QDirPrivate::updateFileLists() const
     }
 }
 
-void QDirPrivate::initFileEngine(const QString &path)
+QString QDirPrivate::initFileEngine(const QString &path)
 {
     detach(false);
     delete data->fileEngine;
     data->fileEngine = 0;
     data->clear();
     data->fileEngine = QAbstractFileEngine::create(path);
+    return data->fileEngine->fileName(QAbstractFileEngine::DefaultName);
 }
 
 void QDirPrivate::detach(bool createFileEngine)
@@ -337,19 +362,14 @@ void QDirPrivate::detach(bool createFileEngine)
 
     Examples of absolute paths:
 
-    \code
-    QDir("/home/user/Documents")
-    QDir("C:/Documents and Settings")
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 0
 
-    On Windows, the second of the examples above will be translated to
-    \c{C:\My Documents} when used to access files.
+    On Windows, the second example above will be translated to
+    \c{C:\Documents and Settings} when used to access files.
 
     Examples of relative paths:
 
-    \code
-    QDir("images/landscape.png")
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 1
 
     You can use the isRelative() or isAbsolute() functions to check if
     a QDir is using a relative or an absolute file path. Call
@@ -366,10 +386,7 @@ void QDirPrivate::detach(bool createFileEngine)
     the location of the directory. However, it can also return "." if
     the QDir represents the current directory.
 
-    \code
-    QDir("Documents/Letters/Applications").dirName() // "Applications"
-    QDir().dirName()                                 // "."
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 2
 
     The path for a directory can also be changed with the cd() and cdUp()
     functions, both of which operate like familiar shell commands.
@@ -405,11 +422,7 @@ void QDirPrivate::detach(bool createFileEngine)
     file or directory. Neither of these functions checks for the
     existence of files or directory; they only construct paths.
 
-    \code
-    QDir directory("Documents/Letters");
-    QString path = directory.filePath("contents.txt");
-    QString absolutePath = directory.absoluteFilePath("contents.txt");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 3
 
     Files can be removed by using the remove() function. Directories
     cannot be removed in the same way as files; use rmdir() to remove
@@ -481,33 +494,19 @@ void QDirPrivate::detach(bool createFileEngine)
 
     Check if a directory exists:
 
-    \code
-        QDir dir("example");
-        if (!dir.exists())
-            qWarning("Cannot find the example directory");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 4
 
     (We could also use the static convenience function
     QFile::exists().)
 
     Traversing directories and reading a file:
 
-    \code
-        QDir dir = QDir::root();                 // "/"
-        if (!dir.cd("tmp")) {                    // "/tmp"
-            qWarning("Cannot find the \"/tmp\" directory");
-        } else {
-            QFile file(dir.filePath("ex1.txt")); // "/tmp/ex1.txt"
-            if (!file.open(QIODevice::ReadWrite))
-                qWarning("Cannot create the file %s", file.name());
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 5
 
     A program that lists all the files in the current directory
     (excluding symbolic links), sorted by size, smallest first:
 
-    \quotefromfile snippets/qdir-listfiles/main.cpp
-    \printuntil /^\}/
+    \snippet doc/src/snippets/qdir-listfiles/main.cpp 0
 
     \sa QFileInfo, QFile, QFileDialog, QApplication::applicationDirPath(), {Find Files Example}
 */
@@ -661,17 +660,7 @@ QString QDir::absolutePath() const
 
     Example:
 
-    \code
-        QString bin = "/local/bin";         // where /local/bin is a symlink to /usr/bin
-        QDir binDir(bin);
-        QString canonicalBin = binDir.canonicalPath();
-        // canonicalBin now equals "/usr/bin"
-
-        QString ls = "/local/bin/ls";       // where ls is the executable "ls"
-        QDir lsDir(ls);
-        QString canonicalLs = lsDir.canonicalPath();
-        // canonicalLS now equals "/usr/bin/ls".
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 6
 
     \sa path(), absolutePath(), exists(), cleanPath(), dirName(),
         absoluteFilePath()
@@ -750,8 +739,10 @@ QString QDir::absoluteFilePath(const QString &fileName) const
         return fileName;
 
     QString ret;
+#ifndef QT_NO_FSFILEENGINE
     if (isRelativePath(d->data->path)) //get pwd
         ret = QFSFileEngine::currentPath(fileName);
+#endif
     if(!d->data->path.isEmpty() && d->data->path != QLatin1String(".")) {
         if (!ret.isEmpty() && !ret.endsWith(QLatin1Char('/')))
             ret += QLatin1Char('/');
@@ -768,13 +759,7 @@ QString QDir::absoluteFilePath(const QString &fileName) const
 /*!
     Returns the path to \a fileName relative to the directory.
 
-    \code
-        QDir dir("/home/bob");
-        QString s;
-
-        s = dir.relativePath("images/file.jpg");     // s is "images/file.jpg"
-        s = dir.relativePath("/home/mary/file.txt"); // s is "../mary/file.txt"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 7
 
     \sa absoluteFilePath() filePath() canonicalPath()
 */
@@ -797,7 +782,9 @@ QString QDir::relativeFilePath(const QString &fileName) const
     }
 
 #ifdef Q_OS_WIN
-    if (fileDrive.toLower() != dirDrive.toLower())
+    if (fileDrive.toLower() != dirDrive.toLower()
+        || (file.startsWith(QLatin1String("//"))
+        && !dir.startsWith(QLatin1String("//"))))
 #else
     if (fileDrive != dirDrive)
 #endif
@@ -862,8 +849,8 @@ QString QDir::toNativeSeparators(const QString &pathName)
     QString n(pathName);
 #if defined(Q_FS_FAT) || defined(Q_OS_OS2EMX)
     for (int i=0; i<(int)n.length(); i++) {
-        if (n[i] == '/')
-            n[i] = '\\';
+        if (n[i] == QLatin1Char('/'))
+            n[i] = QLatin1Char('\\');
     }
 #endif
     return n;
@@ -873,7 +860,7 @@ QString QDir::toNativeSeparators(const QString &pathName)
     \since 4.2
 
     Returns \a pathName using '/' as file separator. On Windows,
-    for instance, fromNativeSeparators("c:\\winnt\\system32") returns
+    for instance, fromNativeSeparators("\c{c:\\winnt\\system32}") returns
     "c:/winnt/system32".
 
     The returned string may be the same as the argument on some
@@ -886,8 +873,8 @@ QString QDir::fromNativeSeparators(const QString &pathName)
     QString n(pathName);
 #if defined(Q_FS_FAT) || defined(Q_OS_OS2EMX)
     for (int i=0; i<(int)n.length(); i++) {
-        if (n[i] == '\\')
-            n[i] = '/';
+        if (n[i] == QLatin1Char('\\'))
+            n[i] = QLatin1Char('/');
     }
 #endif
     return n;
@@ -989,9 +976,7 @@ QStringList QDir::nameFilters() const
     to ensure that only files with extensions typically used for C++
     source files are listed:
 
-    \quotefromfile snippets/qdir-namefilters/main.cpp
-    \skipto QStringList
-    \printuntil setNameFilters
+    \snippet doc/src/snippets/qdir-namefilters/main.cpp 0
 
     \sa nameFilters(), setFilter()
 */
@@ -1004,10 +989,13 @@ void QDir::setNameFilters(const QStringList &nameFilters)
 }
 
 /*!
+    \obsolete
     Adds \a path to the search paths searched in to find resources
     that are not specified with an absolute path. The default search
     path is to search only in the root (\c{:/}).
 
+    Use QDir::addSearchPath() with a prefix instead.
+
     \sa {The Qt Resource System}, QResource::addSearchPath()
 */
 
@@ -1020,6 +1008,80 @@ void QDir::addResourceSearchPath(const QString &path)
 #endif
 }
 
+#ifdef QT_BUILD_CORE_LIB
+/*!
+    \since 4.3
+
+    Sets or replaces Qt's search paths for file names with the prefix \a prefix
+    to \a searchPaths.
+
+    To specify a prefix for a file name, prepend the prefix followed by a single
+    colon (e.g., "images:undo.png", "xmldocs:books.xml"). \a prefix can only
+    contain letters or numbers (e.g., it cannot contain a colon, nor a slash).
+
+    Qt uses this search path to locate files with a known prefix. The search
+    path entries are tested in order, starting with the first entry.
+
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 8
+
+    File name prefix must be at least 2 characters long to avoid conflicts with
+    Windows drive letters.
+
+    Search paths may contain paths to \l{The Qt Resource System}.
+*/
+void QDir::setSearchPaths(const QString &prefix, const QStringList &searchPaths)
+{
+    if (prefix.length() < 2) {
+        qWarning("QDir::setSearchPaths: Prefix must be longer than 1 character");
+        return;
+    }
+
+    for (int i = 0; i < prefix.count(); i++) {
+        if (!prefix.at(i).isLetterOrNumber()) {
+            qWarning("QDir::setSearchPaths: Prefix can only contain letters or numbers");
+            return;
+        }
+    }
+
+    QWriteLocker lock(&QCoreGlobalData::instance()->dirSearchPathsLock);
+    QMap<QString, QStringList> &paths = QCoreGlobalData::instance()->dirSearchPaths;
+    if (searchPaths.isEmpty()) {
+        paths.remove(prefix);
+    } else {
+        paths.insert(prefix, searchPaths);
+    }
+}
+
+/*!
+    \since 4.3
+
+    Adds \a path to the search path for \a prefix.
+
+    \sa setSearchPaths()
+*/
+void QDir::addSearchPath(const QString &prefix, const QString &path)
+{
+    if (path.isEmpty())
+        return;
+
+    QWriteLocker lock(&QCoreGlobalData::instance()->dirSearchPathsLock);
+    QCoreGlobalData::instance()->dirSearchPaths[prefix] += path;
+}
+
+/*!
+    \since 4.3
+
+    Returns the search paths for \a prefix.
+
+    \sa setSearchPaths(), addSearchPath()
+*/
+QStringList QDir::searchPaths(const QString &prefix)
+{
+    QReadLocker lock(&QCoreGlobalData::instance()->dirSearchPathsLock);
+    return QCoreGlobalData::instance()->dirSearchPaths.value(prefix);
+}
+
+#endif // QT_BUILD_CORE_LIB
 
 /*!
     Returns the value set by setFilter()
@@ -1043,7 +1105,7 @@ QDir::Filters QDir::filter() const
     \value Dirs    List directories that match the filters.
     \value AllDirs  List all directories; i.e. don't apply the filters
                     to directory names.
-    \value Files   List files only.
+    \value Files   List files.
     \value Drives  List disk drives (ignored under Unix).
     \value NoSymLinks  Do not list symbolic links (ignored by operating
                        systems that don't support symbolic links).
@@ -1064,8 +1126,7 @@ QDir::Filters QDir::filter() const
     \value Hidden  List hidden files (on Unix, files starting with a .).
     \value System  List system files (on Unix, FIFOs, sockets and
                    device files)
-    \value CaseSensitive  The filter should be case sensitive if the file system
-                          is case sensitive.
+    \value CaseSensitive  The filter should be case sensitive.
 
     \omitvalue DefaultFilter
     \omitvalue TypeMask
@@ -1294,6 +1355,9 @@ QStringList QDir::entryList(const QStringList &nameFilters, Filters filters,
         return d->data->files;
     }
     QStringList l = d->data->fileEngine->entryList(filters, nameFilters);
+    if ((sort & QDir::SortByMask) == QDir::Unsorted)
+        return l;
+
     QStringList ret;
     d->sortFileList(sort, l, &ret, 0);
     return ret;
@@ -1479,8 +1543,11 @@ bool QDir::exists() const
 
     if(!d->data->fileEngine)
         return false;
-    const QAbstractFileEngine::FileFlags info = d->data->fileEngine->fileFlags(QAbstractFileEngine::DirectoryType
-                                                                       |QAbstractFileEngine::ExistsFlag);
+    const QAbstractFileEngine::FileFlags info =
+        d->data->fileEngine->fileFlags(
+            QAbstractFileEngine::DirectoryType
+            | QAbstractFileEngine::ExistsFlag
+            | QAbstractFileEngine::Refresh);
     if(!(info & QAbstractFileEngine::DirectoryType))
         return false;
     return info & QAbstractFileEngine::ExistsFlag;
@@ -1494,12 +1561,7 @@ bool QDir::exists() const
     this function returns false. If you want to test for this use
     canonicalPath(), e.g.
 
-    \code
-        QDir dir("/tmp/root_link");
-        dir = dir.canonicalPath();
-        if (dir.isRoot())
-            qWarning("It is a root link");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 9
 
     \sa root(), rootPath()
 */
@@ -1581,13 +1643,7 @@ bool QDir::makeAbsolute() // ### What do the return values signify?
 
     Example:
 
-    \code
-        // The current directory is "/usr/local"
-        QDir d1("/usr/local/bin");
-        QDir d2("bin");
-        if (d1 == d2)
-            qDebug("They're the same");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 10
 */
 
 bool QDir::operator==(const QDir &dir) const
@@ -1654,13 +1710,7 @@ QDir &QDir::operator=(const QString &path)
 
     Example:
 
-    \code
-        // The current directory is "/usr/local"
-        QDir d1("/usr/local/bin");
-        QDir d2("bin");
-        if (d1 != d2)
-            qDebug("They differ");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 11
 */
 
 
@@ -1711,8 +1761,9 @@ bool QDir::rename(const QString &oldName, const QString &newName)
 }
 
 /*!
-    Returns true if the directory called \a name exists; otherwise returns
-    false.
+    Returns true if the file called \a name exists; otherwise returns
+    false. Unless \a name contains an absolute file path, the file
+    name is assumed to be relative to the current directory.
 
     \sa QFileInfo::exists(), QFile::exists()
 */
@@ -1739,7 +1790,11 @@ bool QDir::exists(const QString &name) const
 
 QFileInfoList QDir::drives()
 {
+#ifdef QT_NO_FSFILEENGINE
+    return QFileInfoList();
+#else
     return QFSFileEngine::drives();
+#endif
 }
 
 /*!
@@ -1776,7 +1831,12 @@ QChar QDir::separator()
 
 bool QDir::setCurrent(const QString &path)
 {
+#ifdef QT_NO_FSFILEENGINE
+    Q_UNUSED(path);
+    return false;
+#else
     return QFSFileEngine::setCurrentPath(path);
+#endif
 }
 
 /*!
@@ -1795,12 +1855,24 @@ bool QDir::setCurrent(const QString &path)
 
     \sa current(), homePath(), rootPath(), tempPath()
 */
-
 QString QDir::currentPath()
 {
+#ifdef QT_NO_FSFILEENGINE
+    return QString();
+#else
     return QFSFileEngine::currentPath();
+#endif
 }
 
+/*!
+  \fn QString QDir::currentDirPath()
+    Returns the absolute path of the application's current directory.
+
+    Use currentPath() instead.
+
+    \sa currentPath()
+*/
+
 /*!
     \fn QDir QDir::home()
 
@@ -1820,9 +1892,7 @@ QString QDir::currentPath()
     Under Windows this function will return the directory of the
     current user's profile. Typically, this is:
 
-    \code
-        C:/Documents and Settings/Username
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 12
 
     Use the toNativeSeparators() function to convert the separators to
     the ones that are appropriate for the underlying operating system.
@@ -1847,12 +1917,25 @@ QString QDir::currentPath()
 
     \sa home(), currentPath(), rootPath(), tempPath()
 */
-
 QString QDir::homePath()
 {
-    return QFSFileEngine::homePath();
+#ifdef QT_NO_FSFILEENGINE
+    return QString();
+#else
+    return cleanPath(QFSFileEngine::homePath());
+#endif
 }
 
+/*!
+  \fn QString QDir::homeDirPath()
+
+  Returns the absolute path of the user's home directory.
+
+  Use homePath() instead.
+
+  \sa homePath()
+ */
+
 /*!
     \fn QDir QDir::temp()
 
@@ -1878,7 +1961,11 @@ QString QDir::homePath()
 */
 QString QDir::tempPath()
 {
-    return QFSFileEngine::tempPath();
+#ifdef QT_NO_FSFILEENGINE
+    return QString();
+#else
+    return cleanPath(QFSFileEngine::tempPath());
+#endif
 }
 
 /*!
@@ -1902,12 +1989,25 @@ QString QDir::tempPath()
 
     \sa root(), drives(), currentPath(), homePath(), tempPath()
 */
-
 QString QDir::rootPath()
 {
+#ifdef QT_NO_FSFILEENGINE
+    return QString();
+#else
     return QFSFileEngine::rootPath();
+#endif
 }
 
+/*!
+  \fn QString QDir::rootDirPath()
+
+  Returns the absolute path of the root directory.
+
+  Use rootPath() instead.
+
+  \sa rootPath()
+*/
+
 #ifndef QT_NO_REGEXP
 /*!
     \overload
@@ -1973,7 +2073,7 @@ QString QDir::cleanPath(const QString &path)
     for(int i = 0, last = -1, iwrite = 0; i < len; i++) {
         if(p[i] == QLatin1Char('/')) {
             while(i < len-1 && p[i+1] == QLatin1Char('/')) {
-#ifdef Q_OS_WIN //allow unc paths
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) //allow unc paths
                 if(!i)
                     break;
 #endif
@@ -2018,6 +2118,12 @@ QString QDir::cleanPath(const QString &path)
                             iwrite = last;
                             last = -1;
                         }
+                    } else if (dotcount == 2 && i > 0 && p[i - 1] != QLatin1Char('.')) {
+                        eaten = true;
+                        used -= iwrite - qMax(0, last);
+                        iwrite = qMax(0, last);
+                        last = -1;
+                        ++i;
                     } else if(dotcount == 1) {
                         eaten = true;
                     }
@@ -2123,27 +2229,13 @@ QStringList QDir::nameFiltersFromString(const QString &nameFilter)
     initialized at startup by adding this line to your \c main()
     function:
 
-    \code
-        Q_INIT_RESOURCE(myapp);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 13
 
     Note: This macro cannot be used in a namespace. It should be called from
     main(). If that is not possible, the following workaround can be used
     to init the resource \c myapp from the function \c{MyNamespace::myFunction}:
 
-    \code
-    inline void initMyResource() { Q_INIT_RESOURCE(myapp); }
-
-    namespace MyNamespace
-    {
-        ...
-
-        void myFunction()
-        {
-            initMyResource();
-        }
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 14
 
     \sa Q_CLEANUP_RESOURCE(), {The Qt Resource System}
 */
@@ -2166,9 +2258,7 @@ QStringList QDir::nameFiltersFromString(const QString &nameFilter)
 
     Example:
 
-    \code
-        Q_CLEANUP_RESOURCE(myapp);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdir.cpp 15
 
     \sa Q_INIT_RESOURCE(), {The Qt Resource System}
 */
@@ -2307,8 +2397,8 @@ void QDir::setNameFilter(const QString &nameFilter)
 
     Use QDir::SortFlags instead.
 */
-
-#ifndef QT_NO_DEBUG
+#endif
+#ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug debug, QDir::Filters filters)
 {
     QStringList flags;
@@ -2347,7 +2437,7 @@ QDebug operator<<(QDebug debug, QDir::SortFlags sorting)
 
         QStringList flags;
         if (sorting & QDir::DirsFirst) flags << QLatin1String("DirsFirst");
-        if (sorting & QDir::DirsLast) flags << QLatin1String("DirsLars");
+        if (sorting & QDir::DirsLast) flags << QLatin1String("DirsLast");
         if (sorting & QDir::IgnoreCase) flags << QLatin1String("IgnoreCase");
         if (sorting & QDir::LocaleAware) flags << QLatin1String("LocaleAware");
         if (sorting & QDir::Type) flags << QLatin1String("Type");
@@ -2371,6 +2461,8 @@ QDebug operator<<(QDebug debug, const QDir &dir)
     return debug.space();
 }
 
-#endif
+
 
 #endif
+
+QT_END_NAMESPACE
