@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -52,18 +57,7 @@
     argument. After construction, the iterator is located before the first
     directory entry. Here's how to iterate over all the entries sequentially:
 
-    \code
-        QDirIterator it("/etc", QDirIterator::Subdirectories);
-        while (it.hasNext()) {
-            qDebug() << it.next();
-
-            // /etc/.
-            // /etc/..
-            // /etc/X11
-            // /etc/X11/fs
-            // ...
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qdiriterator.cpp 0
 
     The next() function returns the path to the next directory entry and
     advances the iterator. You can also call filePath() to get the current
@@ -105,6 +99,8 @@
 #include <QtCore/qstack.h>
 #include <QtCore/qvariant.h>
 
+QT_BEGIN_NAMESPACE
+
 class QDirIteratorPrivate
 {
 public:
@@ -115,6 +111,7 @@ public:
     void pushSubDirectory(const QString &path, const QStringList &nameFilters,
                           QDir::Filters filters);
     void advance();
+    bool shouldFollowDirectory(const QFileInfo &);
     bool matchesFilters(const QAbstractFileEngineIterator *it) const;
 
     QSet<QString> visitedLinks;
@@ -210,69 +207,74 @@ void QDirIteratorPrivate::advance()
         pushSubDirectory(subDir, it->nameFilters(), it->filters());
     }
 
-    if (fileEngineIterators.isEmpty())
-        done = true;
-
-    bool foundValidEntry = false;
     while (!fileEngineIterators.isEmpty()) {
         QAbstractFileEngineIterator *it = fileEngineIterators.top();
 
         // Find the next valid iterator that matches the filters.
-        foundValidEntry = false;
+        bool foundDirectory = false;
         while (it->hasNext()) {
             it->next();
             if (matchesFilters(it)) {
-                foundValidEntry = true;
+                fileInfo = it->currentFileInfo();
+                // Signal that we want to follow this entry.
+                followNextDir = shouldFollowDirectory(fileInfo);
+                
+                //We found a matching entry.
+                return;
+
+            } else if (iteratorFlags & QDirIterator::Subdirectories) {
+                QFileInfo fileInfo = it->currentFileInfo();
+                if (!shouldFollowDirectory(fileInfo))
+                    continue;
+                QString subDir = it->currentFilePath();
+#ifdef Q_OS_WIN
+                if (fileInfo.isSymLink())
+                    subDir = fileInfo.canonicalFilePath();
+#endif
+                pushSubDirectory(subDir, it->nameFilters(), it->filters());
+                
+                foundDirectory = true;
                 break;
             }
         }
+        if (!foundDirectory)
+            delete fileEngineIterators.pop();
+    }
+    done = true;
+}
 
-        if (!foundValidEntry) {
-            // If this iterator is done, pop and delete it, and continue
-            // iteration on the parent. Otherwise break, we're done.
-            if (!fileEngineIterators.isEmpty()) {
-                delete it;
-                it = fileEngineIterators.pop();
-                continue;
-            }
-            break;
-        }
+/*!
+    \internal
+ */
+bool QDirIteratorPrivate::shouldFollowDirectory(const QFileInfo &fileInfo)
+{
+    // If we're doing flat iteration, we're done.
+    if (!(iteratorFlags & QDirIterator::Subdirectories))
+        return false;
+    
+    // Never follow non-directory entries
+    if (!fileInfo.isDir())
+        return false;
 
-        fileInfo = it->currentFileInfo();
 
-        // If we're doing flat iteration, we're done.
-        if (!(iteratorFlags & QDirIterator::Subdirectories))
-            break;
+    // Never follow . and ..
+    if (fileInfo.fileName() == QLatin1String(".") || fileInfo.fileName() == QLatin1String(".."))
+        return false;
 
-        // Subdirectory iteration.
-        QString filePath = fileInfo.filePath();
-
-        // Never follow . and ..
-        if (fileInfo.fileName() == QLatin1String(".") || fileInfo.fileName() == QLatin1String(".."))
-            break;
-
-        // Never follow non-directory entries
-        if (!fileInfo.isDir())
-            break;
       
-        // Check symlinks
-        if (fileInfo.isSymLink() && !(iteratorFlags & QDirIterator::FollowSymlinks)) {
-            // Follow symlinks only if FollowSymlinks was passed
-            break;
-        }
-
-        // Stop link loops
-        if (visitedLinks.contains(fileInfo.canonicalFilePath()))
-            break;
-
-        // Signal that we want to follow this entry.
-        followNextDir = true;
-        break;
+    // Check symlinks
+    if (fileInfo.isSymLink() && !(iteratorFlags & QDirIterator::FollowSymlinks)) {
+        // Follow symlinks only if FollowSymlinks was passed
+        return false;
     }
 
-    if (!foundValidEntry)
-        done = true;
+    // Stop link loops
+    if (visitedLinks.contains(fileInfo.canonicalFilePath()))
+        return false;
+    
+    return true;
 }
+    
 
 /*!
     \internal
@@ -483,8 +485,6 @@ bool QDirIterator::hasNext() const
     if (d->first) {
         d->first = false;
         d->advance();
-        if (!d->fileEngineIterators.isEmpty())
-            d->currentFilePath = d->fileEngineIterators.top()->currentFilePath();
     }
     return !d->done;
 }
@@ -527,7 +527,7 @@ QString QDirIterator::filePath() const
 */
 QFileInfo QDirIterator::fileInfo() const
 {
-    if (d->fileInfo.path() != d->currentFilePath)
+    if (d->fileInfo.filePath() != d->currentFilePath)
         d->fileInfo.setFile(d->currentFilePath);
     return d->fileInfo;
 }
@@ -539,3 +539,5 @@ QString QDirIterator::path() const
 {
     return d->path;
 }
+
+QT_END_NAMESPACE
