@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -59,6 +59,8 @@
 #include "private/qiodevice_p.h"
 #include "private/qringbuffer_p.h"
 
+QT_BEGIN_NAMESPACE
+
 class QFilePrivate : public QIODevicePrivate
 {
     Q_DECLARE_PUBLIC(QFile)
@@ -90,4 +92,6 @@ private:
     static QFile::DecoderFn decoder;
 };
 
+QT_END_NAMESPACE
+
 #endif // QFILE_P_H
