@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,11 +45,14 @@
 #include "qfileinfo.h"
 #include "qdatetime.h"
 #include "qabstractfileengine.h"
+#include "qfsfileengine_p.h"
 #include "qglobal.h"
 #include "qatomic.h"
 #include "qhash.h"
 #include "qdir.h"
 
+QT_BEGIN_NAMESPACE
+
 class QFileInfoPrivate
 {
 public:
@@ -38,11 +61,18 @@ public:
 
     void initFileEngine(const QString &);
 
+    enum Access {
+        ReadAccess,
+        WriteAccess,
+        ExecuteAccess
+    };
+    bool hasAccess(Access access) const;
+
     uint getFileFlags(QAbstractFileEngine::FileFlags) const;
     QDateTime &getFileTime(QAbstractFileEngine::FileTime) const;
     QString getFileName(QAbstractFileEngine::FileName) const;
 
-    enum { CachedFileFlags=0x01, CachedLinkTypeFlag=0x02,
+    enum { CachedFileFlags=0x01, CachedLinkTypeFlag=0x02, CachedBundleTypeFlag=0x04,
            CachedMTime=0x10, CachedCTime=0x20, CachedATime=0x40,
            CachedSize =0x08 };
     struct Data {
@@ -59,7 +89,7 @@ public:
             fileFlags = 0;
             cachedFlags = 0;
         }
-        mutable QAtomic ref;
+        mutable QAtomicInt ref;
 
         QAbstractFileEngine *fileEngine;
         mutable QString fileName;
@@ -111,6 +141,53 @@ QFileInfoPrivate::initFileEngine(const QString &file)
     data->fileName = file;
 }
 
+bool QFileInfoPrivate::hasAccess(Access access) const
+{
+    if (!(data->fileEngine->fileFlags() & QAbstractFileEngine::LocalDiskFlag)) {
+        switch (access) {
+        case ReadAccess:
+            return getFileFlags(QAbstractFileEngine::ReadUserPerm);
+        case WriteAccess:
+            return getFileFlags(QAbstractFileEngine::WriteUserPerm);
+        case ExecuteAccess:
+            return getFileFlags(QAbstractFileEngine::ExeUserPerm);
+        default:
+            return false;
+        }
+    }
+
+    int mode = 0;
+    switch (access) {
+    case ReadAccess:
+        mode = R_OK;
+        break;
+    case WriteAccess:
+        mode = W_OK;
+        break;
+    case ExecuteAccess:
+        mode = X_OK;
+        break;
+    };
+#ifdef Q_OS_UNIX
+    return QT_ACCESS(QFile::encodeName(data->fileName).data(), mode) == 0;
+#endif
+#ifdef Q_OS_WIN
+    if ((access == ReadAccess && !getFileFlags(QAbstractFileEngine::ReadUserPerm))
+        || (access == WriteAccess && !getFileFlags(QAbstractFileEngine::WriteUserPerm))) {
+        return false;
+    }
+    if (access == ExecuteAccess)
+        return getFileFlags(QAbstractFileEngine::ExeUserPerm);
+
+    QT_WA( {
+        return ::_waccess((TCHAR *)QFSFileEnginePrivate::longFileName(data->fileName).utf16(), mode) == 0;
+    } , {
+        return QT_ACCESS(QFSFileEnginePrivate::win95Name(data->fileName), mode) == 0;
+    } );
+#endif
+    return false;
+}
+
 void QFileInfoPrivate::detach()
 { qAtomicDetach(data); }
 
@@ -128,14 +205,17 @@ QFileInfoPrivate::getFileName(QAbstractFileEngine::FileName name) const
 uint
 QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
 {
-    // we split the testing for LinkType and the rest because, in order to
-    // determine if a file is a symlink or not, we have to lstat(). If we're not
-    // interested in that information, we might as well avoid one extra syscall.
+    // We split the testing into tests for for LinkType, BundleType and the rest.
+    // In order to determine if a file is a symlink or not, we have to lstat().
+    // If we're not interested in that information, we might as well avoid one
+    // extra syscall. Bundle detecton on Mac can be slow, expecially on network
+    // paths, so we separate out that as well.
 
     QAbstractFileEngine::FileFlags flags;
     if (!data->getCachedFlag(CachedFileFlags)) {
         QAbstractFileEngine::FileFlags req = QAbstractFileEngine::FileInfoAll;
         req &= (~QAbstractFileEngine::LinkType);
+        req &= (~QAbstractFileEngine::BundleType);
 
         flags = data->fileEngine->fileFlags(req);
         data->setCachedFlag(CachedFileFlags);
@@ -154,6 +234,18 @@ QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
             flags |= linkflag;
         }
     }
+
+    if (request & QAbstractFileEngine::BundleType) {
+        if (!data->getCachedFlag(CachedBundleTypeFlag)) {
+            QAbstractFileEngine::FileFlags bundleflag;
+            bundleflag = data->fileEngine->fileFlags(QAbstractFileEngine::BundleType);
+
+            data->setCachedFlag(CachedBundleTypeFlag);
+            data->fileFlags |= uint(bundleflag);
+            flags |= bundleflag;
+        }
+    }
+
     // no else branch
     // if we had it cached, it was caught in the previous else branch
 
@@ -192,6 +284,7 @@ QDateTime
     \brief The QFileInfo class provides system-independent file information.
 
     \ingroup io
+    \ingroup shared
 
     QFileInfo provides information about a file's name and position
     (path) in the file system, its access rights and whether it is a
@@ -215,15 +308,8 @@ QDateTime
     later with setFile(). Use exists() to see if the file exists and
     size() to get its size.
 
-    To speed up performance, QFileInfo caches information about the
-    file. Because files can be changed by other users or programs, or
-    even by other parts of the same program, there is a function that
-    refreshes the file information: refresh(). If you want to switch
-    off a QFileInfo's caching and force it to access the file system
-    every time you request information from it call setCaching(false).
-
     The file's type is obtained with isFile(), isDir() and
-    isSymLink(). The readLink() function provides the name of the file
+    isSymLink(). The symLinkTarget() function provides the name of the file
     the symlink points to.
 
     On Unix (including Mac OS X), the symlink has the same size() has
@@ -231,48 +317,22 @@ QDateTime
     transparently; similarly, opening a symlink using QFile
     effectively opens the link's target. For example:
 
-    \code
-        #ifdef Q_OS_UNIX
-
-        QFileInfo info1("/home/bob/bin/untabify");
-        info1.isSymLink();          // returns true
-        info1.absoluteFilePath();   // returns "/home/bob/bin/untabify"
-        info1.size();               // returns 56201
-        info1.readLink();           // returns "/opt/pretty++/bin/untabify"
-
-        QFileInfo info2(info1.readLink());
-        info1.isSymLink();          // returns false
-        info1.absoluteFilePath();   // returns "/opt/pretty++/bin/untabify"
-        info1.size();               // returns 56201
-
-        #endif
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 0
 
     On Windows, symlinks (shortcuts) are \c .lnk files. The reported
     size() is that of the symlink (not the link's target), and
     opening a symlink using QFile opens the \c .lnk file. For
     example:
 
-    \code
-        #ifdef Q_OS_WIN
-
-        QFileInfo info1("C:\\Documents and Settings\\Bob\\untabify.lnk");
-        info1.isSymLink();          // returns true
-        info1.absoluteFilePath();   // returns "C:\\Documents and Settings\\Bob\\untabify.lnk"
-        info1.size();               // returns 743
-        info1.readLink();           // returns "C:\\Pretty++\\untabify"
-
-        QFileInfo info2(info1.readLink());
-        info1.isSymLink();          // returns false
-        info1.absoluteFilePath();   // returns "C:\\Pretty++\\untabify"
-        info1.size();               // returns 63942
-
-        #endif
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 1
 
     Elements of the file's name can be extracted with path() and
     fileName(). The fileName()'s parts can be extracted with
-    baseName() and extension().
+    baseName(), suffix() or completeSuffix(). QFileInfo objects to
+    directories created by Qt classes will not have a trailing file
+    separator. If you wish to use trailing separators in your own file
+    info objects, just append one to the file name given to the constructors
+    or setFile().
 
     The file's dates are returned by created(), lastModified() and
     lastRead(). Information about the file's access permissions is
@@ -281,6 +341,25 @@ QDateTime
     groupId(). You can examine a file's permissions and ownership in a
     single statement using the permission() function.
 
+    \section1 Performance Issues
+
+    Some of QFileInfo's functions query the file system, but for
+    performance reasons, some functions only operate on the
+    file name itself. For example: To return the absolute path of
+    a relative file name, absolutePath() has to query the file system.
+    The path() function, however, can work on the file name directly,
+    and so it is faster.
+
+    \note To speed up performance, QFileInfo caches information about
+    the file.
+
+    To speed up performance, QFileInfo caches information about the
+    file. Because files can be changed by other users or programs, or
+    even by other parts of the same program, there is a function that
+    refreshes the file information: refresh(). If you want to switch
+    off a QFileInfo's caching and force it to access the file system
+    every time you request information from it call setCaching(false).
+
     \sa QDir, QFile
 */
 
@@ -380,6 +459,8 @@ bool
 QFileInfo::operator==(const QFileInfo &fileinfo) const
 {
     Q_D(const QFileInfo);
+    // ### Qt 5: understand long and short file names on Windows
+    // ### (GetFullPathName()).
     if(fileinfo.d_func()->data == d->data)
         return true;
     if(!d->data->fileEngine || !fileinfo.d_func()->data->fileEngine)
@@ -413,6 +494,9 @@ QFileInfo::operator==(const QFileInfo &fileinfo) const
     \warning This will not compare two different symbolic links
     pointing to the same file.
 
+    \warning Long and short file names that refer to the same file on Windows
+    are treated as if they referred to different files.
+
     \sa operator!=()
 */
 bool QFileInfo::operator==(const QFileInfo &fileinfo)
@@ -442,19 +526,7 @@ QFileInfo &QFileInfo::operator=(const QFileInfo &fileinfo)
     path relative to the current directory.
 
     Example:
-    \code
-    QString absolute = "/local/bin";
-    QString relative = "local/bin";
-    QFileInfo absFile(absolute);
-    QFileInfo relFile(relative);
-
-    QDir::setCurrent(QDir::rootPath());
-    // absFile and relFile now point to the same file
-
-    QDir::setCurrent("/tmp");
-    // absFile now points to "/local/bin",
-    // while relFile points to "/tmp/local/bin"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 2
 
     \sa isRelative(), QDir::setCurrent(), QDir::isRelativePath()
 */
@@ -636,9 +708,9 @@ QFileInfo::isRelative() const
 
 
 /*!
-    Converts the file's path to an absolute path.
-
-    If it is already absolute, nothing is done.
+    Converts the file's path to an absolute path if it is not already in that form.
+    Returns true to indicate that the path was converted; otherwise returns false
+    to indicate that the path was already absolute.
 
     \sa filePath(), isRelative()
 */
@@ -670,6 +742,9 @@ QFileInfo::exists() const
 /*!
     Refreshes the information about the file, i.e. reads in information
     from the file system the next time a cached property is fetched.
+
+   \note On Windows CE, there might be a delay for the file system driver
+    to detect changes on the file.
 */
 
 void
@@ -699,10 +774,7 @@ QFileInfo::filePath() const
     Returns the name of the file, excluding the path.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        QString name = fi.fileName();                // name = "archive.tar.gz"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 3
 
     \sa isRelative(), filePath(), baseName(), extension()
 */
@@ -716,6 +788,28 @@ QFileInfo::fileName() const
     return d->getFileName(QAbstractFileEngine::BaseName);
 }
 
+/*!
+    \since 4.3
+    Returns the name of the bundle.
+
+    On Mac OS X this returns the proper localized name for a bundle if the
+    path isBundle(). On all other platforms an empty QString is returned.
+
+    Example:
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 4
+
+    \sa isBundle(), filePath(), baseName(), extension()
+*/
+
+QString
+QFileInfo::bundleName() const
+{
+    Q_D(const QFileInfo);
+    if(!d->data->fileEngine)
+        return QLatin1String("");
+    return d->getFileName(QAbstractFileEngine::BundleName);
+}
+
 /*!
     Returns the base name of the file without the path.
 
@@ -723,10 +817,12 @@ QFileInfo::fileName() const
     not including) the \e first '.' character.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        QString base = fi.baseName();  // base = "archive"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 5
+
+
+    The base name of a file is computed equally on all platforms, independent
+    of file naming conventions (e.g., ".bashrc" on Unix has an empty base
+    name, and the suffix is "bashrc").
 
     \sa fileName(), suffix(), completeSuffix(), completeBaseName()
 */
@@ -747,10 +843,7 @@ QFileInfo::baseName() const
     to (but not including) the \e last '.' character.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        QString base = fi.completeBaseName();  // base = "archive.tar"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 6
 
     \sa fileName(), suffix(), completeSuffix(), baseName()
 */
@@ -773,10 +866,7 @@ QFileInfo::completeBaseName() const
     (but not including) the first '.'.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        QString ext = fi.completeSuffix();  // ext = "tar.gz"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 7
 
     \sa fileName(), suffix(), baseName(), completeBaseName()
 */
@@ -801,10 +891,11 @@ QFileInfo::completeSuffix() const
     including) the last '.'.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        QString ext = fi.suffix();  // ext = "gz"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 8
+
+    The suffix of a file is computed equally on all platforms, independent of
+    file naming conventions (e.g., ".bashrc" on Unix has an empty base name,
+    and the suffix is "bashrc").
 
     \sa fileName(), completeSuffix(), baseName(), completeBaseName()
 */
@@ -835,6 +926,7 @@ QFileInfo::suffix() const
 QDir
 QFileInfo::dir() const
 {
+    // ### Qt5: Maybe rename this to parentDirectory(), considering what it actually do?
     return QDir(path());
 }
 
@@ -875,7 +967,7 @@ QFileInfo::isReadable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::ReadUserPerm);
+    return d->hasAccess(QFileInfoPrivate::ReadAccess);
 }
 
 /*!
@@ -891,7 +983,7 @@ QFileInfo::isWritable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::WriteUserPerm);
+    return d->hasAccess(QFileInfoPrivate::WriteAccess);
 }
 
 /*!
@@ -906,11 +998,14 @@ QFileInfo::isExecutable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::ExeUserPerm);
+    return d->hasAccess(QFileInfoPrivate::ExecuteAccess);
 }
 
 /*!
     Returns true if this is a `hidden' file; otherwise returns false.
+
+    \bold{Note:} This function returns true for the special entries
+    "." and ".." on Unix, even though QDir::entryList threats them as shown.
 */
 bool
 QFileInfo::isHidden() const
@@ -926,7 +1021,7 @@ QFileInfo::isHidden() const
     link to a file. Returns false if the
     object points to something which isn't a file, such as a directory.
 
-    \sa isDir(), isSymLink()
+    \sa isDir(), isSymLink(), isBundle()
 */
 
 bool
@@ -942,7 +1037,7 @@ QFileInfo::isFile() const
     Returns true if this object points to a directory or to a symbolic
     link to a directory; otherwise returns false.
 
-    \sa isFile(), isSymLink()
+    \sa isFile(), isSymLink(), isBundle()
 */
 
 bool
@@ -954,23 +1049,37 @@ QFileInfo::isDir() const
     return d->getFileFlags(QAbstractFileEngine::DirectoryType);
 }
 
+
+/*!
+    \since 4.3
+    Returns true if this object points to a bundle or to a symbolic
+    link to a bundle on Mac OS X; otherwise returns false.
+
+    \sa isDir(), isSymLink(), isFile()
+*/
+
+bool
+QFileInfo::isBundle() const
+{
+    Q_D(const QFileInfo);
+    if(!d->data->fileEngine)
+        return false;
+    return d->getFileFlags(QAbstractFileEngine::BundleType);
+}
+
 /*!
     Returns true if this object points to a symbolic link (or to a
     shortcut on Windows); otherwise returns false.
 
     On Unix (including Mac OS X), opening a symlink effectively opens
-    the \l{readLink()}{link's target}. On Windows, it opens the \c
+    the \l{symLinkTarget()}{link's target}. On Windows, it opens the \c
     .lnk file itself.
 
     Example:
 
-    \code
-        QFileInfo info(fileName);
-        if (info.isSymLink())
-            fileName = info.readLink();
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 9
 
-    \sa isFile(), isDir(), readLink()
+    \sa isFile(), isDir(), symLinkTarget()
 */
 
 bool
@@ -998,6 +1107,9 @@ QFileInfo::isRoot() const
 }
 
 /*!
+    \fn QString QFileInfo::symLinkTarget() const
+    \since 4.2
+
     Returns the absolute path to the file or directory a symlink (or shortcut
     on Windows) points to, or a an empty string if the object isn't a symbolic
     link.
@@ -1009,6 +1121,11 @@ QFileInfo::isRoot() const
     \sa exists(), isSymLink(), isDir(), isFile()
 */
 
+/*!
+    \obsolete
+
+    Use symLinkTarget() instead.
+*/
 QString
 QFileInfo::readLink() const
 {
@@ -1103,13 +1220,7 @@ QFileInfo::groupId() const
     always returns true.
 
     Example:
-    \code
-        QFileInfo fi("/tmp/archive.tar.gz");
-        if (fi.permission(QFile::WriteUser | QFile::ReadGroup))
-            qWarning("I can change the file; my group can read the file");
-        if (fi.permission(QFile::WriteGroup | QFile::WriteOther))
-            qWarning("The group or others can change the file");
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qfileinfo.cpp 10
 
     \sa isReadable(), isWritable(), isExecutable()
 */
@@ -1139,8 +1250,10 @@ QFileInfo::permissions() const
 
 
 /*!
-    Returns the file size in bytes, or 0 if the file does not exist or
-    if the size is 0 or if the size cannot be fetched.
+    Returns the file size in bytes. If the file does not exist or cannot be
+    fetched, 0 is returned.
+
+    \sa exists()
 */
 
 qint64
@@ -1275,6 +1388,13 @@ QFileInfo::setCaching(bool enable)
     Use absoluteFilePath() instead.
 */
 
+/*!
+    \fn QString QFileInfo::dirPath(bool absPath) const
+
+    Use absoluteFilePath() if the absolute path is wanted (\a absPath
+    is true) or path() if it's not necessary (\a absPath is false).
+*/
+
 /*!
     \fn bool QFileInfo::convertToAbs()
 
@@ -1306,3 +1426,12 @@ QFileInfo::setCaching(bool enable)
 
     Use permission() instead.
 */
+
+/*!
+    \typedef QFileInfoList
+    \relates QFileInfo
+
+    Synonym for QList<QFileInfo>.
+*/
+
+QT_END_NAMESPACE
