@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,7 @@
 
 #include <qdatetime.h>
 #include <qdebug.h>
+#include <qdir.h>
 #include <qfileinfo.h>
 #include <qmutex.h>
 #include <qset.h>
@@ -57,10 +58,13 @@
 #  include "qfilesystemwatcher_win_p.h"
 #elif defined(Q_OS_LINUX)
 #  include "qfilesystemwatcher_inotify_p.h"
+#  include "qfilesystemwatcher_dnotify_p.h"
 #elif defined(Q_OS_FREEBSD) || defined(Q_OS_MAC)
 #  include "qfilesystemwatcher_kqueue_p.h"
 #endif
 
+QT_BEGIN_NAMESPACE
+
 enum { PollingInterval = 1000 };
 
 class QPollingFileSystemWatcherEngine : public QFileSystemWatcherEngine
@@ -73,6 +77,7 @@ class QPollingFileSystemWatcherEngine : public QFileSystemWatcherEngine
         uint groupId;
         QFile::Permissions permissions;
         QDateTime lastModified;
+        QStringList entries;
 
     public:
         FileInfo(const QFileInfo &fileInfo)
@@ -80,7 +85,11 @@ class QPollingFileSystemWatcherEngine : public QFileSystemWatcherEngine
               groupId(fileInfo.groupId()),
               permissions(fileInfo.permissions()),
               lastModified(fileInfo.lastModified())
-        { }
+        { 
+            if (fileInfo.isDir()) {
+                entries = fileInfo.absoluteDir().entryList(QDir::AllEntries);
+            }
+        }
         FileInfo &operator=(const QFileInfo &fileInfo)
         {
             *this = FileInfo(fileInfo);
@@ -89,6 +98,8 @@ class QPollingFileSystemWatcherEngine : public QFileSystemWatcherEngine
 
         bool operator!=(const QFileInfo &fileInfo) const
         {
+            if (fileInfo.isDir() && entries != fileInfo.absoluteDir().entryList(QDir::AllEntries))
+                return true;
             return (ownerId != fileInfo.ownerId()
                     || groupId != fileInfo.groupId()
                     || permissions != fileInfo.permissions()
@@ -115,7 +126,9 @@ private slots:
 
 QPollingFileSystemWatcherEngine::QPollingFileSystemWatcherEngine()
 {
+#ifndef QT_NO_THREAD
     moveToThread(this);
+#endif
 }
 
 void QPollingFileSystemWatcherEngine::run()
@@ -211,6 +224,7 @@ void QPollingFileSystemWatcherEngine::timeout()
             x.value() = fi;
             emit directoryChanged(path, false);
         }
+        
     }
 }
 
@@ -222,7 +236,10 @@ QFileSystemWatcherEngine *QFileSystemWatcherPrivate::createNativeEngine()
 #if defined(Q_OS_WIN)
     return new QWindowsFileSystemWatcherEngine;
 #elif defined(Q_OS_LINUX)
-    return QInotifyFileSystemWatcherEngine::create();
+    QFileSystemWatcherEngine *eng = QInotifyFileSystemWatcherEngine::create();
+    if(!eng)
+        eng = QDnotifyFileSystemWatcherEngine::create();
+    return eng;
 #elif defined(Q_OS_FREEBSD) || defined(Q_OS_MAC)
     return QKqueueFileSystemWatcherEngine::create();
 #else
@@ -231,7 +248,7 @@ QFileSystemWatcherEngine *QFileSystemWatcherPrivate::createNativeEngine()
 }
 
 QFileSystemWatcherPrivate::QFileSystemWatcherPrivate()
-    : native(0), poller(0)
+    : native(0), poller(0), forced(0)
 {
 }
 
@@ -251,6 +268,52 @@ void QFileSystemWatcherPrivate::init()
     }
 }
 
+void QFileSystemWatcherPrivate::initForcedEngine(const QString &forceName)
+{
+    if(forced)
+        return;
+
+    Q_Q(QFileSystemWatcher);
+
+#if defined(Q_OS_LINUX)
+    if(forceName == QLatin1String("inotify")) {
+        forced = QInotifyFileSystemWatcherEngine::create();
+    } else if(forceName == QLatin1String("dnotify")) {
+        forced = QDnotifyFileSystemWatcherEngine::create();
+    }
+#else
+    Q_UNUSED(forceName);
+#endif
+
+    if(forced) {
+        QObject::connect(forced,
+                         SIGNAL(fileChanged(QString,bool)),
+                         q,
+                         SLOT(_q_fileChanged(QString,bool)));
+        QObject::connect(forced,
+                         SIGNAL(directoryChanged(QString,bool)),
+                         q,
+                         SLOT(_q_directoryChanged(QString,bool)));
+    }
+}
+
+void QFileSystemWatcherPrivate::initPollerEngine()
+{
+    if(poller)
+        return;
+
+    Q_Q(QFileSystemWatcher);
+    poller = new QPollingFileSystemWatcherEngine; // that was a mouthful
+    QObject::connect(poller,
+                     SIGNAL(fileChanged(QString,bool)),
+                     q,
+                     SLOT(_q_fileChanged(QString,bool)));
+    QObject::connect(poller,
+                     SIGNAL(directoryChanged(QString,bool)),
+                     q,
+                     SLOT(_q_directoryChanged(QString,bool)));
+}
+
 void QFileSystemWatcherPrivate::_q_fileChanged(const QString &path, bool removed)
 {
     Q_Q(QFileSystemWatcher);
@@ -301,6 +364,12 @@ void QFileSystemWatcherPrivate::_q_directoryChanged(const QString &path, bool re
     Note that QFileSystemWatcher stops monitoring files and directories
     once they have been removed from disk.
 
+    \bold{Note:} On systems running a Linux kernel without inotify support,
+    file systems that contain watched paths cannot be unmounted.
+
+    \note Windows CE does not support directory monitoring by
+    default as this depends on the file system driver installed.
+
     \sa QFile, QDir
 */
 
@@ -343,6 +412,12 @@ QFileSystemWatcher::~QFileSystemWatcher()
         delete d->poller;
         d->poller = 0;
     }
+    if (d->forced) {
+        d->forced->stop();
+        d->forced->wait();
+        delete d->forced;
+        d->forced = 0;
+    }
 }
 
 /*!
@@ -385,32 +460,39 @@ void QFileSystemWatcher::addPaths(const QStringList &paths)
         qWarning("QFileSystemWatcher::addPaths: list is empty");
         return;
     }
+
     QStringList p = paths;
-    if (objectName() != QLatin1String("_qt_autotest_force_engine_poller")) {
-        if (d->native)
-            p = d->native->addPaths(p, &d->files, &d->directories);
-        if (p.isEmpty())
-            return;
+    QFileSystemWatcherEngine *engine = 0;
+
+    if(!objectName().startsWith(QLatin1String("_qt_autotest_force_engine_"))) {
+        // Normal runtime case - search intelligently for best engine
+        if(d->native) {
+            engine = d->native;
+        } else {
+            d_func()->initPollerEngine();
+            engine = d->poller;
+        }
+
     } else {
-        qDebug() << "QFileSystemWatcher: skipping native engine, using only polling engine";
-    }
-    if (objectName() != QLatin1String("_qt_autotest_force_engine_native")) {
-        // try polling instead
-        if (!d->poller) {
-            d->poller = new QPollingFileSystemWatcherEngine; // that was a mouthful
-            QObject::connect(d->poller,
-                             SIGNAL(fileChanged(QString,bool)),
-                             this,
-                             SLOT(_q_fileChanged(QString,bool)));
-            QObject::connect(d->poller,
-                             SIGNAL(directoryChanged(QString,bool)),
-                             this,
-                             SLOT(_q_directoryChanged(QString,bool)));
+        // Autotest override case - use the explicitly selected engine only
+        QString forceName = objectName().mid(26);
+        if(forceName == QLatin1String("poller")) {
+            qDebug() << "QFileSystemWatcher: skipping native engine, using only polling engine";
+            d_func()->initPollerEngine();
+            engine = d->poller;
+        } else if(forceName == QLatin1String("native")) {
+            qDebug() << "QFileSystemWatcher: skipping polling engine, using only native engine";
+            engine = d->native;
+        } else {
+            qDebug() << "QFileSystemWatcher: skipping polling and native engine, using only explicit" << forceName << "engine";
+            d_func()->initForcedEngine(forceName);
+            engine = d->forced;
         }
-        p = d->poller->addPaths(p, &d->files, &d->directories);
-    } else{
-        qDebug("QFileSystemWatcher: skipping polling engine, using only native engine");
     }
+
+    if(engine)
+        p = engine->addPaths(p, &d->files, &d->directories);
+
     if (!p.isEmpty())
         qWarning("QFileSystemWatcher: failed to add paths: %s",
                  qPrintable(p.join(QLatin1String(", "))));
@@ -446,7 +528,9 @@ void QFileSystemWatcher::removePaths(const QStringList &paths)
     if (d->native)
         p = d->native->removePaths(p, &d->files, &d->directories);
     if (d->poller)
-        (void) d->poller->removePaths(p, &d->files, &d->directories);
+        p = d->poller->removePaths(p, &d->files, &d->directories);
+    if (d->forced)
+        p = d->forced->removePaths(p, &d->files, &d->directories);
 }
 
 /*!
@@ -498,7 +582,11 @@ QStringList QFileSystemWatcher::files() const
     return d->files;
 }
 
+QT_END_NAMESPACE
+
 #include "moc_qfilesystemwatcher.cpp"
+
 #include "qfilesystemwatcher.moc"
 
 #endif // QT_NO_FILESYSTEMWATCHER
+
