@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,10 +45,17 @@
 #include "qfsfileengine_iterator_p.h"
 #include "qdatetime.h"
 #include "qdiriterator.h"
+#include "qset.h"
 
+#ifndef QT_NO_FSFILEENGINE
+
+#if !defined(Q_OS_WINCE)
 #include <errno.h>
+#endif
 #include <stdio.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifdef Q_OS_WIN
 #  ifndef S_ISREG
 #    define S_ISREG(x)   (((x) & S_IFMT) == S_IFREG)
@@ -114,6 +121,60 @@ void QFSFileEnginePrivate::init()
     fileAttrib = INVALID_FILE_ATTRIBUTES;
     fileHandle = INVALID_HANDLE_VALUE;
 #endif
+#ifdef Q_USE_DEPRECATED_MAP_API
+    fileMapHandle = INVALID_HANDLE_VALUE;
+#endif
+}
+
+/*!
+    \internal
+
+    Returns the canonicalized form of \a path (i.e., with all symlinks
+    resolved, and all redundant path elements removed.
+*/
+QString QFSFileEnginePrivate::canonicalized(const QString &path)
+{
+    if (path.isEmpty())
+        return path;
+
+    QFileInfo fi;
+    const QChar slash(QLatin1Char('/'));
+    QString tmpPath = path;
+    int separatorPos = 0;
+    QSet<QString> nonSymlinks;
+    QSet<QString> known;
+
+    do {
+#ifdef Q_OS_WIN
+        // UNC, skip past the first two elements
+        if (separatorPos == 0 && tmpPath.startsWith(QLatin1String("//")))
+            separatorPos = tmpPath.indexOf(slash, 2);
+        if (separatorPos != -1)
+#endif
+        separatorPos = tmpPath.indexOf(slash, separatorPos + 1);
+        QString prefix = separatorPos == -1 ? tmpPath : tmpPath.left(separatorPos);
+        if (!nonSymlinks.contains(prefix)) {
+            if (known.contains(prefix))
+                return QString();
+            known.insert(prefix);
+
+            fi.setFile(prefix);
+            if (fi.isSymLink()) {
+                QString target = fi.symLinkTarget();
+                if (separatorPos != -1) {
+                    if (fi.isDir() && !target.endsWith(slash))
+                        target.append(slash);
+                    target.append(tmpPath.mid(separatorPos));
+                }
+                tmpPath = target;
+                separatorPos = 0;
+            } else {
+                nonSymlinks.insert(prefix);
+            }
+        }
+    } while (separatorPos != -1);
+
+    return QDir::cleanPath(tmpPath);
 }
 
 /*!
@@ -160,6 +221,9 @@ QFSFileEngine::~QFSFileEngine()
             } while (ret == -1 && errno == EINTR);
         }
     }
+    QList<uchar*> keys = d->maps.keys();
+    for (int i = 0; i < keys.count(); ++i)
+        unmap(keys.at(i));
 }
 
 /*!
@@ -579,6 +643,7 @@ qint64 QFSFileEnginePrivate::readFdFh(char *data, qint64 len)
     if (len) {
         int result;
         qint64 read = 0;
+        errno = 0;
 
         // Read in blocks of 4k to avoid platform limitations (Windows
         // commonly bails out if you read or write too large blocks at once).
@@ -595,7 +660,7 @@ qint64 QFSFileEnginePrivate::readFdFh(char *data, qint64 len)
         // if an error occurred.
         if (read > 0) {
             ret += read;
-        } else {
+        } else if (read == 0 && result < 0) {
             ret = -1;
             q->setError(QFile::ReadError, qt_error_string(errno));
         }
@@ -642,8 +707,9 @@ qint64 QFSFileEnginePrivate::readLineFdFh(char *data, qint64 maxlen)
     // does the same, so we'd get two '\0' at the end - passing maxlen + 1
     // solves this.
     if (!fgets(data, int(maxlen + 1), fh)) {
-        q->setError(QFile::ReadError, qt_error_string(int(errno)));
-        return 0;
+        if (!feof(fh))
+            q->setError(QFile::ReadError, qt_error_string(int(errno)));
+        return -1;              // error
     }
 
 #ifdef Q_OS_WIN
@@ -771,8 +837,17 @@ bool QFSFileEngine::extension(Extension extension, const ExtensionOption *option
     if (extension == AtEndExtension && d->fh && isSequential())
         return feof(d->fh);
 
-    Q_UNUSED(option);
-    Q_UNUSED(output);
+    if (extension == MapExtension) {
+        const MapExtensionOption *options = (MapExtensionOption*)(option);
+        MapExtensionReturn *returnValue = static_cast<MapExtensionReturn*>(output);
+        returnValue->address = d->map(options->offset, options->size, options->flags);
+        return (returnValue->address != 0);
+    }
+    if (extension == UnMapExtension) {
+        UnMapExtensionOption *options = (UnMapExtensionOption*)option;
+        return d->unmap(options->address);
+    }
+
     return false;
 }
 
@@ -788,5 +863,11 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
         return true;
     if (extension == FastReadLineExtension && d->fd != -1 && isSequential())
         return true;
+    if (extension == UnMapExtension || extension == MapExtension)
+        return true;
     return false;
 }
+
+QT_END_NAMESPACE
+
+#endif // QT_NO_FSFILEENGINE
