@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,6 +50,8 @@
 #include "qstringlist.h"
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifdef QIODEVICE_DEBUG
 void debugBinaryString(const QByteArray &input)
 {
@@ -80,7 +82,9 @@ void debugBinaryString(const char *data, qint64 maxlen)
 }
 #endif
 
-static const qint64 QIODEVICE_BUFFERSIZE = 16384;
+#ifndef QIODEVICE_BUFFERSIZE
+#define QIODEVICE_BUFFERSIZE Q_INT64_C(16384)
+#endif
 
 #define Q_VOID
 
@@ -206,18 +210,7 @@ QIODevicePrivate::~QIODevicePrivate()
     Calling these functions from the main, GUI thread, may cause your
     user interface to freeze. Example:
 
-    \code
-        QProcess gzip;
-        gzip.start("gzip", QStringList() << "-c");
-        if (!gzip.waitForStarted())
-            return false;
-
-        gzip.write("uncompressed data");
-
-        QByteArray compressed;
-        while (gzip.waitForReadyRead())
-            compressed += gzip.readAll();
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 0
 
     By subclassing QIODevice, you can provide the same interface to
     your own I/O devices. Subclasses of QIODevice are only required to
@@ -318,6 +311,11 @@ QIODevicePrivate::~QIODevicePrivate()
     the signal will not be reemitted (although waitForReadyRead() may still
     return true).
 
+    Note for developers implementing classes derived from QIODevice:
+    you should always emit readyRead() when new data has arrived (do not
+    emit it only because there's data still to be read in your
+    buffers). Do not emit readyRead() in other conditions.
+
     \sa bytesWritten()
 */
 
@@ -329,6 +327,18 @@ QIODevicePrivate::~QIODevicePrivate()
     buffer that needs to be written to the device).
 */
 
+/*!
+    \fn QIODevice::readChannelFinished()
+    \since 4.4
+
+    This signal is emitted when the input (reading) stream is closed
+    in this device. It is emitted as soon as the closing is detected,
+    which means that there might still be data available for reading
+    with read().
+
+    \sa atEnd(), read()
+*/
+
 #ifdef QT_NO_QOBJECT
 QIODevice::QIODevice()
     : d_ptr(new QIODevicePrivate)
@@ -517,7 +527,7 @@ bool QIODevice::open(OpenMode mode)
     Q_D(QIODevice);
     d->openMode = mode;
     d->pos = (mode & Append) ? size() : qint64(0);
-    d_func()->accessMode = QIODevicePrivate::Unset;
+    d->accessMode = QIODevicePrivate::Unset;
 #if defined QIODEVICE_DEBUG
     printf("%p QIODevice::open(0x%x)\n", this, quint32(mode));
 #endif
@@ -691,12 +701,7 @@ bool QIODevice::reset()
     Subclasses that reimplement this function must call the base
     implementation in order to include the size of QIODevices' buffer. Example:
 
-    \code
-        qint64 CustomDevice::bytesAvailable() const
-        {
-            return buffer.size() + QIODevice::bytesAvailable();
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 1
 
     \sa bytesToWrite(), readyRead(), isSequential()
 */
@@ -726,7 +731,10 @@ qint64 QIODevice::bytesToWrite() const
     attempting to read from a device opened in WriteOnly mode, this
     function returns -1.
 
-    0 is returned when no more data is available for reading.
+    0 is returned when no more data is available for reading. However,
+    reading past the end of the stream is considered an error, so this
+    function returns -1 in those cases (that is, reading on a closed
+    socket or after a process has died).
 
     \sa readData() readLine() write()
 */
@@ -819,8 +827,12 @@ qint64 QIODevice::read(char *data, qint64 maxSize)
                 return qint64(-1);
             qint64 readFromDevice = readData(data + readSoFar, maxSize - readSoFar);
 #if defined QIODEVICE_DEBUG
-            printf("%p \treading %d bytes from device\n", this, int(readFromDevice));
+            printf("%p \treading %d bytes from device (total %d)\n", this, int(readFromDevice), int(readSoFar));
 #endif
+            if (readFromDevice == -1 && readSoFar == 0) {
+                // error and we haven't read anything: return immediately
+                return -1;
+            }
             if (readFromDevice <= 0) {
                 moreToRead = false;
             } else {
@@ -954,9 +966,12 @@ QByteArray QIODevice::readAll()
 }
 
 /*!
-    This function reads a line of ASCII characters from the device, up to a
-    maximum of \a maxSize - 1 bytes, stores the characters in \a data, and
-    returns the number of bytes read. If an error occurred, -1 is returned.
+    This function reads a line of ASCII characters from the device, up
+    to a maximum of \a maxSize - 1 bytes, stores the characters in \a
+    data, and returns the number of bytes read. If a line could not be
+    read but no error ocurred, this function returns 0. If an error
+    occurs, this function returns what it could the length of what
+    could be read, or -1 if nothing was read.
 
     A terminating '\0' byte is always appended to \a data, so \a
     maxSize must be larger than 1.
@@ -972,16 +987,7 @@ QByteArray QIODevice::readAll()
     For example, the following code reads a line of characters from a
     file:
 
-    \code
-        QFile file("box.txt");
-        if (file.open(QFile::ReadOnly)) {
-            char buf[1024];
-            qint64 lineLength = file.readLine(buf, sizeof(buf));
-            if (lineLength != -1) {
-                // the line is available in buf
-            }
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 2
 
     The newline character ('\n') is included in the buffer. If a
     newline is not encountered before maxSize - 1 bytes are read, a
@@ -1048,7 +1054,7 @@ qint64 QIODevice::readLine(char *data, qint64 maxSize)
         debugBinaryString(data, int(readSoFar + readBytes));
     }
 #endif
-    if (readBytes <= 0) {
+    if (readBytes < 0) {
         data[readSoFar] = '\0';
         return readSoFar ? readSoFar : -1;
     }
@@ -1117,7 +1123,10 @@ QByteArray QIODevice::readLine(qint64 maxSize)
              readSoFar + 1 == tmp.size() &&   // +1 due to the ending null
              tmp.at(readSoFar - 1) != '\n');
 
-    tmp.resize(int(readSoFar));
+    if (readSoFar == 0 && readBytes == -1)
+        tmp.clear();            // return Null if we found an error
+    else
+        tmp.resize(int(readSoFar));
     return tmp;
 }
 
@@ -1131,15 +1140,22 @@ QByteArray QIODevice::readLine(qint64 maxSize)
 
     readLine() appends a '\0' byte to \a data; readLineData() does not
     need to do this.
+
+    If you reimplement this function, be careful to return the correct
+    value: it should return the number of bytes read in this line,
+    including the terminating newline, or 0 if there is no line to be
+    read at this point. If an error occurs, it should return -1 if and
+    only if no bytes were read. Reading past EOF is considered an error.
 */
 qint64 QIODevice::readLineData(char *data, qint64 maxSize)
 {
+    Q_D(QIODevice);
     qint64 readSoFar = 0;
     char c;
-    bool lastGetSucceeded = false;
-    d_func()->baseReadLineDataCalled = true;
+    int lastReadReturn = 0;
+    d->baseReadLineDataCalled = true;
 
-    while (readSoFar < maxSize && (lastGetSucceeded = getChar(&c))) {
+    while (readSoFar < maxSize && (lastReadReturn = read(&c, 1)) == 1) {
         *data++ = c;
         ++readSoFar;
         if (c == '\n')
@@ -1147,12 +1163,11 @@ qint64 QIODevice::readLineData(char *data, qint64 maxSize)
     }
 
 #if defined QIODEVICE_DEBUG
-    Q_D(QIODevice);
     printf("%p QIODevice::readLineData(%p, %d), d->pos = %d, d->buffer.size() = %d, returns %d\n",
            this, data, int(maxSize), int(d->pos), int(d->buffer.size()), int(readSoFar));
 #endif
-    if (!lastGetSucceeded && readSoFar == 0)
-        return qint64(-1);
+    if (lastReadReturn != 1 && readSoFar == 0)
+        return isSequential() ? lastReadReturn : -1;
     return readSoFar;
 }
 
@@ -1169,12 +1184,7 @@ qint64 QIODevice::readLineData(char *data, qint64 maxSize)
     Subclasses that reimplement this function must call the base
     implementation in order to include the size of the QIODevice's buffer. Example:
 
-    \code
-        bool CustomDevice::canReadLine() const
-        {
-            return buffer.contains('\n') || QIODevice::canReadLine();
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 3
 
     \sa readyRead(), readLine()
 */
@@ -1376,15 +1386,7 @@ bool QIODevice::getChar(char *c)
 
     Example:
 
-    \code
-        bool isExeFile(QFile *file)
-        {
-            char buf[2];
-            if (file->peek(buf, sizeof(buf)) == sizeof(buf))
-                return (buf[0] == 'M' && buf[1] == 'Z');
-            return false;
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 4
 
     \sa read()
 */
@@ -1406,12 +1408,7 @@ qint64 QIODevice::peek(char *data, qint64 maxSize)
 
     Example:
 
-    \code
-        bool isExeFile(QFile *file)
-        {
-            return file->peek(2) == "MZ";
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.io.qiodevice.cpp 5
 
     This function has no way of reporting errors; returning an empty
     QByteArray() can mean either that no data was currently available
@@ -1524,7 +1521,10 @@ QString QIODevice::errorString() const
     \fn qint64 QIODevice::readData(char *data, qint64 maxSize)
 
     Reads up to \a maxSize bytes from the device into \a data, and
-    returns the number of bytes read or -1 if an error occurred.
+    returns the number of bytes read or -1 if an error occurred. If
+    there are no bytes to be read, this function should return -1 if
+    there can never be more bytes available (for example: socket
+    closed, pipe closed, sub-process finished).
 
     This function is called by QIODevice. Reimplement this function
     when creating a subclass of QIODevice.
@@ -1722,3 +1722,5 @@ QDebug operator<<(QDebug debug, QIODevice::OpenMode modes)
     return debug;
 }
 #endif
+
+QT_END_NAMESPACE
