@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -71,6 +71,8 @@ typedef int Q_PIPE;
 
 #ifndef QT_NO_PROCESS
 
+QT_BEGIN_NAMESPACE
+
 class QSocketNotifier;
 class QWindowsPipeWriter;
 class QWinEventNotifier;
@@ -95,26 +97,8 @@ public:
             pipe[0] = INVALID_Q_PIPE;
             pipe[1] = INVALID_Q_PIPE;
         }
-        
-        void clear()
-        {
-            switch (type) {
-            case PipeSource:
-                Q_ASSERT(process);
-                process->stdinChannel.type = Normal;
-                process->stdinChannel.process = 0;
-                break;
-            case PipeSink:
-                Q_ASSERT(process);
-                process->stdoutChannel.type = Normal;
-                process->stdoutChannel.process = 0;
-                break;
-            }
 
-            type = Normal;
-            file.clear();
-            process = 0;
-        }
+        void clear();
 
         Channel &operator=(const QString &fileName)
         {
@@ -241,6 +225,8 @@ public:
 #endif
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_PROCESS
 
 #endif // QPROCESS_P_H
