@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -49,6 +54,7 @@
     Returns a human readable representation of the first \a len
     characters in \a data.
 */
+QT_BEGIN_NAMESPACE
 static QByteArray qt_prettyDebug(const char *data, int len, int maxSize)
 {
     if (!data) return "(null)";
@@ -73,6 +79,7 @@ static QByteArray qt_prettyDebug(const char *data, int len, int maxSize)
 
     return out;
 }
+QT_END_NAMESPACE
 #endif
 
 #include "qplatformdefs.h"
@@ -100,6 +107,8 @@ static QByteArray qt_prettyDebug(const char *data, int len, int maxSize)
 #include <stdlib.h>
 #include <string.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifdef Q_OS_INTEGRITY
 static inline char *strdup(const char *data)
 {
@@ -269,6 +278,16 @@ QProcessManager::~QProcessManager()
 
     qDeleteAll(children.values());
     children.clear();
+
+    struct sigaction oldAction;
+    struct sigaction action;
+    memset(&action, 0, sizeof(action));
+    action.sa_handler = qt_sa_old_sigchld_handler;
+    action.sa_flags = SA_NOCLDSTOP;
+    qt_native_sigaction(SIGCHLD, &action, &oldAction);
+    if (oldAction.sa_handler != qt_sa_sigchld_handler) {
+        qt_native_sigaction(SIGCHLD, &oldAction, 0);
+    }
 }
 
 void QProcessManager::run()
@@ -324,7 +343,7 @@ void QProcessManager::catchDeadChildren()
     }
 }
 
-static QBasicAtomic idCounter = Q_ATOMIC_INIT(1);
+static QBasicAtomicInt idCounter = Q_BASIC_ATOMIC_INITIALIZER(1);
 
 void QProcessManager::add(pid_t pid, QProcess *process)
 {
@@ -339,7 +358,7 @@ void QProcessManager::add(pid_t pid, QProcess *process)
     info->exitResult = 0;
     info->pid = pid;
 
-    int serial = idCounter.fetchAndAdd(1);
+    int serial = idCounter.fetchAndAddRelaxed(1);
     process->d_func()->serial = serial;
     children.insert(serial, info);
 }
@@ -548,6 +567,16 @@ static char **_q_dupEnvironment(const QStringList &environment, int *envc)
     return envp;
 }
 
+// under QNX RTOS we have to use vfork() when multithreading
+inline pid_t qt_fork()
+{
+#if defined(Q_OS_QNX)
+    return vfork();
+#else
+    return fork();
+#endif
+}
+
 void QProcessPrivate::startProcess()
 {
     Q_Q(QProcess);
@@ -583,8 +612,7 @@ void QProcessPrivate::startProcess()
         return;
 
     // Start the process (platform dependent)
-    processState = QProcess::Starting;
-    emit q->stateChanged(processState);
+    q->setProcessState(QProcess::Starting);
 
     // Create argument list with right number of elements, and set the final
     // one to 0.
@@ -648,7 +676,7 @@ void QProcessPrivate::startProcess()
                 pathc = pathEntries.size();
                 path = new char *[pathc + 1];
                 path[pathc] = 0;
-                
+
                 for (int k = 0; k < pathEntries.size(); ++k) {
                     QByteArray tmp = QFile::encodeName(pathEntries.at(k));
                     if (!tmp.endsWith('/')) tmp += '/';
@@ -661,7 +689,7 @@ void QProcessPrivate::startProcess()
 
     // Start the process manager, and fork off the child process.
     processManager()->lock();
-    pid_t childPid = fork();
+    pid_t childPid = qt_fork();
     if (childPid != 0) {
         // Clean up duplicated memory.
         free(dupProgramName);
@@ -678,8 +706,7 @@ void QProcessPrivate::startProcess()
     if (childPid < 0) {
         // Cleanup, report error and return
         processManager()->unlock();
-        processState = QProcess::NotRunning;
-        emit q->stateChanged(processState);
+        q->setProcessState(QProcess::NotRunning);
         processError = QProcess::FailedToStart;
         q->setErrorString(QLatin1String(QT_TRANSLATE_NOOP(QProcess, "Resource error (fork failure)")));
         emit q->error(processError);
@@ -798,7 +825,7 @@ bool QProcessPrivate::processStarted()
     int i = qt_native_read(childStartedPipe[0], &c, 1);
     if (startupSocketNotifier) {
         startupSocketNotifier->setEnabled(false);
-        delete startupSocketNotifier;
+        startupSocketNotifier->deleteLater();
         startupSocketNotifier = 0;
     }
     qt_native_close(childStartedPipe[0]);
@@ -857,8 +884,8 @@ qint64 QProcessPrivate::readFromStderr(char *data, qint64 maxlen)
 static void qt_ignore_sigpipe()
 {
     // Set to ignore SIGPIPE once only.
-    static QBasicAtomic atom = Q_ATOMIC_INIT(0);
-    if (atom.testAndSet(0, 1)) {
+    static QBasicAtomicInt atom = Q_BASIC_ATOMIC_INITIALIZER(0);
+    if (atom.testAndSetRelaxed(0, 1)) {
         struct sigaction noaction;
         memset(&noaction, 0, sizeof(noaction));
         noaction.sa_handler = SIG_IGN;
@@ -1188,7 +1215,10 @@ bool QProcessPrivate::waitForDeadChild()
 
     // check if our process is dead
     int exitStatus;
-    pid_t waitResult = waitpid(pid_t(pid), &exitStatus, WNOHANG);
+    pid_t waitResult = 0;
+    do {
+        waitResult = waitpid(pid_t(pid), &exitStatus, WNOHANG);
+    } while ((waitResult == -1 && errno == EINTR));
     if (waitResult > 0) {
         processManager()->remove(q);
         crashed = !WIFEXITED(exitStatus);
@@ -1224,7 +1254,7 @@ bool QProcessPrivate::startDetached(const QString &program, const QStringList &a
     int pidPipe[2];
     ::pipe(pidPipe);
 
-    pid_t childPid = fork();
+    pid_t childPid = qt_fork();
     if (childPid == 0) {
         struct sigaction noaction;
         memset(&noaction, 0, sizeof(noaction));
@@ -1236,7 +1266,7 @@ bool QProcessPrivate::startDetached(const QString &program, const QStringList &a
         qt_native_close(startedPipe[0]);
         qt_native_close(pidPipe[0]);
 
-        pid_t doubleForkPid = fork();
+        pid_t doubleForkPid = qt_fork();
         if (doubleForkPid == 0) {
             ::fcntl(startedPipe[1], F_SETFD, FD_CLOEXEC);
             qt_native_close(pidPipe[1]);
@@ -1312,7 +1342,8 @@ bool QProcessPrivate::startDetached(const QString &program, const QStringList &a
     int startResult = qt_native_read(startedPipe[0], &reply, 1);
     int result;
     qt_native_close(startedPipe[0]);
-    ::waitpid(childPid, &result, 0);
+    while (::waitpid(childPid, &result, 0) == -1 && errno == EINTR)
+    { }
     bool success = (startResult != -1 && reply == '\0');
     if (success && pid) {
         pid_t actualPid = 0;
@@ -1331,6 +1362,8 @@ void QProcessPrivate::initializeProcessManager()
     (void) processManager();
 }
 
+QT_END_NAMESPACE
+
 #include "qprocess_unix.moc"
 
 #endif // QT_NO_PROCESS
