@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,44 +48,25 @@
 #include <private/qthread_p.h>
 #include <private/qcoreapplication_p.h>
 
+QT_BEGIN_NAMESPACE
 
-static QBasicAtomic timerId = Q_ATOMIC_INIT(1);
-static int nextTimerId()
-{
-    register int id;
-    forever {
-        id = timerId;
-        if (timerId.testAndSet(id, id + 1))
-            break;
-    }
-    Q_ASSERT_X(id >= 1, "QTimer", "timer id overflow, please contact qt-bugs@trolltech.com");
-    return id;
-}
+static QBasicAtomicInt timerId = Q_BASIC_ATOMIC_INITIALIZER(1);
 
 
 void QAbstractEventDispatcherPrivate::init()
 {
     Q_Q(QAbstractEventDispatcher);
-    QThread *thread = q->thread();
-    if (!thread) {
-        if (!QCoreApplicationPrivate::eventDispatcher) {
-            QCoreApplicationPrivate::eventDispatcher = q;
-        } else {
-            Q_ASSERT_X(false, "QAbstractEventDispatcher",
-                       "An event dispatcher has already been created for the application");
-        }
+    if (threadData->eventDispatcher != 0) {
+        qWarning("QAbstractEventDispatcher: An event dispatcher has already been created for this thread");
     } else {
-        QThreadData *data = QThreadData::get(thread);
-        Q_ASSERT_X(!data->eventDispatcher, "QAbstractEventDispatcher",
-                   "An event dispatcher has already been created for this thread");
-        Q_UNUSED(data);
+        threadData->eventDispatcher = q;
     }
 }
 
 
 /*!
     \class QAbstractEventDispatcher
-    \brief The QAbstractEventDispatcher class manages Qt's event queue, excluding GUI-related events.
+    \brief The QAbstractEventDispatcher class provides an interface to manage Qt's event queue.
 
     \ingroup application
     \ingroup events
@@ -95,9 +96,10 @@ void QAbstractEventDispatcherPrivate::init()
     values to control which events should be delivered.
 
     QAbstractEventDispatcher also allows the integration of an
-    external event loop with the Qt event loop. For example, the Motif
-    Extension included with Qt includes a reimplementation of
-    QAbstractEventDispatcher that merges Qt and Motif events together.
+    external event loop with the Qt event loop. For example, the
+    \l{Qt Solutions}{Motif Extension Qt Solution} includes a
+    reimplementation of QAbstractEventDispatcher that merges Qt and
+    Motif events together.
 
     \sa QEventLoop, QCoreApplication
 */
@@ -140,14 +142,11 @@ QAbstractEventDispatcher::~QAbstractEventDispatcher()
  */
 QAbstractEventDispatcher *QAbstractEventDispatcher::instance(QThread *thread)
 {
-    if (!thread)
-        thread = QThread::currentThread();
-    if (!thread)
-        return 0;
-    return QThreadData::get(thread)->eventDispatcher;
+    QThreadData *data = thread ? QThreadData::get2(thread) : QThreadData::current();
+    return data->eventDispatcher;
 }
 
-/*! 
+/*!
     \fn bool QAbstractEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
 
     Processes pending events that match \a flags until there are no
@@ -210,7 +209,7 @@ QAbstractEventDispatcher *QAbstractEventDispatcher::instance(QThread *thread)
 */
 int QAbstractEventDispatcher::registerTimer(int interval, QObject *object)
 {
-    int id = nextTimerId();
+    int id = timerId.fetchAndAddRelaxed(1);
     registerTimer(id, interval, object);
     return id;
 }
@@ -226,12 +225,18 @@ int QAbstractEventDispatcher::registerTimer(int interval, QObject *object)
     \fn bool QAbstractEventDispatcher::unregisterTimer(int timerId)
 
     Unregisters the timer with the given \a timerId.
+    Returns true if successful; otherwise returns false.
+
+    \sa registerTimer(), unregisterTimers()
 */
 
 /*!
     \fn bool QAbstractEventDispatcher::unregisterTimers(QObject *object)
 
     Unregisters all the timers associated with the given \a object.
+    Returns true if all timers were successful removed; otherwise returns false.
+
+    \sa unregisterTimer(), registeredTimers()
 */
 
 /*!
@@ -249,7 +254,7 @@ int QAbstractEventDispatcher::registerTimer(int interval, QObject *object)
     \sa awake()
 */
 
-/*! 
+/*!
     \fn void QAbstractEventDispatcher::interrupt()
 
     Interrupts event dispatching; i.e. the event dispatcher will
@@ -287,9 +292,7 @@ void QAbstractEventDispatcher::closingDown()
 
     Typedef for a function with the signature
 
-    \code
-        bool myEventFilter(void *message);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qabstracteventdispatcher.cpp 0
 
     \sa setEventFilter(), filterEvent()
 */
@@ -354,3 +357,5 @@ bool QAbstractEventDispatcher::filterEvent(void *message)
 
     \sa awake()
 */
+
+QT_END_NAMESPACE
