@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,6 +59,8 @@
 #include "QtCore/qtranslator.h"
 #include "private/qobject_p.h"
 
+QT_BEGIN_NAMESPACE
+
 typedef QList<QTranslator*> QTranslatorList;
 
 class QAbstractEventDispatcher;
@@ -51,6 +73,8 @@ public:
     QCoreApplicationPrivate(int &aargc,  char **aargv);
     ~QCoreApplicationPrivate();
 
+    bool sendThroughApplicationEventFilters(QObject *, QEvent *);
+    bool sendThroughObjectEventFilters(QObject *, QEvent *);
     bool notify_helper(QObject *, QEvent *);
 
     virtual QString appName() const;
@@ -60,15 +84,19 @@ public:
     static void removePostedTimerEvent(QObject *object, int timerId);
 #endif
 
+#ifdef Q_OS_MAC
+    static QString macMenuBarName();
+#endif
+
     static QThread *theMainThread;
     static QThread *mainThread();
     static bool checkInstance(const char *method);
+    static void sendPostedEvents(QObject *receiver, int event_type, QThreadData *data);
+    static void removePostedEvents_unlocked(QObject *receiver, int type, QThreadData *data);
 
-#ifdef QT3_SUPPORT
-    void removePostedChildInsertedEvents(QObject *receiver, QObject *child);
-#endif
+#if !defined (QT_NO_DEBUG) || defined (QT_MAC_FRAMEWORK_BUILD)
     void checkReceiverThread(QObject *receiver);
-
+#endif
     int &argc;
     char **argv;
     void appendApplicationPathToLibraryPaths(void);
@@ -81,6 +109,8 @@ public:
     QCoreApplication::EventFilter eventFilter;
 
     bool in_exec;
+    QString cachedApplicationDirPath;
+    QString cachedApplicationFilePath;
 
     static bool isTranslatorInstalled(QTranslator *translator);
 
@@ -92,4 +122,6 @@ public:
     static inline bool testAttribute(uint flag) { return attribs & (1 << flag); }
 };
 
+QT_END_NAMESPACE
+
 #endif // QCOREAPPLICATION_P_H
