@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -45,6 +50,8 @@
 #include "qobject_p.h"
 #include <private/qthread_p.h>
 
+QT_BEGIN_NAMESPACE
+
 class QEventLoopPrivate : public QObjectPrivate
 {
     Q_DECLARE_PUBLIC(QEventLoop)
@@ -73,8 +80,9 @@ public:
     This enum controls the types of events processed by the
     processEvents() functions.
 
-    \value AllEvents All events except
-    \l{QEvent::DeferredDelete}{DeferredDelete} are processed.
+    \value AllEvents All events. Note that
+    \l{QEvent::DeferredDelete}{DeferredDelete} events are processed
+    specially. See QObject::deleteLater() for more details.
 
     \value ExcludeUserInputEvents Do not process user input events,
     such as ButtonPress and KeyPress. Note that the events are not
@@ -89,13 +97,10 @@ public:
     \value WaitForMoreEvents Wait for events if no pending events are
     available.
 
-    \value DeferredDeletion Allow objects to be queued for deletion
-    at a later time.
-
-    \value X11ExcludeTimers
-
+    \omitvalue X11ExcludeTimers
     \omitvalue ExcludeUserInput
     \omitvalue WaitForMore
+    \value DeferredDeletion deprecated - do not use.
 
     \sa processEvents()
 */
@@ -139,6 +144,8 @@ bool QEventLoop::processEvents(ProcessEventsFlags flags)
     Q_D(QEventLoop);
     if (!d->threadData->eventDispatcher)
         return false;
+    if (flags & DeferredDeletion)
+        QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
     return d->threadData->eventDispatcher->processEvents(flags);
 }
 
@@ -155,7 +162,7 @@ bool QEventLoop::processEvents(ProcessEventsFlags flags)
 
     Generally speaking, no user interaction can take place before
     calling exec(). As a special case, modal widgets like QMessageBox
-    can be used before calling exec(), because modal widgets 
+    can be used before calling exec(), because modal widgets
     use their own local event loop.
 
     To make your application perform idle processing (i.e. executing a
@@ -177,15 +184,20 @@ int QEventLoop::exec(ProcessEventsFlags flags)
     }
     d->inExec = true;
     d->exit = false;
+    ++d->threadData->loopLevel;
     d->threadData->eventLoops.push(this);
 
+    // remove posted quit events when entering a new event loop
+    if (qApp->thread() == thread())
+        QCoreApplication::removePostedEvents(qApp, QEvent::Quit);
+
 #if defined(QT_NO_EXCEPTIONS)
     while (!d->exit)
-        processEvents(flags | WaitForMoreEvents | ProcessEventsFlag(QEventLoop::DeferredDeletion));
+        processEvents(flags | WaitForMoreEvents);
 #else
     try {
         while (!d->exit)
-            processEvents(flags | WaitForMoreEvents | ProcessEventsFlag(QEventLoop::DeferredDeletion));
+            processEvents(flags | WaitForMoreEvents);
     } catch (...) {
         qWarning("Qt has caught an exception thrown from an event handler. Throwing\n"
                  "exceptions from an event handler is not supported in Qt. You must\n"
@@ -199,6 +211,7 @@ int QEventLoop::exec(ProcessEventsFlags flags)
     Q_UNUSED(eventLoop); // --release warning
 
     d->inExec = false;
+    --d->threadData->loopLevel;
 
     return d->returnCode;
 }
@@ -227,9 +240,13 @@ void QEventLoop::processEvents(ProcessEventsFlags flags, int maxTime)
 
     QTime start;
     start.start();
+    if (flags & DeferredDeletion)
+        QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
     while (processEvents(flags & ~WaitForMoreEvents)) {
         if (start.elapsed() > maxTime)
             break;
+        if (flags & DeferredDeletion)
+            QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
     }
 }
 
@@ -294,3 +311,5 @@ void QEventLoop::wakeUp()
 */
 void QEventLoop::quit()
 { exit(0); }
+
+QT_END_NAMESPACE
