@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,12 +54,15 @@
 #include <qvariant.h>
 #include <qhash.h>
 #include <qdebug.h>
+#include <qsemaphore.h>
 
 #include "private/qobject_p.h"
 #include "private/qmetaobject_p.h"
 
 #include <ctype.h>
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QMetaObject
 
@@ -107,7 +130,11 @@
 /*!
     \enum QMetaMethod::Access
 
-    \internal
+    This enum describes the access level of a method, following the conventions used in C++.
+
+    \value Private
+    \value Protected
+    \value Public
 */
 
 // do not touch without touching the moc as well
@@ -317,7 +344,7 @@ int QMetaObject::classInfoOffset() const
 
 /*!
     Returns the number of methods in this class. These include
-    ordinary methods, signals, and slots.
+    signals and slots.
 
     \sa method(), methodOffset(), indexOfMethod()
 */
@@ -658,17 +685,7 @@ QMetaProperty QMetaObject::userProperty() const
 
     Example:
 
-    \code
-        class MyClass
-        {
-            Q_OBJECT
-            Q_CLASSINFO("author", "Sabrina Schweinsteiger")
-            Q_CLASSINFO("url", "http://doc.moosesoft.co.uk/1.0/")
-
-        public:
-            ...
-        };
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 0
 
     \sa classInfoCount(), classInfoOffset(), indexOfClassInfo()
  */
@@ -720,8 +737,10 @@ static void qRemoveWhitespace(const char *s, char *d)
             last = *d++ = *s++;
         while (*s && is_space(*s))
             s++;
-        if (*s && is_ident_char(*s) && is_ident_char(last))
+        if (*s && ((is_ident_char(*s) && is_ident_char(last))
+                   || ((*s == ':') && (last == '<')))) {
             last = *d++ = ' ';
+        }
     }
     *d = '\0';
 }
@@ -754,10 +773,7 @@ static char *qNormalizeType(char *d, int &templdepth, QByteArray &result)
 
     Example:
 
-    \code
-    QByteArray normType = QMetaObject::normalizedType(" int    const  *");
-    // normType is now "const int*"
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 1
 
     \sa normalizedSignature()
  */
@@ -768,7 +784,7 @@ QByteArray QMetaObject::normalizedType(const char *type)
     if (!type || !*type)
         return result;
 
-    QVarLengthArray<char> stackbuf(strlen(type));
+    QVarLengthArray<char> stackbuf((int)strlen(type));
     qRemoveWhitespace(type, stackbuf.data());
     int templdepth = 0;
     qNormalizeType(stackbuf.data(), templdepth, result);
@@ -792,7 +808,7 @@ QByteArray QMetaObject::normalizedSignature(const char *method)
     QByteArray result;
     if (!method || !*method)
         return result;
-    int len = strlen(method);
+    int len = int(strlen(method));
     char stackbuf[64];
     char *buf = (len >= 64 ? new char[len+1] : stackbuf);
     qRemoveWhitespace(method, buf);
@@ -854,10 +870,7 @@ QByteArray QMetaObject::normalizedSignature(const char *method)
     \l{QPushButton::animateClick()}{animateClick()} slot on a
     QPushButton:
 
-    \code
-        QMetaObject::invokeMethod(pushButton, "animateClick",
-                                  Qt::QueuedConnection);
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 2
 
     With asynchronous method invocations, the parameters must be of
     types that are known to Qt's meta-object system, because Qt needs
@@ -865,9 +878,7 @@ QByteArray QMetaObject::normalizedSignature(const char *method)
     scenes. If you try to use a queued connection and get the error
     message
 
-    \code
-        QMetaObject::invokeMethod: Unable to handle unregistered datatype 'MyType'
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 3
 
     call qRegisterMetaType() to register the data type before you
     call invokeMethod().
@@ -875,14 +886,7 @@ QByteArray QMetaObject::normalizedSignature(const char *method)
     To synchronously invoke the \c compute(QString, int, double) slot on
     some arbitrary object \c obj retrieve its return value:
 
-    \code
-        QString retVal;
-        QMetaObject::invokeMethod(obj, "compute", Qt::DirectConnection,
-                                  Q_RETURN_ARG(QString, retVal),
-                                  Q_ARG(QString, "sqrt"),
-                                  Q_ARG(int, 42),
-                                  Q_ARG(double, 9.7));
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 4
 
     If the "compute" slot does not take exactly one QString, one int
     and one double in the specified order, the call will fail.
@@ -913,20 +917,20 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
     sig.append(member, len);
     sig.append('(');
 
-    enum { ParamCount = 11 };
+    enum { MaximumParamCount = 11 };
     const char *typeNames[] = {ret.name(), val0.name(), val1.name(), val2.name(), val3.name(),
                                val4.name(), val5.name(), val6.name(), val7.name(), val8.name(),
                                val9.name()};
 
-    int i;
-    for (i = 1; i < ParamCount; ++i) {
-        len = qstrlen(typeNames[i]);
+    int paramCount;
+    for (paramCount = 1; paramCount < MaximumParamCount; ++paramCount) {
+        len = qstrlen(typeNames[paramCount]);
         if (len <= 0)
             break;
-        sig.append(typeNames[i], len);
+        sig.append(typeNames[paramCount], len);
         sig.append(',');
     }
-    if (i == 1)
+    if (paramCount == 1)
         sig.append(')'); // no parameters
     else
         sig[sig.size() - 1] = ')';
@@ -970,7 +974,7 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
                : Qt::QueuedConnection;
     }
 
-    if (type != Qt::QueuedConnection) {
+    if (type == Qt::DirectConnection) {
         return obj->qt_metacall(QMetaObject::InvokeMetaMethod, idx, param) < 0;
     } else {
         if (ret.data()) {
@@ -979,11 +983,11 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
             return false;
         }
         int nargs = 1; // include return type
-        void **args = (void **) qMalloc(ParamCount * sizeof(void *));
-        int *types = (int *) qMalloc(ParamCount * sizeof(int));
+        void **args = (void **) qMalloc(paramCount * sizeof(void *));
+        int *types = (int *) qMalloc(paramCount * sizeof(int));
         types[0] = 0; // return type
         args[0] = 0;
-        for (i = 1; i < ParamCount; ++i) {
+        for (int i = 1; i < paramCount; ++i) {
             types[i] = QMetaType::type(typeNames[i]);
             if (types[i]) {
                 args[i] = QMetaType::construct(types[i], param[i]);
@@ -991,11 +995,34 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
             } else if (param[i]) {
                 qWarning("QMetaObject::invokeMethod: Unable to handle unregistered datatype '%s'",
                          typeNames[i]);
+                for (int x = 1; x < i; ++x) {
+                    if (types[x] && args[x])
+                        QMetaType::destroy(types[x], args[x]);
+                }
+                qFree(types);
+                qFree(args);
                 return false;
             }
         }
 
-        QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, -1, nargs, types, args));
+        if (type == Qt::QueuedConnection) {
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, nargs, types, args));
+        } else {
+            if (QThread::currentThread() == obj->thread()) {
+                qWarning("QMetaObject::invokeMethod: Dead lock detected in BlockingQueuedConnection: "
+                         "Receiver is %s(%p)",
+                         obj->metaObject()->className(), obj);
+            }
+
+            // blocking queued connection
+#ifdef QT_NO_THREAD
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, nargs, types, args));
+#else
+            QSemaphore semaphore;
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, nargs, types, args, &semaphore));
+            semaphore.acquire();
+#endif // QT_NO_THREAD
+        }
     }
     return true;
 }
@@ -1079,6 +1106,11 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
     \value Scriptable
 */
 
+/*!
+    \fn const QMetaObject *QMetaMethod::enclosingMetaObject() const
+    \internal
+*/
+
 /*!
     \enum QMetaMethod::MethodType
 
@@ -1256,6 +1288,12 @@ QMetaMethod::MethodType QMetaMethod::methodType() const
     \sa name()
 */
 
+/*!
+    \fn const QMetaObject *QMetaEnum::enclosingMetaObject() const
+    \internal
+*/
+
+
 /*!
     \fn QMetaEnum::QMetaEnum()
     \internal
@@ -1363,19 +1401,19 @@ int QMetaEnum::keyToValue(const char *key) const
 {
     if (!mobj || !key)
         return -1;
-    int scope = 0;
+    uint scope = 0;
     const char *qualified_key = key;
-    const char *s = key;
-    while (*s  && *s != ':')
-        ++s;
-    if (*s && *(s+1)==':') {
-        scope = s - key;
+    const char *s = key + qstrlen(key);
+    while (s > key && *s != ':')
+        --s;
+    if (s > key && *(s-1)==':') {
+        scope = s - key - 1;
         key += scope + 2;
     }
     int count = mobj->d.data[handle + 2];
     int data = mobj->d.data[handle + 3];
     for (int i = 0; i < count; ++i)
-        if ((!scope || strncmp(qualified_key, mobj->d.stringdata, scope) == 0)
+        if ((!scope || (qstrlen(mobj->d.stringdata) == scope && strncmp(qualified_key, mobj->d.stringdata, scope) == 0))
              && strcmp(key, mobj->d.stringdata + mobj->d.data[data + 2*i]) == 0)
             return mobj->d.data[data + 2*i + 1];
     return -1;
@@ -1421,17 +1459,17 @@ int QMetaEnum::keysToValue(const char *keys) const
         QString trimmed = l.at(li).trimmed();
         QByteArray qualified_key = trimmed.toLatin1();
         const char *key = qualified_key.constData();
-        int scope = 0;
-        const char *s = key;
-        while (*s  && *s != ':')
-            ++s;
-        if (*s && *(s+1)==':') {
-            scope = s - key;
+        uint scope = 0;
+        const char *s = key + qstrlen(key);
+        while (s > key && *s != ':')
+            --s;
+        if (s > key && *(s-1)==':') {
+            scope = s - key - 1;
             key += scope + 2;
         }
         int i;
         for (i = count-1; i >= 0; --i)
-            if ((!scope || strncmp(qualified_key.constData(), mobj->d.stringdata, scope) == 0)
+            if ((!scope || (qstrlen(mobj->d.stringdata) == scope && strncmp(qualified_key.constData(), mobj->d.stringdata, scope) == 0))
                  && strcmp(key, mobj->d.stringdata + mobj->d.data[data + 2*i]) == 0) {
                 value |= mobj->d.data[data + 2*i + 1];
                 break;
@@ -1456,7 +1494,7 @@ QByteArray QMetaEnum::valueToKeys(int value) const
     int count = mobj->d.data[handle + 2];
     int data = mobj->d.data[handle + 3];
     int v = value;
-    for(int i = count - 1; i >= 0; --i) {
+    for(int i = 0; i < count; i++) {
         int k = mobj->d.data[data + 2*i + 1];
         if ((k != 0 && (v & k) == k ) ||  (k == value))  {
             v = v & ~k;
@@ -1468,6 +1506,10 @@ QByteArray QMetaEnum::valueToKeys(int value) const
     return keys;
 }
 
+static QByteArray qualifiedName(const QMetaEnum &e)
+{
+    return QByteArray(e.scope()) + "::" + e.name();
+}
 
 /*!
     \class QMetaProperty
@@ -1506,6 +1548,11 @@ QByteArray QMetaEnum::valueToKeys(int value) const
     \sa isReadable()
 */
 
+/*!
+    \fn const QMetaObject *QMetaProperty::enclosingMetaObject() const
+    \internal
+*/
+
 /*!
     \internal
 */
@@ -1559,8 +1606,11 @@ QVariant::Type QMetaProperty::type() const
         type = QVariant::LastType;
     if (type)
         return QVariant::Type(type);
-    if (isEnumType())
-        return QVariant::Int;
+    if (isEnumType()) {
+        int enumMetaTypeId = QMetaType::type(qualifiedName(menum));
+        if (enumMetaTypeId == 0)
+            return QVariant::Int;
+    }
 
     return QVariant::UserType;
 }
@@ -1579,6 +1629,10 @@ int QMetaProperty::userType() const
     QVariant::Type tp = type();
     if (tp != QVariant::UserType)
         return tp;
+    if (isEnumType()) {
+        int enumMetaTypeId = QMetaType::type(qualifiedName(menum));
+        return enumMetaTypeId;
+    }
     return QMetaType::type(typeName());
 }
 
@@ -1652,8 +1706,17 @@ QVariant QMetaProperty::read(const QObject *object) const
     if (!object || !mobj)
         return QVariant();
 
-    int  t = QVariant::Int;
-    if (!isEnumType()) {
+    uint t = QVariant::Int;
+    if (isEnumType()) {
+        /*
+          try to create a QVariant that can be converted to this enum
+          type (only works if the enum has already been registered
+          with QMetaType)
+        */
+        int enumMetaTypeId = QMetaType::type(qualifiedName(menum));
+        if (enumMetaTypeId != 0)
+            t = enumMetaTypeId;
+    } else {
         int handle = priv(mobj->d.data)->propertyData + 3*idx;
         uint flags = mobj->d.data[handle + 2];
         const char *typeName = mobj->d.stringdata + mobj->d.data[handle + 1];
@@ -1671,8 +1734,8 @@ QVariant QMetaProperty::read(const QObject *object) const
         }
     }
     QVariant value;
-    void *argv[1];
-    if (t == int(QVariant::LastType)) {
+    void *argv[2] = { 0, &value };
+    if (t == QVariant::LastType) {
         argv[0] = &value;
     } else {
         value = QVariant(t, (void*)0);
@@ -1681,7 +1744,11 @@ QVariant QMetaProperty::read(const QObject *object) const
     const_cast<QObject*>(object)->qt_metacall(QMetaObject::ReadProperty,
                                               idx + mobj->propertyOffset(),
                                               argv);
-    if (t != int(QVariant::LastType) && argv[0] != value.data())
+    if (argv[1] == 0)
+        // "value" was changed
+        return value;
+    if (t != QVariant::LastType && argv[0] != value.data())
+        // pointer or reference
         return QVariant((QVariant::Type)t, argv[0]);
     return value;
 }
@@ -1710,7 +1777,10 @@ bool QMetaProperty::write(QObject *object, const QVariant &value) const
             else
                 v = QVariant(menum.keyToValue(value.toByteArray()));
         } else if (v.type() != QVariant::Int && v.type() != QVariant::UInt) {
-            return false;
+            int enumMetaTypeId = QMetaType::type(qualifiedName(menum));
+            if ((enumMetaTypeId == 0) || (v.userType() != enumMetaTypeId) || !v.constData())
+                return false;
+            v = QVariant(*reinterpret_cast<const int *>(v.constData()));
         }
         v.convert(QVariant::Int);
     } else {
@@ -1733,7 +1803,7 @@ bool QMetaProperty::write(QObject *object, const QVariant &value) const
             return false;
     }
 
-    void *argv[1];
+    void *argv[2] = { 0, &v };
     if (t == QVariant::LastType)
         argv[0] = &v;
     else
@@ -1876,13 +1946,15 @@ bool QMetaProperty::isStored(const QObject *object) const
 }
 
 /*!
-    Returns true if this is the property that the user can editable for \a object;
-    otherwise returns false. I.e. the text property is the user editable property
+    Returns true if this property is designated as the \c USER
+    property, i.e., the one that the user can edit for \a object or
+    that is significant in some other way.  Otherwise it returns
+    false. e.g., the \c text property is the \c USER editable property
     of a QLineEdit.
 
-    If no \a object is given, the function returns false if the
-    \c{Q_PROPERTY()}'s \c USER attribute is false; otherwise returns
-    true (if the attribute is true or is a function or expression).
+    If \a object is null, the function returns false if the \c
+    {Q_PROPERTY()}'s \c USER attribute is false. Otherwise it returns
+    true.
 
     \sa QMetaObject::userProperty(), isDesignable(), isScriptable()
 */
@@ -1938,17 +2010,7 @@ bool QMetaProperty::isEditable(const QObject *object) const
     are specified using Q_CLASSINFO() in the source code. The
     information can be retrieved using name() and value(). For example:
 
-    \code
-        class MyClass
-        {
-            Q_OBJECT
-            Q_CLASSINFO("author", "Sabrina Schweinsteiger")
-            Q_CLASSINFO("url", "http://doc.moosesoft.co.uk/1.0/")
-
-        public:
-            ...
-        };
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmetaobject.cpp 5
 
     This mechanism is free for you to use in your Qt applications. Qt
     doesn't use it for any of its classes.
@@ -1962,6 +2024,11 @@ bool QMetaProperty::isEditable(const QObject *object) const
     \internal
 */
 
+/*!
+    \fn const QMetaObject *QMetaClassInfo::enclosingMetaObject() const
+    \internal
+*/
+
 /*!
     Returns the name of this item.
 
@@ -2056,3 +2123,5 @@ const char* QMetaClassInfo::value() const
     Constructs a QGenericReturnArgument object with the given \a name
     and \a data.
 */
+
+QT_END_NAMESPACE
