@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +48,8 @@
 #include "qstringlist.h"
 #include "qtextcodec.h"
 
+QT_BEGIN_NAMESPACE
+
 struct QMimeDataStruct
 {
     QString format;
@@ -38,6 +60,7 @@ class QMimeDataPrivate : public QObjectPrivate
 {
     Q_DECLARE_PUBLIC(QMimeData)
 public:
+    void removeData(const QString &format);
     void setData(const QString &format, const QVariant &data);
     QVariant getData(const QString &format) const;
 
@@ -46,15 +69,20 @@ public:
     QList<QMimeDataStruct> dataList;
 };
 
-void QMimeDataPrivate::setData(const QString &format, const QVariant &data)
+void QMimeDataPrivate::removeData(const QString &format)
 {
-    // remove it first if the format is already here.
     for (int i=0; i<dataList.size(); i++) {
         if (dataList.at(i).format == format) {
             dataList.removeAt(i);
-            break;
+            return;
         }
     }
+}
+
+void QMimeDataPrivate::setData(const QString &format, const QVariant &data)
+{
+    // remove it first if the format is already here.
+    removeData(format);
     QMimeDataStruct mimeData;
     mimeData.format = format;
     mimeData.data = data;
@@ -117,11 +145,19 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QVariant::Ty
             // fall through
         }
         case QVariant::Url: {
+            QByteArray ba = data.toByteArray();
+            // Qt 3.x will send text/uri-list with a trailing
+            // null-terminator (that is *not* sent for any other
+            // text/* mime-type), so chop it off
+            if (ba.endsWith('\0'))
+                ba.chop(1);
+
+            QList<QByteArray> urls = ba.split('\n');
             QList<QVariant> list;
-            QList<QByteArray> urls = data.toByteArray().split('\n');
             for (int i = 0; i < urls.size(); ++i) {
                 QByteArray ba = urls.at(i).trimmed();
-                list.append(QUrl::fromEncoded(ba));
+                if (!ba.isEmpty())
+                    list.append(QUrl::fromEncoded(ba));
             }
             return list;
         }
@@ -201,35 +237,16 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QVariant::Ty
     For example, if your write a widget that accepts URL drags, you
     would end up writing code like this:
 
-    \code
-        void MyWidget::dragEnterEvent(QDragEnterEvent *event)
-        {
-            if (event->mimeData()->hasUrl())
-                event->acceptProposedEvent();
-        }
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 0
 
-        void MyWidget::dropEvent(QDropEvent *event)
-        {
-            if (event->mimeData()->hasUrl()) {
-                QUrl url = event->mimeData()->url();
-                ...
-            }
-        }
-    \endcode
-
-    There are three appraches for storing custom data in a QMimeData
+    There are three approaches for storing custom data in a QMimeData
     object:
 
     \list 1
     \o  Custom data can be stored directly in a QMimeData object as a
         QByteArray using setData(). For example:
 
-        \code
-            QByteArray csvData = ...;
-
-            QMimeData *mimeData = new QMimeData;
-            mimeData->setData("text/csv", csvData);
-        \endcode
+        \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 1
 
     \o  We can subclass QMimeData and reimplement hasFormat(),
         formats(), and retrieveData().
@@ -239,20 +256,32 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QVariant::Ty
         it, and use a qobject_cast() in the receiver's drop event
         handler. For example:
 
-        \code
-        void MyWidget::dropEvent(QDropEvent *event)
-        {
-            const MyMimeData *myData =
-                    qobject_cast<const MyMimeData *>(event->mimeData());
-            if (myData) {
-                // access myData's data directly (not through QMimeData's API)
-            }
-        }
-        \endcode
+        \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 2
     \endlist
 
+    \section1 Platform-Specific MIME Types
+
+    On Windows, formats() will also return custom formats available
+    in the MIME data, using the \c{x-qt-windows-mime} subtype to
+    indicate that they represent data in non-standard formats.
+    The formats will take the following form:
+
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 3
+
+    The following are examples of custom MIME types:
+
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 4
+
+    The \c value declaration of each format describes the way in which the
+    data is encoded.
+
+    On Windows, the MIME format does not always map directly to the
+    clipboard formats. Qt provides QWindowsMime to map clipboard
+    formats to open-standard MIME formats. Similarly, the
+    QMacPasteboardMime maps MIME to Mac flavors.
+
     \sa QClipboard, QDragEnterEvent, QDragMoveEvent, QDropEvent, QDrag,
-        {Drag and Drop}
+        QWindowsMime, QMacPasteboardMime, {Drag and Drop}
 */
 
 /*!
@@ -405,12 +434,7 @@ bool QMimeData::hasHtml() const
     library, whereas QImage belongs to \l QtGui. To convert the
     QVariant to a QImage, simply use qvariant_cast(). For example:
 
-    \code
-        if (event->mimeData()->hasImage()) {
-            QImage image = qvariant_cast<QImage>(event->mimeData()->imageData());
-            ...
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 5
 
     \sa hasImage()
 */
@@ -427,9 +451,7 @@ QVariant QMimeData::imageData() const
     library, whereas QImage belongs to \l QtGui. The conversion
     from QImage to QVariant is implicit. For example:
 
-    \code
-        mimeData->setImageData(QImage("beautifulfjord.png"));
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 6
 
     \sa hasImage(), setData()
 */
@@ -457,14 +479,9 @@ bool QMimeData::hasImage() const
 
     A QVariant is used because QMimeData belongs to the \l QtCore
     library, whereas QColor belongs to \l QtGui. To convert the
-    QVariant to a QImage, simply use qvariant_cast(). For example:
+    QVariant to a QColor, simply use qvariant_cast(). For example:
 
-    \code
-        if (event->mimeData()->hasColor()) {
-            QColor color = qvariant_cast<QColor>(event->mimeData()->colorData());
-            ...
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.kernel.qmimedata.cpp 7
 
     \sa hasColor(), setColorData(), data()
 */
@@ -591,3 +608,14 @@ void QMimeData::clear()
     Q_D(QMimeData);
     d->dataList.clear();
 }
+
+/*!
+    Removes the data entry for \a mimeType in the object.
+*/
+void QMimeData::removeFormat(const QString &mimeType)
+{
+    Q_D(QMimeData);
+    d->removeData(mimeType);
+}
+
+QT_END_NAMESPACE
