@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +56,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Core)
 
 class QEvent;
@@ -84,13 +106,9 @@ public:
     uint ownObjectName : 1;
     uint sendChildEvents : 1;
     uint receiveChildEvents : 1;
-    uint unused : 25;
+    uint inEventHandler : 1;
+    uint unused : 24;
     int postedEvents;
-#ifdef QT3_SUPPORT
-    int postedChildInsertedEvents;
-#else
-    int reserved;
-#endif
 };
 
 
@@ -108,14 +126,19 @@ public:
     virtual bool eventFilter(QObject *, QEvent *);
 
 #ifdef qdoc
-    static QString tr(const char *, const char *);
-    static QString trUtf8(const char *, const char *);
+    static QString tr(const char *sourceText, const char *comment = 0, int n = -1);
+    static QString trUtf8(const char *sourceText, const char *comment = 0, int n = -1);
     virtual const QMetaObject *metaObject() const;
+    static const QMetaObject staticMetaObject;
 #endif
 #ifdef QT_NO_TRANSLATION
+    static QString tr(const char *sourceText, const char *, int)
+        { return QString::fromLatin1(sourceText); }
     static QString tr(const char *sourceText, const char * = 0)
         { return QString::fromLatin1(sourceText); }
 #ifndef QT_NO_TEXTCODEC
+    static QString trUtf8(const char *sourceText, const char *, int)
+        { return QString::fromUtf8(sourceText); }
     static QString trUtf8(const char *sourceText, const char * = 0)
         { return QString::fromUtf8(sourceText); }
 #endif
@@ -137,12 +160,12 @@ public:
 
 #ifndef QT_NO_MEMBER_TEMPLATES
     template<typename T>
-    inline T findChild(const QString &name = QString()) const
-    { return qFindChild<T>(this, name); }
+    inline T findChild(const QString &aName = QString()) const
+    { return qFindChild<T>(this, aName); }
 
     template<typename T>
-    inline QList<T> findChildren(const QString &name = QString()) const
-    { return qFindChildren<T>(this, name); }
+    inline QList<T> findChildren(const QString &aName = QString()) const
+    { return qFindChildren<T>(this, aName); }
 
 #ifndef QT_NO_REGEXP
     template<typename T>
@@ -197,6 +220,7 @@ public:
 #ifndef QT_NO_PROPERTIES
     bool setProperty(const char *name, const QVariant &value);
     QVariant property(const char *name) const;
+    QList<QByteArray> dynamicPropertyNames() const;
 #endif // QT_NO_PROPERTIES
 
 #ifndef QT_NO_USERDATA
@@ -241,7 +265,7 @@ public:
     inline QT3_SUPPORT const char *name() const { return objectName().latin1_helper(); }
     inline QT3_SUPPORT const char *name(const char *defaultName) const
         { QString s = objectName(); return s.isEmpty()?defaultName:s.latin1_helper(); }
-    inline QT3_SUPPORT void setName(const char *name) { setObjectName(QLatin1String(name)); }
+    inline QT3_SUPPORT void setName(const char *aName) { setObjectName(QLatin1String(aName)); }
 protected:
     inline QT3_SUPPORT bool checkConnectArgs(const char *signal,
                                   const QObject *,
@@ -283,12 +307,10 @@ public:
 };
 #endif
 
-
 Q_CORE_EXPORT void qt_qFindChildren_helper(const QObject *parent, const QString &name, const QRegExp *re,
-                         const QMetaObject &mo, QList<void*> *list);
+                                           const QMetaObject &mo, QList<void *> *list);
 Q_CORE_EXPORT QObject *qt_qFindChild_helper(const QObject *parent, const QString &name, const QMetaObject &mo);
 
-
 #if defined Q_CC_MSVC && _MSC_VER < 1300
 
 template<typename T>
@@ -299,8 +321,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QString &name, T)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, name, 0, ((T)0)->staticMetaObject,
-                        reinterpret_cast<QList<void *>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, name, 0, ((T)0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -325,8 +351,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QRegExp &re, T)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, 0, &re, ((T)0)->staticMetaObject,
-                        reinterpret_cast<QList<void*>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, 0, &re, ((T)0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -336,6 +366,11 @@ inline QList<T> qFindChildren(const QObject *o, const QRegExp &re)
 
 #endif
 
+#ifdef Q_MOC_RUN
+# define Q_DECLARE_INTERFACE(IFace, IId) Q_DECLARE_INTERFACE(IFace, IId)
+#endif // Q_MOC_RUN
+
+
 template <class T> inline T qobject_cast_helper(QObject *object, T)
 { return static_cast<T>(((T)0)->staticMetaObject.cast(object)); }
 
@@ -350,11 +385,13 @@ template <class T>
 inline T qobject_cast(const QObject *object)
 { return qobject_cast_helper<T>(object, T(0)); }
 
-#define Q_DECLARE_INTERFACE(IFace, IId) \
-template <> inline IFace *qobject_cast_helper<IFace *>(QObject *object, IFace *) \
-{ return (IFace *)(object ? object->qt_metacast(IId) : 0); } \
-template <> inline IFace *qobject_cast_helper<IFace *>(const QObject *object, IFace *) \
-{ return (IFace *)(object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0); }
+#ifndef Q_MOC_RUN
+#  define Q_DECLARE_INTERFACE(IFace, IId) \
+    template <> inline IFace *qobject_cast_helper<IFace *>(QObject *object, IFace *) \
+    { return (IFace *)(object ? object->qt_metacast(IId) : 0); } \
+    template <> inline IFace *qobject_cast_helper<IFace *>(const QObject *object, IFace *) \
+    { return (IFace *)(object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0); }
+#endif // Q_MOC_RUN
 
 #else
 
@@ -366,8 +403,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QString &name)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, name, 0, reinterpret_cast<T>(0)->staticMetaObject,
-                         reinterpret_cast<QList<void *>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, name, 0, reinterpret_cast<T>(0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -376,26 +417,42 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QRegExp &re)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, 0, &re, reinterpret_cast<T>(0)->staticMetaObject,
-                        reinterpret_cast<QList<void*>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, QString(), &re, reinterpret_cast<T>(0)->staticMetaObject, u.voidList);
     return list;
 }
 #endif
 
 template <class T>
 inline T qobject_cast(QObject *object)
-{ return static_cast<T>(reinterpret_cast<T>(0)->staticMetaObject.cast(object)); }
+{
+#if !defined(QT_NO_MEMBER_TEMPLATES) && !defined(QT_NO_QOBJECT_CHECK)
+    reinterpret_cast<T>(0)->qt_check_for_QOBJECT_macro(*reinterpret_cast<T>(object));
+#endif
+    return static_cast<T>(reinterpret_cast<T>(0)->staticMetaObject.cast(object));
+}
 
 template <class T>
 inline T qobject_cast(const QObject *object)
-{ return static_cast<T>(const_cast<const QObject *>(reinterpret_cast<T>(0)->staticMetaObject.cast(const_cast<QObject *>(object)))); }
+{
+#if !defined(QT_NO_MEMBER_TEMPLATES) && !defined(QT_NO_QOBJECT_CHECK)
+    reinterpret_cast<T>(0)->qt_check_for_QOBJECT_macro(*reinterpret_cast<T>(object));
+#endif
+    return static_cast<T>(const_cast<const QObject *>(reinterpret_cast<T>(0)->staticMetaObject.cast(const_cast<QObject *>(object))));
+}
 
 
-#define Q_DECLARE_INTERFACE(IFace, IId) \
-template <> inline IFace *qobject_cast<IFace *>(QObject *object) \
-{ return reinterpret_cast<IFace *>((object ? object->qt_metacast(IId) : 0)); } \
-template <> inline IFace *qobject_cast<IFace *>(const QObject *object) \
-{ return reinterpret_cast<IFace *>((object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0)); }
+#ifndef Q_MOC_RUN
+#  define Q_DECLARE_INTERFACE(IFace, IId) \
+    template <> inline IFace *qobject_cast<IFace *>(QObject *object) \
+    { return reinterpret_cast<IFace *>((object ? object->qt_metacast(IId) : 0)); } \
+    template <> inline IFace *qobject_cast<IFace *>(const QObject *object) \
+    { return reinterpret_cast<IFace *>((object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0)); }
+#endif // Q_MOC_RUN
 
 #endif
 
@@ -403,6 +460,8 @@ template <> inline IFace *qobject_cast<IFace *>(const QObject *object) \
 Q_CORE_EXPORT QDebug operator<<(QDebug, const QObject *);
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif
