@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -61,12 +61,10 @@
 
     Example:
 
-    \quotefromfile snippets/pointer/pointer.cpp
-    \skipto QPointer<QLabel> label
-    \printuntil setText(
+    \snippet doc/src/snippets/pointer/pointer.cpp 0
     \dots
-    \printline if (label)
-    \printline show()
+    \snippet doc/src/snippets/pointer/pointer.cpp 1
+    \snippet doc/src/snippets/pointer/pointer.cpp 2
 
     If the QLabel is deleted in the meantime, the \c label variable
     will hold 0 instead of an invalid address, and the last line will
@@ -147,6 +145,13 @@
     same object that \a p points to.
 */
 
+/*!
+    \fn T* QPointer::data() const
+    \since 4.4
+
+    Returns the pointer to the object being guarded.
+*/
+
 /*!
     \fn bool QPointer::isNull() const
 
