@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -43,7 +43,7 @@
 
 #include "qfactoryloader_p.h"
 
-#ifndef QT_NO_LIBRARY
+#if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
 #include "qfactoryinterface.h"
 #include "qmap.h"
 #include <qdir.h>
@@ -55,6 +55,12 @@
 #include "private/qobject_p.h"
 #include "private/qcoreapplication_p.h"
 
+QT_BEGIN_NAMESPACE
+
+Q_GLOBAL_STATIC(QList<QFactoryLoader *>, qt_factory_loaders);
+
+Q_GLOBAL_STATIC_WITH_ARGS(QMutex, qt_factoryloader_mutex, (QMutex::Recursive))
+
 class QFactoryLoaderPrivate : public QObjectPrivate
 {
     Q_DECLARE_PUBLIC(QFactoryLoader)
@@ -65,24 +71,49 @@ public:
     QList<QLibraryPrivate*> libraryList;
     QMap<QString,QLibraryPrivate*> keyMap;
     QStringList keyList;
+    QString suffix;
+    Qt::CaseSensitivity cs;
+    QStringList loadedPaths;
+
+    void unloadPath(const QString &path);
 };
 
 QFactoryLoader::QFactoryLoader(const char *iid,
-                               const QStringList &paths, const QString &suffix,
+                               const QString &suffix,
                                Qt::CaseSensitivity cs)
     : QObject(*new QFactoryLoaderPrivate)
 {
     moveToThread(QCoreApplicationPrivate::mainThread());
     Q_D(QFactoryLoader);
     d->iid = iid;
+    d->cs = cs;
+    d->suffix = suffix;
 
+
+    QMutexLocker locker(qt_factoryloader_mutex());
+    qt_factory_loaders()->append(this);
+    update();
+}
+
+
+
+void QFactoryLoader::update()
+{
 #ifdef QT_SHARED
+    Q_D(QFactoryLoader);
+    QStringList paths = QCoreApplication::libraryPaths();
     QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
-
     for (int i = 0; i < paths.count(); ++i) {
-        QString path = paths.at(i) + suffix;
+        const QString &pluginDir = paths.at(i);
+        // Already loaded, skip it...
+        if (d->loadedPaths.contains(pluginDir))
+            continue;
+        d->loadedPaths << pluginDir;
+
+        QString path = pluginDir + d->suffix;
         if (!QDir(path).exists(QLatin1String(".")))
             continue;
+
         QStringList plugins = QDir(path).entryList(QDir::Files);
         QLibraryPrivate *library = 0;
         for (int j = 0; j < plugins.count(); ++j) {
@@ -91,7 +122,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
                 qDebug() << "QFactoryLoader::QFactoryLoader() looking at" << fileName;
             }
             library = QLibraryPrivate::findOrCreate(QFileInfo(fileName).canonicalFilePath());
-            if (!library->isPlugin()) {
+            if (!library->isPlugin(&settings)) {
                 if (qt_debug_component()) {
                     qDebug() << library->errorString;
                     qDebug() << "         not a plugin";
@@ -102,7 +133,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
             QString regkey = QString::fromLatin1("Qt Factory Cache %1.%2/%3:/%4")
                              .arg((QT_VERSION & 0xff0000) >> 16)
                              .arg((QT_VERSION & 0xff00) >> 8)
-                             .arg(QLatin1String(iid))
+                             .arg(QLatin1String(d->iid))
                              .arg(fileName);
             QStringList reg, keys;
             reg = settings.value(regkey).toStringList();
@@ -123,7 +154,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
                     // ignore plugins that have a valid signature but cannot be loaded.
                     continue;
                 QFactoryInterface *factory = qobject_cast<QFactoryInterface*>(instance);
-                if (instance && factory && instance->qt_metacast(iid))
+                if (instance && factory && instance->qt_metacast(d->iid))
                     keys = factory->keys();
                 if (keys.isEmpty())
                     library->unload();
@@ -147,7 +178,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
                 // whereas the new one has a Qt version that fits
                 // better
                 QString key = keys.at(k);
-                if (!cs)
+                if (!d->cs)
                     key = key.toLower();
                 QLibraryPrivate *previous = d->keyMap.value(key);
                 if (!previous || (previous->qt_version > QT_VERSION && library->qt_version <= QT_VERSION)) {
@@ -158,9 +189,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
         }
     }
 #else
-    Q_UNUSED(paths);
-    Q_UNUSED(suffix);
-    Q_UNUSED(cs);
+    Q_D(QFactoryLoader);
     if (qt_debug_component()) {
         qDebug() << "QFactoryLoader::QFactoryLoader() ignoring" << d->iid
                  << "since plugins are disabled in static builds";
@@ -173,6 +202,9 @@ QFactoryLoader::~QFactoryLoader()
     Q_D(QFactoryLoader);
     for (int i = 0; i < d->libraryList.count(); ++i)
         d->libraryList.at(i)->release();
+
+    QMutexLocker locker(qt_factoryloader_mutex());
+    qt_factory_loaders()->removeAll(this);
 }
 
 QStringList QFactoryLoader::keys() const
@@ -198,7 +230,8 @@ QObject *QFactoryLoader::instance(const QString &key) const
             if (instances.at(i)->qt_metacast(d->iid) && factory->keys().contains(key, Qt::CaseInsensitive))
                 return instances.at(i);
 
-    if (QLibraryPrivate* library = d->keyMap.value(key)) {
+    QString lowered = d->cs ? key : key.toLower();
+    if (QLibraryPrivate* library = d->keyMap.value(lowered)) {
         if (library->instance || library->loadPlugin()) {
             if (QObject *obj = library->instance()) {
                 if (obj && !obj->parent())
@@ -209,4 +242,17 @@ QObject *QFactoryLoader::instance(const QString &key) const
     }
     return 0;
 }
+
+void QFactoryLoader::refreshAll()
+{
+    QMutexLocker locker(qt_factoryloader_mutex());
+    QList<QFactoryLoader *> *loaders = qt_factory_loaders();
+    for (QList<QFactoryLoader *>::const_iterator it = loaders->constBegin();
+         it != loaders->constEnd(); ++it) {
+        (*it)->update();
+    }
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_LIBRARY
