@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,12 +53,33 @@
 
 #include "qthread_p.h"
 
+/*
+#ifdef Q_OS_WIN32
+# include "qt_windows.h"
+#else
+# include <unistd.h>
+# include <netinet/in.h>
+# include <sys/utsname.h>
+# include <sys/socket.h>
+*/
+/*
+#  elif defined(Q_OS_HPUX)
+#   include <sys/pstat.h>
+#  elif defined(Q_OS_FREEBSD) || defined(Q_OS_OPENBSD) || defined(Q_OS_MAC)
+#   include <sys/sysctl.h>
+#  endif
+#endif
+*/
+
+QT_BEGIN_NAMESPACE
+
 /*
   QThreadData
 */
 
-QThreadData::QThreadData()
-    : _ref(1), thread(0), quitNow(false), eventDispatcher(0), canWait(true), tls(0)
+QThreadData::QThreadData(int initialRefCount)
+    : _ref(initialRefCount), thread(0),
+      quitNow(false), loopLevel(0), eventDispatcher(0), canWait(true)
 {
     // fprintf(stderr, "QThreadData %p created\n", this);
 }
@@ -55,10 +96,6 @@ QThreadData::~QThreadData()
         const QPostEvent &pe = postEventList.at(i);
         if (pe.event) {
             --pe.receiver->d_func()->postedEvents;
-#ifdef QT3_SUPPORT
-            if (pe.event->type() == QEvent::ChildInserted)
-                --pe.receiver->d_func()->postedChildInsertedEvents;
-#endif
             pe.event->posted = false;
             delete pe.event;
         }
@@ -148,6 +185,12 @@ QThread *QAdoptedThread::createThreadForAdoption()
     return t;
 }
 
+void QAdoptedThread::run()
+{
+    // this function should never be called
+    qFatal("QAdoptedThread::run(): Internal error, this implementation should never be called.");
+}
+
 /*!
     \class QThread
     \brief The QThread class provides platform-independent threads.
@@ -160,26 +203,12 @@ QThread *QAdoptedThread::createThreadForAdoption()
     program; it shares data with all the other threads within the
     process but executes independently in the way that a separate
     program does on a multitasking operating system. Instead of
-    starting in \c main(), QThreads begin executing in run().
-    To create your own threads, subclass QThread and reimplement
-    run(). For example:
+    starting in \c main(), QThreads begin executing in run().  By
+    default, run() starts the event loop by calling exec() (see
+    below). To create your own threads, subclass QThread and
+    reimplement run(). For example:
 
-    \code
-        class MyThread : public QThread
-        {
-        public:
-            void run();
-        };
-
-        void MyThread::run()
-        {
-            QTcpSocket socket;
-            // connect QTcpSocket's signals somewhere meaningful
-            ...
-            socket.connectToHost(hostName, portNumber);
-            exec();
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.thread.qthread.cpp 0
 
     This will create a QTcpSocket in the thread and then execute the
     thread's event loop. Use the start() method to begin execution.
@@ -197,11 +226,12 @@ QThread *QAdoptedThread::createThreadForAdoption()
     Each QThread can have its own event loop. You can start the event
     loop by calling exec(); you can stop it by calling exit() or
     quit(). Having an event loop in a thread makes it possible to
-    connect signals from other threads to slots in this threads,
-    using a mechanism called \l{Qt::QueuedConnection}{queued
+    connect signals from other threads to slots in this threads, using
+    a mechanism called \l{Qt::QueuedConnection}{queued
     connections}. It also makes it possible to use classes that
     require the event loop, such as QTimer and QTcpSocket, in the
-    thread.
+    thread. Note, however, that is is not possible to use any widget
+    classes in the thread.
 
     In extreme cases, you may want to forcibly terminate() an
     executing thread. However, doing so is dangerous and discouraged.
@@ -385,9 +415,9 @@ uint QThread::stackSize() const
 }
 
 /*!
-    Enters the event loop and waits until exit() is called or the main
-    widget is destroyed, and returns the value that was set to exit()
-    (which is 0 if exit() is called via quit()).
+    Enters the event loop and waits until exit() is called, returning the value
+    that was passed to exit(). The value returned is 0 if exit() is called via
+    quit().
 
     It is necessary to call this function to start event handling.
 
@@ -397,6 +427,7 @@ int QThread::exec()
 {
     Q_D(QThread);
     d->mutex.lock();
+    d->data->quitNow = false;
     QEventLoop eventLoop;
     d->mutex.unlock();
     int returnCode = eventLoop.exec();
@@ -446,27 +477,35 @@ void QThread::quit()
 { exit(); }
 
 /*!
-    \fn void QThread::run()
+    The starting point for the thread. After calling start(), the
+    newly created thread calls this function. The default
+    implementation simply calls exec().
 
-    This method is pure virtual and must be implemented in derived
-    classes in order to do useful work. Returning from this method
-    will end the execution of the thread.
+    You can reimplemented this function to do other useful
+    work. Returning from this method will end the execution of the
+    thread.
 
-    \sa wait()
+    \sa start() wait()
 */
-
+void QThread::run()
+{
+    (void) exec();
+}
 
 /*! \internal
     Initializes the QThread system.
 */
+#if defined (Q_OS_WIN)
+void qt_create_tls();
+#endif
+
 void QThread::initialize()
 {
     if (qt_global_mutexpool)
         return;
-    qt_global_mutexpool = new QMutexPool(true);
+    qt_global_mutexpool = QMutexPool::instance();
 
 #if defined (Q_OS_WIN)
-    extern void qt_create_tls();
     qt_create_tls();
 #endif
 }
@@ -477,7 +516,6 @@ void QThread::initialize()
 */
 void QThread::cleanup()
 {
-    delete qt_global_mutexpool;
     qt_global_mutexpool = 0;
 }
 
@@ -524,7 +562,9 @@ QThread::Priority QThread::priority() const
 
 #else // QT_NO_THREAD
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include <private/qcoreapplication_p.h>
+QT_END_INCLUDE_NAMESPACE
 
 QThread* QThread::instance = 0;
 
@@ -543,3 +583,5 @@ QThreadData* QThreadData::current()
 }
 
 #endif // QT_NO_THREAD
+
+QT_END_NAMESPACE
