@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -52,6 +57,8 @@
 
 #include "qunicodetables.cpp"
 
+QT_BEGIN_NAMESPACE
+
 #define LAST_UNICODE_CHAR 0x10ffff
 
 #ifndef QT_NO_CODEC_FOR_C_STRINGS
@@ -170,17 +177,17 @@
     Specifies which version of the \l{http://www.unicode.org/}{Unicode standard}
     introduced a certain character.
 
-    \value Unicode_1_1  Version 1.1.
-    \value Unicode_2_0  Version 2.0.
-    \value Unicode_2_1_2  Version 2.1.2.
-    \value Unicode_3_0  Version 3.0.
-    \value Unicode_3_1  Version 3.1.
-    \value Unicode_3_2  Version 3.2.
-    \value Unicode_4_0  Version 4.0.
-    \value Unicode_4_1  Version 4.1.
-    \value Unicode_5_0  Version 5.0.
+    \value Unicode_1_1  Version 1.1
+    \value Unicode_2_0  Version 2.0
+    \value Unicode_2_1_2  Version 2.1.2
+    \value Unicode_3_0  Version 3.0
+    \value Unicode_3_1  Version 3.1
+    \value Unicode_3_2  Version 3.2
+    \value Unicode_4_0  Version 4.0
+    \value Unicode_4_1  Version 4.1
+    \value Unicode_5_0  Version 5.0
     \value Unicode_Unassigned  The value is not assigned to any character
-        in version 4.0 of Unicode.
+        in version 5.0 of Unicode.
 
     \sa unicodeVersion()
 */
@@ -545,6 +552,8 @@ bool QChar::isSpace() const
 /*!
     Returns true if the character is a mark (Mark_* categories);
     otherwise returns false.
+    
+    See QChar::Category for more information regarding marks.
 */
 bool QChar::isMark() const
 {
@@ -656,8 +665,8 @@ bool QChar::isSymbol() const
 /*!
   \fn static uint QChar::surrogateToUcs4(ushort high, ushort low)
 
-  Converts a utf16 surrogate pair (\a high, \a low) to it's ucs4
-  code point.
+  Converts a UTF16 surrogate pair with the given \a high and \a low values
+  to its UCS-4 code point.
 */
 
 /*!
@@ -721,8 +730,9 @@ QChar::Category QChar::category() const
 }
 
 /*! 
-\overload
-Returns the category of the UCS-4-encoded character specified by \a ucs4.
+    \overload
+    \since 4.3
+    Returns the category of the UCS-4-encoded character specified by \a ucs4.
  */
 QChar::Category QChar::category(uint ucs4)
 {
@@ -732,8 +742,8 @@ QChar::Category QChar::category(uint ucs4)
 }
 
 /*! 
-\overload
-Returns the category of the UCS-2-encoded character specified by \a ucs2.
+    \overload
+    Returns the category of the UCS-2-encoded character specified by \a ucs2.
  */
 QChar::Category QChar::category(ushort ucs2)
 {
@@ -894,7 +904,8 @@ enum {
 };
 
 // buffer has to have a length of 3. It's needed for Hangul decomposition
-static const unsigned short * QT_FASTCALL decomposition(uint ucs4, int *length, int *tag, unsigned short *buffer)
+static const unsigned short * QT_FASTCALL decompositionHelper
+    (uint ucs4, int *length, int *tag, unsigned short *buffer)
 {
     *length = 0;
     if (ucs4 > LAST_UNICODE_CHAR)
@@ -937,7 +948,7 @@ QString QChar::decomposition(uint ucs4)
     unsigned short buffer[3];
     int length;
     int tag;
-    const unsigned short *d = ::decomposition(ucs4, &length, &tag, buffer);
+    const unsigned short *d = decompositionHelper(ucs4, &length, &tag, buffer);
     return QString::fromUtf16(d, length);
 }
 
@@ -1227,6 +1238,7 @@ ushort QChar::toCaseFolded(ushort ucs2)
 */
 
 /*!
+    \fn char QChar::toAscii() const
     Returns the character value of the QChar obtained using the current
     codec used to read C strings, or 0 if the character is not representable
     using this codec. The default codec handles Latin-1 encoded text,
@@ -1239,7 +1251,11 @@ ushort QChar::toCaseFolded(ushort ucs2)
 
     \sa toLatin1(), unicode(), QTextCodec::codecForCStrings()
 */
+#ifdef Q_COMPILER_MANGLES_RETURN_TYPE
 const char QChar::toAscii() const
+#else
+char QChar::toAscii() const
+#endif
 {
 #ifndef QT_NO_CODEC_FOR_C_STRINGS
     if (QTextCodec::codecForCStrings())
@@ -1407,7 +1423,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 // ---------------------------------------------------------------------------
 
 
-static QString decompose(const QString &str, bool canonical, QChar::UnicodeVersion version)
+static QString decomposeHelper
+    (const QString &str, bool canonical, QChar::UnicodeVersion version)
 {
     unsigned short buffer[3];
 
@@ -1428,7 +1445,7 @@ static QString decompose(const QString &str, bool canonical, QChar::UnicodeVersi
             continue;
         int length;
         int tag;
-        const unsigned short *d = decomposition(ucs4, &length, &tag, buffer);
+        const unsigned short *d = decompositionHelper(ucs4, &length, &tag, buffer);
         if (!d || (canonical && tag != QChar::Canonical))
             continue;
 
@@ -1443,7 +1460,7 @@ static QString decompose(const QString &str, bool canonical, QChar::UnicodeVersi
 }
 
 
-static ushort ligature(ushort u1, ushort u2)
+static ushort ligatureHelper(ushort u1, ushort u2)
 {
     // hangul L-V pair
     int LIndex = u1 - Hangul_LBase;
@@ -1474,7 +1491,7 @@ static ushort ligature(ushort u1, ushort u2)
     return 0;
 }
 
-static QString compose(const QString &str)
+static QString composeHelper(const QString &str)
 {
     QString s = str;
 
@@ -1497,7 +1514,7 @@ static QString compose(const QString &str)
         int combining = QChar::combiningClass(uc);
         if (starter == pos - 1 || combining > lastCombining) {
             // allowed to form ligature with S
-            QChar ligature = ::ligature(s.utf16()[starter], uc);
+            QChar ligature = ligatureHelper(s.utf16()[starter], uc);
             if (ligature.unicode()) {
                 s[starter] = ligature;
                 s.remove(pos, 1);
@@ -1513,7 +1530,8 @@ static QString compose(const QString &str)
 }
 
 
-static QString canonicalOrder(const QString &str, QChar::UnicodeVersion version)
+static QString canonicalOrderHelper
+    (const QString &str, QChar::UnicodeVersion version)
 {
     QString s = str;
     const int l = s.length()-1;
@@ -1598,3 +1616,5 @@ Q_CORE_EXPORT QUnicodeTables::LineBreakClass QUnicodeTables::lineBreakClass(uint
     return (QUnicodeTables::LineBreakClass) qGetProp(ucs4)->line_break_class;
 }
 
+
+QT_END_NAMESPACE
