@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,6 +48,8 @@
 
 #include <math.h>
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QLine
     \ingroup multimedia
@@ -195,6 +202,60 @@
     Translates this line the distance specified by \a dx and \a dy.
 */
 
+/*!
+    \fn QLine QLine::translated(const QPoint &offset) const
+
+    \since 4.4
+
+    Returns this line translated by the given \a offset.
+*/
+
+/*!
+    \fn QLine QLine::translated(int dx, int dy) const
+    \overload
+    \since 4.4
+
+    Returns this line translated the distance specified by \a dx and \a dy.
+*/
+
+
+/*!
+    \fn void QLine::setP1(const QPoint &p1)
+
+    Sets the starting point of this line to \a p1.
+
+    \sa setP2(), p1()
+*/
+
+
+/*!
+    \fn void QLine::setP2(const QPoint &p2)
+
+    Sets the end point of this line to \a p2.
+
+    \sa setP1(), p2()
+*/
+
+
+/*!
+    \fn void QLine::setPoints(const QPoint &p1, const QPoint &p2)
+
+    Sets the start point of this line to \a p1 and the end point of this line to \a p2.
+
+    \sa setP1(), setP2(), p1(), p2()
+*/
+
+
+/*!
+    \fn void QLine::setLine(int x1, int y1, int x2, int y2)
+
+    Sets this line to the start in \a x1, \a y1 and end in \a x2, \a y2.
+
+    \sa setP1(), setP2(), p1(), p2()
+*/
+
+
+
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug d, const QLine &p)
 {
@@ -270,7 +331,9 @@ QDataStream &operator>>(QDataStream &stream, QLine &line)
     components of the line, respectively.
 
     The line's length can be retrieved using the length() function,
-    and altered using the setLength() function.  Use the isNull()
+    and altered using the setLength() function.  Similarly, angle()
+    and setAngle() are respectively used for retrieving and altering
+    the angle of the line. Use the isNull()
     function to determine whether the QLineF represents a valid line
     or a null line.
 
@@ -490,6 +553,70 @@ qreal QLineF::length() const
     return sqrt(x*x + y*y);
 }
 
+/*!
+    \since 4.4
+
+    Returns the angle of the line in degrees.
+
+    Positive values for the angles mean counter-clockwise while negative values
+    mean the clockwise direction. Zero degrees is at the 3 o'clock position.
+
+    \sa setAngle()
+*/
+qreal QLineF::angle() const
+{
+    const qreal dx = pt2.x() - pt1.x();
+    const qreal dy = pt2.y() - pt1.y();
+
+    const qreal theta = atan2(-dy, dx) * 360.0 / M_2PI;
+
+    const qreal theta_normalized = theta < 0 ? theta + 360 : theta;
+
+    if (qFuzzyCompare(theta_normalized, qreal(360)))
+        return qreal(0);
+    else
+        return theta_normalized;
+}
+
+/*!
+    \since 4.4
+
+    Sets the angle of the line to the given \a angle (in degrees).
+    This will change the position of the second point of the line such that
+    the line has the given angle.
+
+    Positive values for the angles mean counter-clockwise while negative values
+    mean the clockwise direction. Zero degrees is at the 3 o'clock position.
+
+    \sa angle()
+*/
+void QLineF::setAngle(qreal angle)
+{
+    const qreal angleR = angle * M_2PI / 360.0;
+    const qreal l = length();
+
+    const qreal dx = cos(angleR) * l;
+    const qreal dy = -sin(angleR) * l;
+
+    pt2.rx() = pt1.x() + dx;
+    pt2.ry() = pt1.y() + dy;
+}
+
+/*!
+    \since 4.4
+
+    Returns a QLineF with the given \a length and \a angle.
+
+    The first point of the line will be on the origin.
+
+    Positive values for the angles mean counter-clockwise while negative values
+    mean the clockwise direction. Zero degrees is at the 3 o'clock position.
+*/
+QLineF QLineF::fromPolar(qreal length, qreal angle)
+{
+    const qreal angleR = angle * M_2PI / 360.0;
+    return QLineF(0, 0, cos(angleR) * length, -sin(angleR) * length);
+}
 
 /*!
     Returns the unit vector for this line, i.e a line starting at the
@@ -568,8 +695,8 @@ QLineF::IntersectType QLineF::intersect(const QLineF &l, QPointF *intersectionPo
                                             l.x1(), l.y1(), l.x2(), l.y2())
                          ? BoundedIntersection : UnboundedIntersection;
 
-    bool dx_zero = qFuzzyCompare(dx(), 0);
-    bool ldx_zero = qFuzzyCompare(l.dx(), 0);
+    bool dx_zero = qFuzzyCompare(dx() + 1, 1);
+    bool ldx_zero = qFuzzyCompare(l.dx() + 1, 1);
 
     // For special case where one of the lines are vertical
     if (dx_zero && ldx_zero) {
@@ -607,13 +734,100 @@ QLineF::IntersectType QLineF::intersect(const QLineF &l, QPointF *intersectionPo
     Translates this line the distance specified by \a dx and \a dy.
 */
 
+/*!
+    \fn QLineF QLineF::translated(const QPointF &offset) const
+
+    \since 4.4
+
+    Returns this line translated by the given \a offset.
+*/
+
+/*!
+    \fn QLineF QLineF::translated(qreal dx, qreal dy) const
+    \overload
+    \since 4.4
+
+    Returns this line translated the distance specified by \a dx and \a dy.
+*/
+
+/*!
+    \fn void QLineF::setP1(const QPointF &p1)
+
+    Sets the starting point of this line to \a p1.
+
+    \sa setP2(), p1()
+*/
+
+
+/*!
+    \fn void QLineF::setP2(const QPointF &p2)
+
+    Sets the end point of this line to \a p2.
+
+    \sa setP1(), p2()
+*/
+
+
+/*!
+    \fn void QLineF::setPoints(const QPointF &p1, const QPointF &p2)
+
+    Sets the start point of this line to \a p1 and the end point of this line to \a p2.
+
+    \sa setP1(), setP2(), p1(), p2()
+*/
+
+
+/*!
+    \fn void QLineF::setLine(qreal x1, qreal y1, qreal x2, qreal y2)
+
+    Sets this line to the start in \a x1, \a y1 and end in \a x2, \a y2.
+
+    \sa setP1(), setP2(), p1(), p2()
+*/
+
+/*!
+  \fn qreal QLineF::angleTo(const QLineF &line) const
+
+  \since 4.4
+
+  Returns the angle (in degrees) from this line to the given \a
+  line, taking the direction of the lines into account. If the lines
+  do not intersect within their range, it is the intersection point of
+  the extended lines that serves as origin (see
+  QLineF::UnboundedIntersection).
+
+  The returned value represents the number of degrees you need to add
+  to this line to make it have the same angle as the given \a line,
+  going counter-clockwise.
+
+  \sa intersect()
+*/
+qreal QLineF::angleTo(const QLineF &l) const
+{
+    if (isNull() || l.isNull())
+        return 0;
+
+    const qreal a1 = angle();
+    const qreal a2 = l.angle();
+
+    const qreal delta = a2 - a1;
+    const qreal delta_normalized = delta < 0 ? delta + 360 : delta;
+
+    if (qFuzzyCompare(delta, qreal(360)))
+        return 0;
+    else
+        return delta_normalized;
+}
+
 /*!
   \fn qreal QLineF::angle(const QLineF &line) const
 
+  \obsolete
+
   Returns the angle (in degrees) between this line and the given \a
   line, taking the direction of the lines into account. If the lines
   do not intersect within their range, it is the intersection point of
-  the extended lines that serves as origo (see
+  the extended lines that serves as origin (see
   QLineF::UnboundedIntersection).
 
   \table
@@ -683,3 +897,5 @@ QDataStream &operator>>(QDataStream &stream, QLineF &line)
 }
 
 #endif // QT_NO_DATASTREAM
+
+QT_END_NAMESPACE
