@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +48,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Core)
 
 class Q_CORE_EXPORT QPoint
@@ -63,7 +85,7 @@ public:
     friend inline const QPoint operator/(const QPoint &, qreal);
 
 private:
-
+    friend class QTransform;
 #if defined(Q_OS_MAC)
     int yp;
     int xp;
@@ -146,7 +168,7 @@ inline const QPoint operator-(const QPoint &p)
 
 inline QPoint &QPoint::operator/=(qreal c)
 {
-    Q_ASSERT(!qFuzzyCompare(c, 0));
+    Q_ASSERT(!qFuzzyCompare(c + 1, 1));
     xp = qRound(xp/c);
     yp = qRound(yp/c);
     return *this;
@@ -154,7 +176,7 @@ inline QPoint &QPoint::operator/=(qreal c)
 
 inline const QPoint operator/(const QPoint &p, qreal c)
 {
-    Q_ASSERT(!qFuzzyCompare(c, 0));
+    Q_ASSERT(!qFuzzyCompare(c + 1, 1));
     return QPoint(qRound(p.xp/c), qRound(p.yp/c));
 }
 
@@ -201,6 +223,7 @@ public:
 
 private:
     friend class QMatrix;
+    friend class QTransform;
 
     qreal xp;
     qreal yp;
@@ -315,7 +338,7 @@ inline const QPointF operator-(const QPointF &p)
 
 inline QPointF &QPointF::operator/=(qreal c)
 {
-    Q_ASSERT(!qFuzzyCompare(c, 0));
+    Q_ASSERT(!qFuzzyCompare(c + 1, 1));
     xp/=c;
     yp/=c;
     return *this;
@@ -323,7 +346,7 @@ inline QPointF &QPointF::operator/=(qreal c)
 
 inline const QPointF operator/(const QPointF &p, qreal c)
 {
-    Q_ASSERT(!qFuzzyCompare(c, 0));
+    Q_ASSERT(!qFuzzyCompare(c + 1, 1));
     return QPointF(p.xp/c, p.yp/c);
 }
 
@@ -336,6 +359,8 @@ inline QPoint QPointF::toPoint() const
 Q_CORE_EXPORT QDebug operator<<(QDebug d, const QPointF &p);
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QPOINT_H
