@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,6 +44,11 @@
 #include "qrect.h"
 #include "qdatastream.h"
 #include "qdebug.h"
+#include "qmath.h"
+
+#include <math.h>
+
+QT_BEGIN_NAMESPACE
 
 /*!
     \class QRect
@@ -41,10 +66,7 @@
     height integers, or from a QPoint and a QSize.  The following code
     creates two identical rectangles.
 
-    \code
-        QRect r1(100, 200, 11, 16);
-        QRect r2(QPoint(100, 200), QSize(11, 16));
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qrect.cpp 0
 
     There is a third constructor that creates a QRect using the
     top-left and bottom-right coordinates, but we recommend that you
@@ -73,8 +95,8 @@
     The contains() function tells whether a given point is inside the
     rectangle or not, and the intersects() function returns true if
     this rectangle intersects with a given rectangle. The QRect class
-    also provides the intersect() function which returns the
-    intersection rectangle, and the unite function() which returns the
+    also provides the intersected() function which returns the
+    intersection rectangle, and the united() function which returns the
     rectangle that encloses the given rectangle and this:
 
     \table
@@ -82,8 +104,8 @@
     \o \inlineimage qrect-intersect.png
     \o \inlineimage qrect-unite.png
     \row
-    \o intersect()
-    \o unite()
+    \o intersected()
+    \o united()
     \endtable
 
     The isEmpty() function returns true if left() > right() or top() >
@@ -92,6 +114,9 @@
     bottom(). A null rectangle (isNull() == true) on the other hand,
     has both width and height set to 0.
 
+    Note that due to the way QRect and QRectF are defined, an
+    empty QRect is defined in essentially the same way as QRectF.
+
     Finally, QRect objects can be streamed as well as compared.
 
     \tableofcontents
@@ -161,7 +186,7 @@
     bottom(). Another solution is to use QRectF: The QRectF class
     defines a rectangle in the plane using floating point accuracy for
     coordinates, and the QRectF::right() and QRectF::bottom()
-    functions \e do return the true bottom-right corner.
+    functions \e do return the right and bottom coordinates.
 
     It is also possible to add offsets to this rectangle's coordinates
     using the adjust() function, as well as retrieve a new rectangle
@@ -227,7 +252,7 @@
     Returns true if the rectangle is a null rectangle, otherwise
     returns false.
 
-    A null rectangle has both the width and the height set to 0 (i.e.
+    A null rectangle has both the width and the height set to 0 (i.e.,
     right() == left() - 1 and bottom() == top() - 1). A null rectangle
     is also empty, and hence is not valid.
 
@@ -240,7 +265,7 @@
     Returns true if the rectangle is empty, otherwise returns false.
 
     An empty rectangle has a left() > right() or top() > bottom(). An
-    empty rectangle is not valid (i.e isEmpty() == !isValid()).
+    empty rectangle is not valid (i.e., isEmpty() == !isValid()).
 
     Use the normalized() function to retrieve a rectangle where the
     corners are swapped.
@@ -256,14 +281,14 @@
     A valid rectangle has a left() < right() and top() <
     bottom(). Note that non-trivial operations like intersections are
     not defined for invalid rectangles. A valid rectangle is not empty
-    (i.e. isValid() == !isEmpty()).
+    (i.e., isValid() == !isEmpty()).
 
     \sa isNull(), isEmpty(), normalized()
 */
 
 
 /*!
-    Returns a normalized rectangle; i.e. a rectangle that has a
+    Returns a normalized rectangle; i.e., a rectangle that has a
     non-negative width and height.
 
     If width() < 0 the function swaps the left and right corners, and
@@ -274,17 +299,15 @@
 
 QRect QRect::normalized() const
 {
-    if (isNull() || width() == 0 || height() == 0)
-        return *this;
     QRect r;
-    if (x2 < x1) {                                // swap bad x values
+    if (x2 < x1 - 1) {                                // swap bad x values
         r.x1 = x2;
         r.x2 = x1;
     } else {
         r.x1 = x1;
         r.x2 = x2;
     }
-    if (y2 < y1) {                                // swap bad y values
+    if (y2 < y1 - 1) {                                // swap bad y values
         r.y1 = y2;
         r.y2 = y1;
     } else {
@@ -299,7 +322,7 @@ QRect QRect::normalized() const
     \fn QRect QRect::normalize() const
     \compat
 
-    Returns a normalized rectangle, i.e. a rectangle that has a
+    Returns a normalized rectangle; i.e., a rectangle that has a
     non-negative width and height.
 
     Use the normalized() function instead
@@ -469,7 +492,7 @@ QRect QRect::normalized() const
     \fn void QRect::setTopLeft(const QPoint &position)
 
     Set the top-left corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     bottom-right corner of the rectangle.
 
     \sa topLeft(), moveTopLeft()
@@ -479,7 +502,7 @@ QRect QRect::normalized() const
     \fn void QRect::setBottomRight(const QPoint &position)
 
     Set the bottom-right corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     top-left corner of the rectangle.
 
     \sa bottomRight(), moveBottomRight()
@@ -489,7 +512,7 @@ QRect QRect::normalized() const
     \fn void QRect::setTopRight(const QPoint &position)
 
     Set the top-right corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     bottom-left corner of the rectangle.
 
     \sa topRight(), moveTopRight()
@@ -499,7 +522,7 @@ QRect QRect::normalized() const
     \fn void QRect::setBottomLeft(const QPoint &position)
 
     Set the bottom-left corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     top-right corner of the rectangle.
 
     \sa bottomLeft(), moveBottomLeft()
@@ -883,7 +906,7 @@ void QRect::moveCenter(const QPoint &p)
     Returns true if the the given \a point is inside or on the edge of
     the rectangle, otherwise returns false. If \a proper is true, this
     function only returns true if the given \a point is \e inside the
-    rectangle (i.e. not on the edge).
+    rectangle (i.e., not on the edge).
 
     \sa intersects()
 */
@@ -945,7 +968,7 @@ bool QRect::contains(const QRect &r, bool proper) const
 
     Unites this rectangle with the given \a rectangle.
 
-    \sa unite(), operator|()
+    \sa united(), operator|()
 */
 
 /*!
@@ -953,7 +976,7 @@ bool QRect::contains(const QRect &r, bool proper) const
 
     Intersects this rectangle with the given \a rectangle.
 
-    \sa intersect(), operator&()
+    \sa intersected(), operator&()
 */
 
 
@@ -963,7 +986,7 @@ bool QRect::contains(const QRect &r, bool proper) const
     Returns the bounding rectangle of this rectangle and the given \a
     rectangle.
 
-    \sa operator|=(),  unite()
+    \sa operator|=(), united()
 */
 
 QRect QRect::operator|(const QRect &r) const
@@ -984,12 +1007,20 @@ QRect QRect::operator|(const QRect &r) const
 
 /*!
     \fn QRect QRect::unite(const QRect &rectangle) const
+    \obsolete
+
+    Use united(\a rectangle) instead.
+*/
+
+/*!
+    \fn QRect QRect::united(const QRect &rectangle) const
+    \since 4.2
 
     Returns the bounding rectangle of this rectangle and the given \a rectangle.
 
     \image qrect-unite.png
 
-    \sa intersect()
+    \sa intersected()
 */
 
 
@@ -999,7 +1030,7 @@ QRect QRect::operator|(const QRect &r) const
     Returns the intersection of this rectangle and the given \a
     rectangle. Returns an empty rectangle if there is no intersection.
 
-    \sa operator&=(), intersect()
+    \sa operator&=(), intersected()
 */
 
 QRect QRect::operator&(const QRect &r) const
@@ -1013,28 +1044,36 @@ QRect QRect::operator&(const QRect &r) const
     tmp.x2 = qMin(r1.x2, r2.x2);
     tmp.y1 = qMax(r1.y1, r2.y1);
     tmp.y2 = qMin(r1.y2, r2.y2);
-    return tmp;
+    return tmp.isEmpty() ? QRect() : tmp;
 }
 
 /*!
     \fn QRect QRect::intersect(const QRect &rectangle) const
+    \obsolete
+
+    Use intersected(\a rectangle) instead.
+*/
+
+/*!
+    \fn QRect QRect::intersected(const QRect &rectangle) const
+    \since 4.2
 
     Returns the intersection of this rectangle and the given \a
-    rectangle. Note that \c{r.intersect(s)} is equivalent to \c{r&s}.
+    rectangle. Note that \c{r.intersected(s)} is equivalent to \c{r & s}.
 
     \image qrect-intersect.png
 
-    \sa intersects(), unite(), operator&=()
+    \sa intersects(), united(), operator&=()
 */
 
 /*!
     \fn bool QRect::intersects(const QRect &rectangle) const
 
     Returns true if this rectangle intersects with the given \a
-    rectangle (i.e. there is at least one pixel that is within both
+    rectangle (i.e., there is at least one pixel that is within both
     rectangles), otherwise returns false.
 
-    The intersection rectangle can be retrieved using the intersect()
+    The intersection rectangle can be retrieved using the intersected()
     function.
 
     \sa  contains()
@@ -1146,10 +1185,7 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     height integers, or from a QPoint and a QSize.  The following code
     creates two identical rectangles.
 
-    \code
-        QRectF r1(100, 200, 11, 16);
-        QRectF r2(QPoint(100, 200), QSize(11, 16));
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qrect.cpp 1
 
     There is also a third constructor creating a QRectF from a QRect,
     and a corresponding toRect() function that returns a QRect object
@@ -1177,8 +1213,8 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     The contains() function tells whether a given point is inside the
     rectangle or not, and the intersects() function returns true if
     this rectangle intersects with a given rectangle (otherwise
-    false). The QRectF class also provides the intersect() function
-    which returns the intersection rectangle, and the unite function()
+    false). The QRectF class also provides the intersected() function
+    which returns the intersection rectangle, and the united() function
     which returns the rectangle that encloses the given rectangle and
     this:
 
@@ -1187,8 +1223,8 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     \o \inlineimage qrect-intersect.png
     \o \inlineimage qrect-unite.png
     \row
-    \o intersect()
-    \o unite()
+    \o intersected()
+    \o united()
     \endtable
 
     The isEmpty() function returns true if the rectangle's width or
@@ -1197,6 +1233,9 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     and height is larger than 0. A null rectangle (isNull() == true)
     on the other hand, has both width and height set to 0.
 
+    Note that due to the way QRect and QRectF are defined, an
+    empty QRectF is defined in essentially the same way as QRect.
+
     Finally, QRectF objects can be streamed as well as compared.
 
     \tableofcontents
@@ -1288,6 +1327,15 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     \sa setTopLeft(), setSize()
 */
 
+/*!
+    \fn QRectF::QRectF(const QPointF &topLeft, const QPointF &bottomRight)
+    \since 4.3
+
+    Constructs a rectangle with the given \a topLeft and \a bottomRight corners.
+
+    \sa setTopLeft(), setBottomRight()
+*/
+
 /*!
     \fn QRectF::QRectF(qreal x, qreal y, qreal width, qreal height)
 
@@ -1322,7 +1370,7 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     Returns true if the rectangle is empty, otherwise returns false.
 
     An empty rectangle has width() <= 0 or height() <= 0.  An empty
-    rectangle is not valid (i.e isEmpty() == !isValid()).
+    rectangle is not valid (i.e., isEmpty() == !isValid()).
 
     Use the normalized() function to retrieve a rectangle where the
     corners are swapped.
@@ -1337,7 +1385,7 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
 
     A valid rectangle has a width() > 0 and height() > 0. Note that
     non-trivial operations like intersections are not defined for
-    invalid rectangles. A valid rectangle is not empty (i.e. isValid()
+    invalid rectangles. A valid rectangle is not empty (i.e., isValid()
     == !isEmpty()).
 
     \sa isNull(), isEmpty(), normalized()
@@ -1345,7 +1393,7 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
 
 
 /*!
-    Returns a normalized rectangle; i.e. a rectangle that has a
+    Returns a normalized rectangle; i.e., a rectangle that has a
     non-negative width and height.
 
     If width() < 0 the function swaps the left and right corners, and
@@ -1460,7 +1508,7 @@ QRectF QRectF::normalized() const
     \fn void QRectF::setTopLeft(const QPointF &position)
 
     Set the top-left corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     bottom-right corner of the rectangle.
 
     \sa topLeft(), moveTopLeft()
@@ -1469,8 +1517,8 @@ QRectF QRectF::normalized() const
 /*!
     \fn void QRectF::setBottomRight(const QPointF &position)
 
-    Set the top-right corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    Set the bottom-right corner of the rectangle to the given \a
+    position. May change the size, but will never change the
     top-left corner of the rectangle.
 
     \sa bottomRight(), moveBottomRight()
@@ -1480,7 +1528,7 @@ QRectF QRectF::normalized() const
     \fn void QRectF::setTopRight(const QPointF &position)
 
     Set the top-right corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     bottom-left corner of the rectangle.
 
     \sa topRight(), moveTopRight()
@@ -1490,7 +1538,7 @@ QRectF QRectF::normalized() const
     \fn void QRectF::setBottomLeft(const QPointF &position)
 
     Set the bottom-left corner of the rectangle to the given \a
-    position. May change the size, but will the never change the
+    position. May change the size, but will never change the
     top-right corner of the rectangle.
 
     \sa bottomLeft(), moveBottomLeft()
@@ -1874,7 +1922,7 @@ bool QRectF::contains(const QRectF &r) const
 
     Unites this rectangle with the given \a rectangle.
 
-    \sa unite(), operator|()
+    \sa united(), operator|()
 */
 
 /*!
@@ -1882,7 +1930,7 @@ bool QRectF::contains(const QRectF &r) const
 
     Intersects this rectangle with the given \a rectangle.
 
-    \sa intersect(), operator|=()
+    \sa intersected(), operator|=()
 */
 
 
@@ -1891,7 +1939,7 @@ bool QRectF::contains(const QRectF &r) const
 
     Returns the bounding rectangle of this rectangle and the given \a rectangle.
 
-    \sa unite(), operator|=()
+    \sa united(), operator|=()
 */
 
 QRectF QRectF::operator|(const QRectF &r) const
@@ -1912,13 +1960,21 @@ QRectF QRectF::operator|(const QRectF &r) const
 
 /*!
     \fn QRectF QRectF::unite(const QRectF &rectangle) const
+    \obsolete
+
+    Use united(\a rectangle) instead.
+*/
+
+/*!
+    \fn QRectF QRectF::united(const QRectF &rectangle) const
+    \since 4.2
 
     Returns the bounding rectangle of this rectangle and the given \a
     rectangle.
 
     \image qrect-unite.png
 
-    \sa intersect()
+    \sa intersected()
 */
 
 
@@ -1928,7 +1984,7 @@ QRectF QRectF::operator|(const QRectF &r) const
     Returns the intersection of this rectangle and the given \a
     rectangle. Returns an empty rectangle if there is no intersection.
 
-    \sa operator&=(), intersect()
+    \sa operator&=(), intersected()
 */
 
 QRectF QRectF::operator&(const QRectF &r) const
@@ -1942,29 +1998,37 @@ QRectF QRectF::operator&(const QRectF &r) const
     tmp.yp = qMax(r1.yp, r2.yp);
     tmp.w = qMin(r1.xp + r1.w, r2.xp + r2.w) - tmp.xp;
     tmp.h = qMin(r1.yp + r1.h, r2.yp + r2.h) - tmp.yp;
-    return tmp;
+    return tmp.isEmpty() ? QRectF() : tmp;
 }
 
 /*!
     \fn QRectF QRectF::intersect(const QRectF &rectangle) const
+    \obsolete
+
+    Use intersected(\a rectangle) instead.
+*/
+
+/*!
+    \fn QRectF QRectF::intersected(const QRectF &rectangle) const
+    \since 4.2
 
     Returns the intersection of this rectangle and the given \a
-    rectangle. Note that \c {r.intersect(s)} is equivalent to \c
-    {r&s}.
+    rectangle. Note that \c {r.intersected(s)} is equivalent to \c
+    {r & s}.
 
     \image qrect-intersect.png
 
-    \sa intersects(), unite(), operator&=()
+    \sa intersects(), united(), operator&=()
 */
 
 /*!
     \fn bool QRectF::intersects(const QRectF &rectangle) const
 
     Returns true if this rectangle intersects with the given \a
-    rectangle (i.e. there is at least one pixel that is within both
-    rectangles), otherwise returns false.
+    rectangle (i.e. there is a non-empty area of overlap between
+    them), otherwise returns false.
 
-    The intersection rectangle can be retrieved using the intersect()
+    The intersection rectangle can be retrieved using the intersected()
     function.
 
     \sa contains()
@@ -1976,8 +2040,8 @@ bool QRectF::intersects(const QRectF &r) const
         return false;
     QRectF r1 = normalized();
     QRectF r2 = r.normalized();
-    return qMax(r1.xp, r2.xp) <= qMin(r1.xp + r1.w, r2.xp + r2.w)
-        && qMax(r1.yp, r2.yp) <= qMin(r1.yp + r1.h, r2.yp + r2.h);
+    return qMax(r1.xp, r2.xp) < qMin(r1.xp + r1.w, r2.xp + r2.w)
+        && qMax(r1.yp, r2.yp) < qMin(r1.yp + r1.h, r2.yp + r2.h);
 }
 
 /*!
@@ -1986,9 +2050,29 @@ bool QRectF::intersects(const QRectF &r) const
     Returns a QRect based on the values of this rectangle.  Note that the
     coordinates in the returned rectangle are rounded to the nearest integer.
 
-    \sa QRectF()
+    \sa QRectF(), toAlignedRect()
 */
 
+/*!
+    \fn QRect QRectF::toAlignedRect() const
+    \since 4.3
+
+    Returns a QRect based on the values of this rectangle that is the
+    smallest possible integer rectangle that completely contains this
+    rectangle.
+
+    \sa toRect()
+*/
+
+QRect QRectF::toAlignedRect() const
+{
+    int xmin = int(qFloor(x()));
+    int xmax = int(qCeil(x() + width()));
+    int ymin = int(qFloor(y()));
+    int ymax = int(qCeil(y() + height()));
+    return QRect(xmin, ymin, xmax - xmin, ymax - ymin);
+}
+
 /*!
     \fn void QRectF::moveCenter(const QPointF &position)
 
@@ -2032,7 +2116,7 @@ bool QRectF::intersects(const QRectF &r) const
 
 QDataStream &operator<<(QDataStream &s, const QRectF &r)
 {
-    s << r.x() << r.y() << r.width() << r.height();
+    s << double(r.x()) << double(r.y()) << double(r.width()) << double(r.height());
     return s;
 }
 
@@ -2054,7 +2138,7 @@ QDataStream &operator>>(QDataStream &s, QRectF &r)
     s >> y;
     s >> w;
     s >> h;
-    r.setRect(x, y, w, h);
+    r.setRect(qreal(x), qreal(y), qreal(w), qreal(h));
     return s;
 }
 
@@ -2063,8 +2147,10 @@ QDataStream &operator>>(QDataStream &s, QRectF &r)
 
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QRectF &r) {
-    dbg.nospace() << "QRectF(" << r.x() << ',' << r.y() << ','
-                  << r.width() << ',' << r.height() << ')';
+    dbg.nospace() << "QRectF(" << r.x() << ',' << r.y() << ' '
+                  << r.width() << 'x' << r.height() << ')';
     return dbg.space();
 }
 #endif
+
+QT_END_NAMESPACE
