@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Core)
 
 class Q_CORE_EXPORT QRect
@@ -149,7 +171,7 @@ public:
 #endif
 
 private:
-#if defined(Q_WS_X11) || defined(Q_OS_TEMP)
+#if defined(Q_WS_X11)
     friend void qt_setCoords(QRect *r, int xp1, int yp1, int xp2, int yp2);
 #endif
 #if defined(Q_OS_MAC)
@@ -457,9 +479,9 @@ inline QRect QRect::intersect(const QRect &r) const
     return *this & r;
 }
 
-inline QRect QRect::intersected(const QRect &other) const 
-{ 
-    return intersect(other); 
+inline QRect QRect::intersected(const QRect &other) const
+{
+    return intersect(other);
 }
 
 inline QRect QRect::unite(const QRect &r) const
@@ -468,8 +490,8 @@ inline QRect QRect::unite(const QRect &r) const
 }
 
 inline QRect QRect::united(const QRect &r) const
-{ 
-     return unite(r); 
+{
+     return unite(r);
 }
 
 inline bool operator==(const QRect &r1, const QRect &r2)
@@ -492,6 +514,7 @@ class Q_CORE_EXPORT QRectF
 public:
     QRectF() { xp = yp = 0.; w = h = 0.; }
     QRectF(const QPointF &topleft, const QSizeF &size);
+    QRectF(const QPointF &topleft, const QPointF &bottomRight);
     QRectF(qreal left, qreal top, qreal width, qreal height);
     QRectF(const QRect &rect);
 
@@ -578,6 +601,7 @@ public:
     friend Q_CORE_EXPORT_INLINE bool operator!=(const QRectF &, const QRectF &);
 
     QRect toRect() const;
+    QRect toAlignedRect() const;
 
 private:
     qreal xp;
@@ -616,6 +640,14 @@ inline QRectF::QRectF(const QPointF &atopLeft, const QSizeF &asize)
     h = asize.height();
 }
 
+inline QRectF::QRectF(const QPointF &atopLeft, const QPointF &abottomRight)
+{
+    xp = atopLeft.x();
+    yp = atopLeft.y();
+    w = abottomRight.x() - xp;
+    h = abottomRight.y() - yp;
+}
+
 inline QRectF::QRectF(const QRect &r)
     : xp(r.x()), yp(r.y()), w(r.width()), h(r.height())
 {
@@ -785,8 +817,8 @@ inline QRectF QRectF::intersect(const QRectF &r) const
 }
 
 inline QRectF QRectF::intersected(const QRectF &r) const
-{ 
-    return intersect(r); 
+{
+    return intersect(r);
 }
 
 inline QRectF QRectF::unite(const QRectF &r) const
@@ -820,6 +852,8 @@ inline QRect QRectF::toRect() const
 Q_CORE_EXPORT QDebug operator<<(QDebug, const QRectF &);
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QRECT_H
