@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -58,6 +58,8 @@
 #include <QtCore/qbytearray.h>
 #include <QtCore/qlist.h>
 
+QT_BEGIN_NAMESPACE
+
 class Q_CORE_EXPORT QRingBuffer
 {
 public:
@@ -314,4 +316,6 @@ private:
     int bufferSize;
 };
 
+QT_END_NAMESPACE
+
 #endif // QRINGBUFFER_P_H
