@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 #include "qdatastream.h"
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QSize
     \ingroup multimedia
@@ -185,19 +187,7 @@ void QSize::transpose()
     \endlist
 
     Example:
-    \code
-        QSize t1(10, 12);
-        t1.scale(60, 60, Qt::IgnoreAspectRatio);
-        // t1 is (60, 60)
-
-        QSize t2(10, 12);
-        t2.scale(60, 60, Qt::KeepAspectRatio);
-        // t2 is (50, 60)
-
-        QSize t3(10, 12);
-        t3.scale(60, 60, Qt::KeepAspectRatioByExpanding);
-        // t3 is (60, 72)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 0
 
     \sa setWidth(), setHeight()
 */
@@ -211,12 +201,12 @@ void QSize::transpose()
 */
 void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
 {
-    if (mode == Qt::IgnoreAspectRatio) {
+    if (mode == Qt::IgnoreAspectRatio || wd == 0 || ht == 0) {
         wd = s.wd;
         ht = s.ht;
     } else {
         bool useHeight;
-        int rw = qint32(qint64(s.ht) * qint64(wd) / qint64(ht));
+        qint64 rw = qint64(s.ht) * qint64(wd) / qint64(ht);
 
         if (mode == Qt::KeepAspectRatio) {
             useHeight = (rw <= s.wd);
@@ -242,12 +232,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
     Using a reference makes it possible to manipulate the width
     directly. For example:
 
-    \code
-        QSize size(100, 10);
-        size.rwidth() += 20;
-
-        // size becomes (120,10)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 1
 
     \sa rheight(), setWidth()
 */
@@ -260,12 +245,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
     Using a reference makes it possible to manipulate the height
     directly. For example:
 
-    \code
-        QSize size(100, 10);
-        size.rheight() += 5;
-
-        // size becomes (100,15)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 2
 
     \sa rwidth(), setHeight()
 */
@@ -276,13 +256,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
     Adds the given \a size to \e this size, and returns a reference to
     this size. For example:
 
-    \code
-        QSize s( 3, 7);
-        QSize r(-1, 4);
-        s += r;
-
-        // s becomes (2,11)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 3
 */
 
 /*!
@@ -291,13 +265,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
     Subtracts the given \a size from \e this size, and returns a
     reference to this size. For example:
 
-    \code
-        QSize s( 3, 7);
-        QSize r(-1, 4);
-        s -= r;
-
-        // s becomes (4,3)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 4
 */
 
 /*!
@@ -454,7 +422,7 @@ QDataStream &operator>>(QDataStream &s, QSize &sz)
 
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QSize &s) {
-    dbg.nospace() << "QSize(" << s.width() << ',' << s.height() << ')';
+    dbg.nospace() << "QSize(" << s.width() << ", " << s.height() << ')';
     return dbg.space();
 }
 #endif
@@ -621,19 +589,7 @@ void QSizeF::transpose()
     \endlist
 
     Example:
-    \code
-        QSizeF t1(10, 12);
-        t1.scale(60, 60, Qt::IgnoreAspectRatio);
-        // t1 is (60, 60)
-
-        QSizeF t2(10, 12);
-        t2.scale(60, 60, Qt::KeepAspectRatio);
-        // t2 is (50, 60)
-
-        QSizeF t3(10, 12);
-        t3.scale(60, 60, Qt::KeepAspectRatioByExpanding);
-        // t3 is (60, 72)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 5
 
     \sa setWidth(), setHeight()
 */
@@ -647,7 +603,7 @@ void QSizeF::transpose()
 */
 void QSizeF::scale(const QSizeF &s, Qt::AspectRatioMode mode)
 {
-    if (mode == Qt::IgnoreAspectRatio) {
+    if (mode == Qt::IgnoreAspectRatio || qIsNull(wd) || qIsNull(ht)) {
         wd = s.wd;
         ht = s.ht;
     } else {
@@ -678,12 +634,7 @@ void QSizeF::scale(const QSizeF &s, Qt::AspectRatioMode mode)
     Using a reference makes it possible to manipulate the width
     directly. For example:
 
-    \code
-        QSizeF size(100.3, 10);
-        size.rwidth() += 20.5;
-
-         // size becomes (120.8,10)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 6
 
     \sa rheight(), setWidth()
 */
@@ -696,12 +647,7 @@ void QSizeF::scale(const QSizeF &s, Qt::AspectRatioMode mode)
     Using a reference makes it possible to manipulate the height
     directly. For example:
 
-    \code
-        QSizeF size(100, 10.2);
-        size.rheight() += 5.5;
-
-        // size becomes (100,15.7)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 7
 
     \sa rwidth(), setHeight()
 */
@@ -712,13 +658,7 @@ void QSizeF::scale(const QSizeF &s, Qt::AspectRatioMode mode)
     Adds the given \a size to this size and returns a reference to
     this size. For example:
 
-    \code
-        QSizeF s( 3, 7);
-        QSizeF r(-1, 4);
-        s += r;
-
-        // s becomes (2,11)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 8
 */
 
 /*!
@@ -727,13 +667,7 @@ void QSizeF::scale(const QSizeF &s, Qt::AspectRatioMode mode)
     Subtracts the given \a size from this size and returns a reference
     to this size. For example:
 
-    \code
-        QSizeF s( 3, 7);
-        QSizeF r(-1, 4);
-        s -= r;
-
-        // s becomes (4,3)
-    \endcode
+    \snippet doc/src/snippets/code/src.corelib.tools.qsize.cpp 9
 */
 
 /*!
@@ -884,7 +818,9 @@ QDataStream &operator>>(QDataStream &s, QSizeF &sz)
 
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QSizeF &s) {
-    dbg.nospace() << "QSizeF(" << s.width() << ',' << s.height() << ')';
+    dbg.nospace() << "QSizeF(" << s.width() << ", " << s.height() << ')';
     return dbg.space();
 }
 #endif
+
+QT_END_NAMESPACE
