@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,6 +47,8 @@
 
 #include "qaccessible.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QAccessiblePlugin
     \brief The QAccessiblePlugin class provides an abstract base for
@@ -102,4 +104,6 @@ QAccessiblePlugin::~QAccessiblePlugin()
     \sa keys()
 */
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_ACCESSIBILITY
