@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,9 +56,13 @@
 //
 //
 
+#include "private/qdialog_p.h"
+
 #ifndef QT_NO_PRINTDIALOG
 
-#include "private/qdialog_p.h"
+#include "qpagesetupdialog.h"
+
+QT_BEGIN_NAMESPACE
 
 class QPrinter;
 
@@ -67,9 +71,18 @@ class QAbstractPageSetupDialogPrivate : public QDialogPrivate
     Q_DECLARE_PUBLIC(QAbstractPageSetupDialog)
 
 public:
+    QAbstractPageSetupDialogPrivate() : printer(0), pageSetupDialogOptions(0) {}
+    void addEnabledOption(QPageSetupDialog::PageSetupDialogOption option) { pageSetupDialogOptions |= option; }
+    void setEnabledOptions(QPageSetupDialog::PageSetupDialogOptions options) { pageSetupDialogOptions = options; }
+    QPageSetupDialog::PageSetupDialogOptions enabledOptions() const { return pageSetupDialogOptions; }
+    bool isOptionEnabled(QPageSetupDialog::PageSetupDialogOption option) const { return pageSetupDialogOptions & option; }
+
     QPrinter *printer;
+    QPageSetupDialog::PageSetupDialogOptions pageSetupDialogOptions;
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_PRINTDIALOG
 
 #endif // QABSTRACTPAGESETUPDIALOG_P_H
