@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +48,8 @@
 
 #ifndef QT_NO_PRINTDIALOG
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QAbstractPrintDialog
     \brief The QAbstractPrintDialog class provides a base implementation for
@@ -55,13 +77,20 @@
 /*!
     \enum QAbstractPrintDialog::PrintDialogOption
 
-    Used to specify which parts of the print dialog should be enabled.
+    Used to specify which parts of the print dialog should be visible.
 
     \value None None of the options are enabled.
     \value PrintToFile The print to file option is enabled.
     \value PrintSelection The print selection option is enalbed.
     \value PrintPageRange The page range selection option is enabled.
     \value PrintCollateCopies
+    \value DontUseSheet Do not make the native print dialog a sheet. By default
+    on Mac OS X, the native dialog is made a sheet if it has a parent that can
+    accept sheets and is visible.  Internally, Mac OS X tracks whether a
+    printing \bold session and not which particular dialog should be a sheet
+    or not.  Therefore, make sure this value matches between the page setup
+    dialog and the print dialog or you can potentially end up in a modal loop that you can't break.
+    \value PrintShowPageSize  Show the page size + margings page only if this is enabled.
 */
 
 /*!
@@ -73,11 +102,7 @@ QAbstractPrintDialog::QAbstractPrintDialog(QPrinter *printer, QWidget *parent)
 {
     Q_D(QAbstractPrintDialog);
     d->printer = printer;
-
-    if (printer->d_func()->printDialog)
-        d->init(printer->d_func()->printDialog->d_func());
-
-    printer->d_func()->printDialog = this;
+    d->pd = printer->d_func();
 }
 
 /*!
@@ -90,9 +115,7 @@ QAbstractPrintDialog::QAbstractPrintDialog(QAbstractPrintDialogPrivate &ptr,
 {
     Q_D(QAbstractPrintDialog);
     d->printer = printer;
-    if (printer->d_func()->printDialog)
-        d->init(printer->d_func()->printDialog->d_func());
-    printer->d_func()->printDialog = this;
+    d->pd = printer->d_func();
 }
 
 
@@ -100,25 +123,25 @@ QAbstractPrintDialog::QAbstractPrintDialog(QAbstractPrintDialogPrivate &ptr,
     Sets the set of options that should be enabled in the print dialog
     to \a options.
 
-    Note that this function has no effect on Mac OS X. See the QPrintDialog
-    documentation for more information.
+    Except for the DontUseSheet option, this function has no effect on Mac OS
+    X. See the QPrintDialog documentation for more information.
 */
 void QAbstractPrintDialog::setEnabledOptions(PrintDialogOptions options)
 {
     Q_D(QAbstractPrintDialog);
-    d->options = options;
+    d->pd->options = options;
 }
 
 /*!
     Adds the option \a option to the set of enabled options in this dialog.
 
-    Note that this function has no effect on Mac OS X. See the QPrintDialog
-    documentation for more information.
+    Except for the DontUseSheet option, this function has no effect on Mac OS
+    X. See the QPrintDialog documentation for more information.
 */
 void QAbstractPrintDialog::addEnabledOption(PrintDialogOption option)
 {
     Q_D(QAbstractPrintDialog);
-    d->options |= option;
+    d->pd->options |= option;
 }
 
 /*!
@@ -127,7 +150,7 @@ void QAbstractPrintDialog::addEnabledOption(PrintDialogOption option)
 QAbstractPrintDialog::PrintDialogOptions QAbstractPrintDialog::enabledOptions() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->options;
+    return d->pd->options;
 }
 
 /*!
@@ -136,7 +159,7 @@ QAbstractPrintDialog::PrintDialogOptions QAbstractPrintDialog::enabledOptions()
 bool QAbstractPrintDialog::isOptionEnabled(PrintDialogOption option) const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->options & option;
+    return d->pd->options & option;
 }
 
 /*!
@@ -145,7 +168,7 @@ bool QAbstractPrintDialog::isOptionEnabled(PrintDialogOption option) const
 void QAbstractPrintDialog::setPrintRange(PrintRange range)
 {
     Q_D(QAbstractPrintDialog);
-    d->printRange = range;
+    d->pd->printRange = range;
 }
 
 /*!
@@ -154,7 +177,7 @@ void QAbstractPrintDialog::setPrintRange(PrintRange range)
 QAbstractPrintDialog::PrintRange QAbstractPrintDialog::printRange() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->printRange;
+    return d->pd->printRange;
 }
 
 /*!
@@ -166,27 +189,30 @@ void QAbstractPrintDialog::setMinMax(int min, int max)
     Q_D(QAbstractPrintDialog);
     Q_ASSERT_X(min <= max, "QAbstractPrintDialog::setMinMax",
                "'min' must be less than or equal to 'max'");
-    d->minPage = min;
-    d->maxPage = max;
-    d->options |= PrintPageRange;
+    d->pd->minPage = min;
+    d->pd->maxPage = max;
+    d->pd->options |= PrintPageRange;
 }
 
 /*!
     Returns the minimum page in the page range.
+    By default, this value is set to 1.
 */
 int QAbstractPrintDialog::minPage() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->minPage;
+    return d->pd->minPage;
 }
 
 /*!
-    Returns the maximum page in the page range.
+    Returns the maximum page in the page range. As of Qt 4.4, this
+    function returns INT_MAX by default. Previous versions returned 1
+    by default.
 */
 int QAbstractPrintDialog::maxPage() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->maxPage;
+    return d->pd->maxPage;
 }
 
 /*!
@@ -197,29 +223,31 @@ void QAbstractPrintDialog::setFromTo(int from, int to)
     Q_D(QAbstractPrintDialog);
     Q_ASSERT_X(from <= to, "QAbstractPrintDialog::setFromTo",
                "'from' must be less than or equal to 'to'");
-    d->fromPage = from;
-    d->toPage = to;
+    d->pd->fromPage = from;
+    d->pd->toPage = to;
 
-    if (d->minPage == 0 && d->maxPage == 0)
+    if (d->pd->minPage == 0 && d->pd->maxPage == 0)
         setMinMax(1, to);
 }
 
 /*!
     Returns the first page to be printed
+    By default, this value is set to 0.
 */
 int QAbstractPrintDialog::fromPage() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->fromPage;
+    return d->pd->fromPage;
 }
 
 /*!
     Returns the last page to be printed.
+    By default, this value is set to 0.
 */
 int QAbstractPrintDialog::toPage() const
 {
     Q_D(const QAbstractPrintDialog);
-    return d->toPage;
+    return d->pd->toPage;
 }
 
 /*!
@@ -257,12 +285,7 @@ QPrinter *QAbstractPrintDialog::printer() const
     Typically, QPrintDialog objects are constructed with a QPrinter
     object, and executed using the exec() function.
 
-    \code
-        QPrintDialog printDialog(printer, parent);
-        if (printDialog.exec() == QDialog::Accepted) {
-            // print ...
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.dialogs.qabstractprintdialog.cpp 0
 
     If the dialog is accepted by the user, the QPrinter object is
     correctly configured for printing.
@@ -286,11 +309,12 @@ QPrinter *QAbstractPrintDialog::printer() const
     settings for each available printer can be modified via the dialog's
     \gui{Properties} push button.
 
-    On Windows and Mac OS X, the native print dialog is used, which
-    means that some QWidget and QDialog properties set on the dialog
-    won't be respected. In addition, the native print dialog on Mac OS X does
-    not support setting printer options, i.e. QAbstractPrintDialog::setEnabledOptions()
-    and QAbstractPrintDialog::addEnabledOption() have no effect.
+    On Windows and Mac OS X, the native print dialog is used, which means that
+    some QWidget and QDialog properties set on the dialog won't be respected.
+    In addition, aside from the DontUseSheet option, the native print dialog on
+    Mac OS X does not support setting printer options, i.e.
+    QAbstractPrintDialog::setEnabledOptions() and
+    QAbstractPrintDialog::addEnabledOption() have no effect.
 
     \sa QPageSetupDialog, QPrinter, {Pixelator Example}, {Order Form Example},
         {Image Viewer Example}, {Scribble Example}
@@ -314,4 +338,21 @@ QPrinter *QAbstractPrintDialog::printer() const
     \reimp
 */
 
+/*!
+    \since 4.4
+
+    Set a list of widgets as \a tabs to be shown on the print dialog, if supported.
+
+    Currently this option is only supported on X11.
+
+    Setting the option tabs will transfer their ownership to the print dialog.
+*/
+void QAbstractPrintDialog::setOptionTabs(const QList<QWidget*> &tabs)
+{
+    Q_D(QAbstractPrintDialog);
+    d->setTabs(tabs);
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_PRINTDIALOG
