@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -61,6 +61,10 @@
 
 #include "QtGui/qabstractprintdialog.h"
 
+QT_BEGIN_NAMESPACE
+
+#ifndef QT_NO_PRINTER
+
 class QPrinter;
 class QPrinterPrivate;
 
@@ -69,14 +73,19 @@ class QAbstractPrintDialogPrivate : public QDialogPrivate
     Q_DECLARE_PUBLIC(QAbstractPrintDialog)
 public:
     QAbstractPrintDialogPrivate()
-        : printer(0)
+        : printer(0), pd(0)
     {
     }
 
     QPrinter *printer;
     QPrinterPrivate *pd;
+    virtual void setTabs(const QList<QWidget *> &) { }
 };
 
+#endif //QT_NO_PRINTER
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_PRINTDIALOG
 
 #endif // QABSTRACTPRINTDIALOG_P_H
