@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -59,6 +64,9 @@
 #include "qvalidator.h"
 #include "qmime.h"
 #include "qspinbox.h"
+#include "qdialogbuttonbox.h"
+
+QT_BEGIN_NAMESPACE
 
 #ifdef Q_WS_MAC
 QRgb macGetRgba(QRgb initial, bool needAlpha, bool *ok, QWidget *parent);
@@ -569,7 +577,7 @@ void QColorWell::mouseMoveEvent(QMouseEvent *e)
         QPixmap pix(cellWidth(), cellHeight());
         pix.fill(col);
         QPainter p(&pix);
-        p.drawRect(0, 0, pix.width(), pix.height());
+        p.drawRect(0, 0, pix.width() - 1, pix.height() - 1);
         p.end();
         QDrag *drg = new QDrag(this);
         drg->setMimeData(mime);
@@ -1003,7 +1011,7 @@ void QColorShowLabel::mouseMoveEvent(QMouseEvent *e)
         QPixmap pix(30, 20);
         pix.fill(col);
         QPainter p(&pix);
-        p.drawRect(0, 0, pix.width(), pix.height());
+        p.drawRect(0, 0, pix.width() - 1, pix.height() - 1);
         p.end();
         QDrag *drg = new QDrag(this);
         drg->setMimeData(mime);
@@ -1056,7 +1064,11 @@ QColorShower::QColorShower(QWidget *parent)
     QGridLayout *gl = new QGridLayout(this);
     gl->setMargin(gl->spacing());
     lab = new QColorShowLabel(this);
+#ifndef Q_OS_WINCE    
     lab->setMinimumWidth(60);
+#else    
+    lab->setMinimumWidth(20);
+#endif    
     gl->addWidget(lab, 0, 0, -1, 1);
     connect(lab, SIGNAL(colorDropped(QRgb)),
              this, SIGNAL(newCol(QRgb)));
@@ -1263,7 +1275,6 @@ public:
     QColorShower *cs;
     QLabel *lblBasicColors;
     QLabel *lblCustomColors;
-    QPushButton *custbut;
     QPushButton *ok;
     QPushButton *cancel;
     QPushButton *addCusBt;
@@ -1321,10 +1332,19 @@ void QColorDialogPrivate::init()
         compact = true;
 
     nextCust = 0;
-    QHBoxLayout *topLay = new QHBoxLayout(q);
-    const int lumSpace = topLay->spacing() / 2;
+    QVBoxLayout *mainLay = new QVBoxLayout(q);
+    QHBoxLayout *topLay = new QHBoxLayout();
+    mainLay->addLayout(topLay);
+
     QVBoxLayout *leftLay = 0;
 
+#if defined(Q_OS_WINCE)
+    compact = true;
+    const int lumSpace = 20;
+#else
+     const int lumSpace = topLay->spacing() / 2;
+#endif
+
     if (!compact) {
         leftLay = new QVBoxLayout;
         topLay->addLayout(leftLay);
@@ -1342,8 +1362,9 @@ void QColorDialogPrivate::init()
         leftLay->addWidget(lblBasicColors);
         leftLay->addWidget(standard);
 
-
+#if !defined(Q_OS_WINCE)
         leftLay->addStretch();
+#endif
 
         custom = new QColorWell(q, 2, 8, cusrgb);
         custom->setAcceptDrops(true);
@@ -1356,9 +1377,9 @@ void QColorDialogPrivate::init()
         leftLay->addWidget(lblCustomColors);
         leftLay->addWidget(custom);
 
-        custbut = new QPushButton(q);
-        custbut->setEnabled(false);
-        leftLay->addWidget(custbut);
+        addCusBt = new QPushButton(q);
+        QObject::connect(addCusBt, SIGNAL(clicked()), q, SLOT(_q_addCustom()));
+        leftLay->addWidget(addCusBt);
     } else {
         // better color picker size for small displays
         pWidth = 150;
@@ -1377,6 +1398,7 @@ void QColorDialogPrivate::init()
     QVBoxLayout *cLay = new QVBoxLayout;
     pickLay->addLayout(cLay);
     cp = new QColorPicker(q);
+
     cp->setFrameStyle(QFrame::Panel + QFrame::Sunken);
     cLay->addSpacing(lumSpace);
     cLay->addWidget(cp);
@@ -1395,29 +1417,16 @@ void QColorDialogPrivate::init()
     QObject::connect(cs, SIGNAL(newCol(QRgb)), q, SLOT(_q_newColorTypedIn(QRgb)));
     rightLay->addWidget(cs);
 
-    QHBoxLayout *buttons;
-    if (compact) {
-        buttons = new QHBoxLayout;
-        rightLay->addLayout(buttons);
-    } else {
-        buttons = new QHBoxLayout;
-        leftLay->addLayout(buttons);
-    }
+    QDialogButtonBox *buttons;
+    buttons = new QDialogButtonBox(q);
+    mainLay->addWidget(buttons);
 
-    ok = new QPushButton(q);
+    ok = buttons->addButton(QDialogButtonBox::Ok);
     QObject::connect(ok, SIGNAL(clicked()), q, SLOT(accept()));
     ok->setDefault(true);
-    cancel = new QPushButton(q);
+    cancel = buttons->addButton(QDialogButtonBox::Cancel);
     QObject::connect(cancel, SIGNAL(clicked()), q, SLOT(reject()));
-    buttons->addWidget(ok);
-    buttons->addWidget(cancel);
-    buttons->addStretch();
 
-    if (!compact) {
-        addCusBt = new QPushButton(q);
-        rightLay->addWidget(addCusBt);
-        QObject::connect(addCusBt, SIGNAL(clicked()), q, SLOT(_q_addCustom()));
-    }
     retranslateStrings();
 }
 
@@ -1434,11 +1443,8 @@ void QColorDialogPrivate::retranslateStrings()
     if (!compact) {
         lblBasicColors->setText(QColorDialog::tr("&Basic colors"));
         lblCustomColors->setText(QColorDialog::tr("&Custom colors"));
-        custbut->setText(QColorDialog::tr("&Define Custom Colors >>"));
         addCusBt->setText(QColorDialog::tr("&Add to Custom Colors"));
     }
-    ok->setText(QColorDialog::tr("OK"));
-    cancel->setText(QColorDialog::tr("Cancel"));
 
     cs->retranslateStrings();
 }
@@ -1708,10 +1714,12 @@ bool QColorDialog::selectColor(const QColor& col)
     return false;
 }
 
+QT_END_NAMESPACE
+
 #include "qcolordialog.moc"
 #include "moc_qcolordialog.cpp"
 
-#endif
+#endif // QT_NO_COLORDIALOG
 
 /*!
     \fn QColor QColorDialog::getColor(const QColor &init, QWidget *parent, const char *name)
