@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -54,6 +59,8 @@
 #include <private/qdialog_p.h>
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
+
 // If the operation is expected to take this long (as predicted by
 // progress time), show the progress dialog.
 static const int defaultShowTime = 4000;
@@ -161,7 +168,8 @@ void QProgressDialogPrivate::layout()
             cs.width(), cs.height());
     }
 
-    label->setGeometry(mlr, 0, q->width()-mlr*2, lh);
+    if (label)
+        label->setGeometry(mlr, 0, q->width()-mlr*2, lh);
     bar->setGeometry(mlr, lh+sp, q->width()-mlr*2, bh.height());
 }
 
@@ -209,9 +217,7 @@ void QProgressDialogPrivate::retranslateStrings()
   to use for the programmer. Do the operation in a loop, call \l setValue() at
   intervals, and check for cancellation with wasCanceled(). For example:
 
-  \quotefromfile snippets/dialogs/dialogs.cpp
-  \skipto QProgressDialog progress("Copying files...", "Abort Copy", 0, numFiles, this);
-  \printuntil progress.setValue(numFiles);
+  \snippet doc/src/snippets/dialogs/dialogs.cpp 3
 
   A modeless progress dialog is suitable for operations that take
   place in the background, where the user is able to interact with the
@@ -224,10 +230,11 @@ void QProgressDialogPrivate::retranslateStrings()
   canceled() signal to a slot that stops the operation, and call \l
   setValue() at intervals. For example:
 
-  \skipto Operation constructor
-  \printuntil }
-  \printuntil }
-  \printuntil }
+  \snippet doc/src/snippets/dialogs/dialogs.cpp 4
+  \codeline
+  \snippet doc/src/snippets/dialogs/dialogs.cpp 5
+  \codeline
+  \snippet doc/src/snippets/dialogs/dialogs.cpp 6
 
   In both modes the progress dialog may be customized by
   replacing the child widgets with custom widgets by using setLabel(),
@@ -235,9 +242,6 @@ void QProgressDialogPrivate::retranslateStrings()
   The functions setLabelText() and setCancelButtonText()
   set the texts shown.
 
-  The \l{dialogs/standarddialogs}{Standard Dialogs} example shows
-  how to use QProgressDialog as well as other built-in Qt dialogs.
-
   \image plastique-progressdialog.png A progress dialog shown in the Plastique widget style.
 
   \sa QDialog, QProgressBar, {fowler}{GUI Design Handbook: Progress Indicator},
@@ -286,8 +290,8 @@ QProgressDialog::QProgressDialog(QWidget *parent, Qt::WindowFlags f)
    setValue(0). As each file is processed call setValue(1), setValue(2),
    etc., finally calling setValue(50) after examining the last file.
 
-   The \a parent argument is the dialog's parent widget. The  and widget flags,
-   \a f, are passed to the QDialog::QDialog() constructor.
+   The \a parent argument is the dialog's parent widget. The parent, \a parent, and
+   widget flags, \a f, are passed to the QDialog::QDialog() constructor.
 
   \sa setLabelText(), setLabel(), setCancelButtonText(), setCancelButton(),
   setMinimum(), setMaximum()
@@ -448,6 +452,10 @@ void QProgressDialog::setCancelButtonText(const QString &cancelButtonText)
 void QProgressDialog::setBar(QProgressBar *bar)
 {
     Q_D(QProgressDialog);
+    if (!bar) {
+        qWarning("QProgressDialog::setBar: Cannot set a null progress bar");
+        return;
+    }
 #ifndef QT_NO_DEBUG
     if (value() > 0)
         qWarning("QProgressDialog::setBar: Cannot set a new progress bar "
@@ -659,7 +667,7 @@ void QProgressDialog::setValue(int progress)
 QSize QProgressDialog::sizeHint() const
 {
     Q_D(const QProgressDialog);
-    QSize sh = d->label->sizeHint();
+    QSize sh = d->label ? d->label->sizeHint() : QSize(0, 0);
     QSize bh = d->bar->sizeHint();
     int margin = style()->pixelMetric(QStyle::PM_DefaultTopLevelMargin);
     int spacing = style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
@@ -804,4 +812,6 @@ void QProgressDialog::forceShow()
     d->shown_once = true;
 }
 
-#endif
+QT_END_NAMESPACE
+
+#endif // QT_NO_PROGRESSDIALOG
