@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -70,9 +70,7 @@
 
     An example animation with a timeline follows:
 
-    \quotefromfile snippets/timeline/main.cpp
-    \skipto QGraphicsItem *ball
-    \printuntil timer->start();
+    \snippet doc/src/snippets/timeline/main.cpp 0
     
     Note that steps lie between 0.0 and 1.0. It may be necessary to use
     \l{QTimeLine::}{setUpdateInterval()}. The default update interval
@@ -95,6 +93,8 @@
 #include <QtCore/qpair.h>
 #include <QtGui/qmatrix.h>
 
+QT_BEGIN_NAMESPACE
+
 class QGraphicsItemAnimationPrivate
 {
 public:
@@ -555,7 +555,11 @@ void QGraphicsItemAnimation::setStep(qreal x)
 }
 
 /*!
-Resets the item to its starting position and transformation.
+    Resets the item to its starting position and transformation.
+
+    \obsolete
+
+    You can call setStep(0) instead.
 */
 void QGraphicsItemAnimation::reset()
 {
@@ -591,4 +595,6 @@ void QGraphicsItemAnimation::afterAnimationStep(qreal step)
     Q_UNUSED(step);
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_GRAPHICSVIEW
