@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,7 +52,7 @@
     drop events (QMouseEvent, QKeyEvent, QDragEvent, etc.), it
     translates them into instances of QGraphicsSceneEvent subclasses
     and forwards them to the QGraphicsScene it displays. The scene
-    then forwards the events to the relevent items.
+    then forwards the events to the relevant items.
 
     For example, when a QGraphicsView receives a QMouseEvent of type
     MousePress as a response to a user click, the view sends a
@@ -88,14 +108,15 @@
     \class QGraphicsSceneWheelEvent
     \brief The QGraphicsSceneWheelEvent class provides wheel events 
 	   in the graphics view framework.
+    \brief The QGraphicsSceneWheelEvent class provides wheel events in the
+    graphics view framework.
     \since 4.2
     \ingroup multimedia
 
-    \l{QWheelEvent}{QWheelEvent}s received by a QGraphicsView are
-    translated into
-    \l{QGraphicsSceneWheelEvent}{QGraphicsSceneWheelEvents}. The
-    QWheelEvent::globalPos() is translated into item, scene, and
-    screen coordinates (pos(), scenePos(), and screenPos()).
+    \l{QWheelEvent}{QWheelEvent}s received by a QGraphicsView are translated
+    into QGraphicsSceneWheelEvents; it translates the QWheelEvent::globalPos()
+    into item, scene, and screen coordinates (pos(), scenePos(), and
+    screenPos()).
 
     \sa QGraphicsSceneMouseEvent, QGraphicsSceneContextMenuEvent,
     QGraphicsSceneHoverEvent, QWheelEvent
@@ -114,7 +135,7 @@
     screen coordinates (pos(), scenePos(), and screenPos()).
 
     \sa QGraphicsSceneMouseEvent, QGraphicsSceneWheelEvent,
-    QGraphicsSceneContextMenuEvent, QContextMenuEvent
+    QContextMenuEvent
 */
 
 /*!
@@ -143,8 +164,8 @@
     it into QGraphicsSceneHoverEvent. The event is then forwarded to
     the QGraphicsScene associated with the view.
 
-    \sa QGraphicsSceneMouseEvent, QGraphicsViewContextMenuEvent,
-    QGraphicsSceneWheelEvent, QHoverEvent
+    \sa QGraphicsSceneMouseEvent, QGraphicsSceneContextMenuEvent,
+        QGraphicsSceneWheelEvent, QHoverEvent
 */
 
 /*!
@@ -195,6 +216,40 @@
     drops with \l{QGraphicsItem::}{setAcceptDrops()}.
 */
 
+/*!
+    \class QGraphicsSceneResizeEvent
+    \brief The QGraphicsSceneResizeEvent class provides events for widget
+    resizing in the graphics view framework.
+    \since 4.4
+    \ingroup multimedia
+
+    A QGraphicsWidget sends itself a QGraphicsSceneResizeEvent immediately
+    when its geometry changes.
+
+    It's similar to QResizeEvent, but its sizes, oldSize() and newSize(), use
+    QSizeF instead of QSize.
+
+    \sa QGraphicsWidget::setGeometry(), QGraphicsWidget::resize()
+*/
+
+/*!
+    \class QGraphicsSceneMoveEvent
+    \brief The QGraphicsSceneMoveEvent class provides events for widget
+    moving in the graphics view framework.
+    \since 4.4
+    \ingroup multimedia
+
+    A QGraphicsWidget sends itself a QGraphicsSceneMoveEvent immediately when
+    its local position changes. The delivery is implemented as part of
+    QGraphicsItem::itemChange().
+
+    It's similar to QMoveEvent, but its positions, oldPos() and newPos(), use
+    QPointF instead of QPoint.
+
+    \sa QGraphicsItem::setPos(), QGraphicsItem::ItemPositionChange,
+    QGraphicsItem::ItemPositionHasChanged
+*/
+
 #include "qgraphicssceneevent.h"
 
 #ifndef QT_NO_GRAPHICSVIEW
@@ -203,8 +258,12 @@
 #include <QtCore/qdebug.h>
 #endif
 #include <QtCore/qmap.h>
+#include <QtCore/qpoint.h>
+#include <QtCore/qsize.h>
 #include <QtCore/qstring.h>
 
+QT_BEGIN_NAMESPACE
+
 class QGraphicsSceneEventPrivate
 {
 public:
@@ -253,8 +312,6 @@ QGraphicsSceneEvent::~QGraphicsSceneEvent()
 /*!
     Returns the widget where the event originated, or 0 if the event
     originates from another application.
-
-    \sa setWidget()
 */
 QWidget *QGraphicsSceneEvent::widget() const
 {
@@ -458,9 +515,10 @@ void QGraphicsSceneMouseEvent::setLastPos(const QPointF &pos)
 }
 
 /*!
-    Returns the last recorded, the scene coordinates of the
-    previous mouse event received by the view, that created the
-    event mouse cursor position in scene coordinates.
+    Returns the last recorded mouse cursor position in scene 
+    coordinates. The last recorded position is the position of
+    the previous mouse event received by the view that created
+    the event.
 
     \sa lastPos(), lastScreenPos(), scenePos()
 */
@@ -569,7 +627,7 @@ class QGraphicsSceneWheelEventPrivate : public QGraphicsSceneEventPrivate
     Q_DECLARE_PUBLIC(QGraphicsSceneWheelEvent)
 public:
     inline QGraphicsSceneWheelEventPrivate()
-        : buttons(0), modifiers(0), delta(0)
+        : buttons(0), modifiers(0), delta(0), orientation(Qt::Horizontal)
     { }
 
     QPointF pos;
@@ -578,6 +636,7 @@ public:
     Qt::MouseButtons buttons;
     Qt::KeyboardModifiers modifiers;
     int delta;
+    Qt::Orientation orientation;
 };
 
 /*!
@@ -602,7 +661,7 @@ QGraphicsSceneWheelEvent::~QGraphicsSceneWheelEvent()
     Returns the position of the cursor in item coordinates when the
     wheel event occurred.
 
-    \sa setPos(), scenePos(), screenPos()
+    \sa scenePos(), screenPos()
 */
 QPointF QGraphicsSceneWheelEvent::pos() const
 {
@@ -620,10 +679,10 @@ void QGraphicsSceneWheelEvent::setPos(const QPointF &pos)
 }
 
 /*!
-    Returns the position of the cursor in item coordinates when the wheel
+    Returns the position of the cursor in scene coordinates when the wheel
     event occurred.
 
-    \sa setScenePos(), pos(), screenPos()
+    \sa pos(), screenPos()
 */
 QPointF QGraphicsSceneWheelEvent::scenePos() const
 {
@@ -644,7 +703,7 @@ void QGraphicsSceneWheelEvent::setScenePos(const QPointF &pos)
     Returns the position of the cursor in screen coordinates when the wheel
     event occurred.
 
-    \sa setScreenPos(), pos(), scenePos()
+    \sa pos(), scenePos()
 */
 QPoint QGraphicsSceneWheelEvent::screenPos() const
 {
@@ -664,7 +723,7 @@ void QGraphicsSceneWheelEvent::setScreenPos(const QPoint &pos)
 /*!
     Returns the mouse buttons that were pressed when the wheel event occurred.
 
-    \sa setButtons(), modifiers()
+    \sa modifiers()
 */
 Qt::MouseButtons QGraphicsSceneWheelEvent::buttons() const
 {
@@ -685,7 +744,7 @@ void QGraphicsSceneWheelEvent::setButtons(Qt::MouseButtons buttons)
     Returns the keyboard modifiers that were active when the wheel event
     occurred.
 
-    \sa setModifiers(), buttons()
+    \sa buttons()
 */
 Qt::KeyboardModifiers QGraphicsSceneWheelEvent::modifiers() const
 {
@@ -710,8 +769,6 @@ void QGraphicsSceneWheelEvent::setModifiers(Qt::KeyboardModifiers modifiers)
 
     Most mouse types work in steps of 15 degrees, in which case the delta
     value is a multiple of 120 (== 15 * 8).
-
-    \sa setDelta()
 */
 int QGraphicsSceneWheelEvent::delta() const
 {
@@ -728,6 +785,24 @@ void QGraphicsSceneWheelEvent::setDelta(int delta)
     d->delta = delta;
 }
 
+/*!
+    Returns the wheel orientation.
+*/
+Qt::Orientation QGraphicsSceneWheelEvent::orientation() const
+{
+    Q_D(const QGraphicsSceneWheelEvent);
+    return d->orientation;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneWheelEvent::setOrientation(Qt::Orientation orientation)
+{
+    Q_D(QGraphicsSceneWheelEvent);
+    d->orientation = orientation;
+}
+
 class QGraphicsSceneContextMenuEventPrivate : public QGraphicsSceneEventPrivate
 {
     Q_DECLARE_PUBLIC(QGraphicsSceneContextMenuEvent)
@@ -859,7 +934,7 @@ void QGraphicsSceneContextMenuEvent::setModifiers(Qt::KeyboardModifiers modifier
 /*!
     Returns the reason for the context menu event.
 
-    \sa setReason(), QGraphicsContextMenu::Reason
+    \sa QGraphicsSceneContextMenuEvent::Reason
 */
 QGraphicsSceneContextMenuEvent::Reason QGraphicsSceneContextMenuEvent::reason() const
 {
@@ -885,6 +960,10 @@ public:
     QPointF pos;
     QPointF scenePos;
     QPoint screenPos;
+    QPointF lastPos;
+    QPointF lastScenePos;
+    QPoint lastScreenPos;
+    Qt::KeyboardModifiers modifiers;
 };
 
 /*!
@@ -979,6 +1058,91 @@ void QGraphicsSceneHoverEvent::setScreenPos(const QPoint &pos)
     d->screenPos = pos;
 }
 
+/*!
+    Returns the last recorded mouse cursor position in item coordinates.
+
+    \sa lastScenePos(), lastScreenPos(), pos()
+*/
+QPointF QGraphicsSceneHoverEvent::lastPos() const
+{
+    Q_D(const QGraphicsSceneHoverEvent);
+    return d->lastPos;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneHoverEvent::setLastPos(const QPointF &pos)
+{
+    Q_D(QGraphicsSceneHoverEvent);
+    d->lastPos = pos;
+}
+
+/*!
+    Returns the last recorded, the scene coordinates of the previous mouse or
+    hover event received by the view, that created the event mouse cursor
+    position in scene coordinates.
+
+    \sa lastPos(), lastScreenPos(), scenePos()
+*/
+QPointF QGraphicsSceneHoverEvent::lastScenePos() const
+{
+    Q_D(const QGraphicsSceneHoverEvent);
+    return d->lastScenePos;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneHoverEvent::setLastScenePos(const QPointF &pos)
+{
+    Q_D(QGraphicsSceneHoverEvent);
+    d->lastScenePos = pos;
+}
+
+/*!
+    Returns the last recorded mouse cursor position in screen coordinates. The
+    last recorded position is the position of the previous mouse or hover
+    event received by the view that created the event.
+
+    \sa lastPos(), lastScenePos(), screenPos()
+*/
+QPoint QGraphicsSceneHoverEvent::lastScreenPos() const
+{
+    Q_D(const QGraphicsSceneHoverEvent);
+    return d->lastScreenPos;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneHoverEvent::setLastScreenPos(const QPoint &pos)
+{
+    Q_D(QGraphicsSceneHoverEvent);
+    d->lastScreenPos = pos;
+}
+
+/*!
+    Returns the keyboard modifiers at the moment the the hover event was sent.
+*/
+Qt::KeyboardModifiers QGraphicsSceneHoverEvent::modifiers() const
+{
+    Q_D(const QGraphicsSceneHoverEvent);
+    return d->modifiers;
+}
+
+/*!
+    \fn void QGraphicsSceneHoverEvent::setModifiers(Qt::KeyboardModifiers modifiers)
+    \internal
+
+    Sets the modifiers for the current hover event to \a modifiers.
+*/
+void QGraphicsSceneHoverEvent::setModifiers(Qt::KeyboardModifiers modifiers)
+{
+    Q_D(QGraphicsSceneHoverEvent);
+    d->modifiers = modifiers;
+}
+
 class QGraphicsSceneHelpEventPrivate : public QGraphicsSceneEventPrivate
 {
 public:
@@ -1171,7 +1335,7 @@ void QGraphicsSceneDragDropEvent::setScreenPos(const QPoint &pos)
     were pressed on the mouse when this mouse event was
     generated.
 
-    \sa Qt::MouseButtons, button()
+    \sa Qt::MouseButtons
 */
 Qt::MouseButtons QGraphicsSceneDragDropEvent::buttons() const
 {
@@ -1277,11 +1441,7 @@ void QGraphicsSceneDragDropEvent::setProposedAction(Qt::DropAction action)
     Sets the proposed action as accepted, i.e, the drop action
     is set to the proposed action. This is equal to:
 
-    \code
-
-    setDropAction(proposedAction());
-
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.graphicsview.qgraphicssceneevent.cpp 0
 
     When using this function, one should not call \c accept().
 
@@ -1337,7 +1497,7 @@ QWidget *QGraphicsSceneDragDropEvent::source() const
 /*!
     \internal
     This function set the source widget, i.e., the widget that
-    created the drop event, to \source.
+    created the drop event, to \a source.
 */
 void QGraphicsSceneDragDropEvent::setSource(QWidget *source)
 {
@@ -1364,4 +1524,140 @@ void QGraphicsSceneDragDropEvent::setMimeData(const QMimeData *data)
     d->mimeData = data;
 }
 
+class QGraphicsSceneResizeEventPrivate : public QGraphicsSceneEventPrivate
+{
+    Q_DECLARE_PUBLIC(QGraphicsSceneResizeEvent)
+public:
+    inline QGraphicsSceneResizeEventPrivate()
+    { }
+
+    QSizeF oldSize;
+    QSizeF newSize;
+};
+
+/*!
+    Constructs a QGraphicsSceneResizeEvent.
+*/
+QGraphicsSceneResizeEvent::QGraphicsSceneResizeEvent()
+    : QGraphicsSceneEvent(*new QGraphicsSceneResizeEventPrivate, QEvent::GraphicsSceneResize)
+{
+}
+
+/*!
+    Destroys the QGraphicsSceneResizeEvent.
+*/
+QGraphicsSceneResizeEvent::~QGraphicsSceneResizeEvent()
+{
+}
+
+/*!
+    Returns the old size (i.e., the size immediately before the widget was
+    resized).
+
+    \sa newSize(), QGraphicsWidget::resize()
+*/
+QSizeF QGraphicsSceneResizeEvent::oldSize() const
+{
+    Q_D(const QGraphicsSceneResizeEvent);
+    return d->oldSize;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneResizeEvent::setOldSize(const QSizeF &size)
+{
+    Q_D(QGraphicsSceneResizeEvent);
+    d->oldSize = size;
+}
+
+/*!
+    Returns the new size (i.e., the current size).
+
+    \sa oldSize(), QGraphicsWidget::resize()
+*/
+QSizeF QGraphicsSceneResizeEvent::newSize() const
+{
+    Q_D(const QGraphicsSceneResizeEvent);
+    return d->newSize;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneResizeEvent::setNewSize(const QSizeF &size)
+{
+    Q_D(QGraphicsSceneResizeEvent);
+    d->newSize = size;
+}
+
+class QGraphicsSceneMoveEventPrivate : public QGraphicsSceneEventPrivate
+{
+    Q_DECLARE_PUBLIC(QGraphicsSceneMoveEvent)
+public:
+    inline QGraphicsSceneMoveEventPrivate()
+    { }
+
+    QPointF oldPos;
+    QPointF newPos;
+};
+
+/*!
+    Constructs a QGraphicsSceneMoveEvent.
+*/
+QGraphicsSceneMoveEvent::QGraphicsSceneMoveEvent()
+    : QGraphicsSceneEvent(*new QGraphicsSceneMoveEventPrivate, QEvent::GraphicsSceneMove)
+{
+}
+
+/*!
+    Destroys the QGraphicsSceneMoveEvent.
+*/
+QGraphicsSceneMoveEvent::~QGraphicsSceneMoveEvent()
+{
+}
+
+/*!
+    Returns the old position (i.e., the position immediatly before the widget
+    was moved).
+
+    \sa newPos(), QGraphicsItem::setPos()
+*/
+QPointF QGraphicsSceneMoveEvent::oldPos() const
+{
+    Q_D(const QGraphicsSceneMoveEvent);
+    return d->oldPos;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneMoveEvent::setOldPos(const QPointF &pos)
+{
+    Q_D(QGraphicsSceneMoveEvent);
+    d->oldPos = pos;
+}
+
+/*!
+    Returns the new position (i.e., the current position).
+
+    \sa oldPos(), QGraphicsItem::setPos()
+*/
+QPointF QGraphicsSceneMoveEvent::newPos() const
+{
+    Q_D(const QGraphicsSceneMoveEvent);
+    return d->newPos;
+}
+
+/*!
+    \internal
+*/
+void QGraphicsSceneMoveEvent::setNewPos(const QPointF &pos)
+{
+    Q_D(QGraphicsSceneMoveEvent);
+    d->newPos = pos;
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_GRAPHICSVIEW
