@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@
 #include <qvariant.h>
 #include <qvector.h>
 
+QT_BEGIN_NAMESPACE
+
 static void swapPixel01(QImage *image)        // 1-bpp: swap 0 and 1 pixels
 {
     int i;
@@ -728,9 +730,24 @@ bool QBmpHandler::read(QImage *image)
 
 bool QBmpHandler::write(const QImage &img)
 {
-    QImage image = img;
-    if (image.format() == QImage::Format_RGB16)
-        image = image.convertToFormat(QImage::Format_RGB32);
+    QImage image;
+    switch (img.format()) {
+    case QImage::Format_ARGB8565_Premultiplied:
+    case QImage::Format_ARGB8555_Premultiplied:
+    case QImage::Format_ARGB6666_Premultiplied:
+    case QImage::Format_ARGB4444_Premultiplied:
+        image = img.convertToFormat(QImage::Format_ARGB32);
+        break;
+    case QImage::Format_RGB16:
+    case QImage::Format_RGB888:
+    case QImage::Format_RGB666:
+    case QImage::Format_RGB555:
+    case QImage::Format_RGB444:
+        image = img.convertToFormat(QImage::Format_RGB32);
+        break;
+    default:
+        image = img;
+    }
 
     QIODevice *d = device();
     QDataStream s(d);
@@ -792,4 +809,6 @@ QByteArray QBmpHandler::name() const
     return "bmp";
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_IMAGEFORMAT_BMP
