@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,6 +54,8 @@
 #include "qvariant.h"
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \enum QIcon::Mode
 
@@ -52,6 +74,9 @@
         functionality represented by the icon is available and
         the user is interacting with the icon, for example, moving the
         mouse over it or clicking it.
+   \value Selected
+        Display the pixmap when the item represented by the icon is
+        selected.
 */
 
 /*!
@@ -64,16 +89,21 @@
   \value On  Display the pixmap when the widget is in an "on" state
 */
 
-static int serialNumCounter = 0;
+extern Q_GUI_EXPORT qint64 qt_pixmap_id(const QPixmap &pixmap);
+
+static QBasicAtomicInt serialNumCounter = Q_BASIC_ATOMIC_INITIALIZER(1);
 
 class QIconPrivate
 {
 public:
-    QIconPrivate():ref(1),engine(0),serialNum(++serialNumCounter){}
+    QIconPrivate():ref(1), engine(0), serialNum(serialNumCounter.fetchAndAddRelaxed(1)), detach_no(0), engine_version(2) {}
+
     ~QIconPrivate() { delete engine; }
-    QAtomic ref;
+    QAtomicInt ref;
     QIconEngine *engine;
     int serialNum;
+    int detach_no;
+    int engine_version;
 };
 
 
@@ -92,9 +122,10 @@ struct QPixmapIconEngineEntry
     bool isNull() const {return (fileName.isEmpty() && pixmap.isNull()); }
 };
 
-class QPixmapIconEngine : public QIconEngine{
+class QPixmapIconEngine : public QIconEngineV2 {
 public:
     QPixmapIconEngine();
+    QPixmapIconEngine(const QPixmapIconEngine &);
     ~QPixmapIconEngine();
     void paint(QPainter *painter, const QRect &rect, QIcon::Mode mode, QIcon::State state);
     QPixmap pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state);
@@ -102,15 +133,29 @@ public:
     QSize actualSize(const QSize &size, QIcon::Mode mode, QIcon::State state);
     void addPixmap(const QPixmap &pixmap, QIcon::Mode mode, QIcon::State state);
     void addFile(const QString &fileName, const QSize &size, QIcon::Mode mode, QIcon::State state);
+
+    // v2 functions
+    QString key() const;
+    QIconEngineV2 *clone() const;
+    bool read(QDataStream &in);
+    bool write(QDataStream &out) const;
+
 private:
     QPixmapIconEngineEntry *tryMatch(const QSize &size, QIcon::Mode mode, QIcon::State state);
     QVector<QPixmapIconEngineEntry> pixmaps;
+
+    friend QDataStream &operator<<(QDataStream &s, const QIcon &icon);
 };
 
 QPixmapIconEngine::QPixmapIconEngine()
 {
 }
 
+QPixmapIconEngine::QPixmapIconEngine(const QPixmapIconEngine &other)
+    : QIconEngineV2(other), pixmaps(other.pixmaps)
+{
+}
+
 QPixmapIconEngine::~QPixmapIconEngine()
 {
 }
@@ -165,17 +210,22 @@ QPixmapIconEngineEntry *QPixmapIconEngine::bestMatch(const QSize &size, QIcon::M
     QPixmapIconEngineEntry *pe = tryMatch(size, mode, state);
     while (!pe){
         QIcon::State oppositeState = (state == QIcon::On) ? QIcon::Off : QIcon::On;
-        if (mode == QIcon::Disabled) {
+        if (mode == QIcon::Disabled || mode == QIcon::Selected) {
+            QIcon::Mode oppositeMode = (mode == QIcon::Disabled) ? QIcon::Selected : QIcon::Disabled;
             if ((pe = tryMatch(size, QIcon::Normal, state)))
                 break;
             if ((pe = tryMatch(size, QIcon::Active, state)))
                 break;
-            if ((pe = tryMatch(size, QIcon::Disabled, oppositeState)))
+            if ((pe = tryMatch(size, mode, oppositeState)))
                 break;
             if ((pe = tryMatch(size, QIcon::Normal, oppositeState)))
                 break;
             if ((pe = tryMatch(size, QIcon::Active, oppositeState)))
                 break;
+            if ((pe = tryMatch(size, oppositeMode, state)))
+                break;
+            if ((pe = tryMatch(size, oppositeMode, oppositeState)))
+                break;
         } else {
             QIcon::Mode oppositeMode = (mode == QIcon::Normal) ? QIcon::Active : QIcon::Normal;
             if ((pe = tryMatch(size, oppositeMode, state)))
@@ -186,8 +236,12 @@ QPixmapIconEngineEntry *QPixmapIconEngine::bestMatch(const QSize &size, QIcon::M
                 break;
             if ((pe = tryMatch(size, QIcon::Disabled, state)))
                 break;
+            if ((pe = tryMatch(size, QIcon::Selected, state)))
+                break;
             if ((pe = tryMatch(size, QIcon::Disabled, oppositeState)))
                 break;
+            if ((pe = tryMatch(size, QIcon::Selected, oppositeState)))
+                break;
         }
 
         if (!pe)
@@ -214,11 +268,12 @@ QPixmap QPixmapIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::St
         return pm;
 
     QSize actualSize = pm.size();
-    if (!actualSize.isNull() && (actualSize.width() > size.width() && actualSize.height() > size.height()))
+    if (!actualSize.isNull() && (actualSize.width() > size.width() || actualSize.height() > size.height()))
         actualSize.scale(size, Qt::KeepAspectRatio);
 
     QString key = QLatin1String("$qt_icon_")
-                  + QString::number(pm.serialNumber())
+                  + QString::number(qt_pixmap_id(pm))
+                  + QString::number(pe->mode)
                   + QString::number(actualSize.width())
                   + QLatin1Char('_')
                   + QString::number(actualSize.height())
@@ -232,21 +287,21 @@ QPixmap QPixmapIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::St
             QStyleOption opt(0);
             opt.palette = QApplication::palette();
             QPixmap active = QApplication::style()->generatedIconPixmap(QIcon::Active, pm, &opt);
-            if (pm.serialNumber() == active.serialNumber())
+            if (qt_pixmap_id(pm) == qt_pixmap_id(active))
                 return pm;
         }
     }
 
     if (!QPixmapCache::find(key + QString::number(mode), pm)) {
-        if (pe->mode != mode && mode != QIcon::Normal && pe->mode != QIcon::Disabled) {
+        if (pm.size() != actualSize)
+            pm = pm.scaled(actualSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+        if (pe->mode != mode && mode != QIcon::Normal) {
             QStyleOption opt(0);
             opt.palette = QApplication::palette();
             QPixmap generated = QApplication::style()->generatedIconPixmap(mode, pm, &opt);
             if (!generated.isNull())
                 pm = generated;
         }
-        if (pm.size() != actualSize)
-            pm = pm.scaled(actualSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
         QPixmapCache::insert(key + QString::number(mode), pm);
     }
     return pm;
@@ -261,32 +316,118 @@ QSize QPixmapIconEngine::actualSize(const QSize &size, QIcon::Mode mode, QIcon::
     if (actualSize.isNull())
         return actualSize;
 
-    if (!actualSize.isNull() && (actualSize.width() > size.width() && actualSize.height() > size.height()))
+    if (!actualSize.isNull() && (actualSize.width() > size.width() || actualSize.height() > size.height()))
         actualSize.scale(size, Qt::KeepAspectRatio);
     return actualSize;
 }
 
 void QPixmapIconEngine::addPixmap(const QPixmap &pixmap, QIcon::Mode mode, QIcon::State state)
 {
-    if (!pixmap.isNull())
-        pixmaps += QPixmapIconEngineEntry(pixmap, mode, state);
-}
-
-void QPixmapIconEngine::addFile(const QString &fileName, const QSize &size, QIcon::Mode mode, QIcon::State state)
-{
-    if (!fileName.isEmpty()) {
-        QString abs = fileName;
-        if (fileName.at(0) != QLatin1Char(':'))
-            abs = QFileInfo(fileName).absoluteFilePath();
-        pixmaps += QPixmapIconEngineEntry(abs, size, mode, state);
+    if (!pixmap.isNull()) {
+        QPixmapIconEngineEntry *pe = tryMatch(pixmap.size(), mode, state);
+        if(pe && pe->size == pixmap.size()) {
+            pe->pixmap = pixmap;
+            pe->fileName.clear();
+        } else {
+            pixmaps += QPixmapIconEngineEntry(pixmap, mode, state);
+        }
     }
 }
 
+void QPixmapIconEngine::addFile(const QString &fileName, const QSize &_size, QIcon::Mode mode, QIcon::State state)
+{
+    if (!fileName.isEmpty()) {
+        QSize size = _size;
+        QPixmap pixmap;
 
+        QString abs = fileName;
+        if (fileName.at(0) != QLatin1Char(':'))
+            abs = QFileInfo(fileName).absoluteFilePath();
 
-#ifndef QT_NO_LIBRARY
+        for (int i = 0; i < pixmaps.count(); ++i) {
+            if (pixmaps.at(i).mode == mode && pixmaps.at(i).state == state) {
+                QPixmapIconEngineEntry *pe = &pixmaps[i];
+                if(size == QSize()) {
+                    pixmap = QPixmap(abs);
+                    size = pixmap.size();
+                }
+                if (pe->size == QSize() && pe->pixmap.isNull()) {
+                    pe->pixmap = QPixmap(pe->fileName);
+                    pe->size = pe->pixmap.size();
+                }
+                if(pe->size == size) {
+                    pe->pixmap = pixmap;
+                    pe->fileName = abs;
+                    return;
+                }
+            }
+        }
+        QPixmapIconEngineEntry e(abs, size, mode, state);
+        e.pixmap = pixmap;
+        pixmaps += e;
+    }
+}
+
+QString QPixmapIconEngine::key() const
+{
+    return QLatin1String("QPixmapIconEngine");
+}
+
+QIconEngineV2 *QPixmapIconEngine::clone() const
+{
+    return new QPixmapIconEngine(*this);
+}
+
+bool QPixmapIconEngine::read(QDataStream &in)
+{
+    int num_entries;
+    QPixmap pm;
+    QString fileName;
+    QSize sz;
+    uint mode;
+    uint state;
+
+    in >> num_entries;
+    for (int i=0; i < num_entries; ++i) {
+        if (in.atEnd()) {
+            pixmaps.clear();
+            return false;
+        }
+        in >> pm;
+        in >> fileName;
+        in >> sz;
+        in >> mode;
+        in >> state;
+        if (pm.isNull())
+            addFile(fileName, sz, QIcon::Mode(mode), QIcon::State(state));
+        else
+            addPixmap(pm, QIcon::Mode(mode), QIcon::State(state));
+    }
+    return true;
+}
+
+bool QPixmapIconEngine::write(QDataStream &out) const
+{
+    int num_entries = pixmaps.size();
+    out << num_entries;
+    for (int i=0; i < num_entries; ++i) {
+        if (pixmaps.at(i).pixmap.isNull())
+            out << QPixmap(pixmaps.at(i).fileName);
+        else
+            out << pixmaps.at(i).pixmap;
+        out << pixmaps.at(i).fileName;
+        out << pixmaps.at(i).size;
+        out << (uint) pixmaps.at(i).mode;
+        out << (uint) pixmaps.at(i).state;
+    }
+    return true;
+}
+
+#if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
-    (QIconEngineFactoryInterface_iid, QCoreApplication::libraryPaths(), "/iconengines", Qt::CaseInsensitive))
+    (QIconEngineFactoryInterface_iid, QLatin1String("/iconengines"), Qt::CaseInsensitive))
+Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loaderV2,
+    (QIconEngineFactoryInterfaceV2_iid, QLatin1String("/iconengines"), Qt::CaseInsensitive))
 #endif
 
 
@@ -309,10 +450,11 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
   resource, and then use it, allowing Qt to work out all the required
   icon styles and sizes. For example:
 
-  \code
-    QToolButton *button = new QToolButton;
-    button->setIcon(QIcon("open.xpm"));
-  \endcode
+  \snippet doc/src/snippets/code/src.gui.image.qicon.cpp 0
+
+  To undo a QIcon, simply set a null icon in its place:
+
+  \snippet doc/src/snippets/code/src.gui.image.qicon.cpp 1
 
   Use the QImageReader::supportedImageFormats() and
   QImageWriter::supportedImageFormats() functions to retrieve a
@@ -321,13 +463,16 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
   When you retrieve a pixmap using pixmap(QSize, Mode, State), and no
   pixmap for this given size, mode and state has been added with
   addFile() or addPixmap(), then QIcon will generate one on the
-  fly. This pixmap generation happens in a QIconEngine. The default
+  fly. This pixmap generation happens in a QIconEngineV2. The default
   engine scales pixmaps down if required, but never up, and it uses
   the current style to calculate a disabled appearance. By using
   custom icon engines, you can customize every aspect of generated
-  icons. With QIconEnginePlugin it is possible to register different
-  icon engines for different file suffixes, so you could provide a SVG
-  icon engine or any other scalable format.
+  icons. With QIconEnginePluginV2 it is possible to register different
+  icon engines for different file suffixes, making it possible for
+  third parties to provide additional icon engines to those included
+  with Qt.
+
+  \note Since Qt 4.2, an icon engine that supports SVG is included.
 
   \section1 Making Classes that Use QIcon
 
@@ -338,17 +483,7 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
   Provide a method to set a QIcon, and when you draw the icon, choose
   whichever pixmap is appropriate for the current state of your widget.
   For example:
-  \code
-    void MyWidget::drawIcon(QPainter *painter, QPoint pos)
-    {
-        QPixmap pixmap = icon.pixmap(QSize(22, 22),
-                                       isEnabled() ? QIcon::Normal
-                                                   : QIcon::Disabled,
-                                       isOn() ? QIcon::On
-                                              : QIcon::Off);
-        painter->drawPixmap(pos, pixmap);
-    }
-  \endcode
+  \snippet doc/src/snippets/code/src.gui.image.qicon.cpp 2
 
   You might also make use of the \c Active mode, perhaps making your
   widget \c Active when the mouse is over the widget (see \l
@@ -359,7 +494,7 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
 
   \img icon.png QIcon
 
-  \sa {fowler}{GUI Design Handbook: Iconic Label}
+  \sa {fowler}{GUI Design Handbook: Iconic Label}, {Icons Example}
 */
 
 
@@ -411,17 +546,6 @@ QIcon::QIcon(const QIcon &other)
 QIcon::QIcon(const QString &fileName)
     : d(0)
 {
-    QFileInfo info(fileName);
-    QString suffix = info.suffix();
-#ifndef QT_NO_LIBRARY
-    if (!suffix.isEmpty())
-        if (QIconEngineFactoryInterface *factory = qobject_cast<QIconEngineFactoryInterface*>(loader()->instance(suffix)))
-            if (QIconEngine *engine = factory->create(fileName)) {
-                d = new QIconPrivate;
-                d->engine = engine;
-                return;
-            }
-#endif
     addFile(fileName);
 }
 
@@ -433,9 +557,22 @@ QIcon::QIcon(const QString &fileName)
 QIcon::QIcon(QIconEngine *engine)
     :d(new QIconPrivate)
 {
+    d->engine_version = 1;
     d->engine = engine;
 }
 
+/*!
+    Creates an icon with a specific icon \a engine. The icon takes
+    ownership of the engine.
+*/
+QIcon::QIcon(QIconEngineV2 *engine)
+    :d(new QIconPrivate)
+{
+    d->engine_version = 2;
+    d->engine = engine;
+}
+
+
 /*!
     Destroys the icon.
 */
@@ -451,12 +588,11 @@ QIcon::~QIcon()
 */
 QIcon &QIcon::operator=(const QIcon &other)
 {
-    QIconPrivate *x = other.d;
-    if (x)
-        x->ref.ref();
-    x = qAtomicSetPtr(&d, x);
-    if (x && !x->ref.deref())
-        delete x;
+    if (other.d)
+        other.d->ref.ref();
+    if (d && !d->ref.deref())
+        delete d;
+    d = other.d;
     return *this;
 }
 
@@ -468,13 +604,20 @@ QIcon::operator QVariant() const
     return QVariant(QVariant::Icon, this);
 }
 
-/*!
-    Returns a number that uniquely identifies the contents of this
-    QIcon object. This means that multiple QIcon objects can have
-    the same serial number as long as they refer to the same contents.
+/*! \obsolete
+
+    Returns a number that identifies the contents of this
+    QIcon object. Distinct QIcon objects can have
+    the same serial number if they refer to the same contents
+    (but they don't have to). Also, the serial number of
+    a QIcon object may change during its lifetime.
+
+    Use cacheKey() instead.
 
     A null icon always has a serial number of 0.
 
+    Serial numbers are mostly useful in conjunction with caching.
+
     \sa QPixmap::serialNumber()
 */
 
@@ -483,7 +626,28 @@ int QIcon::serialNumber() const
     return d ? d->serialNum : 0;
 }
 
-/*!  Returns a pixmap with the requested \a size, \a mode, and \a
+/*!
+    Returns a number that identifies the contents of this QIcon
+    object. Distinct QIcon objects can have the same key if
+    they refer to the same contents.
+    \since 4.3
+
+    The cacheKey() will change when the icon is altered via
+    addPixmap() or addFile().
+
+    Cache keys are mostly useful in conjunction with caching.
+
+    \sa QPixmap::cacheKey()
+*/
+qint64 QIcon::cacheKey() const
+{
+    if (!d)
+        return 0;
+    return (((qint64) d->serialNum) << 32) | ((qint64) (d->detach_no));
+}
+
+/*!
+  Returns a pixmap with the requested \a size, \a mode, and \a
   state, generating one if necessary. The pixmap might be smaller than
   requested, but never larger.
 
@@ -501,7 +665,8 @@ QPixmap QIcon::pixmap(const QSize &size, Mode mode, State state) const
 
     \overload
 
-    Returns a pixmap of size QSize(\a w, \a h).
+    Returns a pixmap of size QSize(\a w, \a h). The pixmap might be smaller than
+    requested, but never larger.
 */
 
 /*!
@@ -509,7 +674,8 @@ QPixmap QIcon::pixmap(const QSize &size, Mode mode, State state) const
 
     \overload
 
-    Returns a pixmap of size QSize(\a extent, \a extent).
+    Returns a pixmap of size QSize(\a extent, \a extent). The pixmap might be smaller
+    than requested, but never larger.
 */
 
 /*!  Returns the actual size of the icon for the requested \a size, \a
@@ -569,6 +735,27 @@ bool QIcon::isDetached() const
     return !d || d->ref == 1;
 }
 
+/*! \internal
+ */
+void QIcon::detach()
+{
+    if (d) {
+        if (d->ref != 1) {
+            QIconPrivate *x = new QIconPrivate;
+            if (d->engine_version > 1) {
+                QIconEngineV2 *engine = static_cast<QIconEngineV2 *>(d->engine);
+                x->engine = engine->clone();
+            } else {
+                x->engine = d->engine;
+            }
+            x->engine_version = d->engine_version;
+            if (!d->ref.deref())
+                delete d;
+            d = x;
+        }
+        ++d->detach_no;
+    }
+}
 
 /*!
     Adds \a pixmap to the icon, as a specialization for \a mode and
@@ -586,12 +773,14 @@ void QIcon::addPixmap(const QPixmap &pixmap, Mode mode, State state)
     if (!d) {
         d = new QIconPrivate;
         d->engine = new QPixmapIconEngine;
+    } else {
+        detach();
     }
     d->engine->addPixmap(pixmap, mode, state);
 }
 
 
-/*!  Adds a pixmap from the file with the given \a fileName to the
+/*!  Adds an image from the file with the given \a fileName to the
      icon, as a specialization for \a size, \a mode and \a state. The
      file will be loaded on demand. Note: custom icon engines are free
      to ignore additionally added pixmaps.
@@ -617,12 +806,150 @@ void QIcon::addFile(const QString &fileName, const QSize &size, Mode mode, State
     if (fileName.isEmpty())
         return;
     if (!d) {
-        d = new QIconPrivate;
-        d->engine = new QPixmapIconEngine;
+#if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
+        QFileInfo info(fileName);
+        QString suffix = info.suffix();
+        if (!suffix.isEmpty()) {
+            // first try version 2 engines..
+            if (QIconEngineFactoryInterfaceV2 *factory = qobject_cast<QIconEngineFactoryInterfaceV2*>(loaderV2()->instance(suffix))) {
+                if (QIconEngine *engine = factory->create(fileName)) {
+                    d = new QIconPrivate;
+                    d->engine = engine;
+                }
+            }
+            // ..then fall back and try to load version 1 engines
+            if (!d) {
+                if (QIconEngineFactoryInterface *factory = qobject_cast<QIconEngineFactoryInterface*>(loader()->instance(suffix))) {
+                    if (QIconEngine *engine = factory->create(fileName)) {
+                        d = new QIconPrivate;
+                        d->engine = engine;
+                        d->engine_version = 1;
+                    }
+                }
+            }
+        }
+#endif
+        // ...then fall back to the default engine
+        if (!d) {
+            d = new QIconPrivate;
+            d->engine = new QPixmapIconEngine;
+        }
+    } else {
+        detach();
     }
     d->engine->addFile(fileName, size, mode, state);
 }
 
+/*****************************************************************************
+  QIcon stream functions
+ *****************************************************************************/
+#if !defined(QT_NO_DATASTREAM)
+/*!
+    \fn QDataStream &operator<<(QDataStream &stream, const QIcon &icon)
+    \relates QIcon
+    \since 4.2
+
+    Writes the given \a icon to the the given \a stream as a PNG
+    image. If the icon contains more than one image, all images will
+    be written to the stream. Note that writing the stream to a file
+    will not produce a valid image file.
+*/
+
+QDataStream &operator<<(QDataStream &s, const QIcon &icon)
+{
+    if (s.version() >= QDataStream::Qt_4_3) {
+        if (icon.isNull()) {
+            s << QString();
+        } else {
+            if (icon.d->engine_version > 1) {
+                QIconEngineV2 *engine = static_cast<QIconEngineV2 *>(icon.d->engine);
+                s << engine->key();
+                engine->write(s);
+            } else {
+                // not really supported
+                qWarning("QIcon: Cannot stream QIconEngine. Use QIconEngineV2 instead.");
+            }
+        }
+    } else if (s.version() == QDataStream::Qt_4_2) {
+        if (icon.isNull()) {
+            s << 0;
+        } else {
+            QPixmapIconEngine *engine = static_cast<QPixmapIconEngine *>(icon.d->engine);
+            int num_entries = engine->pixmaps.size();
+            s << num_entries;
+            for (int i=0; i < num_entries; ++i) {
+                s << engine->pixmaps.at(i).pixmap;
+                s << engine->pixmaps.at(i).fileName;
+                s << engine->pixmaps.at(i).size;
+                s << (uint) engine->pixmaps.at(i).mode;
+                s << (uint) engine->pixmaps.at(i).state;
+            }
+        }
+    } else {
+        s << QPixmap(icon.pixmap(22,22));
+    }
+    return s;
+}
+
+/*!
+    \fn QDataStream &operator>>(QDataStream &stream, QIcon &icon)
+    \relates QIcon
+    \since 4.2
+
+    Reads an image, or a set of images, from the given \a stream into
+    the given \a icon.
+*/
+
+QDataStream &operator>>(QDataStream &s, QIcon &icon)
+{
+    if (s.version() >= QDataStream::Qt_4_3) {
+        icon = QIcon();
+        QString key;
+        s >> key;
+        if (key == QLatin1String("QPixmapIconEngine")) {
+            icon.d = new QIconPrivate;
+            QIconEngineV2 *engine = new QPixmapIconEngine;
+            icon.d->engine = engine;
+            engine->read(s);
+#if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
+        } else if (QIconEngineFactoryInterfaceV2 *factory = qobject_cast<QIconEngineFactoryInterfaceV2*>(loaderV2()->instance(key))) {
+            if (QIconEngineV2 *engine= factory->create()) {
+                icon.d = new QIconPrivate;
+                icon.d->engine = engine;
+                engine->read(s);
+            }
+#endif
+        }
+    } else if (s.version() == QDataStream::Qt_4_2) {
+        icon = QIcon();
+        int num_entries;
+        QPixmap pm;
+        QString fileName;
+        QSize sz;
+        uint mode;
+        uint state;
+
+        s >> num_entries;
+        for (int i=0; i < num_entries; ++i) {
+            s >> pm;
+            s >> fileName;
+            s >> sz;
+            s >> mode;
+            s >> state;
+            if (pm.isNull())
+                icon.addFile(fileName, sz, QIcon::Mode(mode), QIcon::State(state));
+            else
+                icon.addPixmap(pm, QIcon::Mode(mode), QIcon::State(state));
+        }
+    } else {
+        QPixmap pm;
+        s >> pm;
+        icon.addPixmap(pm);
+    }
+    return s;
+}
+
+#endif //QT_NO_DATASTREAM
 
 
 #ifdef QT3_SUPPORT
@@ -630,7 +957,8 @@ void QIcon::addFile(const QString &fileName, const QSize &size, Mode mode, State
 static int widths[2] = { 22, 32 };
 static int heights[2] = { 22, 32 };
 
-static QSize pixmapSize(QIcon::Size which) {
+static QSize pixmapSizeHelper(QIcon::Size which)
+{
     int i = 0;
     if (which == QIcon::Large)
         i = 1;
@@ -653,7 +981,7 @@ static QSize pixmapSize(QIcon::Size which) {
     \sa pixmapSize()
 */
 QPixmap QIcon::pixmap(Size size, Mode mode, State state) const
-{ return pixmap(::pixmapSize(size), mode, state); }
+{ return pixmap(pixmapSizeHelper(size), mode, state); }
 
 /*!
     Use pixmap(QSize(...), mode, \a state), where the first argument
@@ -664,13 +992,13 @@ QPixmap QIcon::pixmap(Size size, Mode mode, State state) const
     \sa pixmapSize()
 */
 QPixmap QIcon::pixmap(Size size, bool enabled, State state) const
-{ return pixmap(::pixmapSize(size), enabled ? Normal : Disabled, state); }
+{ return pixmap(pixmapSizeHelper(size), enabled ? Normal : Disabled, state); }
 
 /*!
     Use one of the other pixmap() overloads.
 */
 QPixmap QIcon::pixmap() const
-{ return pixmap(::pixmapSize(Small), Normal, Off); }
+{ return pixmap(pixmapSizeHelper(Small), Normal, Off); }
 
 /*!
     The pixmap() function now takes a QSize instead of a QIcon::Size,
@@ -692,7 +1020,7 @@ void QIcon::setPixmapSize(Size which, const QSize &size)
 */
 QSize QIcon::pixmapSize(Size which)
 {
-    return ::pixmapSize(which);
+    return pixmapSizeHelper(which);
 }
 
 /*!
@@ -716,3 +1044,15 @@ QSize QIcon::pixmapSize(Size which)
 */
 
 #endif // QT3_SUPPORT
+
+/*!
+    \fn DataPtr &QIcon::data_ptr()
+    \internal
+*/
+
+/*!
+    \typedef QIcon::DataPtr
+    \internal
+*/
+
+QT_END_NAMESPACE
