@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,7 @@
     \class QImageIOHandler
     \brief The QImageIOHandler class defines the common image I/O
     interface for all image formats in Qt.
+    \reentrant
 
     Qt uses QImageIOHandler for reading and writing images through
     QImageReader and QImageWriter. You can also derive from this class
@@ -160,6 +161,7 @@
     \class QImageIOPlugin
     \brief The QImageIOPlugin class defines an interface for writing
     an image format plugin.
+    \reentrant
 
     \ingroup plugins
 
@@ -206,6 +208,7 @@
     \class QImageIOHandlerFactoryInterface
     \brief The QImageIOHandlerFactoryInterface class provides the factory
     interface for QImageIOPlugin.
+    \reentrant
 
     \internal
 
@@ -218,6 +221,8 @@
 #include <qimage.h>
 #include <qvariant.h>
 
+QT_BEGIN_NAMESPACE
+
 class QIODevice;
 
 class QImageIOHandlerPrivate
@@ -557,3 +562,5 @@ QImageIOPlugin::~QImageIOPlugin()
 
     \sa keys()
 */
+
+QT_END_NAMESPACE
