@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,13 +59,7 @@
     To display the movie in your application, you can pass your QMovie object
     to QLabel::setMovie(). Example:
 
-    \code
-        QLabel label;
-        QMovie *movie = new QMovie("animations/fire.gif");
-
-        label.setMovie(movie);
-        movie->start();
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.image.qmovie.cpp 0
 
     Whenever a new frame is available in the movie, QMovie will emit
     updated(). If the size of the frame changes, resized() is emitted. You can
@@ -77,7 +91,7 @@
 
     Call supportedFormats() for a list of formats that QMovie supports.
 
-    \sa QLabel, QImageReader
+    \sa QLabel, QImageReader, {Movie Example}
 */
 
 /*! \enum QMovie::MovieState
@@ -175,6 +189,8 @@
 
 #define QMOVIE_INVALID_DELAY -1
 
+QT_BEGIN_NAMESPACE
+
 class QFrameInfo
 {
 public:
@@ -913,10 +929,7 @@ void QMovie::setPaused(bool paused)
     The default speed is 100%.
     Example:
 
-    \code
-        QMovie movie("racecar.gif");
-        movie.setSpeed(200); // 2x speed
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.image.qmovie.cpp 1
 */
 void QMovie::setSpeed(int percentSpeed)
 {
@@ -1034,9 +1047,9 @@ QList<QByteArray> QMovie::supportedFormats()
     \sa QMovie::CacheMode
 */
 
-QMovie::CacheMode QMovie::cacheMode()
+QMovie::CacheMode QMovie::cacheMode() const
 {
-    Q_D(QMovie);
+    Q_D(const QMovie);
     return d->cacheMode;
 }
 
@@ -1046,5 +1059,17 @@ void QMovie::setCacheMode(CacheMode cacheMode)
     d->cacheMode = cacheMode;
 }
 
+/*!
+  \internal
+*/
+QMovie::CacheMode QMovie::cacheMode()
+{
+    Q_D(QMovie);
+    return d->cacheMode;
+}
+
+QT_END_NAMESPACE
+
 #include "moc_qmovie.cpp"
+
 #endif // QT_NO_MOVIE
