@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -46,6 +51,7 @@
 #include "qbuffer.h"
 #include "qbytearray.h"
 #include "qdatastream.h"
+#include "qmath.h"
 #include "qpaintengine_pic_p.h"
 #include "qpicture.h"
 #include "qpolygon.h"
@@ -55,7 +61,8 @@
 //#define QT_PICTURE_DEBUG
 #include <qdebug.h>
 
-#include <private/qmath_p.h>
+
+QT_BEGIN_NAMESPACE
 
 class QPicturePaintEnginePrivate : public QPaintEnginePrivate
 {
@@ -160,7 +167,13 @@ void QPicturePaintEngine::updatePen(const QPen &pen)
 #endif
     int pos;
     SERIALIZE_CMD(QPicturePrivate::PdcSetPen);
-    d->s << pen;
+    if (d->pic_d->in_memory_only) {
+        int index = d->pic_d->pen_list.size();
+        d->pic_d->pen_list.append(pen);
+        d->s << index;
+    } else {
+        d->s << pen;
+    }
     writeCmdLength(pos, QRect(), false);
 }
 
@@ -208,7 +221,13 @@ void QPicturePaintEngine::updateBrush(const QBrush &brush)
 #endif
     int pos;
     SERIALIZE_CMD(QPicturePrivate::PdcSetBrush);
-    d->s << brush;
+    if (d->pic_d->in_memory_only) {
+        int index = d->pic_d->brush_list.size();
+        d->pic_d->brush_list.append(brush);
+        d->s << index;
+    } else {
+        d->s << brush;
+    }
     writeCmdLength(pos, QRect(), false);
 }
 
@@ -401,7 +420,7 @@ void QPicturePaintEngine::drawPixmap(const QRectF &r, const QPixmap &pm, const Q
     int pos;
     SERIALIZE_CMD(QPicturePrivate::PdcDrawPixmap);
 
-    if (d->pic_d->dont_stream_pixmaps) {
+    if (d->pic_d->in_memory_only) {
         int index = d->pic_d->pixmap_list.size();
         d->pic_d->pixmap_list.append(pm);
         d->s << r << index << sr;
@@ -419,7 +438,7 @@ void QPicturePaintEngine::drawTiledPixmap(const QRectF &r, const QPixmap &pixmap
 #endif
     int pos;
     SERIALIZE_CMD(QPicturePrivate::PdcDrawTiledPixmap);
-    if (d->pic_d->dont_stream_pixmaps) {
+    if (d->pic_d->in_memory_only) {
         int index = d->pic_d->pixmap_list.size();
         d->pic_d->pixmap_list.append(pixmap);
         d->s << r << index << s;
@@ -485,4 +504,6 @@ void QPicturePaintEngine::updateState(const QPaintEngineState &state)
     if (flags & DirtyOpacity) updateOpacity(state.opacity());
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_PICTURE
