@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +46,14 @@
 #include "qobject.h"
 #include "qdebug.h"
 
+#ifdef Q_OS_WIN
+#include "qpaintengine.h"
+#include <private/qimage_p.h>
+#include <private/qpixmap_raster_p.h>
+#endif
+
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QPixmapCache
 
@@ -62,15 +90,37 @@
     \sa QCache, QPixmap
 */
 
-static int cache_limit = 1024;                        // 1024 KB cache limit
+#if defined(Q_WS_QWS) || defined(Q_OS_WINCE)
+static int cache_limit = 2048; // 2048 KB cache limit for embedded
+#else
+static int cache_limit = 10240; // 10 MB cache limit for desktop
+#endif
 
-class QPMCache : public QObject, public QCache<int, QPixmap>
+// XXX: hw: is this a general concept we need to abstract?
+class QDetachedPixmap : public QPixmap
+{
+public:
+    QDetachedPixmap(const QPixmap &pix) : QPixmap(pix)
+    {
+#ifdef Q_OS_WIN
+        QRasterPixmapData *d = static_cast<QRasterPixmapData*>(data);
+        if (!d->image.isNull() && d->image.d->paintEngine
+            && !d->image.d->paintEngine->isActive())
+        {
+            delete d->image.d->paintEngine;
+            d->image.d->paintEngine = 0;
+        }
+#endif
+    }
+};
+
+class QPMCache : public QObject, public QCache<qint64, QDetachedPixmap>
 {
     Q_OBJECT
 public:
     QPMCache()
         : QObject(0),
-          QCache<int, QPixmap>(cache_limit * 1024),
+          QCache<qint64, QDetachedPixmap>(cache_limit * 1024),
           id(0), ps(0), t(false) { }
     ~QPMCache() { }
 
@@ -81,12 +131,14 @@ public:
     QPixmap *object(const QString &key) const;
 
 private:
-    QHash<QString, int> serialNumbers;
+    QHash<QString, qint64> cacheKeys;
     int id;
     int ps;
     bool t;
 };
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "qpixmapcache.moc"
+QT_END_INCLUDE_NAMESPACE
 
 /*
   This is supposed to cut the cache size down by about 80-90% in a
@@ -107,16 +159,15 @@ void QPMCache::timerEvent(QTimerEvent *)
     setMaxCost(mc);
     ps = totalCost();
 
-    QHash<QString,int>::iterator it = serialNumbers.begin();
-    while (it != serialNumbers.end()) {
+    QHash<QString, qint64>::iterator it = cacheKeys.begin();
+    while (it != cacheKeys.end()) {
         if (!contains(it.value())) {
-            it = serialNumbers.erase(it);
-        }
-        else
+            it = cacheKeys.erase(it);
+        } else {
             ++it;
+        }
     }
 
-
     if (!size()) {
         killTimer(id);
         id = 0;
@@ -129,20 +180,20 @@ void QPMCache::timerEvent(QTimerEvent *)
 
 QPixmap *QPMCache::object(const QString &key) const
 {
-    return QCache<int,QPixmap>::object(serialNumbers.value(key, -1));
+    return QCache<qint64, QDetachedPixmap>::object(cacheKeys.value(key, -1));
 }
 
 
 bool QPMCache::insert(const QString& key, const QPixmap &pixmap, int cost)
 {
-    int serialNumber = pixmap.serialNumber();
-    if (contains(serialNumber)) {
-        serialNumbers.insert(key, serialNumber);
+    qint64 cacheKey = pixmap.cacheKey();
+    if (QCache<qint64, QDetachedPixmap>::object(cacheKey)) {
+        cacheKeys.insert(key, cacheKey);
         return true;
     }
-    bool success = QCache<int, QPixmap>::insert(serialNumber, new QPixmap(pixmap), cost);
+    bool success = QCache<qint64, QDetachedPixmap>::insert(cacheKey, new QDetachedPixmap(pixmap), cost);
     if (success) {
-        serialNumbers.insert(key, serialNumber);
+        cacheKeys.insert(key, cacheKey);
         if (!id) {
             id = startTimer(30000);
             t = false;
@@ -153,9 +204,9 @@ bool QPMCache::insert(const QString& key, const QPixmap &pixmap, int cost)
 
 bool QPMCache::remove(const QString &key)
 {
-    int serialNumber = serialNumbers.value(key, -1);
-    serialNumbers.remove(key);
-    return QCache<int, QPixmap>::remove(serialNumber);
+    qint64 cacheKey = cacheKeys.value(key, -1);
+    cacheKeys.remove(key);
+    return QCache<qint64, QDetachedPixmap>::remove(cacheKey);
 }
 
 Q_GLOBAL_STATIC(QPMCache, pm_cache)
@@ -173,17 +224,7 @@ Q_GLOBAL_STATIC(QPMCache, pm_cache)
     find(const QString&, QPixmap&) instead.
 
     Example:
-    \code
-        QPixmap* pp;
-        QPixmap p;
-        if ((pp=QPixmapCache::find("my_big_image", pm))) {
-            p = *pp;
-        } else {
-            p.load("bigimage.png");
-            QPixmapCache::insert("my_big_image", new QPixmap(p));
-        }
-        painter->drawPixmap(0, 0, p);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.image.qpixmapcache.cpp 0
 */
 
 QPixmap *QPixmapCache::find(const QString &key)
@@ -198,14 +239,7 @@ QPixmap *QPixmapCache::find(const QString &key)
     returns true; otherwise it leaves \a pm alone and returns false.
 
     Example:
-    \code
-        QPixmap pm;
-        if (!QPixmapCache::find("my_big_image", pm)) {
-            pm.load("bigimage.png");
-            QPixmapCache::insert("my_big_image", pm);
-        }
-        painter->drawPixmap(0, 0, pm);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.image.qpixmapcache.cpp 1
 */
 
 bool QPixmapCache::find(const QString &key, QPixmap& pm)
@@ -287,3 +321,4 @@ void QPixmapCache::clear()
     pm_cache()->clear();
 }
 
+QT_END_NAMESPACE
