@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,6 +51,10 @@
 #include <qtextstream.h>
 #include <qvariant.h>
 
+#if defined(Q_OS_WINCE)
+#include "qguifunctions_wince.h"
+#endif
+
 #if defined(Q_CC_BOR)
 // needed for qsort() because of a std namespace problem on Borland
 #include "qplatformdefs.h"
@@ -38,6 +62,8 @@
 
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 static quint64 xpmHash(const QString &str)
 {
     unsigned int hashValue = 0;
@@ -763,9 +789,9 @@ static QString fbname(const QString &fileName) // get file basename (sort of)
     QString s = fileName;
     if (!s.isEmpty()) {
         int i;
-        if ((i = s.lastIndexOf('/')) >= 0)
+        if ((i = s.lastIndexOf(QLatin1Char('/'))) >= 0)
             s = s.mid(i);
-        if ((i = s.lastIndexOf('\\')) >= 0)
+        if ((i = s.lastIndexOf(QLatin1Char('\\'))) >= 0)
             s = s.mid(i);
         QRegExp r(QLatin1String("[a-zA-Z][a-zA-Z0-9_]*"));
         int p = r.indexIn(s);
@@ -839,7 +865,7 @@ static bool read_xpm_header(
     if (!read_xpm_string(buf, device, source, index, state))
         return false;
 
-#if defined(_MSC_VER) && _MSC_VER >= 1400
+#if defined(_MSC_VER) && _MSC_VER >= 1400 && !defined(Q_OS_WINCE)
 	if (sscanf_s(buf, "%d %d %d %d", w, h, ncols, cpp) < 4)
 #else
     if (sscanf(buf, "%d %d %d %d", w, h, ncols, cpp) < 4)
@@ -866,10 +892,12 @@ static bool read_xpm_body(
     // create it in correct format (Format_RGB32 vs Format_ARGB32,
     // depending on absence or presence of "c none", respectively)
     if (ncols <= 256) {
-        image = QImage(w, h, QImage::Format_Indexed8);
+        if (image.size() != QSize(w, h) || image.format() != QImage::Format_Indexed8) {
+            image = QImage(w, h, QImage::Format_Indexed8);
+            if (image.isNull())
+                return false;
+        }
         image.setNumColors(ncols);
-        if (image.isNull())
-            return false;
     }
 
     QMap<quint64, int> colorMap;
@@ -910,9 +938,9 @@ static bool read_xpm_body(
             int transparentColor = currentColor;
             if (ncols <= 256) {
                 image.setColor(transparentColor, 0);
-                colorMap.insert(xpmHash((const char *)index.constData()), transparentColor);
+                colorMap.insert(xpmHash(QLatin1String(index.constData())), transparentColor);
             } else {
-                colorMap.insert(xpmHash((const char *)index.constData()), 0);
+                colorMap.insert(xpmHash(QLatin1String(index.constData())), 0);
             }
         } else {
             QRgb c_rgb;
@@ -926,9 +954,9 @@ static bool read_xpm_body(
             }
             if (ncols <= 256) {
                 image.setColor(currentColor, 0xff000000 | c_rgb);
-                colorMap.insert(xpmHash((const char *)index.constData()), currentColor);
+                colorMap.insert(xpmHash(QLatin1String(index.constData())), currentColor);
             } else {
-                colorMap.insert(xpmHash((const char *)index.constData()), 0xff000000 | c_rgb);
+                colorMap.insert(xpmHash(QLatin1String(index.constData())), 0xff000000 | c_rgb);
             }
         }
     }
@@ -937,9 +965,11 @@ static bool read_xpm_body(
         // Now we can create 32-bit image of appropriate format
         QImage::Format format = hasTransparency ?
                                 QImage::Format_ARGB32 : QImage::Format_RGB32;
-        image = QImage(w, h, format);
-        if (image.isNull())
-            return false;
+        if (image.size() != QSize(w, h) || image.format() != format) {
+            image = QImage(w, h, format);
+            if (image.isNull())
+                return false;
+        }
     }
 
     // Read pixels
@@ -964,7 +994,7 @@ static bool read_xpm_body(
                 char b[16];
                 b[cpp] = '\0';
                 for (x=0; x<w && d<end; x++) {
-                    memcpy(b, (char *)d, sizeof(char)*cpp);
+                    memcpy(b, (char *)d, cpp);
                     *p++ = (uchar)colorMap[xpmHash(b)];
                     d += cpp;
                 }
@@ -977,12 +1007,21 @@ static bool read_xpm_body(
             char b[16];
             b[cpp] = '\0';
             for (x=0; x<w && d<end; x++) {
-                memcpy(b, (char *)d, sizeof(char)*cpp);
+                memcpy(b, (char *)d, cpp);
                 *p++ = (QRgb)colorMap[xpmHash(b)];
                 d += cpp;
             }
         }
     }
+
+    if (device) {
+        // Rewind unused characters, and skip to the end of the XPM struct.
+        for (int i = state.size() - 1; i >= 0; --i)
+            device->ungetChar(state[i]);
+        char c;
+        while (device->getChar(&c) && c != ';');
+        while (device->getChar(&c) && c != '\n');
+    }
     return true;
 }
 
@@ -995,6 +1034,9 @@ static bool read_xpm_body(
 
 bool qt_read_xpm_image_or_array(QIODevice *device, const char * const * source, QImage &image)
 {
+    if (!source)
+        return true;
+    
     QByteArray buf(200, 0);
     QByteArray state;
 
@@ -1123,13 +1165,13 @@ static bool write_xpm_image(const QImage &sourceImage, QIODevice *device, const
         for(x=0; x<w; x++) {
             int color = (int)(*(yp + x));
             QByteArray chars(xpm_color_name(cpp, colorMap[color]));
-            line[cc++] = chars[0];
+            line[cc++] = QLatin1Char(chars[0]);
             if (cpp > 1) {
-                line[cc++] = chars[1];
+                line[cc++] = QLatin1Char(chars[1]);
                 if (cpp > 2) {
-                    line[cc++] = chars[2];
+                    line[cc++] = QLatin1Char(chars[2]);
                     if (cpp > 3) {
-                        line[cc++] = chars[3];
+                        line[cc++] = QLatin1Char(chars[3]);
                     }
                 }
             }
@@ -1239,4 +1281,6 @@ QByteArray QXpmHandler::name() const
     return "xpm";
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_IMAGEFORMAT_XPM
