@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,7 +48,7 @@
 ** Copyright (C) 2003-2004 immodule for Qt Project.  All rights reserved.
 **
 ** This file is written to contribute to Trolltech ASA under their own
-** licence. You may use this file under your Qt license. Following
+** license. You may use this file under your Qt license. Following
 ** description is copied from their original file headers. Contact
 ** immodule-qt@freedesktop.org if any conditions of this licensing are
 ** not clear to you.
@@ -40,8 +60,11 @@
 
 #if !defined(QT_NO_IM)
 
+QT_BEGIN_NAMESPACE
+
 #if !defined(QT_NO_XIM)
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "qplatformdefs.h"
 
 #include "qapplication.h"
@@ -56,6 +79,7 @@
 
 #include <stdlib.h>
 #include <limits.h>
+QT_END_INCLUDE_NAMESPACE
 
 // #define QT_XIM_DEBUG
 #ifdef QT_XIM_DEBUG
@@ -585,7 +609,7 @@ bool QXIMInputContext::x11FilterEvent(QWidget *keywidget, XEvent *event)
     int xkey_keycode = event->xkey.keycode;
     if (!keywidget->testAttribute(Qt::WA_WState_Created))
         return false;
-    if (XFilterEvent(event, keywidget->winId())) {
+    if (XFilterEvent(event, keywidget->effectiveWinId())) {
         qt_ximComposingKeycode = xkey_keycode; // ### not documented in xlib
 
         return true;
@@ -614,8 +638,15 @@ bool QXIMInputContext::x11FilterEvent(QWidget *keywidget, XEvent *event)
         count = XmbLookupString(data->ic, &event->xkey, string.data(), string.size(),
                                 &key, &status);
     }
-    if (count > 0)
+    if (count > 0) {
+        // XmbLookupString() gave us some text, convert it to unicode
         text = qt_input_mapper->toUnicode(string.constData() , count);
+        if (text.isEmpty()) {
+            // codec couldn't convert to unicode? this can happen when running in the
+            // C locale (or with no LANG set). try converting from latin-1
+            text = QString::fromLatin1(string.constData(), count);
+        }
+    }
 
 #if 0
     if (!(xim_style & XIMPreeditCallbacks) || !isComposing()) {
@@ -678,18 +709,17 @@ QXIMInputContext::ICData *QXIMInputContext::createICData(QWidget *w)
                                            (char *) 0);
     }
 
-    Q_ASSERT(w->testAttribute(Qt::WA_WState_Created));
     if (preedit_attr) {
         data->ic = XCreateIC(xim,
                              XNInputStyle, xim_style,
-                             XNClientWindow, w->winId(),
+                             XNClientWindow, w->effectiveWinId(),
                              XNPreeditAttributes, preedit_attr,
                              (char *) 0);
         XFree(preedit_attr);
     } else {
         data->ic = XCreateIC(xim,
                              XNInputStyle, xim_style,
-                             XNClientWindow, w->winId(),
+                             XNClientWindow, w->effectiveWinId(),
                              (char *) 0);
     }
 
@@ -771,6 +801,9 @@ QXIMInputContext::~QXIMInputContext() {}
 void QXIMInputContext::widgetDestroyed(QWidget *) {}
 QString QXIMInputContext::language() { return QString(); }
 bool QXIMInputContext::x11FilterEvent(QWidget *, XEvent *) { return true; }
+
 #endif //QT_NO_XIM
 
+QT_END_NAMESPACE
+
 #endif //QT_NO_IM
