@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -50,6 +70,8 @@
 
 #ifndef QT_NO_ITEMVIEWS
 
+QT_BEGIN_NAMESPACE
+
 typedef QList<QPair<QPersistentModelIndex, QPointer<QWidget> > > _q_abstractitemview_editor_container;
 typedef _q_abstractitemview_editor_container::const_iterator _q_abstractitemview_editor_const_iterator;
 typedef _q_abstractitemview_editor_container::iterator _q_abstractitemview_editor_iterator;
@@ -81,14 +103,16 @@ public:
     void _q_columnsAboutToBeRemoved(const QModelIndex &parent, int start, int end);
     void _q_columnsRemoved(const QModelIndex &parent, int start, int end);
     void _q_modelDestroyed();
+    void _q_layoutChanged();
+    void _q_fetchMore();
 
-    void fetchMore();
     bool shouldEdit(QAbstractItemView::EditTrigger trigger, const QModelIndex &index) const;
     bool shouldForwardEvent(QAbstractItemView::EditTrigger trigger, const QEvent *event) const;
     bool shouldAutoScroll(const QPoint &pos) const;
     void doDelayedItemsLayout();
 
     bool dropOn(QDropEvent *event, int *row, int *col, QModelIndex *index);
+    bool droppingOnItself(QDropEvent *event, const QModelIndex &index);
 
     QWidget *editor(const QModelIndex &index, const QStyleOptionViewItem &options);
     bool sendDelegateEvent(const QModelIndex &index, QEvent *event) const;
@@ -101,6 +125,7 @@ public:
                                                                  const QEvent *event) const;
     QItemSelectionModel::SelectionFlags contiguousSelectionCommand(const QModelIndex &index,
                                                                    const QEvent *event) const;
+    void selectAll(QItemSelectionModel::SelectionFlags command);
 
     inline QItemSelectionModel::SelectionFlags selectionBehaviorFlags() const
     {
@@ -112,7 +137,7 @@ public:
     }
 
 #ifndef QT_NO_DRAGANDDROP
-    QAbstractItemView::DropIndicatorPosition position(const QPoint &pos, const QRect &rect) const;
+    QAbstractItemView::DropIndicatorPosition position(const QPoint &pos, const QRect &rect, const QModelIndex &idx) const;
     inline bool canDecode(QDropEvent *e) const {
         QStringList modelTypes = model->mimeTypes();
         const QMimeData *mime = e->mimeData();
@@ -125,10 +150,16 @@ public:
 
     inline void paintDropIndicator(QPainter *painter)
     {
-        if (showDropIndicator && state == QAbstractItemView::DraggingState)
-            if (dropIndicatorRect.height() == 0) // FIXME: should be painted by style
-                painter->drawLine(dropIndicatorRect.topLeft(), dropIndicatorRect.topRight());
-            else painter->drawRect(dropIndicatorRect);
+        if (showDropIndicator && state == QAbstractItemView::DraggingState
+#ifndef QT_NO_CURSOR
+            && viewport->cursor().shape() != Qt::ForbiddenCursor
+#endif
+            ) {
+            QStyleOption opt;
+            opt.init(q_func());
+            opt.rect = dropIndicatorRect;
+            q_func()->style()->drawPrimitive(QStyle::PE_IndicatorItemViewItemDrop, &opt, painter, q_func());
+        }
     }
 #endif
 
@@ -137,6 +168,7 @@ public:
             QObject::disconnect(editor, SIGNAL(destroyed(QObject*)),
                                 q_func(), SLOT(editorDestroyed(QObject*)));
             editor->removeEventFilter(itemDelegate);
+            editor->hide();
             editor->deleteLater();
         }
     }
@@ -172,6 +204,8 @@ public:
     }
 
     void clearOrRemove();
+    void checkPersistentEditorFocus();
+
     QPixmap renderToPixmap(const QModelIndexList &indexes, QRect *r = 0) const;
 
     inline bool isIndexValid(const QModelIndex &index) const {
@@ -228,6 +262,34 @@ public:
         return QPoint(q->horizontalOffset(), q->verticalOffset());
     }
 
+    /**
+     * For now, assume that we have few editors, if we need a more efficient implementation 
+     * we should add a QMap<QAbstractItemDelegate*, int> member.
+     */
+    int delegateRefCount(const QAbstractItemDelegate *delegate) const
+    {
+        int ref = 0;
+        if (itemDelegate == delegate)
+            ++ref;
+
+        for (int maps = 0; maps < 2; ++maps) {
+            const QMap<int, QPointer<QAbstractItemDelegate> > *delegates = maps ? &columnDelegates : &rowDelegates;
+            for (QMap<int, QPointer<QAbstractItemDelegate> >::const_iterator it = delegates->begin(); 
+                it != delegates->end(); ++it) {
+                    if (it.value() == delegate) {
+                        ++ref;
+                        // optimization, we are only interested in the ref count values 0, 1 or >=2
+                        if (ref >= 2) {
+                            return ref;
+                        }
+                    }
+            }
+        }
+        return ref;
+    }
+
+    QStyleOptionViewItemV4 viewOptionsV4() const;
+
     QAbstractItemModel *model;
     QPointer<QAbstractItemDelegate> itemDelegate;
     QMap<int, QPointer<QAbstractItemDelegate> > rowDelegates;
@@ -238,7 +300,8 @@ public:
     QAbstractItemView::SelectionBehavior selectionBehavior;
 
     _q_abstractitemview_editor_container editors;
-    QList<QWidget*> persistent;
+    QSet<QWidget*> persistent;
+    QWidget *currentlyCommittingEditor;
 
     QPersistentModelIndex enteredIndex;
     QPersistentModelIndex pressedIndex;
@@ -248,6 +311,7 @@ public:
 
     QAbstractItemView::State state;
     QAbstractItemView::EditTriggers editTriggers;
+    QAbstractItemView::EditTrigger lastTrigger;
 
     QPersistentModelIndex root;
     QPersistentModelIndex hover;
@@ -269,7 +333,6 @@ public:
     bool autoScroll;
     QBasicTimer autoScrollTimer;
     int autoScrollMargin;
-    int autoScrollInterval;
     int autoScrollCount;
 
     bool alternatingColors;
@@ -283,13 +346,20 @@ public:
     QBasicTimer updateTimer;
     QBasicTimer delayedEditing;
     mutable QBasicTimer delayedLayout;
+    QBasicTimer delayedAutoScroll; //used when an item is clicked
     QTimeLine timeline;
 
     QAbstractItemView::ScrollMode verticalScrollMode;
     QAbstractItemView::ScrollMode horizontalScrollMode;
+
+    bool currentIndexSet;
+
+    bool wrapItemText;
 };
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include <qvector.h>
+QT_END_INCLUDE_NAMESPACE
 
 template <typename T>
 inline int qBinarySearch(const QVector<T> &vec, const T &item, int start, int end)
@@ -305,6 +375,8 @@ inline int qBinarySearch(const QVector<T> &vec, const T &item, int start, int en
     return i;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_ITEMVIEWS
 
 #endif // QABSTRACTITEMVIEW_P_H
