@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,26 +57,6 @@
 #include <qapplication.h>
 #include <private/qabstractitemmodel_p.h>
 #include <qdebug.h>
-#if defined(Q_WS_WIN)
-#  include "qt_windows.h"
-#endif
-
-/*!
-  \class QFileIconProvider
-
-  \brief The QFileIconProvider class provides file icon for the QDirModel class.
-*/
-
-/*!
-  \enum QFileIconProvider::IconType
-  \value Computer
-  \value Desktop
-  \value Trashcan
-  \value Network
-  \value Drive
-  \value Folder
-  \value File
-*/
 
 /*!
     \enum QDirModel::Roles
@@ -65,158 +65,7 @@
     \value FileNameRole
 */
 
-class QFileIconProviderPrivate
-{
-    Q_DECLARE_PUBLIC(QFileIconProvider)
-public:
-    QFileIconProviderPrivate();
-
-    QIcon file;
-    QIcon fileLink;
-    QIcon directory;
-    QIcon directoryLink;
-    QIcon harddisk;
-    QIcon floppy;
-    QIcon cdrom;
-    QIcon ram;
-    QIcon network;
-    QIcon computer;
-    QIcon desktop;
-    QIcon trashcan;
-    QIcon generic;
-
-    QFileIconProvider *q_ptr;
-};
-
-QFileIconProviderPrivate::QFileIconProviderPrivate()
-{
-    QStyle *style = QApplication::style();
-
-    file = QIcon(style->standardPixmap(QStyle::SP_FileIcon));
-    fileLink = QIcon(style->standardPixmap(QStyle::SP_FileLinkIcon));
-
-    directory = QIcon(style->standardPixmap(QStyle::SP_DirClosedIcon));
-    directory.addPixmap(style->standardPixmap(QStyle::SP_DirOpenIcon), QIcon::Normal, QIcon::On);
-    directoryLink = QIcon(style->standardPixmap(QStyle::SP_DirLinkIcon));
-
-    harddisk = QIcon(style->standardPixmap(QStyle::SP_DriveHDIcon));
-    floppy = QIcon(style->standardPixmap(QStyle::SP_DriveFDIcon));
-    cdrom = QIcon(style->standardPixmap(QStyle::SP_DriveCDIcon));
-    generic = ram = harddisk; // FIXME
-    network = QIcon(style->standardPixmap(QStyle::SP_DriveNetIcon));
-    computer = QIcon(style->standardPixmap(QStyle::SP_ComputerIcon));
-    desktop = QIcon(style->standardPixmap(QStyle::SP_DesktopIcon));
-    trashcan = QIcon(style->standardPixmap(QStyle::SP_TrashIcon));
-}
-
-/*!
-  Constructs a file icon provider.
-*/
-
-QFileIconProvider::QFileIconProvider()
-    : d_ptr(new QFileIconProviderPrivate)
-{
-}
-
-/*!
-  Destroys the file icon provider.
-
-*/
-
-QFileIconProvider::~QFileIconProvider()
-{
-    delete d_ptr;
-}
-
-/*!
-  Returns an icon set for the given \a type.
-*/
-
-QIcon QFileIconProvider::icon(IconType type) const
-{
-    switch (type) {
-    case Computer:
-        return d_ptr->computer;
-    case Desktop:
-        return d_ptr->desktop;
-    case Trashcan:
-        return d_ptr->trashcan;
-    case Network:
-        return d_ptr->network;
-    case Drive:
-        return d_ptr->generic;
-    case Folder:
-        return d_ptr->directory;
-    case File:
-        return d_ptr->file;
-    default:
-        break;
-    };
-    return QIcon();
-}
-
-/*!
-  Returns an icon for the file described by \a info.
-*/
-
-QIcon QFileIconProvider::icon(const QFileInfo &info) const
-{
-    if (info.isRoot())
-#ifdef Q_OS_WIN
-    {
-        uint type = DRIVE_UNKNOWN;
-	QT_WA({ type = GetDriveTypeW((wchar_t *)info.absoluteFilePath().utf16()); },
-        { type = GetDriveTypeA(info.absoluteFilePath().toLocal8Bit()); });
-	switch (type) {
-	case DRIVE_REMOVABLE:
-            return d_ptr->floppy;
-	case DRIVE_FIXED:
-            return d_ptr->harddisk;
-	case DRIVE_REMOTE:
-            return d_ptr->network;
-	case DRIVE_CDROM:
-            return d_ptr->cdrom;
-	case DRIVE_RAMDISK:
-            return d_ptr->ram;
-	case DRIVE_UNKNOWN:
-            return d_ptr->generic;
-	case DRIVE_NO_ROOT_DIR:
-        default:
-            return d_ptr->generic;
-	}
-    }
-#else
-    return d_ptr->generic;
-#endif
-  if (info.isFile())
-    if (info.isSymLink())
-      return d_ptr->fileLink;
-    else
-      return d_ptr->file;
-  if (info.isDir())
-    if (info.isSymLink())
-      return d_ptr->directoryLink;
-    else
-      return d_ptr->directory;
-  return QIcon();
-}
-
-/*!
-  Returns the type of the file described by \a info.
-*/
-
-QString QFileIconProvider::type(const QFileInfo &info) const
-{
-    if (info.isRoot())
-        return QApplication::translate("QFileDialog", "Drive");
-    if (info.isFile())
-        return info.suffix() + QLatin1String(" ") + QApplication::translate("QFileDialog", "File");
-    if (info.isDir())
-        return QApplication::translate("QFileDialog", "Directory");
-    if (info.isSymLink())
-        return QApplication::translate("QFileDialog", "Symbolic Link");
-    return QApplication::translate("QFileDialog", "Unknown");
-}
+QT_BEGIN_NAMESPACE
 
 class QDirModelPrivate : public QAbstractItemModelPrivate
 {
@@ -283,8 +132,13 @@ public:
     QFileIconProvider *iconProvider;
     QFileIconProvider defaultProvider;
 
-    QList< QPair<QString,int> > savedPaths;
-    QList< QPersistentModelIndex > savedPersistentIndexes;
+    struct SavedPersistent {
+        QString path;
+        int column;
+        QPersistentModelIndexData *data;
+        QPersistentModelIndex index;
+    };
+    QList<SavedPersistent> savedPersistent;
     QPersistentModelIndex toBeRefreshed;
 
     bool shouldStat; // use the "carefull not to stat directories" mode
@@ -342,24 +196,22 @@ void QDirModelPrivate::invalidate()
   directories. In the simplest case, it can be used with a suitable display
   widget as part of a browser or filer.
 
-  QDirModel does not store file information internally or cache file data.
+  QDirModel keeps a cache with file information. The cache needs to be
+  updated with refresh().
 
   A directory model that displays the contents of a default directory
   is usually constructed with a parent object:
 
-  \quotefromfile snippets/shareddirmodel/main.cpp
-  \skipto QDirModel *model
-  \printuntil QDirModel *model
+  \snippet doc/src/snippets/shareddirmodel/main.cpp 2
 
   A tree view can be used to display the contents of the model
 
-  \skipto QTreeView *tree
-  \printuntil tree->setModel(
+  \snippet doc/src/snippets/shareddirmodel/main.cpp 4
 
   and the contents of a particular directory can be displayed by
   setting the tree view's root index:
 
-  \printuntil tree->setRootIndex(
+  \snippet doc/src/snippets/shareddirmodel/main.cpp 7
 
   The view's root index can be used to control how much of a
   hierarchical model is displayed. QDirModel provides a convenience
@@ -367,15 +219,16 @@ void QDirModelPrivate::invalidate()
   directory within the model.
 
   QDirModel can be accessed using the standard interface provided by
-  QAbstractItemModel, but it also provides some convenience functions that are
-  specific to a directory model.
-  The fileInfo(), isDir(), name(), and path() functions provide information
-  about the underlying files and directories related to items in the model.
+  QAbstractItemModel, but it also provides some convenience functions
+  that are specific to a directory model. The fileInfo() and isDir()
+  functions provide information about the underlying files and directories
+  related to items in the model.
+  
   Directories can be created and removed using mkdir(), rmdir(), and the
   model will be automatically updated to take the changes into account.
 
-  \sa nameFilters(), setFilter(), filter(), {Model/View Programming}, QListView, QTreeView
-
+  \sa nameFilters(), setFilter(), filter(), QListView, QTreeView,
+      {Dir View Example}, {Model Classes}
 */
 
 /*!
@@ -393,7 +246,7 @@ QDirModel::QDirModel(const QStringList &nameFilters,
 {
     Q_D(QDirModel);
     // we always start with QDir::drives()
-    d->nameFilters = nameFilters.isEmpty() ? QStringList("*") : nameFilters;
+    d->nameFilters = nameFilters.isEmpty() ? QStringList(QLatin1String("*")) : nameFilters;
     d->filters = filters;
     d->sort = sort;
     d->root.parent = 0;
@@ -441,17 +294,17 @@ QModelIndex QDirModel::index(int row, int column, const QModelIndex &parent) con
 {
     Q_D(const QDirModel);
     // note that rowCount does lazy population
-    if (column < 0 || column >= 4 || row < 0 || parent.column() > 0)
+    if (column < 0 || column >= columnCount(parent) || row < 0 || parent.column() > 0)
         return QModelIndex();
     // make sure the list of children is up to date
-    QDirModelPrivate::QDirNode *p = (parent.isValid() ? d->node(parent) : &d->root);
+    QDirModelPrivate::QDirNode *p = (d->indexValid(parent) ? d->node(parent) : &d->root);
     Q_ASSERT(p);
     if (!p->populated)
         d->populate(p); // populate without stat'ing
     if (row >= p->children.count())
         return QModelIndex();
     // now get the internal pointer for the index
-    QDirModelPrivate::QDirNode *n = d->node(row, parent.isValid() ? p : 0);
+    QDirModelPrivate::QDirNode *n = d->node(row, d->indexValid(parent) ? p : 0);
     Q_ASSERT(n);
 
     return createIndex(row, column, n);
@@ -465,7 +318,7 @@ QModelIndex QDirModel::parent(const QModelIndex &child) const
 {
     Q_D(const QDirModel);
 
-    if (!child.isValid())
+    if (!d->indexValid(child))
 	return QModelIndex();
     QDirModelPrivate::QDirNode *node = d->node(child);
     QDirModelPrivate::QDirNode *par = (node ? node->parent : 0);
@@ -498,6 +351,8 @@ int QDirModel::rowCount(const QModelIndex &parent) const
             d->populate(&d->root);
         return d->root.children.count();
     }
+    if (parent.model() != this)
+        return 0;
     QDirModelPrivate::QDirNode *p = d->node(parent);
     if (p->info.isDir() && !p->populated) // lazy population
         d->populate(p);
@@ -522,7 +377,7 @@ int QDirModel::columnCount(const QModelIndex &parent) const
 QVariant QDirModel::data(const QModelIndex &index, int role) const
 {
     Q_D(const QDirModel);
-    if (!index.isValid())
+    if (!d->indexValid(index))
         return QVariant();
 
     if (role == Qt::DisplayRole || role == Qt::EditRole) {
@@ -545,6 +400,10 @@ QVariant QDirModel::data(const QModelIndex &index, int role) const
         if (role == FileNameRole)
             return fileName(index);
     }
+
+    if (index.column() == 1 && Qt::TextAlignmentRole == role) {
+        return Qt::AlignRight;
+    }
     return QVariant();
 }
 
@@ -559,7 +418,7 @@ QVariant QDirModel::data(const QModelIndex &index, int role) const
 bool QDirModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     Q_D(QDirModel);
-    if (!index.isValid() || index.column() != 0
+    if (!d->indexValid(index) || index.column() != 0
         || (flags(index) & Qt::ItemIsEditable) == 0 || role != Qt::EditRole)
         return false;
 
@@ -572,8 +431,7 @@ bool QDirModel::setData(const QModelIndex &index, const QVariant &value, int rol
         emit dataChanged(index, sibling);
 
         d->toBeRefreshed = index.parent();
-        int slot = metaObject()->indexOfSlot("_q_refresh()");
-        QApplication::postEvent(this, new QMetaCallEvent(slot));
+        QMetaObject::invokeMethod(this, "_q_refresh", Qt::QueuedConnection);
 
         return true;
     }
@@ -594,8 +452,17 @@ QVariant QDirModel::headerData(int section, Qt::Orientation orientation, int rol
 	switch (section) {
         case 0: return tr("Name");
         case 1: return tr("Size");
-        case 2: return tr("Type");
-        case 3: return tr("Modified");
+        case 2: return
+#ifdef Q_OS_MAC
+                       tr("Kind", "Match OS X Finder");
+#else
+                       tr("Type", "All other platforms");
+#endif
+        // Windows   - Type
+        // OS X      - Kind
+        // Konqueror - File Type
+        // Nautilus  - Type
+        case 3: return tr("Date Modified");
         default: return QVariant();
         }
     }
@@ -632,7 +499,7 @@ Qt::ItemFlags QDirModel::flags(const QModelIndex &index) const
 {
     Q_D(const QDirModel);
     Qt::ItemFlags flags = QAbstractItemModel::flags(index);
-    if (!index.isValid())
+    if (!d->indexValid(index))
         return flags;
     flags |= Qt::ItemIsDragEnabled;
     if (d->readOnly)
@@ -653,7 +520,7 @@ Qt::ItemFlags QDirModel::flags(const QModelIndex &index) const
 
 void QDirModel::sort(int column, Qt::SortOrder order)
 {
-    QDir::SortFlags sort = QDir::DirsFirst;
+    QDir::SortFlags sort = QDir::DirsFirst | QDir::IgnoreCase;
     if (order == Qt::DescendingOrder)
         sort |= QDir::Reversed;
 
@@ -717,49 +584,50 @@ QMimeData *QDirModel::mimeData(const QModelIndexList &indexes) const
 */
 
 bool QDirModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
-                             int row, int /* column */, const QModelIndex &parent)
+                             int /* row */, int /* column */, const QModelIndex &parent)
 {
     Q_D(QDirModel);
-    if (!parent.isValid() || isReadOnly())
+    if (!d->indexValid(parent) || isReadOnly())
         return false;
 
-    QDirModelPrivate::QDirNode *p = d->node(parent);
-
     bool success = true;
     QString to = filePath(parent) + QDir::separator();
 
     QList<QUrl> urls = data->urls();
-    QList<QUrl>::const_iterator it = urls.begin();
-
-    int last = row + urls.count() - 1;
-    beginInsertRows(parent, row, last);
+    QList<QUrl>::const_iterator it = urls.constBegin();
 
     switch (action) {
     case Qt::CopyAction:
-        for (; it != urls.end(); ++it) {
+        for (; it != urls.constEnd(); ++it) {
             QString path = (*it).toLocalFile();
             success = QFile::copy(path, to + QFileInfo(path).fileName()) && success;
         }
         break;
     case Qt::LinkAction:
-        for (; it != urls.end(); ++it) {
+        for (; it != urls.constEnd(); ++it) {
             QString path = (*it).toLocalFile();
             success = QFile::link(path, to + QFileInfo(path).fileName()) && success;
         }
         break;
     case Qt::MoveAction:
-        for (; it != urls.end(); ++it) {
+        for (; it != urls.constEnd(); ++it) {
             QString path = (*it).toLocalFile();
-            success = QFile::copy(path, to + QFileInfo(path).fileName())
-                      && QFile::remove(path) && success;
+            if(QFile::copy(path, to + QFileInfo(path).fileName())
+               && QFile::remove(path)) {
+                QModelIndex idx=index(QFileInfo(path).path());
+                if(idx.isValid())
+                    refresh(idx);
+            } else {
+                success = false;
+            }
         }
         break;
     default:
         return false;
     }
 
-    d->populate(p);
-    endInsertRows();
+    if (success)
+        refresh(parent);
 
     return success;
 }
@@ -804,6 +672,7 @@ void QDirModel::setNameFilters(const QStringList &filters)
 {
     Q_D(QDirModel);
     d->nameFilters = filters;
+    emit layoutAboutToBeChanged();
     if (d->shouldStat)
        refresh(QModelIndex());
     else
@@ -834,6 +703,7 @@ void QDirModel::setFilter(QDir::Filters filters)
 {
     Q_D(QDirModel);
     d->filters = filters;
+    emit layoutAboutToBeChanged();
     if (d->shouldStat)
         refresh(QModelIndex());
     else
@@ -863,6 +733,7 @@ void QDirModel::setSorting(QDir::SortFlags sort)
 {
     Q_D(QDirModel);
     d->sort = sort;
+    emit layoutAboutToBeChanged();
     if (d->shouldStat)
         refresh(QModelIndex());
     else
@@ -927,7 +798,7 @@ bool QDirModel::isReadOnly() const
   \brief Whether the directory model optimizes the hasChildren function
   to only check if the item is a directory.
 
-  If this property is set to true, the directory model will make sure that a directory
+  If this property is set to false, the directory model will make sure that a directory
   actually containes any files before reporting that it has children.
   Otherwise the directory model will report that an item has children if the item
   is a directory.
@@ -948,29 +819,35 @@ bool QDirModel::lazyChildCount() const
 }
 
 /*!
-  Refreshes (rereads) the children of \a parent.
+  QDirModel caches file information. This function updates the
+  cache. The \a parent parameter is the directory from which the
+  model is updated; the default value will update the model from
+  root directory of the file system (the entire model).
 */
 
 void QDirModel::refresh(const QModelIndex &parent)
 {
     Q_D(QDirModel);
 
-    QDirModelPrivate::QDirNode *n = parent.isValid() ? d->node(parent) : &(d->root);
+    QDirModelPrivate::QDirNode *n = d->indexValid(parent) ? d->node(parent) : &(d->root);
 
     int rows = n->children.count();
     if (rows == 0) {
+        emit layoutAboutToBeChanged();
         n->stat = true; // make sure that next time we read all the info
         n->populated = false;
         emit layoutChanged();
         return;
     }
 
+    emit layoutAboutToBeChanged();
     d->savePersistentIndexes();
-    beginRemoveRows(parent, 0, rows - 1);
+    d->rowsAboutToBeRemoved(parent, 0, rows - 1);
     n->stat = true; // make sure that next time we read all the info
     d->clear(n);
-    endRemoveRows();
+    d->rowsRemoved(parent, 0, rows - 1);
     d->restorePersistentIndexes();
+    emit layoutChanged();
 }
 
 /*!
@@ -987,7 +864,7 @@ QModelIndex QDirModel::index(const QString &path, int column) const
         return QModelIndex();
 
     QString absolutePath = QDir(path).absolutePath();
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
     absolutePath = absolutePath.toLower();
     // On Windows, "filename......." and "filename" are equivalent
     if (absolutePath.endsWith(QLatin1Char('.'))) {
@@ -1000,9 +877,9 @@ QModelIndex QDirModel::index(const QString &path, int column) const
     }
 #endif
 
-    QStringList pathElements = absolutePath.split(QChar('/'), QString::SkipEmptyParts);
+    QStringList pathElements = absolutePath.split(QLatin1Char('/'), QString::SkipEmptyParts);
     if ((pathElements.isEmpty() || !QFileInfo(path).exists())
-#ifndef Q_OS_WIN
+#if !defined(Q_OS_WIN) || defined(Q_OS_WINCE)
         && path != QLatin1String("/")
 #endif
         )
@@ -1012,17 +889,22 @@ QModelIndex QDirModel::index(const QString &path, int column) const
     if (!d->root.populated) // make sure the root is populated
         d->populate(&d->root);
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
     if (absolutePath.startsWith(QLatin1String("//"))) { // UNC path
         QString host = pathElements.first();
         int r = 0;
         for (; r < d->root.children.count(); ++r)
             if (d->root.children.at(r).info.fileName() == host)
                 break;
-        if (r >= d->root.children.count() && d->allowAppendChild)
+        bool childAppended = false;
+        if (r >= d->root.children.count() && d->allowAppendChild) {
             d->appendChild(&d->root, QLatin1String("//") + host);
+            childAppended = true;
+        }
         idx = index(r, 0, QModelIndex());
         pathElements.pop_front();
+        if (childAppended)
+            emit const_cast<QDirModel*>(this)->layoutChanged();
     } else if (pathElements.at(0).endsWith(QLatin1Char(':'))) {
         pathElements[0] += QLatin1Char('/');
     }
@@ -1045,7 +927,7 @@ QModelIndex QDirModel::index(const QString &path, int column) const
         for (int j = parent->children.count() - 1; j >= 0; --j) {
             const QFileInfo& fi = parent->children.at(j).info;
             QString childFileName;
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
             childFileName = idx.isValid() ? fi.fileName() : fi.absoluteFilePath();
             childFileName = childFileName.toLower();
 #else
@@ -1061,7 +943,15 @@ QModelIndex QDirModel::index(const QString &path, int column) const
 
         // we couldn't find the path element, we create a new node since we _know_ that the path is valid
         if (row == -1) {
-            QString newPath = parent->info.absoluteFilePath() + "/" + element;
+#if defined(Q_OS_WINCE)
+            QString newPath;
+            if (parent->info.isRoot())
+                newPath = parent->info.absoluteFilePath() + element;
+            else
+                newPath= parent->info.absoluteFilePath() + QLatin1Char('/') + element;
+#else
+            QString newPath = parent->info.absoluteFilePath() + QLatin1Char('/') + element;
+#endif
             if (!d->allowAppendChild || !QFileInfo(newPath).isDir())
                 return QModelIndex();
             d->appendChild(parent, newPath);
@@ -1089,7 +979,7 @@ QModelIndex QDirModel::index(const QString &path, int column) const
 bool QDirModel::isDir(const QModelIndex &index) const
 {
     Q_D(const QDirModel);
-    Q_ASSERT(index.isValid());
+    Q_ASSERT(d->indexValid(index));
     QDirModelPrivate::QDirNode *node = d->node(index);
     return node->info.isDir();
 }
@@ -1101,7 +991,7 @@ bool QDirModel::isDir(const QModelIndex &index) const
 QModelIndex QDirModel::mkdir(const QModelIndex &parent, const QString &name)
 {
     Q_D(QDirModel);
-    if (!parent.isValid() || isReadOnly())
+    if (!d->indexValid(parent) || isReadOnly())
         return QModelIndex();
 
     QDirModelPrivate::QDirNode *p = d->node(parent);
@@ -1130,13 +1020,20 @@ QModelIndex QDirModel::mkdir(const QModelIndex &parent, const QString &name)
 
 /*!
   Removes the directory corresponding to the model item \a index in the
-  directory model, returning true if successful. If the directory
-  cannot be removed, false is returned.
+  directory model and \bold{deletes the corresponding directory from the
+  file system}, returning true if successful. If the directory cannot be
+  removed, false is returned.
+
+  \warning This function deletes directories from the file system; it does
+  \bold{not} move them to a location where they can be recovered.
+
+  \sa remove()
 */
 
 bool QDirModel::rmdir(const QModelIndex &index)
 {
-    if (!index.isValid() || isReadOnly())
+    Q_D(QDirModel);
+    if (!d->indexValid(index) || isReadOnly())
         return false;
 
     QDirModelPrivate::QDirNode *n = d_func()->node(index);
@@ -1158,13 +1055,20 @@ bool QDirModel::rmdir(const QModelIndex &index)
 }
 
 /*!
-  Removes the model item \a index from the directory model, returning
-  true if successful. If the item cannot be removed, false is returned.
+  Removes the model item \a index from the directory model and \bold{deletes the
+  corresponding file from the file system}, returning true if successful. If the
+  item cannot be removed, false is returned.
+
+  \warning This function deletes files from the file system; it does \bold{not}
+  move them to a location where they can be recovered.
+
+  \sa rmdir()
 */
 
 bool QDirModel::remove(const QModelIndex &index)
 {
-    if (!index.isValid() || isReadOnly())
+    Q_D(QDirModel);
+    if (!d->indexValid(index) || isReadOnly())
         return false;
 
     QDirModelPrivate::QDirNode *n = d_func()->node(index);
@@ -1192,7 +1096,7 @@ bool QDirModel::remove(const QModelIndex &index)
 QString QDirModel::filePath(const QModelIndex &index) const
 {
     Q_D(const QDirModel);
-    if (index.isValid()) {
+    if (d->indexValid(index)) {
         QFileInfo fi = fileInfo(index);
         if (d->resolveSymlinks && fi.isSymLink())
             fi = d->resolvedInfo(fi);
@@ -1210,7 +1114,7 @@ QString QDirModel::filePath(const QModelIndex &index) const
 QString QDirModel::fileName(const QModelIndex &index) const
 {
     Q_D(const QDirModel);
-    if (!index.isValid())
+    if (!d->indexValid(index))
         return QString();
     QFileInfo info = fileInfo(index);
     if (info.isRoot())
@@ -1228,7 +1132,7 @@ QString QDirModel::fileName(const QModelIndex &index) const
 QIcon QDirModel::fileIcon(const QModelIndex &index) const
 {
     Q_D(const QDirModel);
-    if (!index.isValid())
+    if (!d->indexValid(index))
         return d->iconProvider->icon(QFileIconProvider::Computer);
     QDirModelPrivate::QDirNode *node = d->node(index);
     if (node->icon.isNull())
@@ -1237,25 +1141,37 @@ QIcon QDirModel::fileIcon(const QModelIndex &index) const
 }
 
 /*!
-  Returns the file information for the model item \a index.
+  Returns the file information for the specified model \a index.
 
+  \bold{Note:} If the model index represents a symbolic link in the
+  underlying filing system, the file information returned will contain
+  information about the symbolic link itself, regardless of whether
+  resolveSymlinks is enabled or not.
+
+  \sa QFileInfo::symLinkTarget()
 */
 
 QFileInfo QDirModel::fileInfo(const QModelIndex &index) const
 {
-    Q_ASSERT(index.isValid());
+    Q_D(const QDirModel);
+    Q_ASSERT(d->indexValid(index));
 
-    QDirModelPrivate::QDirNode *node = d_func()->node(index);
+    QDirModelPrivate::QDirNode *node = d->node(index);
     return node->info;
 }
 
+/*!
+  \fn QObject *QDirModel::parent() const
+  \internal
+*/
+
 /*
   The root node is never seen outside the model.
 */
 
 void QDirModelPrivate::init()
 {
-    filters = QDir::AllEntries;
+    filters = QDir::AllEntries | QDir::NoDotAndDotDot;
     sort = QDir::Name;
     nameFilters << QLatin1String("*");
     root.parent = 0;
@@ -1289,8 +1205,9 @@ QVector<QDirModelPrivate::QDirNode> QDirModelPrivate::children(QDirNode *parent,
         parent = 0;
         infoList = QDir::drives();
     } else if (parent->info.isDir()) {
-        if (parent->info.isSymLink()) {
-            QString link = parent->info.readLink();
+        //resolve directory links only if requested.
+        if (parent->info.isSymLink() && resolveSymlinks) {
+            QString link = parent->info.symLinkTarget();
             if (link.size() > 1 && link.at(link.size() - 1) == QDir::separator())
                 link.chop(1);
             if (stat)
@@ -1327,13 +1244,15 @@ void QDirModelPrivate::_q_refresh()
 void QDirModelPrivate::savePersistentIndexes()
 {
     Q_Q(QDirModel);
-    savedPaths.clear();
-    const QList<QPersistentModelIndexData*> indexes = persistent.indexes;
-    for (int i = 0; i < indexes.count(); ++i) {
-        QModelIndex idx = indexes.at(i)->index;
-        QString path = q->filePath(idx);
-        savedPaths.append(qMakePair(path, idx.column()));
-        savedPersistentIndexes.append(idx); // make sure the ref  >= 1
+    savedPersistent.clear();
+    for (int i = 0; i < persistent.indexes.count(); ++i) {
+        savedPersistent.append(SavedPersistent());
+        QPersistentModelIndexData *data = persistent.indexes.at(i);
+        QModelIndex index = data->index;
+        savedPersistent[i].path = q->filePath(index);
+        savedPersistent[i].column = index.column();
+        savedPersistent[i].data = data;
+        savedPersistent[i].index = index;
     }
 }
 
@@ -1342,27 +1261,26 @@ void QDirModelPrivate::restorePersistentIndexes()
     Q_Q(QDirModel);
     bool allow = allowAppendChild;
     allowAppendChild = false;
-    for (int i = 0; i < persistent.indexes.count(); ++i) {
-        QModelIndex index;
-        if (i < savedPaths.count())
-            index = q->index(savedPaths.at(i).first, savedPaths.at(i).second);
-        persistent.indexes.at(i)->index = index;
+    for (int i = 0; i < savedPersistent.count(); ++i) {
+        QPersistentModelIndexData *data = savedPersistent.at(i).data;
+        QString path = savedPersistent.at(i).path;
+        int column = savedPersistent.at(i).column;
+        data->index = q->index(path, column);
     }
-    savedPersistentIndexes.clear();
-    savedPaths.clear();
+    savedPersistent.clear();
     allowAppendChild = allow;
 }
 
 QFileInfoList QDirModelPrivate::entryInfoList(const QString &path) const
 {
     const QDir dir(path);
-    return dir.entryInfoList(nameFilters, filters | QDir::NoDotAndDotDot, sort);
+    return dir.entryInfoList(nameFilters, filters, sort);
 }
 
 QStringList QDirModelPrivate::entryList(const QString &path) const
 {
     const QDir dir(path);
-    return dir.entryList(nameFilters, filters | QDir::NoDotAndDotDot, sort);
+    return dir.entryList(nameFilters, filters, sort);
 }
 
 QString QDirModelPrivate::name(const QModelIndex &index) const
@@ -1371,7 +1289,7 @@ QString QDirModelPrivate::name(const QModelIndex &index) const
     const QFileInfo info = n->info;
     if (info.isRoot()) {
         QString name = info.absoluteFilePath();
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
         if (name.startsWith(QLatin1Char('/'))) // UNC host
             return info.fileName();
         if (name.endsWith(QLatin1Char('/')))
@@ -1384,14 +1302,35 @@ QString QDirModelPrivate::name(const QModelIndex &index) const
 
 QString QDirModelPrivate::size(const QModelIndex &index) const
 {
-    quint64 bytes = node(index)->info.size();
-    if (bytes >= 1000000000)
-        return QLocale().toString(bytes / 1000000000) + QString(" GB");
-    if (bytes >= 1000000)
-        return QLocale().toString(bytes / 1000000) + QString(" MB");
-    if (bytes >= 1000)
-        return QLocale().toString(bytes / 1000) + QString(" KB");
-    return QLocale().toString(bytes) + QString(" bytes");
+    const QDirNode *n = node(index);
+    if (n->info.isDir()) {
+#ifdef Q_OS_MAC
+        return QLatin1String("--");
+#else
+        return QLatin1String("");
+#endif
+    // Windows   - ""
+    // OS X      - "--"
+    // Konqueror - "4 KB"
+    // Nautilus  - "9 items" (the number of children)
+    }
+
+    // According to the Si standard KB is 1000 bytes, KiB is 1024
+    // but on windows sizes are calulated by dividing by 1024 so we do what they do.
+    const quint64 kb = 1024;
+    const quint64 mb = 1024 * kb;
+    const quint64 gb = 1024 * mb;
+    const quint64 tb = 1024 * gb;
+    quint64 bytes = n->info.size();
+    if (bytes >= tb)
+        return QLocale().toString(bytes / tb) + QString::fromLatin1(" TB");
+    if (bytes >= gb)
+        return QLocale().toString(bytes / gb) + QString::fromLatin1(" GB");
+    if (bytes >= mb)
+        return QLocale().toString(bytes / mb) + QString::fromLatin1(" MB");
+    if (bytes >= kb)
+        return QLocale().toString(bytes / kb) + QString::fromLatin1(" KB");
+    return QLocale().toString(bytes) + QString::fromLatin1(" bytes");
 }
 
 QString QDirModelPrivate::type(const QModelIndex &index) const
@@ -1402,7 +1341,7 @@ QString QDirModelPrivate::type(const QModelIndex &index) const
 QString QDirModelPrivate::time(const QModelIndex &index) const
 {
 #ifndef QT_NO_DATESTRING
-    return node(index)->info.lastModified().toString("yyyy-MM-dd hh:mm:ss");
+    return node(index)->info.lastModified().toString(Qt::LocalDate);
 #else
     Q_UNUSED(index);
     return QString();
@@ -1435,11 +1374,11 @@ QFileInfo QDirModelPrivate::resolvedInfo(QFileInfo info)
 {
 #ifdef Q_OS_WIN
     // On windows, we cannot create a shortcut to a shortcut.
-    return QFileInfo(info.readLink());
+    return QFileInfo(info.symLinkTarget());
 #else
     QStringList paths;
     do {
-        QFileInfo link(info.readLink());
+        QFileInfo link(info.symLinkTarget());
         if (link.isRelative())
             info.setFile(info.absolutePath(), link.filePath());
         else
@@ -1452,6 +1391,8 @@ QFileInfo QDirModelPrivate::resolvedInfo(QFileInfo info)
 #endif
 }
 
+QT_END_NAMESPACE
+
 #include "moc_qdirmodel.cpp"
 
 #endif // QT_NO_DIRMODEL
