@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -51,9 +51,12 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_ITEMVIEWS
+
 class QItemDelegatePrivate;
 class QItemEditorFactory;
 
@@ -133,6 +136,8 @@ private:
 
 #endif // QT_NO_ITEMVIEWS
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QITEMDELEGATE_H
