@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +46,9 @@
 #include <qdebug.h>
 
 #ifndef QT_NO_ITEMVIEWS
+
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QItemSelectionRange
 
@@ -291,19 +314,13 @@ QModelIndexList QItemSelectionRange::indexes() const
   a selection that contains a range of items from the given \c model,
   beginning at the \c topLeft, and ending at the \c bottomRight.
 
-  \code
-    QItemSelection *selection = new QItemSelection(topLeft, bottomRight);
-  \endcode
+  \snippet doc/src/snippets/code/src.gui.itemviews.qitemselectionmodel.cpp 0
 
   An empty item selection can be constructed, and later populated as
   required. So, if the model is going to be unavailable when we construct
   the item selection, we can rewrite the above code in the following way:
 
-  \code
-    QItemSelection *selection = new QItemSelection();
-    ...
-    selection->select(topLeft, bottomRight);
-  \endcode
+  \snippet doc/src/snippets/code/src.gui.itemviews.qitemselectionmodel.cpp 1
 
   QItemSelection saves memory, and avoids unnecessary work, by working with
   selection ranges rather than recording the model item index for each
@@ -401,10 +418,10 @@ QModelIndexList QItemSelection::indexes() const
 */
 void QItemSelection::merge(const QItemSelection &other, QItemSelectionModel::SelectionFlags command)
 {
-    if (!(command & QItemSelectionModel::Select ||
+    if (other.isEmpty() ||
+          !(command & QItemSelectionModel::Select ||
           command & QItemSelectionModel::Deselect ||
-          command & QItemSelectionModel::Toggle) ||
-        other.isEmpty())
+          command & QItemSelectionModel::Toggle))
         return;
 
     QItemSelection newSelection = other;
@@ -611,7 +628,7 @@ void QItemSelectionModelPrivate::_q_columnsAboutToBeInserted(const QModelIndex &
     QList<QItemSelectionRange>::iterator it = ranges.begin();
     for (; it != ranges.end(); ) {
         if ((*it).isValid() && (*it).parent() == parent
-            && (*it).left() <= start && (*it).right() >= start) {
+            && (*it).left() < start && (*it).right() >= start) {
             QModelIndex bottomMiddle = model->index((*it).bottom(), start - 1, (*it).parent());
             QItemSelectionRange left((*it).topLeft(), bottomMiddle);
             QModelIndex topMiddle = model->index((*it).top(), start, (*it).parent());
@@ -640,7 +657,7 @@ void QItemSelectionModelPrivate::_q_rowsAboutToBeInserted(const QModelIndex &par
     QList<QItemSelectionRange>::iterator it = ranges.begin();
     for (; it != ranges.end(); ) {
         if ((*it).isValid() && (*it).parent() == parent
-            && (*it).top() <= start && (*it).bottom() >= start) {
+            && (*it).top() < start && (*it).bottom() >= start) {
             QModelIndex middleRight = model->index(start - 1, (*it).right(), (*it).parent());
             QItemSelectionRange top((*it).topLeft(), middleRight);
             QModelIndex middleLeft = model->index(start, (*it).left(), (*it).parent());
@@ -779,7 +796,7 @@ void QItemSelectionModelPrivate::_q_layoutChanged()
   To update the currently selected items, use the bitwise OR of
   QItemSelectionModel::Current and any of the other SelectionFlags.
   If you omit the QItemSelectionModel::Current command, a new current
-  selection will be created, and the previous one added to the committed
+  selection will be created, and the previous one added to the whole
   selection. All functions operate on both layers; for example,
   selectedItems() will return items from both layers.
 
@@ -860,6 +877,21 @@ QItemSelectionModel::QItemSelectionModel(QItemSelectionModelPrivate &dd, QAbstra
 */
 QItemSelectionModel::~QItemSelectionModel()
 {
+    Q_D(QItemSelectionModel);
+    if (d->model) {
+        disconnect(d->model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
+                this, SLOT(_q_rowsAboutToBeRemoved(QModelIndex,int,int)));
+        disconnect(d->model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
+                this, SLOT(_q_columnsAboutToBeRemoved(QModelIndex,int,int)));
+        disconnect(d->model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
+                this, SLOT(_q_rowsAboutToBeInserted(QModelIndex,int,int)));
+        disconnect(d->model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
+                this, SLOT(_q_columnsAboutToBeInserted(QModelIndex,int,int)));
+        disconnect(d->model, SIGNAL(layoutAboutToBeChanged()),
+                this, SLOT(_q_layoutAboutToBeChanged()));
+        disconnect(d->model, SIGNAL(layoutChanged()),
+                this, SLOT(_q_layoutChanged()));
+    }
 }
 
 /*!
@@ -870,10 +902,8 @@ QItemSelectionModel::~QItemSelectionModel()
 */
 void QItemSelectionModel::select(const QModelIndex &index, QItemSelectionModel::SelectionFlags command)
 {
-    if (index.isValid()) {
-        QItemSelection selection(index, index);
-        select(selection, command);
-    }
+    QItemSelection selection(index, index);
+    select(selection, command);
 }
 
 /*!
@@ -1055,9 +1085,11 @@ void QItemSelectionModel::setCurrentIndex(const QModelIndex &index, QItemSelecti
     if (command != NoUpdate)
         select(d->currentIndex, command); // select item
     emit currentChanged(d->currentIndex, previous);
-    if (d->currentIndex.row() != previous.row())
+    if (d->currentIndex.row() != previous.row() ||
+            d->currentIndex.parent() != previous.parent())
         emit currentRowChanged(d->currentIndex, previous);
-    if (d->currentIndex.column() != previous.column())
+    if (d->currentIndex.column() != previous.column() ||
+            d->currentIndex.parent() != previous.parent())
         emit currentColumnChanged(d->currentIndex, previous);
 }
 
@@ -1083,7 +1115,7 @@ bool QItemSelectionModel::isSelected(const QModelIndex &index) const
     //  search model ranges
     QList<QItemSelectionRange>::const_iterator it = d->ranges.begin();
     for (; it != d->ranges.end(); ++it) {
-        if ((*it).contains(index)) {
+        if ((*it).isValid() && (*it).contains(index)) {
             selected = true;
             break;
         }
@@ -1223,16 +1255,13 @@ bool QItemSelectionModel::rowIntersectsSelection(int row, const QModelIndex &par
     Q_D(const QItemSelectionModel);
     if (parent.isValid() && d->model != parent.model())
          return false;
-    // check current selection
-    for (int i = 0; i < d->currentSelection.count(); ++i)
-        if (d->currentSelection.at(i).top() <= row
-            && d->currentSelection.at(i).bottom() >= row)
-            return true;
-    // check the ranges
-    for (int i = 0; i < d->ranges.count(); ++i)
-        if (d->ranges.at(i).top() <= row
-            && d->ranges.at(i).bottom() >= row)
+
+    QItemSelection sel = d->ranges;
+    sel.merge(d->currentSelection, d->currentCommand);
+    for (int i = 0; i < sel.count(); ++i)
+        if (sel.at(i).top() <= row && sel.at(i).bottom() >= row)
             return true;
+
     return false;
 }
 
@@ -1245,16 +1274,13 @@ bool QItemSelectionModel::columnIntersectsSelection(int column, const QModelInde
     Q_D(const QItemSelectionModel);
     if (parent.isValid() && d->model != parent.model())
         return false;
-    // check current selection
-    for (int i = 0; i < d->currentSelection.count(); ++i)
-        if (d->currentSelection.at(i).left() <= column
-            && d->currentSelection.at(i).right() >= column)
-            return true;
-    // check the ranges
-    for (int i = 0; i < d->ranges.count(); ++i)
-        if (d->ranges.at(i).left() <= column
-            && d->ranges.at(i).right() >= column)
+
+    QItemSelection sel = d->ranges;
+    sel.merge(d->currentSelection, d->currentCommand);
+    for (int i = 0; i < sel.count(); ++i)
+        if (sel.at(i).left() <= column && sel.at(i).right() >= column)
             return true;
+
     return false;
 }
 
@@ -1267,7 +1293,14 @@ bool QItemSelectionModel::columnIntersectsSelection(int column, const QModelInde
 bool QItemSelectionModel::hasSelection() const
 {
     Q_D(const QItemSelectionModel);
-    return !(d->ranges.isEmpty() && d->currentSelection.isEmpty());
+    if (d->currentCommand == Toggle || d->currentCommand == Deselect) {
+        QItemSelection sel = d->ranges;
+        sel.merge(d->currentSelection, d->currentCommand);
+        return !sel.isEmpty();
+    }
+    else {
+        return !(d->ranges.isEmpty() && d->currentSelection.isEmpty());
+    }
 }
 
 /*!
@@ -1439,6 +1472,8 @@ QDebug operator<<(QDebug dbg, const QItemSelectionRange &range)
 }
 #endif
 
+QT_END_NAMESPACE
+
 #include "moc_qitemselectionmodel.cpp"
 
 #endif // QT_NO_ITEMVIEWS
