@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -61,9 +61,10 @@
 #include <private/qlistview_p.h>
 #include <private/qwidgetitemdata_p.h>
 
-
 #ifndef QT_NO_LISTWIDGET
 
+QT_BEGIN_NAMESPACE
+
 class QListModelLessThan
 {
 public:
@@ -144,7 +145,7 @@ class QListWidgetPrivate : public QListViewPrivate
     Q_DECLARE_PUBLIC(QListWidget)
 public:
     QListWidgetPrivate() : QListViewPrivate(), sortOrder(Qt::AscendingOrder), sortingEnabled(false) {}
-    inline QListModel *model() const { return ::qobject_cast<QListModel*>(q_func()->model()); }
+    inline QListModel *model() const { return qobject_cast<QListModel*>(q_func()->model()); }
     void setup();
     void _q_emitItemPressed(const QModelIndex &index);
     void _q_emitItemClicked(const QModelIndex &index);
@@ -165,8 +166,11 @@ public:
     QListWidgetItemPrivate(QListWidgetItem *item) : q(item), id(-1) {}
     QListWidgetItem *q;
     int id;
+    QVector<QWidgetItemData> values;
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_LISTWIDGET
 
 #endif // QLISTWIDGET_P_H
