@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 #include <qsize.h>
 #include <qstringlist.h>
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QProxyModel
     \obsolete
@@ -540,5 +542,8 @@ void QProxyModelPrivate::_q_sourceColumnsRemoved(const QModelIndex &, int, int)
     q->endRemoveColumns();
 }
 
+QT_END_NAMESPACE
+
 #include "moc_qproxymodel.cpp"
+
 #endif // QT_NO_PROXYMODEL
