@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +56,8 @@
 #include <private/qstandarditemmodel_p.h>
 #include <qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 class QStandardItemModelLessThan
 {
 public:
@@ -82,20 +104,10 @@ QStandardItemPrivate::~QStandardItemPrivate()
 /*!
   \internal
 */
-int QStandardItemPrivate::childIndex(int row, int column) const
+QPair<int, int> QStandardItemPrivate::position() const
 {
-    if ((row < 0) || (column < 0) || (row >= rowCount()) || (column >= columnCount()))
-        return -1;
-    return (row * columnCount()) + column;
-}
-
-/*!
-  \internal
-*/
-QPair<int, int> QStandardItemPrivate::itemPosition(const QStandardItem *item) const
-{
-    if (QStandardItem *par = item->d_func()->parent) {
-        int idx = par->d_func()->childIndex(item);
+    if (QStandardItem *par = parent) {
+        int idx = par->d_func()->childIndex(q_func());
         if (idx == -1)
             return QPair<int, int>(-1, -1);
         return QPair<int, int>(idx / par->columnCount(), idx % par->columnCount());
@@ -151,13 +163,12 @@ void QStandardItemPrivate::setChild(int row, int column, QStandardItem *item,
 void QStandardItemPrivate::changeFlags(bool enable, Qt::ItemFlags f)
 {
     Q_Q(QStandardItem);
-    Qt::ItemFlags oldFlags = flags;
+    Qt::ItemFlags flags = q->flags();
     if (enable)
         flags |= f;
     else
         flags &= ~f;
-    if ((flags != oldFlags) && model)
-        model->d_func()->itemChanged(q);
+    q->setFlags(flags);
 }
 
 /*!
@@ -175,12 +186,25 @@ void QStandardItemPrivate::childDeleted(QStandardItem *child)
 */
 void QStandardItemPrivate::setItemData(const QMap<int, QVariant> &roles)
 {
-    values.clear();
+    Q_Q(QStandardItem);
+
+    //let's build the vector of new values
+    QVector<QWidgetItemData> newValues;
     QMap<int, QVariant>::const_iterator it;
     for (it = roles.begin(); it != roles.end(); ++it) {
-        int role = it.key();
-        role = (role == Qt::EditRole) ? Qt::DisplayRole : role;
-        values.append(QWidgetItemData(role, it.value()));
+        QVariant value = it.value();
+        if (value.isValid()) {
+            int role = it.key();
+            role = (role == Qt::EditRole) ? Qt::DisplayRole : role;
+            QWidgetItemData wid(role,it.value());
+            newValues.append(wid);
+        }
+    }
+
+    if (values!=newValues) {
+        values=newValues;
+        if (model)
+            model->d_func()->itemChanged(q);
     }
 }
 
@@ -316,6 +340,37 @@ void QStandardItemModelPrivate::_q_emitItemChanged(const QModelIndex &topLeft,
 /*!
     \internal
 */
+bool QStandardItemPrivate::insertRows(int row, const QList<QStandardItem*> &items)
+{
+    Q_Q(QStandardItem);
+    if ((row < 0) || (row > rowCount()))
+        return false;
+    int count = items.count();
+    if (model)
+        model->d_func()->rowsAboutToBeInserted(q, row, row + count - 1);
+    if (rowCount() == 0) {
+        if (columnCount() == 0)
+            q->setColumnCount(1);
+        children.resize(columnCount() * count);
+        rows = count;
+    } else {
+        rows += count;
+        int index = childIndex(row, 0);
+        if (index != -1)
+            children.insert(index, columnCount() * count, 0);
+    }
+    for (int i = 0; i < items.count(); ++i) {
+        QStandardItem *item = items.at(i);
+        item->d_func()->model = model;
+        item->d_func()->parent = q;
+        int index = childIndex(i + row, 0);
+        children.replace(index, item);
+    }
+    if (model)
+        model->d_func()->rowsInserted(q, row, count);
+    return true;
+}
+
 bool QStandardItemPrivate::insertRows(int row, int count, const QList<QStandardItem*> &items)
 {
     Q_Q(QStandardItem);
@@ -689,7 +744,6 @@ QStandardItem &QStandardItem::operator=(const QStandardItem &other)
 {
     Q_D(QStandardItem);
     d->values = other.d_func()->values;
-    d->flags = other.d_func()->flags;
     return *this;
 }
 
@@ -719,23 +773,30 @@ QStandardItem *QStandardItem::parent() const
 /*!
     Sets the item's data for the given \a role to the specified \a value.
 
+    If you subclass QStandardItem and reimplement this function, your
+    reimplementation should call emitDataChanged() if you do not call
+    the base implementation of setData(). This will ensure that e.g.
+    views using the model are notified of the changes.
+    
+    \note The default implementation treats Qt::EditRole and Qt::DisplayRole
+    as referring to the same data.
+
     \sa Qt::ItemDataRole, data(), setFlags()
 */
 void QStandardItem::setData(const QVariant &value, int role)
 {
     Q_D(QStandardItem);
     role = (role == Qt::EditRole) ? Qt::DisplayRole : role;
-    QVariant oldValue = data(role);
-    if (value == oldValue)
-        return;
-
     QVector<QWidgetItemData>::iterator it;
     for (it = d->values.begin(); it != d->values.end(); ++it) {
         if ((*it).role == role) {
-            if (value.isValid())
+            if (value.isValid()) {
+                if ((*it).value == value)
+                    return;
                 (*it).value = value;
-            else
+            } else {
                 d->values.erase(it);
+            }
             if (d->model)
                 d->model->d_func()->itemChanged(this);
             return;
@@ -749,6 +810,9 @@ void QStandardItem::setData(const QVariant &value, int role)
 /*!
     Returns the item's data for the given \a role, or an invalid
     QVariant if there is no data for the role.
+    
+    \note The default implementation treats Qt::EditRole and Qt::DisplayRole
+    as referring to the same data.  
 */
 QVariant QStandardItem::data(int role) const
 {
@@ -762,6 +826,25 @@ QVariant QStandardItem::data(int role) const
     return QVariant();
 }
 
+/*!
+  \since 4.4
+
+  Causes the model associated with this item to emit a
+  \l{QAbstractItemModel::dataChanged()}{dataChanged}() signal for this
+  item.
+
+  You normally only need to call this function if you have subclassed
+  QStandardItem and reimplemented data() and/or setData().
+
+  \sa setData()
+*/
+void QStandardItem::emitDataChanged()
+{
+    Q_D(QStandardItem);
+    if (d->model)
+        d->model->d_func()->itemChanged(this);
+}
+
 /*!
   Sets the item flags for the item to \a flags.
 
@@ -772,12 +855,7 @@ QVariant QStandardItem::data(int role) const
 */
 void QStandardItem::setFlags(Qt::ItemFlags flags)
 {
-    Q_D(QStandardItem);
-    if (flags != d->flags) {
-        d->flags = flags;
-        if (d->model)
-            d->model->d_func()->itemChanged(this);
-    }
+    setData((int)flags, Qt::UserRole - 1);
 }
 
 /*!
@@ -792,8 +870,11 @@ void QStandardItem::setFlags(Qt::ItemFlags flags)
 */
 Qt::ItemFlags QStandardItem::flags() const
 {
-    Q_D(const QStandardItem);
-    return d->flags;
+    QVariant v = data(Qt::UserRole - 1);
+    if (!v.isValid())
+        return (Qt::ItemIsSelectable|Qt::ItemIsEnabled|Qt::ItemIsEditable
+                |Qt::ItemIsDragEnabled|Qt::ItemIsDropEnabled);
+    return ((Qt::ItemFlags)(v.toInt()));
 }
 
 /*!
@@ -1156,7 +1237,7 @@ void QStandardItem::setTristate(bool tristate)
 /*!
   \fn bool QStandardItem::isTristate() const
 
-  Returns whether the item is tristate; that is, if it's checkable with tree
+  Returns whether the item is tristate; that is, if it's checkable with three
   separate states.
 
   The default value is false.
@@ -1164,6 +1245,8 @@ void QStandardItem::setTristate(bool tristate)
   \sa setTristate(), isCheckable(), checkState()
 */
 
+#ifndef QT_NO_DRAGANDDROP
+
 /*!
   Sets whether the item is drag enabled. If \a dragEnabled is true, the item
   can be dragged by the user; otherwise, the user cannot drag the item.
@@ -1220,6 +1303,8 @@ void QStandardItem::setDropEnabled(bool dropEnabled)
   \sa setDropEnabled(), isDragEnabled(), flags()
 */
 
+#endif // QT_NO_DRAGANDDROP
+
 /*!
   Returns the row where the item is located in its parent's child table, or
   -1 if the item has no parent.
@@ -1229,7 +1314,7 @@ void QStandardItem::setDropEnabled(bool dropEnabled)
 int QStandardItem::row() const
 {
     Q_D(const QStandardItem);
-    QPair<int, int> pos = d->itemPosition(this);
+    QPair<int, int> pos = d->position();
     return pos.first;
 }
 
@@ -1242,7 +1327,7 @@ int QStandardItem::row() const
 int QStandardItem::column() const
 {
     Q_D(const QStandardItem);
-    QPair<int, int> pos = d->itemPosition(this);
+    QPair<int, int> pos = d->position();
     return pos.second;
 }
 
@@ -1350,6 +1435,19 @@ void QStandardItem::insertRow(int row, const QList<QStandardItem*> &items)
     d->insertRows(row, 1, items);
 }
 
+/*!
+    Inserts \a items at \a row. The column count wont be changed.
+
+    \sa insertRow(), insertColumn()
+*/
+void QStandardItem::insertRows(int row, const QList<QStandardItem*> &items)
+{
+    Q_D(QStandardItem);
+    if (row < 0)
+        return;
+    d->insertRows(row, items);
+}
+
 /*!
     Inserts a column at \a column containing \a items. If necessary,
     the row count is increased to the size of \a items.
@@ -1405,6 +1503,14 @@ void QStandardItem::insertColumns(int column, int count)
     \sa insertRow()
 */
 
+/*!
+    \fn void QStandardItem::appendRows(const QList<QStandardItem*> &items)
+
+    Appends rows containing \a items.  The column count will not change.
+
+    \sa insertRow()
+*/
+
 /*!
     \fn void QStandardItem::appendColumn(const QList<QStandardItem*> &items)
 
@@ -1416,12 +1522,22 @@ void QStandardItem::insertColumns(int column, int count)
 
 /*!
     \fn bool QStandardItemModel::insertRow(int row, const QModelIndex &parent)
-    \internal
+
+    Inserts a single row before the given \a row in the child items of the
+    \a parent specified. Returns true if the row is inserted; otherwise
+    returns false.
+
+    \sa insertRows(), insertColumn(), removeRow()
 */
 
 /*!
     \fn bool QStandardItemModel::insertColumn(int column, const QModelIndex &parent)
-    \internal
+
+    Inserts a single column before the given \a column in the child items of
+    the \a parent specified. Returns true if the column is inserted; otherwise
+    returns false.
+
+    \sa insertColumns(), insertRow(), removeColumn()
 */
 
 /*!
@@ -1688,8 +1804,11 @@ bool QStandardItem::operator<(const QStandardItem &other) const
 }
 
 /*!
-    Sorts the children of the item using the given \a order,
-    by the values in the given \a column.
+    Sorts the children of the item using the given \a order, by the values in
+    the given \a column.
+    
+    \note This function is recursive, therefore it sorts the children of the
+    item, its grandchildren, etc.
 
     \sa {operator<()}
 */
@@ -1746,7 +1865,7 @@ void QStandardItem::read(QDataStream &in)
     in >> d->values;
     qint32 flags;
     in >> flags;
-    d->flags = Qt::ItemFlags(flags);
+    setFlags((Qt::ItemFlags)flags);
 }
 
 /*!
@@ -1759,7 +1878,7 @@ void QStandardItem::write(QDataStream &out) const
 {
     Q_D(const QStandardItem);
     out << d->values;
-    out << qint32(d->flags);
+    out << flags();
 }
 
 /*!
@@ -1837,27 +1956,11 @@ QDataStream &operator<<(QDataStream &out, const QStandardItem &item)
 
     An example usage of QStandardItemModel to create a table:
 
-    \code
-            QStandardItemModel model(4, 4);
-            for (int row = 0; row < 4; ++row) {
-                for (int column = 0; column < 4; ++column) {
-                    QStandardItem *item = new QStandardItem(QString("row %0, column %1").arg(row).arg(column));
-                    model.setItem(row, column, item);
-                }
-            }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.itemviews.qstandarditemmodel.cpp 0
 
     An example usage of QStandardItemModel to create a tree:
 
-    \code
-            QStandardItemModel model;
-            QStandardItem *parentItem = model.invisibleRootItem();
-            for (int i = 0; i < 4; ++i) {
-                QStandardItem *item = new QStandardItem(QString("item %0").arg(i));
-                parentItem->appendRow(item);
-                parentItem = item;
-            }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.itemviews.qstandarditemmodel.cpp 1
 
     After setting the model on a view, you typically want to react to user
     actions, such as an item being clicked. Since a QAbstractItemView provides
@@ -1869,32 +1972,19 @@ QDataStream &operator<<(QDataStream &out, const QStandardItem &item)
     a QAbstractItemView signal, such as QAbstractItemView::clicked(). First
     you connect the view's signal to a slot in your class:
 
-    \code
-        QTreeView *treeView = new QTreeView(this);
-        treeView->setModel(myStandardItemModel);
-        connect(treeView, SIGNAL(clicked(QModelIndex)),
-                this, SLOT(clicked(QModelIndex)));
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.itemviews.qstandarditemmodel.cpp 2
 
     When you receive the signal, you call itemFromIndex() on the given model
     index to get a pointer to the item:
 
-    \code
-        void MyWidget::clicked(const QModelIndex &index)
-        {
-            QStandardItem *item = myStandardItemModel->itemFromIndex(index);
-            // Do stuff with the item ...
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.itemviews.qstandarditemmodel.cpp 3
 
     Conversely, you must obtain the QModelIndex of an item when you want to
     invoke a model/view function that takes an index as argument. You can
     obtain the index either by using the model's indexFromItem() function, or,
     equivalently, by calling QStandardItem::index():
 
-    \code
-        treeView->scrollTo(item->index());
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.itemviews.qstandarditemmodel.cpp 4
 
     You are, of course, not required to use the item-based approach; you could
     instead rely entirely on the QAbstractItemModel interface when working with
@@ -1932,9 +2022,11 @@ QStandardItemModel::QStandardItemModel(int rows, int columns, QObject *parent)
 {
     Q_D(QStandardItemModel);
     d->init();
+    d->root->insertColumns(0, columns);
+    d->columnHeaderItems.insert(0, columns, 0);
+    d->root->insertRows(0, rows);
+    d->rowHeaderItems.insert(0, rows, 0);
     d->root->d_func()->setModel(this);
-    d->root->setRowCount(rows);
-    d->root->setColumnCount(columns);
 }
 
 /*!
@@ -2023,8 +2115,10 @@ QStandardItem *QStandardItemModel::itemFromIndex(const QModelIndex &index) const
 */
 QModelIndex QStandardItemModel::indexFromItem(const QStandardItem *item) const
 {
-    if (item && item->d_func()->parent)
-        return createIndex(item->row(), item->column(), item->d_func()->parent);
+    if (item && item->d_func()->parent) {
+        QPair<int, int> pos = item->d_func()->position();
+        return createIndex(pos.first, pos.second, item->d_func()->parent);
+    }
     return QModelIndex();
 }
 
@@ -2102,6 +2196,9 @@ QStandardItem *QStandardItemModel::item(int row, int column) const
     through the QStandardItem API, making it possible to write functions that
     can treat top-level items and their children in a uniform way; for
     example, recursive functions involving a tree model.
+
+    \note Calling \l{QAbstractItemModel::index()}{index()} on the QStandardItem object
+    retrieved from this function is not valid.
 */
 QStandardItem *QStandardItemModel::invisibleRootItem() const
 {
@@ -2374,7 +2471,7 @@ void QStandardItemModel::insertRow(int row, const QList<QStandardItem*> &items)
 /*!
     \since 4.2
 
-    \fn QStandardItemModel::insertRow(int row, QStandardItem *item)
+    \fn void QStandardItemModel::insertRow(int row, QStandardItem *item)
     \overload
 
     Inserts a row at \a row containing \a item.
@@ -2753,6 +2850,13 @@ void QStandardItemModel::sort(int column, Qt::SortOrder order)
     d->root->sortChildren(column, order);
 }
 
+/*!
+  \fn QObject *QStandardItemModel::parent() const
+  \internal
+*/
+
+QT_END_NAMESPACE
+
 #include "moc_qstandarditemmodel.cpp"
 
 #endif // QT_NO_STANDARDITEMMODEL
