@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,6 +48,9 @@
 #include "qstringlistmodel.h"
 
 #ifndef QT_NO_STRINGLISTMODEL
+
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QStringListModel
     \brief The QStringListModel class provides a model that supplies strings to views.
@@ -75,9 +83,7 @@
 
     An example usage of QStringListModel:
 
-    \quotefromfile snippets/qstringlistmodel/main.cpp
-    \skipto QStringListModel
-    \printuntil model->setStringList(list)
+    \snippet doc/src/snippets/qstringlistmodel/main.cpp 0
 
     \sa QAbstractListModel, QAbstractItemModel, {Model Classes}
 */
@@ -189,7 +195,6 @@ bool QStringListModel::setData(const QModelIndex &index, const QVariant &value,
 
 bool QStringListModel::insertRows(int row, int count, const QModelIndex &parent)
 {
-    Q_UNUSED(parent);
     if (count < 1 || row < 0 || row > rowCount(parent))
         return false;
 
@@ -216,7 +221,6 @@ bool QStringListModel::insertRows(int row, int count, const QModelIndex &parent)
 
 bool QStringListModel::removeRows(int row, int count, const QModelIndex &parent)
 {
-    Q_UNUSED(parent);
     if (count <= 0 || row < 0 || (row + count) > rowCount(parent))
         return false;
 
@@ -230,16 +234,45 @@ bool QStringListModel::removeRows(int row, int count, const QModelIndex &parent)
     return true;
 }
 
+static bool ascendingLessThan(const QPair<QString, int> &s1, const QPair<QString, int> &s2)
+{
+    return s1.first < s2.first;
+}
+
+static bool decendingLessThan(const QPair<QString, int> &s1, const QPair<QString, int> &s2)
+{
+    return s1.first > s2.first;
+}
+
 /*!
   \reimp
 */
 void QStringListModel::sort(int, Qt::SortOrder order)
 {
     emit layoutAboutToBeChanged();
+
+    QList<QPair<QString, int> > list;
+    for (int i = 0; i < lst.count(); ++i)
+        list.append(QPair<QString, int>(lst.at(i), i));
+
     if (order == Qt::AscendingOrder)
-        qSort(lst.begin(), lst.end(), qLess<QString>());
+        qSort(list.begin(), list.end(), ascendingLessThan);
     else
-        qSort(lst.begin(), lst.end(), qGreater<QString>());
+        qSort(list.begin(), list.end(), decendingLessThan);
+
+    lst.clear();
+    QVector<int> forwarding(list.count());
+    for (int i = 0; i < list.count(); ++i) {
+        lst.append(list.at(i).first);
+        forwarding[list.at(i).second] = i;
+    }
+
+    QModelIndexList oldList = persistentIndexList();
+    QModelIndexList newList;
+    for (int i = 0; i < oldList.count(); ++i)
+        newList.append(index(forwarding.at(oldList.at(i).row()), 0));
+    changePersistentIndexList(oldList, newList);
+
     emit layoutChanged();
 }
 
@@ -263,4 +296,14 @@ void QStringListModel::setStringList(const QStringList &strings)
     reset();
 }
 
+/*!
+  \reimp
+*/
+Qt::DropActions QStringListModel::supportedDropActions() const
+{
+    return QAbstractItemModel::supportedDropActions() | Qt::MoveAction;
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_STRINGLISTMODEL
