@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -61,6 +61,8 @@
 
 #ifndef QT_NO_TREEWIDGET
 
+QT_BEGIN_NAMESPACE
+
 class QTreeWidgetItem;
 class QTreeWidgetItemIterator;
 class QTreeModelPrivate;
@@ -162,7 +164,9 @@ private:
     Q_DECLARE_PRIVATE(QTreeModel)
 };
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "private/qabstractitemmodel_p.h"
+QT_END_INCLUDE_NAMESPACE
 
 class QTreeModelPrivate : public QAbstractItemModelPrivate
 {
@@ -173,13 +177,18 @@ class QTreeWidgetItemPrivate
 {
 public:
     QTreeWidgetItemPrivate(QTreeWidgetItem *item)
-        : q(item), disabled(false), policy(QTreeWidgetItem::DontShowIndicatorWhenChildless) {}
+        : q(item), disabled(false), selected(false), rowGuess(-1), policy(QTreeWidgetItem::DontShowIndicatorWhenChildless) {}
     void propagateDisabled(QTreeWidgetItem *item);
     QTreeWidgetItem *q;
     QVariantList display;
     uint disabled : 1;
+    uint selected : 1;
+    int rowGuess;
     QTreeWidgetItem::ChildIndicatorPolicy policy;
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_TREEWIDGET
+
 #endif // QTREEWIDGET_P_H
