@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,10 +48,12 @@
 
 #ifndef QT_NO_TREEWIDGET
 
+QT_BEGIN_NAMESPACE
+
 /*!
   \class QTreeWidgetItemIterator
   \ingroup model-view
-  \brief The QTreeWidgetItemIterator class provides a way to iterate over the 
+  \brief The QTreeWidgetItemIterator class provides a way to iterate over the
   items in a QTreeWidget instance.
 
   The iterator will walk the items in a pre-order traversal order, thus visiting the
@@ -40,10 +62,7 @@
   For example, the following code examples each item in a tree, checking the
   text in the first column against a user-specified search string:
 
-  \quotefromfile snippets/qtreewidgetitemiterator-using/mainwindow.cpp
-  \skipto findItems(
-  \skipto QTreeWidgetItemIterator
-  \printuntil }
+  \snippet doc/src/snippets/qtreewidgetitemiterator-using/mainwindow.cpp 0
 
   It is also possible to filter out certain types of node by passing certain
   \l{IteratorFlag}{flags} to the constructor of QTreeWidgetItemIterator.
@@ -98,12 +117,12 @@ QTreeWidgetItemIterator::QTreeWidgetItemIterator(QTreeWidget *widget, IteratorFl
 
 QTreeWidgetItemIterator::QTreeWidgetItemIterator(QTreeWidgetItem *item, IteratorFlags flags)
     : d_ptr(new QTreeWidgetItemIteratorPrivate(
-                this, ::qobject_cast<QTreeModel*>(item->view->model()))),
+                this, qobject_cast<QTreeModel*>(item->view->model()))),
       current(item), flags(flags)
 {
     Q_D(QTreeWidgetItemIterator);
     Q_ASSERT(item);
-    QTreeModel *model = ::qobject_cast<QTreeModel*>(item->view->model());
+    QTreeModel *model = qobject_cast<QTreeModel*>(item->view->model());
     Q_ASSERT(model);
     model->iterators.append(this);
 
@@ -155,7 +174,7 @@ QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator=(const QTreeWidgetIte
 }
 
 /*!
-    The prefix ++ operator (++it) advances the iterator to the next matching item 
+    The prefix ++ operator (++it) advances the iterator to the next matching item
     and returns a reference to the resulting iterator.
     Sets the current pointer to 0 if the current item is the last matching item.
 */
@@ -170,7 +189,7 @@ QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator++()
 }
 
 /*!
-    The prefix -- operator (--it) advances the iterator to the previous matching item 
+    The prefix -- operator (--it) advances the iterator to the previous matching item
     and returns a reference to the resulting iterator.
     Sets the current pointer to 0 if the current item is the first matching item.
 */
@@ -218,9 +237,9 @@ bool QTreeWidgetItemIterator::matchesFlags(const QTreeWidgetItem *item) const
         if ((flags & NotEditable) && (itemFlags & Qt::ItemIsEditable))
             return false;
     }
-
-    {
-        // ### We only test the check state for column 0
+    
+    if (flags & (Checked|NotChecked)) {
+        // ### We only test the check state for column 0 
         Qt::CheckState check = item->checkState(0);
         // PartiallyChecked matches as Checked.
         if ((flags & Checked) && (check == Qt::Unchecked))
@@ -234,22 +253,15 @@ bool QTreeWidgetItemIterator::matchesFlags(const QTreeWidgetItem *item) const
     if ((flags & NoChildren) && item->childCount())
         return false;
 
-    {
-        QTreeWidget *widget = item->view;
-        Q_ASSERT(widget);
+    if ((flags & Hidden) && !item->isHidden())
+        return false;
+    if ((flags & NotHidden) && item->isHidden())
+        return false;
 
-        bool hidden = widget->isItemHidden(item);
-        if ((flags & Hidden) && !hidden)
-            return false;
-        if ((flags & NotHidden) && hidden)
-            return false;
-
-        bool selected = widget->isItemSelected(item);
-        if ((flags & Selected) && !selected)
-            return false;
-        if ((flags & Unselected) && selected)
-            return false;
-    }
+    if ((flags & Selected) && !item->isSelected())
+        return false;
+    if ((flags & Unselected) && item->isSelected())
+        return false;
 
     return true;
 }
@@ -327,7 +339,7 @@ void QTreeWidgetItemIteratorPrivate::ensureValidIterator(const QTreeWidgetItem *
 {
     Q_Q(QTreeWidgetItemIterator);
     Q_ASSERT(itemToBeRemoved);
-    
+
     if (!q->current) return;
     QTreeWidgetItem *nextItem = q->current;
 
@@ -360,10 +372,10 @@ void QTreeWidgetItemIteratorPrivate::ensureValidIterator(const QTreeWidgetItem *
     if (nextItem->parent() == itemToBeRemoved->parent()) {
         // They have the same parent, i.e. we have to adjust the m_currentIndex member of the iterator
         // if the deleted item is to the left of the nextItem.
-        
+
         QTreeWidgetItem *par = itemToBeRemoved->parent();   // We know they both have the same parent.
         QTreeWidget *tw = itemToBeRemoved->treeWidget();    // ..and widget
-        int indexOfItemToBeRemoved = par ? par->indexOfChild(const_cast<QTreeWidgetItem *>(itemToBeRemoved)) 
+        int indexOfItemToBeRemoved = par ? par->indexOfChild(const_cast<QTreeWidgetItem *>(itemToBeRemoved))
             : tw->indexOfTopLevelItem(const_cast<QTreeWidgetItem *>(itemToBeRemoved));
         int indexOfNextItem = par ? par->indexOfChild(nextItem) : tw->indexOfTopLevelItem(nextItem);
 
@@ -378,7 +390,7 @@ void QTreeWidgetItemIteratorPrivate::ensureValidIterator(const QTreeWidgetItem *
 /*!
   \fn const QTreeWidgetItemIterator QTreeWidgetItemIterator::operator++(int)
 
-  The postfix ++ operator (it++) advances the iterator to the next matching item 
+  The postfix ++ operator (it++) advances the iterator to the next matching item
   and returns an iterator to the previously current item.
 */
 
@@ -443,4 +455,7 @@ void QTreeWidgetItemIteratorPrivate::ensureValidIterator(const QTreeWidgetItem *
     \value NotEditable
     \value UserFlag
 */
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_TREEWIDGET
