@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -59,6 +59,8 @@
 #include "QtGui/qmenu.h"
 #include "private/qobject_p.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_ACTION
 
 #ifdef QT3_SUPPORT
@@ -101,6 +103,7 @@ public:
     uint separator : 1;
     uint fontSet : 1;
     QAction::MenuRole menuRole;
+    int iconVisibleInMenu : 3;  // Only has values -1, 0, and 1
     QList<QWidget *> widgets;
 #ifndef QT_NO_SHORTCUT
     void redoGrab(QShortcutMap &map);
@@ -119,4 +122,6 @@ public:
 
 #endif // QT_NO_ACTION
 
+QT_END_NAMESPACE
+
 #endif // QACTION_P_H
