@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -60,6 +80,8 @@
 #include "qvariant.h"
 #include "qdnd_p.h"
 
+QT_BEGIN_NAMESPACE
+
 /*****************************************************************************
   Internal QClipboard functions for X11.
  *****************************************************************************/
@@ -126,44 +148,69 @@ public:
     mutable QByteArray format_atoms;
 };
 
-
-
 class QClipboardData
 {
+private:
+    QMimeData *&mimeDataRef() const
+    {
+        if(mode == QClipboard::Selection)
+            return selectionData;
+        return clipboardData;
+    }
+
 public:
-    QClipboardData();
+    QClipboardData(QClipboard::Mode mode);
     ~QClipboardData();
 
     void setSource(QMimeData* s)
     {
-        delete src;
-        src = s;
+        if ((mode == QClipboard::Selection && selectionData == s)
+            || clipboardData == s) {
+            return;
+        }
+
+        if (selectionData != clipboardData) {
+            delete mimeDataRef();
+        }
+
+        mimeDataRef() = s;
     }
 
-    QMimeData *source() const { return src; }
+    QMimeData *source() const
+    {
+        return mimeDataRef();
+    }
 
-    void clear();
+    void clear()
+    {
+        timestamp = CurrentTime;
+        if (selectionData == clipboardData) {
+            mimeDataRef() = 0;
+        } else {
+            QMimeData *&src = mimeDataRef();
+            delete src;
+            src = 0;
+        }
+    }
 
-    QMimeData *src;
+    static QMimeData *selectionData;
+    static QMimeData *clipboardData;
     Time timestamp;
+    QClipboard::Mode mode;
 };
 
-QClipboardData::QClipboardData()
+QMimeData *QClipboardData::selectionData = 0;
+QMimeData *QClipboardData::clipboardData = 0;
+
+QClipboardData::QClipboardData(QClipboard::Mode clipboardMode)
 {
-    src = 0;
     timestamp = CurrentTime;
+    mode = clipboardMode;
 }
 
 QClipboardData::~QClipboardData()
 { clear(); }
 
-void QClipboardData::clear()
-{
-    delete src;
-    src = 0;
-    timestamp = CurrentTime;
-}
-
 
 static QClipboardData *internalCbData = 0;
 static QClipboardData *internalSelData = 0;
@@ -177,7 +224,7 @@ static void cleanupClipboardData()
 static QClipboardData *clipboardData()
 {
     if (internalCbData == 0) {
-        internalCbData = new QClipboardData;
+        internalCbData = new QClipboardData(QClipboard::Clipboard);
         qAddPostRoutine(cleanupClipboardData);
     }
     return internalCbData;
@@ -192,7 +239,7 @@ static void cleanupSelectionData()
 static QClipboardData *selectionData()
 {
     if (internalSelData == 0) {
-        internalSelData = new QClipboardData;
+        internalSelData = new QClipboardData(QClipboard::Selection);
         qAddPostRoutine(cleanupSelectionData);
     }
     return internalSelData;
@@ -356,14 +403,25 @@ static bool qt_x11_clipboard_event_filter(void *message, long *)
     return false;
 }
 
+static Bool checkForClipboardEvents(Display *, XEvent *e, XPointer)
+{
+    return ((e->type == SelectionRequest && (e->xselectionrequest.selection == XA_PRIMARY
+                                             || e->xselectionrequest.selection == ATOM(CLIPBOARD)))
+            || (e->type == SelectionClear && (e->xselectionclear.selection == XA_PRIMARY
+                                              || e->xselectionclear.selection == ATOM(CLIPBOARD))));
+}
+
 bool QX11Data::clipboardWaitForEvent(Window win, int type, XEvent *event, int timeout)
 {
     QTime started = QTime::currentTime();
     QTime now = started;
 
-    if (QAbstractEventDispatcher::instance()->inherits("QMotif")) {
-        if (waiting_for_data)
-            qFatal("QClipboard: internal error, qt_xclb_wait_for_event recursed");
+    if (QAbstractEventDispatcher::instance()->inherits("QtMotif")
+        || QApplication::clipboard()->property("useEventLoopWhenWaiting").toBool()) {
+        if (waiting_for_data) {
+            Q_ASSERT(!"QClipboard: internal error, qt_xclb_wait_for_event recursed");
+            return false;
+        }
         waiting_for_data = true;
 
 
@@ -410,6 +468,11 @@ bool QX11Data::clipboardWaitForEvent(Window win, int type, XEvent *event, int ti
             if (XCheckTypedWindowEvent(X11->display,win,type,event))
                 return true;
 
+            // process other clipboard events, since someone is probably requesting data from us
+            XEvent e;
+            if (XCheckIfEvent(X11->display, &e, checkForClipboardEvents, 0))
+                qApp->x11ProcessEvent(&e);
+
             now = QTime::currentTime();
             if ( started > now )                        // crossed midnight
                 started = now;
@@ -638,6 +701,14 @@ static Atom send_selection(QClipboardData *d, Atom target, Window window, Atom p
     Atom atomFormat = target;
     int dataFormat = 0;
     QByteArray data;
+
+    QByteArray fmt = X11->xdndAtomToString(target);
+    if (fmt.isEmpty() || !QInternalMimeData::hasFormatHelper(QString::fromAscii(fmt), d->source())) { // Not a MIME type we have
+        DEBUG("QClipboard: send_selection(): converting to type '%s' is not supported", fmt.data());
+        return XNone;
+    }
+    DEBUG("QClipboard: send_selection(): converting to type '%s'", fmt.data());
+
     if (X11->xdndMimeDataForAtom(target, d->source(), &data, &atomFormat, &dataFormat)) {
 
         VDEBUG("QClipboard: send_selection():\n"
@@ -756,7 +827,7 @@ bool QClipboard::event(QEvent *e)
             QClipboardData *d = selectionData();
 
             // ignore the event if it was generated before we gained selection ownership
-            if (d->timestamp != CurrentTime && xevent->xselectionclear.time < d->timestamp)
+            if (d->timestamp != CurrentTime && xevent->xselectionclear.time <= d->timestamp)
                 break;
 
             DEBUG("QClipboard: new selection owner 0x%lx at time %lx (ours %lx)",
@@ -775,7 +846,7 @@ bool QClipboard::event(QEvent *e)
             QClipboardData *d = clipboardData();
 
             // ignore the event if it was generated before we gained selection ownership
-            if (d->timestamp != CurrentTime && xevent->xselectionclear.time < d->timestamp)
+            if (d->timestamp != CurrentTime && xevent->xselectionclear.time <= d->timestamp)
                 break;
 
             DEBUG("QClipboard: new clipboard owner 0x%lx at time %lx (%lx)",
@@ -904,7 +975,7 @@ bool QClipboard::event(QEvent *e)
                     ;
                 } else if (target == xa_timestamp) {
                     if (d->timestamp != CurrentTime) {
-                        XChangeProperty(dpy, req->requestor, property, xa_timestamp, 32,
+                        XChangeProperty(dpy, req->requestor, property, XA_INTEGER, 32,
                                         PropModeReplace, (uchar *) &d->timestamp, 1);
                         ret = property;
                     } else {
@@ -1040,7 +1111,7 @@ bool QClipboardWatcher::hasFormat_sys(const QString &format) const
     return list.contains(format);
 }
 
-QVariant QClipboardWatcher::retrieveData_sys(const QString &fmt, QVariant::Type) const
+QVariant QClipboardWatcher::retrieveData_sys(const QString &fmt, QVariant::Type requestedType) const
 {
     if (fmt.isEmpty() || empty())
         return QByteArray();
@@ -1054,12 +1125,13 @@ QVariant QClipboardWatcher::retrieveData_sys(const QString &fmt, QVariant::Type)
     for (int i = 0; i < size; ++i)
         atoms.append(targets[i]);
 
-    Atom fmtatom = X11->xdndMimeAtomForFormat(fmt, atoms);
+    QByteArray encoding;
+    Atom fmtatom = X11->xdndMimeAtomForFormat(fmt, requestedType, atoms, &encoding);
 
     if (fmtatom == 0)
         return QVariant();
 
-    return X11->xdndMimeConvertToFormat(fmtatom, getDataInFormat(fmtatom), fmt);
+    return X11->xdndMimeConvertToFormat(fmtatom, getDataInFormat(fmtatom), fmt, requestedType, encoding);
 }
 
 QByteArray QClipboardWatcher::getDataInFormat(Atom fmtatom) const
@@ -1235,7 +1307,7 @@ bool qt_check_selection_sentinel()
           and have already emitted dataChanged() as a result of that)
         */
 
-        Window* owners;
+        unsigned char *retval;
         Atom actualType;
         int actualFormat;
         ulong nitems;
@@ -1245,7 +1317,8 @@ bool qt_check_selection_sentinel()
                                QApplication::desktop()->screen(0)->internalWinId(),
                                ATOM(_QT_SELECTION_SENTINEL), 0, 2, False, XA_WINDOW,
                                &actualType, &actualFormat, &nitems,
-                               &bytesLeft, (unsigned char**)&owners) == Success) {
+                               &bytesLeft, &retval) == Success) {
+            Window *owners = (Window *)retval;
             if (actualType == XA_WINDOW && actualFormat == 32 && nitems == 2) {
                 Window win = owner->internalWinId();
                 if (owners[0] == win || owners[1] == win)
@@ -1275,7 +1348,7 @@ bool qt_check_clipboard_sentinel()
 {
     bool doIt = true;
     if (owner) {
-        Window *owners;
+        unsigned char *retval;
         Atom actualType;
         int actualFormat;
         unsigned long nitems, bytesLeft;
@@ -1284,7 +1357,8 @@ bool qt_check_clipboard_sentinel()
                                QApplication::desktop()->screen(0)->internalWinId(),
                                ATOM(_QT_CLIPBOARD_SENTINEL), 0, 2, False, XA_WINDOW,
                                &actualType, &actualFormat, &nitems, &bytesLeft,
-                               (unsigned char **) &owners) == Success) {
+                               &retval) == Success) {
+            Window *owners = (Window *)retval;
             if (actualType == XA_WINDOW && actualFormat == 32 && nitems == 2) {
                 Window win = owner->internalWinId();
                 if (owners[0] == win || owners[1] == win)
@@ -1309,4 +1383,6 @@ bool qt_check_clipboard_sentinel()
     return doIt;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_CLIPBOARD
