@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -41,17 +41,21 @@
 **
 ****************************************************************************/
 
-#include "qvariant.h"
-#include "qdesktopwidget.h"
 #include "qapplication.h"
+#include "qdesktopwidget.h"
+#include "qlibrary.h"
 #include "qt_x11_p.h"
-#include "qx11info_x11.h"
+#include "qvariant.h"
 #include "qwidget_p.h"
+#include "qx11info_x11.h"
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
+
 // defined in qwidget_x11.cpp
 extern int qt_x11_create_desktop_on_screen;
 
+
 // function to update the workarea of the screen
 static bool qt_desktopwidget_workarea_dirty = true;
 void qt_desktopwidget_update_workarea()
@@ -114,7 +118,7 @@ QDesktopWidgetPrivate::~QDesktopWidgetPrivate()
             screens[i] = 0;
         }
 
-        delete [] screens;
+        free (screens);
     }
 
     if (rects)     delete [] rects;
@@ -124,30 +128,42 @@ QDesktopWidgetPrivate::~QDesktopWidgetPrivate()
 void QDesktopWidgetPrivate::init()
 {
     // get the screen count
+    int newScreenCount;
 #ifndef QT_NO_XINERAMA
+
     XineramaScreenInfo *xinerama_screeninfo = 0;
-    int unused;
-    use_xinerama = (XineramaQueryExtension(X11->display, &unused, &unused) && XineramaIsActive(X11->display));
+
+    if (X11->ptrXineramaQueryExtension
+        && X11->ptrXineramaIsActive
+        && X11->ptrXineramaQueryScreens) {
+        int unused;
+        use_xinerama = (X11->ptrXineramaQueryExtension(X11->display, &unused, &unused)
+                        && X11->ptrXineramaIsActive(X11->display));
+    }
 
     if (use_xinerama) {
         xinerama_screeninfo =
-            XineramaQueryScreens(X11->display, &screenCount);
+            X11->ptrXineramaQueryScreens(X11->display, &newScreenCount);
+    }
+
+    if (xinerama_screeninfo) {
         defaultScreen = 0;
-    } else
+     } else
 #endif // QT_NO_XINERAMA
     {
         defaultScreen = DefaultScreen(X11->display);
-        screenCount = ScreenCount(X11->display);
+        newScreenCount = ScreenCount(X11->display);
+        use_xinerama = false;
     }
 
     delete [] rects;
-    rects     = new QRect[screenCount];
+    rects     = new QRect[newScreenCount];
     delete [] workareas;
-    workareas = new QRect[screenCount];
+    workareas = new QRect[newScreenCount];
 
     // get the geometry of each screen
-    int i, x, y, w, h;
-    for (i = 0; i < screenCount; i++) {
+    int i, j, x, y, w, h;
+    for (i = 0, j = 0; i < newScreenCount; i++, j++) {
 
 #ifndef QT_NO_XINERAMA
         if (use_xinerama) {
@@ -164,11 +180,31 @@ void QDesktopWidgetPrivate::init()
                 h = HeightOfScreen(ScreenOfDisplay(X11->display, i));
             }
 
-        rects[i].setRect(x, y, w, h);
+        // merge a "cloned" screen with the previous, hiding all crtcs
+        // that are currently showing a sub-rect of the previous screen
+        rects[j].setRect(x, y, w, h);
+        if (j > 0 && rects[j-1].intersects(rects[j]) &&
+                (rects[j].width()*rects[j].height()) >
+                (rects[j-1].width()*rects[j-1].height())) {
+            rects[j-1] = rects[j];
+           j--;
+        }
         workareas[i] = QRect();
     }
 
+    if (screens) {
+        // leaks QWidget* pointers on purpose, can't delete them as pointer escapes
+        screens = (QWidget**) realloc(screens, j * sizeof(QWidget*));
+        if (j > screenCount)
+            memset(&screens[screenCount], 0, (j-screenCount) * sizeof(QWidget*));
+    }
+
+    screenCount = j;
+
 #ifndef QT_NO_XINERAMA
+    if (use_xinerama && screenCount == 1)
+        use_xinerama = false;
+
     if (xinerama_screeninfo)
         XFree(xinerama_screeninfo);
 #endif // QT_NO_XINERAMA
@@ -216,8 +252,7 @@ QWidget *QDesktopWidget::screen(int screen)
         screen = d->defaultScreen;
 
     if (! d->screens) {
-        d->screens = new QWidget*[d->screenCount];
-        memset(d->screens, 0, d->screenCount * sizeof(QWidget *));
+        d->screens = (QWidget**) calloc( d->screenCount, sizeof(QWidget*));
         d->screens[d->defaultScreen] = this;
     }
 
@@ -247,7 +282,8 @@ const QRect QDesktopWidget::availableGeometry(int screen) const
     if (d->workareas[screen].isValid())
         return d->workareas[screen];
 
-    if (! isVirtualDesktop() && X11->isSupportedByWM(ATOM(_NET_WORKAREA))) {
+    if ((d->screenCount == 1 || !isVirtualDesktop())
+        && X11->isSupportedByWM(ATOM(_NET_WORKAREA))) {
         Atom ret;
         int format, e;
         unsigned char *data = 0;
@@ -337,3 +373,5 @@ void QDesktopWidget::resizeEvent(QResizeEvent *event)
     qt_desktopwidget_workarea_dirty = true;
     QWidget::resizeEvent(event);
 }
+
+QT_END_NAMESPACE
