@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -63,6 +63,8 @@
 #include "qdebug.h"
 #include <ctype.h>
 
+QT_BEGIN_NAMESPACE
+
 // These pixmaps approximate the images in the Windows User Interface Guidelines.
 
 // XPM
@@ -327,7 +329,7 @@ QDragManager::QDragManager()
 
 #ifdef Q_WS_WIN
     n_cursor = 4;
-#else 
+#else
     n_cursor = 3;
 #endif
 
@@ -387,7 +389,7 @@ QPixmap QDragManager::dragCursor(Qt::DropAction action) const
     else if (action == Qt::IgnoreAction)
         return pm_cursor[3];
 #endif
-    return 0;
+    return QPixmap();
 }
 
 bool QDragManager::hasCustomDragCursors() const
@@ -688,3 +690,5 @@ QDropData::~QDropData()
 {
 }
 #endif // QT_NO_DRAGANDDROP
+
+QT_END_NAMESPACE
