@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -53,6 +73,8 @@
 
 #ifndef QT_NO_DRAGANDDROP
 
+QT_BEGIN_NAMESPACE
+
 class QEventLoop;
 
 class QDragPrivate : public QObjectPrivate
@@ -66,6 +88,7 @@ public:
     Qt::DropActions possible_actions;
     Qt::DropAction executed_action;
     QMap<Qt::DropAction, QPixmap> customCursors;
+    Qt::DropAction defaultDropAction;
 };
 
 class QInternalMimeData : public QMimeData
@@ -136,6 +159,7 @@ public:
     void updatePixmap();
     QWidget *source() const { return object ? object->d_func()->source : 0; }
     QDragPrivate *dragPrivate() const { return object ? object->d_func() : 0; }
+    static QDragPrivate *dragPrivate(QDrag *drag) { return drag ? drag->d_func() : 0; }
 
     static QDragManager *self();
     Qt::DropAction defaultAction(Qt::DropActions possibleActions,
@@ -169,6 +193,9 @@ public:
 private:
     QPixmap *pm_cursor;
     int n_cursor;
+#ifdef Q_WS_QWS
+    Qt::DropAction currentActionForOverrideCursor;
+#endif
 
     QWidget *currentDropTarget;
 
@@ -244,8 +271,41 @@ private:
     bool m_isNull;
 };
 
+class QOleDropTarget : public IDropTarget
+{
+public:
+    QOleDropTarget(QWidget* w);
+    virtual ~QOleDropTarget() {}
+
+    void releaseQt();
+
+    // IUnknown methods
+    STDMETHOD(QueryInterface)(REFIID riid, void FAR* FAR* ppvObj);
+    STDMETHOD_(ULONG, AddRef)(void);
+    STDMETHOD_(ULONG, Release)(void);
+
+    // IDropTarget methods
+    STDMETHOD(DragEnter)(LPDATAOBJECT pDataObj, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect);
+    STDMETHOD(DragOver)(DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect);
+    STDMETHOD(DragLeave)();
+    STDMETHOD(Drop)(LPDATAOBJECT pDataObj, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect);
+
+private:
+    ULONG m_refs;
+    QWidget* widget;
+    QPointer<QWidget> currentWidget;
+    QRect answerRect;
+    QPoint lastPoint;
+    DWORD choosenEffect;
+    DWORD lastKeyState;
+
+    void sendDragEnterEvent(QWidget *to, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect);
+};
+
 #endif
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_DRAGANDDROP
 
 #endif // QDND_P_H
