@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QGridLayoutPrivate;
@@ -41,6 +63,9 @@ class Q_GUI_EXPORT QGridLayout : public QLayout
 {
     Q_OBJECT
     Q_DECLARE_PRIVATE(QGridLayout)
+    QDOC_PROPERTY(int horizontalSpacing READ horizontalSpacing WRITE setHorizontalSpacing)
+    QDOC_PROPERTY(int verticalSpacing READ verticalSpacing WRITE setVerticalSpacing)
+
 public:
     explicit QGridLayout(QWidget *parent);
     QGridLayout();
@@ -58,6 +83,13 @@ public:
     QSize minimumSize() const;
     QSize maximumSize() const;
 
+    void setHorizontalSpacing(int spacing);
+    int horizontalSpacing() const;
+    void setVerticalSpacing(int spacing);
+    int verticalSpacing() const;
+    void setSpacing(int spacing);
+    int spacing() const;
+
     void setRowStretch(int row, int stretch);
     void setColumnStretch(int column, int stretch);
     int rowStretch(int row) const;
@@ -96,8 +128,9 @@ public:
     inline QT3_SUPPORT void setOrigin(Qt::Corner corner) { setOriginCorner(corner); }
     inline QT3_SUPPORT Qt::Corner origin() const { return originCorner(); }
 #endif
-    QLayoutItem *itemAt(int) const;
-    QLayoutItem *takeAt(int);
+    QLayoutItem *itemAt(int index) const;
+    QLayoutItem *itemAtPosition(int row, int column) const;
+    QLayoutItem *takeAt(int index);
     int count() const;
     void setGeometry(const QRect&);
 
@@ -138,6 +171,8 @@ public:
 #endif
 };
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QGRIDLAYOUT_H
