@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,8 @@
 #include "qicon.h"
 #include "qimage.h"
 #include "qkeysequence.h"
+#include "qtransform.h"
+#include "qmatrix.h"
 #include "qpalette.h"
 #include "qpen.h"
 #include "qpixmap.h"
@@ -43,6 +65,8 @@
 
 #include "private/qvariant_p.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifdef QT3_SUPPORT
 extern QDataStream &qt_stream_out_qcolorgroup(QDataStream &s, const QColorGroup &g);
 extern QDataStream &qt_stream_in_qcolorgroup(QDataStream &s, QColorGroup &g);
@@ -90,6 +114,12 @@ static void construct(QVariant::Private *x, const void *copy)
         v_construct<QIcon>(x, copy);
         break;
 #endif
+    case QVariant::Matrix:
+        v_construct<QMatrix>(x, copy);
+        break;
+    case QVariant::Transform:
+        v_construct<QTransform>(x, copy);
+        break;
     case QVariant::TextFormat:
         v_construct<QTextFormat>(x, copy);
         break;
@@ -112,6 +142,14 @@ static void construct(QVariant::Private *x, const void *copy)
         v_construct<QCursor>(x, copy);
         break;
 #endif
+    case 62: {
+        // small 'trick' to let a QVariant(Qt::blue) create a variant
+        // of type QColor
+        x->type = QVariant::Color;
+        QColor color(*reinterpret_cast<const Qt::GlobalColor *>(copy));
+        v_construct<QColor>(x, &color);
+        break;
+    }
     default:
         qcoreVariantHandler()->construct(x, copy);
         return;
@@ -162,6 +200,12 @@ static void clear(QVariant::Private *d)
         v_clear<QIcon>(d);
         break;
 #endif
+    case QVariant::Matrix:
+        v_clear<QMatrix>(d);
+        break;
+    case QVariant::Transform:
+        v_clear<QTransform>(d);
+        break;
     case QVariant::TextFormat:
         v_clear<QTextFormat>(d);
         break;
@@ -207,6 +251,7 @@ static bool isNull(const QVariant::Private *d)
     case QVariant::Icon:
         return v_cast<QIcon>(d)->isNull();
 #endif
+    case QVariant::Matrix:
     case QVariant::TextFormat:
     case QVariant::TextLength:
     case QVariant::Cursor:
@@ -230,164 +275,6 @@ static bool isNull(const QVariant::Private *d)
     return d->is_null;
 }
 
-#ifndef QT_NO_DATASTREAM
-static void load(QVariant::Private *d, QDataStream &s)
-{
-    switch (d->type) {
-#ifndef QT_NO_CURSOR
-    case QVariant::Cursor:
-        s >> *v_cast<QCursor>(d);
-        break;
-#endif
-    case QVariant::Bitmap: 
-        s >> *v_cast<QBitmap>(d);
-        break;
-    case QVariant::Region:
-        s >> *v_cast<QRegion>(d);
-        break;
-    case QVariant::Polygon:
-        s >> *v_cast<QPolygon>(d);
-        break;
-    case QVariant::Font:
-        s >> *v_cast<QFont>(d);
-        break;
-    case QVariant::Pixmap:
-        s >> *v_cast<QPixmap>(d);
-        break;
-    case QVariant::Image:
-        s >> *v_cast<QImage>(d);
-        break;
-    case QVariant::Brush:
-        s >> *v_cast<QBrush>(d);
-        break;
-    case QVariant::Color:
-        s >> *v_cast<QColor>(d);
-        break;
-    case QVariant::Palette:
-        s >> *v_cast<QPalette>(d);
-        break;
-#ifdef QT3_SUPPORT
-    case QVariant::ColorGroup:
-        qt_stream_in_qcolorgroup(s, *v_cast<QColorGroup>(d));
-        break;
-#endif
-#ifndef QT_NO_ICON
-    case QVariant::Icon: {
-        QPixmap x;
-        s >> x;
-        *v_cast<QIcon>(d) = QIcon(x);
-        break;
-    }
-#endif
-    case QVariant::TextFormat: {
-        QTextFormat x;
-        s >> x;
-        *v_cast<QTextFormat>(d) = x;
-        break;
-    }
-    case QVariant::TextLength: {
-        QTextLength x;
-        s >> x;
-        *v_cast<QTextLength>(d) = x;
-        break;
-    }
-    case QVariant::SizePolicy: {
-        int h, v;
-        qint8 hfw;
-        s >> h >> v >> hfw;
-        QSizePolicy *sp = v_cast<QSizePolicy>(d);
-        *sp = QSizePolicy(QSizePolicy::Policy(h), QSizePolicy::Policy(v));
-        sp->setHeightForWidth(bool(hfw));
-        break;
-    }
-#ifndef QT_NO_SHORTCUT
-    case QVariant::KeySequence:
-        s >> *v_cast<QKeySequence>(d);
-        break;
-#endif // QT_NO_SHORTCUT
-    case QVariant::Pen:
-        s >> *v_cast<QPen>(d);
-        break;
-    default:
-        qcoreVariantHandler()->load(d, s);
-        return;
-    }
-}
-
-
-static void save(const QVariant::Private *d, QDataStream &s)
-{
-    switch (d->type) {
-#ifndef QT_NO_CURSOR
-    case QVariant::Cursor:
-        s << *v_cast<QCursor>(d);
-        break;
-#endif
-    case QVariant::Bitmap:
-        s << *v_cast<QBitmap>(d);
-        break;
-    case QVariant::Polygon:
-        s << *v_cast<QPolygon>(d);
-        break;
-    case QVariant::Region:
-        s << *v_cast<QRegion>(d);
-        break;
-    case QVariant::Font:
-        s << *v_cast<QFont>(d);
-        break;
-    case QVariant::Pixmap:
-        s << *v_cast<QPixmap>(d);
-        break;
-    case QVariant::Image:
-        s << *v_cast<QImage>(d);
-        break;
-    case QVariant::Brush:
-        s << *v_cast<QBrush>(d);
-        break;
-    case QVariant::Color:
-        s << *v_cast<QColor>(d);
-        break;
-    case QVariant::Palette:
-        s << *v_cast<QPalette>(d);
-        break;
-#ifdef QT3_SUPPORT
-    case QVariant::ColorGroup:
-        qt_stream_out_qcolorgroup(s, *v_cast<QColorGroup>(d));
-        break;
-#endif
-#ifndef QT_NO_ICON
-    case QVariant::Icon:
-        //### add stream operator to icon
-        s << v_cast<QIcon>(d)->pixmap(QSize(22, 22)); //FIXME
-        break;
-#endif
-    case QVariant::TextFormat:
-        s << *v_cast<QTextFormat>(d);
-        break;
-    case QVariant::TextLength:
-        s << *v_cast<QTextLength>(d);
-        break;
-    case QVariant::SizePolicy:
-    {
-        const QSizePolicy *p = v_cast<QSizePolicy>(d);
-        s << (int) p->horizontalPolicy() << (int) p->verticalPolicy()
-          << (qint8) p->hasHeightForWidth();
-    }
-    break;
-#ifndef QT_NO_SHORTCUT
-    case QVariant::KeySequence:
-        s << *v_cast<QKeySequence>(d);
-        break;
-#endif
-    case QVariant::Pen:
-        s << *v_cast<QPen>(d);
-        break;
-    default:
-        qcoreVariantHandler()->save(d, s);
-    }
-}
-#endif // QT_NO_DATASTREAM
-
 static bool compare(const QVariant::Private *a, const QVariant::Private *b)
 {
     Q_ASSERT(a->type == b->type);
@@ -397,8 +284,8 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
         return v_cast<QCursor>(a)->shape() == v_cast<QCursor>(b)->shape();
 #endif
     case QVariant::Bitmap:
-        return v_cast<QBitmap>(a)->serialNumber()
-            == v_cast<QBitmap>(b)->serialNumber();
+        return v_cast<QBitmap>(a)->cacheKey()
+            == v_cast<QBitmap>(b)->cacheKey();
     case QVariant::Polygon:
         return *v_cast<QPolygon>(a) == *v_cast<QPolygon>(b);
     case QVariant::Region:
@@ -406,7 +293,7 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
     case QVariant::Font:
         return *v_cast<QFont>(a) == *v_cast<QFont>(b);
     case QVariant::Pixmap:
-        return v_cast<QPixmap>(a)->serialNumber() == v_cast<QPixmap>(b)->serialNumber();
+        return v_cast<QPixmap>(a)->cacheKey() == v_cast<QPixmap>(b)->cacheKey();
     case QVariant::Image:
         return *v_cast<QImage>(a) == *v_cast<QImage>(b);
     case QVariant::Brush:
@@ -421,8 +308,14 @@ static bool compare(const QVariant::Private *a, const QVariant::Private *b)
 #endif
 #ifndef QT_NO_ICON
     case QVariant::Icon:
-        return false; // #### FIXME
+        /* QIcon::operator==() cannot be reasonably implemented for QIcon,
+         * so we always return false. */
+        return false;
 #endif
+    case QVariant::Matrix:
+        return *v_cast<QMatrix>(a) == *v_cast<QMatrix>(b);
+    case QVariant::Transform:
+        return *v_cast<QTransform>(a) == *v_cast<QTransform>(b);
     case QVariant::TextFormat:
         return *v_cast<QTextFormat>(a) == *v_cast<QTextFormat>(b);
     case QVariant::TextLength:
@@ -479,6 +372,11 @@ static bool convert(const QVariant::Private *d, QVariant::Type t,
         } else if (d->type == QVariant::Bitmap) {
             *static_cast<QPixmap *>(result) = *v_cast<QBitmap>(d);
             return true;
+        } else if (d->type == QVariant::Brush) {
+            if (v_cast<QBrush>(d)->style() == Qt::TexturePattern) {
+                *static_cast<QPixmap *>(result) = v_cast<QBrush>(d)->texture();
+                return true;
+            }
         }
         break;
     case QVariant::Image:
@@ -517,11 +415,25 @@ static bool convert(const QVariant::Private *d, QVariant::Type t,
     case QVariant::Color:
         if (d->type == QVariant::String) {
             static_cast<QColor *>(result)->setNamedColor(*v_cast<QString>(d));
-            return true;
+            return static_cast<QColor *>(result)->isValid();
         } else if (d->type == QVariant::ByteArray) {
             static_cast<QColor *>(result)->setNamedColor(QString::fromLatin1(
                                 *v_cast<QByteArray>(d)));
             return true;
+        } else if (d->type == QVariant::Brush) {
+            if (v_cast<QBrush>(d)->style() == Qt::SolidPattern) {
+                *static_cast<QColor *>(result) = v_cast<QBrush>(d)->color();
+                return true;
+            }
+        }
+        break;
+    case QVariant::Brush:
+        if (d->type == QVariant::Color) {
+            *static_cast<QBrush *>(result) = QBrush(*v_cast<QColor>(d));
+            return true;
+        } else if (d->type == QVariant::Pixmap) {
+            *static_cast<QBrush *>(result) = QBrush(*v_cast<QPixmap>(d));
+            return true;
         }
         break;
 #ifndef QT_NO_SHORTCUT
@@ -545,42 +457,6 @@ static bool convert(const QVariant::Private *d, QVariant::Type t,
     return qcoreVariantHandler()->convert(d, t, result, ok);
 }
 
-static bool canConvert(const QVariant::Private *d, QVariant::Type t)
-{
-    if (d->type == uint(t))
-        return true;
-
-    switch (t) {
-    case QVariant::Int:
-        if (d->type == QVariant::KeySequence)
-            return true;
-        break;
-    case QVariant::Image:
-        return d->type == QVariant::Pixmap || d->type == QVariant::Bitmap;
-    case QVariant::Pixmap:
-        return d->type == QVariant::Image || d->type == QVariant::Bitmap;
-    case QVariant::Bitmap:
-        return d->type == QVariant::Pixmap || d->type == QVariant::Image;
-    case QVariant::ByteArray:
-        if (d->type == QVariant::Color)
-            return true;
-        break;
-    case QVariant::String:
-        if (d->type == QVariant::KeySequence || d->type == QVariant::Font || d->type == QVariant::Color)
-            return true;
-        break;
-    case QVariant::KeySequence:
-        return d->type == QVariant::String || d->type == QVariant::Int;
-    case QVariant::Font:
-        return d->type == QVariant::String;
-    case QVariant::Color:
-        return d->type == QVariant::String || d->type == QVariant::ByteArray;
-    default:
-        break;
-    }
-    return qcoreVariantHandler()->canConvert(d, t);
-}
-
 #if !defined(QT_NO_DEBUG_STREAM) && !defined(Q_BROKEN_DEBUG_STREAM)
 static void streamDebug(QDebug dbg, const QVariant &v)
 {
@@ -602,6 +478,12 @@ static void streamDebug(QDebug dbg, const QVariant &v)
     case QVariant::Font:
 //        dbg.nospace() << qvariant_cast<QFont>(v);  //FIXME
         break;
+    case QVariant::Matrix:
+        dbg.nospace() << qvariant_cast<QMatrix>(v);
+        break;
+    case QVariant::Transform:
+        dbg.nospace() << qvariant_cast<QTransform>(v);
+        break;
     case QVariant::Pixmap:
 //        dbg.nospace() << qvariant_cast<QPixmap>(v); //FIXME
         break;
@@ -645,12 +527,12 @@ const QVariant::Handler qt_gui_variant_handler = {
     clear,
     isNull,
 #ifndef QT_NO_DATASTREAM
-    load,
-    save,
+    0,
+    0,
 #endif
     compare,
     convert,
-    canConvert,
+    0,
 #if !defined(QT_NO_DEBUG_STREAM) && !defined(Q_BROKEN_DEBUG_STREAM)
     streamDebug
 #else
@@ -658,10 +540,137 @@ const QVariant::Handler qt_gui_variant_handler = {
 #endif
 };
 
+struct QMetaTypeGuiHelper
+{
+    QMetaType::Constructor constr;
+    QMetaType::Destructor destr;
+#ifndef QT_NO_DATASTREAM
+    QMetaType::SaveOperator saveOp;
+    QMetaType::LoadOperator loadOp;
+#endif
+};
+
+#ifdef QT_MAKEDLL
+#  define Q_VARIANT_DECL_IMPORT Q_DECL_IMPORT
+#else
+#  define Q_VARIANT_DECL_IMPORT
+#endif
+extern Q_VARIANT_DECL_IMPORT const QMetaTypeGuiHelper *qMetaTypeGuiHelper;
+
+
+#ifdef QT_NO_DATASTREAM
+#  define Q_DECL_METATYPE_HELPER(TYPE) \
+     typedef void *(*QConstruct##TYPE)(const TYPE *); \
+     static const QConstruct##TYPE qConstruct##TYPE = qMetaTypeConstructHelper<TYPE>; \
+     typedef void (*QDestruct##TYPE)(TYPE *); \
+     static const QDestruct##TYPE qDestruct##TYPE = qMetaTypeDeleteHelper<TYPE>;
+#else
+#  define Q_DECL_METATYPE_HELPER(TYPE) \
+     typedef void *(*QConstruct##TYPE)(const TYPE *); \
+     static const QConstruct##TYPE qConstruct##TYPE = qMetaTypeConstructHelper<TYPE>; \
+     typedef void (*QDestruct##TYPE)(TYPE *); \
+     static const QDestruct##TYPE qDestruct##TYPE = qMetaTypeDeleteHelper<TYPE>; \
+     typedef void (*QSave##TYPE)(QDataStream &, const TYPE *); \
+     static const QSave##TYPE qSave##TYPE = qMetaTypeSaveHelper<TYPE>; \
+     typedef void (*QLoad##TYPE)(QDataStream &, TYPE *); \
+     static const QLoad##TYPE qLoad##TYPE = qMetaTypeLoadHelper<TYPE>;
+#endif
+
+#ifdef QT3_SUPPORT
+Q_DECL_METATYPE_HELPER(QColorGroup)
+#endif
+Q_DECL_METATYPE_HELPER(QFont)
+Q_DECL_METATYPE_HELPER(QPixmap)
+Q_DECL_METATYPE_HELPER(QBrush)
+Q_DECL_METATYPE_HELPER(QColor)
+Q_DECL_METATYPE_HELPER(QPalette)
+#ifndef QT_NO_ICON
+Q_DECL_METATYPE_HELPER(QIcon)
+#endif
+Q_DECL_METATYPE_HELPER(QImage)
+Q_DECL_METATYPE_HELPER(QPolygon)
+Q_DECL_METATYPE_HELPER(QRegion)
+Q_DECL_METATYPE_HELPER(QBitmap)
+#ifndef QT_NO_CURSOR
+Q_DECL_METATYPE_HELPER(QCursor)
+#endif
+Q_DECL_METATYPE_HELPER(QSizePolicy)
+#ifndef QT_NO_SHORTCUT
+Q_DECL_METATYPE_HELPER(QKeySequence)
+#endif
+Q_DECL_METATYPE_HELPER(QPen)
+Q_DECL_METATYPE_HELPER(QTextLength)
+Q_DECL_METATYPE_HELPER(QTextFormat)
+Q_DECL_METATYPE_HELPER(QMatrix)
+Q_DECL_METATYPE_HELPER(QTransform)
+
+#ifdef QT_NO_DATASTREAM
+#  define Q_IMPL_METATYPE_HELPER(TYPE) \
+     { reinterpret_cast<QMetaType::Constructor>(qConstruct##TYPE), \
+       reinterpret_cast<QMetaType::Destructor>(qDestruct##TYPE) }
+#else
+#  define Q_IMPL_METATYPE_HELPER(TYPE) \
+     { reinterpret_cast<QMetaType::Constructor>(qConstruct##TYPE), \
+       reinterpret_cast<QMetaType::Destructor>(qDestruct##TYPE), \
+       reinterpret_cast<QMetaType::SaveOperator>(qSave##TYPE), \
+       reinterpret_cast<QMetaType::LoadOperator>(qLoad##TYPE) \
+     }
+#endif
+
+static const QMetaTypeGuiHelper qVariantGuiHelper[] = {
+#ifdef QT3_SUPPORT
+    Q_IMPL_METATYPE_HELPER(QColorGroup),
+#else
+    {0, 0, 0, 0},
+#endif
+    Q_IMPL_METATYPE_HELPER(QFont),
+    Q_IMPL_METATYPE_HELPER(QPixmap),
+    Q_IMPL_METATYPE_HELPER(QBrush),
+    Q_IMPL_METATYPE_HELPER(QColor),
+    Q_IMPL_METATYPE_HELPER(QPalette),
+#ifdef QT_NO_ICON
+    {0, 0, 0, 0},
+#else
+    Q_IMPL_METATYPE_HELPER(QIcon),
+#endif
+    Q_IMPL_METATYPE_HELPER(QImage),
+    Q_IMPL_METATYPE_HELPER(QPolygon),
+    Q_IMPL_METATYPE_HELPER(QRegion),
+    Q_IMPL_METATYPE_HELPER(QBitmap),
+#ifdef QT_NO_CURSOR
+    {0, 0, 0, 0},
+#else
+    Q_IMPL_METATYPE_HELPER(QCursor),
+#endif
+    Q_IMPL_METATYPE_HELPER(QSizePolicy),
+#ifdef QT_NO_SHORTCUT
+    {0, 0, 0, 0},
+#else
+    Q_IMPL_METATYPE_HELPER(QKeySequence),
+#endif
+    Q_IMPL_METATYPE_HELPER(QPen),
+    Q_IMPL_METATYPE_HELPER(QTextLength),
+    Q_IMPL_METATYPE_HELPER(QTextFormat),
+    Q_IMPL_METATYPE_HELPER(QMatrix),
+    Q_IMPL_METATYPE_HELPER(QTransform)
+};
+
+static const QVariant::Handler *qt_guivariant_last_handler = 0;
 int qRegisterGuiVariant()
 {
+    qt_guivariant_last_handler = QVariant::handler;
     QVariant::handler = &qt_gui_variant_handler;
+    qMetaTypeGuiHelper = qVariantGuiHelper;
     return 1;
 }
-
 Q_CONSTRUCTOR_FUNCTION(qRegisterGuiVariant)
+
+int qUnregisterGuiVariant()
+{
+    QVariant::handler = qt_guivariant_last_handler;
+    qMetaTypeGuiHelper = 0;
+    return 1;
+}
+Q_DESTRUCTOR_FUNCTION(qUnregisterGuiVariant)
+
+QT_END_NAMESPACE
