@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -45,6 +65,8 @@
 # define QMAC_SHIFT QChar(kShiftUnicode)
 #endif
 
+QT_BEGIN_NAMESPACE
+
 #ifdef Q_WS_MAC
 static bool qt_sequence_no_mnemonics = true;
 #else
@@ -87,12 +109,7 @@ void Q_GUI_EXPORT qt_set_sequence_auto_mnemonic(bool b) { qt_sequence_no_mnemoni
     printing a document, and can be specified in any of the following
     ways:
 
-    \code
-    QKeySequence(QKeySequence::Print}
-    QKeySequence(tr("Ctrl+P"))
-    QKeySequence(tr("Ctrl+p"))
-    QKeySequence(Qt::CTRL + Qt::Key_P)
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.kernel.qkeysequence.cpp 0
 
     Note that, for letters, the case used in the specification string does not
     matter. In the above examples, the user does not need to hold down the
@@ -160,7 +177,7 @@ void Q_GUI_EXPORT qt_set_sequence_auto_mnemonic(bool b) { qt_sequence_no_mnemoni
     \row    \i PreviousChild    \i Ctrl+Shift+Tab, Back, Ctrl+Shift+F6  \i Ctrl+{, Back, Ctrl+Shift+Tab \i Ctrl+Shift+Tab, Back, Ctrl+Period \i Ctrl+Shift+Tab, Back
     \row    \i Find             \i Ctrl+F                               \i Ctrl+F                   \i Ctrl+F         \i Ctrl+F
     \row    \i FindNext         \i F3, Ctrl+G                           \i Ctrl+G                   \i F3             \i Ctrl+G, F3
-    \row    \i FindPrevious     \i Shift+F3, Ctrl+Shift+G               \i Ctrl+Shift+G             \i Shift+F3       \i Ctrl+Shift+G, F3
+    \row    \i FindPrevious     \i Shift+F3, Ctrl+Shift+G               \i Ctrl+Shift+G             \i Shift+F3       \i Ctrl+Shift+G, Shift+F3
     \row    \i Replace          \i Ctrl+H                               \i (none)                   \i Ctrl+R         \i Ctrl+H
     \row    \i SelectAll        \i Ctrl+A                               \i Ctrl+A                   \i Ctrl+A         \i Ctrl+A 
     \row    \i Bold             \i Ctrl+B                               \i Ctrl+B                   \i Ctrl+B         \i Ctrl+B
@@ -259,10 +276,7 @@ void Q_GUI_EXPORT qt_set_sequence_auto_mnemonic(bool b) { qt_sequence_no_mnemoni
     For example, the key sequence, \key{Ctrl X} followed by \key{Ctrl C}, can
     be specified using either of the following ways:
 
-    \code
-    QKeySequence(tr("Ctrl+X, Ctrl+C"))
-    QKeySequence(Qt::CTRL + Qt::Key_X, Qt::CTRL + Qt::Key_C)
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.kernel.qkeysequence.cpp 1
 
     \sa QShortcut
 */
@@ -283,7 +297,7 @@ void Q_GUI_EXPORT qt_set_sequence_auto_mnemonic(bool b) { qt_sequence_no_mnemoni
 
     \value NativeText The key sequence as a platform specific string.
     This means that it will be shown translated and on the Mac it will
-    resemble a keysequence from the menu bar. This enum is best used when you
+    resemble a key sequence from the menu bar. This enum is best used when you
     want to display the string to the user.
 
     \value PortableText The key sequence is given in a "portable" format,
@@ -498,8 +512,8 @@ const QKeyBinding QKeySequencePrivate::keyBindings[] = {
     {QKeySequence::AddTab,                  1,          Qt::CTRL | Qt::SHIFT | Qt::Key_N,       QApplicationPrivate::KB_KDE},
     {QKeySequence::Redo,                    0,          Qt::CTRL | Qt::SHIFT | Qt::Key_Z,       QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
     {QKeySequence::Redo,                    1,          Qt::CTRL | Qt::SHIFT | Qt::Key_Z,       QApplicationPrivate::KB_Mac}, //different priority from above
-    {QKeySequence::PreviousChild,           1,          Qt::CTRL | Qt::SHIFT | Qt::Key_Tab,     QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
-    {QKeySequence::PreviousChild,           0,          Qt::CTRL | Qt::SHIFT | Qt::Key_Tab,     QApplicationPrivate::KB_Mac },//different priority from above 
+    {QKeySequence::PreviousChild,           1,          Qt::CTRL | Qt::SHIFT | Qt::Key_Backtab, QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
+    {QKeySequence::PreviousChild,           0,          Qt::CTRL | Qt::SHIFT | Qt::Key_Backtab, QApplicationPrivate::KB_Mac },//different priority from above 
     {QKeySequence::SelectStartOfDocument,   0,          Qt::CTRL | Qt::SHIFT | Qt::Key_Home,    QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
     {QKeySequence::SelectEndOfDocument,     0,          Qt::CTRL | Qt::SHIFT | Qt::Key_End,     QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
     {QKeySequence::SelectPreviousWord,      0,          Qt::CTRL | Qt::SHIFT | Qt::Key_Left,    QApplicationPrivate::KB_Win | QApplicationPrivate::KB_X11},
@@ -660,11 +674,7 @@ QKeySequence::QKeySequence()
     \endlink(), so that shortcut keys can be replaced in
     translations:
 
-    \code
-        QMenu *file = new QMenu(this);
-        file->addAction(tr("&Open..."), this, SLOT(open()),
-                          QKeySequence(tr("Ctrl+O", "File|Open")));
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.kernel.qkeysequence.cpp 2
 
     Note the "File|Open" translator comment. It is by no means
     necessary, but it provides some context for the human translator.
@@ -788,6 +798,8 @@ bool QKeySequence::isEmpty() const
     in English. At the time of writing, Microsoft and Open Group do
     not appear to have issued equivalent recommendations for other
     languages.
+
+    \sa qt_set_sequence_auto_mnemonic()
 */
 QKeySequence QKeySequence::mnemonic(const QString &text)
 {
@@ -935,7 +947,7 @@ int QKeySequencePrivate::decodeString(const QString &str, QKeySequence::Sequence
 #else
     int i = 0;
     int lastI = 0;
-    while ((i = sl.indexOf('+', i + 1)) != -1) {
+    while ((i = sl.indexOf(QLatin1Char('+'), i + 1)) != -1) {
         const QString sub = sl.mid(lastI, i - lastI + 1);
         // Just shortcut the check here if we only have one character.
         // Rational: A modifier will contain the name AND +, so longer than 1, a length of 1 is just
@@ -1001,7 +1013,7 @@ static inline void addKey(QString &str, const QString &theKey, QKeySequence::Seq
 {
     if (!str.isEmpty())
         str += (format == QKeySequence::NativeText) ? QShortcut::tr("+")
-                                                    : QString(QLatin1String("+"));
+                                                    : QString::fromLatin1("+");
     str += theKey;
 }
 
@@ -1025,13 +1037,13 @@ QString QKeySequencePrivate::encodeString(int key, QKeySequence::SequenceFormat
     {
         // On other systems the order is Meta, Control, Alt, Shift
         if ((key & Qt::META) == Qt::META)
-            s = nativeText ? QShortcut::tr("Meta") : QString(QLatin1String("Meta"));
+            s = nativeText ? QShortcut::tr("Meta") : QString::fromLatin1("Meta");
         if ((key & Qt::CTRL) == Qt::CTRL)
-            addKey(s, nativeText ? QShortcut::tr("Ctrl") : QString(QLatin1String("Ctrl")), format);
+            addKey(s, nativeText ? QShortcut::tr("Ctrl") : QString::fromLatin1("Ctrl"), format);
         if ((key & Qt::ALT) == Qt::ALT)
-            addKey(s, nativeText ? QShortcut::tr("Alt") : QString(QLatin1String("Alt")), format);
+            addKey(s, nativeText ? QShortcut::tr("Alt") : QString::fromLatin1("Alt"), format);
         if ((key & Qt::SHIFT) == Qt::SHIFT)
-            addKey(s, nativeText ? QShortcut::tr("Shift") : QString(QLatin1String("Shift")), format);
+            addKey(s, nativeText ? QShortcut::tr("Shift") : QString::fromLatin1("Shift"), format);
     }
 
 
@@ -1047,13 +1059,13 @@ QString QKeySequencePrivate::encodeString(int key, QKeySequence::SequenceFormat
         }
     } else if (key >= Qt::Key_F1 && key <= Qt::Key_F35) {
             p = nativeText ? QShortcut::tr("F%1").arg(key - Qt::Key_F1 + 1)
-                           : QString(QLatin1String("F%1")).arg(key - Qt::Key_F1 + 1);
+                           : QString::fromLatin1("F%1").arg(key - Qt::Key_F1 + 1);
     } else if (key) {
         int i=0;
         while (keyname[i].name) {
             if (key == keyname[i].key) {
                 p = nativeText ? QShortcut::tr(keyname[i].name)
-                               : QString(QLatin1String(keyname[i].name));
+                               : QString::fromLatin1(keyname[i].name);
                 break;
             }
             ++i;
@@ -1111,7 +1123,11 @@ QKeySequence::SequenceMatch QKeySequence::matches(const QKeySequence &seq) const
 /*!
     \obsolete
 
-    Use toString() instead.
+    Use toString() instead. 
+    
+    Returns the key sequence as a QString. This is equivalent to 
+    calling toString(QKeySequence::NativeText). Note that the
+    result is not platform independent.
 */
 QKeySequence::operator QString() const
 {
@@ -1251,7 +1267,7 @@ bool QKeySequence::isDetached() const
     If the key sequence has no keys, an empty string is returned.
 
     On Mac OS X, the string returned resembles the sequence that is
-    shown in the menubar.
+    shown in the menu bar.
 
     \sa fromString()
 */
@@ -1348,3 +1364,16 @@ QDebug operator<<(QDebug dbg, const QKeySequence &p)
 #endif
 
 #endif // QT_NO_SHORTCUT
+
+
+/*!
+    \typedef QKeySequence::DataPtr
+    \internal
+*/
+
+ /*!
+    \fn DataPtr &QKeySequence::data_ptr()
+    \internal
+*/
+
+QT_END_NAMESPACE
