@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -57,6 +57,8 @@
 
 #include "qkeysequence.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_SHORTCUT
 struct Q_AUTOTEST_EXPORT QKeyBinding
 {
@@ -82,15 +84,17 @@ public:
         key[2] = copy.key[2];
         key[3] = copy.key[3];
     }
-    QAtomic ref;
+    QAtomicInt ref;
     int key[4];
     static QString encodeString(int key, QKeySequence::SequenceFormat format);
     static int decodeString(const QString &keyStr, QKeySequence::SequenceFormat format);
 
     static const QKeyBinding keyBindings[];
     static const uint numberOfKeyBindings;
-    
+
 };
 #endif // QT_NO_SHORTCUT
 
+QT_END_NAMESPACE
+
 #endif //QKEYSEQUENCE_P_H
