@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -53,6 +53,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QLayout;
@@ -158,7 +160,7 @@ public:
     Qt::Orientations expandingDirections() const;
     QSize minimumSize() const;
     QSize maximumSize() const;
-    void setGeometry(const QRect&) = 0;
+    virtual void setGeometry(const QRect&);
     virtual QLayoutItem *itemAt(int index) const = 0;
     virtual QLayoutItem *takeAt(int index) = 0;
     virtual int indexOf(QWidget *) const;
@@ -229,10 +231,14 @@ inline void QLayoutIterator::deleteCurrent() { delete  layout->takeAt(index); }
 
 //### support old includes
 #if 1 //def QT3_SUPPORT
+QT_BEGIN_INCLUDE_NAMESPACE
 #include <QtGui/qboxlayout.h>
 #include <QtGui/qgridlayout.h>
+QT_END_INCLUDE_NAMESPACE
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QLAYOUT_H
