@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +48,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class Q_GUI_EXPORT QMimeSource
@@ -42,11 +64,13 @@ public:
 
 #if defined(Q_WS_WIN)
 
+QT_BEGIN_INCLUDE_NAMESPACE
 typedef struct tagFORMATETC FORMATETC;
 typedef struct tagSTGMEDIUM STGMEDIUM;
 struct IDataObject;
 
 #include <QtCore/qvariant.h>
+QT_END_INCLUDE_NAMESPACE
 
 /*
   Encapsulation of conversion between MIME and Windows CLIPFORMAT.
@@ -54,7 +78,8 @@ struct IDataObject;
   directly.
 */
 
-class Q_GUI_EXPORT QWindowsMime {
+class Q_GUI_EXPORT QWindowsMime
+{
 public:
     QWindowsMime();
     virtual ~QWindowsMime();
@@ -88,22 +113,22 @@ private:
 
 /*
   Encapsulation of conversion between MIME and Mac flavor.
-  Not need on X11, as the underlying protocol uses the MIME standard
+  Not needed on X11, as the underlying protocol uses the MIME standard
   directly.
 */
 
-class Q_GUI_EXPORT QMacMime {
+class Q_GUI_EXPORT QMacMime { //Obsolete
     char type;
 public:
     enum QMacMimeType { MIME_DND=0x01, MIME_CLIP=0x02, MIME_QT_CONVERTOR=0x04, MIME_ALL=MIME_DND|MIME_CLIP };
-    explicit QMacMime(char);
-    virtual ~QMacMime();
+    explicit QMacMime(char) { }
+    virtual ~QMacMime() { }
 
-    static void initialize();
+    static void initialize() { }
 
-    static QList<QMacMime*> all(QMacMimeType);
-    static QMacMime *convertor(QMacMimeType, const QString &mime, int flav);
-    static QString flavorToMime(QMacMimeType, int flav);
+    static QList<QMacMime*> all(QMacMimeType) { return QList<QMacMime*>(); }
+    static QMacMime *convertor(QMacMimeType, const QString &, int) { return 0; }
+    static QString flavorToMime(QMacMimeType, int) { return QString(); }
 
     virtual QString convertorName()=0;
     virtual int countFlavors()=0;
@@ -115,8 +140,37 @@ public:
     virtual QList<QByteArray> convertFromMime(const QString &mime, QVariant data, int flav)=0;
 };
 
+class Q_GUI_EXPORT QMacPasteboardMime {
+    char type;
+public:
+    enum QMacPasteboardMimeType { MIME_DND=0x01,
+                                  MIME_CLIP=0x02,
+                                  MIME_QT_CONVERTOR=0x04,
+                                  MIME_QT3_CONVERTOR=0x08,
+                                  MIME_ALL=MIME_DND|MIME_CLIP
+    };
+    explicit QMacPasteboardMime(char);
+    virtual ~QMacPasteboardMime();
+
+    static void initialize();
+
+    static QList<QMacPasteboardMime*> all(uchar);
+    static QMacPasteboardMime *convertor(uchar, const QString &mime, QString flav);
+    static QString flavorToMime(uchar, QString flav);
+
+    virtual QString convertorName() = 0;
+
+    virtual bool canConvert(const QString &mime, QString flav) = 0;
+    virtual QString mimeFor(QString flav) = 0;
+    virtual QString flavorFor(const QString &mime) = 0;
+    virtual QVariant convertToMime(const QString &mime, QList<QByteArray> data, QString flav) = 0;
+    virtual QList<QByteArray> convertFromMime(const QString &mime, QVariant data, QString flav) = 0;
+};
+
 #endif // Q_WS_MAC
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QMIME_H
