@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,11 +46,14 @@
 #include "qdatastream.h"
 #include "qvariant.h"
 
+QT_BEGIN_NAMESPACE
+
 static int qt_palette_count = 1;
+
 class QPalettePrivate {
 public:
     QPalettePrivate() : ref(1), ser_no(qt_palette_count++), detach_no(0) { }
-    QAtomic ref;
+    QAtomicInt ref;
     QBrush br[QPalette::NColorGroups][QPalette::NColorRoles];
     int ser_no;
     int detach_no;
@@ -74,8 +77,10 @@ QDataStream &qt_stream_out_qcolorgroup(QDataStream &s, const QColorGroup &g)
           << g.color(QPalette::Mid) << g.color(QPalette::Text) << g.color(QPalette::Base);
     } else {
         int max = QPalette::NColorRoles;
-        if(s.version() <= 3) // Qt 2.x
-            max = 14;
+        if (s.version() <= QDataStream::Qt_2_1)
+            max = QPalette::HighlightedText + 1;
+        if (s.version() <= QDataStream::Qt_4_3)
+            max = QPalette::AlternateBase + 1;
         for(int r = 0 ; r < max ; r++)
             s << g.brush((QPalette::ColorRole)r);
     }
@@ -98,10 +103,12 @@ QDataStream &qt_stream_in_qcolorgroup(QDataStream &s, QColorGroup &g)
         g.setCurrentColorGroup(QPalette::Active);
     } else {
         int max = QPalette::NColorRoles;
-        if (s.version() <= 3) // Qt 2.x
-            max = 14;
-        else if (s.version() <= 4) // Qt 3.x
-            max = 16;
+        if (s.version() <= QDataStream::Qt_2_1)
+            max = QPalette::HighlightedText + 1;
+        else if (s.version() <= QDataStream::Qt_3_0)
+            max = QPalette::LinkVisited + 1;
+        else if (s.version() <= QDataStream::Qt_4_3)
+            max = QPalette::AlternateBase + 1;
         QBrush tmp;
         for(int r = 0 ; r < max; r++) {
             s >> tmp;
@@ -129,6 +136,7 @@ QDataStream &operator>>(QDataStream &s, QColorGroup &g)
 QPalette::QPalette(const QColorGroup &active, const QColorGroup &disabled,
                    const QColorGroup &inactive)
 {
+    Q_ASSERT(QPalette::NColorRoles == QPalette::ToolTipText + 1);
     init();
     setColorGroup(Active, active);
     setColorGroup(Disabled, disabled);
@@ -149,7 +157,7 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
                   g.brush(Base), g.brush(AlternateBase), g.brush(Window),
                   g.brush(Midlight), g.brush(ButtonText), g.brush(Shadow),
                   g.brush(Highlight), g.brush(HighlightedText), g.brush(Link),
-                  g.brush(LinkVisited));
+                  g.brush(LinkVisited), g.brush(ToolTipBase), g.brush(ToolTipText));
 }
 
 #endif
@@ -292,6 +300,24 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
     \sa ColorRole brush()
 */
 
+/*!
+    \fn const QBrush & QPalette::toolTipBase() const
+
+    Returns the tool tip base brush of the current color group. This brush is
+    used by QToolTip and QWhatsThis.
+
+    \sa ColorRole brush()
+*/
+
+/*!
+    \fn const QBrush & QPalette::toolTipText() const
+
+    Returns the tool tip text brush of the current color group. This brush is
+    used by QToolTip and QWhatsThis.
+
+    \sa ColorRole brush()
+*/
+
 /*!
     \fn const QBrush & QPalette::background() const
     \obsolete
@@ -461,6 +487,12 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
                           alternating row colors (see
                           QAbstractItemView::setAlternatingRowColors()).
 
+    \value ToolTipBase Used as the background color for QToolTip and
+                          QWhatsThis.
+
+    \value ToolTipText Used as the foreground color for QToolTip and
+                          QWhatsThis.
+
     \value Text  The foreground color used with \c Base. This is usually
                  the same as the \c WindowText, in which case it must provide
                  good contrast with \c Window and \c Base.
@@ -521,12 +553,12 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
     and the QTextDocument::setDefaultStyleSheet() function to alter
     the appearance of links. For example:
 
-    \quotefromfile doc/src/snippets/textdocument-css/main.cpp
-    \skipto QTextBrowser
-    \printuntil browser.document
+    \snippet doc/src/snippets/textdocument-css/main.cpp 0
+
+    \value NoRole No role; this special role is often used to indicate that a
+    role has not been assigned.
 
     \omitvalue NColorRoles
-    \omitvalue NoRole
 */
 
 /*!
@@ -695,13 +727,12 @@ void QPalette::init() {
 */
 QPalette &QPalette::operator=(const QPalette &p)
 {
-    QPalettePrivate *x = p.d;
-    x->ref.ref();
+    p.d->ref.ref();
     resolve_mask = p.resolve_mask;
     current_group = p.current_group;
-    x = qAtomicSetPtr(&d, x);
-    if(!x->ref.deref())
-        delete x;
+    if(!d->ref.deref())
+        delete d;
+    d = p.d;
     return *this;
 }
 
@@ -808,9 +839,9 @@ void QPalette::detach()
             for(int role = 0; role < (int)NColorRoles; role++)
                 x->br[grp][role] = d->br[grp][role];
         }
-        x = qAtomicSetPtr(&d, x);
-        if(!x->ref.deref())
-            delete x;
+        if(!d->ref.deref())
+            delete d;
+        d = x;
     }
     ++d->detach_no;
 }
@@ -987,9 +1018,11 @@ QDataStream &operator<<(QDataStream &s, const QPalette &p)
             for (int i = 0; i < NumOldRoles; ++i)
                 s << p.d->br[grp][oldRoles[i]].color();
         } else {
-            int max = QPalette::NColorRoles;
-            if (s.version() <= 3) // Qt 2.x
-                max = 14;
+            int max = QPalette::ToolTipText + 1;
+            if (s.version() <= QDataStream::Qt_2_1)
+                max = QPalette::HighlightedText + 1;
+            if (s.version() <= QDataStream::Qt_4_3)
+                max = QPalette::AlternateBase + 1;
             for (int r = 0; r < max; r++)
                 s << p.d->br[grp][r];
         }
@@ -1024,9 +1057,13 @@ QDataStream &operator>>(QDataStream &s, QPalette &p)
         readV1ColorGroup(s, p, QPalette::Inactive);
     } else {
         int max = QPalette::NColorRoles;
-        if (s.version() <= 3) { // Qt 2.x
+        if (s.version() <= QDataStream::Qt_2_1) {
             p = QPalette();
-            max = 14;
+            max = QPalette::HighlightedText + 1;
+        }
+        if (s.version() <= QDataStream::Qt_4_3) {
+            p = QPalette();
+            max = QPalette::AlternateBase + 1;
         }
 
         QBrush tmp;
@@ -1070,10 +1107,14 @@ void QPalette::setColorGroup(ColorGroup cg, const QBrush &windowText, const QBru
 {
     QBrush alt_base = QBrush(qt_mix_colors(base.color(), button.color()));
     QBrush mid_light = QBrush(qt_mix_colors(button.color(), light.color()));
+    QColor toolTipBase(255, 255, 220);
+    QColor toolTipText(0, 0, 0);
+
     setColorGroup(cg, windowText, button, light, dark, mid, text, bright_text, base,
                   alt_base, window, mid_light, text,
                   QBrush(Qt::black), QBrush(Qt::darkBlue), QBrush(Qt::white),
-                  QBrush(Qt::blue), QBrush(Qt::magenta));
+                  QBrush(Qt::blue), QBrush(Qt::magenta), QBrush(toolTipBase),
+                  QBrush(toolTipText));
 
     resolve_mask &= ~(1 << Highlight);
     resolve_mask &= ~(1 << HighlightedText);
@@ -1092,6 +1133,23 @@ QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBrush &b
                         const QBrush &button_text, const QBrush &shadow,
                         const QBrush &highlight, const QBrush &highlighted_text,
                         const QBrush &link, const QBrush &link_visited)
+{
+    setColorGroup(cg, foreground, button, light, dark, mid,
+                  text, bright_text, base, alternate_base, background,
+                  midlight, button_text, shadow, highlight, highlighted_text,
+                  link, link_visited, background, foreground);
+}
+
+/*!\internal*/
+void QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBrush &button,
+                             const QBrush &light, const QBrush &dark, const QBrush &mid,
+                             const QBrush &text, const QBrush &bright_text,
+                             const QBrush &base, const QBrush &alternate_base,
+                             const QBrush &background, const QBrush &midlight,
+                             const QBrush &button_text, const QBrush &shadow,
+                             const QBrush &highlight, const QBrush &highlighted_text,
+                             const QBrush &link, const QBrush &link_visited,
+                             const QBrush &toolTipBase, const QBrush &toolTipText)
 {
     detach();
     setBrush(cg, WindowText, foreground);
@@ -1111,6 +1169,8 @@ QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBrush &b
     setBrush(cg, HighlightedText, highlighted_text);
     setBrush(cg, Link, link);
     setBrush(cg, LinkVisited, link_visited);
+    setBrush(cg, ToolTipBase, toolTipBase);
+    setBrush(cg, ToolTipText, toolTipText);
 }
 
 /*!
@@ -1299,3 +1359,5 @@ QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBrush &b
     Returns true if this color group is not equal to \a other;
     otherwise returns false.
 */
+
+QT_END_NAMESPACE
