@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -30,6 +50,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifdef QT3_SUPPORT
@@ -64,10 +86,13 @@ public:
     enum ColorGroup { Active, Disabled, Inactive, NColorGroups, Current, All, Normal = Active };
     enum ColorRole { WindowText, Button, Light, Midlight, Dark, Mid,
                      Text, BrightText, ButtonText, Base, Window, Shadow,
-                     Highlight, HighlightedText, Link, LinkVisited, AlternateBase,
-                     NColorRoles, NoRole = NColorRoles
-                     // ## deprecated
-                     , Foreground = WindowText, Background = Window
+                     Highlight, HighlightedText,
+                     Link, LinkVisited, // ### Qt 5: remove
+                     AlternateBase,
+                     NoRole, // ### Qt 5: value should be 0 or -1
+                     ToolTipBase, ToolTipText,
+                     NColorRoles = ToolTipText + 1,
+                     Foreground = WindowText, Background = Window // ### Qt 5: remove
                    };
 
     inline ColorGroup currentColorGroup() const { return static_cast<ColorGroup>(current_group); }
@@ -79,6 +104,7 @@ public:
     inline void setColor(ColorGroup cg, ColorRole cr, const QColor &color);
     inline void setColor(ColorRole cr, const QColor &color);
     inline void setBrush(ColorRole cr, const QBrush &brush);
+    bool isBrushSet(ColorGroup cg, ColorRole cr) const;
     void setBrush(ColorGroup cg, ColorRole cr, const QBrush &brush);
     void setColorGroup(ColorGroup cr, const QBrush &windowText, const QBrush &button,
                        const QBrush &light, const QBrush &dark, const QBrush &mid,
@@ -88,7 +114,7 @@ public:
 
     inline const QColor &color(ColorRole cr) const { return color(Current, cr); }
     inline const QBrush &brush(ColorRole cr) const { return brush(Current, cr); }
-    inline const QBrush &foreground() const { return brush(Foreground); }
+    inline const QBrush &foreground() const { return brush(WindowText); }
     inline const QBrush &windowText() const { return brush(WindowText); }
     inline const QBrush &button() const { return brush(Button); }
     inline const QBrush &light() const { return brush(Light); }
@@ -97,7 +123,9 @@ public:
     inline const QBrush &text() const { return brush(Text); }
     inline const QBrush &base() const { return brush(Base); }
     inline const QBrush &alternateBase() const { return brush(AlternateBase); }
-    inline const QBrush &background() const { return brush(Background); }
+    inline const QBrush &toolTipBase() const { return brush(ToolTipBase); }
+    inline const QBrush &toolTipText() const { return brush(ToolTipText); }
+    inline const QBrush &background() const { return brush(Window); }
     inline const QBrush &window() const { return brush(Window); }
     inline const QBrush &midlight() const { return brush(Midlight); }
     inline const QBrush &brightText() const { return brush(BrightText); }
@@ -126,6 +154,7 @@ public:
     bool isCopyOf(const QPalette &p) const;
 
     int serialNumber() const;
+    qint64 cacheKey() const;
 
     QPalette resolve(const QPalette &) const;
     inline uint resolve() const { return resolve_mask; }
@@ -140,6 +169,15 @@ private:
                        const QBrush &button_text, const QBrush &shadow,
                        const QBrush &highlight, const QBrush &highlighted_text,
                        const QBrush &link, const QBrush &link_visited);
+    void setColorGroup(ColorGroup cr, const QBrush &windowText, const QBrush &button,
+                       const QBrush &light, const QBrush &dark, const QBrush &mid,
+                       const QBrush &text, const QBrush &bright_text,
+                       const QBrush &base, const QBrush &alternate_base,
+                       const QBrush &window, const QBrush &midlight,
+                       const QBrush &button_text, const QBrush &shadow,
+                       const QBrush &highlight, const QBrush &highlighted_text,
+                       const QBrush &link, const QBrush &link_visited,
+                       const QBrush &toolTipBase, const QBrush &toolTipText);
 #ifdef QT3_SUPPORT
     friend class QColorGroup;
     void setColorGroup(ColorGroup, const QColorGroup &);
@@ -181,14 +219,14 @@ public:
     inline bool operator!=(const QColorGroup &other) const { return !(operator==(other)); }
     operator QVariant() const;
 
-    inline QT3_SUPPORT const QColor &foreground() const { return color(Foreground); }
+    inline QT3_SUPPORT const QColor &foreground() const { return color(WindowText); }
     inline QT3_SUPPORT const QColor &button() const { return color(Button); }
     inline QT3_SUPPORT const QColor &light() const { return color(Light); }
     inline QT3_SUPPORT const QColor &dark() const { return color(Dark); }
     inline QT3_SUPPORT const QColor &mid() const { return color(Mid); }
     inline QT3_SUPPORT const QColor &text() const { return color(Text); }
     inline QT3_SUPPORT const QColor &base() const { return color(Base); }
-    inline QT3_SUPPORT const QColor &background() const { return color(Background); }
+    inline QT3_SUPPORT const QColor &background() const { return color(Window); }
     inline QT3_SUPPORT const QColor &midlight() const { return color(Midlight); }
     inline QT3_SUPPORT const QColor &brightText() const { return color(BrightText); }
     inline QT3_SUPPORT const QColor &buttonText() const { return color(ButtonText); }
@@ -219,6 +257,8 @@ Q_GUI_EXPORT QDataStream &operator<<(QDataStream &ds, const QPalette &p);
 Q_GUI_EXPORT QDataStream &operator>>(QDataStream &ds, QPalette &p);
 #endif // QT_NO_DATASTREAM
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QPALETTE_H
