@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +49,8 @@
 #include <private/qobject_p.h>
 #include "qsound_p.h"
 
+QT_BEGIN_NAMESPACE
+
 static QList<QAuServer*> *servers=0;
 
 QAuServer::QAuServer(QObject* parent)
@@ -92,17 +114,12 @@ public:
     applications: asynchronously playing a sound file. This is most
     easily accomplished using the static play() function:
 
-    \code
-        QSound::play("mysounds/bells.wav");
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.kernel.qsound.cpp 0
 
     Alternatively, create a QSound object from the sound file first
     and then call the play() slot:
 
-    \code
-        QSound bells("mysounds/bells.wav");
-        bells.play();
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.kernel.qsound.cpp 1
 
     Once created a QSound object can be queried for its fileName() and
     total number of loops() (i.e. the number of times the sound will
@@ -132,10 +149,10 @@ public:
     supports WAVE and AU files.
     \row
     \o Mac OS X
-    \o \l{http://quicktime.apple.com/}{QuickTime} is used. All QuickTime
-    formats are supported by Qt/Mac.
+    \o NSSound is used. All formats that NSSound supports, including QuickTime formats, 
+    are supported by Qt/Mac.
     \row
-    \o Qtopia Core
+    \o Qt for Embedded Linux
     \o A built-in mixing sound server is used, accessing \c /dev/dsp
     directly. Only the WAVE format is supported.
     \endtable
@@ -366,5 +383,6 @@ QAuBucket::~QAuBucket()
     Use the isAvailable() function instead.
 */
 
+QT_END_NAMESPACE
 
 #endif // QT_NO_SOUND
