@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,10 +50,14 @@
 #include "qapplication.h"
 #include "qsound_p.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_NAS
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include <audio/audiolib.h>
 #include <audio/soundlib.h>
+QT_END_INCLUDE_NAMESPACE
 
 static AuServer *nas=0;
 
@@ -253,7 +257,9 @@ void QAuServerNAS::dataReceived()
     AuHandleEvents(nas);
 }
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "qsound_x11.moc"
+QT_END_INCLUDE_NAMESPACE
 
 #endif
 
@@ -287,4 +293,6 @@ QAuServer* qt_new_audio_server()
     return new QAuServerNull(qApp);
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SOUND
