@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,14 +44,15 @@
 #include "qwidget.h"
 #include "qt_x11_p.h"
 
-
 /*
   Internal Qt functions to create X windows.  We have put them in
   separate functions to allow the programmer to reimplement them by
   custom versions.
 */
 
-Window qt_XCreateWindow(const QWidget*, Display *display, Window parent,
+QT_BEGIN_NAMESPACE
+
+Window qt_XCreateWindow(const QWidget *, Display *display, Window parent,
                          int x, int y, uint w, uint h,
                          int borderwidth, int depth,
                          uint windowclass, Visual *visual,
@@ -62,7 +63,7 @@ Window qt_XCreateWindow(const QWidget*, Display *display, Window parent,
 }
 
 
-Window qt_XCreateSimpleWindow(const QWidget*, Display *display, Window parent,
+Window qt_XCreateSimpleWindow(const QWidget *, Display *display, Window parent,
                                int x, int y, uint w, uint h, int borderwidth,
                                ulong border, ulong background)
 {
@@ -71,7 +72,10 @@ Window qt_XCreateSimpleWindow(const QWidget*, Display *display, Window parent,
 }
 
 
-void qt_XDestroyWindow(const QWidget*, Display *display, Window window)
+void qt_XDestroyWindow(const QWidget *, Display *display, Window window)
 {
-    XDestroyWindow(display, window);
+    if (window)
+        XDestroyWindow(display, window);
 }
+
+QT_END_NAMESPACE
