@@ -93,7 +93,6 @@ static QT_FT_Long QT_FT_MulDiv(QT_FT_Long  a, QT_FT_Long  b, QT_FT_Long  c)
 #define QT_FT_MEM_ZERO(x, len) memset(x, 0, len);
 #include <stdio.h>
 
-
 #define MAX(x, y) (x > y ? x : y)
 #define MIN(x, y) (x < y ? x : y)
 #define MAX_SPANS 256
@@ -376,11 +375,15 @@ typedef struct  TBand_
 
 typedef struct TRaster_Instance_  TRaster_Instance;
 
-#define precision_bits 12
+#define precision_bits 8
 #define precision_step (1 << (precision_bits - 1))
 #define precision (1 << precision_bits)
 #define precision_shift (precision_bits - Pixel_Bits)
 
+/* only used for debugging */
+#define FIXED_TO_FLOAT(x) ((x)/(float) precision)
+#define INT_TO_FIXED(x) ((x) << precision_bits)
+
 /* NOTE: These operations are only valid on 2's complement processors */
 
 #define FLOOR( x )    ( (x) & -precision )
@@ -388,6 +391,8 @@ typedef struct TRaster_Instance_  TRaster_Instance;
 #define TRUNC( x )    ( (signed long)(x) >> precision_bits )
 #define FRAC( x )     ( (x) & ( precision - 1 ) )
 #define SCALED( x )   ( ( (x) << precision_shift ) )
+#define ROUND( x )    ( TRUNC((x) + precision_step) )
+#define HALF          ( (Int)(0.5 * precision) )
 
 /* Note that I have moved the location of some fields in the */
 /* structure to ensure that the most used variables are used */
@@ -481,12 +486,12 @@ New_Profile( TRaster_Instance*  raster, TStates  aState )
     {
     case Ascending_State:
         ras.cProfile->flow = Flow_Up;
-        QT_FT_TRACE6( "\nNew ascending profile = %lx\n", (long)ras.cProfile );
+        QT_FT_TRACE6( "\nNew ascending profile = %p\n", ras.cProfile );
         break;
 
     case Descending_State:
         ras.cProfile->flow = Flow_Down;
-        QT_FT_TRACE6( "\nNew descending profile = %lx\n", (long)ras.cProfile );
+        QT_FT_TRACE6( "\nNew descending profile = %p\n", ras.cProfile );
         break;
 
     default:
@@ -537,8 +542,8 @@ End_Profile( TRaster_Instance*  raster )
 
     if ( h > 0 )
     {
-        QT_FT_TRACE6( "Ending profile %lx, start = %ld, height = %ld top=%p\n\n",
-                   (long)ras.cProfile, ras.cProfile->start, h,ras.top );
+        QT_FT_TRACE6( "Ending profile %p, start = %ld, height = %ld top=%p\n\n",
+                   ras.cProfile, ras.cProfile->start, h,ras.top );
 
         ras.cProfile->height = (long)h;
         ras.cProfile         = (PProfile)ras.top;
@@ -830,7 +835,8 @@ Line_Up( TRaster_Instance*  raster, Long  x1,
         if (FRAC(y2) == 0 && ras.cProfile->flow == Flow_Up)
             --e2;
     }
-    QT_FT_TRACE6("Line_Up y1=%f, y2=%f, e1=%d, e2=%d f1=%d\n", y1/64.,  y2/64., e1, e2, f1);
+
+    QT_FT_TRACE6("Line_Up y1=%f, y2=%f, e1=%d, e2=%d f1=%d\n", FIXED_TO_FLOAT(y1),  FIXED_TO_FLOAT(y2), e1, e2, f1);
 
     if ( f1 > 0 )
     {
@@ -871,13 +877,13 @@ Line_Up( TRaster_Instance*  raster, Long  x1,
     top = ras.top;
 
     QT_FT_TRACE6("line_up (%f/%f)->(%f/%f), flow=%s\n",
-              x1/64., y1/64., x2/64., y2/64.,
-              ras.cProfile->flow == Flow_Up ? "Flow_Up" : "Flow_Down");
+                 FIXED_TO_FLOAT(x1), FIXED_TO_FLOAT(y1), FIXED_TO_FLOAT(x2), FIXED_TO_FLOAT(y2),
+		 ras.cProfile->flow == Flow_Up ? "Flow_Up" : "Flow_Down");
 
     while ( size > 0 )
     {
         *top++ = x1;
-        QT_FT_TRACE6("    x=%f y=%d\n", x1/64., e2+1-size);
+        QT_FT_TRACE6("    x=%f y=%d\n", FIXED_TO_FLOAT(x1), e2+1-size);
 
         x1 += Ix;
         Ax += Rx;
@@ -1011,7 +1017,7 @@ Bezier_Up( TRaster_Instance*  raster, Int        degree,
         ras.fresh = FALSE;
     }
 
-    QT_FT_TRACE6("bezier_up: y1=%f, y2=%f, e1=%f, e2=%f\n", y1/64.,  y2/64., e/64., e2/64.);
+    QT_FT_TRACE6("bezier_up: y1=%f, y2=%f, e1=%f, e2=%f\n", FIXED_TO_FLOAT(y1),  FIXED_TO_FLOAT(y2), FIXED_TO_FLOAT(e), FIXED_TO_FLOAT(e2));
 
     if ( e2 < e )
         goto Fin;
@@ -1025,7 +1031,7 @@ Bezier_Up( TRaster_Instance*  raster, Int        degree,
 
     QT_FT_TRACE6("       Flow = %s start=%ld, e=%f, e2=%f y1=%f, y2=%f\n",
               ras.cProfile->flow == Flow_Up ? "Flow_Up" : "Flow_Down",
-              ras.cProfile->start, e/64., e2/64.,y1/64.,y2/64.);
+                 ras.cProfile->start, FIXED_TO_FLOAT(e), FIXED_TO_FLOAT(e2),FIXED_TO_FLOAT(y1),FIXED_TO_FLOAT(y2));
 
     start_arc = arc;
 
@@ -1047,7 +1053,7 @@ Bezier_Up( TRaster_Instance*  raster, Int        degree,
                                                   e - y1, y2 - y1 );
                 QT_FT_TRACE6("  x=%f y=%f\n",
                           (arc[degree].x + FMulDiv( arc[0].x-arc[degree].x,
-                                                    e - y1, y2 - y1 ))/64., e/64.);
+                                                    e - y1, y2 - FIXED_TO_FLOAT(y1) )), FIXED_TO_FLOAT(e));
                 arc -= degree;
                 e   += precision;
             }
@@ -1057,7 +1063,7 @@ Bezier_Up( TRaster_Instance*  raster, Int        degree,
             if ( y2 == e )
             {
                 *top++     = arc[0].x;
-                QT_FT_TRACE6("  x=%f y=%f\n", arc[0].x/64., e/64.);
+                QT_FT_TRACE6("  x=%f y=%f\n", FIXED_TO_FLOAT(arc[0].x), FIXED_TO_FLOAT(e));
 
                 e += precision;
             }
@@ -1146,7 +1152,7 @@ Line_To( TRaster_Instance*  raster, Long  x,
          Long  y )
 {
     /* First, detect a change of direction */
-    QT_FT_TRACE6( "Line_To (%f/%f)->(%f/%f)\n", ras.lastX/64., ras.lastY/64., x/64., y/64. );
+    QT_FT_TRACE6( "Line_To (%f/%f)->(%f/%f)\n", FIXED_TO_FLOAT(ras.lastX), FIXED_TO_FLOAT(ras.lastY), FIXED_TO_FLOAT(x), FIXED_TO_FLOAT(y) );
 
     switch ( ras.state )
     {
@@ -1245,7 +1251,7 @@ Conic_To( TRaster_Instance*  raster, Long  cx,
     Long     y1, y2, y3, x3, ymin, ymax;
     TStates  state_bez;
 
-    QT_FT_TRACE6( "Conic_To (%f/%f)-(%f/%f)\n", cx/64., cy/64., x/64., y/64. );
+    QT_FT_TRACE6( "Conic_To (%f/%f)-(%f/%f)\n", FIXED_TO_FLOAT(cx), FIXED_TO_FLOAT(cy), FIXED_TO_FLOAT(x), FIXED_TO_FLOAT(y) );
 
     ras.arc      = ras.arcs;
     ras.arc[2].x = ras.lastX;
@@ -1363,7 +1369,7 @@ Cubic_To( TRaster_Instance*  raster, Long  cx1,
     TStates  state_bez;
 
     QT_FT_TRACE6( "Cubic_To (%f/%f)-(%f/%f)-(%f/%f)-(%f/%f)\n",
-               ras.lastX/64., ras.lastY/64., cx1/64., cy1/64., cx2/64., cy2/64., x/64., y/64. );
+                  FIXED_TO_FLOAT(ras.lastX), FIXED_TO_FLOAT(ras.lastY), FIXED_TO_FLOAT(cx1), FIXED_TO_FLOAT(cy1), FIXED_TO_FLOAT(cx2), FIXED_TO_FLOAT(cy2), FIXED_TO_FLOAT(x), FIXED_TO_FLOAT(y) );
 
     ras.arc      = ras.arcs;
     ras.arc[3].x = ras.lastX;
@@ -1486,8 +1492,8 @@ Fail:
 /*    SUCCESS on success, FAILURE on error.                              */
 /*                                                                       */
 static Bool
-Decompose_Curve( TRaster_Instance*  raster, UShort  first,
-                 UShort  last,
+Decompose_Curve( TRaster_Instance*  raster, UInt first,
+                 UInt last,
                  int     flipped )
 {
     QT_FT_Vector   v_last;
@@ -1728,8 +1734,8 @@ Convert_Glyph( TRaster_Instance*  raster, int  flipped )
         ras.state    = Unknown_State;
         ras.gProfile = NULL;
 
-        if ( Decompose_Curve( raster, (unsigned short)start,
-                              (unsigned short)ras.outline.contours[i],
+        if ( Decompose_Curve( raster, start,
+                              ras.outline.contours[i],
                               flipped ) )
             return FAILURE;
 
@@ -1836,7 +1842,6 @@ DelOld( PProfileList  list,
     /* the list.                                                     */
 }
 
-
 /*************************************************************************/
 /*                                                                       */
 /*  Sort                                                                 */
@@ -1849,9 +1854,11 @@ static void
 Sort( PProfileList  list )
 {
     PProfile  *old, current, next;
-
+    PProfile q, p, e, tail;
+    int insize, nmerges, psize, qsize, i;
 
     /* First, set the new X coordinate of each profile */
+    int lsize = 0;
     current = *list;
     while ( current )
     {
@@ -1859,38 +1866,91 @@ Sort( PProfileList  list )
         current->offset += current->flow;
         current->height--;
         current = current->link;
+	++lsize;
     }
 
-    /* Then sort them */
-    old     = list;
-    current = *old;
+    if ( !*list )
+	return;
 
-    if ( !current )
-        return;
+    /* If we have 20 or less elements in the list, use
+       Bubblesort, otherwise use Mergesort */
+    if (lsize < 20) {
+	old     = list;
+	current = *old;
 
-    next = current->link;
+	next = current->link;
 
-    while ( next )
-    {
-        if ( current->X <= next->X )
-        {
-            old     = &current->link;
-            current = *old;
+	while ( next )
+	{
+	    if ( current->X <= next->X )
+	    {
+		old     = &current->link;
+		current = *old;
 
-            if ( !current )
-                return;
-        }
-        else
-        {
-            *old          = next;
-            current->link = next->link;
-            next->link    = current;
+		if ( !current )
+		    return;
+	    }
+	    else
+	    {
+		*old = next;
+		current->link = next->link;
+		next->link    = current;
 
-            old     = list;
-            current = *old;
-        }
+		old     = list;
+		current = *old;
+	    }
 
-        next = current->link;
+	    next = current->link;
+	}
+    } else {
+
+	/* The below algoritm is a modified version of a Mergesort
+	   algorithm described by Simon Tatham, which can be found
+	   here: http://www.chiark.greenend.org.uk/~sgtatham/algorithms */
+
+	insize = 1;
+	while (1) {
+	    p = *list;
+	    *list = 0;
+	    tail = 0;
+	    nmerges = 0;
+
+	    while (p) {
+		++nmerges;
+		q = p;
+		psize = 0;
+		for (i = 0; i < insize; ++i) {
+		    ++psize;
+		    q = q->link;
+		    if (!q)
+			break;
+		}
+		qsize = insize;
+
+		while (psize > 0 || (qsize > 0 && q)) {
+		    if (psize == 0) {
+			e = q; q = q->link; --qsize;
+		    } else if ((qsize == 0 || !q) || (p->X <= q->X)) {
+			e = p; p = p->link; --psize;
+		    } else {
+			e = q; q = q->link; --qsize;
+		    }
+
+		    if (tail)
+			tail->link = e;
+		    else
+			*list = e;
+		    tail = e;
+		}
+		p = q;
+	    }
+	    if (tail)
+		tail->link = 0;
+
+	    if (nmerges <= 1)
+		break;
+	    insize *= 2;
+	}
     }
 }
 
@@ -2012,7 +2072,7 @@ Draw_Sweep( TRaster_Instance*  raster )
             Q = profile_list;
             printf("y=%d ::", y);
             while (Q) {
-                printf(" %.2f", Q->X / 64.);
+                printf(" %.2f", FIXED_TO_FLOAT(Q->X));
                 Q = Q->link;
             }
             printf("\n");
@@ -2312,5 +2372,4 @@ const QT_FT_Raster_Funcs  qt_ft_standard_raster =
     (QT_FT_Raster_Done_Func)    qt_ft_black_done
 };
 
-
 /* END */
