@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -62,6 +62,10 @@
 #include <QtCore/qlibrary.h>
 #include <cups/cups.h>
 
+QT_BEGIN_NAMESPACE
+
+Q_DECLARE_TYPEINFO(cups_option_t, Q_MOVABLE_TYPE | Q_PRIMITIVE_TYPE);
+
 class QCUPSSupport
 {
 public:
@@ -78,7 +82,7 @@ public:
     const ppd_file_t* currentPPD() const;
     const ppd_option_t* ppdOption(const char *key) const;
 
-//     const cups_option_t* printerOption(const QString &key) const;
+    const cups_option_t* printerOption(const QString &key) const;
     const ppd_option_t* pageSizes() const;
 
     int markOption(const char* name, const char* value);
@@ -93,6 +97,10 @@ public:
 
     QString unicodeString(const char *s);
 
+    QPair<int, QString> tempFd();
+    int printFile(const char * printerName, const char * filename, const char * title,
+                  int num_options, cups_option_t * options);
+
 private:
     void collectMarkedOptions(QStringList& list, const ppd_group_t* group = 0) const;
     void collectMarkedOptionsHelper(QStringList& list, const ppd_group_t* group) const;
@@ -107,5 +115,8 @@ private:
 #endif
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_CUPS
+
 #endif
