@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,7 +45,9 @@
 
 #include <private/qdrawhelper_mmx_p.h>
 
-const CompositionFunctionSolid qt_functionForModeSolid_MMX[] = {
+QT_BEGIN_NAMESPACE
+
+CompositionFunctionSolid qt_functionForModeSolid_MMX[numCompositionFunctions] = {
     comp_func_solid_SourceOver<QMMXIntrinsics>,
     comp_func_solid_DestinationOver<QMMXIntrinsics>,
     comp_func_solid_Clear<QMMXIntrinsics>,
@@ -57,22 +59,10 @@ const CompositionFunctionSolid qt_functionForModeSolid_MMX[] = {
     comp_func_solid_DestinationOut<QMMXIntrinsics>,
     comp_func_solid_SourceAtop<QMMXIntrinsics>,
     comp_func_solid_DestinationAtop<QMMXIntrinsics>,
-    comp_func_solid_XOR<QMMXIntrinsics>,
-    comp_func_solid_Plus,
-    comp_func_solid_Multiply,
-    comp_func_solid_Screen,
-    comp_func_solid_Overlay,
-    comp_func_solid_Darken,
-    comp_func_solid_Lighten,
-    comp_func_solid_ColorDodge,
-    comp_func_solid_ColorBurn,
-    comp_func_solid_HardLight,
-    comp_func_solid_SoftLight,
-    comp_func_solid_Difference,
-    comp_func_solid_Exclusion
+    comp_func_solid_XOR<QMMXIntrinsics>
 };
 
-const CompositionFunction qt_functionForMode_MMX[] = {
+CompositionFunction qt_functionForMode_MMX[numCompositionFunctions] = {
     comp_func_SourceOver<QMMXIntrinsics>,
     comp_func_DestinationOver<QMMXIntrinsics>,
     comp_func_Clear<QMMXIntrinsics>,
@@ -84,19 +74,7 @@ const CompositionFunction qt_functionForMode_MMX[] = {
     comp_func_DestinationOut<QMMXIntrinsics>,
     comp_func_SourceAtop<QMMXIntrinsics>,
     comp_func_DestinationAtop<QMMXIntrinsics>,
-    comp_func_XOR<QMMXIntrinsics>,
-    comp_func_Plus,
-    comp_func_Multiply,
-    comp_func_Screen,
-    comp_func_Overlay,
-    comp_func_Darken,
-    comp_func_Lighten,
-    comp_func_ColorDodge,
-    comp_func_ColorBurn,
-    comp_func_HardLight,
-    comp_func_SoftLight,
-    comp_func_Difference,
-    comp_func_Exclusion
+    comp_func_XOR<QMMXIntrinsics>
 };
 
 void qt_blend_color_argb_mmx(int count, const QSpan *spans, void *userData)
@@ -106,3 +84,5 @@ void qt_blend_color_argb_mmx(int count, const QSpan *spans, void *userData)
 }
 
 #endif // QT_HAVE_MMX
+
+QT_END_NAMESPACE
