@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +48,8 @@
 #include "qpainter.h"
 #include "qpalette.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \fn void qDrawShadeLine(QPainter *painter, int x1, int y1, int x2, int y2,
                      const QPalette &palette, bool sunken,
@@ -56,10 +78,7 @@
     Alternatively you can use a QFrame widget and apply the
     QFrame::setFrameStyle() function to display a shaded line:
 
-    \code
-        QFrame frame:
-        frame.setFrameStyle(QFrame::HLine | QFrame::Sunken);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 0
 
     \sa qDrawShadeRect(), qDrawShadePanel(), QStyle
 */
@@ -69,7 +88,7 @@ void qDrawShadeLine(QPainter *p, int x1, int y1, int x2, int y2,
                      int lineWidth, int midLineWidth)
 {
     if (!(p && lineWidth >= 0 && midLineWidth >= 0))        {
-        qWarning("qDrawShadeLine invalid parameters.");
+        qWarning("qDrawShadeLine: Invalid parameters");
         return;
     }
     int tlw = lineWidth*2 + midLineWidth;        // total line width
@@ -172,10 +191,7 @@ void qDrawShadeLine(QPainter *p, int x1, int y1, int x2, int y2,
     Alternatively you can use a QFrame widget and apply the
     QFrame::setFrameStyle() function to display a shaded rectangle:
 
-    \code
-        QFrame frame:
-        frame.setFrameStyle(QFrame::Box | QFrame::Raised);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 1
 
     \sa qDrawShadeLine(), qDrawShadePanel(), qDrawPlainRect(), QStyle
 */
@@ -188,7 +204,7 @@ void qDrawShadeRect(QPainter *p, int x, int y, int w, int h,
     if (w == 0 || h == 0)
         return;
     if (! (w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0)) {
-        qWarning("qDrawShadeRect(): Invalid parameters");
+        qWarning("qDrawShadeRect: Invalid parameters");
         return;
     }
     QPen oldPen = p->pen();
@@ -276,10 +292,7 @@ void qDrawShadeRect(QPainter *p, int x, int y, int w, int h,
     Alternatively you can use a QFrame widget and apply the
     QFrame::setFrameStyle() function to display a shaded panel:
 
-    \code
-        QFrame frame:
-        frame.setFrameStyle( QFrame::Panel | QFrame::Sunken);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 2
 
     \sa qDrawWinPanel(), qDrawShadeLine(), qDrawShadeRect(), QStyle
 */
@@ -291,7 +304,7 @@ void qDrawShadePanel(QPainter *p, int x, int y, int w, int h,
     if (w == 0 || h == 0)
         return;
     if (!(w > 0 && h > 0 && lineWidth >= 0)) {
-        qWarning("qDrawShadePanel() Invalid parameters.");
+        qWarning("qDrawShadePanel: Invalid parameters");
     }
     QColor shade = pal.dark().color();
     QColor light = pal.light().color();
@@ -450,10 +463,7 @@ void qDrawWinButton(QPainter *p, int x, int y, int w, int h,
     Alternatively you can use a QFrame widget and apply the
     QFrame::setFrameStyle() function to display a shaded panel:
 
-    \code
-        QFrame frame:
-        frame.setFrameStyle(QFrame::WinPanel | QFrame::Raised);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 3
 
     \sa qDrawShadePanel(), qDrawWinButton(), QStyle
 */
@@ -478,8 +488,8 @@ void qDrawWinPanel(QPainter *p, int x, int y, int w, int h,
     \relates QPainter
 
     Draws the plain rectangle beginning at (\a x, \a y) with the given
-    \a width and \a height, using the provided \a painter, \a lineColor
-    and \a lineWidth . The rectangle's interior is filled with the \a
+    \a width and \a height, using the specified \a painter, \a lineColor
+    and \a lineWidth. The rectangle's interior is filled with the \a
     fill brush unless \a fill is 0.
 
     \warning This function does not look at QWidget::style() or
@@ -489,10 +499,7 @@ void qDrawWinPanel(QPainter *p, int x, int y, int w, int h,
     Alternatively you can use a QFrame widget and apply the
     QFrame::setFrameStyle() function to display a plain rectangle:
 
-    \code
-        QFrame frame:
-        frame.setFrameStyle(QFrame::Box | QFrame::Plain);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 4
 
     \sa qDrawShadeRect(), QStyle
 */
@@ -503,7 +510,7 @@ void qDrawPlainRect(QPainter *p, int x, int y, int w, int h, const QColor &c,
     if (w == 0 || h == 0)
         return;
     if (!(w > 0 && h > 0 && lineWidth >= 0)) {
-        qWarning("qDrawPlainRect() Invalid parameters.");
+        qWarning("qDrawPlainRect: Invalid parameters");
     }
     QPen   oldPen   = p->pen();
     QBrush oldBrush = p->brush();
@@ -525,10 +532,35 @@ void qDrawPlainRect(QPainter *p, int x, int y, int w, int h, const QColor &c,
  *****************************************************************************/
 
 /*!
-    \fn void qDrawShadeLine(QPainter *p, const QPoint &p1, const QPoint &p2,
-    const QPalette &pal, bool sunken, int lineWidth, int midLineWidth)
-
+    \fn void qDrawShadeLine(QPainter *painter, const QPoint &p1, const QPoint &p2,
+             const QPalette &palette, bool sunken, int lineWidth, int midLineWidth)
+    \relates QPainter
     \overload
+
+    Draws a horizontal or vertical shaded line between \a p1 and \a p2
+    using the given \a painter.  Note that nothing is drawn if the line
+    between the points would be neither horizontal nor vertical.
+
+    The provided \a palette specifies the shading colors (\l
+    {QPalette::light()}{light}, \l {QPalette::dark()}{dark} and \l
+    {QPalette::mid()}{middle} colors).  The given \a lineWidth
+    specifies the line width for each of the lines; it is not the
+    total line width. The given \a midLineWidth specifies the width of
+    a middle line drawn in the QPalette::mid() color.
+
+    The line appears sunken if \a sunken is true, otherwise raised.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style().  Use the drawing functions in QStyle to
+    make widgets that follow the current GUI style.
+
+
+    Alternatively you can use a QFrame widget and apply the
+    QFrame::setFrameStyle() function to display a shaded line:
+
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 5
+
+    \sa qDrawShadeRect(), qDrawShadePanel(), QStyle
 */
 
 void qDrawShadeLine(QPainter *p, const QPoint &p1, const QPoint &p2,
@@ -540,10 +572,34 @@ void qDrawShadeLine(QPainter *p, const QPoint &p1, const QPoint &p2,
 }
 
 /*!
-    \fn void qDrawShadeRect(QPainter *p, const QRect &r, const QPalette &pal,
-    bool sunken, int lineWidth, int midLineWidth, const QBrush *fill)
-
+    \fn void qDrawShadeRect(QPainter *painter, const QRect &rect, const QPalette &palette,
+             bool sunken, int lineWidth, int midLineWidth, const QBrush *fill)
+    \relates QPainter
     \overload
+
+    Draws the shaded rectangle specified by \a rect using the given \a painter.
+
+    The provide \a palette specifies the shading colors (\l
+    {QPalette::light()}{light}, \l {QPalette::dark()}{dark} and \l
+    {QPalette::mid()}{middle} colors.  The given \a lineWidth
+    specifies the line width for each of the lines; it is not the
+    total line width.  The \a midLineWidth specifies the width of a
+    middle line drawn in the QPalette::mid() color.  The rectangle's
+    interior is filled with the \a fill brush unless \a fill is 0.
+
+    The rectangle appears sunken if \a sunken is true, otherwise
+    raised.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style(). Use the drawing functions in QStyle to make
+    widgets that follow the current GUI style.
+
+    Alternatively you can use a QFrame widget and apply the
+    QFrame::setFrameStyle() function to display a shaded rectangle:
+
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 6
+
+    \sa qDrawShadeLine(), qDrawShadePanel(), qDrawPlainRect(), QStyle
 */
 
 void qDrawShadeRect(QPainter *p, const QRect &r,
@@ -556,8 +612,31 @@ void qDrawShadeRect(QPainter *p, const QRect &r,
 }
 
 /*!
-    \fn void qDrawShadePanel(QPainter *p, const QRect &r, const QPalette &pal, bool sunken, int lineWidth, const QBrush *fill)
+    \fn void qDrawShadePanel(QPainter *painter, const QRect &rect, const QPalette &palette,
+             bool sunken, int lineWidth, const QBrush *fill)
+    \relates QPainter
     \overload
+
+    Draws the shaded panel at the rectangle specified by \a rect using the
+    given \a painter and the given \a lineWidth.
+
+    The given \a palette specifies the shading colors (\l
+    {QPalette::light()}{light}, \l {QPalette::dark()}{dark} and \l
+    {QPalette::mid()}{middle} colors).  The panel's interior is filled
+    with the \a fill brush unless \a fill is 0.
+
+    The panel appears sunken if \a sunken is true, otherwise raised.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style(). Use the drawing functions in QStyle to make
+    widgets that follow the current GUI style.
+
+    Alternatively you can use a QFrame widget and apply the
+    QFrame::setFrameStyle() function to display a shaded panel:
+
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 7
+
+    \sa qDrawWinPanel(), qDrawShadeLine(), qDrawShadeRect(), QStyle
 */
 
 void qDrawShadePanel(QPainter *p, const QRect &r,
@@ -569,8 +648,26 @@ void qDrawShadePanel(QPainter *p, const QRect &r,
 }
 
 /*!
-    \fn void qDrawWinButton(QPainter *p, const QRect &r, const QPalette &pal, bool sunken, const QBrush *fill)
+    \fn void qDrawWinButton(QPainter *painter, const QRect &rect, const QPalette &palette,
+             bool sunken, const QBrush *fill)
+    \relates QPainter
     \overload
+
+    Draws the Windows-style button at the rectangle specified by \a rect using
+    the given \a painter with a line width of 2 pixels. The button's interior
+    is filled with the \a{fill} brush unless \a fill is 0.
+
+    The given \a palette specifies the shading colors (\l
+    {QPalette::light()}{light}, \l {QPalette::dark()}{dark} and \l
+    {QPalette::mid()}{middle} colors).
+
+    The button appears sunken if \a sunken is true, otherwise raised.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style()-> Use the drawing functions in QStyle to make
+    widgets that follow the current GUI style.
+
+    \sa qDrawWinPanel(), QStyle
 */
 
 void qDrawWinButton(QPainter *p, const QRect &r,
@@ -580,8 +677,27 @@ void qDrawWinButton(QPainter *p, const QRect &r,
 }
 
 /*!
-    \fn void qDrawWinPanel(QPainter *p, const QRect &r, const QPalette &pal, bool sunken, const QBrush *fill)
+    \fn void qDrawWinPanel(QPainter *painter, const QRect &rect, const QPalette &palette,
+             bool sunken, const QBrush *fill)
     \overload
+
+    Draws the Windows-style panel at the rectangle specified by \a rect using
+    the given \a painter with a line width of 2 pixels. The button's interior
+    is filled with the \a fill brush unless \a fill is 0.
+
+    The given \a palette specifies the shading colors.  The panel
+    appears sunken if \a sunken is true, otherwise raised.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style(). Use the drawing functions in QStyle to make
+    widgets that follow the current GUI style.
+
+    Alternatively you can use a QFrame widget and apply the
+    QFrame::setFrameStyle() function to display a shaded panel:
+
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 8
+
+    \sa qDrawShadePanel(), qDrawWinButton(), QStyle
 */
 
 void qDrawWinPanel(QPainter *p, const QRect &r,
@@ -591,8 +707,24 @@ void qDrawWinPanel(QPainter *p, const QRect &r,
 }
 
 /*!
-    \fn void qDrawPlainRect(QPainter *p, const QRect &r, const QColor &c, int lineWidth, const QBrush *fill)
+    \fn void qDrawPlainRect(QPainter *painter, const QRect &rect, const QColor &lineColor, int lineWidth, const QBrush *fill)
+    \relates QPainter
     \overload
+
+    Draws the plain rectangle specified by \a rect using the given \a painter,
+    \a lineColor and \a lineWidth. The rectangle's interior is filled with the
+    \a fill brush unless \a fill is 0.
+
+    \warning This function does not look at QWidget::style() or
+    QApplication::style(). Use the drawing functions in QStyle to make
+    widgets that follow the current GUI style.
+
+    Alternatively you can use a QFrame widget and apply the
+    QFrame::setFrameStyle() function to display a plain rectangle:
+
+    \snippet doc/src/snippets/code/src.gui.painting.qdrawutil.cpp 9
+
+    \sa qDrawShadeRect(), QStyle
 */
 
 void qDrawPlainRect(QPainter *p, const QRect &r, const QColor &c,
@@ -676,7 +808,7 @@ static void qDrawMotifArrow(QPainter *p, Qt::ArrowType type, bool down,
     QPolygon bTop;                                // top shadow.
     QPolygon bBot;                                // bottom shadow.
     QPolygon bLeft;                                // left shadow.
-    QMatrix        matrix;                                // xform matrix
+    QTransform matrix;                            // xform matrix
     bool vertical = type == Qt::UpArrow || type == Qt::DownArrow;
     bool horizontal = !vertical;
     int         dim = w < h ? w : h;
@@ -763,13 +895,13 @@ static void qDrawMotifArrow(QPainter *p, Qt::ArrowType type, bool down,
 
     QPen     savePen   = p->pen();                // save current pen
     QBrush   saveBrush = p->brush();                // save current brush
-    QMatrix wxm = p->matrix();
+    QTransform wxm = p->transform();
     QPen     pen(Qt::NoPen);
     const QBrush &brush = pal.brush(QPalette::Button);
 
     p->setPen(pen);
     p->setBrush(brush);
-    p->setMatrix(matrix, true);                // set transformation matrix
+    p->setTransform(matrix, true);                // set transformation matrix
     p->drawPolygon(bFill);                        // fill arrow
     p->setBrush(Qt::NoBrush);                        // don't fill
 
@@ -780,7 +912,7 @@ static void qDrawMotifArrow(QPainter *p, Qt::ArrowType type, bool down,
     p->setPen(CBOT);
     p->drawLines(bBot);
 
-    p->setMatrix(wxm);
+    p->setTransform(wxm);
     p->setBrush(saveBrush);                        // restore brush
     p->setPen(savePen);                        // restore pen
 
@@ -839,7 +971,7 @@ void qDrawArrow(QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
             break;
 #endif
         default:
-            qWarning("qDrawArrow: Requested GUI style not supported");
+            qWarning("qDrawArrow: Requested unsupported GUI style");
     }
 }
 
@@ -879,7 +1011,7 @@ void qDrawItem(QPainter *p, Qt::GUIStyle gs,
 #ifndef QT_NO_IMAGE_HEURISTIC_MASK
             } else {                                // color pixmap, no mask
                 QString k;
-                k.sprintf("$qt-drawitem-%x", pm.serialNumber());
+                k.sprintf("$qt-drawitem-%llx", pm.cacheKey());
                 if (!QPixmapCache::find(k, pm)) {
                     pm = pm.createHeuristicMask();
                     pm.setMask((QBitmap&)pm);
@@ -905,5 +1037,7 @@ void qDrawItem(QPainter *p, Qt::GUIStyle gs,
         p->drawText(x, y, w, h, flags, text.left(len));
     }
 }
+
 #endif
 
+QT_END_NAMESPACE
