@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +49,8 @@
 #include <private/qt_x11_p.h>
 #include "qx11info_x11.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QPaintDevice
     \brief The QPaintDevice class is the base class of objects that
@@ -131,6 +153,8 @@ QPaintDevice::QPaintDevice()
     painters = 0;
 }
 
+extern void qt_painter_removePaintDevice(QPaintDevice *); //qpainter.cpp
+
 /*!
     Destroys the paint device and frees window system resources.
 */
@@ -140,6 +164,7 @@ QPaintDevice::~QPaintDevice()
     if (paintingActive())
         qWarning("QPaintDevice: Cannot destroy paint device that is being "
                   "painted");
+    qt_painter_removePaintDevice(this);
 }
 
 /*!
@@ -227,10 +252,10 @@ int QPaintDevice::metric(PaintDeviceMetric) const
     \oldcode
         Display *display = widget->x11Display();
     \newcode
-        Display *display = widget->x11Info().display();
+        Display *display = QX11Info::display();
     \endcode
 
-    \sa QWidget::x11Info(), QPixmap::x11Info()
+    \sa QWidget::x11Info(), QX11Info::display()
 */
 Display *QPaintDevice::x11Display() const
 {
@@ -601,8 +626,7 @@ int QPaintDevice::x11AppDpiY(int screen)
 
     Returns the horizontal resolution of the device in dots per inch,
     which is used when computing font sizes. For X11, this is usually
-    the same as could be computed from widthMM(), but it varies on
-    Windows.
+    the same as could be computed from widthMM().
 
     Note that if the logicalDpiX() doesn't equal the physicalDpiX(),
     the corresponding QPaintEngine must handle the resolution mapping.
@@ -615,8 +639,7 @@ int QPaintDevice::x11AppDpiY(int screen)
 
     Returns the vertical resolution of the device in dots per inch,
     which is used when computing font sizes. For X11, this is usually
-    the same as could be computed from heightMM(), but it varies on
-    Windows.
+    the same as could be computed from heightMM().
 
     Note that if the logicalDpiY() doesn't equal the physicalDpiY(),
     the corresponding QPaintEngine must handle the resolution mapping.
@@ -628,6 +651,9 @@ int QPaintDevice::x11AppDpiY(int screen)
     \fn int QPaintDevice::physicalDpiX() const
 
     Returns the horizontal resolution of the device in dots per inch.
+    For example, when printing, this resolution refers to the physical
+    printer's resolution. The logical DPI on the other hand, refers to
+    the resolution used by the actual paint engine.
 
     Note that if the physicalDpiX() doesn't equal the logicalDpiX(),
     the corresponding QPaintEngine must handle the resolution mapping.
@@ -639,6 +665,9 @@ int QPaintDevice::x11AppDpiY(int screen)
     \fn int QPaintDevice::physicalDpiY() const
 
     Returns the horizontal resolution of the device in dots per inch.
+    For example, when printing, this resolution refers to the physical
+    printer's resolution. The logical DPI on the other hand, refers to
+    the resolution used by the actual paint engine.
 
     Note that if the physicalDpiY() doesn't equal the logicalDpiY(),
     the corresponding QPaintEngine must handle the resolution mapping.
@@ -646,3 +675,4 @@ int QPaintDevice::x11AppDpiY(int screen)
     \sa  physicalDpiX(),  logicalDpiY()
 */
 
+QT_END_NAMESPACE
