@@ -1,26 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
-
 #include "qpaintengine.h"
 #include "qpaintengine_p.h"
 #include "qpainter_p.h"
@@ -28,46 +47,96 @@
 #include "qbitmap.h"
 #include "qapplication.h"
 #include <qdebug.h>
+#include <qmath.h>
 #include <private/qtextengine_p.h>
 #include <qvarlengtharray.h>
+#include <private/qfontengine_p.h>
 
-#include <math.h>
+
+QT_BEGIN_NAMESPACE
 
 /*!
     \class QTextItem
     \internal
+
+    \brief The QTextItem class provides all the information required to draw
+    text in a custom paint engine.
+
+    When you reimplement your own paint engine, you must reimplement
+    QPaintEngine::drawTextItem(), a function that takes a QTextItem as
+    one of its arguments.
 */
 
+/*!
+  \enum QTextItem::RenderFlag
+
+  \value  RightToLeft         Indicates that the text should be rendered from right to left.
+  \value  Overline               Indicates that QPainter has painted a line above the text.
+  \value  Underline             Indicates that QPainter has painted a line under the text.
+  \value  StrikeOut            Indicates that QPainter has painted a line through the text.
+*/
+
+
+/*!
+    \fn qreal QTextItem::descent() const
+
+    Corresponds to the \l{QFontMetrics::descent()}{descent} of the piece of text that is drawn.
+*/
 qreal QTextItem::descent() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
     return ti->descent.toReal();
 }
 
+/*!
+    \fn qreal QTextItem::ascent() const
+
+    Corresponds to the \l{QFontMetrics::ascent()}{ascent} of the piece of text that is drawn.
+*/
 qreal QTextItem::ascent() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
     return ti->ascent.toReal();
 }
 
+/*!
+    \fn qreal QTextItem::width() const
+
+    Specifies the total width of the text to be drawn.
+*/
 qreal QTextItem::width() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
     return ti->width.toReal();
 }
 
+/*!
+    \fn QTextItem::RenderFlags QTextItem::renderFlags() const
+
+    Returns the render flags used.
+*/
 QTextItem::RenderFlags QTextItem::renderFlags() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
     return ti->flags;
 }
 
+/*!
+    \fn QString QTextItem::text() const
+
+    Returns the text that should be drawn.
+*/
 QString QTextItem::text() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
     return QString(ti->chars, ti->num_chars);
 }
 
+/*!
+    \fn QFont QTextItem::font() const
+
+    Returns the font that should be used to draw the text.
+*/
 QFont QTextItem::font() const
 {
     const QTextItemInt *ti = static_cast<const QTextItemInt *>(this);
@@ -75,8 +144,6 @@ QFont QTextItem::font() const
 }
 
 
-#include <private/qfontengine_p.h>
-
 /*!
   \class QPaintEngine
   \ingroup multimedia
@@ -118,34 +185,37 @@ QFont QTextItem::font() const
   means and pass on an alpha blended QImage to the engine with the
   emulated results. Some features cannot be emulated: AlphaBlend and PorterDuff.
 
-  \value PrimitiveTransform The engine has support for transforming
-                            drawing primitives.
-  \value PatternTransform   The engine has support for transforming brush
-                            patterns.
-  \value PixmapTransform    The engine can transform pixmaps, including
-                            rotation and shearing.
-  \value PatternBrush       The engine is capable of rendering brushes with
-                            the brush patterns specified in Qt::BrushStyle.
-  \value LinearGradientFill The engine supports linear gradient fills.
-  \value RadialGradientFill The engine supports radial gradient fills.
-  \value ConicalGradientFill The engine supports conical gradient fills.
   \value AlphaBlend         The engine can alpha blend primitives.
-  \value PorterDuff         The engine supports Porter-Duff operations
-  \value PainterPaths       The engine has path support.
   \value Antialiasing       The engine can use antialising to improve the appearance
                             of rendered primitives.
+  \value BlendModes         The engine supports blending modes.
   \value BrushStroke        The engine supports drawing strokes that
                             contain brushes as fills, not just solid
                             colors (e.g. a dashed gradient line of
                             width 2).
+  \value ConicalGradientFill The engine supports conical gradient fills.
   \value ConstantOpacity    The engine supports the feature provided by
                             QPainter::setOpacity().
+  \value LinearGradientFill The engine supports linear gradient fills.
   \value MaskedBrush        The engine is capable of rendering brushes that has a
                             texture with an alpha channel or a mask.
-
+  \value PainterPaths       The engine has path support.
   \value PaintOutsidePaintEvent The engine is capable of painting outside of
                                 paint events.
-  \value AllFeatures
+  \value PatternBrush       The engine is capable of rendering brushes with
+                            the brush patterns specified in Qt::BrushStyle.
+  \value PatternTransform   The engine has support for transforming brush
+                            patterns.
+  \value PerspectiveTransform The engine has support for performing perspective
+                            transformations on primitives.
+  \value PixmapTransform    The engine can transform pixmaps, including
+                            rotation and shearing.
+  \value PorterDuff         The engine supports Porter-Duff operations
+  \value PrimitiveTransform The engine has support for transforming
+                            drawing primitives.
+  \value RadialGradientFill The engine supports radial gradient fills.
+  \value AllFeatures        All of the above features. This enum value is usually
+                            used as a bit mask.
 */
 
 /*!
@@ -223,7 +293,8 @@ QFont QTextItem::font() const
     from QPainters state to the native state is required.
 */
 
-static int qt_polygon_recursion;
+
+static QPaintEngine *qt_polygon_recursion = 0;
 struct QT_Point {
     int x;
     int y;
@@ -243,9 +314,9 @@ struct QT_Point {
 */
 void QPaintEngine::drawPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode)
 {
-    Q_ASSERT_X(!qt_polygon_recursion, "QPaintEngine::drawPolygon",
+    Q_ASSERT_X(qt_polygon_recursion != this, "QPaintEngine::drawPolygon",
                "At least one drawPolygon function must be implemented");
-    qt_polygon_recursion = 1;
+    qt_polygon_recursion = this;
     Q_ASSERT(sizeof(QT_Point) == sizeof(QPoint));
     QVarLengthArray<QT_Point> p(pointCount);
     for (int i = 0; i < pointCount; ++i) {
@@ -272,9 +343,9 @@ struct QT_PointF {
 */
 void QPaintEngine::drawPolygon(const QPoint *points, int pointCount, PolygonDrawMode mode)
 {
-    Q_ASSERT_X(!qt_polygon_recursion, "QPaintEngine::drawPolygon",
+    Q_ASSERT_X(qt_polygon_recursion != this, "QPaintEngine::drawPolygon",
                "At least one drawPolygon function must be implemented");
-    qt_polygon_recursion = 1;
+    qt_polygon_recursion = this;
     Q_ASSERT(sizeof(QT_PointF) == sizeof(QPointF));
     QVarLengthArray<QT_PointF> p(pointCount);
     for (int i=0; i<pointCount; ++i) {
@@ -293,12 +364,14 @@ void QPaintEngine::drawPolygon(const QPoint *points, int pointCount, PolygonDraw
     \value MacPrinter
     \value CoreGraphics Mac OS X's Quartz2D (CoreGraphics)
     \value QuickDraw Mac OS X's QuickDraw
-    \value QWindowSystem Qtopia Core
+    \value QWindowSystem Qt for Embedded Linux
     \value PostScript
     \value OpenGL
     \value Picture QPicture format
     \value SVG Scalable Vector Graphics XML format
     \value Raster
+    \value Direct3D Windows only, Direct3D based engine
+    \value Pdf Portable Document Format
     \value User First user type ID
     \value MaxUser Last user type ID
 */
@@ -371,7 +444,7 @@ void QPaintEngine::drawPoints(const QPoint *points, int pointCount)
             fp[i].y = points[i].y();
             ++i;
         }
-        drawPoints((QPointF *)fp, i);
+        drawPoints((QPointF *)(void *)fp, i);
         points += i;
         pointCount -= i;
     }
@@ -477,16 +550,12 @@ void QPaintEngine::drawTiledPixmap(const QRectF &rect, const QPixmap &pixmap, co
         while (tw*th < 32678 && th < rect.height()/2)
             th *= 2;
         QPixmap tile;
-        if (pixmap.hasAlphaChannel()) {
-            // ####################
-            QImage image(tw, th, QImage::Format_ARGB32_Premultiplied);
-            image.fill(qRgba(127, 0, 0, 127));
-            tile = QPixmap::fromImage(image);
+        if (pixmap.depth() == 1) {
+            tile = QBitmap(tw, th);
         } else {
-            if (pixmap.depth() == 1)
-                tile = QBitmap(tw, th);
-            else
-                tile = QPixmap(tw, th);
+            tile = QPixmap(tw, th);
+            if (pixmap.hasAlphaChannel())
+                tile.fill(Qt::transparent);
         }
         qt_fill_tile(&tile, pixmap);
         qt_draw_tile(this, rect.x(), rect.y(), rect.width(), rect.height(), tile, p.x(), p.y());
@@ -510,10 +579,8 @@ void QPaintEngine::drawImage(const QRectF &r, const QImage &image, const QRectF
     QRectF baseSize(0, 0, image.width(), image.height());
     QImage im = image;
     if (baseSize != sr)
-        im = im.copy((int)floor(sr.x()), (int)floor(sr.y()),
-                     (int)ceil(sr.width()), (int)ceil(sr.height()));
-    if (im.depth() == 1)
-        im = im.convertToFormat(QImage::Format_RGB32);
+        im = im.copy(qFloor(sr.x()), qFloor(sr.y()),
+                     qCeil(sr.width()), qCeil(sr.height()));
     QPixmap pm = QPixmap::fromImage(im, flags);
     drawPixmap(r, pm, QRectF(QPointF(0, 0), pm.size()));
 }
@@ -636,27 +703,11 @@ void QPaintEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
     const QTextItemInt &ti = static_cast<const QTextItemInt &>(textItem);
 
     QPainterPath path;
-    QPainterPath effects;
+#ifndef Q_WS_MAC
     path.setFillRule(Qt::WindingFill);
-    effects.setFillRule(Qt::WindingFill);
+#endif
     if (ti.num_glyphs)
         ti.fontEngine->addOutlineToPath(p.x(), p.y(), ti.glyphs, ti.num_glyphs, &path, ti.flags);
-    if (ti.flags) {
-        const QFontEngine *fe = ti.fontEngine;
-        const qreal lw = fe->lineThickness().toReal();
-        if (ti.flags & QTextItem::Underline) {
-            qreal pos = fe->underlinePosition().toReal();
-            effects.addRect(p.x(), p.y() + pos, ti.width.toReal(), lw);
-        }
-        if (ti.flags & QTextItem::Overline) {
-            qreal pos = fe->ascent().toReal() + 1;
-            effects.addRect(p.x(), p.y() - pos, ti.width.toReal(), lw);
-        }
-        if (ti.flags & QTextItem::StrikeOut) {
-            qreal pos = fe->ascent().toReal() / 3;
-            effects.addRect(p.x(), p.y() - pos, ti.width.toReal(), lw);
-        }
-    }
     if (!path.isEmpty()) {
         painter()->save();
         painter()->setRenderHint(QPainter::Antialiasing,
@@ -665,7 +716,6 @@ void QPaintEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
         painter()->setBrush(state->pen().brush());
         painter()->setPen(Qt::NoPen);
         painter()->drawPath(path);
-        painter()->drawPath(effects);
         painter()->restore();
     }
 }
@@ -679,6 +729,13 @@ void QPaintEngine::drawLines(const QLineF *lines, int lineCount)
 {
     for (int i=0; i<lineCount; ++i) {
         QPointF pts[2] = { lines[i].p1(), lines[i].p2() };
+
+        if (pts[0] == pts[1]) {
+            if (state->pen().capStyle() != Qt::FlatCap)
+                drawPoints(pts, 1);
+            continue;
+        }
+
         drawPolygon(pts, 2, PolylineMode);
     }
 }
@@ -712,7 +769,7 @@ void QPaintEngine::drawLines(const QLine *lines, int lineCount)
             fl[i].p2.y = lines[i].y2();
             ++i;
         }
-        drawLines((QLineF *)fl, i);
+        drawLines((QLineF *)(void *)fl, i);
         lines += i;
         lineCount -= i;
     }
@@ -745,7 +802,7 @@ void QPaintEngine::drawRects(const QRect *rects, int rectCount)
             fr[i].h = rects[i].height();
             ++i;
         }
-        drawRects((QRectF *)fr, i);
+        drawRects((QRectF *)(void *)fr, i);
         rects += i;
         rectCount -= i;
     }
@@ -758,7 +815,9 @@ void QPaintEngine::drawRects(const QRect *rects, int rectCount)
 */
 void QPaintEngine::drawRects(const QRectF *rects, int rectCount)
 {
-    if (hasFeature(PainterPaths)) {
+    if (hasFeature(PainterPaths) &&
+        !state->penNeedsResolving() &&
+        !state->brushNeedsResolving()) {
         for (int i=0; i<rectCount; ++i) {
             QPainterPath path;
             path.addRect(rects[i]);
@@ -788,7 +847,7 @@ void QPaintEngine::setPaintDevice(QPaintDevice *device)
 }
 
 /*!
-    Returns the engine that this engine is painting on, if painting is
+    Returns the device that this engine is painting on, if painting is
     active; otherwise returns 0.
 */
 QPaintDevice *QPaintEngine::paintDevice() const
@@ -892,3 +951,33 @@ QRect QPaintEngine::systemRect() const
 {
     return d_func()->systemRect;
 }
+
+void QPaintEnginePrivate::drawBoxTextItem(const QPointF &p, const QTextItemInt &ti)
+{
+    if (!ti.num_glyphs)
+        return;
+
+    // any fixes here should probably also be done in QFontEngineBox::draw
+    const int size = qRound(ti.fontEngine->ascent());
+    QVarLengthArray<QFixedPoint> positions;
+    QVarLengthArray<glyph_t> glyphs;
+    QTransform matrix;
+    matrix.translate(p.x(), p.y() - size);
+    ti.fontEngine->getGlyphPositions(ti.glyphs, ti.num_glyphs, matrix, ti.flags, glyphs, positions);
+    if (glyphs.size() == 0)
+        return;
+
+    QSize s(size - 3, size - 3);
+
+    QPainter *painter = q_func()->state->painter();
+    painter->save();
+    painter->setBrush(Qt::NoBrush);
+    QPen pen = painter->pen();
+    pen.setWidthF(ti.fontEngine->lineThickness().toReal());
+    painter->setPen(pen);
+    for (int k = 0; k < positions.size(); k++)
+        painter->drawRect(QRectF(positions[k].toPointF(), s));
+    painter->restore();
+}
+
+QT_END_NAMESPACE
