@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,10 +60,13 @@
 #include "private/qdatabuffer_p.h"
 #include "private/qdrawhelper_p.h"
 #include "private/qpaintengine_p.h"
+#include "private/qrasterizer_p.h"
 #include "private/qstroker_p.h"
 
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 class QFTOutlineMapper;
 class QRasterPaintEnginePrivate;
 class QRasterBuffer;
@@ -66,7 +89,7 @@ public:
     bool end();
 
     void updateState(const QPaintEngineState &state);
-    void updateMatrix(const QMatrix &matrix);
+    void updateMatrix(const QTransform &matrix);
 
     void updateClipRegion(const QRegion &region, Qt::ClipOperation op);
     void updateClipPath(const QPainterPath &path, Qt::ClipOperation op);
@@ -75,11 +98,13 @@ public:
     void drawPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode);
     void drawPolygon(const QPoint *points, int pointCount, PolygonDrawMode mode);
     void fillPath(const QPainterPath &path, QSpanData *fillData);
+    void fillPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode);
 
     void drawEllipse(const QRectF &rect);
 
     void drawRects(const QRect  *rects, int rectCount);
     void drawRects(const QRectF *rects, int rectCount);
+    void fastFillRect(const QRect &rect, const QBrush &brush);
 
     void drawPixmap(const QRectF &r, const QPixmap &pm, const QRectF &sr);
     void drawImage(const QRectF &r, const QImage &pm, const QRectF &sr,
@@ -122,9 +147,9 @@ public:
 
     void disableClearType();
 #endif
-#ifdef Q_WS_QWS
     //QWS hack
     void alphaPenBlt(const void* src, int bpl, bool mono, int rx,int ry,int w,int h);
+#ifdef Q_WS_QWS
     void qwsFillRect(int x, int y, int w, int h);
 #endif
 
@@ -163,22 +188,32 @@ public:
 
     void drawBitmap(const QPointF &pos, const QPixmap &image, QSpanData *fill);
 
+    void rasterize(QT_FT_Outline *outline, ProcessSpans callback, QSpanData *spanData, QRasterBuffer *rasterBuffer);
     void rasterize(QT_FT_Outline *outline, ProcessSpans callback, void *userData, QRasterBuffer *rasterBuffer);
-#ifdef QT_EXPERIMENTAL_REGIONS
-    void setSimpleClip(const QRect &rect);
-#endif
-    void updateMatrixData(QSpanData *spanData, const QBrush &brush, const QMatrix &brushMatrix);
+    void setClipRect(const QRect &rect);
+    void setClipRegion(const QRegion &region);
+    void updateMatrixData(QSpanData *spanData, const QBrush &brush, const QTransform &brushMatrix);
 
-    QMatrix brushMatrix() const {
-        QMatrix m(matrix);
+    QTransform brushMatrix() const {
+        QTransform m(matrix);
         m.translate(brushOffset.x(), brushOffset.y());
         return m;
     }
 
+    bool isUnclipped_normalized(const QRect &rect) const;
+    bool isUnclipped(const QRect &rect, int penWidth) const;
+    bool isUnclipped(const QRectF &rect, int penWidth) const;
+    ProcessSpans getPenFunc(const QRect &rect, const QSpanData *data) const;
+    ProcessSpans getPenFunc(const QRectF &rect, const QSpanData *data) const;
+    ProcessSpans getBrushFunc(const QRect &rect, const QSpanData *data) const;
+    ProcessSpans getBrushFunc(const QRectF &rect, const QSpanData *data) const;
+
+    void initializeRasterizer(QSpanData *data);
+
     QPointF brushOffset;
     QBrush brush;
     QPen pen;
-    QMatrix matrix;
+    QTransform matrix;
     int opacity;
 
     QPaintDevice *device;
@@ -191,9 +226,8 @@ public:
 
     QPainterPath baseClip;
     QRect deviceRect;
-#ifdef QT_EXPERIMENTAL_REGIONS
     QRegion clipRegion;
-#endif
+    QRegion disabledClipRegion;
 
     QSpanData penData;
     QSpanData brushData;
@@ -224,7 +258,14 @@ public:
     uint mono_surface : 1;
     uint int_xform : 1;
     uint user_clip_enabled : 1;
+    uint fast_text : 1;
+    uint paint_unclipped : 1;
+    uint tx_noshear : 1;
+#ifdef Q_WS_WIN
+    uint isPlain45DegreeRotation : 1;
+#endif
 
+    QRasterizer rasterizer;
 };
 
 class QClipData {
@@ -244,9 +285,8 @@ public:
 
     void appendSpan(int x, int length, int y, int coverage);
     void appendSpans(const QSpan *s, int num);
-#ifdef QT_EXPERIMENTAL_REGIONS
-    void setSimpleClip(const QRect &rect);
-#endif
+    void setClipRect(const QRect &rect);
+    void setClipRegion(const QRegion &region);
     void fixup();
 };
 
@@ -265,9 +305,10 @@ inline void QClipData::appendSpan(int x, int length, int y, int coverage)
 
 inline void QClipData::appendSpans(const QSpan *s, int num)
 {
-    if (count + num >= allocated) {
-        while (allocated < count + num)
+    if (count + num > allocated) {
+        do {
             allocated *= 2;
+        } while (count + num > allocated);
         spans = (QSpan *)realloc(spans, allocated*sizeof(QSpan));
     }
     memcpy(spans+count, s, num*sizeof(QSpan));
@@ -312,6 +353,8 @@ public:
         : clip(0),
           m_hdc(0),
           m_bitmap(0),
+          m_null_bitmap(0),
+          m_text_buffer(false),
           m_width(0),
           m_height(0),
           m_buffer(0)
@@ -335,17 +378,16 @@ public:
     void init();
 
     void prepare(QImage *image);
-#ifdef Q_WS_QWS
-void prepare(QCustomRasterPaintDevice *device);
-#endif
-#if defined(Q_WS_QWS) || defined(Q_WS_MAC)
     void prepare(QPixmap *pix);
+#ifdef Q_WS_QWS
+    void prepare(QCustomRasterPaintDevice *device);
 #endif
     void prepare(int w, int h);
     void prepareBuffer(int w, int h);
 
 #ifdef Q_WS_WIN
     void setupHDC(bool clear_type);
+    inline void setTextBuffer(bool t);
 #endif
 
     void resetBuffer(int val=0);
@@ -362,13 +404,22 @@ void prepare(QCustomRasterPaintDevice *device);
     int width() const { return m_width; }
     int height() const { return m_height; }
     int bytesPerLine() const { return bytes_per_line; }
+    int bytesPerPixel() const { return bytes_per_pixel; }
 
     uchar *buffer() const { return m_buffer; }
 
+    QRect clipRect;
+    QRect disabledClipRect;
+    QRegion clipRegion;
+    QRegion disabledClipRegion;
     QClipData *clip;
     QClipData *disabled_clip;
     bool clipEnabled;
 
+    bool monoDestinationWithClut;
+    QRgb destColor0;
+    QRgb destColor1;
+
     QPainter::CompositionMode compositionMode;
     QImage::Format format;
     DrawHelper *drawHelper;
@@ -379,13 +430,19 @@ private:
 #if defined(Q_WS_WIN)
     HDC m_hdc;
     HBITMAP m_bitmap;
+    HBITMAP m_null_bitmap;
+    bool m_text_buffer;
 #endif
-
     int m_width;
     int m_height;
     int bytes_per_line;
+    int bytes_per_pixel;
     uchar *m_buffer;
-
 };
 
+#ifdef Q_WS_WIN
+inline void QRasterBuffer::setTextBuffer(bool t) { m_text_buffer = t; }
+#endif
+
+QT_END_NAMESPACE
 #endif // QPAINTENGINE_RASTER_P_H
