@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -59,6 +64,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QBrush;
@@ -84,7 +91,8 @@ public:
         Antialiasing = 0x01,
         TextAntialiasing = 0x02,
         SmoothPixmapTransform = 0x04,
-        HighQualityAntialiasing = 0x08
+        HighQualityAntialiasing = 0x08,
+        NonCosmeticDefaultPen = 0x10
     };
 
     Q_DECLARE_FLAGS(RenderHints, RenderHint)
@@ -193,7 +201,7 @@ public:
 
     void setWorldMatrix(const QMatrix &matrix, bool combine = false);
     const QMatrix &worldMatrix() const;
-    
+
     void setWorldTransform(const QTransform &matrix, bool combine = false);
     const QTransform &worldTransform() const;
 
@@ -267,6 +275,9 @@ public:
     void drawEllipse(const QRect &r);
     inline void drawEllipse(int x, int y, int w, int h);
 
+    inline void drawEllipse(const QPointF &center, qreal rx, qreal ry);
+    inline void drawEllipse(const QPoint &center, int rx, int ry);
+
     void drawPolyline(const QPointF *points, int pointCount);
     inline void drawPolyline(const QPolygonF &polyline);
     void drawPolyline(const QPoint *points, int pointCount);
@@ -294,6 +305,13 @@ public:
     inline void drawChord(int x, int y, int w, int h, int a, int alen);
     inline void drawChord(const QRect &, int a, int alen);
 
+    void drawRoundedRect(const QRectF &rect, qreal xRadius, qreal yRadius,
+                         Qt::SizeMode mode = Qt::AbsoluteSize);
+    inline void drawRoundedRect(int x, int y, int w, int h, qreal xRadius, qreal yRadius,
+                                Qt::SizeMode mode = Qt::AbsoluteSize);
+    inline void drawRoundedRect(const QRect &rect, qreal xRadius, qreal yRadius,
+                                Qt::SizeMode mode = Qt::AbsoluteSize);
+
     void drawRoundRect(const QRectF &r, int xround = 25, int yround = 25);
     inline void drawRoundRect(int x, int y, int w, int h, int = 25, int = 25);
     inline void drawRoundRect(const QRect &r, int xround = 25, int yround = 25);
@@ -343,6 +361,8 @@ public:
     inline void drawText(const QPoint &p, const QString &s);
     inline void drawText(int x, int y, const QString &s);
 
+    void drawText(const QPointF &p, const QString &str, int tf, int justificationPadding);
+
     void drawText(const QRectF &r, int flags, const QString &text, QRectF *br=0);
     void drawText(const QRect &r, int flags, const QString &text, QRect *br=0);
     inline void drawText(int x, int y, int w, int h, int flags, const QString &text, QRect *br=0);
@@ -472,6 +492,7 @@ private:
     friend class QWin32PaintEnginePrivate;
     friend class QRasterPaintEngine;
     friend class QAlphaPaintEngine;
+    friend class QPreviewPaintEngine;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QPainter::RenderHints)
@@ -618,11 +639,33 @@ inline void QPainter::drawRoundRect(const QRect &rect, int xRnd, int yRnd)
     drawRoundRect(QRectF(rect), xRnd, yRnd);
 }
 
+inline void QPainter::drawRoundedRect(int x, int y, int w, int h, qreal xRadius, qreal yRadius,
+                            Qt::SizeMode mode)
+{
+    drawRoundedRect(QRectF(x, y, w, h), xRadius, yRadius, mode);
+}
+
+inline void QPainter::drawRoundedRect(const QRect &rect, qreal xRadius, qreal yRadius,
+                            Qt::SizeMode mode)
+{
+    drawRoundedRect(QRectF(rect), xRadius, yRadius, mode);
+}
+
 inline void QPainter::drawEllipse(int x, int y, int w, int h)
 {
     drawEllipse(QRect(x, y, w, h));
 }
 
+inline void QPainter::drawEllipse(const QPointF &center, qreal rx, qreal ry)
+{
+    drawEllipse(QRectF(center.x() - rx, center.y() - ry, 2 * rx, 2 * ry));
+}
+
+inline void QPainter::drawEllipse(const QPoint &center, int rx, int ry)
+{
+    drawEllipse(QRect(center.x() - rx, center.y() - ry, 2 * rx, 2 * ry));
+}
+
 inline void QPainter::drawArc(const QRect &r, int a, int alen)
 {
     drawArc(QRectF(r), a, alen);
@@ -852,6 +895,8 @@ inline void QPainter::drawPicture(const QPoint &pt, const QPicture &p)
 }
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QPAINTER_H
