@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,6 +59,8 @@
 #include "private/qdatabuffer_p.h"
 #include "private/qnumeric_p.h"
 
+QT_BEGIN_NAMESPACE
+
 // #define QFIXED_IS_26_6
 
 #if defined QFIXED_IS_26_6
@@ -95,6 +117,8 @@ struct qfixed2d
 QPointF qt_curves_for_arc(const QRectF &rect, qreal startAngle, qreal sweepLength,
                           QPointF *controlPoints, int *point_count);
 
+qreal qt_t_for_arc_angle(qreal angle);
+
 typedef void (*qStrokerMoveToHook)(qfixed x, qfixed y, void *data);
 typedef void (*qStrokerLineToHook)(qfixed x, qfixed y, void *data);
 typedef void (*qStrokerCubicToHook)(qfixed c1x, qfixed c1y,
@@ -131,10 +155,10 @@ public:
     inline void lineTo(qfixed x, qfixed y);
     inline void cubicTo(qfixed x1, qfixed y1, qfixed x2, qfixed y2, qfixed ex, qfixed ey);
 
-    void strokePath(const QPainterPath &path, void *data, const QMatrix &matrix);
+    void strokePath(const QPainterPath &path, void *data, const QTransform &matrix);
     void strokePolygon(const QPointF *points, int pointCount, bool implicit_close,
-                       void *data, const QMatrix &matrix);
-    void strokeEllipse(const QRectF &ellipse, void *data, const QMatrix &matrix);
+                       void *data, const QTransform &matrix);
+    void strokeEllipse(const QRectF &ellipse, void *data, const QTransform &matrix);
 
     QRectF clipRect() const { return m_clip_rect; }
     void setClipRect(const QRectF &clip) { m_clip_rect = clip; }
@@ -166,7 +190,8 @@ public:
         SquareJoin,
         MiterJoin,
         RoundJoin,
-        RoundCap
+        RoundCap,
+        SvgMiterJoin
     };
 
     QStroker();
@@ -229,6 +254,9 @@ public:
     void setDashPattern(const QVector<qfixed> &dashPattern) { m_dashPattern = dashPattern; }
     QVector<qfixed> dashPattern() const { return m_dashPattern; }
 
+    void setDashOffset(qreal offset) { m_dashOffset = offset; }
+    qreal dashOffset() const { return m_dashOffset; }
+
     virtual void begin(void *data);
     virtual void end();
 
@@ -237,6 +265,7 @@ protected:
 
     QStroker *m_stroker;
     QVector<qfixed> m_dashPattern;
+    qreal m_dashOffset;
 };
 
 
@@ -312,8 +341,18 @@ inline void QStroker::emitCubicTo(qfixed c1x, qfixed c1y,
                                   qfixed c2x, qfixed c2y,
                                   qfixed ex, qfixed ey)
 {
-    m_back2X = c2x;
-    m_back2Y = c2y;
+    if (c2x == ex && c2y == ey) {
+        if (c1x == ex && c1y == ey) {
+            m_back2X = m_back1X;
+            m_back2Y = m_back1Y;
+        } else {
+            m_back2X = c1x;
+            m_back2Y = c1y;
+        }
+    } else {
+        m_back2X = c2x;
+        m_back2Y = c2y;
+    }
     m_back1X = ex;
     m_back1Y = ey;
     QStrokerOps::emitCubicTo(c1x, c1y, c2x, c2y, ex, ey);
@@ -336,4 +375,6 @@ inline void QDashStroker::end()
     m_stroker->end();
 }
 
+QT_END_NAMESPACE
+
 #endif // QSTROKER_P_H
