@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,8 +47,10 @@
 #include <QList>
 #include <QDebug>
 
+#include <qmath.h>
 #include <limits.h>
-#include <private/qmath_p.h>
+
+QT_BEGIN_NAMESPACE
 
 //#define DEBUG
 #ifdef DEBUG
@@ -917,6 +924,8 @@ void QTessellatorPrivate::removeEdges()
         if (v->flags & LineBeforeEnds) {
             QDEBUG() << "    removing edge" << vertices.prevPos(v);
             int pos = scanline.findEdge(vertices.prevPos(v));
+            if (pos == -1)
+                continue;
             scanline.edges[pos]->mark = true;
             if (pos > 0)
                 scanline.edges[pos - 1]->intersect_right = true;
@@ -927,6 +936,8 @@ void QTessellatorPrivate::removeEdges()
         if (v->flags & LineAfterEnds) {
             QDEBUG() << "    removing edge" << vertices.position(v);
             int pos = scanline.findEdge(vertices.position(v));
+            if (pos == -1)
+                continue;
             scanline.edges[pos]->mark = true;
             if (pos > 0)
                 scanline.edges[pos - 1]->intersect_right = true;
@@ -1029,12 +1040,14 @@ bool QTessellatorPrivate::edgeInChain(Intersection i, int edge)
 
         Intersection i2 = i;
         i2.edge = l.next;
-        IntersectionLink l2 = intersections.value(i2);
 
+#ifndef QT_NO_DEBUG
+        IntersectionLink l2 = intersections.value(i2);
         Q_ASSERT(l2.next != -1);
         Q_ASSERT(l2.prev != -1);
         Q_ASSERT(l.next == i2.edge);
         Q_ASSERT(l2.prev == i.edge);
+#endif
         i = i2;
     }
     return false;
@@ -1424,7 +1437,7 @@ void QTessellator::tessellateRect(const QPointF &a_, const QPointF &b_, qreal wi
         QPointF perp(pb.y() - pa.y(), pa.x() - pb.x());
         qreal length = qSqrt(perp.x() * perp.x() + perp.y() * perp.y());
 
-        if (qFuzzyCompare(length, static_cast<qreal>(0)))
+        if (qFuzzyCompare(length + 1, static_cast<qreal>(1)))
             return;
 
         // need the half of the width
@@ -1483,3 +1496,5 @@ void QTessellator::tessellateRect(const QPointF &a_, const QPointF &b_, qreal wi
         }
     }
 }
+
+QT_END_NAMESPACE
