@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -68,6 +68,7 @@
 #include "qdebug.h"
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
 
 /*!
     \class QCDEStyle
@@ -301,4 +302,6 @@ QIcon QCDEStyle::standardIconImplementation(StandardPixmap standardIcon, const Q
     return QMotifStyle::standardIconImplementation(standardIcon, opt, widget);
 }
 
+QT_END_NAMESPACE
+
 #endif
