@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -62,10 +62,21 @@
 #ifndef QT_NO_STYLE_WINDOWSVISTA
 #include "qwindowsvistastyle.h"
 #endif
+#ifndef QT_NO_STYLE_WINDOWSCE
+#include "qwindowscestyle.h"
+#endif
+#ifndef QT_NO_STYLE_WINDOWSMOBILE
+#include "qwindowsmobilestyle.h"
+#endif
+
+QT_BEGIN_NAMESPACE
 
 #if !defined(QT_NO_STYLE_MAC) && defined(Q_WS_MAC)
+QT_BEGIN_INCLUDE_NAMESPACE
 #  include <private/qt_mac_p.h>
 #  include "qmacstyle_mac.h"
+QT_END_INCLUDE_NAMESPACE
+
 QString qt_mac_get_style_name()
 {
     QString ret;
@@ -84,9 +95,9 @@ QString qt_mac_get_style_name()
 }
 #endif
 
-#ifndef QT_NO_LIBRARY
+#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
-    (QStyleFactoryInterface_iid, QCoreApplication::libraryPaths(), QLatin1String("/styles"), Qt::CaseInsensitive))
+    (QStyleFactoryInterface_iid, QLatin1String("/styles"), Qt::CaseInsensitive))
 #endif
 
 /*!
@@ -127,6 +138,16 @@ QStyle *QStyleFactory::create(const QString& key)
         ret = new QWindowsStyle;
     else
 #endif
+#ifndef QT_NO_STYLE_WINDOWSCE
+    if (style == QLatin1String("windowsce"))
+        ret = new QWindowsCEStyle;
+    else
+#endif
+#ifndef QT_NO_STYLE_WINDOWSMOBILE
+    if (style == QLatin1String("windowsmobile"))
+        ret = new QWindowsMobileStyle;
+    else
+#endif
 #ifndef QT_NO_STYLE_WINDOWSXP
     if (style == QLatin1String("windowsxp"))
         ret = new QWindowsXPStyle;
@@ -167,7 +188,7 @@ QStyle *QStyleFactory::create(const QString& key)
     } else
 #endif
     { } // Keep these here - they make the #ifdefery above work
-#ifndef QT_NO_LIBRARY
+#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     if(!ret) {
         if (QStyleFactoryInterface *factory = qobject_cast<QStyleFactoryInterface*>(loader()->instance(style)))
             ret = factory->create(style);
@@ -186,7 +207,7 @@ QStyle *QStyleFactory::create(const QString& key)
 */
 QStringList QStyleFactory::keys()
 {
-#ifndef QT_NO_LIBRARY
+#if !defined(QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     QStringList list = loader()->keys();
 #else
     QStringList list;
@@ -195,6 +216,14 @@ QStringList QStyleFactory::keys()
     if (!list.contains(QLatin1String("Windows")))
         list << QLatin1String("Windows");
 #endif
+#ifndef QT_NO_STYLE_WINDOWSCE
+    if (!list.contains(QLatin1String("WindowsCE")))
+        list << QLatin1String("WindowsCE");
+#endif
+#ifndef QT_NO_STYLE_WINDOWSMOBILE
+    if (!list.contains(QLatin1String("WindowsMobile")))
+        list << QLatin1String("WindowsMobile");
+#endif
 #ifndef QT_NO_STYLE_WINDOWSXP
     if (!list.contains(QLatin1String("WindowsXP")) &&
         (QSysInfo::WindowsVersion >= QSysInfo::WV_XP && QSysInfo::WindowsVersion < QSysInfo::WV_NT_based))
@@ -231,3 +260,5 @@ QStringList QStyleFactory::keys()
 #endif
     return list;
 }
+
+QT_END_NAMESPACE
