@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QStringList;
@@ -146,6 +148,8 @@ public:
     static bool removeApplicationFont(int id);
     static bool removeAllApplicationFonts();
 
+    static bool supportsThreadedFontRendering();
+
 private:
     static void createDatabase();
     static void parseFontName(const QString &name, QString &foundry, QString &family);
@@ -166,6 +170,8 @@ private:
     QFontDatabasePrivate *d;
 };
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QFONTDATABASE_H
