@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,11 @@
 
 #ifdef Q_WS_X11
 #include "qx11info_x11.h"
+#endif
+
+QT_BEGIN_NAMESPACE
+
+#ifdef Q_WS_X11
 extern const QX11Info *qt_x11Info(const QPaintDevice *pd);
 #endif
 
@@ -48,6 +73,8 @@ extern int qt_defaultDpi();
 
 /*!
     \class QFontMetrics
+    \reentrant
+
     \brief The QFontMetrics class provides font metrics information.
 
     \ingroup multimedia
@@ -105,12 +132,7 @@ extern int qt_defaultDpi();
     and size(), to return the size of that rectangle.
 
     Example:
-    \code
-    QFont font("times", 24);
-    QFontMetrics fm(font);
-    int pixelsWide = fm.width("What's the width of this text?");
-    int pixelsHigh = fm.height();
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qfontmetrics.cpp 0
 
     \sa QFont, QFontInfo, QFontDatabase, QFontComboBox, {Character Map Example}
 */
@@ -287,8 +309,7 @@ int QFontMetrics::ascent() const
     Returns the descent of the font.
 
     The descent is the distance from the base line to the lowest point
-    characters extend to. (Note that this is different from X, which
-    adds 1 pixel.) In practice, some font designers break this rule,
+    characters extend to. In practice, some font designers break this rule,
     e.g. to accommodate an unusual character in an exotic language, so
     it is possible (though rare) that this value will be too small.
 
@@ -396,6 +417,8 @@ int QFontMetrics::xHeight() const
 {
     QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
     Q_ASSERT(engine != 0);
+    if (d->capital == QFont::SmallCaps)
+        return qRound(d->smallCapsFontPrivate()->engineForScript(QUnicodeTables::Common)->ascent());
     return qRound(engine->xHeight());
 }
 
@@ -440,11 +463,17 @@ bool QFontMetrics::inFont(QChar ch) const
 int QFontMetrics::leftBearing(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
         return 0;
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -468,11 +497,17 @@ int QFontMetrics::leftBearing(QChar ch) const
 int QFontMetrics::rightBearing(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
         return 0;
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -533,20 +568,27 @@ int QFontMetrics::width(const QString &text, int len) const
 */
 int QFontMetrics::width(QChar ch) const
 {
-    if (QUnicodeTables::category(ch) == QChar::Mark_NonSpacing)
+    if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing)
         return 0;
 
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[8];
     int nglyphs = 7;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
     return qRound(glyphs[0].advance.x);
 }
 
-/*!
+/*! \obsolete
+
     Returns the width of the character at position \a pos in the
     string \a text.
 
@@ -562,7 +604,7 @@ int QFontMetrics::charWidth(const QString &text, int pos) const
     if (pos < 0 || pos > (int)text.length())
         return 0;
 
-    const QChar &ch = text.unicode()[pos];
+    QChar ch = text.unicode()[pos];
     const int script = QUnicodeTables::script(ch);
     int width;
 
@@ -575,12 +617,18 @@ int QFontMetrics::charWidth(const QString &text, int pos) const
         layout.ignoreBidi = true;
         layout.itemize();
         width = qRound(layout.width(pos-from, 1));
-    } else if (QUnicodeTables::category(ch) == QChar::Mark_NonSpacing) {
+    } else if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing) {
         width = 0;
     } else {
-        QFontEngine *engine = d->engineForScript(script);
+        QFontEngine *engine;
+        if (d->capital == QFont::SmallCaps && ch.isLower())
+            engine = d->smallCapsFontPrivate()->engineForScript(script);
+        else
+            engine = d->engineForScript(script);
         Q_ASSERT(engine != 0);
 
+        d->alterCharForCapitalization(ch);
+
         QGlyphLayout glyphs[8];
         int nglyphs = 7;
         engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -607,7 +655,7 @@ int QFontMetrics::charWidth(const QString &text, int pos) const
     The height of the bounding rectangle is at least as large as the
     value returned by height().
 
-    \sa width(), height(), QPainter::boundingRect()
+    \sa width(), height(), QPainter::boundingRect(), tightBoundingRect()
 */
 QRect QFontMetrics::boundingRect(const QString &text) const
 {
@@ -623,7 +671,7 @@ QRect QFontMetrics::boundingRect(const QString &text) const
 
 /*!
     Returns the rectangle that is covered by ink if character \a ch
-    where to be drawn at the origin of the coordinate system.
+    were to be drawn at the origin of the coordinate system.
 
     Note that the bounding rectangle may extend to the left of (0, 0),
     e.g. for italicized fonts, and that the text output may cover \e
@@ -641,9 +689,15 @@ QRect QFontMetrics::boundingRect(const QString &text) const
 QRect QFontMetrics::boundingRect(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -674,7 +728,7 @@ QRect QFontMetrics::boundingRect(QChar ch) const
     \o Qt::TextSingleLine ignores newline characters in the text.
     \o Qt::TextExpandTabs expands tabs (see below)
     \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
-    \o Qt::TextWordBreak breaks the text to fit the rectangle.
+    \o Qt::TextWordWrap breaks the text to fit the rectangle.
     \endlist
 
     Qt::Horizontal alignment defaults to Qt::AlignLeft and vertical
@@ -720,11 +774,7 @@ QRect QFontMetrics::boundingRect(const QRect &rect, int flags, const QString &te
     qt_format_text(QFont(d), rr, flags | Qt::TextDontPrint, text, &rb, tabStops, tabArray,
                    tabArrayLen, 0);
 
-    int xmin = floor(rb.x());
-    int xmax = ceil(rb.x() + rb.width());
-    int ymin = floor(rb.y());
-    int ymax = ceil(rb.y() + rb.height());
-    return QRect(xmin, ymin, xmax - xmin, ymax - ymin);
+    return rb.toAlignedRect();
 }
 
 /*!
@@ -755,6 +805,41 @@ QSize QFontMetrics::size(int flags, const QString &text, int tabStops, int *tabA
     return boundingRect(QRect(0,0,0,0), flags, text, tabStops, tabArray).size();
 }
 
+/*!
+  \since 4.3
+
+    Returns a tight bounding rectangle around the characters in the
+    string specified by \a text. The bounding rectangle always covers
+    at least the set of pixels the text would cover if drawn at (0,
+    0).
+
+    Note that the bounding rectangle may extend to the left of (0, 0),
+    e.g. for italicized fonts, and that the width of the returned
+    rectangle might be different than what the width() method returns.
+
+    If you want to know the advance width of the string (to layout
+    a set of strings next to each other), use width() instead.
+
+    Newline characters are processed as normal characters, \e not as
+    linebreaks.
+
+    \warning Calling this method is very slow on Windows.
+
+    \sa width(), height(), boundingRect()
+*/
+QRect QFontMetrics::tightBoundingRect(const QString &text) const
+{
+    if (text.length() == 0)
+        return QRect();
+
+    QTextEngine layout(text, d);
+    layout.ignoreBidi = true;
+    layout.itemize();
+    glyph_metrics_t gm = layout.tightBoundingBox(0, text.length());
+    return QRect(qRound(gm.x), qRound(gm.y), qRound(gm.width), qRound(gm.height));
+}
+
+
 /*!
     \since 4.2
 
@@ -835,6 +920,8 @@ int QFontMetrics::lineWidth() const
 
 /*!
     \class QFontMetricsF qfontmetrics.h
+    \reentrant
+
     \brief The QFontMetricsF class provides font metrics information.
 
     \ingroup multimedia
@@ -873,12 +960,7 @@ int QFontMetrics::lineWidth() const
     and size(), to return the size of that rectangle.
 
     Example:
-    \code
-    QFont font("times", 24);
-    QFontMetricsF fm(font);
-    qreal pixelsWide = fm.width("What's the width of this text?");
-    qreal pixelsHigh = fm.height();
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qfontmetrics.cpp 1
 
     \sa QFont QFontInfo QFontDatabase
 */
@@ -1164,6 +1246,8 @@ qreal QFontMetricsF::xHeight() const
 {
     QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
     Q_ASSERT(engine != 0);
+    if (d->capital == QFont::SmallCaps)
+        return d->smallCapsFontPrivate()->engineForScript(QUnicodeTables::Common)->ascent().toReal();
     return engine->xHeight().toReal();
 }
 
@@ -1208,11 +1292,17 @@ bool QFontMetricsF::inFont(QChar ch) const
 qreal QFontMetricsF::leftBearing(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
         return 0;
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -1236,11 +1326,17 @@ qreal QFontMetricsF::leftBearing(QChar ch) const
 qreal QFontMetricsF::rightBearing(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
         return 0;
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -1294,13 +1390,19 @@ qreal QFontMetricsF::width(const QString &text) const
 */
 qreal QFontMetricsF::width(QChar ch) const
 {
-    if (QUnicodeTables::category(ch) == QChar::Mark_NonSpacing)
+    if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing)
         return 0.;
 
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[8];
     int nglyphs = 7;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -1357,9 +1459,15 @@ QRectF QFontMetricsF::boundingRect(const QString &text) const
 QRectF QFontMetricsF::boundingRect(QChar ch) const
 {
     const int script = QUnicodeTables::script(ch);
-    QFontEngine *engine = d->engineForScript(script);
+    QFontEngine *engine;
+    if (d->capital == QFont::SmallCaps && ch.isLower())
+        engine = d->smallCapsFontPrivate()->engineForScript(script);
+    else
+        engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
 
+    d->alterCharForCapitalization(ch);
+
     QGlyphLayout glyphs[10];
     int nglyphs = 9;
     engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
@@ -1389,7 +1497,7 @@ QRectF QFontMetricsF::boundingRect(QChar ch) const
     \o Qt::TextSingleLine ignores newline characters in the text.
     \o Qt::TextExpandTabs expands tabs (see below)
     \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
-    \o Qt::TextWordBreak breaks the text to fit the rectangle.
+    \o Qt::TextWordWrap breaks the text to fit the rectangle.
     \endlist
 
     Qt::Horizontal alignment defaults to Qt::AlignLeft and vertical
@@ -1473,6 +1581,40 @@ QSizeF QFontMetricsF::size(int flags, const QString &text, int tabStops, int *ta
     return boundingRect(QRectF(), flags, text, tabStops, tabArray).size();
 }
 
+/*!
+  \since 4.3
+
+    Returns a tight bounding rectangle around the characters in the
+    string specified by \a text. The bounding rectangle always covers
+    at least the set of pixels the text would cover if drawn at (0,
+    0).
+
+    Note that the bounding rectangle may extend to the left of (0, 0),
+    e.g. for italicized fonts, and that the width of the returned
+    rectangle might be different than what the width() method returns.
+
+    If you want to know the advance width of the string (to layout
+    a set of strings next to each other), use width() instead.
+
+    Newline characters are processed as normal characters, \e not as
+    linebreaks.
+
+    \warning Calling this method is very slow on Windows.
+
+    \sa width(), height(), boundingRect()
+*/
+QRectF QFontMetricsF::tightBoundingRect(const QString &text) const
+{
+    if (text.length() == 0)
+        return QRect();
+
+    QTextEngine layout(text, d);
+    layout.ignoreBidi = true;
+    layout.itemize();
+    glyph_metrics_t gm = layout.tightBoundingBox(0, text.length());
+    return QRectF(gm.x.toReal(), gm.y.toReal(), gm.width.toReal(), gm.height.toReal());
+}
+
 /*!
     \since 4.2
 
@@ -1589,3 +1731,5 @@ qreal QFontMetricsF::lineWidth() const
         QRect rect = boundingRect(text.left(len));
     \endcode
 */
+
+QT_END_NAMESPACE
