@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,8 +46,22 @@
 #include <qpainterpath.h>
 #include "private/qpdf_p.h"
 
+#ifdef Q_WS_X11
+#include "private/qfontengine_x11_p.h"
+#endif
+
+#ifndef QT_NO_FREETYPE
+#if defined(Q_WS_X11) || defined(Q_WS_QWS)
+#    include "private/qfontengine_ft_p.h"
+#endif
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#endif
+
 #ifndef QT_NO_PRINTER
 
+QT_BEGIN_NAMESPACE
+
 static const char * const agl =
 ".notdef\0space\0exclam\0quotedbl\0numbersign\0dollar\0percent\0ampersand\0"
 "quotesingle\0parenleft\0parenright\0asterisk\0plus\0comma\0hyphen\0period\0"
@@ -269,7 +303,7 @@ static FT_Face ft_face(const QFontEngine *engine)
 #ifdef Q_WS_QWS
     if (engine->type() == QFontEngine::Freetype) {
         const QFontEngineFT *ft = static_cast<const QFontEngineFT *>(engine);
-        return ft->face;
+        return ft->non_locked_face();
     }
 #endif
     return 0;
@@ -279,6 +313,10 @@ static FT_Face ft_face(const QFontEngine *engine)
 QByteArray QFontSubset::glyphName(unsigned int glyph, const QVector<int> reverseMap) const
 {
     uint glyphIndex = glyph_indices[glyph];
+
+    if (glyphIndex == 0)
+        return "/.notdef";
+
     QByteArray ba;
     QPdf::ByteStream s(&ba);
 #ifndef QT_NO_FREETYPE
@@ -292,6 +330,8 @@ QByteArray QFontSubset::glyphName(unsigned int glyph, const QVector<int> reverse
             glyphIndex = static_cast<QFontEngineXLFD *>(fontEngine)->glyphIndexToFreetypeGlyphIndex(glyphIndex);
 #endif
         FT_Get_Glyph_Name(face, glyphIndex, &name, 32);
+        if (name[0] == '.') // fix broken PS fonts returning .notdef for many glyphs
+            name[0] = 0;
     }
     if (name[0]) {
         s << "/" << name;
@@ -303,11 +343,8 @@ QByteArray QFontSubset::glyphName(unsigned int glyph, const QVector<int> reverse
         s << "/" << glyphName(uc, false /* ### */);
     } else
 #endif
-    if (glyph == 0) {
-        s << "/.notdef ";
-    } else if (reverseMap[glyphIndex]) {
-        char tmp[8];
-        s << "/uni" << QPdf::toHex((ushort)reverseMap[glyphIndex], tmp);
+    if (reverseMap[glyphIndex] && reverseMap[glyphIndex] < 0x10000) {
+        s << "/" << glyphName(reverseMap[glyphIndex], false);
     } else {
         s << "/gl" << (int)glyphIndex;
     }
@@ -530,12 +567,6 @@ private:
     uchar *start;
 };
 
-#define MAKE_TAG( str )                      \
-    ( ( (quint32)str[0] << 24 ) |           \
-      ( (quint32)str[1] << 16 ) |           \
-      ( (quint32)str[2] <<  8 ) |           \
-      (quint32)str[3]         )
-
 struct QTtfTable {
     Tag tag;
     QByteArray data;
@@ -646,7 +677,7 @@ static QTtfTable generateHead(const qttf_head_table &head)
 {
     const int head_size = 54;
     QTtfTable t;
-    t.tag = MAKE_TAG("head");
+    t.tag = MAKE_TAG('h', 'e', 'a', 'd');
     t.data.resize(head_size);
 
     QTtfStream s(t.data);
@@ -717,7 +748,7 @@ static QTtfTable generateHhea(const qttf_hhea_table &hhea)
 {
     const int hhea_size = 36;
     QTtfTable t;
-    t.tag = MAKE_TAG("hhea");
+    t.tag = MAKE_TAG('h', 'h', 'e', 'a');
     t.data.resize(hhea_size);
 
     QTtfStream s(t.data);
@@ -768,7 +799,7 @@ static QTtfTable generateMaxp(const qttf_maxp_table &maxp)
 {
     const int maxp_size = 32;
     QTtfTable t;
-    t.tag = MAKE_TAG("maxp");
+    t.tag = MAKE_TAG('m', 'a', 'x', 'p');
     t.data.resize(maxp_size);
 
     QTtfStream s(t.data);
@@ -830,8 +861,8 @@ static QTtfTable generateName(const qttf_name_table &name)
     list.append(rec);
     rec.nameId = 4;
     rec.value = name.family;
-    if (name.subfamily != "Regular")
-        rec.value += " " + name.subfamily;
+    if (name.subfamily != QLatin1String("Regular"))
+        rec.value += QLatin1Char(' ') + name.subfamily;
     list.append(rec);
     rec.nameId = 6;
     rec.value = name.postscript_name;
@@ -846,7 +877,7 @@ static QTtfTable generateName(const QList<QTtfNameRecord> &name)
     const int char_size = 2;
 
     QTtfTable t;
-    t.tag = MAKE_TAG("name");
+    t.tag = MAKE_TAG('n', 'a', 'm', 'e');
 
     const int name_size = 6 + 12*name.size();
     int string_size = 0;
@@ -1204,15 +1235,15 @@ static QList<QTtfTable> generateGlyphTables(qttf_font_tables &tables, const QLis
     tables.hhea.numberOfHMetrics = nGlyphs;
 
     QTtfTable glyf;
-    glyf.tag = MAKE_TAG("glyf");
+    glyf.tag = MAKE_TAG('g', 'l', 'y', 'f');
 
     QTtfTable loca;
-    loca.tag = MAKE_TAG("loca");
+    loca.tag = MAKE_TAG('l', 'o', 'c', 'a');
     loca.data.resize(glyf_size < max_size_small ? (nGlyphs+1)*sizeof(quint16) : (nGlyphs+1)*sizeof(quint32));
     QTtfStream ls(loca.data);
 
     QTtfTable hmtx;
-    hmtx.tag = MAKE_TAG("hmtx");
+    hmtx.tag = MAKE_TAG('h', 'm', 't', 'x');
     hmtx.data.resize(nGlyphs*4);
     QTtfStream hs(hmtx.data);
 
@@ -1311,7 +1342,7 @@ static QByteArray bindFont(const QList<QTtfTable>& _tables)
         for (int i = 0; i < tables.size(); ++i) {
             const QTtfTable &t = tables.at(i);
             const quint32 size = (t.data.size() + 3) & ~3;
-            if (t.tag == MAKE_TAG("head"))
+            if (t.tag == MAKE_TAG('h', 'e', 'a', 'd'))
                 head_offset = table_offset;
             f << t.tag
               << checksum(t.data)
@@ -1330,7 +1361,7 @@ static QByteArray bindFont(const QList<QTtfTable>& _tables)
     }
 
     if (!head_offset) {
-        qWarning("Font misses 'head' table");
+        qWarning("QFontSubset: Font misses 'head' table");
         return QByteArray();
     }
 
@@ -1435,25 +1466,25 @@ QByteArray QFontSubset::toTruetype() const
     tables.append(generateMaxp(font.maxp));
     // name
     QTtfTable name_table;
-    name_table.tag = MAKE_TAG("name");
+    name_table.tag = MAKE_TAG('n', 'a', 'm', 'e');
     if (!noEmbed)
         name_table.data = fontEngine->getSfntTable(name_table.tag);
     if (name_table.data.isEmpty()) {
         qttf_name_table name;
         if (noEmbed)
-            name.copyright = "Fake font";
+            name.copyright = QLatin1String("Fake font");
         else
-            name.copyright = properties.copyright;
+            name.copyright = QLatin1String(properties.copyright);
         name.family = fontEngine->fontDef.family;
         name.subfamily = QLatin1String("Regular"); // ######
-        name.postscript_name = properties.postscriptName;
+        name.postscript_name = QLatin1String(properties.postscriptName);
         name_table = generateName(name);
     }
     tables.append(name_table);
 
     if (!noEmbed) {
         QTtfTable os2;
-        os2.tag = MAKE_TAG("OS/2");
+        os2.tag = MAKE_TAG('O', 'S', '/', '2');
         os2.data = fontEngine->getSfntTable(os2.tag);
         if (!os2.data.isEmpty())
             tables.append(os2);
@@ -1583,23 +1614,30 @@ static QByteArray charString(const QPainterPath &path, qreal advance, qreal lsb,
     if (openpath)
         charstring += closepath;
     charstring += endchar;
+    if (charstring.length() > 240) {
+        int pos = 240;
+        while (pos < charstring.length()) {
+            charstring.insert(pos, '\n');
+            pos += 241;
+        }
+    }
     return charstring;
 }
 
 #ifndef QT_NO_FREETYPE
-const char *helvetica_styles[4] = {
+static const char *helvetica_styles[4] = {
     "Helvetica",
     "Helvetica-Bold",
     "Helvetica-Oblique",
     "Helvetica-BoldOblique"
 };
-const char *times_styles[4] = {
+static const char *times_styles[4] = {
     "Times-Regular",
     "Times-Bold",
     "Times-Italic",
     "Times-BoldItalic"
 };
-const char *courier_styles[4] = {
+static const char *courier_styles[4] = {
     "Courier",
     "Courier-Bold",
     "Courier-Oblique",
@@ -1615,13 +1653,11 @@ QByteArray QFontSubset::toType1() const
     QByteArray font;
     QPdf::ByteStream s(&font);
 
-    int nGlyphs = this->nGlyphs();
-
     QByteArray id = QByteArray::number(object_id);
     QByteArray psname = properties.postscriptName;
     psname.replace(" ", "");
 
-    bool standard_font = false;
+    standard_font = false;
 
 #ifndef QT_NO_FREETYPE
     FT_Face face = ft_face(fontEngine);
@@ -1643,8 +1679,12 @@ QByteArray QFontSubset::toType1() const
         }
     }
 #endif
-    if (!standard_font) {
-        s << "/F" << id << "-Base <<\n";
+    s << "/F" << id << "-Base\n";
+    if (standard_font) {
+            s << "/" << psname << " findfont\n"
+                "0 dict copy dup /NumGlyphs 0 put dup /CMap 256 array put def\n";
+    } else {
+        s << "<<\n";
         if(!psname.isEmpty())
             s << "/FontName /" << psname << "\n";
         s << "/FontInfo <</FsType " << (int)fontEngine->fsType << ">>\n"
@@ -1658,58 +1698,46 @@ QByteArray QFontSubset::toType1() const
             "/BlueValues []\n"
             "/lenIV -1\n"
             ">>\n"
-            "/CharStrings <<\n";
-
-        for (int i = 0; i < nGlyphs; ++i) {
-            glyph_t g = glyph_indices.at(i);
-            QPainterPath path;
-            glyph_metrics_t metric;
-            fontEngine->getUnscaledGlyph(g, &path, &metric);
-            QByteArray charstring = ::charString(path, metric.xoff.toReal(), metric.x.toReal(),
-                                                 properties.emSquare.toReal());
-            s << glyphName(i, reverseMap)
-              << " <" << charstring << ">\n";
-        }
-        s << ">>\n"
+            "/CharStrings << >>\n"
+            "/NumGlyphs 0\n"
+            "/CMap 256 array\n"
             ">> def\n";
     }
-    int page = 0;
-    while (nGlyphs > 0) {
-        s << "/F" << id << "-" << page;
-        if (standard_font)
-            s << "/" << psname << " findfont\n";
-        else
-            s << "F" << id << "-Base\n";
-
-        s << "0 dict copy dup /Encoding 256 array\n"
-            "0 1 255 {1 index exch /.notdef put} for\n";
-        for (int i = 0; i < nGlyphs; ++i)
-            s << "dup " << i << glyphName(i, reverseMap) << " put\n";
-        s << "put definefont pop\n";
-        ++page;
-        nGlyphs -= 256;
-    }
-    s << "/F" << id << " <<\n"
-        "/FontType 0\n"
-        "/FMapType 2\n"
-        "/FontMatrix[1 0 0 1 0 0]\n"
-        "/Encoding [";
-    for (int i = 0; i < page; ++i) {
-        if (page % 16 == 0)
-            s << "\n";
-        s << i;
-    }
-    s << "]\n"
-        "/FDepVector [\n";
-    for (int i = 0; i < page; ++i) {
-        s << "/F" << id << "-" << i
-          << " findfont\n";
-    }
-    s << "]\n"
-        ">> definefont pop\n";
+    s << type1AddedGlyphs();
+    downloaded_glyphs = glyph_indices.size();
 
     return font;
 }
 
-#endif
+QByteArray QFontSubset::type1AddedGlyphs() const
+{
+    if (downloaded_glyphs == glyph_indices.size())
+        return QByteArray();
 
+    QFontEngine::Properties properties = fontEngine->properties();
+    QVector<int> reverseMap = getReverseMap();
+    QByteArray glyphs;
+    QPdf::ByteStream s(&glyphs);
+
+    int nGlyphs = glyph_indices.size();
+    QByteArray id = QByteArray::number(object_id);
+
+    s << "F" << id << "-Base [\n";
+    for (int i = downloaded_glyphs; i < nGlyphs; ++i) {
+        glyph_t g = glyph_indices.at(i);
+        QPainterPath path;
+        glyph_metrics_t metric;
+        fontEngine->getUnscaledGlyph(g, &path, &metric);
+        QByteArray charstring = charString(path, metric.xoff.toReal(), metric.x.toReal(),
+                                             properties.emSquare.toReal());
+        s << glyphName(i, reverseMap);
+        if (!standard_font)
+          s << "\n<" << charstring << ">\n";
+    }
+    s << (standard_font ? "] T1AddMapping\n" : "] T1AddGlyphs\n");
+    return glyphs;
+}
+
+QT_END_NAMESPACE
+
+#endif // QT_NO_PRINTER
