@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,6 +55,8 @@
 #include <qtextedit.h>
 #include <qtimer.h>
 
+QT_BEGIN_NAMESPACE
+
 class QSyntaxHighlighterPrivate : public QObjectPrivate
 {
     Q_DECLARE_PUBLIC(QSyntaxHighlighter)
@@ -136,7 +158,7 @@ void QSyntaxHighlighterPrivate::_q_reformatBlocks(int from, int charsRemoved, in
     QTextBlock block = doc->findBlock(from);
     if (!block.isValid())
         return;
-    
+
     int endPosition;
     QTextBlock lastBlock = doc->findBlock(from + charsAdded);
     if (lastBlock.isValid())
@@ -179,6 +201,7 @@ void QSyntaxHighlighterPrivate::reformatBlock(QTextBlock block)
 
 /*!
     \class QSyntaxHighlighter
+    \reentrant
 
     \brief The QSyntaxHighlighter class allows you to define syntax
     highlighting rules, and in addition you can use the class to query
@@ -202,10 +225,7 @@ void QSyntaxHighlighterPrivate::reformatBlock(QTextBlock block)
     pass it the QTextEdit or QTextDocument that you want the syntax
     highlighting to be applied to. For example:
 
-    \code
-           QTextEdit *editor = new QTextEdit;
-           MyHighlighter *highlighter = new MyHighlighter(editor->document());
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qsyntaxhighlighter.cpp 0
 
     After this your highlightBlock() function will be called
     automatically whenever necessary. Use your highlightBlock()
@@ -214,23 +234,7 @@ void QSyntaxHighlighterPrivate::reformatBlock(QTextBlock block)
     setFormat() function which applies a given QTextCharFormat on
     the current text block. For example:
 
-    \code
-        void MyHighlighter::highlightBlock(const QString &text)
-        {
-            QTextCharFormat myClassFormat;
-            myClassFormat.setFontWeight(QFont::Bold);
-            myClassFormat.setForeground(Qt::darkMagenta);
-            QString pattern = "\\bMy[A-Za-z]+\\b";
-
-            QRegExp expression(pattern);
-            int index = text.indexOf(expression);
-            while (index >= 0) {
-                int length = expression.matchedLength();
-                setFormat(index, length, myClassFormat);
-                index = text.indexOf(expression, index + length);
-             }
-         }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qsyntaxhighlighter.cpp 1
 
     Some syntaxes can have constructs that span several text
     blocks. For example, a C++ syntax highlighter should be able to
@@ -253,34 +257,7 @@ void QSyntaxHighlighterPrivate::reformatBlock(QTextBlock block)
     For example, if you're writing a simple C++ syntax highlighter,
     you might designate 1 to signify "in comment":
 
-    \code
-        QTextCharFormat multiLineCommentFormat;
-        multiLineCommentFormat.setForeground(Qt::red);
-
-        QRegExp startExpression("/\\*");
-        QRegExp endExpression("\\* /");
-
-        setCurrentBlockState(0);
-
-        int startIndex = 0;
-        if (previousBlockState() != 1)
-            startIndex = text.indexOf(startExpression);
-
-        while (startIndex >= 0) {
-           int endIndex = text.indexOf(endExpression, startIndex);
-           int commentLength;
-           if (endIndex == -1) {
-               setCurrentBlockState(1);
-               commentLength = text.length() - startIndex;
-           } else {
-               commentLength = endIndex - startIndex
-                               + endExpression.matchedLength();
-           }
-           setFormat(startIndex, commentLength, multiLineCommentFormat);
-           startIndex = text.indexOf(startExpression,
-                                     startIndex + commentLength);
-        }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qsyntaxhighlighter.cpp 2
 
     In the example above, we first set the current block state to
     0. Then, if the previous block ended within a comment, we higlight
@@ -388,11 +365,15 @@ void QSyntaxHighlighter::rehighlight()
     if (!d->doc)
         return;
 
+    disconnect(d->doc, SIGNAL(contentsChange(int,int,int)),
+               this, SLOT(_q_reformatBlocks(int,int,int)));
     QTextCursor cursor(d->doc);
     cursor.beginEditBlock();
     cursor.movePosition(QTextCursor::End);
     d->_q_reformatBlocks(0, 0, cursor.position());
     cursor.endEditBlock();
+    connect(d->doc, SIGNAL(contentsChange(int,int,int)),
+            this, SLOT(_q_reformatBlocks(int,int,int)));
 }
 
 /*!
@@ -408,23 +389,7 @@ void QSyntaxHighlighter::rehighlight()
     setFormat() as often as necessary to apply any font and color
     changes that you require. For example:
 
-    \code
-        void MyHighlighter::highlightBlock(const QString &text)
-        {
-            QTextCharFormat myClassFormat;
-            myClassFormat.setFontWeight(QFont::Bold);
-            myClassFormat.setForeground(Qt::darkMagenta);
-            QString pattern = "\\bMy[A-Za-z]+\\b";
-
-            QRegExp expression(pattern);
-            int index = text.indexOf(expression);
-            while (index >= 0) {
-                int length = expression.matchedLength();
-                setFormat(index, length, myClassFormat);
-                index = text.indexOf(expression, index + length);
-             }
-         }
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qsyntaxhighlighter.cpp 3
 
     Some syntaxes can have constructs that span several text
     blocks. For example, a C++ syntax highlighter should be able to
@@ -595,18 +560,7 @@ void QSyntaxHighlighter::setCurrentBlockState(int newState)
     and store their relative position and the actual QChar in a simple
     class derived from QTextBlockUserData:
 
-    \code
-        struct ParenthesisInfo
-        {
-            QChar char;
-            int position;
-        };
-
-        struct BlockData : public QTextBlockUserData
-        {
-            QVector<ParenthesisInfo> parentheses;
-        };
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.text.qsyntaxhighlighter.cpp 4
 
     During cursor navigation in the associated editor, you can ask the
     current QTextBlock (retrieved using the QTextCursor::block()
@@ -648,6 +602,17 @@ QTextBlockUserData *QSyntaxHighlighter::currentBlockUserData() const
     return d->currentBlock.userData();
 }
 
+/*!
+    Returns the current text block.
+ */
+QTextBlock QSyntaxHighlighter::currentBlock() const
+{
+    Q_D(const QSyntaxHighlighter);
+    return d->currentBlock;
+}
+
+QT_END_NAMESPACE
+
 #include "moc_qsyntaxhighlighter.cpp"
 
 #endif // QT_NO_SYNTAXHIGHLIGHTER
