@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,6 +51,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QTextFormatCollection;
@@ -49,6 +71,7 @@ class QTextCodec;
 class QUrl;
 class QVariant;
 class QRectF;
+class QTextOption;
 
 template<typename T> class QVector;
 
@@ -97,8 +120,12 @@ class Q_GUI_EXPORT QTextDocument : public QObject
     Q_PROPERTY(QSizeF size READ size)
     Q_PROPERTY(qreal textWidth READ textWidth WRITE setTextWidth)
     Q_PROPERTY(int blockCount READ blockCount)
+    Q_PROPERTY(qreal indentWidth READ indentWidth WRITE setIndentWidth)
+#ifndef QT_NO_CSSPARSER
     Q_PROPERTY(QString defaultStyleSheet READ defaultStyleSheet WRITE setDefaultStyleSheet)
+#endif
     Q_PROPERTY(int maximumBlockCount READ maximumBlockCount WRITE setMaximumBlockCount)
+    QDOC_PROPERTY(QTextOption defaultTextOption READ defaultTextOption WRITE setDefaultTextOption)
 
 public:
     explicit QTextDocument(QObject *parent = 0);
@@ -116,17 +143,22 @@ public:
     bool isUndoAvailable() const;
     bool isRedoAvailable() const;
 
+    int revision() const;
+
     void setDocumentLayout(QAbstractTextDocumentLayout *layout);
     QAbstractTextDocumentLayout *documentLayout() const;
 
     enum MetaInformation {
-        DocumentTitle
+        DocumentTitle,
+        DocumentUrl
     };
     void setMetaInformation(MetaInformation info, const QString &);
     QString metaInformation(MetaInformation info) const;
 
+#ifndef QT_NO_TEXTHTMLPARSER
     QString toHtml(const QByteArray &encoding = QByteArray()) const;
     void setHtml(const QString &html);
+#endif
 
     QString toPlainText() const;
     void setPlainText(const QString &text);
@@ -152,9 +184,13 @@ public:
     QTextObject *objectForFormat(const QTextFormat &) const;
 
     QTextBlock findBlock(int pos) const;
+    QTextBlock findBlockByNumber(int blockNumber) const;
     QTextBlock begin() const;
     QTextBlock end() const;
 
+    QTextBlock firstBlock() const;
+    QTextBlock lastBlock() const;
+
     void setPageSize(const QSizeF &size);
     QSizeF pageSize() const;
 
@@ -194,13 +230,18 @@ public:
 
     qreal idealWidth() const;
 
+    qreal indentWidth() const;
+    void setIndentWidth(qreal width);
+
     void adjustSize();
     QSizeF size() const;
 
     int blockCount() const;
 
+#ifndef QT_NO_CSSPARSER
     void setDefaultStyleSheet(const QString &sheet);
     QString defaultStyleSheet() const;
+#endif
 
     void undo(QTextCursor *cursor);
     void redo(QTextCursor *cursor);
@@ -208,13 +249,20 @@ public:
     int maximumBlockCount() const;
     void setMaximumBlockCount(int maximum);
 
+    QTextOption defaultTextOption() const;
+    void setDefaultTextOption(const QTextOption &option);
+
 Q_SIGNALS:
     void contentsChange(int from, int charsRemoves, int charsAdded);
     void contentsChanged();
     void undoAvailable(bool);
     void redoAvailable(bool);
+    void undoCommandAdded();
     void modificationChanged(bool m);
     void cursorPositionChanged(const QTextCursor &cursor);
+    void blockCountChanged(int newBlockCount);
+
+    void documentLayoutChanged();
 
 public Q_SLOTS:
     void undo();
@@ -236,6 +284,8 @@ private:
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QTextDocument::FindFlags)
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QTEXTDOCUMENT_H
