@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,7 +57,8 @@
 #include "qtextengine_p.h"
 
 #include <stdlib.h>
-#include <new>
+
+QT_BEGIN_NAMESPACE
 
 #define PMDEBUG if(0) qDebug
 
@@ -155,6 +176,8 @@ bool QTextUndoCommand::tryMerge(const QTextUndoCommand &other)
 }
 
 QTextDocumentPrivate::QTextDocumentPrivate()
+    : wasUndoAvailable(false)
+    , wasRedoAvailable(false)
 {
     editBlock = 0;
     docChangeFrom = -1;
@@ -168,9 +191,15 @@ QTextDocumentPrivate::QTextDocumentPrivate()
 
     undoEnabled = true;
     inContentsChange = false;
+    defaultTextOption.setTabStop(80); // same as in qtextengine.cpp
+    defaultTextOption.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+
+    indentWidth = 40;
 
-    useDesignMetrics = false;
     maximumBlockCount = 0;
+    needsEnsureMaximumBlockCount = false;
+    unreachableCharacterCount = 0;
+    lastBlockCount = 0;
 }
 
 void QTextDocumentPrivate::init()
@@ -216,6 +245,7 @@ void QTextDocumentPrivate::clear()
     undoState = 0;
     truncateUndoStack();
     text = QString();
+    unreachableCharacterCount = 0;
     modifiedState = 0;
     modified = false;
     formats = QTextFormatCollection();
@@ -223,12 +253,14 @@ void QTextDocumentPrivate::clear()
     fragments.clear();
     blocks.clear();
     cachedResources.clear();
-    q->contentsChange(0, len, 0);
-    if (lout)
-        lout->documentChanged(0, len, 0);
     delete rtFrame;
     init();
     cursors = oldCursors;
+    inContentsChange = true;
+    q->contentsChange(0, len, 0);
+    inContentsChange = false;
+    if (lout)
+        lout->documentChanged(0, len, 0);
 }
 
 QTextDocumentPrivate::~QTextDocumentPrivate()
@@ -246,12 +278,15 @@ void QTextDocumentPrivate::setLayout(QAbstractTextDocumentLayout *layout)
     Q_Q(QTextDocument);
     if (lout == layout)
         return;
+    const bool firstLayout = !lout;
     delete lout;
     lout = layout;
 
-    for (BlockMap::Iterator it = blocks.begin(); !it.atEnd(); ++it)
-        it->free();
+    if (!firstLayout)
+        for (BlockMap::Iterator it = blocks.begin(); !it.atEnd(); ++it)
+            it->free();
 
+    emit q->documentLayoutChanged();
     inContentsChange = true;
     emit q->contentsChange(0, 0, length());
     inContentsChange = false;
@@ -350,14 +385,28 @@ int QTextDocumentPrivate::insertBlock(const QChar &blockSeparator,
 
     Q_ASSERT(blocks.length() == fragments.length());
 
+    int b = blocks.findNode(pos);
+    QTextBlockData *B = blocks.fragment(b);
+
     QTextUndoCommand c = { QTextUndoCommand::BlockInserted, true,
-                      op, charFormat, strPos, pos, { blockFormat } };
+                           op, charFormat, strPos, pos, { blockFormat },
+                           B->revision };
 
     appendUndoItem(c);
     Q_ASSERT(undoState == undoStack.size());
 
+    // update revision numbers of the modified blocks. Close to the
+    // truth, but we may want to special case breaking a block before
+    // the first or behind the last character
+    B->revision = undoState;
+    b = blocks.next(b);
+    if (b) {
+        B = blocks.fragment(b);
+        B->revision = undoState;
+    }
+
     if (formats.charFormat(charFormat).objectIndex() == -1)
-        ensureMaximumBlockCount();
+        needsEnsureMaximumBlockCount = true;
 
     endEditBlock();
     return fragment;
@@ -380,9 +429,14 @@ void QTextDocumentPrivate::insert(int pos, int strPos, int strLength, int format
 
     beginEditBlock();
 
+    int b = blocks.findNode(pos);
+    QTextBlockData *B = blocks.fragment(b);
+
     QTextUndoCommand c = { QTextUndoCommand::Inserted, true,
-                      QTextUndoCommand::MoveCursor, format, strPos, pos, { strLength } };
+                           QTextUndoCommand::MoveCursor, format, strPos, pos, { strLength },
+                           B->revision };
     appendUndoItem(c);
+    B->revision = undoState;
     Q_ASSERT(undoState == undoStack.size());
 
     endEditBlock();
@@ -423,6 +477,9 @@ int QTextDocumentPrivate::remove_string(int pos, uint length, QTextUndoCommand::
 
     const int w = fragments.erase_single(x);
 
+    if (!undoEnabled)
+        unreachableCharacterCount += length;
+
     adjustDocumentChangesAndCursors(pos, -int(length), op);
 
     return w;
@@ -486,11 +543,17 @@ static bool isAncestorFrame(QTextFrame *possibleAncestor, QTextFrame *child)
 }
 #endif
 
-void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operation op)
+void QTextDocumentPrivate::move(int pos, int to, int length, QTextUndoCommand::Operation op)
 {
+    Q_ASSERT(to <= fragments.length() && to <= pos);
     Q_ASSERT(pos >= 0 && pos+length <= fragments.length());
     Q_ASSERT(blocks.length() == fragments.length());
 
+    if (pos == to)
+        return;
+
+    bool needsInsert = to != -1;
+
 #if !defined(QT_NO_DEBUG)
     const bool startAndEndInSameFrame = (frameAt(pos) == frameAt(pos + length - 1));
 
@@ -513,6 +576,9 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
     split(pos);
     split(pos+length);
 
+    uint dst = needsInsert ? fragments.findNode(to) : 0;
+    uint dstKey = needsInsert ? fragments.position(dst) : 0;
+
     uint x = fragments.findNode(pos);
     uint end = fragments.findNode(pos+length);
 
@@ -522,25 +588,47 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
 
         uint key = fragments.position(x);
         uint b = blocks.findNode(key+1);
+        QTextBlockData *B = blocks.fragment(b);
+        int blockRevision = B->revision;
 
         QTextFragmentData *X = fragments.fragment(x);
         QTextUndoCommand c = { QTextUndoCommand::Removed, true,
-                          op, X->format, X->stringPosition, key, { X->size } };
+                               op, X->format, X->stringPosition, key, { X->size },
+                               blockRevision };
+        QTextUndoCommand cInsert = { QTextUndoCommand::Inserted, true,
+                                     op, X->format, X->stringPosition, dstKey, { X->size },
+                                     blockRevision };
 
         if (key+1 != blocks.position(b)) {
 //	    qDebug("remove_string from %d length %d", key, X->size);
             Q_ASSERT(noBlockInString(text.mid(X->stringPosition, X->size)));
             w = remove_string(key, X->size, op);
+
+            if (needsInsert) {
+                insert_string(dstKey, X->stringPosition, X->size, X->format, op);
+                dstKey += X->size;
+            }
         } else {
 //	    qDebug("remove_block at %d", key);
             Q_ASSERT(X->size == 1 && isValidBlockSeparator(text.at(X->stringPosition)));
             b = blocks.previous(b);
+            B = 0;
             c.command = blocks.size(b) == 1 ? QTextUndoCommand::BlockDeleted : QTextUndoCommand::BlockRemoved;
             w = remove_block(key, &c.blockFormat, QTextUndoCommand::BlockAdded, op);
+
+            if (needsInsert) {
+                insert_block(dstKey++, X->stringPosition, X->format, c.blockFormat, op, QTextUndoCommand::BlockRemoved);
+                cInsert.command = blocks.size(b) == 1 ? QTextUndoCommand::BlockAdded : QTextUndoCommand::BlockInserted;
+                cInsert.blockFormat = c.blockFormat;
+            }
         }
         appendUndoItem(c);
+        if (B)
+            B->revision = undoState;
         x = n;
 
+        if (needsInsert)
+            appendUndoItem(cInsert);
     }
     if (w)
         unite(w);
@@ -550,6 +638,11 @@ void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operati
     endEditBlock();
 }
 
+void QTextDocumentPrivate::remove(int pos, int length, QTextUndoCommand::Operation op)
+{
+    move(pos, -1, length, op);
+}
+
 void QTextDocumentPrivate::setCharFormat(int pos, int length, const QTextCharFormat &newFormat, FormatChangeMode mode)
 {
     beginEditBlock();
@@ -615,7 +708,7 @@ void QTextDocumentPrivate::setCharFormat(int pos, int length, const QTextCharFor
         }
 
         QTextUndoCommand c = { QTextUndoCommand::CharFormatChanged, true, QTextUndoCommand::MoveCursor, oldFormat,
-                          0, pos, { length } };
+                               0, pos, { length }, 0 };
         appendUndoItem(c);
 
         pos += length;
@@ -675,7 +768,7 @@ void QTextDocumentPrivate::setBlockFormat(const QTextBlock &from, const QTextBlo
         block(it)->invalidate();
 
         QTextUndoCommand c = { QTextUndoCommand::BlockFormatChanged, true, QTextUndoCommand::MoveCursor, oldFormat,
-                              0, it.position(), { 1 } };
+                               0, it.position(), { 1 }, 0 };
         appendUndoItem(c);
 
         if (group != oldGroup) {
@@ -752,6 +845,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
         if (undo)
             --undoState;
         QTextUndoCommand &c = undoStack[undoState];
+        int resetBlockRevision = c.pos;
 
 	switch(c.command) {
         case QTextUndoCommand::Inserted:
@@ -777,12 +871,14 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 	case QTextUndoCommand::BlockDeleted:
             PMDEBUG("   blockinsert: charformat %d blockformat %d (pos %d, strpos=%d)", c.format, c.blockFormat, c.pos, c.strPos);
             insert_block(c.pos, c.strPos, c.format, c.blockFormat, (QTextUndoCommand::Operation)c.operation, c.command);
+            resetBlockRevision += 1;
 	    if (c.command == QTextUndoCommand::BlockRemoved)
 		c.command = QTextUndoCommand::BlockInserted;
 	    else
 		c.command = QTextUndoCommand::BlockAdded;
 	    break;
 	case QTextUndoCommand::CharFormatChanged: {
+            resetBlockRevision = -1; // ## TODO
             PMDEBUG("   charFormat: format %d (from %d, length %d)", c.format, c.pos, c.length);
             FragmentIterator it = find(c.pos);
             Q_ASSERT(!it.atEnd());
@@ -793,6 +889,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 	    break;
 	}
 	case QTextUndoCommand::BlockFormatChanged: {
+            resetBlockRevision = -1; // ## TODO
             PMDEBUG("   blockformat: format %d pos %d", c.format, c.pos);
             QTextBlock it = blocksFind(c.pos);
             Q_ASSERT(it.isValid());
@@ -814,6 +911,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 	    break;
 	}
 	case QTextUndoCommand::GroupFormatChange: {
+            resetBlockRevision = -1; // ## TODO
             PMDEBUG("   group format change");
             QTextObject *object = objectForIndex(c.objectIndex);
             int oldFormat = formats.objectFormatIndex(c.objectIndex);
@@ -822,6 +920,7 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 	    break;
 	}
 	case QTextUndoCommand::Custom:
+            resetBlockRevision = -1; // ## TODO
             if (undo)
                 c.custom->undo();
             else
@@ -830,6 +929,13 @@ int QTextDocumentPrivate::undoRedo(bool undo)
 	default:
 	    Q_ASSERT(false);
         }
+
+        if (resetBlockRevision >= 0) {
+            int b = blocks.findNode(resetBlockRevision);
+            QTextBlockData *B = blocks.fragment(b);
+            B->revision = c.revision;
+        }
+
         if (undo) {
             if (undoState == 0 || !undoStack[undoState-1].block)
                 break;
@@ -845,9 +951,8 @@ int QTextDocumentPrivate::undoRedo(bool undo)
         editPos = qMin(docChangeFrom + docChangeLength, length() - 1);
     }
     endEditBlock();
-    Q_Q(QTextDocument);
-    emit q->undoAvailable(isUndoAvailable());
-    emit q->redoAvailable(isRedoAvailable());
+    emitUndoAvailable(isUndoAvailable());
+    emitRedoAvailable(isRedoAvailable());
     return editPos;
 }
 
@@ -876,7 +981,6 @@ void QTextDocumentPrivate::appendUndoItem(QAbstractUndoItem *item)
 
 void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
 {
-    Q_Q(QTextDocument);
     PMDEBUG("appendUndoItem, command=%d enabled=%d", c.command, undoEnabled);
     if (!undoEnabled)
         return;
@@ -888,10 +992,13 @@ void QTextDocumentPrivate::appendUndoItem(const QTextUndoCommand &c)
         if (last.tryMerge(c))
             return;
     }
+    if (modifiedState > undoState)
+        modifiedState = -1;
     undoStack.append(c);
     undoState++;
-    emit q->undoAvailable(true);
-    emit q->redoAvailable(false);
+    emitUndoAvailable(true);
+    emitRedoAvailable(false);
+    emit document()->undoCommandAdded();
 }
 
 void QTextDocumentPrivate::truncateUndoStack() {
@@ -915,17 +1022,39 @@ void QTextDocumentPrivate::truncateUndoStack() {
     undoStack.resize(undoState);
 }
 
+void QTextDocumentPrivate::emitUndoAvailable(bool available)
+{
+    if (available != wasUndoAvailable) {
+        Q_Q(QTextDocument);
+        emit q->undoAvailable(available);
+        wasUndoAvailable = available;
+    }
+}
+
+void QTextDocumentPrivate::emitRedoAvailable(bool available)
+{
+    if (available != wasRedoAvailable) {
+        Q_Q(QTextDocument);
+        emit q->redoAvailable(available);
+        wasRedoAvailable = available;
+    }
+}
+
 void QTextDocumentPrivate::enableUndoRedo(bool enable)
 {
-    Q_Q(QTextDocument);
+    if (enable && maximumBlockCount > 0)
+        return;
+
     if (!enable) {
         undoState = 0;
         truncateUndoStack();
-        emit q->undoAvailable(false);
-        emit q->redoAvailable(false);
+        emitUndoAvailable(false);
+        emitRedoAvailable(false);
     }
     modifiedState = modified ? -1 : undoState;
     undoEnabled = enable;
+    if (!undoEnabled)
+        compressPieceTable();
 }
 
 void QTextDocumentPrivate::joinPreviousEditBlock()
@@ -959,12 +1088,31 @@ void QTextDocumentPrivate::endEditBlock()
 
     docChangeFrom = -1;
 
+    if (needsEnsureMaximumBlockCount) {
+        needsEnsureMaximumBlockCount = false;
+        if (ensureMaximumBlockCount()) {
+            // if ensureMaximumBlockCount() returns true
+            // it will have called endEditBlock() and
+            // compressPieceTable() itself, so we return here
+            // to prevent getting two contentsChanged emits
+            return;
+        }
+    }
+
     while (!changedCursors.isEmpty()) {
         QTextCursorPrivate *curs = changedCursors.takeFirst();
         emit q->cursorPositionChanged(QTextCursor(curs));
     }
 
     contentsChanged();
+
+    if (blocks.numNodes() != lastBlockCount) {
+        lastBlockCount = blocks.numNodes();
+        emit q->blockCountChanged(lastBlockCount);
+    }
+
+    if (!undoEnabled && unreachableCharacterCount)
+        compressPieceTable();
 }
 
 void QTextDocumentPrivate::documentChange(int from, int length)
@@ -990,10 +1138,12 @@ void QTextDocumentPrivate::adjustDocumentChangesAndCursors(int from, int addedOr
     for (int i = 0; i < cursors.size(); ++i) {
         QTextCursorPrivate *curs = cursors.at(i);
         if (curs->adjustPosition(from, addedOrRemoved, op) == QTextCursorPrivate::CursorMoved) {
-            if (editBlock)
-                changedCursors.append(curs);
-            else
+            if (editBlock) {
+                if (!changedCursors.contains(curs))
+                    changedCursors.append(curs);
+            } else {
                 emit q->cursorPositionChanged(QTextCursor(curs));
+            }
         }
     }
 
@@ -1096,19 +1246,14 @@ void QTextDocumentPrivate::changeObjectFormat(QTextObject *obj, int format)
 
     QTextBlockGroup *b = qobject_cast<QTextBlockGroup *>(obj);
     if (b) {
-        QList<QTextBlock> blocks = b->blockList();
-        for (int i = 0; i < blocks.size(); ++i) {
-            // invalidate blocks and tell layout
-            const QTextBlock &block = blocks.at(i);
-            documentChange(block.position(), block.length());
-        }
+        b->d_func()->markBlocksDirty();
     }
     QTextFrame *f = qobject_cast<QTextFrame *>(obj);
     if (f)
         documentChange(f->firstPosition(), f->lastPosition() - f->firstPosition());
 
     QTextUndoCommand c = { QTextUndoCommand::GroupFormatChange, true, QTextUndoCommand::MoveCursor, oldFormatIndex,
-                      0, 0, { obj->d_func()->objectIndex } };
+                           0, 0, { obj->d_func()->objectIndex }, 0 };
     appendUndoItem(c);
 
     endEditBlock();
@@ -1130,7 +1275,7 @@ QTextFrame *QTextDocumentPrivate::rootFrame() const
 {
     if (!rtFrame) {
         QTextFrameFormat defaultRootFrameFormat;
-        defaultRootFrameFormat.setMargin(2);
+        defaultRootFrameFormat.setMargin(DefaultRootFrameMargin);
         rtFrame = qobject_cast<QTextFrame *>(const_cast<QTextDocumentPrivate *>(this)->createObject(defaultRootFrameFormat));
     }
     return rtFrame;
@@ -1342,6 +1487,39 @@ void QTextDocumentPrivate::contentsChanged()
     emit q->contentsChanged();
 }
 
+void QTextDocumentPrivate::compressPieceTable()
+{
+    if (undoEnabled)
+        return;
+
+    const uint garbageCollectionThreshold = 96 * 1024; // bytes
+
+    //qDebug() << "unreachable bytes:" << unreachableCharacterCount * sizeof(QChar) << " -- limit" << garbageCollectionThreshold << "text size =" << text.size() << "capacity:" << text.capacity();
+
+    bool compressTable = unreachableCharacterCount * sizeof(QChar) > garbageCollectionThreshold
+                         && text.size() >= text.capacity() * 0.9;
+    if (!compressTable)
+        return;
+
+    QString newText;
+    newText.resize(text.size());
+    QChar *newTextPtr = newText.data();
+    int newLen = 0;
+
+    for (FragmentMap::Iterator it = fragments.begin(); !it.atEnd(); ++it) {
+        qMemCopy(newTextPtr, text.constData() + it->stringPosition, it->size * sizeof(QChar));
+        it->stringPosition = newLen;
+        newTextPtr += it->size;
+        newLen += it->size;
+    }
+
+    newText.resize(newLen);
+    newText.squeeze();
+    //qDebug() << "removed" << text.size() - newText.size() << "characters";
+    text = newText;
+    unreachableCharacterCount = 0;
+}
+
 void QTextDocumentPrivate::setModified(bool m)
 {
     Q_Q(QTextDocument);
@@ -1357,12 +1535,12 @@ void QTextDocumentPrivate::setModified(bool m)
     emit q->modificationChanged(modified);
 }
 
-void QTextDocumentPrivate::ensureMaximumBlockCount()
+bool QTextDocumentPrivate::ensureMaximumBlockCount()
 {
     if (maximumBlockCount <= 0)
-        return;
+        return false;
     if (blocks.numNodes() <= maximumBlockCount)
-        return;
+        return false;
 
     beginEditBlock();
 
@@ -1370,11 +1548,18 @@ void QTextDocumentPrivate::ensureMaximumBlockCount()
     QTextCursor cursor(this, 0);
     cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor, blocksToRemove);
 
+    unreachableCharacterCount += cursor.selectionEnd() - cursor.selectionStart();
+
     // preserve the char format of the paragraph that is to become the new first one
     QTextCharFormat charFmt = cursor.blockCharFormat();
     cursor.removeSelectedText();
     cursor.setBlockCharFormat(charFmt);
 
     endEditBlock();
+
+    compressPieceTable();
+
+    return true;
 }
 
+QT_END_NAMESPACE
