@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -59,6 +59,8 @@
 #include "QtCore/qvector.h"
 #include "QtCore/qset.h"
 
+QT_BEGIN_NAMESPACE
+
 class Q_GUI_EXPORT QTextFormatCollection
 {
 public:
@@ -106,4 +108,6 @@ private:
     QFont defaultFnt;
 };
 
+QT_END_NAMESPACE
+
 #endif // QTEXTFORMAT_P_H
