@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 class QTextEngine;
@@ -137,6 +139,7 @@ public:
 
     void beginLayout();
     void endLayout();
+    void clearLayout();
 
     QTextLine createLine();
 
@@ -166,6 +169,7 @@ public:
     qreal maximumWidth() const;
 
     QTextEngine *engine() const { return d; }
+    void setFlags(int flags);
 private:
     QTextLayout(QTextEngine *e) : d(e) {}
     Q_DISABLE_COPY(QTextLayout)
@@ -234,6 +238,8 @@ private:
     QTextEngine *eng;
 };
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QTEXTLAYOUT_H
