@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,12 +49,17 @@
 #include "qtextcursor.h"
 #include "qtextlist.h"
 #include "qabstracttextdocumentlayout.h"
+#include "qtextengine_p.h"
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 // ### DOC: We ought to explain the CONCEPT of objectIndexes if
 // relevant to the public API
 /*!
     \class QTextObject
+    \reentrant
+
     \brief The QTextObject class is a base class for different kinds
     of objects that can group parts of a QTextDocument together.
 
@@ -142,7 +167,7 @@ int QTextObject::objectIndex() const
 */
 QTextDocument *QTextObject::document() const
 {
-    return qobject_cast<QTextDocument *>(parent());
+    return static_cast<QTextDocument *>(parent());
 }
 
 /*!
@@ -150,11 +175,13 @@ QTextDocument *QTextObject::document() const
 */
 QTextDocumentPrivate *QTextObject::docHandle() const
 {
-    return qobject_cast<const QTextDocument *>(parent())->docHandle();
+    return static_cast<const QTextDocument *>(parent())->docHandle();
 }
 
 /*!
     \class QTextBlockGroup
+    \reentrant
+
     \brief The QTextBlockGroup class provides a container for text blocks within
     a QTextDocument.
 
@@ -179,6 +206,14 @@ QTextDocumentPrivate *QTextObject::docHandle() const
     \sa QTextBlock QTextDocument
 */
 
+void QTextBlockGroupPrivate::markBlocksDirty()
+{
+    for (int i = 0; i < blocks.count(); ++i) {
+        const QTextBlock &block = blocks.at(i);
+        pieceTable->documentChange(block.position(), block.length());
+    }
+}
+
 /*!
     \fn QTextBlockGroup::QTextBlockGroup(QTextDocument *document)
 
@@ -220,6 +255,7 @@ void QTextBlockGroup::blockInserted(const QTextBlock &block)
     Q_D(QTextBlockGroup);
     QTextBlockGroupPrivate::BlockList::Iterator it = qLowerBound(d->blocks.begin(), d->blocks.end(), block);
     d->blocks.insert(it, block);
+    d->markBlocksDirty();
 }
 
 // ### DOC: Shouldn't this be removeBlock()?
@@ -231,6 +267,7 @@ void QTextBlockGroup::blockRemoved(const QTextBlock &block)
 {
     Q_D(QTextBlockGroup);
     d->blocks.removeAll(block);
+    d->markBlocksDirty();
     if (d->blocks.isEmpty()) {
         document()->docHandle()->deleteObject(this);
         return;
@@ -266,6 +303,8 @@ QTextFrameLayoutData::~QTextFrameLayoutData()
 
 /*!
     \class QTextFrame
+    \reentrant
+
     \brief The QTextFrame class represents a frame in a QTextDocument.
 
     \ingroup text
@@ -567,7 +606,9 @@ void QTextFramePrivate::remove_me()
 
 /*!
     \class QTextFrame::iterator
-    \brief The QTextFrame::iterator class provides an iterator for reading
+    \reentrant
+
+    \brief The iterator class provides an iterator for reading
     the contents of a QTextFrame.
 
     \ingroup text
@@ -703,17 +744,19 @@ QTextFrame::iterator &QTextFrame::iterator::operator++()
         if (cb == e)
             return *this;
 
-        int pos = map.position(cb);
-        // check if we entered a frame
-        QTextDocumentPrivate::FragmentIterator frag = priv->find(pos-1);
-        if (priv->buffer().at(frag->stringPosition) != QChar::ParagraphSeparator) {
-            QTextFrame *nf = qobject_cast<QTextFrame *>(priv->objectForFormat(frag->format));
-            if (nf) {
-                if (priv->buffer().at(frag->stringPosition) == QTextBeginningOfFrame && nf != f) {
-                    cf = nf;
-                    cb = 0;
-                } else {
-                    Q_ASSERT(priv->buffer().at(frag->stringPosition) != QTextEndOfFrame);
+        if (!f->d_func()->childFrames.isEmpty()) {
+            int pos = map.position(cb);
+            // check if we entered a frame
+            QTextDocumentPrivate::FragmentIterator frag = priv->find(pos-1);
+            if (priv->buffer().at(frag->stringPosition) != QChar::ParagraphSeparator) {
+                QTextFrame *nf = qobject_cast<QTextFrame *>(priv->objectForFormat(frag->format));
+                if (nf) {
+                    if (priv->buffer().at(frag->stringPosition) == QTextBeginningOfFrame && nf != f) {
+                        cf = nf;
+                        cb = 0;
+                    } else {
+                        Q_ASSERT(priv->buffer().at(frag->stringPosition) != QTextEndOfFrame);
+                    }
                 }
             }
         }
@@ -763,6 +806,8 @@ QTextFrame::iterator &QTextFrame::iterator::operator--()
 
 /*!
     \class QTextBlockUserData
+    \reentrant
+
     \brief The QTextBlockUserData class is used to associate custom data with blocks of text.
     \since 4.1
 
@@ -791,6 +836,8 @@ QTextBlockUserData::~QTextBlockUserData()
 
 /*!
     \class QTextBlock qtextblock.h
+    \reentrant
+
     \brief The QTextBlock class provides a container for text fragments in a
     QTextDocument.
 
@@ -822,8 +869,8 @@ QTextBlockUserData::~QTextBlockUserData()
     by calling isValid().
 
     QTextBlock provides comparison operators to make it easier to work with
-    blocks: operator==() compares two block for equality, operator!=()
-    compares two blocks for inequality, and operator<() determines whether
+    blocks: \l operator==() compares two block for equality, \l operator!=()
+    compares two blocks for inequality, and \l operator<() determines whether
     a block precedes another in the same document.
 
     \img qtextblock-sequence.png
@@ -884,6 +931,8 @@ QTextBlockUserData::~QTextBlockUserData()
 
 /*!
     \class QTextBlock::iterator
+    \reentrant
+
     \brief The QTextBlock::iterator class provides an iterator for reading
     the contents of a QTextBlock.
 
@@ -896,11 +945,8 @@ QTextBlockUserData::~QTextBlockUserData()
     An iterator can be constructed and used to access the fragments within
     a text block in the following way:
 
-    \quotefromfile snippets/textblock-fragments/xmlwriter.cpp
-    \skipto QTextBlock::iterator
-    \printuntil processFragment
-    \skipuntil }
-    \printline }
+    \snippet doc/src/snippets/textblock-fragments/xmlwriter.cpp 4
+    \snippet doc/src/snippets/textblock-fragments/xmlwriter.cpp 7
 
     \sa QTextFragment
 */
@@ -978,6 +1024,9 @@ int QTextBlock::position() const
 /*!
     Returns the length of the block in characters.
 
+    \note The length returned includes all formatting characters,
+    for example, newline.
+
     \sa text() charFormat() blockFormat()
  */
 int QTextBlock::length() const
@@ -1005,6 +1054,12 @@ bool QTextBlock::contains(int position) const
 /*!
     Returns the QTextLayout that is used to lay out and display the
     block's contents.
+
+    Note that the returned QTextLayout object can only be modified from the
+    documentChanged implementation of a QAbstractTextDocumentLayout subclass.
+    Any changes applied from the outside cause undefined behavior.
+
+    \sa clearLayout()
  */
 QTextLayout *QTextBlock::layout() const
 {
@@ -1017,6 +1072,23 @@ QTextLayout *QTextBlock::layout() const
     return b->layout;
 }
 
+/*!
+    \since 4.4
+    Clears the QTextLayout that is used to lay out and display the
+    block's contents.
+
+    \sa layout()
+ */
+void QTextBlock::clearLayout()
+{
+    if (!p || !n)
+        return;
+
+    const QTextBlockData *b = p->blockMap().fragment(n);
+    if (b->layout)
+        b->layout->clearLayout();
+}
+
 /*!
     Returns the QTextBlockFormat that describes block-specific properties.
 
@@ -1046,8 +1118,8 @@ int QTextBlock::blockFormatIndex() const
 
 /*!
     Returns the QTextCharFormat that describes the block's character
-    format. This is mainly used to draw block-specific additions such
-    as e.g. list markers.
+    format. The block's character format is used when inserting text into
+    an empty block.
 
     \sa blockFormat()
  */
@@ -1168,6 +1240,8 @@ void QTextBlock::setUserData(QTextBlockUserData *data)
         return;
 
     const QTextBlockData *b = p->blockMap().fragment(n);
+    if (data != b->userData)
+        delete b->userData;
     b->userData = data;
 }
 
@@ -1200,6 +1274,86 @@ void QTextBlock::setUserState(int state)
     b->userState = state;
 }
 
+/*!
+    \since 4.4
+
+    Returns the blocks revision.
+
+    \sa setRevision(), QTextDocument::revision()
+*/
+int QTextBlock::revision() const
+{
+    if (!p || !n)
+        return -1;
+
+    const QTextBlockData *b = p->blockMap().fragment(n);
+    return b->revision;
+}
+
+/*!
+    \since 4.4
+
+    Sets a blocks revision to \a rev.
+
+    \sa revision(), QTextDocument::revision()
+*/
+void QTextBlock::setRevision(int rev)
+{
+    if (!p || !n)
+        return;
+
+    const QTextBlockData *b = p->blockMap().fragment(n);
+    b->revision = rev;
+}
+
+/*!
+    \since 4.4
+
+    Returns true if the block is visible; otherwise returns false.
+
+    \sa setVisible()
+*/
+bool QTextBlock::isVisible() const
+{
+    if (!p || !n)
+        return true;
+
+    const QTextBlockData *b = p->blockMap().fragment(n);
+    return !b->hidden;
+}
+
+/*!
+    \since 4.4
+
+    Sets the block's visibility to \a visible.
+
+    \sa isVisible()
+*/
+void QTextBlock::setVisible(bool visible)
+{
+    if (!p || !n)
+        return;
+
+    const QTextBlockData *b = p->blockMap().fragment(n);
+    b->hidden = !visible;
+}
+
+
+/*!
+\since 4.4
+
+    Returns the number of this block, or -1 if the block is invalid.
+
+    \sa QTextCursor::blockNumber()
+
+*/
+int QTextBlock::blockNumber() const
+{
+    if (!p || !n)
+        return -1;
+    return p->blockMap().index(n);
+}
+
 /*!
     Returns a text block iterator pointing to the beginning of the
     text block.
@@ -1327,6 +1481,8 @@ QTextBlock::iterator &QTextBlock::iterator::operator--()
 
 /*!
     \class QTextFragment
+    \reentrant
+
     \brief The QTextFragment class holds a piece of text in a
     QTextDocument with a single QTextCharFormat.
 
@@ -1502,3 +1658,5 @@ QString QTextFragment::text() const
     }
     return result;
 }
+
+QT_END_NAMESPACE
