@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,9 +45,14 @@
 #define QTEXTOPTION_H
 
 #include <QtCore/qnamespace.h>
+#include <QtCore/qchar.h>
+#include <QtCore/qmetatype.h>
+
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 template <typename T> class QList;
@@ -36,6 +61,31 @@ struct QTextOptionPrivate;
 class Q_GUI_EXPORT QTextOption
 {
 public:
+    enum TabType {
+        LeftTab,
+        RightTab,
+        CenterTab,
+        DelimiterTab
+    };
+
+    struct Q_GUI_EXPORT Tab {
+        inline Tab() : position(80), type(QTextOption::LeftTab) { }
+
+        inline bool operator==(const Tab &other) const {
+            return type == other.type
+                   && qFuzzyCompare(position, other.position)
+                   && delimiter == other.delimiter;
+        }
+
+        inline bool operator!=(const Tab &other) const {
+            return !operator==(other);
+        }
+
+        qreal position;
+        TabType type;
+        QChar delimiter;
+    };
+
     QTextOption();
     QTextOption(Qt::Alignment alignment);
     ~QTextOption();
@@ -72,6 +122,9 @@ public:
     void setTabArray(QList<qreal> tabStops);
     QList<qreal> tabArray() const;
 
+    void setTabs(QList<Tab> tabStops);
+    QList<Tab> tabs() const;
+
     void setUseDesignMetrics(bool b) { design = b; }
     bool useDesignMetrics() const { return design; }
 
@@ -97,6 +150,10 @@ inline void QTextOption::setFlags(Flags aflags)
 inline void QTextOption::setTabStop(qreal atabStop)
 { tab = atabStop; }
 
+QT_END_NAMESPACE
+
+Q_DECLARE_METATYPE( QTextOption::Tab )
+
 QT_END_HEADER
 
 #endif // QTEXTOPTION_H
