@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -23,7 +43,7 @@
 
 /*!
     \class QCompleter
-    \brief The QCompleter class provides completions based on a item model.
+    \brief The QCompleter class provides completions based on an item model.
     \since 4.2
 
     You can use QCompleter to provide auto completions in any Qt
@@ -42,25 +62,12 @@
     For example, here's how to provide auto completions from a simple
     word list in a QLineEdit:
 
-    \code
-        QStringList wordList;
-        wordList << "alpha" << "omega" << "omicron" << "zeta";
-
-        QLineEdit *lineEdit = new QLineEdit(this);
-
-        QCompleter *completer = new QCompleter(wordList, this);
-        completer->setCaseSensitivity(Qt::CaseInsensitive);
-        lineEdit->setCompleter(completer);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.util.qcompleter.cpp 0
 
     A QDirModel can be used to provide auto completion of file names.
     For example:
 
-    \code
-        QCompleter *completer = new QCompleter(this);
-        completer->setModel(new QDirModel(completer));
-        lineEdit->setCompleter(completer);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.util.qcompleter.cpp 1
 
     To set the model on which QCompleter should operate, call
     setModel(). By default, QCompleter will attempt to match the \l
@@ -93,10 +100,7 @@
     currentCompletion(). You can iterate through the list of
     completions as below:
 
-    \code
-        for (int i = 0; completer->setCurrentRow(i); i++)
-            qDebug() << completer->currentCompletion() << " is match number " << i;
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.util.qcompleter.cpp 2
 
     completionCount() returns the total number of completions for the
     current prefix. completionCount() should be avoided when possible,
@@ -152,23 +156,41 @@
 #include "QtGui/qheaderview.h"
 #include "QtGui/qdesktopwidget.h"
 
+QT_BEGIN_NAMESPACE
+
+QCompletionModel::QCompletionModel(QCompleterPrivate *c, QObject *parent)
+    : QAbstractProxyModel(*new QCompletionModelPrivate, parent),
+      c(c), engine(0), showAll(false)
+{
+    createEngine();
+}
+
+int QCompletionModel::columnCount(const QModelIndex &) const
+{
+    Q_D(const QCompletionModel);
+    return d->model->columnCount();
+}
+
 void QCompletionModel::setSourceModel(QAbstractItemModel *source)
 {
-    if (model)
-        QObject::disconnect(model, 0, this, 0);
+    bool hadModel = (sourceModel() != 0);
+
+    if (hadModel)
+        QObject::disconnect(sourceModel(), 0, this, 0);
 
     QAbstractProxyModel::setSourceModel(source);
-    model = sourceModel();
 
-    // TODO: Optimize updates in the source model
-    connect(model, SIGNAL(modelReset()), this, SLOT(invalidate()));
-    connect(model, SIGNAL(destroyed()), this, SLOT(modelDestroyed()));
-    connect(model, SIGNAL(layoutChanged()), this, SLOT(invalidate()));
-    connect(model, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(invalidate()));
-    connect(model, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(invalidate()));
-    connect(model, SIGNAL(columnsInserted(QModelIndex,int,int)), this, SLOT(invalidate()));
-    connect(model, SIGNAL(columnsRemoved(QModelIndex,int,int)), this, SLOT(invalidate()));
-    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(invalidate()));
+    if (source) {
+        // TODO: Optimize updates in the source model
+        connect(source, SIGNAL(modelReset()), this, SLOT(invalidate()));
+        connect(source, SIGNAL(destroyed()), this, SLOT(modelDestroyed()));
+        connect(source, SIGNAL(layoutChanged()), this, SLOT(invalidate()));
+        connect(source, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(rowsInserted()));
+        connect(source, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(invalidate()));
+        connect(source, SIGNAL(columnsInserted(QModelIndex,int,int)), this, SLOT(invalidate()));
+        connect(source, SIGNAL(columnsRemoved(QModelIndex,int,int)), this, SLOT(invalidate()));
+        connect(source, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(invalidate()));
+    }
 
     invalidate();
 }
@@ -197,6 +219,7 @@ void QCompletionModel::createEngine()
 
 QModelIndex QCompletionModel::mapToSource(const QModelIndex& index) const
 {
+    Q_D(const QCompletionModel);
     if (!index.isValid())
         return QModelIndex();
 
@@ -217,7 +240,7 @@ QModelIndex QCompletionModel::mapToSource(const QModelIndex& index) const
         row = index.row();
     }
 
-    return model->index(row, index.column(), parent);
+    return d->model->index(row, index.column(), parent);
 }
 
 QModelIndex QCompletionModel::mapFromSource(const QModelIndex& idx) const
@@ -289,6 +312,7 @@ QModelIndex QCompletionModel::currentIndex(bool sourceIndex) const
 
 QModelIndex QCompletionModel::index(int row, int column, const QModelIndex& parent) const
 {
+    Q_D(const QCompletionModel);
     if (row < 0 || column < 0 || column >= columnCount(parent) || parent.isValid())
         return QModelIndex();
 
@@ -303,7 +327,7 @@ QModelIndex QCompletionModel::index(int row, int column, const QModelIndex& pare
                 return QModelIndex();
         }
     } else {
-        if (row >= model->rowCount(engine->curParent))
+        if (row >= d->model->rowCount(engine->curParent))
             return QModelIndex();
     }
 
@@ -321,6 +345,7 @@ int QCompletionModel::completionCount() const
 
 int QCompletionModel::rowCount(const QModelIndex &parent) const
 {
+    Q_D(const QCompletionModel);
     if (parent.isValid())
         return 0;
 
@@ -329,7 +354,7 @@ int QCompletionModel::rowCount(const QModelIndex &parent) const
         if (engine->curParts.count() != 1  && !engine->matchCount()
             && !engine->curParent.isValid())
             return 0;
-        return model->rowCount(engine->curParent);
+        return d->model->rowCount(engine->curParent);
     }
 
     return completionCount();
@@ -340,16 +365,17 @@ void QCompletionModel::setFiltered(bool filtered)
     if (showAll == !filtered)
         return;
     showAll = !filtered;
-    reset();
+    resetModel();
 }
 
 bool QCompletionModel::hasChildren(const QModelIndex &parent) const
 {
+    Q_D(const QCompletionModel);
     if (parent.isValid())
         return false;
 
     if (showAll)
-        return model->hasChildren(mapToSource(parent));
+        return d->model->hasChildren(mapToSource(parent));
 
     if (!engine->matchCount())
         return false;
@@ -359,16 +385,22 @@ bool QCompletionModel::hasChildren(const QModelIndex &parent) const
 
 QVariant QCompletionModel::data(const QModelIndex& index, int role) const
 {
-    return model->data(mapToSource(index), role);
+    Q_D(const QCompletionModel);
+    return d->model->data(mapToSource(index), role);
 }
 
 void QCompletionModel::modelDestroyed()
 {
     QAbstractProxyModel::setSourceModel(0); // switch to static empty model
-    model = sourceModel();
     invalidate();
 }
 
+void QCompletionModel::rowsInserted()
+{
+    invalidate();
+    emit rowsAdded();
+}
+
 void QCompletionModel::invalidate()
 {
     engine->cache.clear();
@@ -377,8 +409,28 @@ void QCompletionModel::invalidate()
 
 void QCompletionModel::filter(const QStringList& parts)
 {
+    Q_D(QCompletionModel);
     engine->filter(parts);
-    reset();
+    resetModel();
+
+    if (d->model->canFetchMore(engine->curParent))
+        d->model->fetchMore(engine->curParent);
+}
+
+void QCompletionModel::resetModel()
+{
+    if (rowCount() == 0) {
+        reset();
+        return;
+    }
+
+    emit layoutAboutToBeChanged();
+    QModelIndexList piList = persistentIndexList();
+    QModelIndexList empty;
+    for (int i = 0; i < piList.size(); i++)
+        empty.append(QModelIndex());
+    changePersistentIndexList(piList, empty);
+    emit layoutChanged();
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -394,6 +446,9 @@ void QCompletionEngine::filter(const QStringList& parts)
     curMatch = QMatchData();
     historyMatch = filterHistory();
 
+    if (!model)
+        return;
+
     QModelIndex parent;
     for (int i = 0; i < curParts.count() - 1; i++) {
         QString part = curParts[i];
@@ -415,9 +470,9 @@ void QCompletionEngine::filter(const QStringList& parts)
 
 QMatchData QCompletionEngine::filterHistory()
 {
-    if (curParts.count() <= 1 || c->proxy->showAll)
+    QAbstractItemModel *source = c->proxy->sourceModel();
+    if (curParts.count() <= 1 || c->proxy->showAll || !source)
         return QMatchData();
-    QAbstractItemModel *source = c->proxy->model;
     bool dirModel = false;
 #ifndef QT_NO_DIRMODEL
     dirModel = (qobject_cast<QDirModel *>(source) != 0);
@@ -429,8 +484,8 @@ QMatchData QCompletionEngine::filterHistory()
     for (int i = 0; i < source->rowCount(); i++) {
         QString str = source->index(i, c->column).data().toString();
         if (str.startsWith(c->prefix, c->cs)
-#ifndef Q_OS_WIN
-            && (!dirModel || str != QDir::separator())
+#if !defined(Q_OS_WIN) || defined(Q_OS_WINCE)
+            && (!dirModel || QDir::toNativeSeparators(str) != QDir::separator())
 #endif
             )
             m.indices.append(i);
@@ -476,7 +531,7 @@ void QCompletionEngine::saveInCache(QString part, const QModelIndex& parent, con
     cost = cost + m.indices.cost() - old.indices.cost();
     if (cost * sizeof(int) > 1024 * 1024) {
         QMap<QModelIndex, CacheItem>::iterator it1 ;
-        for (it1 = cache.begin(); it1 != cache.end(); it1++) {
+        for (it1 = cache.begin(); it1 != cache.end(); ++it1) {
             CacheItem& ci = it1.value();
             int sz = ci.count()/2;
             QMap<QString, QMatchData>::iterator it2 = ci.begin();
@@ -495,7 +550,7 @@ void QCompletionEngine::saveInCache(QString part, const QModelIndex& parent, con
 }
 
 ///////////////////////////////////////////////////////////////////////////////////
-QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& parent)
+QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& parent, Qt::SortOrder order)
 {
     const QAbstractItemModel *model = c->proxy->sourceModel();
 
@@ -513,7 +568,11 @@ QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& pare
     for(CacheItem::const_iterator it1 = it; it1-- != map.constBegin();) {
         const QMatchData& value = it1.value();
         if (value.isValid()) {
-            from = value.indices.last() + 1;
+            if (order == Qt::AscendingOrder) {
+                from = value.indices.last() + 1;
+            } else {
+                to = value.indices.first() - 1;
+            }
             break;
         }
     }
@@ -522,7 +581,11 @@ QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& pare
     for(CacheItem::const_iterator it2 = it; it2 != map.constEnd(); ++it2) {
         const QMatchData& value = it2.value();
         if (value.isValid() && !it2.key().startsWith(part)) {
-            to = value.indices[0] - 1;
+            if (order == Qt::AscendingOrder) {
+                to = value.indices.first() - 1;
+            } else {
+                from = value.indices.first() + 1;
+            }
             break;
         }
     }
@@ -530,6 +593,18 @@ QIndexMapper QSortedModelEngine::indexHint(QString part, const QModelIndex& pare
     return QIndexMapper(from, to);
 }
 
+Qt::SortOrder QSortedModelEngine::sortOrder(const QModelIndex &parent) const
+{
+    const QAbstractItemModel *model = c->proxy->sourceModel();
+
+    int rowCount = model->rowCount(parent);
+    if (rowCount < 2)
+        return Qt::AscendingOrder;
+    QString first = model->data(model->index(0, c->column, parent), c->role).toString();
+    QString last = model->data(model->index(rowCount - 1, c->column, parent), c->role).toString();
+    return QString::compare(first, last, c->cs) <= 0 ? Qt::AscendingOrder : Qt::DescendingOrder;
+}
+
 QMatchData QSortedModelEngine::filter(const QString& part, const QModelIndex& parent, int)
 {
     const QAbstractItemModel *model = c->proxy->sourceModel();
@@ -539,13 +614,14 @@ QMatchData QSortedModelEngine::filter(const QString& part, const QModelIndex& pa
         return hint;
 
     QIndexMapper indices;
+    Qt::SortOrder order = sortOrder(parent);
 
     if (matchHint(part, parent, &hint)) {
         if (!hint.isValid())
             return QMatchData();
         indices = hint.indices;
     } else {
-        indices = indexHint(part, parent);
+        indices = indexHint(part, parent, order);
     }
 
     // binary search the model within 'indices' for 'part' under 'parent'
@@ -560,42 +636,58 @@ QMatchData QSortedModelEngine::filter(const QString& part, const QModelIndex& pa
         probe = (high + low) / 2;
         probeIndex = model->index(probe, c->column, parent);
         probeData = model->data(probeIndex, c->role).toString();
-        if (QString::compare(probeData, part, c->cs) >= 0)
+        const int cmp = QString::compare(probeData, part, c->cs);
+        if ((order == Qt::AscendingOrder && cmp >= 0)
+            || (order == Qt::DescendingOrder && cmp < 0)) {
             high = probe;
-        else
+        } else {
             low = probe;
+        }
     }
 
-    if (low == indices.to()) { // not found
+    if ((order == Qt::AscendingOrder && low == indices.to())
+        || (order == Qt::DescendingOrder && high == indices.from())) { // not found
         saveInCache(part, parent, QMatchData());
         return QMatchData();
     }
 
-    probeIndex = model->index(low + 1, c->column, parent);
+    probeIndex = model->index(order == Qt::AscendingOrder ? low+1 : high-1, c->column, parent);
     probeData = model->data(probeIndex, c->role).toString();
     if (!probeData.startsWith(part, c->cs)) {
         saveInCache(part, parent, QMatchData());
         return QMatchData();
     }
 
-    int emi = QString::compare(probeData, part, c->cs) == 0 ? low+1 : -1;
+    const bool exactMatch = QString::compare(probeData, part, c->cs) == 0;
+    int emi =  exactMatch ? (order == Qt::AscendingOrder ? low+1 : high-1) : -1;
 
-    int from = low + 1;
-    high = indices.to() + 1;
-    low = from;
+    int from = 0;
+    int to = 0;
+    if (order == Qt::AscendingOrder) {
+        from = low + 1;
+        high = indices.to() + 1;
+        low = from;
+    } else {
+        to = high - 1;
+        low = indices.from() - 1;
+        high = to;
+    }
 
     while (high - low > 1)
     {
         probe = (high + low) / 2;
         probeIndex = model->index(probe, c->column, parent);
         probeData = model->data(probeIndex, c->role).toString();
-        if (probeData.startsWith(part, c->cs))
+        const bool startsWith = probeData.startsWith(part, c->cs);
+        if ((order == Qt::AscendingOrder && startsWith)
+            || (order == Qt::DescendingOrder && !startsWith)) {
             low = probe;
-        else
+        } else {
             high = probe;
+        }
     }
 
-    QMatchData m(QIndexMapper(from, high - 1), emi, false);
+    QMatchData m(order == Qt::AscendingOrder ? QIndexMapper(from, high - 1) : QIndexMapper(low+1, to), emi, false);
     saveInCache(part, parent, m);
     return m;
 }
@@ -612,7 +704,7 @@ int QUnsortedModelEngine::buildIndices(const QString& str, const QModelIndex& pa
     for (i = 0; i < indices.count() && count != n; ++i) {
         QModelIndex idx = model->index(indices[i], c->column, parent);
         QString data = model->data(idx, c->role).toString();
-        if (!data.startsWith(str, c->cs))
+        if (!data.startsWith(str, c->cs) || !(model->flags(idx) & Qt::ItemIsSelectable))
             continue;
         m->indices.append(indices[i]);
         ++count;
@@ -682,7 +774,7 @@ QMatchData QUnsortedModelEngine::filter(const QString& part, const QModelIndex&
 ///////////////////////////////////////////////////////////////////////////////
 QCompleterPrivate::QCompleterPrivate()
 : widget(0), proxy(0), popup(0), cs(Qt::CaseSensitive), role(Qt::EditRole), column(0),
-  sorting(QCompleter::UnsortedModel), eatFocusOut(true)
+  sorting(QCompleter::UnsortedModel), wrap(true), eatFocusOut(true)
 {
 }
 
@@ -690,23 +782,20 @@ void QCompleterPrivate::init(QAbstractItemModel *m)
 {
     Q_Q(QCompleter);
     proxy = new QCompletionModel(this, q);
+    QObject::connect(proxy, SIGNAL(rowsAdded()), q, SLOT(_q_autoResizePopup()));
     q->setModel(m);
 #ifdef QT_NO_LISTVIEW
     q->setCompletionMode(QCompleter::InlineCompletion);
 #else
-    QListView *listView = new QListView;
-    listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
-    listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    listView->setSelectionBehavior(QAbstractItemView::SelectRows);
-    listView->setSelectionMode(QAbstractItemView::SingleSelection);
-    listView->setModelColumn(column);
-    q->setPopup(listView);
     q->setCompletionMode(QCompleter::PopupCompletion);
 #endif // QT_NO_LISTVIEW
 }
 
 void QCompleterPrivate::setCurrentIndex(QModelIndex index, bool select)
 {
+    Q_Q(QCompleter);
+    if (!q->popup())
+        return;
     if (!select) {
         popup->selectionModel()->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
     } else {
@@ -761,6 +850,13 @@ void QCompleterPrivate::_q_complete(QModelIndex index, bool highlighted)
     }
 }
 
+void QCompleterPrivate::_q_autoResizePopup()
+{
+    if (!popup || !popup->isVisible())
+        return;
+    showPopup(popupRect);
+}
+
 void QCompleterPrivate::showPopup(const QRect& rect)
 {
     const QRect screen = QApplication::desktop()->availableGeometry(widget);
@@ -849,10 +945,15 @@ QCompleter::~QCompleter()
 void QCompleter::setWidget(QWidget *widget)
 {
     Q_D(QCompleter);
+    if (d->widget)
+        d->widget->removeEventFilter(this);
     d->widget = widget;
-    if (d->popup)
+    if (d->widget)
+        d->widget->installEventFilter(this);
+    if (d->popup) {
+        d->popup->hide();
         d->popup->setFocusProxy(d->widget);
-    setCompletionMode(d->mode); // will install event filter depending on mode
+    }
 }
 
 /*!
@@ -880,7 +981,7 @@ QWidget *QCompleter::widget() const
 void QCompleter::setModel(QAbstractItemModel *model)
 {
     Q_D(QCompleter);
-    QAbstractItemModel *oldModel = d->proxy->model;
+    QAbstractItemModel *oldModel = d->proxy->sourceModel();
     d->proxy->setSourceModel(model);
     if (d->popup)
         setPopup(d->popup); // set the model and make new connections
@@ -888,7 +989,7 @@ void QCompleter::setModel(QAbstractItemModel *model)
         delete oldModel;
 #ifndef QT_NO_DIRMODEL
     if (qobject_cast<QDirModel *>(model)) {
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
         setCaseSensitivity(Qt::CaseInsensitive);
 #else
         setCaseSensitivity(Qt::CaseSensitive);
@@ -929,18 +1030,20 @@ QAbstractItemModel *QCompleter::model() const
 void QCompleter::setCompletionMode(QCompleter::CompletionMode mode)
 {
     Q_D(QCompleter);
-
     d->mode = mode;
     d->proxy->setFiltered(mode != QCompleter::UnfilteredPopupCompletion);
 
     if (mode == QCompleter::InlineCompletion) {
         if (d->widget)
             d->widget->removeEventFilter(this);
-        return;
+        if (d->popup) {
+            d->popup->deleteLater();
+            d->popup = 0;
+        }
+    } else {
+        if (d->widget)
+            d->widget->installEventFilter(this);
     }
-
-    if (d->widget)
-        d->widget->installEventFilter(this);
 }
 
 QCompleter::CompletionMode QCompleter::completionMode() const
@@ -1000,6 +1103,18 @@ void QCompleter::setPopup(QAbstractItemView *popup)
 QAbstractItemView *QCompleter::popup() const
 {
     Q_D(const QCompleter);
+#ifndef QT_NO_LISTVIEW
+    if (!d->popup && completionMode() != QCompleter::InlineCompletion) {
+        QListView *listView = new QListView;
+        listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
+        listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        listView->setSelectionBehavior(QAbstractItemView::SelectRows);
+        listView->setSelectionMode(QAbstractItemView::SingleSelection);
+        listView->setModelColumn(d->column);
+        QCompleter *that = const_cast<QCompleter*>(this);
+        that->setPopup(listView);
+    }
+#endif // QT_NO_LISTVIEW
     return d->popup;
 }
 
@@ -1057,7 +1172,8 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
                 d->setCurrentIndex(lastIndex);
                 return true;
             } else if (curIndex.row() == 0) {
-                d->setCurrentIndex(QModelIndex());
+                if (d->wrap)
+                    d->setCurrentIndex(QModelIndex());
                 return true;
             }
             return false;
@@ -1068,7 +1184,8 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
                 d->setCurrentIndex(firstIndex);
                 return true;
             } else if (curIndex.row() == d->proxy->rowCount() - 1) {
-                d->setCurrentIndex(QModelIndex());
+                if (d->wrap)
+                    d->setCurrentIndex(QModelIndex());
                 return true;
             }
             return false;
@@ -1085,13 +1202,23 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
         d->eatFocusOut = true;
         if (!d->widget || e->isAccepted() || !d->popup->isVisible()) {
             // widget lost focus, hide the popup
-            if (d->widget && !d->widget->hasFocus())
+            if (d->widget && (!d->widget->hasFocus()
+#ifdef QT_KEYPAD_NAVIGATION
+                || (QApplication::keypadNavigationEnabled() && !d->widget->hasEditFocus())
+#endif
+                ))
                 d->popup->hide();
-            return true;
+            if (e->isAccepted())
+                return true;
         }
 
         // default implementation for keys not handled by the widget when popup is open
         switch (key) {
+#ifdef QT_KEYPAD_NAVIGATION
+        case Qt::Key_Select:
+            if (!QApplication::keypadNavigationEnabled())
+                break;
+#endif
         case Qt::Key_Return:
         case Qt::Key_Enter:
         case Qt::Key_Tab:
@@ -1117,16 +1244,58 @@ bool QCompleter::eventFilter(QObject *o, QEvent *e)
         return true;
     }
 
-    case QEvent::MouseButtonPress:
+#ifdef QT_KEYPAD_NAVIGATION
+    case QEvent::KeyRelease: {
+        QKeyEvent *ke = static_cast<QKeyEvent *>(e);
+        if (QApplication::keypadNavigationEnabled() && ke->key() == Qt::Key_Back) {
+            // Send the event to the 'widget'. This is what we did for KeyPress, so we need
+            // to do the same for KeyRelease, in case the widget's KeyPress event set
+            // up something (such as a timer) that is relying on also receiving the
+            // key release. I see this as a bug in Qt, and should really set it up for all
+            // the affected keys. However, it is difficult to tell how this will affect
+            // existing code, and I can't test for every combination!
+            d->eatFocusOut = false;
+            static_cast<QObject *>(d->widget)->event(ke);
+            d->eatFocusOut = true;
+        }
+        break;
+    }
+#endif
+
+    case QEvent::MouseButtonPress: {
+#ifdef QT_KEYPAD_NAVIGATION
+        if (QApplication::keypadNavigationEnabled()) {
+            // if we've clicked in the widget (or its descendant), let it handle the click
+            QWidget *source = qobject_cast<QWidget *>(o);
+            if (source) {
+                QPoint pos = source->mapToGlobal((static_cast<QMouseEvent *>(e))->pos());
+                QWidget *target = QApplication::widgetAt(pos);
+                if (target && (d->widget->isAncestorOf(target) ||
+                    target == d->widget)) {
+                    d->eatFocusOut = false;
+                    static_cast<QObject *>(target)->event(e);
+                    d->eatFocusOut = true;
+                    return true;
+                }
+            }
+        }
+#endif
         if (!d->popup->underMouse()) {
             d->popup->hide();
             return true;
         }
+        }
         return false;
 
+    case QEvent::InputMethod:
+    case QEvent::ShortcutOverride:
+        QApplication::sendEvent(d->widget, e);
+        break;
+
     default:
         return false;
     }
+    return false;
 }
 
 /*!
@@ -1152,14 +1321,17 @@ void QCompleter::complete(const QRect& rect)
     Q_ASSERT(d->widget != 0);
     if ((d->mode == QCompleter::PopupCompletion && !idx.isValid())
         || (d->mode == QCompleter::UnfilteredPopupCompletion && d->proxy->rowCount() == 0)) {
-        d->popup->hide(); // no suggestion, hide
+        if (d->popup)
+            d->popup->hide(); // no suggestion, hide
         return;
     }
 
+    popup();
     if (d->mode == QCompleter::UnfilteredPopupCompletion)
         d->setCurrentIndex(idx, false);
 
     d->showPopup(rect);
+    d->popupRect = rect;
 }
 
 /*!
@@ -1224,6 +1396,9 @@ int QCompleter::completionCount() const
     significant performance improvements because the completer object can
     then use a binary search algorithm instead of linear search algorithm.
 
+    The sort order (i.e ascending or descending order) of the model is determined
+    dynamically by inspecting the contents of the model.
+
     \bold{Note:} The performance improvements described above cannot take place
     when the completer's \l caseSensitivity is different to the case sensitivity
     used by the model's when sorting.
@@ -1299,6 +1474,27 @@ int QCompleter::completionRole() const
     return d->role;
 }
 
+/*!
+    \property QCompleter::wrapAround
+    \brief the completions wrap around when navigating through items
+    \since 4.3
+
+    The default is true.
+*/
+void QCompleter::setWrapAround(bool wrap)
+{
+    Q_D(QCompleter);
+    if (d->wrap == wrap)
+        return;
+    d->wrap = wrap;
+}
+
+bool QCompleter::wrapAround() const
+{
+    Q_D(const QCompleter);
+    return d->wrap;
+}
+
 /*!
     \property QCompleter::caseSensitivity
     \brief the case sensitivity of the matching
@@ -1397,6 +1593,8 @@ QString QCompleter::pathFromIndex(const QModelIndex& index) const
         return QString();
 
     QAbstractItemModel *sourceModel = d->proxy->sourceModel();
+    if (!sourceModel)
+        return QString();
 #ifndef QT_NO_DIRMODEL
     QDirModel *dirModel = qobject_cast<QDirModel *>(sourceModel);
     if (!dirModel)
@@ -1412,7 +1610,7 @@ QString QCompleter::pathFromIndex(const QModelIndex& index) const
         idx = parent.sibling(parent.row(), index.column());
     } while (idx.isValid());
 
-#ifndef Q_OS_WIN
+#if !defined(Q_OS_WIN) || defined(Q_OS_WINCE)
     if (list.count() == 1) // only the separator or some other text
         return list[0];
     list[0].clear() ; // the join below will provide the separator
@@ -1435,9 +1633,9 @@ QString QCompleter::pathFromIndex(const QModelIndex& index) const
 */
 QStringList QCompleter::splitPath(const QString& path) const
 {
-    Q_D(const QCompleter);
     bool isDirModel = false;
 #ifndef QT_NO_DIRMODEL
+    Q_D(const QCompleter);
     isDirModel = qobject_cast<QDirModel *>(d->proxy->sourceModel()) != 0;
 #endif
 
@@ -1446,7 +1644,7 @@ QStringList QCompleter::splitPath(const QString& path) const
 
     QString pathCopy = QDir::toNativeSeparators(path);
     QString sep = QDir::separator();
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
     if (pathCopy == QLatin1String("\\") || pathCopy == QLatin1String("\\\\"))
         return QStringList(pathCopy);
     QString doubleSlash(QLatin1String("\\\\"));
@@ -1459,12 +1657,12 @@ QStringList QCompleter::splitPath(const QString& path) const
     QRegExp re(QLatin1String("[") + QRegExp::escape(sep) + QLatin1String("]"));
     QStringList parts = pathCopy.split(re);
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
     if (!doubleSlash.isEmpty())
         parts[0].prepend(doubleSlash);
 #else
-    if (path[0] == sep[0]) // readd the "/" at the beginning as the split removed it
-        parts[0] = sep[0];
+    if (pathCopy[0] == sep[0]) // readd the "/" at the beginning as the split removed it
+        parts[0] = QDir::fromNativeSeparators(QString(sep[0]));
 #endif
 
     return parts;
@@ -1504,6 +1702,8 @@ QStringList QCompleter::splitPath(const QString& path) const
     set to QCOmpleter::InlineCompletion. The item's \a text is given.
 */
 
+QT_END_NAMESPACE
+
 #include "moc_qcompleter.cpp"
 
 #endif // QT_NO_COMPLETER
