@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -48,6 +53,9 @@
 #include "qtimer.h"
 #include "qsystemtrayicon_p.h"
 
+#ifndef QT_NO_SYSTEMTRAYICON
+QT_BEGIN_NAMESPACE
+
 Window QSystemTrayIconSys::sysTrayWindow = None;
 QList<QSystemTrayIconSys *> QSystemTrayIconSys::trayIcons;
 QCoreApplication::EventFilter QSystemTrayIconSys::oldEventFilter = 0;
@@ -111,7 +119,6 @@ QSystemTrayIconSys::QSystemTrayIconSys(QSystemTrayIcon *q)
 {
     setAttribute(Qt::WA_AlwaysShowToolTips);
     setAttribute(Qt::WA_QuitOnClose, false);
-    setAttribute(Qt::WA_PaintOnScreen, true);
     setAttribute(Qt::WA_NoSystemBackground, true);
     static bool eventFilterAdded = false;
     Display *display = QX11Info::display();
@@ -177,7 +184,7 @@ void QSystemTrayIconSys::addToTray()
 
 void QSystemTrayIconSys::updateIcon()
 {
-    XClearArea(QX11Info::display(), winId(), 0, 0, width(), height(), True);
+    update();
 }
 
 void QSystemTrayIconSys::resizeEvent(QResizeEvent *re)
@@ -189,6 +196,7 @@ void QSystemTrayIconSys::resizeEvent(QResizeEvent *re)
 void QSystemTrayIconSys::paintEvent(QPaintEvent *)
 {
     QPainter p(this);
+    p.drawPixmap(0, 0, background);
     q->icon().paint(&p, rect());
 }
 
@@ -229,6 +237,12 @@ bool QSystemTrayIconSys::x11Event(XEvent *event)
 {
     if (event->type == ReparentNotify)
         show();
+    else if (event->type == ConfigureNotify || event->type == Expose) {
+        XClearArea(QX11Info::display(), winId(), 0, 0, width(), height(), False);
+        qApp->syncX();
+        background = QPixmap::grabWindow(winId());
+        update();
+    }
     return QWidget::x11Event(event);
 }
 
@@ -293,3 +307,6 @@ void QSystemTrayIconPrivate::showMessage_sys(const QString &message, const QStri
                              QPoint(g.x() + sys->width()/2, g.y() + sys->height()/2),
                              msecs);
 }
+
+QT_END_NAMESPACE
+#endif //QT_NO_SYSTEMTRAYICON
