@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -47,6 +67,8 @@
 #include "private/qwidget_p.h"
 #include "private/qdatetime_p.h"
 
+QT_BEGIN_NAMESPACE
+
 QVariant operator+(const QVariant &arg1, const QVariant &arg2);
 QVariant operator-(const QVariant &arg1, const QVariant &arg2);
 QVariant operator*(const QVariant &arg1, double multiplier);
@@ -81,16 +103,13 @@ public:
     void updateState(bool up);
     QString stripped(const QString &text, int *pos = 0) const;
     bool specialValue() const;
-    QVariant getZeroVariant() const;
-    void setRange(const QVariant &min, const QVariant &max);
+    virtual QVariant getZeroVariant() const;
+    virtual void setRange(const QVariant &min, const QVariant &max);
     void setValue(const QVariant &val, EmitPolicy ep, bool updateEdit = true);
     virtual QVariant bound(const QVariant &val, const QVariant &old = QVariant(), int steps = 0) const;
     QLineEdit *lineEdit();
-    void updateButtons();
     virtual void updateEdit();
 
-    virtual QStyleOptionSpinBox getStyleOption() const;
-
     virtual void emitSignals(EmitPolicy ep, const QVariant &old);
     virtual void interpret(EmitPolicy ep);
     virtual QString textFromValue(const QVariant &n) const;
@@ -99,12 +118,14 @@ public:
     void _q_editorTextChanged(const QString &);
     virtual void _q_editorCursorPositionChanged(int oldpos, int newpos);
 
-    QStyle::SubControl newHoverControl(const QPoint &pos);
+    virtual QStyle::SubControl newHoverControl(const QPoint &pos);
     bool updateHoverControl(const QPoint &pos);
 
-    void clearCache() const;
+    virtual void clearCache() const;
+    virtual void updateEditFieldGeometry();
 
     static int variantCompare(const QVariant &arg1, const QVariant &arg2);
+    static QVariant variantBound(const QVariant &min, const QVariant &value, const QVariant &max);
 
     QLineEdit *edit;
     QString prefix, suffix, specialValueText;
@@ -115,6 +136,7 @@ public:
     mutable QString cachedText;
     mutable QVariant cachedValue;
     mutable QValidator::State cachedState;
+    mutable QSize cachedSizeHint, cachedMinimumSizeHint;
     uint pendingEmit : 1;
     uint spindownEnabled : 1;
     uint spinupEnabled : 1;
@@ -122,6 +144,11 @@ public:
     uint wrapping : 1;
     uint ignoreCursorPositionChanged : 1;
     uint frame : 1;
+    uint accelerate : 1;
+    uint keyboardTracking : 1;
+    uint cleared : 1;
+    QAbstractSpinBox::CorrectionMode correctionMode;
+    int acceleration;
     QStyle::SubControl hoverControl;
     QRect hoverRect;
     QAbstractSpinBox::ButtonSymbols buttonSymbols;
@@ -139,6 +166,8 @@ private:
     QAbstractSpinBoxPrivate *dptr;
 };
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SPINBOX
 
 #endif // QABSTRACTSPINBOX_P_H
