@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -62,6 +62,8 @@
 #include "qaccessible.h"
 #endif
 
+QT_BEGIN_NAMESPACE
+
 class QDialPrivate : public QAbstractSliderPrivate
 {
     Q_DECLARE_PUBLIC(QDial)
@@ -71,7 +73,7 @@ public:
         wrapping = false;
         tracking = true;
         doNotEmit = false;
-        target = 3.7;
+        target = qreal(3.7);
     }
 
     qreal target;
@@ -208,6 +210,10 @@ int QDialPrivate::valueFromPoint(const QPoint &p) const
     \l pageStep, and the \key Home and \key End keys set the value to the
     defined \l minimum and \l maximum values.
 
+    If you are using the mouse wheel to adjust the dial, the increment value
+    is determined by the lesser value of \l{QApplication::wheelScrollLines()}
+    {wheelScrollLines} multipled by \l singleStep, and \l pageStep.
+
     \table
     \row \o \inlineimage plastique-dial.png Screenshot of a dial in the Plastique widget style
     \o \inlineimage windowsxp-dial.png Screenshot of a dial in the Windows XP widget style
@@ -512,5 +518,6 @@ bool QDial::event(QEvent *e)
     Use QAbstractSlider::sliderReleased() instead.
 */
 
+QT_END_NAMESPACE
 
 #endif // QT_NO_DIAL
