@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,10 +45,13 @@
 #include <QtGui/qpushbutton.h>
 #include <QtGui/qstyle.h>
 #include <QtGui/qlayout.h>
+#include <QtGui/qdialog.h>
 #include <QtGui/private/qwidget_p.h>
 
 #include "qdialogbuttonbox.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QDialogButtonBox
     \since 4.2
@@ -80,16 +88,12 @@
     the buttons (or button texts) yourself and add them to the button box,
     specifying their role.
 
-    \quotefromfile dialogs/extension/finddialog.cpp
-    \skipto findButton
-    \printuntil buttonBox->addButton(moreButton, QDialogButtonBox::ActionRole);
+    \snippet examples/dialogs/extension/finddialog.cpp 1
 
     Alternatively, QDialogButtonBox provides several standard buttons (e.g. OK, Cancel, Save)
     that you can use. They exist as flags so you can OR them together in the constructor.
 
-    \quotefromfile dialogs/tabdialog/tabdialog.cpp
-    \skipto buttonBox
-    \printuntil connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+    \snippet examples/dialogs/tabdialog/tabdialog.cpp 2
 
     You can mix and match normal buttons and standard buttons.
 
@@ -167,13 +171,13 @@ static QDialogButtonBox::ButtonRole roleFor(QDialogButtonBox::StandardButton but
     case QDialogButtonBox::Save:
     case QDialogButtonBox::Open:
     case QDialogButtonBox::SaveAll:
-    case QDialogButtonBox::Abort:
     case QDialogButtonBox::Retry:
+    case QDialogButtonBox::Ignore:
         return QDialogButtonBox::AcceptRole;
 
     case QDialogButtonBox::Cancel:
     case QDialogButtonBox::Close:
-    case QDialogButtonBox::Ignore:
+    case QDialogButtonBox::Abort:
         return QDialogButtonBox::RejectRole;
 
     case QDialogButtonBox::Discard:
@@ -355,7 +359,7 @@ void QDialogButtonBoxPrivate::layoutButtons()
     int tmpPolicy = layoutPolicy;
 
     static const int M = 5;
-    static int ModalRoles[M] = { AcceptRole, RejectRole, DestructiveRole, YesRole, NoRole };
+    static const int ModalRoles[M] = { AcceptRole, RejectRole, DestructiveRole, YesRole, NoRole };
     if (tmpPolicy == QDialogButtonBox::MacLayout) {
         bool hasModalButton = false;
         for (int i = 0; i < M; ++i) {
@@ -548,7 +552,7 @@ const char *QDialogButtonBoxPrivate::standardButtonText(QDialogButtonBox::Standa
     const char *buttonText = 0;
     switch (sbutton) {
     case QDialogButtonBox::Ok:
-        buttonText = QT_TRANSLATE_NOOP("QDialogButtonBox", "OK");
+        buttonText = QT_TRANSLATE_NOOP("QDialogButtonBox", "&OK");
         break;
     case QDialogButtonBox::Save:
         buttonText = QT_TRANSLATE_NOOP("QDialogButtonBox", "Save");
@@ -1091,14 +1095,18 @@ bool QDialogButtonBox::event(QEvent *event)
         QList<QAbstractButton *> acceptRoleList = d->buttonLists[AcceptRole];
         QPushButton *firstAcceptButton = acceptRoleList.isEmpty() ? 0 : qobject_cast<QPushButton *>(acceptRoleList.at(0));
         bool hasDefault = false;
+        QWidget *dialog = 0;
+        QWidget *p = this;
+        while (p && !p->isWindow()) {
+            p = p->parentWidget();
+            if ((dialog = qobject_cast<QDialog *>(p)))
+                break;
+        }
 
-        for (int i = d->buttonLayout->count() - 1; i >= 0; --i) {
-            QLayoutItem *item = d->buttonLayout->itemAt(i);
-            if (QPushButton *pb = qobject_cast<QPushButton *>(item->widget())) {
-                if (pb->isDefault() && pb != firstAcceptButton) {
-                    hasDefault = true;
-                    break;
-                }
+        foreach (QPushButton *pb, qFindChildren<QPushButton *>(dialog ? dialog : this)) {
+            if (pb->isDefault() && pb != firstAcceptButton) {
+                hasDefault = true;
+                break;
             }
         }
         if (!hasDefault && firstAcceptButton)
@@ -1110,4 +1118,6 @@ bool QDialogButtonBox::event(QEvent *event)
     return QWidget::event(event);
 }
 
+QT_END_NAMESPACE
+
 #include "moc_qdialogbuttonbox.cpp"
