@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,6 +54,8 @@
 #include "qtimer.h"
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 /*
   Internal class to get access to protected QWidget-members
 */
@@ -66,6 +88,7 @@ protected:
     void paintEvent(QPaintEvent* e);
     void closeEvent(QCloseEvent*);
     void alphaBlend();
+    bool eventFilter(QObject *, QEvent *);
 
 protected slots:
     void render();
@@ -82,6 +105,7 @@ private:
     bool showWidget;
     QTimer anim;
     QTime checkTime;
+    double windowOpacity;
 };
 
 static QAlphaWidget* q_blend = 0;
@@ -95,9 +119,9 @@ QAlphaWidget::QAlphaWidget(QWidget* w, Qt::WindowFlags f)
 #ifndef Q_WS_WIN
     setEnabled(false);
 #endif
-
     setAttribute(Qt::WA_NoSystemBackground, true);
     widget = (QAccessWidget*)w;
+    windowOpacity = w->windowOpacity();
     alpha = 0;
 }
 
@@ -128,30 +152,81 @@ void QAlphaWidget::run(int time)
     checkTime.start();
 
     showWidget = true;
-    qApp->installEventFilter(this);
-
-    move(widget->geometry().x(),widget->geometry().y());
-    resize(widget->size().width(), widget->size().height());
-
-    front = QPixmap::grabWidget(widget).toImage();
-    back = QPixmap::grabWindow(QApplication::desktop()->winId(),
-                                widget->geometry().x(), widget->geometry().y(),
-                                widget->geometry().width(), widget->geometry().height()).toImage();
-
-    if (!back.isNull() && checkTime.elapsed() < duration / 2) {
-        mixed = back.copy();
-        pm = QPixmap::fromImage(mixed);
-        show();
-        setEnabled(false);
-
+#if defined(Q_OS_WIN)
+    if (QSysInfo::WindowsVersion >= QSysInfo::WV_2000 && QSysInfo::WindowsVersion < QSysInfo::WV_NT_based) {
+        qApp->installEventFilter(this);
+        widget->setWindowOpacity(0.0);
+        widget->show();
         connect(&anim, SIGNAL(timeout()), this, SLOT(render()));
         anim.start(1);
-    } else {
-        duration = 0;
-        render();
+    } else
+#endif
+    {
+        //This is roughly equivalent to calling setVisible(true) without actually showing the widget
+        widget->setAttribute(Qt::WA_WState_ExplicitShowHide, true);
+        widget->setAttribute(Qt::WA_WState_Hidden, false);
+
+        qApp->installEventFilter(this);
+
+        move(widget->geometry().x(),widget->geometry().y());
+        resize(widget->size().width(), widget->size().height());
+
+        front = QPixmap::grabWidget(widget).toImage();
+        back = QPixmap::grabWindow(QApplication::desktop()->winId(),
+                                    widget->geometry().x(), widget->geometry().y(),
+                                    widget->geometry().width(), widget->geometry().height()).toImage();
+
+        if (!back.isNull() && checkTime.elapsed() < duration / 2) {
+            mixed = back.copy();
+            pm = QPixmap::fromImage(mixed);
+            show();
+            setEnabled(false);
+
+            connect(&anim, SIGNAL(timeout()), this, SLOT(render()));
+            anim.start(1);
+        } else {
+           duration = 0;
+           render();
+        }
     }
 }
 
+/*
+  \reimp
+*/
+bool QAlphaWidget::eventFilter(QObject *o, QEvent *e)
+{
+    switch (e->type()) {
+    case QEvent::Move:
+	    if (o != widget)
+	        break;
+	    move(widget->geometry().x(),widget->geometry().y());
+	    update();
+	    break;
+    case QEvent::Hide:
+    case QEvent::Close:
+	    if (o != widget)
+	        break;
+    case QEvent::MouseButtonPress:
+	case QEvent::MouseButtonDblClick:
+	    showWidget = false;
+	    render();
+	    break;
+    case QEvent::KeyPress: {
+	        QKeyEvent *ke = (QKeyEvent*)e;
+            if (ke->key() == Qt::Key_Escape) {
+		        showWidget = false;
+            } else {
+		        duration = 0;
+            }
+	        render();
+	        break;
+	}
+    default:
+	    break;
+    }
+    return QWidget::eventFilter(o, e);
+}
 
 /*
   \reimp
@@ -186,6 +261,20 @@ void QAlphaWidget::render()
         alpha = tempel / double(duration);
     else
         alpha = 1;
+
+#if defined(Q_OS_WIN)
+    if (QSysInfo::WindowsVersion >= QSysInfo::WV_2000 && QSysInfo::WindowsVersion < QSysInfo::WV_NT_based) {
+        if (alpha >= windowOpacity || !showWidget) {
+            anim.stop();
+            qApp->removeEventFilter(this);
+            widget->setWindowOpacity(windowOpacity);
+            q_blend = 0;
+            deleteLater();
+        } else {
+            widget->setWindowOpacity(alpha);
+        }
+    } else
+#endif
     if (alpha >= 1 || !showWidget) {
         anim.stop();
         qApp->removeEventFilter(this);
@@ -198,6 +287,9 @@ void QAlphaWidget::render()
 #endif
                 widget->hide();
             } else {
+                //Since we are faking the visibility of the widget 
+                //we need to unset the hidden state on it before calling show
+                widget->setAttribute(Qt::WA_WState_Hidden, true);
                 widget->show();
                 lower();
             }
@@ -385,6 +477,10 @@ void QRollEffect::run(int time)
     move(widget->geometry().x(),widget->geometry().y());
     resize(qMin(currentWidth, totalWidth), qMin(currentHeight, totalHeight));
 
+    //This is roughly equivalent to calling setVisible(true) without actually showing the widget
+    widget->setAttribute(Qt::WA_WState_ExplicitShowHide, true);
+    widget->setAttribute(Qt::WA_WState_Hidden, false);
+
     show();
     setEnabled(false);
 
@@ -458,6 +554,9 @@ void QRollEffect::scroll()
 #endif
                 widget->hide();
             } else {
+                //Since we are faking the visibility of the widget 
+                //we need to unset the hidden state on it before calling show
+                widget->setAttribute(Qt::WA_WState_Hidden, true);
                 widget->show();
                 lower();
             }
@@ -467,12 +566,6 @@ void QRollEffect::scroll()
     }
 }
 
-/*
-  Delete this after timeout
-*/
-
-#include "qeffects.moc"
-
 /*!
     Scroll widget \a w in \a time ms. \a orient may be 1 (vertical), 2
     (horizontal) or 3 (diagonal).
@@ -519,4 +612,13 @@ void qFadeEffect(QWidget* w, int time)
 
     q_blend->run(time);
 }
+
+QT_END_NAMESPACE
+
+/*
+  Delete this after timeout
+*/
+
+#include "qeffects.moc"
+
 #endif //QT_NO_EFFECTS
