@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,6 +47,8 @@
 #include "qpainter.h"
 #include "private/qframe_p.h"
 
+QT_BEGIN_NAMESPACE
+
 class QLCDNumberPrivate : public QFramePrivate
 {
     Q_DECLARE_PUBLIC(QLCDNumber)
@@ -1271,4 +1273,6 @@ bool QLCDNumber::event(QEvent *e)
     \sa setMargin(), QWidget::getContentsMargins()
 */
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_LCDNUMBER
