@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +49,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_LINEEDIT
@@ -36,6 +58,10 @@ QT_MODULE(Gui)
 class QValidator;
 class QMenu;
 class QLineEditPrivate;
+class QCompleter;
+class QStyleOptionFrame;
+class QAbstractSpinBox;
+class QDateTimeEdit;
 
 class Q_GUI_EXPORT QLineEdit : public QWidget
 {
@@ -79,11 +105,7 @@ public:
     void setFrame(bool);
     bool hasFrame() const;
 
-    enum EchoMode { Normal, NoEcho, Password 
-#ifdef Q_WS_QWS
-    ,PasswordEchoOnEdit
-#endif
-    };
+    enum EchoMode { Normal, NoEcho, Password, PasswordEchoOnEdit };
     EchoMode echoMode() const;
     void setEchoMode(EchoMode);
 
@@ -94,7 +116,12 @@ public:
     void setValidator(const QValidator *);
     const QValidator * validator() const;
 #endif
-    
+
+#ifndef QT_NO_COMPLETER
+    void setCompleter(QCompleter *completer);
+    QCompleter *completer() const;
+#endif
+
     QSize sizeHint() const;
     QSize minimumSizeHint() const;
 
@@ -147,10 +174,10 @@ public Q_SLOTS:
 public:
     void deselect();
     void insert(const QString &);
-#ifndef QT_NO_MENU
+#ifndef QT_NO_CONTEXTMENU
     QMenu *createStandardContextMenu();
 #endif
-    
+
 Q_SIGNALS:
     void textChanged(const QString &);
     void textEdited(const QString &);
@@ -175,7 +202,7 @@ protected:
     void dropEvent(QDropEvent *);
 #endif
     void changeEvent(QEvent *);
-#ifndef QT_NO_MENU
+#ifndef QT_NO_CONTEXTMENU
     void contextMenuEvent(QContextMenuEvent *);
 #endif
 #ifdef QT3_SUPPORT
@@ -183,10 +210,12 @@ protected:
 #endif
 
     void inputMethodEvent(QInputMethodEvent *);
+    void initStyleOption(QStyleOptionFrame *option) const;
 public:
     QVariant inputMethodQuery(Qt::InputMethodQuery) const;
     bool event(QEvent *);
 protected:
+    QRect cursorRect() const;
 
 public:
 #ifdef QT3_SUPPORT
@@ -205,19 +234,49 @@ public:
     QT3_SUPPORT int characterAt(int, QChar*) const;
     QT3_SUPPORT bool getSelection(int *, int *);
 
+    QT3_SUPPORT void setFrameRect(QRect) {}
+    QT3_SUPPORT QRect frameRect() const { return QRect(); }
+    enum DummyFrame { Box, Sunken, Plain, Raised, MShadow, NoFrame, Panel, StyledPanel,
+                      HLine, VLine, GroupBoxPanel, WinPanel, ToolBarPanel, MenuBarPanel,
+                      PopupPanel, LineEditPanel, TabWidgetPanel, MShape };
+    QT3_SUPPORT void setFrameShadow(DummyFrame) {}
+    QT3_SUPPORT DummyFrame frameShadow() const { return Plain; }
+    QT3_SUPPORT void setFrameShape(DummyFrame) {}
+    QT3_SUPPORT DummyFrame frameShape() const { return NoFrame; }
+    QT3_SUPPORT void setFrameStyle(int) {}
+    QT3_SUPPORT int frameStyle() const  { return 0; }
+    QT3_SUPPORT int frameWidth() const { return 0; }
+    QT3_SUPPORT void setLineWidth(int) {}
+    QT3_SUPPORT int lineWidth() const { return 0; }
+    QT3_SUPPORT void setMargin(int margin) { setContentsMargins(margin, margin, margin, margin); }
+    QT3_SUPPORT int margin() const
+    { int margin; int dummy; getContentsMargins(&margin, &dummy, &dummy, &dummy);  return margin; }
+    QT3_SUPPORT void setMidLineWidth(int) {}
+    QT3_SUPPORT int midLineWidth() const { return 0; }
+
 Q_SIGNALS:
     QT_MOC_COMPAT void lostFocus();
 #endif
 
 private:
+    friend class QAbstractSpinBox;
+#ifdef QT_KEYPAD_NAVIGATION
+    friend class QDateTimeEdit;
+#endif
     Q_DISABLE_COPY(QLineEdit)
     Q_DECLARE_PRIVATE(QLineEdit)
     Q_PRIVATE_SLOT(d_func(), void _q_clipboardChanged())
+    Q_PRIVATE_SLOT(d_func(), void _q_handleWindowActivate())
     Q_PRIVATE_SLOT(d_func(), void _q_deleteSelected())
+#ifndef QT_NO_COMPLETER
+    Q_PRIVATE_SLOT(d_func(), void _q_completionHighlighted(QString))
+#endif
 };
 
 #endif // QT_NO_LINEEDIT
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QLINEEDIT_H
