@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,6 +60,14 @@
 #include <private/qwidget_p.h>
 #include "qtoolbar_p.h"
 #include "qwidgetanimator_p.h"
+#ifdef Q_WS_MAC
+#include <private/qt_mac_p.h>
+QT_BEGIN_NAMESPACE
+extern WindowRef qt_mac_window_for(const QWidget *); // qwidget_mac.cpp
+QT_END_NAMESPACE
+#endif
+
+QT_BEGIN_NAMESPACE
 
 class QMainWindowPrivate : public QWidgetPrivate
 {
@@ -47,11 +75,20 @@ class QMainWindowPrivate : public QWidgetPrivate
 public:
     inline QMainWindowPrivate()
         : layout(0), toolButtonStyle(Qt::ToolButtonIconOnly)
+#ifdef Q_WS_MAC
+            , useHIToolBar(false)
+#endif
+#if !defined(QT_NO_DOCKWIDGET) && !defined(QT_NO_CURSOR)
+            , hasOldCursor(false) , cursorAdjusted(false)
+#endif
     { }
     QMainWindowLayout *layout;
     QSize iconSize;
     bool explicitIconSize;
     Qt::ToolButtonStyle toolButtonStyle;
+#ifdef Q_WS_MAC
+    bool useHIToolBar;
+#endif
     void init();
     QList<int> hoverSeparator;
     QPoint hoverPos;
@@ -59,6 +96,9 @@ public:
 #if !defined(QT_NO_DOCKWIDGET) && !defined(QT_NO_CURSOR)
     QCursor separatorCursor(const QList<int> &path) const;
     void adjustCursor(const QPoint &pos);
+    QCursor oldCursor;
+    uint hasOldCursor : 1;
+    uint cursorAdjusted : 1;
 #endif
 };
 
@@ -66,7 +106,7 @@ void QMainWindowPrivate::init()
 {
     Q_Q(QMainWindow);
     layout = new QMainWindowLayout(q);
-    const int metric = q->style()->pixelMetric(QStyle::PM_ToolBarIconSize);
+    const int metric = q->style()->pixelMetric(QStyle::PM_ToolBarIconSize, 0, q);
     iconSize = QSize(metric, metric);
     explicitIconSize = false;
 
@@ -113,125 +153,122 @@ void QMainWindowPrivate::init()
     \mainclass
 
     \tableofcontents
-    
+
     \section1 Qt Main Window Framework
 
     A main window provides a framework for building an
     application's user interface. Qt has QMainWindow and its \l{Main
     Window and Related Classes}{related classes} for main window
     management. QMainWindow has its own layout to which you can add
-    \l{QToolBar}{QToolBar}s, \l{QDockWindow}{QDockWindow}s, a
+    \l{QToolBar}s, \l{QDockWidget}s, a
     QMenuBar, and a QStatusBar. The layout has a center area that can
     be occupied by any kind of widget. You can see an image of the
     layout below.
+
+    \image mainwindowlayout.png
+
+    \note Creating a main window without a central widget is not supported.
+    You must have a central widget even if it is just a placeholder.
     
-    \image mainwindowlayout.png 
- 
     \section1 Creating Main Window Components
-    
+
     A central widget will typically be a standard Qt widget such
     as a QTextEdit or a QGraphicsView. Custom widgets can also be
     used for advanced applications. You set the central widget with \c
-    setCentralWidget(). 
-    
+    setCentralWidget().
+
     Main windows have either a single (SDI) or multiple (MDI)
     document interface. You create MDI applications in Qt by using a
-    QWorkspace as the central widget.
-    
+    QMdiArea as the central widget.
+
     We will now examine each of the other widgets that can be
     added to a main window. We give examples on how to create and add
     them.
-    
+
     \section2 Creating Menus
-    
+
     Qt implements menus in QMenu and QMainWindow keeps them in a
     QMenuBar. \l{QAction}{QAction}s are added to the menus, which
     display them as menu items.
-    
+
     You can add new menus to the main window's menu bar by calling
     \c menuBar(), which returns the QMenuBar for the window, and then
     add a menu with QMenuBar::addMenu().
-    
+
     QMainWindow comes with a default menu bar, but you can also
     set one yourself with \c setMenuBar(). If you wish to implement a
     custom menu bar (i.e., not use the QMenuBar widget), you can set it
     with \c setMenuWidget().
 
     An example of how to create menus follows:
-    
-    \quotefromfile mainwindows/application/mainwindow.cpp
-    \skipto /::createMenus/
-    \printuntil /saveAct/
-    
+
+    \snippet examples/mainwindows/application/mainwindow.cpp 26
+
     The \c createPopupMenu() function creates popup menus when the
     main window receives context menu events.  The default
     implementation generates a menu with the checkable actions from
     the dock widgets and toolbars. You can reimplement \c
     createPopupMenu() for a custom menu.
-    
+
     \section2 Creating Toolbars
-    
+
     Toolbars are implemented in the QToolBar class.  You add a
     toolbar to a main window with \c addToolBar().
-    
+
     You control the initial position of toolbars by assigning them
     to a specific Qt::ToolBarArea. You can split an area by inserting
     a toolbar break - think of this as a line break in text editing -
     with \c addToolBarBreak() or \c insertToolBarBreak(). You can also
     restrict placement by the user with QToolBar::setAllowedAreas()
-    and QToolBar::setMovable(). 
-    
+    and QToolBar::setMovable().
+
     The size of toolbar icons can be retrieved with \c iconSize().
     The sizes are platform dependent; you can set a fixed size with \c
     setIconSize(). You can alter the appearance of all tool buttons in
-    the toolbars with \c setToolButtonStyle(). 
-    
+    the toolbars with \c setToolButtonStyle().
+
     An example of toolbar creation follows:
-    
-    \quotefromfile mainwindows/application/mainwindow.cpp
-    \skipto /::createToolBars/
-    \printuntil /fileToolBar->addAction/
-    
+
+    \snippet examples/mainwindows/application/mainwindow.cpp 29
+
     \section2 Creating Dock Widgets
-    
+
     Dock widgets are implemented in the QDockWidget class. A dock
     widget is a window that can be docked into the main window.  You
     add dock widgets to a main window with \c addDockWidget().
-    
+
     There are four dock widget areas as given by the
     Qt::DockWidgetArea enum: left, right, top, and bottom. You can
     specify which dock widget area that should occupy the corners
-    where the areas overlap with \c setDockWidgetCorner(). By default
+    where the areas overlap with \c setCorner(). By default
     each area can only contain one row (vertical or horizontal) of
     dock widgets, but if you enable nesting with \c
     setDockNestingEnabled(), dock widgets can be added in either
     direction.
-    
+
     Two dock widgets may also be stacked on top of each other. A
     QTabBar is then used to select which of the widgets that should be
     displayed.
-    
+
     We give an example of how to create and add dock widgets to a
     main window:
-    
-    \quotefromfile snippets/mainwindowsnippet.cpp
-    \skipto /QDockWidget \*dockWidget/
-    \printuntil /addDockWidget/
 
-    \section2 The Status Bar    
+    \snippet doc/src/snippets/mainwindowsnippet.cpp 0
+
+    \section2 The Status Bar
 
     You can set a status bar with \c setStatusBar(), but one is
     created the first time \c statusBar() (which returns the main
     window's status bar) is called. See QStatusBar for information on
     how to use it.
-    
+
     \section1 Storing State
-    
+
     QMainWindow can store the state of its layout with \c
     saveState(); it can later be retrieved with \c restoreState(). It
     is the position and size (relative to the size of the main window)
     of the toolbars and dock widgets that are stored.
-    
+
     \sa QMenuBar, QToolBar, QStatusBar, QDockWidget, {Application
     Example}, {Dock Widgets Example}, {MDI Example}, {SDI Example},
     {Menus Example}
@@ -295,8 +332,61 @@ QMainWindow::~QMainWindow()
     \brief size of toolbar icons in this mainwindow.
 
     The default is the default tool bar icon size of the GUI style.
+    Note that the icons used must be at least of this size as the
+    icons are only scaled down.
 */
 
+/*!
+    \property QMainWindow::dockOptions
+    \brief the docking behavior of QMainWindow
+    \since 4.3
+
+    The default value is AnimatedDocks | AllowTabbedDocks.
+*/
+
+/*!
+    \enum QMainWindow::DockOption
+    \since 4.3
+
+    This enum contains flags that specify the docking behavior of QMainWindow.
+
+    \value AnimatedDocks    Identical to the \l animated property.
+
+    \value AllowNestedDocks Identical to the \l dockNestingEnabled property.
+
+    \value AllowTabbedDocks The user can drop one dock widget "on top" of
+                            another. The two widgets are stacked and a tab
+                            bar appears for selecting which one is visible.
+
+    \value ForceTabbedDocks Each dock area contains a single stack of tabbed
+                            dock widgets. In other words, dock widgets cannot
+                            be placed next to each other in a dock area. If
+                            this option is set, AllowNestedDocks has no effect.
+
+    \value VerticalTabs     The two vertical dock areas on the sides of the
+                            main window show their tabs vertically. If this
+                            option is not set, all dock areas show their tabs
+                            at the bottom. Implies AllowTabbedDocks.
+
+    These options only control how dock widgets may be dropped in a QMainWindow.
+    They do not re-arrange the dock widgets to conform with the specified
+    options. For this reason they should be set before any dock widgets
+    are added to the main window. Exceptions to this are the AnimatedDocks and
+    VerticalTabs options, which may be set at any time.
+*/
+
+void QMainWindow::setDockOptions(DockOptions opt)
+{
+    Q_D(QMainWindow);
+    d->layout->setDockOptions(opt);
+}
+
+QMainWindow::DockOptions QMainWindow::dockOptions() const
+{
+    Q_D(const QMainWindow);
+    return d->layout->dockOptions;
+}
+
 QSize QMainWindow::iconSize() const
 { return d_func()->iconSize; }
 
@@ -305,7 +395,7 @@ void QMainWindow::setIconSize(const QSize &iconSize)
     Q_D(QMainWindow);
     QSize sz = iconSize;
     if (!sz.isValid()) {
-        const int metric = style()->pixelMetric(QStyle::PM_ToolBarIconSize);
+        const int metric = style()->pixelMetric(QStyle::PM_ToolBarIconSize, 0, this);
         sz = QSize(metric, metric);
     }
     if (d->iconSize != sz) {
@@ -362,8 +452,21 @@ QMenuBar *QMainWindow::menuBar() const
 void QMainWindow::setMenuBar(QMenuBar *menuBar)
 {
     Q_D(QMainWindow);
-    if (d->layout->menuBar() && d->layout->menuBar() != menuBar)
-        delete d->layout->menuBar();
+    if (d->layout->menuBar() && d->layout->menuBar() != menuBar) {
+        // Reparent corner widgets before we delete the old menu bar.
+        QMenuBar *oldMenuBar = qobject_cast<QMenuBar *>(d->layout->menuBar());
+        if (menuBar) {
+            // TopLeftCorner widget.
+            QWidget *cornerWidget = oldMenuBar->cornerWidget(Qt::TopLeftCorner);
+            if (cornerWidget)
+                menuBar->setCornerWidget(cornerWidget, Qt::TopLeftCorner);
+            // TopRightCorner widget.
+            cornerWidget = oldMenuBar->cornerWidget(Qt::TopRightCorner);
+            if (cornerWidget)
+                menuBar->setCornerWidget(cornerWidget, Qt::TopRightCorner);
+        }
+        delete oldMenuBar;
+    }
     d->layout->setMenuBar(menuBar);
 }
 
@@ -371,7 +474,7 @@ void QMainWindow::setMenuBar(QMenuBar *menuBar)
     \since 4.2
 
     Returns the menu bar for the main window. This function returns
-    null if a menubar hasn't been constructed yet.
+    null if a menu bar hasn't been constructed yet.
 */
 QWidget *QMainWindow::menuWidget() const
 {
@@ -532,6 +635,16 @@ void QMainWindow::addToolBarBreak(Qt::ToolBarArea area)
 void QMainWindow::insertToolBarBreak(QToolBar *before)
 { d_func()->layout->insertToolBarBreak(before); }
 
+/*!
+    Removes a toolbar break previously inserted before the toolbar specified by \a before.
+*/
+
+void QMainWindow::removeToolBarBreak(QToolBar *before)
+{
+    Q_D(QMainWindow);
+    d->layout->removeToolBarBreak(before);
+}
+
 /*!
     Adds the \a toolbar into the specified \a area in this main
     window. The \a toolbar is placed at the end of the current tool
@@ -547,15 +660,28 @@ void QMainWindow::addToolBar(Qt::ToolBarArea area, QToolBar *toolbar)
 
     Q_D(QMainWindow);
 
-    if (!toolbar->isAreaAllowed(area))
-        qWarning("QMainWIndow::addToolBar(): specified 'area' is not an allowed for this toolbar");
-
     disconnect(this, SIGNAL(iconSizeChanged(QSize)),
                toolbar, SLOT(_q_updateIconSize(QSize)));
     disconnect(this, SIGNAL(toolButtonStyleChanged(Qt::ToolButtonStyle)),
                toolbar, SLOT(_q_updateToolButtonStyle(Qt::ToolButtonStyle)));
 
-    d->layout->removeWidget(toolbar);
+    if(toolbar->d_func()->state && toolbar->d_func()->state->dragging) {
+        //removing a toolbar which is dragging will cause crash
+#ifndef QT_NO_DOCKWIDGET
+        bool animated = isAnimated();
+        setAnimated(false);
+#endif
+        toolbar->d_func()->endDrag();
+#ifndef QT_NO_DOCKWIDGET
+        setAnimated(animated);
+#endif
+    }
+
+    if (!d->layout->usesHIToolBar(toolbar)) {
+        d->layout->removeWidget(toolbar);
+    } else {
+        d->layout->removeToolBar(toolbar);
+    }
 
     toolbar->d_func()->_q_updateIconSize(d->iconSize);
     toolbar->d_func()->_q_updateToolButtonStyle(d->toolButtonStyle);
@@ -565,9 +691,6 @@ void QMainWindow::addToolBar(Qt::ToolBarArea area, QToolBar *toolbar)
             toolbar, SLOT(_q_updateToolButtonStyle(Qt::ToolButtonStyle)));
 
     d->layout->addToolBar(area, toolbar);
-
-    if (isVisible())
-        d->layout->relayout();
 }
 
 /*! \overload
@@ -592,20 +715,6 @@ QToolBar *QMainWindow::addToolBar(const QString &title)
     return toolBar;
 }
 
-/* Removes the toolbar from the mainwindow so that it can be added again. Does not
-   explicitly hide the toolbar. */
-static void qt_remove_toolbar_from_layout(QToolBar *toolbar, QMainWindowPrivate *d)
-{
-    if (toolbar) {
-        QObject::disconnect(d->q_ptr, SIGNAL(iconSizeChanged(QSize)),
-                   toolbar, SLOT(_q_updateIconSize(QSize)));
-        QObject::disconnect(d->q_ptr, SIGNAL(toolButtonStyleChanged(Qt::ToolButtonStyle)),
-                   toolbar, SLOT(_q_updateToolButtonStyle(Qt::ToolButtonStyle)));
-
-        d->layout->removeWidget(toolbar);
-    }
-}
-
 /*!
     Inserts the \a toolbar into the area occupied by the \a before toolbar
     so that it appears before it. For example, in normal left-to-right
@@ -617,10 +726,8 @@ static void qt_remove_toolbar_from_layout(QToolBar *toolbar, QMainWindowPrivate
 void QMainWindow::insertToolBar(QToolBar *before, QToolBar *toolbar)
 {
     Q_D(QMainWindow);
-    Q_ASSERT_X(toolbar->isAreaAllowed(toolBarArea(before)),
-               "QMainWIndow::insertToolBar", "specified 'area' is not an allowed area");
 
-    qt_remove_toolbar_from_layout(toolbar, d);
+    d->layout->removeToolBar(toolbar);
 
     toolbar->d_func()->_q_updateIconSize(d->iconSize);
     toolbar->d_func()->_q_updateToolButtonStyle(d->toolButtonStyle);
@@ -630,9 +737,6 @@ void QMainWindow::insertToolBar(QToolBar *before, QToolBar *toolbar)
             toolbar, SLOT(_q_updateToolButtonStyle(Qt::ToolButtonStyle)));
 
     d->layout->insertToolBar(before, toolbar);
-
-    if (isVisible())
-        d->layout->relayout();
 }
 
 /*!
@@ -642,7 +746,7 @@ void QMainWindow::insertToolBar(QToolBar *before, QToolBar *toolbar)
 void QMainWindow::removeToolBar(QToolBar *toolbar)
 {
     if (toolbar) {
-        qt_remove_toolbar_from_layout(toolbar, d_func());
+        d_func()->layout->removeToolBar(toolbar);
         toolbar->hide();
     }
 }
@@ -657,56 +761,125 @@ void QMainWindow::removeToolBar(QToolBar *toolbar)
 Qt::ToolBarArea QMainWindow::toolBarArea(QToolBar *toolbar) const
 { return d_func()->layout->toolBarArea(toolbar); }
 
+/*!
+
+    Returns whether there is a toolbar
+    break before the \a toolbar.
+
+    \sa  addToolBarBreak(), insertToolBarBreak()
+*/
+bool QMainWindow::toolBarBreak(QToolBar *toolbar) const
+{
+    return d_func()->layout->toolBarBreak(toolbar);
+}
+
 #endif // QT_NO_TOOLBAR
 
 #ifndef QT_NO_DOCKWIDGET
 
 /*! \property QMainWindow::animated
-    \brief whether manipulating dock widgets is animated
+    \brief whether manipulating dock widgets and tool bars is animated
     \since 4.2
 
-    When a dock widget is dragged over the main window, other dock widgets in the
-    window will adjust themselves to make space for the dragged widget. If this
-    property is set to true, their movement will be animated. The default value
-    is true.
+    When a dock widget or tool bar is dragged over the
+    main window, the main window adjusts its contents
+    to indicate where the dock widget or tool bar will
+    be docked if it is dropped. Setting this property
+    causes QMainWindow to move its contents in a smooth
+    animation. Clearing this property causes the contents
+    to snap into their new positions.
+
+    By default, this property is set. It may be cleared if
+    the main window contains widgets which are slow at resizing
+    or repainting themselves.
+
+    Setting this property is identical to setting the AnimatedDocks
+    option using setDockOptions().
 */
 
 bool QMainWindow::isAnimated() const
 {
-    return d_func()->layout->animationEnabled;
+    Q_D(const QMainWindow);
+    return d->layout->dockOptions & AnimatedDocks;
 }
 
 void QMainWindow::setAnimated(bool enabled)
 {
-    d_func()->layout->animationEnabled = enabled;
+    Q_D(QMainWindow);
+
+    DockOptions opts = d->layout->dockOptions;
+    if (enabled)
+        opts |= AnimatedDocks;
+    else
+        opts &= ~AnimatedDocks;
+
+    d->layout->setDockOptions(opts);
 }
 
 /*! \property QMainWindow::dockNestingEnabled
     \brief whether docks can be nested
     \since 4.2
 
-    If this property is set to false, dock areas can only contain a single row
-    (horizontal or vertical) of dock widgets. If this property is set to true,
+    If this property is false, dock areas can only contain a single row
+    (horizontal or vertical) of dock widgets. If this property is true,
     the area occupied by a dock widget can be split in either direction to contain
     more dock widgets.
 
     Dock nesting is only necessary in applications that contain a lot of
-    dock widgets. It gives the user greater freedom in organizing their main window.
-    However, dock nesting leads to more complex (and less intuitive) behavior when
-    a dock widget is dragged over the main window, since there are more ways in which
-    a dropped dock widget may be placed in the dock area.
+    dock widgets. It gives the user greater freedom in organizing their
+    main window. However, dock nesting leads to more complex
+    (and less intuitive) behavior when a dock widget is dragged over the
+    main window, since there are more ways in which a dropped dock widget
+    may be placed in the dock area.
+
+    Setting this property is identical to setting the AllowNestedDocks option
+    using setDockOptions().
 */
 
 bool QMainWindow::isDockNestingEnabled() const
 {
-    return d_func()->layout->dockNestingEnabled;
+    Q_D(const QMainWindow);
+    return d->layout->dockOptions & AllowNestedDocks;
 }
 
 void QMainWindow::setDockNestingEnabled(bool enabled)
 {
-    d_func()->layout->dockNestingEnabled = enabled;
+    Q_D(QMainWindow);
+
+    DockOptions opts = d->layout->dockOptions;
+    if (enabled)
+        opts |= AllowNestedDocks;
+    else
+        opts &= ~AllowNestedDocks;
+
+    d->layout->setDockOptions(opts);
 }
 
+#if 0
+/*! \property QMainWindow::verticalTabsEnabled
+    \brief whether left and right dock areas use vertical tabs
+    \since 4.2
+
+    If this property is set to false, dock areas containing tabbed dock widgets
+    display horizontal tabs, simmilar to Visual Studio.
+
+    If this property is set to true, then the right and left dock areas display vertical
+    tabs, simmilar to KDevelop.
+
+    This property should be set before any dock widgets are added to the main window.
+*/
+
+bool QMainWindow::verticalTabsEnabled() const
+{
+    return d_func()->layout->verticalTabsEnabled();
+}
+
+void QMainWindow::setVerticalTabsEnabled(bool enabled)
+{
+    d_func()->layout->setVerticalTabsEnabled(enabled);
+}
+#endif
+
 static bool checkDockWidgetArea(Qt::DockWidgetArea area, const char *where)
 {
     switch (area) {
@@ -730,9 +903,6 @@ void QMainWindow::addDockWidget(Qt::DockWidgetArea area, QDockWidget *dockwidget
     if (!checkDockWidgetArea(area, "QMainWindow::addDockWidget"))
         return;
 
-    if (!dockwidget->isAreaAllowed(area))
-        qWarning("QMainWindow::addDockWidget(): specified 'area' is not an allowed for this widget");
-
     Qt::Orientation orientation = Qt::Vertical;
     switch (area) {
     case Qt::TopDockWidgetArea:
@@ -760,6 +930,11 @@ void QMainWindow::addDockWidget(Qt::DockWidgetArea area, QDockWidget *dockwidget
 #endif
 }
 
+bool QMainWindow::restoreDockWidget(QDockWidget *dockwidget)
+{
+    return d_func()->layout->restoreDockWidget(dockwidget);
+}
+
 /*!
     Adds \a dockwidget into the given \a area in the direction
     specified by the \a orientation.
@@ -770,13 +945,8 @@ void QMainWindow::addDockWidget(Qt::DockWidgetArea area, QDockWidget *dockwidget
     if (!checkDockWidgetArea(area, "QMainWindow::addDockWidget"))
         return;
 
-    if (!dockwidget->isAreaAllowed(area))
-        qWarning("QMainWindow::addDockWidget(): specified 'area' is not an allowed for this widget");
-
     // add a window to an area, placing done relative to the previous
     d_func()->layout->addDockWidget(area, dockwidget, orientation);
-    if (isVisible())
-        d_func()->layout->relayout();
 }
 
 /*!
@@ -803,11 +973,7 @@ void QMainWindow::addDockWidget(Qt::DockWidgetArea area, QDockWidget *dockwidget
 void QMainWindow::splitDockWidget(QDockWidget *after, QDockWidget *dockwidget,
                                   Qt::Orientation orientation)
 {
-    if (!dockwidget->isAreaAllowed(dockWidgetArea(after)))
-        qWarning("QMainWindow::splitDockWidget(): specified 'area' is not an allowed for this widget");
     d_func()->layout->splitDockWidget(after, dockwidget, orientation);
-    if (isVisible())
-        d_func()->layout->relayout();
 }
 
 /*!
@@ -818,11 +984,7 @@ void QMainWindow::splitDockWidget(QDockWidget *after, QDockWidget *dockwidget,
 */
 void QMainWindow::tabifyDockWidget(QDockWidget *first, QDockWidget *second)
 {
-    if (!second->isAreaAllowed(dockWidgetArea(first)))
-        qWarning("QMainWindow::splitDockWidget(): specified 'area' is not an allowed for this widget");
     d_func()->layout->tabifyDockWidget(first, second);
-    if (isVisible())
-        d_func()->layout->relayout();
 }
 
 /*!
@@ -894,21 +1056,30 @@ bool QMainWindow::restoreState(const QByteArray &state, int version)
     if (stream.status() != QDataStream::Ok || marker != QMainWindowLayout::VersionMarker || v != version)
         return false;
     bool restored = d_func()->layout->restoreState(stream);
-    if (isVisible())
-        QApplication::postEvent(this, new QResizeEvent(size(), size()));
     return restored;
 }
 
 #if !defined(QT_NO_DOCKWIDGET) && !defined(QT_NO_CURSOR)
 QCursor QMainWindowPrivate::separatorCursor(const QList<int> &path) const
 {
-    QDockAreaLayoutInfo *info = layout->dockWidgetLayout.info(path);
+    QDockAreaLayoutInfo *info = layout->layoutState.dockAreaLayout.info(path);
     Q_ASSERT(info != 0);
-    if (path.size() == 1) {
-        return info->o == Qt::Horizontal
-                ? Qt::SplitVCursor : Qt::SplitHCursor;
+    if (path.size() == 1) { // is this the "top-level" separator which separates a dock area
+                            // from the central widget?
+        switch (path.first()) {
+            case QInternal::LeftDock:
+            case QInternal::RightDock:
+                return Qt::SplitHCursor;
+            case QInternal::TopDock:
+            case QInternal::BottomDock:
+                return Qt::SplitVCursor;
+            default:
+                break;
+        }
     }
 
+    // no, it's a splitter inside a dock area, separating two dock widgets
+
     return info->o == Qt::Horizontal
             ? Qt::SplitHCursor : Qt::SplitVCursor;
 }
@@ -921,25 +1092,44 @@ void QMainWindowPrivate::adjustCursor(const QPoint &pos)
 
     if (pos == QPoint(0, 0)) {
         if (!hoverSeparator.isEmpty())
-            q->update(layout->dockWidgetLayout.separatorRect(hoverSeparator));
+            q->update(layout->layoutState.dockAreaLayout.separatorRect(hoverSeparator));
         hoverSeparator.clear();
-        q->unsetCursor();
+        
+        if (cursorAdjusted) {
+            cursorAdjusted = false;
+            if (hasOldCursor)
+                q->setCursor(oldCursor);
+            else
+                q->unsetCursor();
+        }
     } else {
         QList<int> pathToSeparator
-            = layout->dockWidgetLayout.findSeparator(pos);
+            = layout->layoutState.dockAreaLayout.findSeparator(pos);
 
         if (pathToSeparator != hoverSeparator) {
             if (!hoverSeparator.isEmpty())
-                q->update(layout->dockWidgetLayout.separatorRect(hoverSeparator));
+                q->update(layout->layoutState.dockAreaLayout.separatorRect(hoverSeparator));
 
             hoverSeparator = pathToSeparator;
 
             if (hoverSeparator.isEmpty()) {
-                q->unsetCursor();
+                if (cursorAdjusted) {
+                    cursorAdjusted = false;
+                    if (hasOldCursor)
+                        q->setCursor(oldCursor);
+                    else
+                        q->unsetCursor();
+                }
             } else {
-                q->update(layout->dockWidgetLayout.separatorRect(hoverSeparator));
+                q->update(layout->layoutState.dockAreaLayout.separatorRect(hoverSeparator));
+                if (!cursorAdjusted) {
+                    oldCursor = q->cursor();
+                    hasOldCursor = q->testAttribute(Qt::WA_SetCursor);
+                }
                 QCursor cursor = separatorCursor(hoverSeparator);
+                cursorAdjusted = false; //to not reset the oldCursor in event(CursorChange)
                 q->setCursor(cursor);
+                cursorAdjusted = true;
             }
         }
     }
@@ -956,7 +1146,7 @@ bool QMainWindow::event(QEvent *event)
         case QEvent::Paint: {
             QPainter p(this);
             QRegion r = static_cast<QPaintEvent*>(event)->region();
-            d->layout->dockWidgetLayout.paintSeparators(&p, this, r, d->hoverPos);
+            d->layout->layoutState.dockAreaLayout.paintSeparators(&p, this, r, d->hoverPos);
             break;
         }
 
@@ -966,7 +1156,14 @@ bool QMainWindow::event(QEvent *event)
             break;
         }
 
+        // We don't want QWidget to call update() on the entire QMainWindow
+        // on HoverEnter and HoverLeave, hence accept the event (return true).
+        case QEvent::HoverEnter:
+            return true;
         case QEvent::HoverLeave:
+            d->adjustCursor(QPoint(0, 0));
+            return true;
+        case QEvent::ShortcutOverride: // when a menu pops up
             d->adjustCursor(QPoint(0, 0));
             break;
 #endif // QT_NO_CURSOR
@@ -1012,16 +1209,7 @@ bool QMainWindow::event(QEvent *event)
 
 #ifndef QT_NO_TOOLBAR
         case QEvent::ToolBarChange: {
-            QList<QToolBar *> toolbars = qFindChildren<QToolBar *>(this);
-            QSize minimumSize = d->layout->minimumSize();
-            for (int i = 0; i < toolbars.size(); ++i) {
-                QToolBar *toolbar = toolbars.at(i);
-                toolbar->setVisible(!toolbar->isVisible());
-            }
-            QApplication::sendPostedEvents(this, QEvent::LayoutRequest);
-            QSize newMinimumSize = d->layout->minimumSize();
-            QSize delta = newMinimumSize - minimumSize;
-            resize(size() + delta);
+            d->layout->toggleToolBarsVisible();
             return true;
         }
 #endif
@@ -1041,6 +1229,21 @@ bool QMainWindow::event(QEvent *event)
             if (!d->explicitIconSize)
                 setIconSize(QSize());
             break;
+#ifdef Q_WS_MAC
+        case QEvent::Show: {
+            if (unifiedTitleAndToolBarOnMac())
+                ShowHideWindowToolbar(qt_mac_window_for(this), true, false);
+            break;
+       }
+#endif
+#if !defined(QT_NO_DOCKWIDGET) && !defined(QT_NO_CURSOR)
+       case QEvent::CursorChange:
+           if (d->cursorAdjusted) {
+               d->oldCursor = cursor();
+               d->hasOldCursor = testAttribute(Qt::WA_SetCursor);
+           }
+           break;
+#endif           
 
         default:
             break;
@@ -1049,15 +1252,74 @@ bool QMainWindow::event(QEvent *event)
     return QWidget::event(event);
 }
 
+#ifndef QT_NO_TOOLBAR
+
+/*!
+    \property QMainWindow::unifiedTitleAndToolBarOnMac
+    \brief whether the window uses the unified title and toolbar look on Mac OS X
+    \since 4.3
+
+    This property is false by default and only has any effect on Mac OS X 10.4 or higher.
+
+    If set to true, then the top toolbar area is replaced with a Carbon
+    HIToolbar and all toolbars in the top toolbar area are moved to that. Any
+    toolbars added afterwards will also be added to the Carbon HIToolbar. This
+    means a couple of things.
+
+    \list
+    \i QToolBars in this toolbar area are not movable and you cannot drag other toolbars to it
+    \i Toolbar breaks are not respected or preserved
+    \i Any custom widgets in the toolbar will not be shown if the toolbar becomes too small
+      (only actions will be shown)
+    \i If you call showFullScreen() on the main window, the QToolbar will
+       disappear since it is considered to be part of the title bar. You can work
+       around this by doing the following by turning off the unified toolbar
+       before you call showFullScrenn() and restore the value after you call
+       showNormal().
+    \endlist
+
+    Setting this back to false will remove these restrictions.
+
+    The Qt::WA_MacBrushedMetal attribute takes precedence over this property.
+*/
+void QMainWindow::setUnifiedTitleAndToolBarOnMac(bool set)
+{
+#ifdef Q_WS_MAC
+    Q_D(QMainWindow);
+    if (!isWindow() || d->useHIToolBar == set || QSysInfo::MacintoshVersion < QSysInfo::MV_10_3)
+        return;
+
+    d->useHIToolBar = set;
+    createWinId(); // We need the hiview for down below.
+
+    d->layout->updateHIToolBarStatus();
+    // Enabling the unified toolbar clears the opaque size grip setting, update it.
+    d->macUpdateOpaqueSizeGrip();
+#else
+    Q_UNUSED(set)
+#endif
+}
+
+bool QMainWindow::unifiedTitleAndToolBarOnMac() const
+{
+#ifdef Q_WS_MAC
+    return d_func()->useHIToolBar && !testAttribute(Qt::WA_MacBrushedMetal);
+#endif
+    return false;
+}
+
+#endif // QT_NO_TOOLBAR
+
 /*!
     \internal
 */
 bool QMainWindow::isSeparator(const QPoint &pos) const
 {
-    Q_D(const QMainWindow);
 #ifndef QT_NO_DOCKWIDGET
-    return !d->layout->dockWidgetLayout.findSeparator(pos).isEmpty();
+    Q_D(const QMainWindow);
+    return !d->layout->layoutState.dockAreaLayout.findSeparator(pos).isEmpty();
 #else
+    Q_UNUSED(pos);
     return false;
 #endif
 }
@@ -1067,10 +1329,18 @@ bool QMainWindow::isSeparator(const QPoint &pos) const
 */
 void QMainWindow::contextMenuEvent(QContextMenuEvent *event)
 {
+    event->ignore();
     // only show the context menu for direct QDockWidget and QToolBar
-    // children
+    // children and for the menu bar as well
     QWidget *child = childAt(event->pos());
     while (child && child != this) {
+#ifndef QT_NO_MENUBAR
+        if (QMenuBar *mb = qobject_cast<QMenuBar *>(child)) {
+            if (mb->parentWidget() != this)
+                return;
+            break;
+        }
+#endif
 #ifndef QT_NO_DOCKWIDGET
         if (QDockWidget *dw = qobject_cast<QDockWidget *>(child)) {
             if (dw->parentWidget() != this)
@@ -1097,18 +1367,19 @@ void QMainWindow::contextMenuEvent(QContextMenuEvent *event)
 
 #ifndef QT_NO_MENU
     QMenu *popup = createPopupMenu();
-    if (!popup)
-	return;
-    popup->exec(event->globalPos());
+    if (popup && !popup->isEmpty()) {
+        popup->exec(event->globalPos());
+        event->accept();
+    }
     delete popup;
-    event->accept();
 #endif
 }
 
 #ifndef QT_NO_MENU
 /*!
     Returns a popup menu containing checkable entries for the toolbars and
-    dock widgets present in the main window.
+    dock widgets present in the main window. If  there are no toolbars and
+    dock widgets present, this function returns a null pointer.
 
     By default, this function is called by the main window when the user
     activates a context menu, typically by right-clicking on a toolbar or a dock
@@ -1131,7 +1402,7 @@ QMenu *QMainWindow::createPopupMenu()
         for (int i = 0; i < dockwidgets.size(); ++i) {
             QDockWidget *dockWidget = dockwidgets.at(i);
             if (dockWidget->parentWidget() == this
-                && d->layout->contains(dockWidget)) {
+                && !d->layout->layoutState.dockAreaLayout.indexOf(dockWidget).isEmpty()) {
                 menu->addAction(dockwidgets.at(i)->toggleViewAction());
             }
         }
@@ -1146,7 +1417,9 @@ QMenu *QMainWindow::createPopupMenu()
         for (int i = 0; i < toolbars.size(); ++i) {
             QToolBar *toolBar = toolbars.at(i);
             if (toolBar->parentWidget() == this
-                && d->layout->contains(toolBar)) {
+                && (!d->layout->layoutState.toolBarAreaLayout.indexOf(toolBar).isEmpty()
+                    || (unifiedTitleAndToolBarOnMac()
+                        && toolBarArea(toolBar) == Qt::TopToolBarArea))) {
                 menu->addAction(toolbars.at(i)->toggleViewAction());
             }
         }
@@ -1157,4 +1430,6 @@ QMenu *QMainWindow::createPopupMenu()
 }
 #endif // QT_NO_MENU
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_MAINWINDOW
