@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -62,6 +67,15 @@
 #include "qdockarealayout_p.h"
 #include "qtoolbararealayout_p.h"
 
+//#define Q_DEBUG_MAINWINDOW_LAYOUT
+
+#ifdef Q_DEBUG_MAINWINDOW_LAYOUT
+QT_BEGIN_NAMESPACE
+class QTextStream;
+Q_GUI_EXPORT void qt_dumpLayout(QTextStream &qout, QMainWindow *window);
+QT_END_NAMESPACE
+#endif // Q_DEBUG_MAINWINDOW_LAYOUT
+
 #ifdef Q_WS_MAC
 // Forward defs to make avoid including Carbon.h (faster compile you know ;).
 struct OpaqueHIObjectRef;
@@ -72,6 +86,8 @@ typedef const struct __CFString * CFStringRef;
 
 #endif
 
+QT_BEGIN_NAMESPACE
+
 class QToolBar;
 class QWidgetAnimator;
 class QTabBar;
@@ -125,13 +141,15 @@ public:
     QList<int> gapIndex(QWidget *widget, const QPoint &pos) const;
     bool insertGap(QList<int> path, QLayoutItem *item);
     void remove(QList<int> path);
+    void remove(QLayoutItem *item);
     void clear();
     bool isValid() const;
 
     QLayoutItem *plug(QList<int> path);
-    QLayoutItem *unplug(QList<int> path);
+    QLayoutItem *unplug(QList<int> path, QMainWindowLayoutState *savedState = 0);
 
     void saveState(QDataStream &stream) const;
+    bool checkFormat(QDataStream &stream, bool pre43);
     bool restoreState(QDataStream &stream, const QMainWindowLayoutState &oldState);
 };
 
@@ -177,6 +195,7 @@ public:
     void getStyleOptionInfo(QStyleOptionToolBar *option, QToolBar *toolBar) const;
     void removeToolBar(QToolBar *toolbar);
     void toggleToolBarsVisible();
+    void moveToolBar(QToolBar *toolbar, int pos); 
 #endif
 
     // dock widgets
@@ -194,6 +213,7 @@ public:
     Qt::DockWidgetArea dockWidgetArea(QDockWidget *dockwidget) const;
     void raise(QDockWidget *widget);
     void setVerticalTabsEnabled(bool enabled);
+    bool restoreDockWidget(QDockWidget *dockwidget);
 
 #ifndef QT_NO_TABBAR
     QTabBar *getTabBar();
@@ -289,9 +309,11 @@ private:
     bool useHIToolBar;
 #endif
 };
+QT_END_NAMESPACE
 
 #endif // QT_NO_MAINWINDOW
 
+QT_BEGIN_NAMESPACE
 static inline int pick(Qt::Orientation o, const QPoint &pos)
 { return o == Qt::Horizontal ? pos.x() : pos.y(); }
 
@@ -319,4 +341,6 @@ static inline int &rperp(Qt::Orientation o, QPoint &pos)
 static inline int &rperp(Qt::Orientation o, QSize &size)
 { return o == Qt::Vertical ? size.rwidth() : size.rheight(); }
 
+QT_END_NAMESPACE
+
 #endif // QDYNAMICMAINWINDOWLAYOUT_P_H
