@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,16 +49,22 @@
 #include <QtGui/qicon.h>
 #include <QtGui/qaction.h>
 
+#ifdef QT3_SUPPORT
+#include <QtGui/qpixmap.h>
+#endif
+
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_MENU
 
 class QMenuPrivate;
+class QStyleOptionMenuItem;
 #ifdef QT3_SUPPORT
 class QMenuItem;
-#include <QtGui/qpixmap.h>
 #endif
 
 class Q_GUI_EXPORT QMenu : public QWidget
@@ -50,6 +76,7 @@ private:
     Q_PROPERTY(bool tearOffEnabled READ isTearOffEnabled WRITE setTearOffEnabled)
     Q_PROPERTY(QString title READ title WRITE setTitle)
     Q_PROPERTY(QIcon icon READ icon WRITE setIcon)
+    Q_PROPERTY(bool separatorsCollapsible READ separatorsCollapsible WRITE setSeparatorsCollapsible)
 
 public:
     explicit QMenu(QWidget *parent = 0);
@@ -75,6 +102,7 @@ public:
     QAction *insertMenu(QAction *before, QMenu *menu);
     QAction *insertSeparator(QAction *before);
 
+    bool isEmpty() const;
     void clear();
 
     void setTearOffEnabled(bool);
@@ -112,8 +140,17 @@ public:
     MenuRef macMenu(MenuRef merge=0);
 #endif
 
+#ifdef Q_OS_WINCE
+    HMENU wceMenu(bool create = false);
+#endif
+
+
+    bool separatorsCollapsible() const;
+    void setSeparatorsCollapsible(bool collapse);
+
 Q_SIGNALS:
     void aboutToShow();
+    void aboutToHide();
     void triggered(QAction *action);
     void hovered(QAction *action);
 
@@ -133,6 +170,12 @@ protected:
     void actionEvent(QActionEvent *);
     void timerEvent(QTimerEvent *);
     bool event(QEvent *);
+    bool focusNextPrevChild(bool next);
+    void initStyleOption(QStyleOptionMenuItem *option, const QAction *action) const;
+
+#ifdef Q_OS_WINCE
+    QAction* wceCommands(uint command);
+#endif
 
 private Q_SLOTS:
     void internalSetSloppyAction();
@@ -141,6 +184,7 @@ private Q_SLOTS:
 private:
     Q_PRIVATE_SLOT(d_func(), void _q_actionTriggered())
     Q_PRIVATE_SLOT(d_func(), void _q_actionHovered())
+    Q_PRIVATE_SLOT(d_func(), void _q_overrideMenuActionDestroyed())
 
 #ifdef QT3_SUPPORT
 public:
@@ -341,7 +385,6 @@ protected:
     }
 
 Q_SIGNALS:
-    QT_MOC_COMPAT void aboutToHide();
     QT_MOC_COMPAT void activated(int itemId);
     QT_MOC_COMPAT void highlighted(int itemId);
 
@@ -352,6 +395,9 @@ private:
     int findIdForAction(QAction*) const;
 #endif
 
+protected:
+    QMenu(QMenuPrivate &dd, QWidget* parent = 0);
+
 private:
     Q_DISABLE_COPY(QMenu)
 
@@ -360,8 +406,11 @@ private:
     friend class QTornOffMenu;
     friend class Q3PopupMenu;
     friend class QComboBox;
+    friend class QAction;
+    friend class QToolButtonPrivate;
 
 #ifdef Q_WS_MAC
+    friend void qt_mac_trayicon_activate_action(QMenu *, QAction *action);
     friend bool qt_mac_watchingAboutToShow(QMenu *);
     friend OSStatus qt_mac_menu_event(EventHandlerCallRef, EventRef, void *);
     friend bool qt_mac_activate_action(MenuRef, uint, QAction::ActionEvent, bool);
@@ -370,6 +419,8 @@ private:
 
 #endif // QT_NO_MENU
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QMENU_H
