@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -68,6 +68,11 @@
 #include "qmenubar_p.h"
 #include "qdebug.h"
 
+#ifdef Q_OS_WINCE
+extern bool qt_wince_is_mobile(); //defined in qguifunctions_wce.cpp
+#endif
+
+QT_BEGIN_NAMESPACE
 
 class QMenuBarExtension : public QToolButton
 {
@@ -85,12 +90,12 @@ QMenuBarExtension::QMenuBarExtension(QWidget *parent)
 #ifndef QT_NO_MENU
     setPopupMode(QToolButton::InstantPopup);
 #endif
-    setIcon(style()->standardIcon(QStyle::SP_ToolBarHorizontalExtensionButton));
+    setIcon(style()->standardIcon(QStyle::SP_ToolBarHorizontalExtensionButton, 0, parentWidget()));
 }
 
 QSize QMenuBarExtension::sizeHint() const
 {
-    int ext = style()->pixelMetric(QStyle::PM_ToolBarExtensionExtent);
+    int ext = style()->pixelMetric(QStyle::PM_ToolBarExtensionExtent, 0, parentWidget());
     return QSize(ext, ext);
 }
 
@@ -272,7 +277,8 @@ void QMenuBarPrivate::setKeyboardMode(bool b)
         if(!popupState)
             setCurrentAction(0);
         if(keyboardFocusWidget) {
-            keyboardFocusWidget->setFocus();
+            if (qApp->focusWidget() == q)
+                keyboardFocusWidget->setFocus();
             keyboardFocusWidget = 0;
         }
     }
@@ -334,29 +340,25 @@ void QMenuBarPrivate::popupAction(QAction *action, bool activateFirst)
     q->update(actionRect(action));
 }
 
-bool QMenuBarPrivate::closeActiveMenu()
-{
-    if(activeMenu) {
-        QMenu *menu = activeMenu;
-        activeMenu = 0;
-        menu->hide();
-        return true;
-    }
-    return false;
-}
-
 void QMenuBarPrivate::setCurrentAction(QAction *action, bool popup, bool activateFirst)
 {
     if(currentAction == action && popup == popupState)
         return;
 
+    autoReleaseTimer.stop();
+
     doChildEffects = (popup && !activeMenu);
     Q_Q(QMenuBar);
     QWidget *fw = 0;
-    if(closeActiveMenu() && popup) {
-        fw = q->window()->focusWidget();
-        q->setFocus(Qt::NoFocusReason);
+    if(QMenu *menu = activeMenu) {
+        activeMenu = 0;
+        if (popup) {
+            fw = q->window()->focusWidget();
+            q->setFocus(Qt::NoFocusReason);
+        }
+        menu->hide();
     }
+
     if(currentAction)
         q->update(actionRect(currentAction));
 
@@ -365,8 +367,6 @@ void QMenuBarPrivate::setCurrentAction(QAction *action, bool popup, bool activat
     QAction *previousAction = currentAction;
 #endif
     currentAction = action;
-    if (fw)
-        fw->setFocus(Qt::NoFocusReason);
     if (action) {
         activateAction(action, QAction::Hover);
         if(popup)
@@ -379,6 +379,8 @@ void QMenuBarPrivate::setCurrentAction(QAction *action, bool popup, bool activat
         QApplication::sendEvent(q, &tip);
 #endif
     }
+    if (fw)
+        fw->setFocus(Qt::NoFocusReason);
 }
 
 void QMenuBarPrivate::calcActionRects(int max_width, int start, QMap<QAction*, QRect> &actionRects, QList<QAction*> &actionList) const
@@ -572,9 +574,7 @@ void QMenuBar::initStyleOption(QStyleOptionMenuItem *option, const QAction *acti
     menu items with addMenu(). For example, asuming that \c menubar
     is a pointer to a QMenuBar and \c fileMenu is a pointer to a
     QMenu, the following statement inserts the menu into the menu bar:
-    \code
-      menubar->addMenu(fileMenu);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qmenubar.cpp 0
 
     The ampersand in the menu item's text sets Alt+F as a shortcut for
     this menu. (You can use "\&\&" to get a real ampersand in the menu
@@ -584,29 +584,13 @@ void QMenuBar::initStyleOption(QStyleOptionMenuItem *option, const QAction *acti
     own geometry to the top of the parent widget and changes it
     appropriately whenever the parent is resized.
 
-    \omit
-    Example of creating a menu bar with menu items (from \l menu/menu.cpp):
-    \quotefile menu/menu.cpp
-    \skipto file = new QMenu
-    \printline
-    \skipto Qt::Key_O
-    \printline
-    \printline
-    \skipto new QMenuBar
-    \printline
-    \skipto addMenu
-    \printline
-    \endomit
-
     In most main window style applications you would use the menuBar()
     provided in QMainWindow, adding \l{QMenu}s to the menu bar and
     adding \l{QAction}s to the popup menus.
 
     Example (from the \l{mainwindows/menus}{Menus} example):
 
-    \quotefile mainwindows/menus/mainwindow.cpp
-    \skipto fileMenu =
-    \printuntil fileMenu->addAction(
+    \snippet examples/mainwindows/menus/mainwindow.cpp 9
 
     Menu items may be removed with removeAction().
 
@@ -664,14 +648,22 @@ void QMenuBar::initStyleOption(QStyleOptionMenuItem *option, const QAction *acti
     same menu bar, you need to create a menu bar that does not have a
     parent. The menu bar is created like this:
 
-    \code
-        QMenuBar *menuBar = new QMenuBar(0);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qmenubar.cpp 1
 
     \bold{Note:} The text used for the application name in the menu bar is
     obtained from the value set in the \c{Info.plist} file in the application's
     bundle. See \l{Deploying an Application on Qt/Mac} for more information.
 
+    \section1 QMenuBar on Qt/CE
+
+    QMenuBar on Qt/CE is a wrapper for using the system-wide menu bar, similar to the Mac.
+    This feature is activated for Windows Mobile and integrates QMenuBar with the native
+    soft keys. The left soft key can be controlled with QMenuBar::setDefaultAction() and the
+    right soft key can be used to access the menu bar.
+
+    The hovered() signal is not supported for the native menu integration. Also, it is not possible
+    to display an icon in a native menu on Windows Mobile.
+
     \section1 Examples
 
     The \l{mainwindows/menus}{Menus} example shows how to use QMenuBar and QMenu.
@@ -693,6 +685,13 @@ void QMenuBarPrivate::init()
     macCreateMenuBar(q->parentWidget());
     if(mac_menubar)
         q->hide();
+#endif
+#ifdef Q_OS_WINCE
+    if (qt_wince_is_mobile()) {
+        wceCreateMenuBar(q->parentWidget());
+        if(wce_menubar)
+            q->hide();
+    }
 #endif
     q->setBackgroundRole(QPalette::Button);
     oldWindow = oldParent = 0;
@@ -738,6 +737,11 @@ QMenuBar::~QMenuBar()
     Q_D(QMenuBar);
     d->macDestroyMenuBar();
 #endif
+#ifdef Q_OS_WINCE
+    Q_D(QMenuBar);
+    if (qt_wince_is_mobile())
+        d->wceDestroyMenuBar();
+#endif
 }
 
 /*!
@@ -1009,7 +1013,10 @@ void QMenuBar::mousePressEvent(QMouseEvent *e)
     d->mouseDown = true;
 
     if(d->currentAction == action && d->popupState) {
-        d->closeActiveMenu();
+        if(QMenu *menu = d->activeMenu) {
+            d->activeMenu = 0;
+            menu->hide();
+        }
 #ifdef Q_WS_WIN
         if((d->closePopupMode = style()->styleHint(QStyle::SH_MenuBar_DismissOnSecondClick)))
             update(d->actionRect(action));
@@ -1070,6 +1077,7 @@ void QMenuBar::keyPressEvent(QKeyEvent *e)
         } else if(key == Qt::Key_Enter || key == Qt::Key_Return || key == Qt::Key_Space) {
             d->activateAction(d->currentAction, QAction::Trigger);
             d->setCurrentAction(d->currentAction, false);
+            d->setKeyboardMode(false);
         }
         key_consumed = true;
         break; }
@@ -1196,6 +1204,16 @@ void QMenuBar::actionEvent(QActionEvent *e)
         else if(e->type() == QEvent::ActionChanged)
             d->mac_menubar->syncAction(e->action());
     }
+#endif
+#ifdef Q_OS_WINCE
+    if(d->wce_menubar) {
+        if(e->type() == QEvent::ActionAdded)
+            d->wce_menubar->addAction(e->action(), d->wce_menubar->findAction(e->before()));
+        else if(e->type() == QEvent::ActionRemoved)
+            d->wce_menubar->removeAction(e->action());
+        else if(e->type() == QEvent::ActionChanged)
+            d->wce_menubar->syncAction(e->action());
+    }
 #endif
     if(e->type() == QEvent::ActionAdded) {
         connect(e->action(), SIGNAL(triggered()), this, SLOT(_q_actionTriggered()));
@@ -1231,6 +1249,18 @@ void QMenuBar::focusOutEvent(QFocusEvent *)
     }
 }
 
+/*!
+  \reimp
+ */
+void QMenuBar::timerEvent (QTimerEvent *e)
+{
+    Q_D(QMenuBar);
+    if (e->timerId() == d->autoReleaseTimer.timerId()) {
+        d->autoReleaseTimer.stop();
+        d->setCurrentAction(0);
+    }
+    QWidget::timerEvent(e);
+}
 
 
 void QMenuBarPrivate::handleReparent()
@@ -1266,6 +1296,11 @@ void QMenuBarPrivate::handleReparent()
     macDestroyMenuBar();
     macCreateMenuBar(newParent);
 #endif
+
+#ifdef Q_OS_WINCE
+    if (qt_wince_is_mobile() && wce_menubar)
+        wce_menubar->rebuild();
+#endif
 }
 
 #ifdef QT3_SUPPORT
@@ -1363,7 +1398,15 @@ bool QMenuBar::event(QEvent *e)
             }
         }
 #endif
-        d->updateGeometries();
+        d->_q_updateLayout();
+    break;
+    case QEvent::ShortcutOverride: {
+        QKeyEvent *kev = static_cast<QKeyEvent*>(e);
+        if (kev->key() == Qt::Key_Escape) {
+            e->accept();
+            return true;
+        }
+    }
     break;
 
 #ifdef QT3_SUPPORT
@@ -1649,8 +1692,14 @@ int QMenuBar::heightForWidth(int) const
 */
 void QMenuBarPrivate::_q_internalShortcutActivated(int id)
 {
+    Q_Q(QMenuBar);
     QAction *act = actionList.at(id);
     setCurrentAction(act, true, true);
+    if (act && !act->menu()) {
+        activateAction(act, QAction::Trigger);
+        //100 is the same as the default value in QPushButton::animateClick
+        autoReleaseTimer.start(100, q);
+    }
 }
 
 void QMenuBarPrivate::_q_updateLayout()
@@ -1721,6 +1770,57 @@ QWidget *QMenuBar::cornerWidget(Qt::Corner corner) const
     return w;
 }
 
+/*!
+  Sets the default action to \a act.
+
+  The default action is assigned to the left soft key. The menu is assigned
+  to the right soft key.
+
+  Currently there is only support for the default action on Windows
+  Mobile. All other platforms ignore the default action.
+
+  \sa defaultAction()
+*/
+
+#ifdef Q_OS_WINCE
+void QMenuBar::setDefaultAction(QAction *act)
+{
+    Q_D(QMenuBar);
+    if (d->defaultAction == act)
+        return;
+#ifdef Q_OS_WINCE
+    if (qt_wince_is_mobile())
+        if (d->defaultAction) {
+            disconnect(d->defaultAction, SIGNAL(changed()), this, SLOT(_q_updateDefaultAction()));
+            disconnect(d->defaultAction, SIGNAL(destroyed ()), this, SLOT(_q_updateDefaultAction()));
+        }
+#endif
+    d->defaultAction = act;
+#ifdef Q_OS_WINCE
+    if (qt_wince_is_mobile())
+        if (d->defaultAction) {
+            connect(d->defaultAction, SIGNAL(changed()), this, SLOT(_q_updateDefaultAction()));
+            connect(d->defaultAction, SIGNAL(destroyed()), this, SLOT(_q_updateDefaultAction()));
+        }
+    if (d->wce_menubar) {
+        d->wce_menubar->rebuild();
+    }
+#endif
+}
+
+/*!
+  \since 4.4
+
+  Returns the current default action.
+
+  \sa setDefaultAction()
+*/
+QAction *QMenuBar::defaultAction() const
+{
+    return d_func()->defaultAction;
+}
+#endif
+
 /*!
     \fn void QMenuBar::triggered(QAction *action)
 
@@ -2210,5 +2310,9 @@ int QMenuBar::findIdForAction(QAction *act) const
 
 // for private slots
 
+
+QT_END_NAMESPACE
+
 #include <moc_qmenubar.cpp>
+
 #endif // QT_NO_MENUBAR
