@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,12 +53,17 @@
 #endif
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
+
 class QProgressBarPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QProgressBar)
+
 public:
     QProgressBarPrivate();
+
     void init();
+    inline void resetLayoutItemMargins();
 
     int minimum;
     int maximum;
@@ -49,14 +74,15 @@ public:
     Qt::Orientation orientation;
     bool invertedAppearance;
     QProgressBar::Direction textDirection;
-    QStyleOptionProgressBarV2 getStyleOption() const;
+    QString format;
     inline int bound(int val) const { return qMax(minimum-1, qMin(maximum, val)); }
     bool repaintRequired() const;
 };
 
 QProgressBarPrivate::QProgressBarPrivate()
     : minimum(0), maximum(100), value(-1), alignment(Qt::AlignLeft), textVisible(true),
-      lastPaintedValue(-1), orientation(Qt::Horizontal), invertedAppearance(false), textDirection(QProgressBar::TopToBottom)
+      lastPaintedValue(-1), orientation(Qt::Horizontal), invertedAppearance(false),
+      textDirection(QProgressBar::TopToBottom), format(QLatin1String("%p%"))
 {
 }
 
@@ -68,24 +94,48 @@ void QProgressBarPrivate::init()
         sp.transpose();
     q->setSizePolicy(sp);
     q->setAttribute(Qt::WA_WState_OwnSizePolicy, false);
+    resetLayoutItemMargins();
 }
 
-QStyleOptionProgressBarV2 QProgressBarPrivate::getStyleOption() const
+void QProgressBarPrivate::resetLayoutItemMargins()
 {
-    QStyleOptionProgressBarV2 opt;
-    opt.init(q_func());
+    Q_Q(QProgressBar);
+    QStyleOptionProgressBar option;
+    q->initStyleOption(&option);
+    setLayoutItemMargins(QStyle::SE_ProgressBarLayoutItem, &option);
+}
 
-    opt.minimum = minimum;
-    opt.maximum = maximum;
-    opt.progress = value;
-    opt.textAlignment = alignment;
-    opt.textVisible = textVisible;
-    opt.text = q_func()->text();
-    opt.orientation = orientation;
-    opt.invertedAppearance = invertedAppearance;
-    opt.bottomToTop = (textDirection == QProgressBar::BottomToTop);
+/*!
+    Initialize \a option with the values from this QProgressBar. This method is useful
+    for subclasses when they need a QStyleOptionProgressBar or QStyleOptionProgressBarV2,
+    but don't want to fill in all the information themselves. This function will check the version
+    of the QStyleOptionProgressBar and fill in the additional values for a
+    QStyleOptionProgressBarV2.
 
-    return opt;
+    \sa QStyleOption::initFrom()
+*/
+void QProgressBar::initStyleOption(QStyleOptionProgressBar *option) const
+{
+    if (!option)
+        return;
+    Q_D(const QProgressBar);
+    option->initFrom(this);
+
+    if (d->orientation == Qt::Horizontal)
+        option->state |= QStyle::State_Horizontal;
+    option->minimum = d->minimum;
+    option->maximum = d->maximum;
+    option->progress = d->value;
+    option->textAlignment = d->alignment;
+    option->textVisible = d->textVisible;
+    option->text = text();
+
+    if (QStyleOptionProgressBarV2 *optionV2
+            = qstyleoption_cast<QStyleOptionProgressBarV2 *>(option)) {
+        optionV2->orientation = d->orientation;  // ### Qt 5: use State_Horizontal instead
+        optionV2->invertedAppearance = d->invertedAppearance;
+        optionV2->bottomToTop = (d->textDirection == QProgressBar::BottomToTop);
+    }
 }
 
 bool QProgressBarPrivate::repaintRequired() const
@@ -97,12 +147,19 @@ bool QProgressBarPrivate::repaintRequired() const
     int valueDifference = qAbs(value - lastPaintedValue);
 
     // Check if the text needs to be repainted
-    if ((value == minimum || value == maximum)
-            || (textVisible && valueDifference >= qAbs((maximum - minimum) / 100)))
+    if (value == minimum || value == maximum)
         return true;
+    if (textVisible) {
+        if ((format.contains(QLatin1String("%v"))))
+            return true;
+        if ((format.contains(QLatin1String("%p"))
+             && valueDifference >= qAbs((maximum - minimum) / 100)))
+            return true;
+    }
 
     // Check if the bar needs to be repainted
-    QStyleOptionProgressBarV2 opt = getStyleOption();
+    QStyleOptionProgressBarV2 opt;
+    q->initStyleOption(&opt);
     int cw = q->style()->pixelMetric(QStyle::PM_ProgressBarChunkWidth, &opt, q);
     QRect groove  = q->style()->subElementRect(QStyle::SE_ProgressBarGroove, &opt, q);
     // This expression is basically
@@ -115,7 +172,7 @@ bool QProgressBarPrivate::repaintRequired() const
     \class QProgressBar qprogressbar.h
     \brief The QProgressBar widget provides a horizontal or vertical progress bar.
 
-    \ingroup advanced
+    \ingroup basicwidgets
     \mainclass
 
     A progress bar is used to give the user an indication of the
@@ -123,7 +180,7 @@ bool QProgressBarPrivate::repaintRequired() const
     is still running.
 
     The progress bar uses the concept of \e steps. You set it up by
-    specifying the minumum and maximum possible step values, and it
+    specifying the minimum and maximum possible step values, and it
     will display the percentage of steps that have been completed
     when you later give it the current step value. The percentage is
     calculated by dividing the progress (value() - minimum()) divided
@@ -148,7 +205,7 @@ bool QProgressBarPrivate::repaintRequired() const
          \o A progress bar shown in the Plastique widget style.
     \endtable
 
-    \sa QProgressDialog, {fowler}{GUI Design Handbook: Progress Indicator}
+    \sa QTimeLine, QProgressDialog, {fowler}{GUI Design Handbook: Progress Indicator}
 */
 
 /*!
@@ -159,6 +216,10 @@ bool QProgressBarPrivate::repaintRequired() const
     \value TopToBottom The text is rotated 90 degrees clockwise.
     \value BottomToTop The text is rotated 90 degrees counter-clockwise.
 
+    Note that whether or not the text is drawn is dependent on the style.
+    Currently CDE, CleanLooks, Motif, and Plastique draw the text. Mac, Windows
+    and WindowsXP style do not.
+
     \sa textDirection
 */
 
@@ -203,7 +264,7 @@ void QProgressBar::reset()
 
     When setting this property, the \l maximum is adjusted if
     necessary to ensure that the range remains valid. If the
-    current value falls outside the new range, the progressbar is reset
+    current value falls outside the new range, the progress bar is reset
     with reset().
 */
 void QProgressBar::setMinimum(int minimum)
@@ -223,7 +284,7 @@ int QProgressBar::minimum() const
 
     When setting this property, the \l minimum is adjusted if
     necessary to ensure that the range remains valid. If the
-    current value falls outside the new range, the progressbar is reset
+    current value falls outside the new range, the progress bar is reset
     with reset().
 */
 
@@ -266,13 +327,13 @@ int QProgressBar::value() const
 }
 
 /*!
-    Sets the progressbar's minimum and maximum values to \a minimum and
+    Sets the progress bar's minimum and maximum values to \a minimum and
     \a maximum respectively.
 
     If \a maximum is smaller than \a minimum, \a minimum becomes the only
     legal value.
 
-    If the current value falls outside the new range, the progressbar is reset
+    If the current value falls outside the new range, the progress bar is reset
     with reset().
 
     \sa minimum maximum
@@ -328,7 +389,8 @@ Qt::Alignment QProgressBar::alignment() const
 void QProgressBar::paintEvent(QPaintEvent *)
 {
     QStylePainter paint(this);
-    QStyleOptionProgressBarV2 opt = d_func()->getStyleOption();
+    QStyleOptionProgressBarV2 opt;
+    initStyleOption(&opt);
     paint.drawControl(QStyle::CE_ProgressBar, opt);
     d_func()->lastPaintedValue = d_func()->value;
 }
@@ -340,9 +402,10 @@ QSize QProgressBar::sizeHint() const
 {
     ensurePolished();
     QFontMetrics fm = fontMetrics();
-    QStyleOptionProgressBarV2 opt = d_func()->getStyleOption();
+    QStyleOptionProgressBarV2 opt;
+    initStyleOption(&opt);
     int cw = style()->pixelMetric(QStyle::PM_ProgressBarChunkWidth, &opt, this);
-    QSize size = QSize(cw * 7 + fm.width('0') * 4, fm.height() + 8);
+    QSize size = QSize(qMax(9, cw) * 7 + fm.width(QLatin1Char('0')) * 4, fm.height() + 8);
     if (opt.orientation == Qt::Vertical)
         size.transpose();
     return style()->sizeFromContents(QStyle::CT_ProgressBar, &opt, size, this);
@@ -383,21 +446,23 @@ QString QProgressBar::text() const
             || (d->value == INT_MIN && d->minimum == INT_MIN))
         return QString();
 
-    int totalSteps = d->maximum - d->minimum;
+    qint64 totalSteps = qint64(d->maximum) - qint64(d->minimum);
+
+    QString result = d->format;
+    result.replace(QLatin1String("%m"), QString::fromLatin1("%1").arg(totalSteps));
+    result.replace(QLatin1String("%v"), QString::fromLatin1("%1").arg(d->value));
 
     // If max and min are equal and we get this far, it means that the
     // progress bar has one step and that we are on that step. Return
     // 100% here in order to avoid division by zero further down.
-    if (totalSteps == 0)
-        return tr("%1%").arg(100);
-
-    int progress = d->value - d->minimum;
-    // Get the values down to something usable.
-    if (totalSteps > INT_MAX / 1000) {
-        progress /= 1000;
-        totalSteps /= 1000;
+    if (totalSteps == 0) {
+        result.replace(QLatin1String("%p"), QString::fromLatin1("%1").arg(100));
+        return result;
     }
-    return tr("%1%").arg(progress * 100 / totalSteps);
+
+    int progress = int(((qreal(d->value) - qreal(d->minimum)) * 100.0) / totalSteps);
+    result.replace(QLatin1String("%p"), QString::fromLatin1("%1").arg(progress));
+    return result;
 }
 
 /*!
@@ -423,6 +488,7 @@ void QProgressBar::setOrientation(Qt::Orientation orientation)
         setSizePolicy(sp);
         setAttribute(Qt::WA_WState_OwnSizePolicy, false);
     }
+    d->resetLayoutItemMargins();
     update();
     updateGeometry();
 }
@@ -484,7 +550,44 @@ QProgressBar::Direction QProgressBar::textDirection()
 /*! \reimp */
 bool QProgressBar::event(QEvent *e)
 {
+    Q_D(QProgressBar);
+    if (e->type() == QEvent::StyleChange
+#ifdef Q_WS_MAC
+            || e->type() == QEvent::MacSizeChange
+#endif
+            )
+        d->resetLayoutItemMargins();
     return QWidget::event(e);
 }
 
+/*!
+    \since 4.2
+    \property QProgressBar::format
+    \brief the string used to generate the current text
+
+    %p - is replaced by the percentage completed.
+    %v - is replaced by the current value.
+    %m - is replaced by the total number of steps.
+
+    The default value is "%p%".
+
+    \sa text()
+*/
+void QProgressBar::setFormat(const QString &format)
+{
+    Q_D(QProgressBar);
+    if (d->format == format)
+        return;
+    d->format = format;
+    update();
+}
+
+QString QProgressBar::format() const
+{
+    Q_D(const QProgressBar);
+    return d->format;
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_PROGRESSBAR
