@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,11 +58,13 @@
 #include <limits.h>
 #include "qabstractslider_p.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QScrollBar
     \brief The QScrollBar widget provides a vertical or horizontal scroll bar.
 
-    \ingroup basic
+    \ingroup basicwidgets
 
     A scroll bar is a control that enables the user to access parts of a
     document that is larger than the widget used to display it. It provides
@@ -99,7 +121,7 @@
     when the user presses the \key{Page Up} and \key{Page Down} keys, and is
     set with setPageStep(). Smaller changes to the value defined by the
     line step are made using the cursor keys, and this quantity is set with
-    setLineStep().
+    \l{QAbstractSlider::}{setSingleStep()}.
 
     Note that the range of values used is independent of the actual size
     of the scroll bar widget. You do not need to take this into account when
@@ -150,10 +172,10 @@
 
     A scroll bar can be controlled by the keyboard, but it has a
     default focusPolicy() of Qt::NoFocus. Use setFocusPolicy() to
-    enable keyboard interaction with the scrollbar:
+    enable keyboard interaction with the scroll bar:
     \list
-         \i Left/Right move a horizontal scrollbar by one single step.
-         \i Up/Down move a vertical scrollbar by one single step.
+         \i Left/Right move a horizontal scroll bar by one single step.
+         \i Up/Down move a vertical scroll bar by one single step.
          \i PageUp moves up one page.
          \i PageDown moves down one page.
          \i Home moves to the start (mininum).
@@ -171,13 +193,13 @@
     \table 100%
     \row \o \inlineimage macintosh-horizontalscrollbar.png Screenshot of a Macintosh style scroll bar
          \o A scroll bar shown in the \l{Macintosh Style Widget Gallery}{Macintosh widget style}.
-    \row \o \inlineimage windows-horizontalscrollbar.png Screenshot of a Windows XP style scroll bar
+    \row \o \inlineimage windowsxp-horizontalscrollbar.png Screenshot of a Windows XP style scroll bar
          \o A scroll bar shown in the \l{Windows XP Style Widget Gallery}{Windows XP widget style}.
     \row \o \inlineimage plastique-horizontalscrollbar.png Screenshot of a Plastique style scroll bar
          \o A scroll bar shown in the \l{Plastique Style Widget Gallery}{Plastique widget style}.
     \endtable
 
-    \sa QScrollArea, QSlider, QDial, QSpinBox, {fowler}{GUI Design Handbook: Scroll Bar}
+    \sa QScrollArea, QSlider, QDial, QSpinBox, {fowler}{GUI Design Handbook: Scroll Bar}, {Sliders Example}
 */
 
 class QScrollBarPrivate : public QAbstractSliderPrivate
@@ -190,8 +212,8 @@ public:
     int clickOffset, snapBackPosition;
 
     void activateControl(uint control, int threshold = 500);
+    void stopRepeatAction();
     int pixelPosToRangeValue(int pos) const;
-    QStyleOptionSlider getStyleOption() const;
     void init();
     bool updateHoverControl(const QPoint &pos);
     QStyle::SubControl newHoverControl(const QPoint &pos);
@@ -217,7 +239,8 @@ bool QScrollBarPrivate::updateHoverControl(const QPoint &pos)
 QStyle::SubControl QScrollBarPrivate::newHoverControl(const QPoint &pos)
 {
     Q_Q(QScrollBar);
-    QStyleOptionSlider opt = getStyleOption();
+    QStyleOptionSlider opt;
+    q->initStyleOption(&opt);
     opt.subControls = QStyle::SC_All;
     hoverControl = q->style()->hitTestComplexControl(QStyle::CC_ScrollBar, &opt, pos, q);
     if (hoverControl == QStyle::SC_None)
@@ -254,29 +277,52 @@ void QScrollBarPrivate::activateControl(uint control, int threshold)
     }
 
     if (action) {
-        q_func()->triggerAction(action);
         q_func()->setRepeatAction(action, threshold);
+        q_func()->triggerAction(action);
     }
 }
 
-QStyleOptionSlider QScrollBarPrivate::getStyleOption() const
+void QScrollBarPrivate::stopRepeatAction()
 {
-    Q_Q(const QScrollBar);
+    Q_Q(QScrollBar);
+    QStyle::SubControl tmp = pressedControl;
+    q->setRepeatAction(QAbstractSlider::SliderNoAction);
+    pressedControl = QStyle::SC_None;
+
+    if (tmp == QStyle::SC_ScrollBarSlider)
+        q->setSliderDown(false);
+
     QStyleOptionSlider opt;
-    opt.init(q);
-    opt.subControls = QStyle::SC_None;
-    opt.activeSubControls = QStyle::SC_None;
-    opt.orientation = orientation;
-    opt.minimum = minimum;
-    opt.maximum = maximum;
-    opt.sliderPosition = position;
-    opt.sliderValue = value;
-    opt.singleStep = singleStep;
-    opt.pageStep = pageStep;
-    opt.upsideDown = invertedAppearance;
-    if (orientation == Qt::Horizontal)
-        opt.state |= QStyle::State_Horizontal;
-    return opt;
+    q->initStyleOption(&opt);
+    q->repaint(q->style()->subControlRect(QStyle::CC_ScrollBar, &opt, tmp, q));
+}
+
+/*!
+    Initialize \a option with the values from this QScrollBar. This method
+    is useful for subclasses when they need a QStyleOptionSlider, but don't want
+    to fill in all the information themselves.
+
+    \sa QStyleOption::initFrom()
+*/
+void QScrollBar::initStyleOption(QStyleOptionSlider *option) const
+{
+    if (!option)
+        return;
+
+    Q_D(const QScrollBar);
+    option->initFrom(this);
+    option->subControls = QStyle::SC_None;
+    option->activeSubControls = QStyle::SC_None;
+    option->orientation = d->orientation;
+    option->minimum = d->minimum;
+    option->maximum = d->maximum;
+    option->sliderPosition = d->position;
+    option->sliderValue = d->value;
+    option->singleStep = d->singleStep;
+    option->pageStep = d->pageStep;
+    option->upsideDown = d->invertedAppearance;
+    if (d->orientation == Qt::Horizontal)
+        option->state |= QStyle::State_Horizontal;
 }
 
 
@@ -376,7 +422,7 @@ void QScrollBarPrivate::init()
     pressedControl = hoverControl = QStyle::SC_None;
     pointerOutsidePressedControl = false;
     q->setFocusPolicy(Qt::NoFocus);
-    QSizePolicy sp(QSizePolicy::Minimum, QSizePolicy::Fixed);
+    QSizePolicy sp(QSizePolicy::Minimum, QSizePolicy::Fixed, QSizePolicy::Slider);
     if (orientation == Qt::Vertical)
         sp.transpose();
     q->setSizePolicy(sp);
@@ -387,7 +433,7 @@ void QScrollBarPrivate::init()
 /*! \reimp */
 void QScrollBar::contextMenuEvent(QContextMenuEvent *event)
 {
-    if (!style()->styleHint(QStyle::SH_ScrollBar_ContextMenu)) {
+    if (!style()->styleHint(QStyle::SH_ScrollBar_ContextMenu, 0, this)) {
         QAbstractSlider::contextMenuEvent(event);
         return ;
     }
@@ -439,7 +485,8 @@ void QScrollBar::contextMenuEvent(QContextMenuEvent *event)
 QSize QScrollBar::sizeHint() const
 {
     ensurePolished();
-    QStyleOptionSlider opt = d_func()->getStyleOption();
+    QStyleOptionSlider opt;
+    initStyleOption(&opt);
 
     int scrollBarExtent = style()->pixelMetric(QStyle::PM_ScrollBarExtent, &opt, this);
     int scrollBarSliderMin = style()->pixelMetric(QStyle::PM_ScrollBarSliderMin, &opt, this);
@@ -456,19 +503,6 @@ QSize QScrollBar::sizeHint() const
 /*!\reimp */
 void QScrollBar::sliderChange(SliderChange change)
 {
-    Q_D(QScrollBar);
-
-    if (change == SliderValueChange && repeatAction()) {
-       QStyleOptionSlider opt = d->getStyleOption();
-        if((d->pressedControl == QStyle::SC_ScrollBarAddPage
-            || d->pressedControl == QStyle::SC_ScrollBarSubPage)
-           && style()->styleHint(QStyle::SH_ScrollBar_StopMouseOverSlider, 0, this)
-           && style()->hitTestComplexControl(QStyle::CC_ScrollBar, &opt,
-                                             mapFromGlobal(QCursor::pos()),
-                                             this) == QStyle::SC_ScrollBarSlider) {
-            setRepeatAction(SliderNoAction);
-        }
-    }
     QAbstractSlider::sliderChange(change);
 }
 
@@ -487,7 +521,7 @@ bool QScrollBar::event(QEvent *event)
     default:
         break;
     }
-    return QWidget::event(event);
+    return QAbstractSlider::event(event);
 }
 
 /*!
@@ -497,7 +531,8 @@ void QScrollBar::paintEvent(QPaintEvent *)
 {
     Q_D(QScrollBar);
     QPainter p(this);
-    QStyleOptionSlider opt = d->getStyleOption();
+    QStyleOptionSlider opt;
+    initStyleOption(&opt);
     opt.subControls = QStyle::SC_All;
     if (d->pressedControl) {
         opt.activeSubControls = (QStyle::SubControl)d->pressedControl;
@@ -515,9 +550,14 @@ void QScrollBar::paintEvent(QPaintEvent *)
 void QScrollBar::mousePressEvent(QMouseEvent *e)
 {
     Q_D(QScrollBar);
+
+    if (d->repeatActionTimer.isActive())
+        d->stopRepeatAction();
+
     bool midButtonAbsPos = style()->styleHint(QStyle::SH_ScrollBar_MiddleClickAbsolutePosition,
                                              0, this);
-    QStyleOptionSlider opt = d->getStyleOption();
+    QStyleOptionSlider opt;
+    initStyleOption(&opt);
 
     if (d->maximum == d->minimum // no range
         || (e->buttons() & (~e->button())) // another button was clicked before
@@ -530,6 +570,9 @@ void QScrollBar::mousePressEvent(QMouseEvent *e)
     QRect sr = style()->subControlRect(QStyle::CC_ScrollBar, &opt,
                                        QStyle::SC_ScrollBarSlider, this);
     QPoint click = e->pos();
+    QPoint pressValue = click - sr.center() + sr.topLeft();
+    d->pressValue = d->orientation == Qt::Horizontal ? d->pixelPosToRangeValue(pressValue.x()) :
+        d->pixelPosToRangeValue(pressValue.y());
     if (d->pressedControl == QStyle::SC_ScrollBarSlider) {
         d->clickOffset = HORIZONTAL ? (click.x()-sr.x()) : (click.y()-sr.y());
         d->snapBackPosition = d->position;
@@ -566,13 +609,7 @@ void QScrollBar::mouseReleaseEvent(QMouseEvent *e)
     if (e->buttons() & (~e->button())) // some other button is still pressed
         return;
 
-    QStyle::SubControl tmp = d->pressedControl;
-    setRepeatAction(SliderNoAction);
-    d->pressedControl = QStyle::SC_None;
-    if (tmp == QStyle::SC_ScrollBarSlider)
-        setSliderDown(false);
-    QStyleOptionSlider opt = d->getStyleOption();
-    repaint(style()->subControlRect(QStyle::CC_ScrollBar, &opt, tmp, this));
+    d->stopRepeatAction();
 }
 
 
@@ -585,7 +622,8 @@ void QScrollBar::mouseMoveEvent(QMouseEvent *e)
     if (!d->pressedControl)
         return;
 
-    QStyleOptionSlider opt = d->getStyleOption();
+    QStyleOptionSlider opt;
+    initStyleOption(&opt);
     if (!(e->buttons() & Qt::LeftButton
           ||  ((e->buttons() & Qt::MidButton)
                && style()->styleHint(QStyle::SH_ScrollBar_MiddleClickAbsolutePosition, &opt, this))))
@@ -639,7 +677,8 @@ void QScrollBar::mouseMoveEvent(QMouseEvent *e)
 int QScrollBarPrivate::pixelPosToRangeValue(int pos) const
 {
     Q_Q(const QScrollBar);
-    QStyleOptionSlider opt = getStyleOption();
+    QStyleOptionSlider opt;
+    q->initStyleOption(&opt);
     QRect gr = q->style()->subControlRect(QStyle::CC_ScrollBar, &opt,
                                           QStyle::SC_ScrollBarGroove, q);
     QRect sr = q->style()->subControlRect(QStyle::CC_ScrollBar, &opt,
@@ -679,4 +718,16 @@ void QScrollBar::hideEvent(QHideEvent *)
     Use isSliderDown() instead.
 */
 
+/*! \internal
+    Returns the style option for scroll bar.
+*/
+Q_GUI_EXPORT QStyleOptionSlider qt_qscrollbarStyleOption(QScrollBar *scrollbar)
+{
+    QStyleOptionSlider opt;
+    scrollbar->initStyleOption(&opt);
+    return opt;
+}
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_SCROLLBAR
