@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,6 +50,7 @@
 #include "qpainter.h"
 #include "qstyle.h"
 #include "qstyleoption.h"
+#include "qlayout.h"
 #include "qdebug.h"
 #include <QDesktopWidget>
 
@@ -70,6 +71,8 @@
 #define SZ_SIZETOPLEFT      0xf004
 #define SZ_SIZETOPRIGHT     0xf005
 
+QT_BEGIN_NAMESPACE
+
 static QWidget *qt_sizegrip_topLevelWidget(QWidget* w)
 {
     while (w && !w->isWindow() && w->windowType() != Qt::SubWindow)
@@ -77,6 +80,15 @@ static QWidget *qt_sizegrip_topLevelWidget(QWidget* w)
     return w;
 }
 
+static inline bool hasHeightForWidth(QWidget *widget)
+{
+    if (!widget)
+        return false;
+    if (QLayout *layout = widget->layout())
+        return layout->hasHeightForWidth();
+    return widget->sizePolicy().hasHeightForWidth();
+}
+
 class QSizeGripPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QSizeGrip)
@@ -87,7 +99,6 @@ public:
     int d;
     int dxMax;
     int dyMax;
-    bool hiddenByUser;
     Qt::Corner m_corner;
     bool gotMousePress;
 #ifdef Q_WS_MAC
@@ -103,6 +114,27 @@ public:
     {
         return m_corner == Qt::BottomLeftCorner || m_corner == Qt::TopLeftCorner;
     }
+
+    // This slot is invoked by QLayout when the size grip is added to
+    // a layout or reparented after the tlw is shown. This re-implementation is basically
+    // the same as QWidgetPrivate::_q_showIfNotHidden except that it checks
+    // for Qt::WindowFullScreen and Qt::WindowMaximized as well.
+    void _q_showIfNotHidden()
+    {
+        Q_Q(QSizeGrip);
+        bool showSizeGrip = !(q->isHidden() && q->testAttribute(Qt::WA_WState_ExplicitShowHide));
+        QWidget *tlw = qt_sizegrip_topLevelWidget(q);
+        if (tlw && showSizeGrip) {
+            Qt::WindowStates sizeGripNotVisibleState = Qt::WindowFullScreen;
+#ifndef Q_WS_MAC
+            sizeGripNotVisibleState |= Qt::WindowMaximized;
+#endif
+            // Don't show the size grip if the tlw is maximized or in full screen mode.
+            showSizeGrip = !(tlw->windowState() & sizeGripNotVisibleState);
+        }
+        if (showSizeGrip)
+            q->setVisible(true);
+    }
 };
 
 #ifdef Q_WS_MAC
@@ -122,7 +154,6 @@ Qt::Corner QSizeGripPrivate::corner() const
     Q_Q(const QSizeGrip);
     QWidget *tlw = qt_sizegrip_topLevelWidget(const_cast<QSizeGrip *>(q));
     const QPoint sizeGripPos = q->mapTo(tlw, QPoint(0, 0));
-    const QPoint globalPos = tlw->mapFromGlobal(q->mapToGlobal(QPoint(0, 0)));
     bool isAtBottom = sizeGripPos.y() >= tlw->height() / 2;
     bool isAtLeft = sizeGripPos.x() <= tlw->width() / 2;
     if (isAtLeft)
@@ -204,8 +235,7 @@ void QSizeGripPrivate::init()
     Q_Q(QSizeGrip);
     dxMax = 0;
     dyMax = 0;
-    hiddenByUser = false;
-    m_corner = corner();
+    m_corner = q->isLeftToRight() ? Qt::BottomRightCorner : Qt::BottomLeftCorner;
     gotMousePress = false;
 
 #if !defined(QT_NO_CURSOR) && !defined(Q_WS_MAC)
@@ -276,7 +306,8 @@ void QSizeGrip::mousePressEvent(QMouseEvent * e)
 
 #ifdef Q_WS_X11
     // Use a native X11 sizegrip for "real" top-level windows if supported.
-    if (tlw->isWindow() && X11->isSupportedByWM(ATOM(_NET_WM_MOVERESIZE))) {
+    if (tlw->isWindow() && X11->isSupportedByWM(ATOM(_NET_WM_MOVERESIZE))
+        && !tlw->testAttribute(Qt::WA_DontShowOnScreen) && !hasHeightForWidth(tlw)) {
         XEvent xev;
         xev.xclient.type = ClientMessage;
         xev.xclient.message_type = ATOM(_NET_WM_MOVERESIZE);
@@ -297,6 +328,20 @@ void QSizeGrip::mousePressEvent(QMouseEvent * e)
         return;
     }
 #endif // Q_WS_X11
+#ifdef Q_WS_WIN
+    if (tlw->isWindow() && !tlw->testAttribute(Qt::WA_DontShowOnScreen) && !hasHeightForWidth(tlw)) {
+        uint orientation = 0;
+        if (d->atBottom())
+            orientation = d->atLeft() ? SZ_SIZEBOTTOMLEFT : SZ_SIZEBOTTOMRIGHT;
+        else
+            orientation = d->atLeft() ? SZ_SIZETOPLEFT : SZ_SIZETOPRIGHT;
+
+        ReleaseCapture();
+        QT_WA_INLINE(PostMessageW(tlw->winId(), WM_SYSCOMMAND, orientation, 0),
+                     PostMessageA(tlw->winId(), WM_SYSCOMMAND, orientation, 0));
+        return;
+    }
+#endif // Q_WS_WIN
 
     // Find available desktop/workspace geometry.
     QRect availableGeometry;
@@ -374,11 +419,12 @@ void QSizeGrip::mouseMoveEvent(QMouseEvent * e)
 
 #ifdef Q_WS_X11
     if (tlw->isWindow() && X11->isSupportedByWM(ATOM(_NET_WM_MOVERESIZE))
-        && tlw->isTopLevel())
+        && tlw->isTopLevel() && !tlw->testAttribute(Qt::WA_DontShowOnScreen) && !hasHeightForWidth(tlw))
         return;
 #endif
 #ifdef Q_WS_WIN
-    if (tlw->isWindow() && ::GetSystemMenu(tlw->winId(), FALSE) != 0) {
+    if (tlw->isWindow() && GetSystemMenu(tlw->winId(), FALSE) != 0 && internalWinId()
+        && !tlw->testAttribute(Qt::WA_DontShowOnScreen) && !hasHeightForWidth(tlw)) {
         MSG msg;
         while(PeekMessage(&msg, winId(), WM_MOUSEMOVE, WM_MOUSEMOVE, PM_REMOVE));
         return;
@@ -399,7 +445,7 @@ void QSizeGrip::mouseMoveEvent(QMouseEvent * e)
     else
         ns.rwidth() = d->r.width() + qMin(np.x() - d->p.x(), d->dxMax);
 
-    ns = ns.expandedTo(tlw->minimumSize()).expandedTo(tlw->minimumSizeHint()).boundedTo(tlw->maximumSize());
+    ns = QLayout::closestAcceptableSize(tlw, ns);
 
     QPoint p;
     QRect nr(p, ns);
@@ -471,30 +517,32 @@ void QSizeGrip::hideEvent(QHideEvent *hideEvent)
     QWidget::hideEvent(hideEvent);
 }
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 void QSizeGrip::setVisible(bool visible)
 {
-    Q_D(QSizeGrip);
-    d->hiddenByUser = !visible;
     QWidget::setVisible(visible);
 }
 
 /*! \reimp */
 bool QSizeGrip::eventFilter(QObject *o, QEvent *e)
 {
-    Q_D(QSizeGrip);
-    if (d->hiddenByUser || e->type() != QEvent::WindowStateChange)
+    if (isHidden() && testAttribute(Qt::WA_WState_ExplicitShowHide)
+            || e->type() != QEvent::WindowStateChange) {
         return QWidget::eventFilter(o, e);
+    }
     QWidget *tlw = qt_sizegrip_topLevelWidget(this);
     if (o != tlw)
         return QWidget::eventFilter(o, e);
-    QWidget::setVisible((tlw->windowState() &
-                             (Qt::WindowFullScreen
+    Qt::WindowStates sizeGripNotVisibleState = Qt::WindowFullScreen;
 #ifndef Q_WS_MAC
-                              | Qt::WindowMaximized
+    sizeGripNotVisibleState |= Qt::WindowMaximized;
 #endif
-                              ))==0);
-    return false;
+    // Don't show the size grip if the tlw is maximized or in full screen mode.
+    setVisible(!(tlw->windowState() & sizeGripNotVisibleState));
+    setAttribute(Qt::WA_WState_ExplicitShowHide, false);
+    return QWidget::eventFilter(o, e);
 }
 
 /*!
@@ -509,26 +557,12 @@ bool QSizeGrip::event(QEvent *event)
 /*! \reimp */
 bool QSizeGrip::winEvent( MSG *m, long *result )
 {
-    if (m->message != WM_LBUTTONDOWN)
-        return QWidget::winEvent(m, result);
-
-    // toplevel windows use the native size grip on Windows
-    QWidget *w = qt_sizegrip_topLevelWidget(this);
-    if (!w->isWindow())
-        return QWidget::winEvent(m, result);
-
-    Q_D(QSizeGrip);
-    uint orientation = 0;
-    if (d->atBottom())
-        orientation = d->atLeft() ? SZ_SIZEBOTTOMLEFT : SZ_SIZEBOTTOMRIGHT;
-    else
-        orientation = d->atLeft() ? SZ_SIZETOPLEFT : SZ_SIZETOPRIGHT;
-
-    QT_WA_INLINE(PostMessageW(w->winId(), WM_SYSCOMMAND, orientation, 0),
-                 PostMessageA(w->winId(), WM_SYSCOMMAND, orientation, 0));
-
-    return true;
+    return QWidget::winEvent(m, result);
 }
 #endif
 
+QT_END_NAMESPACE
+
+#include "moc_qsizegrip.cpp"
+
 #endif //QT_NO_SIZEGRIP
