@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 #include "private/qabstractslider_p.h"
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 class QSliderPrivate : public QAbstractSliderPrivate
 {
     Q_DECLARE_PUBLIC(QSlider)
@@ -672,3 +674,5 @@ Q_GUI_EXPORT QStyleOptionSlider qt_qsliderStyleOption(QSlider *slider)
 }
 
 #endif
+
+QT_END_NAMESPACE
