@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,8 @@
 
 #include <math.h>
 
+QT_BEGIN_NAMESPACE
+
 //#define QSPINBOX_QSBDEBUG
 #ifdef QSPINBOX_QSBDEBUG
 #  define QSBDEBUG qDebug
@@ -46,7 +68,7 @@ class QSpinBoxPrivate : public QAbstractSpinBoxPrivate
 {
     Q_DECLARE_PUBLIC(QSpinBox)
 public:
-    QSpinBoxPrivate();
+    QSpinBoxPrivate(QWidget *parent = 0);
     void emitSignals(EmitPolicy ep, const QVariant &);
 
     virtual QVariant valueFromText(const QString &n) const;
@@ -55,13 +77,17 @@ public:
                                   QValidator::State &state) const;
     bool isIntermediateValue(const QString &str) const;
     QChar thousand;
+
+    inline void init() {
+        setLayoutItemMargins(QStyle::SE_SpinBoxLayoutItem);
+    }
 };
 
 class QDoubleSpinBoxPrivate : public QAbstractSpinBoxPrivate
 {
     Q_DECLARE_PUBLIC(QDoubleSpinBox)
 public:
-    QDoubleSpinBoxPrivate();
+    QDoubleSpinBoxPrivate(QWidget *parent = 0);
     void emitSignals(EmitPolicy ep, const QVariant &);
     bool isIntermediateValue(const QString &str) const;
 
@@ -80,7 +106,7 @@ public:
     \class QSpinBox
     \brief The QSpinBox class provides a spin box widget.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     QSpinBox is designed to handle integers and discrete sets of
@@ -138,11 +164,9 @@ public:
     for a custom spin box that allows the user to enter icon sizes
     (e.g., "32 x 32"):
 
-    \quotefromfile widgets/icons/iconsizespinbox.cpp
-    \skipto ::valueFromText
-    \printuntil /^\}$/
-    \skipto ::textFromValue
-    \printuntil /^\}$/
+    \snippet examples/widgets/icons/iconsizespinbox.cpp 1
+    \codeline
+    \snippet examples/widgets/icons/iconsizespinbox.cpp 2
 
     See the \l{widgets/icons}{Icons} example for the full source
     code.
@@ -175,8 +199,10 @@ public:
 */
 
 QSpinBox::QSpinBox(QWidget *parent)
-    : QAbstractSpinBox(*new QSpinBoxPrivate, parent)
+    : QAbstractSpinBox(*new QSpinBoxPrivate(parent), parent)
 {
+    Q_D(QSpinBox);
+    d->init();
 }
 
 #ifdef QT3_SUPPORT
@@ -185,9 +211,11 @@ QSpinBox::QSpinBox(QWidget *parent)
     argument and then use setObjectName() instead.
 */
 QSpinBox::QSpinBox(QWidget *parent, const char *name)
-    : QAbstractSpinBox(*new QSpinBoxPrivate, parent)
+    : QAbstractSpinBox(*new QSpinBoxPrivate(parent), parent)
 {
+    Q_D(QSpinBox);
     setObjectName(QString::fromAscii(name));
+    d->init();
 }
 
 /*!
@@ -195,13 +223,14 @@ QSpinBox::QSpinBox(QWidget *parent, const char *name)
     argument and then use setObjectName() instead.
 */
 QSpinBox::QSpinBox(int minimum, int maximum, int step, QWidget *parent, const char *name)
-    : QAbstractSpinBox(*new QSpinBoxPrivate, parent)
+    : QAbstractSpinBox(*new QSpinBoxPrivate(parent), parent)
 {
     Q_D(QSpinBox);
     d->minimum = QVariant(qMin<int>(minimum, maximum));
     d->maximum = QVariant(qMax<int>(minimum, maximum));
     d->singleStep = QVariant(step);
     setObjectName(QString::fromAscii(name));
+    d->init();
 }
 
 #endif
@@ -234,9 +263,7 @@ void QSpinBox::setValue(int value)
     Typical use is to display a unit of measurement or a currency
     symbol. For example:
 
-    \code
-        sb->setPrefix("$");
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 0
 
     To turn off the prefix display, set this property to an empty
     string. The default is no prefix. The prefix is not displayed when
@@ -269,9 +296,7 @@ void QSpinBox::setPrefix(const QString &prefix)
     use is to display a unit of measurement or a currency symbol. For
     example:
 
-    \code
-        sb->setSuffix(" km");
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 1
 
     To turn off the suffix display, set this property to an empty
     string. The default is no suffix. The suffix is not displayed for
@@ -399,14 +424,9 @@ void QSpinBox::setMaximum(int maximum)
     Convenience function to set the \a minimum, and \a maximum values
     with a single function call.
 
-    \code
-    setRange(minimum, maximum);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 2
     is equivalent to:
-    \code
-    setMinimum(minimum);
-    setMaximum(maximum);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 3
 
     \sa minimum maximum
 */
@@ -421,7 +441,7 @@ void QSpinBox::setRange(int minimum, int maximum)
     This virtual function is used by the spin box whenever it needs
     to display the given \a value. The default implementation returns
     a string containing \a value printed in the standard way using
-    QLocale().toString(). Reimplementations may return anything. (See
+    QWidget::locale().toString(). Reimplementations may return anything. (See
     the example in the detailed description.)
 
     Note: QSpinBox does not call this function for specialValueText()
@@ -437,7 +457,7 @@ void QSpinBox::setRange(int minimum, int maximum)
 QString QSpinBox::textFromValue(int value) const
 {
     Q_D(const QSpinBox);
-    QString str = QLocale().toString(value);
+    QString str = locale().toString(value);
     if (qAbs(value) >= 1000) {
         str.remove(d->thousand);
     }
@@ -501,19 +521,15 @@ void QSpinBox::fixup(QString &input) const
     \brief The QDoubleSpinBox class provides a spin box widget that
     takes doubles.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     QDoubleSpinBox allows the user to choose a value by clicking the
     up and down buttons or by pressing Up or Down on the keyboard to
     increase or decrease the value currently displayed. The user can
-    also type the value in manually. If the value is entered directly
-    into the spin box, the value will be changed and valueChanged()
-    will be emitted with the new value when Enter or Return is
-    pressed, when the spin box loses focus or when the spin box is
-    deactivated (see QWidget::windowActivationChanged()). The spin box
-    supports double values but can be extended to use different
-    strings with validate(), textFromValue() and valueFromText().
+    also type the value in manually. The spin box supports double
+    values but can be extended to use different strings with
+    validate(), textFromValue() and valueFromText().
 
     Every time the value changes QDoubleSpinBox emits the
     valueChanged() signal. The current value can be fetched with
@@ -576,7 +592,7 @@ void QSpinBox::fixup(QString &input) const
     \sa setMinimum(), setMaximum(), setSingleStep()
 */
 QDoubleSpinBox::QDoubleSpinBox(QWidget *parent)
-    : QAbstractSpinBox(*new QDoubleSpinBoxPrivate, parent)
+    : QAbstractSpinBox(*new QDoubleSpinBoxPrivate(parent), parent)
 {
 }
 
@@ -613,9 +629,7 @@ void QDoubleSpinBox::setValue(double value)
     Typical use is to display a unit of measurement or a currency
     symbol. For example:
 
-    \code
-        spinbox->setPrefix("$");
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 4
 
     To turn off the prefix display, set this property to an empty
     string. The default is no prefix. The prefix is not displayed when
@@ -649,9 +663,7 @@ void QDoubleSpinBox::setPrefix(const QString &prefix)
     use is to display a unit of measurement or a currency symbol. For
     example:
 
-    \code
-        spinbox->setSuffix(" km");
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 5
 
     To turn off the suffix display, set this property to an empty
     string. The default is no suffix. The suffix is not displayed for
@@ -786,14 +798,9 @@ void QDoubleSpinBox::setMaximum(double maximum)
     Note: The maximum and minimum values will be rounded to match the
     decimals property.
 
-    \code
-    setRange(minimum, maximum);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 6
     is equivalent to:
-    \code
-    setMinimum(minimum);
-    setMaximum(maximum);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qspinbox.cpp 7
 
     \sa minimum maximum
 */
@@ -810,9 +817,10 @@ void QDoubleSpinBox::setRange(double minimum, double maximum)
      \brief the precision of the spin box, in decimals
 
      Sets how many decimals the spinbox will use for displaying and
-     interpreting doubles. The valid decimal range is 0-13. The
-     default is 2. \a decimals will be bounded to a value that is
-     within the valid range.
+     interpreting doubles.
+
+     \warning The results might not be reliable with very high values
+     for \a decimals.
 
      Note: The maximum, minimum and value might change as a result of
      changing this property.
@@ -828,7 +836,7 @@ int QDoubleSpinBox::decimals() const
 void QDoubleSpinBox::setDecimals(int decimals)
 {
     Q_D(QDoubleSpinBox);
-    d->decimals = qBound(0, decimals, 13);
+    d->decimals = qMax(0, decimals);
 
     setRange(minimum(), maximum()); // make sure values are rounded
     setValue(value());
@@ -837,7 +845,7 @@ void QDoubleSpinBox::setDecimals(int decimals)
 /*!
     This virtual function is used by the spin box whenever it needs to
     display the given \a value. The default implementation returns a string
-    containing \a value printed using QLocale().toString(\a value,
+    containing \a value printed using QWidget::locale().toString(\a value,
     QLatin1Char('f'), decimals()) and will remove the thousand
     separator. Reimplementations may return anything.
 
@@ -855,7 +863,7 @@ void QDoubleSpinBox::setDecimals(int decimals)
 QString QDoubleSpinBox::textFromValue(double value) const
 {
     Q_D(const QDoubleSpinBox);
-    QString str = QLocale().toString(value, 'f', d->decimals);
+    QString str = locale().toString(value, 'f', d->decimals);
     if (qAbs(value) >= 1000.0) {
         str.remove(d->thousand);
     }
@@ -914,14 +922,14 @@ void QDoubleSpinBox::fixup(QString &input) const
     Constructs a QSpinBoxPrivate object
 */
 
-QSpinBoxPrivate::QSpinBoxPrivate()
+QSpinBoxPrivate::QSpinBoxPrivate(QWidget *parent)
 {
     minimum = QVariant((int)0);
     maximum = QVariant((int)99);
     value = minimum;
     singleStep = QVariant((int)1);
     type = QVariant::Int;
-    const QString str = QLocale().toString(4567);
+    const QString str = (parent ? parent->locale() : QLocale()).toString(4567);
     if (str.size() == 5) {
         thousand = QChar(str.at(1));
     }
@@ -977,7 +985,7 @@ QVariant QSpinBoxPrivate::valueFromText(const QString &text) const
 
 bool QSpinBoxPrivate::isIntermediateValue(const QString &str) const
 {
-    const int num = QLocale().toInt(str, 0, 10);
+    const int num = q_func()->locale().toInt(str, 0, 10);
     const int min = minimum.toInt();
     const int max = maximum.toInt();
 
@@ -1047,13 +1055,13 @@ QVariant QSpinBoxPrivate::validateAndInterpret(QString &input, int &pos,
     } else {
         bool ok = false;
         bool removedThousand = false;
-        num = QLocale().toInt(copy, &ok, 10);
+        num = q_func()->locale().toInt(copy, &ok, 10);
         if (!ok && copy.contains(thousand) && (max >= 1000 || min <= -1000)) {
             const int s = copy.size();
             copy.remove(thousand);
             pos = qMax(0, pos - (s - copy.size()));
             removedThousand = true;
-            num = QLocale().toInt(copy, &ok, 10);
+            num = q_func()->locale().toInt(copy, &ok, 10);
         }
         QSBDEBUG() << __FILE__ << __LINE__<< "num is set to" << num;
         if (!ok) {
@@ -1092,7 +1100,7 @@ QVariant QSpinBoxPrivate::validateAndInterpret(QString &input, int &pos,
     Constructs a QSpinBoxPrivate object
 */
 
-QDoubleSpinBoxPrivate::QDoubleSpinBoxPrivate()
+QDoubleSpinBoxPrivate::QDoubleSpinBoxPrivate(QWidget *parent)
 {
     minimum = QVariant(0.0);
     maximum = QVariant(99.99);
@@ -1100,7 +1108,7 @@ QDoubleSpinBoxPrivate::QDoubleSpinBoxPrivate()
     singleStep = QVariant(1.0);
     decimals = 2;
     type = QVariant::Double;
-    const QString str = QLocale().toString(4567.1);
+    const QString str = (parent ? parent->locale() : QLocale()).toString(4567.1);
     if (str.size() == 6) {
         delimiter = str.at(4);
         thousand = QChar((ushort)0);
@@ -1243,8 +1251,9 @@ QVariant QDoubleSpinBoxPrivate::valueFromText(const QString &f) const
 
 double QDoubleSpinBoxPrivate::round(double value) const
 {
-    const QString strDbl = QString::number(value, 'f', decimals);
-    return strDbl.toDouble();
+    Q_Q(const QDoubleSpinBox);
+    const QString strDbl = q->locale().toString(value, 'f', decimals);
+    return q->locale().toDouble(strDbl);
 }
 
 
@@ -1336,7 +1345,7 @@ QVariant QDoubleSpinBoxPrivate::validateAndInterpret(QString &input, int &pos,
 
     {
         bool ok = false;
-        QLocale loc;
+        QLocale loc(q_func()->locale());
         num = loc.toDouble(copy, &ok);
         QSBDEBUG() << __FILE__ << __LINE__ << loc << copy << num << ok;
         bool notAcceptable = false;
@@ -1509,7 +1518,16 @@ static bool isIntermediateValueHelper(qint64 num, qint64 min, qint64 max, qint64
 /*! \reimp */
 bool QSpinBox::event(QEvent *event)
 {
+    Q_D(QSpinBox);
+    if (event->type() == QEvent::StyleChange
+#ifdef Q_WS_MAC
+            || event->type() == QEvent::MacSizeChange
+#endif
+            )
+        d->setLayoutItemMargins(QStyle::SE_SpinBoxLayoutItem);
     return QAbstractSpinBox::event(event);
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SPINBOX
