@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,9 +63,9 @@
 
 #include <ctype.h>
 
-//#define QSPLITTER_DEBUG
+QT_BEGIN_NAMESPACE
 
-static int mouseOffset;
+//#define QSPLITTER_DEBUG
 
 /*!
     \class QSplitterHandle
@@ -56,14 +76,37 @@ static int mouseOffset;
     QSplitterHandle is typically what people think about when they think about
     a splitter. It is the handle that is used to resize the widgets.
 
-    A typical developer using QSplitter will never have to worry about QSplitterHandle.
-    It is provided for developers who want splitter handles that do more. The typical
-    way one would create splitter handles is to subclass QSplitter and then
-    reimplement QSplitter::createHandle() to instatiate the custom splitter
-    handle.
+    A typical developer using QSplitter will never have to worry about
+    QSplitterHandle. It is provided for developers who want splitter handles
+    that provide extra features, such as popup menus.
 
-    Most of the functions inside QSplitterHandle are forwards to QSplitter or
-    like orientation() and opaqueResize(), controlled by the QSplitter.
+    The typical way one would create splitter handles is to subclass QSplitter then
+    reimplement QSplitter::createHandle() to instantiate the custom splitter
+    handle. For example, a minimum QSplitter subclass might look like this:
+
+    \snippet doc/src/snippets/splitterhandle/splitter.h 0
+
+    The \l{QSplitter::}{createHandle()} implementation simply constructs a
+    custom splitter handle, called \c Splitter in this example:
+
+    \snippet doc/src/snippets/splitterhandle/splitter.cpp 1
+
+    Information about a given handle can be obtained using functions like
+    orientation() and opaqueResize(), and is retrieved from its parent splitter.
+    Details like these can be used to give custom handles different appearances
+    depending on the splitter's orientation.
+
+    The complexity of a custom handle subclass depends on the tasks that it
+    needs to perform. A simple subclass might only provide a paintEvent()
+    implementation:
+
+    \snippet doc/src/snippets/splitterhandle/splitter.cpp 0
+
+    In this example, a predefined gradient is set up differently depending on
+    the orientation of the handle. QSplitterHandle provides a reasonable
+    size hint for the handle, so the subclass does not need to provide a
+    reimplementation of sizeHint() unless the handle has special size
+    requirements.
 
     \sa QSplitter
 */
@@ -213,7 +256,7 @@ void QSplitterHandle::mouseMoveEvent(QMouseEvent *e)
     if (!(e->buttons() & Qt::LeftButton))
         return;
     int pos = d->pick(parentWidget()->mapFromGlobal(e->globalPos()))
-                 - mouseOffset;
+                 - d->mouseOffset;
     if (opaqueResize()) {
         moveSplitter(pos);
     } else {
@@ -228,7 +271,7 @@ void QSplitterHandle::mousePressEvent(QMouseEvent *e)
 {
     Q_D(QSplitterHandle);
     if (e->button() == Qt::LeftButton)
-        mouseOffset = d->pick(e->pos());
+        d->mouseOffset = d->pick(e->pos());
 }
 
 /*!
@@ -239,7 +282,7 @@ void QSplitterHandle::mouseReleaseEvent(QMouseEvent *e)
     Q_D(QSplitterHandle);
     if (!opaqueResize() && e->button() == Qt::LeftButton) {
         int pos = d->pick(parentWidget()->mapFromGlobal(e->globalPos()))
-                     - mouseOffset;
+                     - d->mouseOffset;
         d->s->setRubberBand(-1);
         moveSplitter(pos);
     }
@@ -431,7 +474,7 @@ void QSplitterPrivate::doResize()
                 a[j].sizeHint = a[j].minimumSize;
                 a[j].expansive = true;
             } else {
-                a[j].sizeHint = s->getWidgetSize(orient);
+                a[j].sizeHint = qMax(s->getWidgetSize(orient), a[j].minimumSize);
             }
         }
         ++j;
@@ -776,6 +819,23 @@ QSplitter::QSplitter(Qt::Orientation orientation, QWidget *parent, const char *n
 }
 #endif
 
+/*!
+    \internal
+*/
+void QSplitterPrivate::insertWidget_helper(int index, QWidget *widget, bool show)
+{
+    Q_Q(QSplitter);
+    QBoolBlocker b(blockChildAdd);
+    bool needShow = show && q->isVisible() &&
+                    !(widget->isHidden() && widget->testAttribute(Qt::WA_WState_ExplicitShowHide));
+    if (widget->parentWidget() != q)
+        widget->setParent(q);
+    if (needShow)
+        widget->show();
+    insertWidget(index, widget);
+    recalc(q->isVisible());
+}
+
 /*
     Inserts the widget \a w at position \a index in the splitter's list of widgets.
 
@@ -843,9 +903,7 @@ QSplitterLayoutStruct *QSplitterPrivate::insertWidget(int index, QWidget *w)
     The following example will show a QListView, QTreeView, and
     QTextEdit side by side, with two splitter handles:
 
-    \quotefromfile snippets/splitter/splitter.cpp
-    \skipto  QSplitter
-    \printuntil splitter->addWidget(textedit);
+    \snippet doc/src/snippets/splitter/splitter.cpp 0
 
     If a widget is already inside a QSplitter when insertWidget() or
     addWidget() is called, it will move to the new position. This can be used
@@ -1007,7 +1065,7 @@ void QSplitter::setCollapsible(int index, bool collapse)
     Q_D(QSplitter);
 
     if (index < 0 || index >= d->list.size()) {
-        qWarning("QSplitter::setCollapsible() index %d out of range", index);
+        qWarning("QSplitter::setCollapsible: Index %d out of range", index);
         return;
     }
     d->list.at(index)->collapsible = collapse ? 1 : 0;
@@ -1020,7 +1078,7 @@ bool QSplitter::isCollapsible(int index) const
 {
     Q_D(const QSplitter);
     if (index < 0 || index >= d->list.size()) {
-        qWarning("QSplitter::isCollapsible() index %d out of range", index);
+        qWarning("QSplitter::isCollapsible: Index %d out of range", index);
         return false;
     }
     return d->list.at(index)->collapsible;
@@ -1062,15 +1120,7 @@ void QSplitter::addWidget(QWidget *widget)
 void QSplitter::insertWidget(int index, QWidget *widget)
 {
     Q_D(QSplitter);
-    QBoolBlocker b(d->blockChildAdd);
-    bool needShow = isVisible() &&
-                    !(widget->isHidden() && widget->testAttribute(Qt::WA_WState_ExplicitShowHide));
-    if (widget->parentWidget() != this)
-        widget->setParent(this);
-    if (needShow)
-        widget->show();
-    d->insertWidget(index, widget);
-    d->recalc(isVisible());
+    d->insertWidget_helper(index, widget, true);
 }
 
 /*!
@@ -1160,7 +1210,7 @@ int QSplitter::count() const
 
     This method is also used to handle the situation where a widget is created
     with the splitter as a parent but not explicitly added with insertWidget()
-    or addWidget(). This is for compatiblity and not the recommended way of
+    or addWidget(). This is for compatibility and not the recommended way of
     putting widgets into a splitter in new code. Please use insertWidget() or
     addWidget() in new code.
 
@@ -1175,8 +1225,11 @@ void QSplitter::childEvent(QChildEvent *c)
     QWidget *w = static_cast<QWidget*>(c->child());
 
     if (c->added() && !d->blockChildAdd && !w->isWindow() && !d->findWidget(w)) {
-        addWidget(w);
-    } else  if (c->type() == QEvent::ChildRemoved) {
+        d->insertWidget_helper(d->list.count(), w, false);
+    } else if (c->polished() && !d->blockChildAdd) {
+        if (isVisible() && !(w->isHidden() && w->testAttribute(Qt::WA_WState_ExplicitShowHide)))
+            w->show();
+    } else if (c->type() == QEvent::ChildRemoved) {
         for (int i = 0; i < d->list.size(); ++i) {
             QSplitterLayoutStruct *s = d->list.at(i);
             if (s->widget == w) {
@@ -1420,6 +1473,23 @@ void QSplitter::setOpaqueResize(bool on)
 
     Use setCollapsible(indexOf(\a widget, \a collapsible)) instead.
 */
+
+/*!
+    \fn void QSplitter::setMargin(int margin)
+    Sets the width of the margin around the contents of the widget to \a margin.
+    
+    Use QWidget::setContentsMargins() instead.
+    \sa margin(), QWidget::setContentsMargins()
+*/
+
+/*!
+    \fn int QSplitter::margin() const
+    Returns the with of the the margin around the contents of the widget.
+    
+    Use QWidget::getContentsMargins() instead.
+    \sa setMargin(), QWidget::getContentsMargins()
+*/
+
 #endif
 
 /*!
@@ -1457,16 +1527,24 @@ QSize QSplitter::minimumSizeHint() const
     ensurePolished();
     int l = 0;
     int t = 0;
-    QObjectList childList = children();
-    for (int i = 0; i < childList.size(); ++i) {
-        if (QWidget *w = qobject_cast<QWidget *>(childList.at(i))) {
-            if (w->isHidden())
-                continue;
-            QSize s = qSmartMinSize(w);
-            if (s.isValid()) {
-                l += d->pick(s);
-                t = qMax(t, d->trans(s));
-            }
+
+    for (int i = 0; i < d->list.size(); ++i) {
+        QSplitterLayoutStruct *s = d->list.at(i);
+        if (!s || !s->widget)
+            continue;
+        if (s->widget->isHidden())
+            continue;
+        QSize widgetSize = qSmartMinSize(s->widget);
+        if (widgetSize.isValid()) {
+            l += d->pick(widgetSize);
+            t = qMax(t, d->trans(widgetSize));
+        }
+        if (!s->handle || s->handle->isHidden())
+            continue;
+        QSize splitterSize = s->handle->sizeHint();
+        if (splitterSize.isValid()) {
+            l += d->pick(splitterSize);
+            t = qMax(t, d->trans(splitterSize));
         }
     }
     return orientation() == Qt::Horizontal ? QSize(l, t) : QSize(t, l);
@@ -1484,11 +1562,11 @@ QSize QSplitter::minimumSizeHint() const
     Giving the values to another splitter's setSizes() function will
     produce a splitter with the same layout as this one.
 
+    Note that invisible widgets have a size of 0.
+
     The easiest way to iterate over the list is to use the Java-style iterators.
 
-    \quotefromfile snippets/splitter/splitter.cpp
-    \skipto   QListIterator<int>
-    \printuntil processSize(it.next());
+    \snippet doc/src/snippets/splitter/splitter.cpp 3
 
     \sa setSizes()
 */
@@ -1517,7 +1595,8 @@ QList<int> QSplitter::sizes() const
     the program will still be well-behaved.
 
     The values in \a list should be the height or width (depending on
-    orientation()) that the widgets should be resized to.
+    orientation()) that the widgets should be resized to. If you specify
+    a size of 0, the widget will be invisible.
 
     \sa sizes()
 */
@@ -1591,10 +1670,7 @@ static const qint32 SplitterMagic = 0xff;
     for a future session. A version number is stored as part of the data.
     Here is an example:
 
-    \quotefromfile snippets/splitter/splitter.cpp
-    \skipto SAVE
-    \skipto QSettings
-    \printuntil saveState()
+    \snippet doc/src/snippets/splitter/splitter.cpp 1
 
     \sa restoreState()
 */
@@ -1616,16 +1692,17 @@ QByteArray QSplitter::saveState() const
 
 /*!
     Restores the splitter's layout to the \a state specified.
+    Returns true if the state is restored; otherwise returns false.
 
     Typically this is used in conjunction with QSettings to restore the size
     from a past session. Here is an example:
 
     Restore the splitters's state:
 
-    \quotefromfile snippets/splitter/splitter.cpp
-    \skipto RESTORE
-    \skipto QSettings
-    \printuntil restoreState(
+    \snippet doc/src/snippets/splitter/splitter.cpp 2
+
+    A failure to restore the splitter's layout may result from either
+    invalid or out-of-date data in the supplied byte array.
 
     \sa saveState()
 */
@@ -1671,13 +1748,7 @@ bool QSplitter::restoreState(const QByteArray &state)
 
     This function is provided for convenience. It is equivalent to
 
-    \code
-        QWidget *widget = splitter->widget(index);
-        QSizePolicy policy = widget->sizePolicy();
-        policy.setHorizontalStretch(stretch);
-        policy.setVerticalStretch(stretch);
-        widget->setSizePolicy(policy);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qsplitter.cpp 0
 
     \sa setSizes(), widget()
 */
@@ -1721,7 +1792,7 @@ QTextStream& operator>>(QTextStream& ts, QSplitter& splitter)
 {
     QString line = ts.readLine();
     line = line.simplified();
-    line.replace(' ', QString());
+    line.replace(QLatin1Char(' '), QString());
     line = line.toUpper();
 
     splitter.restoreState(line.toAscii());
@@ -1729,4 +1800,7 @@ QTextStream& operator>>(QTextStream& ts, QSplitter& splitter)
 }
 #endif // QT_NO_TEXTSTREAM
 //#endif // QT3_SUPPORT
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_SPLITTER
