@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +49,8 @@
 #include <qevent.h>
 #include <private/qframe_p.h>
 
+QT_BEGIN_NAMESPACE
+
 class QStackedWidgetPrivate : public QFramePrivate
 {
     Q_DECLARE_PUBLIC(QStackedWidget)
@@ -55,22 +77,16 @@ public:
     Like QStackedLayout, QStackedWidget can be constructed and
     populated with a number of child widgets ("pages"):
 
-    \quotefromfile snippets/qstackedwidget/main.cpp
-    \skipto firstPageWidget
-    \printto QComboBox
-    \skipto QVBoxLayout
-    \printline QVBoxLayout
-    \skipto layout->addWidget(stackedWidget)
-    \printuntil setLayout
+    \snippet doc/src/snippets/qstackedwidget/main.cpp 0
+    \snippet doc/src/snippets/qstackedwidget/main.cpp 2
+    \snippet doc/src/snippets/qstackedwidget/main.cpp 3
 
     QStackedWidget provides no intrinsic means for the user to switch
     page. This is typically done through a QComboBox or a QListWidget
     that stores the titles of the QStackedWidget's pages. For
     example:
 
-    \quotefromfile snippets/qstackedwidget/main.cpp
-    \skipto QComboBox
-    \printto QVBoxLayout
+    \snippet doc/src/snippets/qstackedwidget/main.cpp 1
 
     When populating a stacked widget, the widgets are added to an
     internal list. The indexOf() function returns the index of a
@@ -267,4 +283,7 @@ bool QStackedWidget::event(QEvent *e)
 {
     return QFrame::event(e);
 }
+
+QT_END_NAMESPACE
+
 #endif // QT_NO_STACKEDWIDGET
