@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +56,8 @@
 
 #include <private/qwidget_p.h>
 
+QT_BEGIN_NAMESPACE
+
 class QStatusBarPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QStatusBar)
@@ -58,6 +80,7 @@ public:
 
 #ifndef QT_NO_SIZEGRIP
     QSizeGrip * resizer;
+    bool showSizeGrip;
 #endif
 
     int savedStrut;
@@ -72,6 +95,20 @@ public:
         }
         return i;
     }
+
+#ifndef QT_NO_SIZEGRIP
+    void tryToShowSizeGrip()
+    {
+        if (!showSizeGrip)
+            return;
+        showSizeGrip = false;
+        if (!resizer || resizer->isVisible())
+            return;
+        resizer->setAttribute(Qt::WA_WState_ExplicitShowHide, false);
+        QMetaObject::invokeMethod(resizer, "_q_showIfNotHidden", Qt::DirectConnection);
+        resizer->setAttribute(Qt::WA_WState_ExplicitShowHide, false);
+    }
+#endif
 };
 
 /*!
@@ -107,16 +144,12 @@ public:
 
     Use the showMessage() slot to display a \e temporary message:
 
-    \quotefromfile mainwindows/dockwidgets/mainwindow.cpp
-    \skipto MainWindow::createStatusBar()
-    \printuntil /^\}/
+    \snippet examples/mainwindows/dockwidgets/mainwindow.cpp 8
 
     To remove a temporary message, use the clearMessage() slot, or set
     a time limit when calling showMessage(). For example:
 
-    \quotefromfile mainwindows/dockwidgets/mainwindow.cpp
-    \skipto void MainWindow::print()
-    \printuntil /^\}/
+    \snippet examples/mainwindows/dockwidgets/mainwindow.cpp 3
 
     Use the currentMessage() function to retrieve the temporary
     message currently shown. The QStatusBar class also provide the
@@ -129,9 +162,7 @@ public:
     addPermanentWidget() function. Use the removeWidget() function to
     remove such messages from the status bar.
 
-    \code
-        statusBar()->addWidget(new MyReadWriteIndication);
-    \endcode
+    \snippet doc/src/snippets/code/src.gui.widgets.qstatusbar.cpp 0
 
     By default QStatusBar provides a QSizeGrip in the lower-right
     corner. You can disable it using the setSizeGripEnabled()
@@ -169,6 +200,7 @@ QStatusBar::QStatusBar(QWidget * parent, const char *name)
 
 #ifndef QT_NO_SIZEGRIP
     d->resizer = 0;
+    d->showSizeGrip = false;
     setSizeGripEnabled(true); // causes reformat()
 #else
     reformat();
@@ -353,8 +385,11 @@ int QStatusBar::insertPermanentWidget(int index, QWidget *widget, int stretch)
 }
 
 /*!
-    Removes the specified \a widget from the status bar (without
-    deleting it).
+    Removes the specified \a widget from the status bar.
+    
+    \note This function does not delete the widget but \e hides it.
+    To add the widget again, you must call both the addWidget() and
+    show() functions.
 
     \sa addWidget(), addPermanentWidget(), clearMessage()
 */
@@ -416,13 +451,17 @@ void QStatusBar::setSizeGripEnabled(bool enabled)
     if (!enabled != !d->resizer) {
         if (enabled) {
             d->resizer = new QSizeGrip(this);
+            d->resizer->hide();
+            d->resizer->installEventFilter(this);
+            d->showSizeGrip = true;
         } else {
             delete d->resizer;
             d->resizer = 0;
+            d->showSizeGrip = false;
         }
         reformat();
         if (d->resizer && isVisible())
-            d->resizer->show();
+            d->tryToShowSizeGrip();
     }
 #endif
 }
@@ -456,8 +495,8 @@ void QStatusBar::reformat()
     vbox->addSpacing(3);
     QBoxLayout* l = new QHBoxLayout;
     vbox->addLayout(l);
-    l->addSpacing(3);
-    l->setSpacing(4);
+    l->addSpacing(2);
+    l->setSpacing(6);
 
     int maxH = fontMetrics().height();
 
@@ -482,7 +521,6 @@ void QStatusBar::reformat()
         int itemH = qMin(qSmartMinSize(item->w).height(), item->w->maximumHeight());
         maxH = qMax(maxH, itemH);
     }
-    l->addSpacing(4);
 #ifndef QT_NO_SIZEGRIP
     if (d->resizer) {
         maxH = qMax(maxH, d->resizer->sizeHint().height());
@@ -539,7 +577,7 @@ void QStatusBar::clearMessage()
     if (d->tempItem.isEmpty())
         return;
     if (d->timer) {
-        delete d->timer;
+        qDeleteInEventHandler(d->timer);
         d->timer = 0;
     }
     d->tempItem.clear();
@@ -596,16 +634,30 @@ void QStatusBar::hideOrShow()
         item = d->items.at(i);
         if (!item || item->p)
             break;
-        if (haveMessage)
+        if (haveMessage && item->w->isVisible()) {
             item->w->hide();
-        else
+            item->w->setAttribute(Qt::WA_WState_ExplicitShowHide, false);
+        } else if (!haveMessage && !item->w->testAttribute(Qt::WA_WState_ExplicitShowHide)) {
             item->w->show();
+        }
     }
 
     emit messageChanged(d->tempItem);
     repaint();
 }
 
+/*!
+  \reimp
+ */
+void QStatusBar::showEvent(QShowEvent *)
+{
+#ifndef QT_NO_SIZEGRIP
+    Q_D(QStatusBar);
+    if (d->resizer && d->showSizeGrip)
+        d->tryToShowSizeGrip();
+#endif
+}
+
 
 /*!
     \reimp
@@ -620,6 +672,10 @@ void QStatusBar::paintEvent(QPaintEvent *)
     bool haveMessage = !d->tempItem.isEmpty();
 
     QPainter p(this);
+    QStyleOption opt;
+    opt.initFrom(this);
+    style()->drawPrimitive(QStyle::PE_PanelStatusBar, &opt, &p, this);
+
     QStatusBarPrivate::SBItem* item = 0;
 
     bool rtl = layoutDirection() == Qt::RightToLeft;
@@ -649,11 +705,11 @@ void QStatusBar::paintEvent(QPaintEvent *)
                         right = qMin(right, item->w->x()-1);
                 }
                 QStyleOption opt(0);
-                opt.rect.setRect(item->w->x() - 1, item->w->y() - 1,
-                                 item->w->width() + 2, item->w->height() + 2);
+                opt.rect.setRect(item->w->x() - 2, item->w->y() - 1,
+                                 item->w->width() + 4, item->w->height() + 2);
                 opt.palette = palette();
                 opt.state = QStyle::State_None;
-                style()->drawPrimitive(QStyle::PE_FrameStatusBar, &opt, &p, item->w);
+                style()->drawPrimitive(QStyle::PE_FrameStatusBarItem, &opt, &p, item->w);
             }
     }
     if (haveMessage) {
@@ -720,4 +776,6 @@ bool QStatusBar::event(QEvent *e)
     return QWidget::event(e);
 }
 
+QT_END_NAMESPACE
+
 #endif
