@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,12 +48,15 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_TABBAR
 
 class QIcon;
 class QTabBarPrivate;
+class QStyleOptionTab;
 
 class Q_GUI_EXPORT QTabBar: public QWidget
 {
@@ -45,6 +68,8 @@ class Q_GUI_EXPORT QTabBar: public QWidget
     Q_PROPERTY(int count READ count)
     Q_PROPERTY(bool drawBase READ drawBase WRITE setDrawBase)
     Q_PROPERTY(QSize iconSize READ iconSize WRITE setIconSize)
+    Q_PROPERTY(Qt::TextElideMode elideMode READ elideMode WRITE setElideMode)
+    Q_PROPERTY(bool usesScrollButtons READ usesScrollButtons WRITE setUsesScrollButtons)
 
 public:
     explicit QTabBar(QWidget* parent=0);
@@ -77,10 +102,13 @@ public:
 
     QColor tabTextColor(int index) const;
     void setTabTextColor(int index, const QColor &color);
-    
+
     QIcon tabIcon(int index) const;
     void setTabIcon(int index, const QIcon &icon);
 
+    Qt::TextElideMode elideMode() const;
+    void setElideMode(Qt::TextElideMode);
+
 #ifndef QT_NO_TOOLTIP
     void setTabToolTip(int index, const QString &tip);
     QString tabToolTip(int index) const;
@@ -95,6 +123,8 @@ public:
     QVariant tabData(int index) const;
 
     QRect tabRect(int index) const;
+    int tabAt(const QPoint &pos) const;
+
 
     int currentIndex() const;
     int count() const;
@@ -108,6 +138,9 @@ public:
     QSize iconSize() const;
     void setIconSize(const QSize &size);
 
+    bool usesScrollButtons() const;
+    void setUsesScrollButtons(bool useButtons);
+
 public Q_SLOTS:
     void setCurrentIndex(int index);
 
@@ -129,6 +162,7 @@ protected:
     void mouseReleaseEvent (QMouseEvent *);
     void keyPressEvent(QKeyEvent *);
     void changeEvent(QEvent *);
+    void initStyleOption(QStyleOptionTab *option, int tabIndex) const;
 
 #ifdef QT3_SUPPORT
 public Q_SLOTS:
@@ -146,6 +180,8 @@ private:
 
 #endif // QT_NO_TABBAR
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QTABBAR_H
