@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,6 +50,8 @@
 #include <qtoolbar.h>
 #include <qpainter.h>
 
+QT_BEGIN_NAMESPACE
+
 void QToolBarSeparator::initStyleOption(QStyleOption *option) const
 {
     option->initFrom(this);
@@ -86,4 +88,6 @@ void QToolBarSeparator::paintEvent(QPaintEvent *)
     style()->drawPrimitive(QStyle::PE_IndicatorToolBarSeparator, &opt, &p, parentWidget());
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_TOOLBAR
