@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,12 +48,15 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_TOOLBUTTON
 
 class QToolButtonPrivate;
 class QMenu;
+class QStyleOptionToolButton;
 
 class Q_GUI_EXPORT QToolButton : public QAbstractButton
 {
@@ -102,12 +125,15 @@ protected:
 
     bool hitButton(const QPoint &pos) const;
     void nextCheckState();
+    void initStyleOption(QStyleOptionToolButton *option) const;
 
 private:
     Q_DISABLE_COPY(QToolButton)
     Q_DECLARE_PRIVATE(QToolButton)
 #ifndef QT_NO_MENU
     Q_PRIVATE_SLOT(d_func(), void _q_buttonPressed())
+    Q_PRIVATE_SLOT(d_func(), void _q_updateButtonDown())
+    Q_PRIVATE_SLOT(d_func(), void _q_menuTriggered(QAction*))
 #endif
     Q_PRIVATE_SLOT(d_func(), void _q_actionTriggered())
 
@@ -131,7 +157,15 @@ public:
     QT3_SUPPORT void setOffIconSet(const QIcon&);
     inline QT3_SUPPORT void setIconSet(const QIcon &icon){setIcon(icon);}
     QT3_SUPPORT void setIconSet(const QIcon &, bool on);
-    inline QT3_SUPPORT void setTextLabel(const QString &text, bool tooltip = true) { setText(text); if (tooltip)setToolTip(text);}
+    inline QT3_SUPPORT void setTextLabel(const QString &text, bool tooltip = true) {
+        setText(text);
+#ifndef QT_NO_TOOLTIP
+        if (tooltip)
+            setToolTip(text);
+#else
+        Q_UNUSED(tooltip);
+#endif
+    }
     inline QT3_SUPPORT QString textLabel() const { return text(); }
     QT3_SUPPORT QIcon onIconSet() const;
     QT3_SUPPORT QIcon offIconSet() const;
@@ -160,6 +194,8 @@ public Q_SLOTS:
 
 #endif // QT_NO_TOOLBUTTON
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QTOOLBUTTON_H
