@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,6 +50,8 @@
 
 #include "qwidgetanimator_p.h"
 
+QT_BEGIN_NAMESPACE
+
 static const int g_animation_steps = 12;
 static const int g_animation_interval = 16;
 
@@ -62,7 +64,7 @@ static const int g_animate_function[] =
 };
 static const int g_animate_function_points = sizeof(g_animate_function)/sizeof(int);
 
-static inline int animate(int start, int stop, int step, int steps)
+static inline int animateHelper(int start, int stop, int step, int steps)
 {
     if (start == stop)
         return start;
@@ -158,13 +160,13 @@ void QWidgetAnimator::animationStep()
 
         item.step = qMin(item.step + steps, g_animation_steps);
 
-        int x = ::animate(item.r1.left(), item.r2.left(),
+        int x = animateHelper(item.r1.left(), item.r2.left(),
                             item.step, g_animation_steps);
-        int y = ::animate(item.r1.top(), item.r2.top(),
+        int y = animateHelper(item.r1.top(), item.r2.top(),
                             item.step, g_animation_steps);
-        int w = ::animate(item.r1.width(), item.r2.width(),
+        int w = animateHelper(item.r1.width(), item.r2.width(),
                             item.step, g_animation_steps);
-        int h = ::animate(item.r1.height(), item.r2.height(),
+        int h = animateHelper(item.r1.height(), item.r2.height(),
                             item.step, g_animation_steps);
 
         item.widget->setGeometry(x, y, w, h);
@@ -194,3 +196,5 @@ bool QWidgetAnimator::animating(QWidget *widget)
 {
     return m_animation_map.contains(widget);
 }
+
+QT_END_NAMESPACE
