@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,6 +56,8 @@
 #include "qdebug.h"
 #include "private/qlayoutengine_p.h"
 
+QT_BEGIN_NAMESPACE
+
 #define RANGE 4
 
 static bool resizeHorizontalDirectionFixed = false;
@@ -542,4 +544,6 @@ void QWidgetResizeHandler::doMove()
     widget->grabKeyboard();
 }
 
+QT_END_NAMESPACE
+
 #endif //QT_NO_RESIZEHANDLER
