@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -71,27 +71,7 @@
     optimization.
 
     Example of use:
-    \code
-    #include <QApplication>
-    #include <QGLColormap>
-
-    int main()
-    {
-        QApplication app(argc, argv);
-
-        MySuperGLWidget widget;     // a QGLWidget in color-index mode
-        QGLColormap colormap;
-
-        // This will fill the colormap with colors ranging from
-        // black to white.
-        for (int i = 0; i < colormap.size(); i++)
-            colormap.setEntry(i, qRgb(i, i, i));
-
-        widget.setColormap(colormap);
-        widget.show();
-        return app.exec();
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.opengl.qglcolormap.cpp 0
 
     \sa QGLWidget::setColormap(), QGLWidget::colormap()
 */
@@ -114,7 +94,9 @@
 
 #include "qglcolormap.h"
 
-QGLColormap::QGLColormapData QGLColormap::shared_null = { Q_ATOMIC_INIT(1), 0, 0 };
+QT_BEGIN_NAMESPACE
+
+QGLColormap::QGLColormapData QGLColormap::shared_null = { Q_BASIC_ATOMIC_INITIALIZER(1), 0, 0 };
 
 /*!
     Construct a QGLColormap.
@@ -157,11 +139,10 @@ void QGLColormap::cleanup(QGLColormap::QGLColormapData *x)
 */
 QGLColormap & QGLColormap::operator=(const QGLColormap &map)
 {
-    QGLColormapData *x = map.d;
-    x->ref.ref();
-    x = qAtomicSetPtr(&d, x);
-    if (!x->ref.deref())
-        cleanup(x);
+    map.d->ref.ref();
+    if (!d->ref.deref())
+        cleanup(d);
+    d = map.d;
     return *this;
 }
 
@@ -175,16 +156,16 @@ QGLColormap & QGLColormap::operator=(const QGLColormap &map)
 void QGLColormap::detach_helper()
 {
     QGLColormapData *x = new QGLColormapData;
-    x->ref.init(1);
+    x->ref = 1;
     x->cmapHandle = 0;
     x->cells = 0;
     if (d->cells) {
         x->cells = new QVector<QRgb>(256);
         *x->cells = *d->cells;
     }
-    x = qAtomicSetPtr(&d, x);
-    if (!x->ref.deref())
-        cleanup(x);
+    if (!d->ref.deref())
+        cleanup(d);
+    d = x;
 }
 
 /*!
@@ -304,3 +285,5 @@ int QGLColormap::findNearest(QRgb color) const
     }
     return idx;
 }
+
+QT_END_NAMESPACE
