@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@
 #include <q3iconview.h>
 #include <q3listbox.h>
 
+QT_BEGIN_NAMESPACE
+
 /*!
 \fn Q3AccessibleScrollView::Q3AccessibleScrollView(QWidget* widget, Role role)
 
@@ -839,3 +841,5 @@ QVector<int> QAccessibleListBox::selection() const
     array.resize(size);
     return array;
 }
+
+QT_END_NAMESPACE
