@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -65,6 +70,9 @@
 #include <QtGui/QFocusFrame>
 
 #ifndef QT_NO_ACCESSIBILITY
+
+QT_BEGIN_NAMESPACE
+
 using namespace QAccessible2;
 
 QList<QWidget*> childWidgets(const QWidget *widget, bool includeTopLevel)
@@ -1061,7 +1069,7 @@ int QAccessibleTitleBar::navigate(RelationFlag relation, int entry, QAccessibleI
             int index = 1;
             int role;
             for (role = QDockWidgetLayout::CloseButton; role <= QDockWidgetLayout::FloatButton; ++role) {
-                QWidget *w = layout->widget((QDockWidgetLayout::Role)role);
+                QWidget *w = layout->widgetForRole((QDockWidgetLayout::Role)role);
                 if (!w->isVisible())
                     continue;
                 if (index == entry)
@@ -1110,7 +1118,7 @@ int QAccessibleTitleBar::childCount() const
     QDockWidgetLayout *layout = dockWidgetLayout();
     int count = 0;
     for (int role = QDockWidgetLayout::CloseButton; role <= QDockWidgetLayout::FloatButton; ++role) {
-        QWidget *w = layout->widget((QDockWidgetLayout::Role)role);
+        QWidget *w = layout->widgetForRole((QDockWidgetLayout::Role)role);
         if (w && w->isVisible())
             ++count;
     }
@@ -1132,7 +1140,7 @@ QAccessible::State QAccessibleTitleBar::state(int child) const
     QAccessible::State state = Normal;
     if (child) {
         QDockWidgetLayout *layout = dockWidgetLayout();
-        QAbstractButton *b = static_cast<QAbstractButton *>(layout->widget((QDockWidgetLayout::Role)child));
+        QAbstractButton *b = static_cast<QAbstractButton *>(layout->widgetForRole((QDockWidgetLayout::Role)child));
         if (b) {
             if (b->isDown())
                 state |= Pressed;
@@ -1171,7 +1179,7 @@ QRect QAccessibleTitleBar::rect (int child ) const
         QDockWidgetLayout *layout = dockWidgetLayout();
         int index = 1;
         for (int role = QDockWidgetLayout::CloseButton; role <= QDockWidgetLayout::FloatButton; ++role) {
-            QWidget *w = layout->widget((QDockWidgetLayout::Role)role);
+            QWidget *w = layout->widgetForRole((QDockWidgetLayout::Role)role);
             if (!w || !w->isVisible())
                 continue;
             if (index == child) {
@@ -1245,7 +1253,7 @@ bool QAccessibleTitleBar::doAction(int action, int child, const QVariantList& /*
     case DefaultAction:
     case Press: {
         QDockWidgetLayout *layout = dockWidgetLayout();
-        QAbstractButton *btn = static_cast<QAbstractButton *>(layout->widget((QDockWidgetLayout::Role)child));
+        QAbstractButton *btn = static_cast<QAbstractButton *>(layout->widgetForRole((QDockWidgetLayout::Role)child));
         if (btn)
             btn->animateClick();
         return true;
@@ -1621,6 +1629,25 @@ int QAccessibleMainWindow::navigate(RelationFlag relation, int entry, QAccessibl
     return QAccessibleWidgetEx::navigate(relation, entry, iface);
 }
 
+int QAccessibleMainWindow::childAt(int x, int y) const
+{
+    QWidget *w = widget();
+    if (!w->isVisible())
+        return -1;
+    QPoint gp = w->mapToGlobal(QPoint(0, 0));
+    if (!QRect(gp.x(), gp.y(), w->width(), w->height()).contains(x, y))
+        return -1;
+
+    QWidgetList kids = childWidgets(mainWindow(), true);
+    QPoint rp = mainWindow()->mapFromGlobal(QPoint(x, y));
+    for (int i = 0; i < kids.size(); ++i) {
+        QWidget *child = kids.at(i);
+        if (!child->isWindow() && !child->isHidden() && child->geometry().contains(rp)) {
+            return i + 1;
+        }
+    }
+    return 0;
+}
 
 QMainWindow *QAccessibleMainWindow::mainWindow() const
 {
@@ -1629,5 +1656,6 @@ QMainWindow *QAccessibleMainWindow::mainWindow() const
 
 #endif //QT_NO_MAINWINDOW
 
+QT_END_NAMESPACE
 
 #endif // QT_NO_ACCESSIBILITY
