@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,6 +47,8 @@
 #include <QtGui/qaccessiblewidget.h>
 #include <QtGui/qaccessible2.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_ACCESSIBILITY
 
 class QAbstractSpinBox;
@@ -211,7 +213,7 @@ public:
     enum DialElements {
         Self  = 0,
         SpeedoMeter,
-        SliderHandle,
+        SliderHandle
     };
 
     int childCount() const;
@@ -228,4 +230,6 @@ protected:
 
 #endif // QT_NO_ACCESSIBILITY
 
+QT_END_NAMESPACE
+
 #endif // RANGECONTROLS_H
