@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,15 +56,17 @@
 #include <qstyle.h>
 #include <qstyleoption.h>
 
+#ifdef Q_OS_MAC
+#include <qfocusframe.h>
+#endif
+
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_ACCESSIBILITY
 
 using namespace QAccessible2;
 extern QList<QWidget*> childWidgets(const QWidget *widget, bool includeTopLevel = false);
 
-#ifdef Q_OS_MAC
-#include <qfocusframe.h>
-#endif
-
 QString Q_GUI_EXPORT qt_accStripAmp(const QString &text);
 QString Q_GUI_EXPORT qt_accHotKey(const QString &text);
 
@@ -753,6 +755,9 @@ void QAccessibleLineEdit::scrollToSubstring(int startIndex, int endIndex)
     lineEdit()->setCursorPosition(startIndex);
 }
 
-
 #endif // QT_NO_LINEEDIT
+
 #endif // QT_NO_ACCESSIBILITY
+
+QT_END_NAMESPACE
+
