@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -77,6 +77,8 @@
 
 #include "qeucjpcodec.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_TEXTCODEC
 
 static const uchar Esc = 0x1b;
@@ -258,3 +260,5 @@ QByteArray QEucJpCodec::_name()
     return "EUC-JP";
 }
 #endif // QT_NO_TEXTCODEC
+
+QT_END_NAMESPACE
