@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@
 
 #include "qbig5codec.h"
 
+QT_BEGIN_NAMESPACE
+
 class TWTextCodecs : public QTextCodecPlugin
 {
 public:
@@ -134,3 +136,5 @@ Q_EXPORT_STATIC_PLUGIN(TWTextCodecs);
 Q_EXPORT_PLUGIN2(qtwcodecs, TWTextCodecs);
 
 #endif // QT_NO_TEXTCODECPLUGIN
+
+QT_END_NAMESPACE
