@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 #include <qiodevice.h>
 #include <qbytearray.h>
 
+QT_BEGIN_NAMESPACE
+
 class QMngPlugin : public QImageIOPlugin
 {
     public:
@@ -93,4 +95,6 @@ QImageIOHandler *QMngPlugin::create(QIODevice *device, const QByteArray &format)
 Q_EXPORT_STATIC_PLUGIN(QMngPlugin)
 Q_EXPORT_PLUGIN2(qmng, QMngPlugin)
 
+QT_END_NAMESPACE
+
 #endif // !QT_NO_IMAGEFORMATPLUGIN
