@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,7 +44,7 @@
 #include <qimageiohandler.h>
 #include <qstringlist.h>
 
-#ifndef QT_NO_IMAGEFORMATPLUGIN
+#if !defined(QT_NO_IMAGEFORMATPLUGIN) && !defined(QT_NO_SVGRENDERER)
 
 #include "qsvgiohandler.h"
 
@@ -52,6 +52,8 @@
 #include <qbytearray.h>
 #include <qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 class QSvgPlugin : public QImageIOPlugin
 {
 public:
@@ -97,4 +99,6 @@ QImageIOHandler *QSvgPlugin::create(QIODevice *device, const QByteArray &format)
 Q_EXPORT_STATIC_PLUGIN(QSvgPlugin)
 Q_EXPORT_PLUGIN2(qsvg, QSvgPlugin)
 
+QT_END_NAMESPACE
+
 #endif // !QT_NO_IMAGEFORMATPLUGIN
