@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -64,6 +64,8 @@
 #include <QtGui/qinputcontextplugin.h>
 #include <QtCore/qstringlist.h>
 
+QT_BEGIN_NAMESPACE
+
 class QMultiInputContextPlugin : public QInputContextPlugin
 {
     Q_OBJECT
@@ -80,4 +82,6 @@ public:
 
 #endif // QT_NO_IM
 
+QT_END_NAMESPACE
+
 #endif // QMULTIINPUTCONTEXTPLUGIN_H
