@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 #include <qstringlist.h>
 #include "../../../../src/sql/drivers/sqlite/qsql_sqlite.h"
 
+QT_BEGIN_NAMESPACE
+
 class QSQLiteDriverPlugin : public QSqlDriverPlugin
 {
 public:
@@ -77,3 +79,5 @@ QStringList QSQLiteDriverPlugin::keys() const
 
 Q_EXPORT_STATIC_PLUGIN(QSQLiteDriverPlugin)
 Q_EXPORT_PLUGIN2(qsqlite, QSQLiteDriverPlugin)
+
+QT_END_NAMESPACE
