@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -52,6 +52,8 @@
 #include "q3widgetstack.h"
 #include "qlayout.h"
 
+QT_BEGIN_NAMESPACE
+
 using namespace Qt;
 
 /*!
@@ -1072,3 +1074,5 @@ void Q3TabDialog::removePage(QWidget * w)
 {
     d->tw->removePage(w);
 }
+
+QT_END_NAMESPACE
