@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -23,7 +43,7 @@
 
 #include "qplatformdefs.h"
 #include "qbytearray.h"
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32) || defined(Q_OS_WINCE) || defined(Q_OS_CYGWIN)
 # include "qt_windows.h"
 #else
 # include <sys/types.h>
@@ -72,9 +92,11 @@ extern "C" int res_init();
 #include "q3cleanuphandler.h"
 #include <limits.h>
 
+QT_BEGIN_NAMESPACE
+
 //#define Q3DNS_DEBUG
 
-static Q_UINT16 id; // ### seeded started by now()
+static Q_UINT16 theId; // ### seeded started by now()
 
 
 static QDateTime * originOfTime = 0;
@@ -87,14 +109,14 @@ static Q_UINT32 now()
 	return originOfTime->secsTo( QDateTime::currentDateTime() );
 
     originOfTime = new QDateTime( QDateTime::currentDateTime() );
-    ::id = originOfTime->time().msec() * 60 + originOfTime->time().second();
+    theId = originOfTime->time().msec() * 60 + originOfTime->time().second();
     q3dns_cleanup_time.add( &originOfTime );
     return 0;
 }
 
 
-static Q3PtrList<QHostAddress> * ns = 0;
-static Q3StrList * domains = 0;
+static Q3PtrList<QHostAddress> * theNs = 0;
+static Q3StrList * theDomains = 0;
 static bool ipv6support = false;
 
 class Q3DnsPrivate {
@@ -338,14 +360,14 @@ QString Q3DnsAnswer::readString(bool multipleLabels)
 	    if ( b == 0 ) {
 		if ( p > pp )
 		    pp = p;
-                return r.isNull() ? QString( "." ) : r;
+                return r.isNull() ? QLatin1String( "." ) : r;
 	    }
 
             // Read a label of size 'b' characters
             if ( !r.isNull() )
-		r += '.';
+		r += QLatin1Char('.');
 	    while( b-- > 0 )
-                r += QChar( answer[p++] );
+                r += QLatin1Char( answer[p++] );
 
             // Return immediately if we were only supposed to read one
             // label.
@@ -929,7 +951,7 @@ Q3DnsManager::Q3DnsManager()
     }
 #endif
 
-    if ( !ns )
+    if ( !theNs )
 	Q3Dns::doResInit();
 
     // O(n*n) stuff here.  but for 3 and 6, O(n*n) with a low k should
@@ -937,9 +959,9 @@ Q3DnsManager::Q3DnsManager()
     // might be hidden in the lists.
     Q3PtrList<QHostAddress> * ns = new Q3PtrList<QHostAddress>;
 
-    ::ns->first();
+    theNs->first();
     QHostAddress * h;
-    while( (h=::ns->current()) != 0 ) {
+    while( (h=theNs->current()) != 0 ) {
 	ns->first();
 	while( ns->current() != 0 && !(*ns->current() == *h) )
 	    ns->next();
@@ -951,18 +973,18 @@ Q3DnsManager::Q3DnsManager()
 	    qDebug( "skipping address %s", h->toString().latin1() );
 #endif
 	}
-	::ns->next();
+	theNs->next();
     }
 
-    delete ::ns;
-    ::ns = ns;
-    ::ns->setAutoDelete( true );
+    delete theNs;
+    theNs = ns;
+    theNs->setAutoDelete( true );
 
     Q3StrList * domains = new Q3StrList( true );
 
-    ::domains->first();
+    theDomains->first();
     const char * s;
-    while( (s=::domains->current()) != 0 ) {
+    while( (s=theDomains->current()) != 0 ) {
 	domains->first();
 	while( domains->current() != 0 && qstrcmp( domains->current(), s ) )
 	    domains->next();
@@ -974,12 +996,12 @@ Q3DnsManager::Q3DnsManager()
 	    qDebug( "skipping domain %s", s );
 #endif
 	}
-	::domains->next();
+	theDomains->next();
     }
 
-    delete ::domains;
-    ::domains = domains;
-    ::domains->setAutoDelete( true );
+    delete theDomains;
+    theDomains = domains;
+    theDomains->setAutoDelete( true );
 }
 
 
@@ -1161,7 +1183,7 @@ void Q3DnsManager::transmitQuery( int i )
     int pp = 12;
     uint lp = 0;
     while( lp < (uint) q->l.length() ) {
-	int le = q->l.find( '.', lp );
+	int le = q->l.find( QLatin1Char('.'), lp );
 	if ( le < 0 )
 	    le = q->l.length();
 	QString component = q->l.mid( lp, le-lp );
@@ -1209,10 +1231,10 @@ void Q3DnsManager::transmitQuery( int i )
     // name servers have recently been defined (like on windows,
     // plugging/unplugging the network cable will change the name
     // server entries)
-    if ( !ns || ns->isEmpty() )
+    if ( !theNs || theNs->isEmpty() )
         Q3Dns::doResInit();
 
-    if ( !ns || ns->isEmpty() ) {
+    if ( !theNs || theNs->isEmpty() ) {
 	// we don't find any name servers. We fake an NXDomain
 	// with a very short life time...
 	Q3DnsAnswer answer( q );
@@ -1228,7 +1250,7 @@ void Q3DnsManager::transmitQuery( int i )
 	return;
     }
 
-    QHostAddress receiver = *ns->at( q->step % ns->count() );
+    QHostAddress receiver = *theNs->at( q->step % theNs->count() );
     if (receiver.isIPv4Address())
 	ipv4Socket->writeBlock( p.data(), pp, receiver, 53 );
 #if !defined (QT_NO_IPV6)
@@ -1240,13 +1262,13 @@ void Q3DnsManager::transmitQuery( int i )
 	    q->id, q->step, q->l.ascii(), q->t,
 	    ns->at( q->step % ns->count() )->toString().ascii() );
 #endif
-    if ( ns->count() > 1 && q->step == 0 && queries.count() == 1 ) {
+    if ( theNs->count() > 1 && q->step == 0 && queries.count() == 1 ) {
 	// if it's the first time, and we don't have any other
 	// outstanding queries, send nonrecursive queries to the other
 	// name servers too.
 	p[2] = 0;
 	QHostAddress * server;
-	while( (server=ns->next()) != 0 ) {
+	while( (server=theNs->next()) != 0 ) {
 	    if (server->isIPv4Address())
 		ipv4Socket->writeBlock( p.data(), pp, *server, 53 );
 #if !defined (QT_NO_IPV6)
@@ -1263,7 +1285,7 @@ void Q3DnsManager::transmitQuery( int i )
     // seconds.  the graph becomes steep around that point, and the
     // number of errors rises... so it seems good to retry at that
     // point.
-    q->start( q->step < ns->count() ? 800 : 1500, true );
+    q->start( q->step < theNs->count() ? 800 : 1500, true );
 }
 
 
@@ -1320,7 +1342,7 @@ Q3PtrList<Q3DnsRR> * Q3DnsDomain::cached( const Q3Dns * r )
 
     // test at first if you have to start a query at all
     if ( r->recordType() == Q3Dns::A ) {
-	if ( r->label().lower() == "localhost" ) {
+	if ( r->label().lower() == QLatin1String("localhost") ) {
 	    // undocumented hack. ipv4-specific. also, may be a memory
 	    // leak? not sure. would be better to do this in doResInit(),
 	    // anyway.
@@ -1415,14 +1437,14 @@ Q3PtrList<Q3DnsRR> * Q3DnsDomain::cached( const Q3Dns * r )
 			// ask the name server again right now.
 			Q3DnsQuery * query = new Q3DnsQuery;
 			query->started = now();
-			query->id = ++::id;
+			query->id = ++theId;
 			query->t = rr->t;
 			query->l = rr->domain->name();
 			// note that here, we don't bother about
 			// notification. but we do bother about
 			// timeouts: we make sure to use high timeouts
 			// and few tramsissions.
-			query->step = ns->count();
+			query->step = theNs->count();
 			QObject::connect( query, SIGNAL(timeout()),
 					  Q3DnsManager::manager(),
 					  SLOT(retransmit()) );
@@ -1467,11 +1489,11 @@ Q3PtrList<Q3DnsRR> * Q3DnsDomain::cached( const Q3Dns * r )
 	    // we haven't done it before, so maybe we should.  but
 	    // wait - if it's an unqualified name, only ask when all
 	    // the other alternatives are exhausted.
-	    if ( q == m->queries.size() && ( s.find( '.' ) >= 0 ||
+	    if ( q == m->queries.size() && ( s.find( QLatin1Char('.') ) >= 0 ||
 					     int(l->count()) >= n.count()-1 ) ) {
 		Q3DnsQuery * query = new Q3DnsQuery;
 		query->started = now();
-		query->id = ++::id;
+		query->id = ++theId;
 		query->t = r->recordType();
 		query->l = s;
 		query->dns->replace( (void*)r, (void*)r );
@@ -1702,23 +1724,23 @@ void Q3Dns::setLabel( const QString & label )
 
     // construct a list of qualified names
     n.clear();
-    if ( l.length() > 1 && l[(int)l.length()-1] == '.' ) {
+    if ( l.length() > 1 && l[(int)l.length()-1] == QLatin1Char('.') ) {
 	n.append( l.left( l.length()-1 ).lower() );
     } else {
 	int i = l.length();
 	int dots = 0;
 	const int maxDots = 2;
 	while( i && dots < maxDots ) {
-	    if ( l[--i] == '.' )
+	    if ( l[--i] == QLatin1Char('.') )
 		dots++;
 	}
 	if ( dots < maxDots ) {
 	    (void)Q3DnsManager::manager(); // create a Q3DnsManager, if it is not already there
-	    Q3StrListIterator it( *domains );
+	    Q3StrListIterator it( *theDomains );
 	    const char * dom;
 	    while( (dom=it.current()) != 0 ) {
 		++it;
-		n.append( l.lower() + "." + dom );
+		n.append( l.lower() + QLatin1String(".") + QLatin1String(dom) );
 	    }
 	}
 	n.append( l.lower() );
@@ -1764,14 +1786,7 @@ void Q3Dns::setLabel( const QHostAddress & address )
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    QStringList list = myDns.qualifiedNames();
-    QStringList::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 0
 
 */
 
@@ -1886,11 +1901,11 @@ QString Q3Dns::toInAddrArpaDomain( const QHostAddress &address )
 	// RFC 3152. (1886 is deprecated, and clients no longer need to
 	// support it, in practice).
 	Q_IPV6ADDR i = address.toIPv6Address();
-	s = "ip6.arpa";
+	s = QLatin1String("ip6.arpa");
 	uint b = 0;
 	while( b < 16 ) {
-	    s = QString::number( i.c[b]%16, 16 ) + "." +
-		QString::number( i.c[b]/16, 16 ) + "." + s;
+	    s = QString::number( i.c[b]%16, 16 ) + QLatin1String(".") +
+		QString::number( i.c[b]/16, 16 ) + QLatin1String(".") + s;
 	    b++;
 	}
     }
@@ -1965,14 +1980,7 @@ bool Q3Dns::isWorking() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    Q3ValueList<QHostAddress> list = myDns.addresses();
-    Q3ValueList<QHostAddress>::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 1
 
 */
 
@@ -2015,14 +2023,7 @@ Q3ValueList<QHostAddress> Q3Dns::addresses() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    Q3ValueList<Q3Dns::MailServer> list = myDns.mailServers();
-    Q3ValueList<Q3Dns::MailServer>::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 2
 
 */
 Q3ValueList<Q3Dns::MailServer> Q3Dns::mailServers() const
@@ -2068,14 +2069,7 @@ Q3ValueList<Q3Dns::MailServer> Q3Dns::mailServers() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    Q3ValueList<Q3Dns::Server> list = myDns.servers();
-    Q3ValueList<Q3Dns::Server>::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 3
 */
 Q3ValueList<Q3Dns::Server> Q3Dns::servers() const
 {
@@ -2106,14 +2100,7 @@ Q3ValueList<Q3Dns::Server> Q3Dns::servers() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    QStringList list = myDns.hostNames();
-    QStringList::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 4
 
 */
 QStringList Q3Dns::hostNames() const
@@ -2145,14 +2132,7 @@ QStringList Q3Dns::hostNames() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    QStringList list = myDns.texts();
-    QStringList::Iterator it = list.begin();
-    while( it != list.end() ) {
-        myProcessing( *it );
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.network.q3dns.cpp 5
 */
 QStringList Q3Dns::texts() const
 {
@@ -2277,7 +2257,7 @@ static QString getWindowsRegString( HKEY key, const QString &subKey )
 	    char *ptr = new char[bsz+1];
 	    r = RegQueryValueEx( key, (TCHAR*)subKey.ucs2(), 0, 0, (LPBYTE)ptr, &bsz );
 	    if ( r == ERROR_SUCCESS )
-		s = ptr;
+		s = QLatin1String(ptr);
 	    delete [] ptr;
 	}
     } , {
@@ -2285,12 +2265,12 @@ static QString getWindowsRegString( HKEY key, const QString &subKey )
 	DWORD bsz = sizeof(buf);
 	int r = RegQueryValueExA( key, subKey.local8Bit(), 0, 0, (LPBYTE)buf, &bsz );
 	if ( r == ERROR_SUCCESS ) {
-	    s = buf;
+	    s = QLatin1String(buf);
 	} else if ( r == ERROR_MORE_DATA ) {
 	    char *ptr = new char[bsz+1];
 	    r = RegQueryValueExA( key, subKey.local8Bit(), 0, 0, (LPBYTE)ptr, &bsz );
 	    if ( r == ERROR_SUCCESS )
-		s = ptr;
+		s = QLatin1String(ptr);
 	    delete [] ptr;
 	}
     } );
@@ -2313,15 +2293,15 @@ static bool getDnsParamsFromRegistry( const QString &path,
     } );
 
     if ( r == ERROR_SUCCESS ) {
-	*domainName = getWindowsRegString( k, "DhcpDomain" );
+	*domainName = getWindowsRegString( k, QLatin1String("DhcpDomain") );
 	if ( domainName->isEmpty() )
-	    *domainName = getWindowsRegString( k, "Domain" );
+	    *domainName = getWindowsRegString( k, QLatin1String("Domain") );
 
-	*nameServer = getWindowsRegString( k, "DhcpNameServer" );
+	*nameServer = getWindowsRegString( k, QLatin1String("DhcpNameServer") );
 	if ( nameServer->isEmpty() )
-	    *nameServer = getWindowsRegString( k, "NameServer" );
+	    *nameServer = getWindowsRegString( k, QLatin1String("NameServer") );
 
-	*searchList = getWindowsRegString( k, "SearchList" );
+	*searchList = getWindowsRegString( k, QLatin1String("SearchList") );
     }
     RegCloseKey( k );
     return r == ERROR_SUCCESS;
@@ -2331,25 +2311,25 @@ void Q3Dns::doResInit()
 {
     char separator = 0;
 
-    if ( ns )
-        delete ns;
-    ns = new Q3PtrList<QHostAddress>;
-    ns->setAutoDelete( true );
-    domains = new Q3StrList( true );
-    domains->setAutoDelete( true );
+    if ( theNs )
+        delete theNs;
+    theNs = new Q3PtrList<QHostAddress>;
+    theNs->setAutoDelete( true );
+    theDomains = new Q3StrList( true );
+    theDomains->setAutoDelete( true );
 
     QString domainName, nameServer, searchList;
 
     bool gotNetworkParams = false;
     // try the API call GetNetworkParams() first and use registry lookup only
     // as a fallback
-#ifdef Q_OS_TEMP
+#ifdef Q_OS_WINCE
     HINSTANCE hinstLib = LoadLibraryW( L"iphlpapi" );
 #else
     HINSTANCE hinstLib = LoadLibraryA( "iphlpapi" );
 #endif
     if ( hinstLib != 0 ) {
-#ifdef Q_OS_TEMP
+#ifdef Q_OS_WINCE
 	GNP getNetworkParams = (GNP) GetProcAddressW( hinstLib, L"GetNetworkParams" );
 #else
 	GNP getNetworkParams = (GNP) GetProcAddress( hinstLib, "GetNetworkParams" );
@@ -2362,16 +2342,16 @@ void Q3Dns::doResInit()
 		FIXED_INFO *finfo = (FIXED_INFO*)new char[l];
 		res = getNetworkParams( finfo, &l );
 		if ( res == ERROR_SUCCESS ) {
-		    domainName = finfo->DomainName;
-		    nameServer = "";
+		    domainName = QLatin1String(finfo->DomainName);
+		    nameServer = QLatin1String("");
 		    IP_ADDR_STRING *dnsServer = &finfo->DnsServerList;
 		    while ( dnsServer != 0 ) {
-			nameServer += dnsServer->IpAddress.String;
+			nameServer += QLatin1String(dnsServer->IpAddress.String);
 			dnsServer = dnsServer->Next;
 			if ( dnsServer != 0 )
-			    nameServer += " ";
+			    nameServer += QLatin1String(" ");
 		    }
-		    searchList = "";
+		    searchList = QLatin1String("");
 		    separator = ' ';
 		    gotNetworkParams = true;
 		}
@@ -2382,20 +2362,20 @@ void Q3Dns::doResInit()
     }
     if ( !gotNetworkParams ) {
 	if ( getDnsParamsFromRegistry(
-	    QString( "System\\CurrentControlSet\\Services\\Tcpip\\Parameters" ),
+	    QString( QLatin1String("System\\CurrentControlSet\\Services\\Tcpip\\Parameters") ),
 		    &domainName, &nameServer, &searchList )) {
 	    // for NT
 	    separator = ' ';
 	} else if ( getDnsParamsFromRegistry(
-	    QString( "System\\CurrentControlSet\\Services\\VxD\\MSTCP" ),
+	    QString( QLatin1String("System\\CurrentControlSet\\Services\\VxD\\MSTCP") ),
 		    &domainName, &nameServer, &searchList )) {
 	    // for Windows 98
 	    separator = ',';
 	} else {
 	    // Could not access the TCP/IP parameters
-	    domainName = "";
-	    nameServer = "127.0.0.1";
-	    searchList = "";
+	    domainName = QLatin1String("");
+	    nameServer = QLatin1String("127.0.0.1");
+	    searchList = QLatin1String("");
 	    separator = ' ';
 	}
     }
@@ -2405,26 +2385,26 @@ void Q3Dns::doResInit()
     if ( !nameServer.isEmpty() ) {
 	first = 0;
 	do {
-	    last = nameServer.find( separator, first );
+	    last = nameServer.find( QLatin1Char(separator), first );
 	    if ( last < 0 )
 		last = nameServer.length();
 	    Q3Dns tmp( nameServer.mid( first, last-first ), Q3Dns::A );
 	    Q3ValueList<QHostAddress> address = tmp.addresses();
 	    Q_LONG i = address.count();
 	    while( i )
-		ns->append( new QHostAddress(address[--i]) );
+		theNs->append( new QHostAddress(address[--i]) );
 	    first = last+1;
 	} while( first < (int)nameServer.length() );
     }
 
-    searchList = searchList + " " + domainName;
+    searchList = searchList + QLatin1String(" ") + domainName;
     searchList = searchList.simplifyWhiteSpace().lower();
     first = 0;
     do {
-	last = searchList.find( separator, first );
+	last = searchList.find( QLatin1Char(separator), first );
 	if ( last < 0 )
 	    last = searchList.length();
-	domains->append( qstrdup( searchList.mid( first, last-first ).latin1() ) );
+	theDomains->append( qstrdup( searchList.mid( first, last-first ).latin1() ) );
 	first = last+1;
     } while( first < (int)searchList.length() );
 }
@@ -2474,7 +2454,7 @@ void Q3Dns::doSynchronousLookup()
 
 		Q3DnsQuery * query = new Q3DnsQuery;
 		query->started = now();
-		query->id = ++::id;
+		query->id = ++theId;
 		query->t = t;
 		query->l = s;
 		Q3DnsAnswer a( ba, query );
@@ -2494,93 +2474,93 @@ void Q3Dns::doSynchronousLookup()
 
 void Q3Dns::doResInit()
 {
-    if ( ns )
+    if ( theNs )
 	return;
-    ns = new Q3PtrList<QHostAddress>;
-    ns->setAutoDelete( true );
-    domains = new Q3StrList( true );
-    domains->setAutoDelete( true );
+    theNs = new Q3PtrList<QHostAddress>;
+    theNs->setAutoDelete( true );
+    theDomains = new Q3StrList( true );
+    theDomains->setAutoDelete( true );
 
     // read resolv.conf manually.
-    QFile resolvConf("/etc/resolv.conf");
+    QFile resolvConf(QLatin1String("/etc/resolv.conf"));
     if (resolvConf.open(QIODevice::ReadOnly)) {
         QTextStream stream( &resolvConf );
 	QString line;
 
 	while ( !stream.atEnd() ) {
             line = stream.readLine();
-	    QStringList list = QStringList::split( " ", line );
-	    if( line.startsWith( "#" ) || list.size() < 2 )
+	    QStringList list = QStringList::split( QLatin1String(" "), line );
+	    if( line.startsWith( QLatin1String("#") ) || list.size() < 2 )
 	       continue;
 	    const QString type = list[0].lower();
 
-	    if ( type == "nameserver" ) {
+	    if ( type == QLatin1String("nameserver") ) {
 		QHostAddress *address = new QHostAddress();
 		if ( address->setAddress( QString(list[1]) ) ) {
 		    // only add ipv6 addresses from resolv.conf if
 		    // this host supports ipv6.
 		    if ( address->isIPv4Address() || ipv6support )
-			ns->append( address );
+			theNs->append( address );
                     else
                         delete address;
 		} else {
 		    delete address;
 		}
-	    } else if ( type == "search" ) {
-		QStringList srch = QStringList::split( " ", list[1] );
+	    } else if ( type == QLatin1String("search") ) {
+		QStringList srch = QStringList::split( QLatin1String(" "), list[1] );
 		for ( QStringList::Iterator i = srch.begin(); i != srch.end(); ++i )
-		    domains->append( (*i).lower().local8Bit() );
+		    theDomains->append( (*i).lower().local8Bit() );
 
-	    } else if ( type == "domain" ) {
-		domains->append( list[1].lower().local8Bit() );
+	    } else if ( type == QLatin1String("domain") ) {
+		theDomains->append( list[1].lower().local8Bit() );
 	    }
 	}
     }
 
-    if (ns->isEmpty()) {
+    if (theNs->isEmpty()) {
 #if defined(Q_MODERN_RES_API)
 	struct __res_state res;
 	res_ninit( &res );
 	int i;
 	// find the name servers to use
 	for( i=0; i < MAXNS && i < res.nscount; i++ )
-	    ns->append( new QHostAddress( ntohl( res.nsaddr_list[i].sin_addr.s_addr ) ) );
+	    theNs->append( new QHostAddress( ntohl( res.nsaddr_list[i].sin_addr.s_addr ) ) );
 #  if defined(MAXDFLSRCH)
 	for( i=0; i < MAXDFLSRCH; i++ ) {
 	    if ( res.dnsrch[i] && *(res.dnsrch[i]) )
-		domains->append( QString::fromLatin1( res.dnsrch[i] ).lower().local8Bit() );
+		theDomains->append( QString::fromLatin1( res.dnsrch[i] ).lower().local8Bit() );
 	    else
 		break;
 	}
 #  endif
 	if ( *res.defdname )
-	    domains->append( QString::fromLatin1( res.defdname ).lower().local8Bit() );
+	    theDomains->append( QString::fromLatin1( res.defdname ).lower().local8Bit() );
 #else
 	res_init();
 	int i;
 	// find the name servers to use
 	for( i=0; i < MAXNS && i < _res.nscount; i++ )
-	    ns->append( new QHostAddress( ntohl( _res.nsaddr_list[i].sin_addr.s_addr ) ) );
+	    theNs->append( new QHostAddress( ntohl( _res.nsaddr_list[i].sin_addr.s_addr ) ) );
 #  if defined(MAXDFLSRCH)
 	for( i=0; i < MAXDFLSRCH; i++ ) {
 	    if ( _res.dnsrch[i] && *(_res.dnsrch[i]) )
-		domains->append( QString::fromLatin1( _res.dnsrch[i] ).lower().local8Bit() );
+		theDomains->append( QString::fromLatin1( _res.dnsrch[i] ).lower().local8Bit() );
 	    else
 		break;
 	}
 #  endif
 	if ( *_res.defdname )
-	    domains->append( QString::fromLatin1( _res.defdname ).lower().local8Bit() );
+	    theDomains->append( QString::fromLatin1( _res.defdname ).lower().local8Bit() );
 #endif
 
 	// the code above adds "0.0.0.0" as a name server at the slightest
 	// hint of trouble. so remove those again.
-	ns->first();
-	while( ns->current() ) {
-	    if ( ns->current()->isNull() )
-		delete ns->take();
+	theNs->first();
+	while( theNs->current() ) {
+	    if ( theNs->current()->isNull() )
+		delete theNs->take();
 	    else
-		ns->next();
+		theNs->next();
 	}
     }
 
@@ -2593,7 +2573,7 @@ void Q3Dns::doResInit()
 	while( !i.atEnd() ) {
 	    line = i.readLine().simplifyWhiteSpace().lower();
 	    uint n = 0;
-	    while( (int) n < line.length() && line[(int)n] != '#' )
+	    while( (int) n < line.length() && line[(int)n] != QLatin1Char('#') )
 		n++;
 	    line.truncate( n );
 	    n = 0;
@@ -2637,4 +2617,6 @@ void Q3Dns::doResInit()
 
 #endif
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_DNS
