@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -78,6 +78,7 @@ static inline int qt_socket_socket(int domain, int type, int protocol)
 #include <errno.h>
 #include <sys/types.h>
 
+QT_BEGIN_NAMESPACE
 
 static inline void qt_socket_getportaddr( struct sockaddr *sa,
 					  Q_UINT16 *port, QHostAddress *addr )
@@ -1076,4 +1077,6 @@ QHostAddress Q3SocketDevice::peerAddress() const
     return pa;
 }
 
+QT_END_NAMESPACE
+
 #endif //QT_NO_NETWORK
