@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,9 @@
 #include "qevent.h"
 #include "qkeysequence.h"
 #include "private/qapplication_p.h"
+
+QT_BEGIN_NAMESPACE
+
 using namespace Qt;
 
 /*!
@@ -75,12 +78,7 @@ using namespace Qt;
     widgets automatically generate accelerators, such as QAbstractButton,
     QGroupBox, QLabel (with QLabel::setBuddy()), QMenuBar, and QTabBar.
     Example:
-    \code
-        QPushButton p("&Exit", parent); // automatic shortcut Alt+E
-        Q3PopupMenu *fileMenu = new fileMenu(parent);
-        fileMenu->insertItem("Undo", parent, SLOT(undo()),
-                             Qt::CTRL + Qt::Key_Z);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 0
 
     A Q3Accel contains a list of accelerator items that can be
     manipulated using insertItem(), removeItem(), clear(), key() and
@@ -123,20 +121,14 @@ using namespace Qt;
     and will consume relevant key events until then.
 
     Please note that the accelerator
-    \code
-        accelerator->insertItem(QKeySequence("M"));
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 1
     can be triggered with both the 'M' key, and with Shift+M,
     unless a second accelerator is defined for the Shift+M
     combination.
 
 
     Example:
-    \code
-        Q3Accel *a = new Q3Accel(myWindow);
-        a->connectItem(a->insertItem(Qt::CTRL + Qt::Key_P),
-                       myWindow, SLOT(printDoc()));
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 2
 
     \sa QKeyEvent QWidget::keyPressEvent()
     QAbstractButton::setAccel() QLabel::setBuddy() QKeySequence
@@ -247,7 +239,7 @@ bool Q3AccelManager::correctSubWindow(QWidget* w, Q3AccelPrivate* d) {
     /* if we live in a floating dock window, keep our parent's
      * accelerators working */
 #ifndef QT_NO_MAINWINDOW
-    if ((tlw->windowType() == Qt::Dialog) && tlw->parentWidget() && ::qobject_cast<QDockWidget*>(tlw))
+    if ((tlw->windowType() == Qt::Dialog) && tlw->parentWidget() && qobject_cast<QDockWidget*>(tlw))
         return tlw->parentWidget()->window() == wtlw;
 
     if (wtlw  != tlw)
@@ -704,14 +696,7 @@ static int get_seq_id()
     If \a id is negative, then the item will be assigned a unique
     negative identifier less than -1.
 
-    \code
-        Q3Accel *a = new Q3Accel(myWindow);	   // create accels for myWindow
-        a->insertItem(CTRL + Key_P, 200);	   // Ctrl+P, e.g. to print document
-        a->insertItem(ALT + Key_X, 201);	   // Alt+X, e.g. to quit
-        a->insertItem(UNICODE_ACCEL + 'q', 202); // Unicode 'q', e.g. to quit
-        a->insertItem(Key_D);			   // gets a unique negative id < -1
-        a->insertItem(CTRL + SHIFT + Key_P);	   // gets a unique negative id < -1
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 3
 */
 
 int Q3Accel::insertItem(const QKeySequence& key, int id)
@@ -802,9 +787,7 @@ void Q3Accel::setItemEnabled(int id, bool enable)
     Connects the accelerator item \a id to the slot \a member of \a
     receiver. Returns true if the connection is successful.
 
-    \code
-        a->connectItem(201, mainView, SLOT(quit()));
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 4
 
     Of course, you can also send a signal as \a member.
 
@@ -933,11 +916,7 @@ QString Q3Accel::keyToString(QKeySequence k)
   \endlink(), so that accelerator keys can be replaced in
   translations:
 
-  \code
-    Q3PopupMenu *file = new Q3PopupMenu(this);
-    file->insertItem(p1, tr("&Open..."), this, SLOT(open()),
-                      Q3Accel::stringToKey(tr("Ctrl+O", "File|Open")));
-  \endcode
+  \snippet doc/src/snippets/code/src.qt3support.other.q3accel.cpp 5
 
   Notice the "File|Open" translator comment. It is by no means
   necessary, but it provides some context for the human translator.
@@ -1001,3 +980,5 @@ bool Q3Accel::ignoreWhatsThis() const
     \fn void Q3Accel::repairEventFilter()
     \internal
 */
+
+QT_END_NAMESPACE
