@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -46,6 +66,12 @@
 #include <private/qobject_p.h>
 
 #include <ctype.h>
+#if defined(Q_OS_WINCE)
+#include "qfunctions_wince.h"
+#include <winsock.h>
+#endif
+
+QT_BEGIN_NAMESPACE
 
 static QWidget *last_target = 0;
 
@@ -66,10 +92,10 @@ class Q3TextDragPrivate : public Q3DragObjectPrivate
 {
     Q_DECLARE_PUBLIC(Q3TextDrag)
 public:
-    Q3TextDragPrivate() { setSubType("plain"); }
+    Q3TextDragPrivate() { setSubType(QLatin1String("plain")); }
     void setSubType(const QString & st) {
         subtype = st;
-        fmt = "text/" + subtype.toLatin1();
+        fmt = QString(QLatin1String("text/")).toLatin1() + subtype.toLatin1();
     }
 
     QString txt;
@@ -265,6 +291,10 @@ QWidget *Q3DragObject::target()
     \c DragMove mode. Be sure to read the constraints described in
     drag().
 
+    Returns true if the data was dragged as a \e move, indicating that
+    the caller should remove the original source of the data (the drag
+    object must continue to have a copy); otherwise returns false.
+
     \sa drag() dragCopy() dragLink()
 */
 bool Q3DragObject::dragMove()
@@ -556,9 +586,9 @@ static QTextCodec *codecForHTML(const QByteArray &ba)
             int end = ba.indexOf('>', pos+1);
             if (end == -1)
                 break;
-            QString str = ba.mid(pos, end-pos);
-            if (str.contains("meta http-equiv=", Qt::CaseInsensitive)) {
-                pos = str.indexOf("charset=", 0, Qt::CaseInsensitive) + int(strlen("charset="));
+            const QString str(QString::fromLatin1(ba.mid(pos, end-pos)));
+            if (str.contains(QLatin1String("meta http-equiv="), Qt::CaseInsensitive)) {
+                pos = str.indexOf(QLatin1String("charset="), 0, Qt::CaseInsensitive) + int(strlen("charset="));
                 if (pos != -1) {
                     int pos2 = ba.indexOf('\"', pos+1);
                     QByteArray cs = ba.mid(pos, pos2-pos);
@@ -652,7 +682,7 @@ bool Q3TextDrag::decode(const QMimeSource* e, QString& str, QString& subtype)
             int semi = m.indexOf(';');
             if (semi < 0)
                 semi = m.length();
-            QString foundst = m.mid(5,semi-5);
+            QString foundst(QString::fromLatin1(m.mid(5,semi-5)));
             if (subtype.isNull() || foundst == subtype) {
                 bool html = !qstrnicmp(mime, "text/html", 9);
                 QTextCodec* codec = 0;
@@ -792,6 +822,7 @@ void Q3ImageDrag::setImage(QImage image)
             format = "image/ppm";
         d->ofmts.append(format);
     }
+    d->ofmts.append("application/x-qt-image");
 }
 
 /*!
@@ -809,8 +840,12 @@ const char * Q3ImageDrag::format(int i) const
 QByteArray Q3ImageDrag::encodedData(const char* fmt) const
 {
     Q_D(const Q3ImageDrag);
-    if (qstrnicmp(fmt, "image/", 6)==0) {
-        QByteArray f(fmt+6);
+    QString imgFormat(fmt);
+    if (imgFormat == QLatin1String("application/x-qt-image"))
+        imgFormat = QLatin1String("image/PNG");
+
+    if (imgFormat.startsWith("image/")){
+        QByteArray f(imgFormat.mid(6).toAscii());
         QByteArray dat;
         QBuffer w(&dat);
         w.open(QIODevice::WriteOnly);
@@ -834,14 +869,7 @@ QByteArray Q3ImageDrag::encodedData(const char* fmt) const
 */
 bool Q3ImageDrag::canDecode(const QMimeSource* e)
 {
-    const QList<QByteArray> fileFormats = QImageReader::supportedImageFormats();
-
-    for (int i = 0; i < fileFormats.count(); ++i) {
-        if (e->provides("image" + fileFormats.at(i).toLower()))
-            return true;
-    }
-
-    return false;
+    return e->provides("application/x-qt-image");
 }
 
 /*!
@@ -857,33 +885,14 @@ bool Q3ImageDrag::decode(const QMimeSource* e, QImage& img)
     if (!e)
         return false;
 
-    QByteArray payload;
-    QList<QByteArray> fileFormats = QImageReader::supportedImageFormats();
-
-    // PNG is best of all
-    // (this is a rather strange hack, but it works now)
-    for (int i=0; i < fileFormats.count(); i++) {
-        if (fileFormats.at(i).toLower() == "png") {
-            fileFormats.prepend("png");
-            break ;
-        }
-    } // move to front
-
-    for (int i = 0; i < fileFormats.count(); ++i) {
-        QByteArray type = "image/" + fileFormats.at(i).toLower();
-        if (! e->provides(type))
-            continue;
-        payload = e->encodedData(type);
-        if (!payload.isEmpty())
-            break;
-    }
-
+    QByteArray payload = e->encodedData("application/x-qt-image");
     if (payload.isEmpty())
         return false;
 
     img.loadFromData(payload);
     if (img.isNull())
         return false;
+
     return true;
 }
 
@@ -1043,7 +1052,7 @@ QByteArray Q3StoredDrag::encodedData(const char* m) const
 Q3UriDrag::Q3UriDrag(const Q3StrList &uris, QWidget * dragSource, const char * name) :
     Q3StoredDrag("text/uri-list", dragSource)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     setUris(uris);
 }
 
@@ -1056,7 +1065,7 @@ Q3UriDrag::Q3UriDrag(const Q3StrList &uris, QWidget * dragSource, const char * n
 Q3UriDrag::Q3UriDrag(QWidget * dragSource, const char * name) :
     Q3StoredDrag("text/uri-list", dragSource)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
 }
 #endif
 
@@ -1213,7 +1222,7 @@ QByteArray Q3UriDrag::unicodeUriToUri(const QString& uuri)
     QByteArray utf8 = uuri.toUtf8();
     QByteArray escutf8;
     int n = utf8.length();
-    bool isFile = uuri.startsWith("file://");
+    bool isFile = uuri.startsWith(QLatin1String("file://"));
     for (int i=0; i<n; i++) {
         if (utf8[i] >= 'a' && utf8[i] <= 'z'
           || utf8[i] == '/'
@@ -1263,20 +1272,20 @@ QByteArray Q3UriDrag::localFileToUri(const QString& filename)
 
     bool hasHost = false;
     // convert form network path
-    if (r.left(2) == "\\\\" || r.left(2) == "//") {
+    if (r.left(2) == QLatin1String("\\\\") || r.left(2) == QLatin1String("//")) {
         r.remove(0, 2);
         hasHost = true;
     }
 
     // Slosh -> Slash
     int slosh;
-    while ((slosh=r.indexOf('\\')) >= 0) {
-        r[slosh] = '/';
+    while ((slosh=r.indexOf(QLatin1Char('\\'))) >= 0) {
+        r[slosh] = QLatin1Char('/');
     }
 
     // Drive
-    if (r[0] != '/' && !hasHost)
-        r.insert(0,'/');
+    if (r[0] != QLatin1Char('/') && !hasHost)
+        r.insert(0,QLatin1Char('/'));
 
 #endif
 #if defined (Q_WS_X11) && 0
@@ -1289,7 +1298,7 @@ QByteArray Q3UriDrag::localFileToUri(const QString& filename)
         r.prepend(QString::fromLatin1(hostname));
     }
 #endif
-    return unicodeUriToUri(QString("file://" + r));
+    return unicodeUriToUri(QString(QLatin1String("file://") + r));
 }
 
 /*!
@@ -1344,7 +1353,7 @@ QString Q3UriDrag::uriToLocalFile(const char* uri)
         return file;
     if (0==qstrnicmp(uri,"file:/",6)) // It is a local file uri
         uri += 6;
-    else if (QString(uri).indexOf(":/") != -1) // It is a different scheme uri
+    else if (QString(QLatin1String(uri)).indexOf(QLatin1String(":/")) != -1) // It is a different scheme uri
         return file;
 
     bool local = uri[0] != '/' || (uri[0] != '\0' && uri[1] == '/');
@@ -1370,13 +1379,13 @@ QString Q3UriDrag::uriToLocalFile(const char* uri)
         if (uri[1] == '/') {
             file.remove((uint)0,1);
         } else {
-                file.insert(0,'/');
+                file.insert(0, QLatin1Char('/'));
         }
 #ifdef Q_WS_WIN
-        if (file.length() > 2 && file[0] == '/' && file[2] == '|') {
-            file[2] = ':';
+        if (file.length() > 2 && file[0] == QLatin1Char('/') && file[2] == QLatin1Char('|')) {
+            file[2] = QLatin1Char(':');
             file.remove(0,1);
-        } else if (file.length() > 2 && file[0] == '/' && file[1].isLetter() && file[2] == ':') {
+        } else if (file.length() > 2 && file[0] == QLatin1Char('/') && file[1].isLetter() && file[2] == QLatin1Char(':')) {
             file.remove(0, 1);
         }
         // Leave slash as slashes.
@@ -1386,7 +1395,7 @@ QString Q3UriDrag::uriToLocalFile(const char* uri)
     else {
         file = uriToUnicodeUri(uri);
         // convert to network path
-        file.insert(1, '/'); // leave as forward slashes
+        file.insert(1, QLatin1Char('/')); // leave as forward slashes
     }
 #endif
 
@@ -1468,7 +1477,7 @@ bool Q3UriDrag::decodeToUnicodeUris(const QMimeSource* e, QStringList& l)
 Q3ColorDrag::Q3ColorDrag(const QColor &col, QWidget *dragsource, const char *name)
     : Q3StoredDrag("application/x-color", dragsource)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     setColor(col);
 }
 
@@ -1480,7 +1489,7 @@ Q3ColorDrag::Q3ColorDrag(const QColor &col, QWidget *dragsource, const char *nam
 Q3ColorDrag::Q3ColorDrag(QWidget *dragsource, const char *name)
     : Q3StoredDrag("application/x-color", dragsource)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     setColor(Qt::white);
 }
 
@@ -1527,7 +1536,9 @@ bool Q3ColorDrag::canDecode(QMimeSource *e)
     \fn bool Q3ColorDrag::decode(QMimeSource *source, QColor &color)
 
     Decodes the MIME \a source, and sets the decoded values to the
-    given \a color.
+    given \a color. Returns true if the decoding is successful.
+    Returns false if the size of the encoded data is not the
+    expected size.
 */
 
 bool Q3ColorDrag::decode(QMimeSource *e, QColor &col)
@@ -1558,3 +1569,5 @@ bool Q3ColorDrag::decode(QMimeSource *e, QColor &col)
     col.setRgb(r, g, b, a);
     return true;
 }
+
+QT_END_NAMESPACE
