@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +56,8 @@
 #include "q3cleanuphandler.h"
 #include "private/qtextimagehandler_p.h"
 
+QT_BEGIN_NAMESPACE
+
 static Q3MimeSourceFactory* defaultfactory = 0;
 static Q3SingleCleanupHandler<Q3MimeSourceFactory> qmime_cleanup_factory;
 
@@ -106,34 +128,20 @@ static QImage richTextImageLoader(const QString &name, const QString &context)
     images that are stored in the program itself, not loaded from the
     hard disk. Your program may, for example, define some image data
     as:
-    \code
-    static const char* myimage_data[]={
-    "...",
-    ...
-    "..."};
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3mimefactory.cpp 0
 
     To be able to use this image within some rich text, for example
     inside a QLabel, you must create a QImage from the raw data and
     insert it into the factory with a unique name:
-    \code
-    Q3MimeSourceFactory::defaultFactory()->setImage("myimage", QImage(myimage_data));
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3mimefactory.cpp 1
 
     Now you can create a rich text QLabel with
 
-    \code
-    QLabel* label = new QLabel(
-        "Rich text with embedded image:<img source=\"myimage\">"
-        "Isn't that <em>cute</em>?");
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3mimefactory.cpp 2
 
     When no longer needed, you can clear the data from the factory:
 
-    \code
-    delete label;
-    Q3MimeSourceFactory::defaultFactory()->setData("myimage", 0);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3mimefactory.cpp 3
 */
 
 
@@ -144,13 +152,13 @@ static QImage richTextImageLoader(const QString &name, const QString &context)
 Q3MimeSourceFactory::Q3MimeSourceFactory() :
     d(new Q3MimeSourceFactoryData)
 {
-    addFilePath(":/qt/q3mimesourcefactory/"); //to get from the resources
+    addFilePath(QLatin1String(":/qt/q3mimesourcefactory/")); //to get from the resources
     // add some reasonable defaults
-    setExtensionType("htm", "text/html;charset=iso8859-1");
-    setExtensionType("html", "text/html;charset=iso8859-1");
-    setExtensionType("txt", "text/plain");
-    setExtensionType("xml", "text/xml;charset=UTF-8");
-    setExtensionType("jpg", "image/jpeg"); // support misspelled jpeg files
+    setExtensionType(QLatin1String("htm"), "text/html;charset=iso8859-1");
+    setExtensionType(QLatin1String("html"), "text/html;charset=iso8859-1");
+    setExtensionType(QLatin1String("txt"), "text/plain");
+    setExtensionType(QLatin1String("xml"), "text/xml;charset=UTF-8");
+    setExtensionType(QLatin1String("jpg"), "image/jpeg"); // support misspelled jpeg files
 }
 
 /*!
@@ -217,12 +225,7 @@ QMimeSource* Q3MimeSourceFactory::dataInternal(const QString& abs_name, const QM
     The factory understands all the image formats supported by
     QImageReader. Any other mime types are determined by the file name
     extension. The default settings are
-    \code
-    setExtensionType("html", "text/html;charset=iso8859-1");
-    setExtensionType("htm", "text/html;charset=iso8859-1");
-    setExtensionType("txt", "text/plain");
-    setExtensionType("xml", "text/xml;charset=UTF-8");
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3mimefactory.cpp 4
     The effect of these is that file names ending in "txt" will be
     treated as text encoded in the local encoding; those ending in
     "xml" will be treated as text encoded in Unicode UTF-8 encoding.
@@ -258,9 +261,9 @@ const QMimeSource *Q3MimeSourceFactory::data(const QString& abs_name) const
     if (abs_name.isEmpty())
         return r;
     QStringList::Iterator it;
-    if (abs_name[0] == '/'
+    if (abs_name[0] == QLatin1Char('/')
 #ifdef Q_WS_WIN
-            || (abs_name[0].isLetter() && abs_name[1] == ':') || abs_name.startsWith("\\\\")
+            || (abs_name[0].isLetter() && abs_name[1] == QLatin1Char(':')) || abs_name.startsWith(QLatin1String("\\\\"))
 #endif
    )
     {
@@ -270,8 +273,8 @@ const QMimeSource *Q3MimeSourceFactory::data(const QString& abs_name) const
     else { // check list of paths
         for (it = d->path.begin(); !r && it != d->path.end(); ++it) {
             QString filename = *it;
-            if (filename[(int)filename.length()-1] != '/')
-                filename += '/';
+            if (filename[(int)filename.length()-1] != QLatin1Char('/'))
+                filename += QLatin1Char('/');
             filename += abs_name;
             r = dataInternal(filename, d->extensions);
         }
@@ -346,7 +349,7 @@ void Q3MimeSourceFactory::addFilePath(const QString& p)
 */
 void Q3MimeSourceFactory::setExtensionType(const QString& ext, const char* mimetype)
 {
-    d->extensions.insert(ext, mimetype);
+    d->extensions.insert(ext, QLatin1String(mimetype));
 }
 
 /*!
@@ -358,9 +361,9 @@ void Q3MimeSourceFactory::setExtensionType(const QString& ext, const char* mimet
 QString Q3MimeSourceFactory::makeAbsolute(const QString& abs_or_rel_name, const QString& context) const
 {
     if (context.isNull() ||
-         !(context[0] == '/'
+         !(context[0] == QLatin1Char('/')
 #ifdef Q_WS_WIN
-         || (context[0].isLetter() && context[1] == ':')
+         || (context[0].isLetter() && context[1] == QLatin1Char(':'))
 #endif
           ))
         return abs_or_rel_name;
@@ -541,4 +544,6 @@ QImage qImageFromMimeSource(const QString &abs_name)
     return img;
 }
 
-#endif // QT_NO_MIME
+QT_END_NAMESPACE
+
+#endif // QT_NO_MIMEFACTORY
