@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,6 +51,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
 
 //#define QT_Q3PROCESS_DEBUG
 
@@ -231,14 +252,7 @@ Q3Process::Q3Process( const QStringList& args, QObject *parent, const char *name
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    QStringList list = myProcess.arguments();
-    QStringList::Iterator it = list.begin();
-    while( it != list.end() ) {
-	myProcessing( *it );
-	++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3process.cpp 0
 
     \sa setArguments() addArgument()
 */
@@ -456,7 +470,7 @@ QString Q3Process::readLineStdout()
 	return QString();
 
       if ( !buf->scanNewline( &a ) )
-	return QString( buf->readAll() );
+	return QString( QLatin1String(buf->readAll()) );
     }
 
     uint size = a.size();
@@ -469,7 +483,7 @@ QString Q3Process::readLineStdout()
       else
 	a.chop(1);
     }
-    return QString( a );
+    return QString(QString::fromLatin1(a.constData()));
 }
 
 /*!
@@ -492,7 +506,7 @@ QString Q3Process::readLineStderr()
 	return QString();
 
       if ( !buf->scanNewline( &a ) )
-	return QString( buf->readAll() );
+	return QString( QString::fromLatin1( buf->readAll().constData() ) );
     }
 
     uint size = a.size();
@@ -505,7 +519,7 @@ QString Q3Process::readLineStderr()
       else
 	a.chop(1);
     }
-    return QString( a );
+    return QString( QString::fromLatin1( a.constData() ) );
 }
 
 /*!
@@ -751,4 +765,6 @@ void Q3Process::disconnectNotify( const char * )
     }
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_PROCESS
