@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -46,6 +66,8 @@
 #include <errno.h>
 #include <sys/types.h>
 
+QT_BEGIN_NAMESPACE
+
 #ifdef __MIPSEL__
 # ifndef SOCK_DGRAM
 #  define SOCK_DGRAM 1
@@ -451,7 +473,9 @@ void Q3ProcessManager::sigchldHnd( int fd )
 	sn->setEnabled( true );
 }
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "q3process_unix.moc"
+QT_END_INCLUDE_NAMESPACE
 
 
 /***********************************************************************
@@ -538,7 +562,7 @@ void Q3ProcessPrivate::newProc( pid_t pid, Q3Process *process )
  * sigchld handler callback
  *
  **********************************************************************/
-static QT_SIGNAL_RETTYPE qt_C_sigchldHnd( QT_SIGNAL_ARGS )
+static QT_SIGNAL_RETTYPE qt_C_sigchldHnd(QT_SIGNAL_ARGS)
 {
     if ( Q3ProcessPrivate::procManager == 0 )
 	return;
@@ -751,7 +775,7 @@ bool Q3Process::start( QStringList *env )
 #ifdef Q_OS_MACX
     if(i) {
 	Q3CString arg_bundle = arglistQ[0];
-	QFileInfo fi(arg_bundle);
+	QFileInfo fi(QString::fromUtf8(arg_bundle.constData()));
 	if(fi.exists() && fi.isDir() && arg_bundle.right(4) == ".app") {
 	    Q3CString exe = arg_bundle;
 	    int lslash = exe.findRev('/');
@@ -759,7 +783,7 @@ bool Q3Process::start( QStringList *env )
 		exe = exe.mid(lslash+1);
 	    exe = Q3CString(arg_bundle + "/Contents/MacOS/" + exe);
 	    exe = exe.left(exe.length() - 4); //chop off the .app
-	    if(QFile::exists(exe)) {
+	    if(QFile::exists(QString::fromLatin1(exe.constData()))) {
 		arglistQ[0] = exe;
 		arglist[0] = arglistQ[0];
 	    }
@@ -802,7 +826,7 @@ bool Q3Process::start( QStringList *env )
 	if ( fd[0] )
 	    ::close( fd[0] );
 	if ( fd[1] )
-	    ::fcntl( fd[1], F_SETFD, FD_CLOEXEC ); // close on exec shows sucess
+	    ::fcntl( fd[1], F_SETFD, FD_CLOEXEC ); // close on exec shows success
 
 	if ( env == 0 ) { // inherit environment and start process
 #ifndef Q_OS_QNX4
@@ -814,12 +838,12 @@ bool Q3Process::start( QStringList *env )
 	    // construct the environment for exec
 	    int numEntries = env->count();
 #if defined(Q_OS_MACX)
-	    QString ld_library_path("DYLD_LIBRARY_PATH");
+	    QString ld_library_path(QLatin1String("DYLD_LIBRARY_PATH"));
 #else
-	    QString ld_library_path("LD_LIBRARY_PATH");
+	    QString ld_library_path(QLatin1String("LD_LIBRARY_PATH"));
 #endif
 	    bool setLibraryPath =
-		env->grep( QRegExp( "^" + ld_library_path + "=" ) ).empty() &&
+		env->grep( QRegExp( QLatin1Char('^') + ld_library_path + QLatin1Char('=') ) ).empty() &&
 		getenv( ld_library_path.local8Bit() ) != 0;
 	    if ( setLibraryPath )
 		numEntries++;
@@ -827,7 +851,7 @@ bool Q3Process::start( QStringList *env )
 	    const char** envlist = new const char*[ numEntries + 1 ];
 	    int i = 0;
 	    if ( setLibraryPath ) {
-		envlistQ[i] = QString( ld_library_path + "=%1" ).arg( getenv( ld_library_path.local8Bit() ) ).local8Bit();
+		envlistQ[i] = QString( ld_library_path + QLatin1String("=%1") ).arg( QString::fromLocal8Bit(getenv( ld_library_path.local8Bit() )) ).local8Bit();
 		envlist[i] = envlistQ[i];
 		i++;
 	    }
@@ -841,13 +865,13 @@ bool Q3Process::start( QStringList *env )
 	    // look for the executable in the search path
 	    if ( _arguments.count()>0 && getenv("PATH")!=0 ) {
 		QString command = _arguments[0];
-		if ( !command.contains( '/' ) ) {
-		    QStringList pathList = QStringList::split( ':', getenv( "PATH" ) );
+		if ( !command.contains( QLatin1Char('/') ) ) {
+		    QStringList pathList = QStringList::split( QLatin1Char(':'), QString::fromLocal8Bit(getenv( "PATH" )) );
 		    for (QStringList::Iterator it = pathList.begin(); it != pathList.end(); ++it ) {
 			QString dir = *it;
 #if defined(Q_OS_MACX) //look in a bundle
-			if(!QFile::exists(dir + "/" + command) && QFile::exists(dir + "/" + command + ".app"))
-			    dir += "/" + command + ".app/Contents/MacOS";
+			if(!QFile::exists(dir + QLatin1Char('/') + command) && QFile::exists(dir + QLatin1Char('/') + command + QLatin1String(".app")))
+			    dir += QLatin1Char('/') + command + QLatin1String(".app/Contents/MacOS");
 #endif
 #ifndef QT_NO_DIR
 			QFileInfo fileInfo( dir, command );
@@ -1002,10 +1026,7 @@ void Q3Process::tryTerminate() const
 
     The nice way to end a process and to be sure that it is finished,
     is to do something like this:
-    \code
-	process->tryTerminate();
-	QTimer::singleShot( 5000, process, SLOT( kill() ) );
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.other.q3process_unix.cpp 0
 
     This tries to terminate the process the nice way. If the process
     is still running after 5 seconds, it terminates the process the
@@ -1386,4 +1407,6 @@ Q3Process::PID Q3Process::processIdentifier()
     return d->proc->pid;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_PROCESS
