@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Qt3SupportLight)
 
 class Q_COMPAT_EXPORT Q3Painter : public QPainter
@@ -114,6 +116,8 @@ void Q_COMPAT_EXPORT Q3Painter::drawChord(const QRect &r, int angle, int arcLeng
     QPainter::drawChord(adjustedRectangle(r), angle, arcLength);
 }
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // Q3PAINTER_H
