@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@
 #include "private/q3sqlmanager_p.h"
 #include "qsqlresult.h"
 
+QT_BEGIN_NAMESPACE
+
 class Q3DataBrowserPrivate
 {
 public:
@@ -250,14 +252,7 @@ bool Q3DataBrowser::boundaryChecking() const
 
     Note that if you want to iterate over the list, you should iterate
     over a copy, e.g.
-    \code
-    QStringList list = myDataBrowser.sort();
-    QStringList::Iterator it = list.begin();
-    while(it != list.end()) {
-        myProcessing(*it);
-        ++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3databrowser.cpp 0
 */
 
 void Q3DataBrowser::setSort(const QStringList& sort)
@@ -1283,4 +1278,6 @@ QSql::Confirm  Q3DataBrowser::confirmCancel(QSql::Op m)
     some auditing information about the deletion.
 */
 
+QT_END_NAMESPACE
+
 #endif
