@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,6 +61,8 @@
 #include "qtimer.h"
 #include "qpointer.h"
 
+QT_BEGIN_NAMESPACE
+
 //#define QT_DEBUG_DATATABLE
 
 class Q3DataTablePrivate
@@ -132,10 +154,10 @@ void qt_debug_buffer( const QString& msg, QSqlRecord* cursor )
     When displaying data, Q3DataTable only retrieves data for visible
     rows. If the driver supports the 'query size' property the
     Q3DataTable will have the correct number of rows and the vertical
-    scrollbar will accurately reflect the number of rows displayed in
+    scroll bar will accurately reflect the number of rows displayed in
     proportion to the number of rows in the dataset. If the driver
     does not support the 'query size' property, rows are dynamically
-    fetched from the database on an as-needed basis with the scrollbar
+    fetched from the database on an as-needed basis with the scroll bar
     becoming more accurate as the user scrolls down through the
     records. This allows extremely large queries to be displayed as
     quickly as possible, with minimum memory usage.
@@ -184,7 +206,7 @@ void qt_debug_buffer( const QString& msg, QSqlRecord* cursor )
     currentRecord(). Use the find() function to search for a string in
     the table.
 
-    Editing actions can be applied programatically. For example, the
+    Editing actions can be applied programmatically. For example, the
     insertCurrent() function reads the fields from the current record
     into the cursor and performs the insert. The updateCurrent() and
     deleteCurrent() functions perform similarly to update and delete
@@ -376,7 +398,7 @@ void Q3DataTable::adjustColumn( int col )
 	d->cur.refresh();
     }
     int oldRow = currentRow();
-    int w = fontMetrics().width( horizontalHeader()->label( col ) + "W" );
+    int w = fontMetrics().width( horizontalHeader()->label( col ) + QLatin1Char('W') );
     cur->seek( QSql::BeforeFirst );
     while ( cur->next() ) {
 	w = QMAX( w, fontMetrics().width( fieldToString( cur->fieldPtr( indexOf( col ) ) ) ) + 10 );
@@ -438,14 +460,7 @@ void Q3DataTable::setFilter( const QString& filter )
 
     Note that if you want to iterate over the sort list, you should
     iterate over a copy, e.g.
-    \code
-    QStringList list = myDataTable.sort();
-    QStringList::Iterator it = list.begin();
-    while( it != list.end() ) {
-	myProcessing( *it );
-	++it;
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3datatable.cpp 0
 
     \sa filter() refresh()
 */
@@ -915,8 +930,7 @@ void Q3DataTable::endInsert()
 }
 
 /*! \internal
-*/
-
+ */
 void Q3DataTable::endUpdate()
 {
     d->dat.setMode( QSql::None );
@@ -928,18 +942,18 @@ void Q3DataTable::endUpdate()
 }
 
 /*!
-    Protected virtual function called when editing is about to begin
-    on a new record. If the table is read-only, or if there's no
-    cursor or the cursor does not allow inserts, nothing happens.
-
-    Editing takes place using the cursor's edit buffer(see
-    Q3SqlCursor::editBuffer()).
-
-    When editing begins, a new row is created in the table marked with
-    an asterisk '*' in the row's vertical header column, i.e. at the
-    left of the row.
+  Protected virtual function called when editing is about to begin
+  on a new record. If the table is read-only, or if there's no cursor
+  or the cursor does not allow inserts, nothing happens and false
+  is returned. Otherwise returns true.
+  
+  Editing takes place using the cursor's edit buffer(see
+  Q3SqlCursor::editBuffer()).
+  
+  When editing begins, a new row is created in the table marked with
+  an asterisk '*' in the row's vertical header column, i.e. at the
+  left of the row.
 */
-
 bool Q3DataTable::beginInsert()
 {
     if ( !sqlCursor() || isReadOnly() || !numCols() )
@@ -970,7 +984,7 @@ bool Q3DataTable::beginInsert()
 	lastRow = numRows() - 1;
     d->insertRowLast = lastRow;
     d->insertHeaderLabelLast = verticalHeader()->label( d->insertRowLast );
-    verticalHeader()->setLabel( row, "*" );
+    verticalHeader()->setLabel( row, QString(QLatin1Char('*')) );
     d->editRow = row;
     // in the db world it's common to allow inserting new records
     // into a table that has read-only columns - temporarily
@@ -1665,7 +1679,7 @@ void Q3DataTable::sortColumn ( int col, bool ascending,
 	if ( !sqlCursor() )
 	    return;
 	QSqlIndex lastSort = sqlCursor()->sort();
-	QSqlIndex newSort( lastSort.cursorName(), "newSort" );
+	QSqlIndex newSort( lastSort.cursorName(), QLatin1String("newSort") );
 	const QSqlField *field = sqlCursor()->fieldPtr( indexOf( col ) );
 	if ( field )
 	    newSort.append( *field );
@@ -2317,3 +2331,5 @@ void  Q3DataTable::selectRow(int row)
 */
 
 #endif
+
+QT_END_NAMESPACE
