@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -34,6 +54,8 @@
 #include "q3sqlrecordinfo.h"
 #include "q3sqlfieldinfo.h"
 
+QT_BEGIN_NAMESPACE
+
 class Q3SqlCursorPrivate
 {
 public:
@@ -73,7 +95,7 @@ QString qOrderByClause(const QSqlIndex & i, const QString& prefix = QString())
     int k = i.count();
     if(k == 0)
         return QString();
-    str = " order by " + i.toString(prefix);
+    str = QLatin1String(" order by ") + i.toString(prefix);
     return str;
 }
 
@@ -82,12 +104,12 @@ QString qWhereClause(const QString& prefix, QSqlField* field, const QSqlDriver*
     QString f;
     if (field && driver) {
         if (!prefix.isEmpty())
-            f += prefix + QString(".");
+            f += prefix + QLatin1Char('.');
         f += field->name();
         if (field->isNull()) {
-            f += " IS NULL";
+            f += QLatin1String(" IS NULL");
         } else {
-            f += " = " + driver->formatValue(field);
+            f += QLatin1String(" = ") + driver->formatValue(field);
         }
     }
     return f;
@@ -96,7 +118,7 @@ QString qWhereClause(const QString& prefix, QSqlField* field, const QSqlDriver*
 QString qWhereClause(QSqlRecord* rec, const QString& prefix, const QString& sep,
                       const QSqlDriver* driver)
 {
-    static QString blank(" ");
+    static QString blank(QLatin1Char(' '));
     QString filter;
     bool separator = false;
     for (int j = 0; j < rec->count(); ++j) {
@@ -144,15 +166,7 @@ QString qWhereClause(QSqlRecord* rec, const QString& prefix, const QString& sep,
 
     For example:
 
-    \code
-    Q3SqlCursor cur( "staff" ); // Specify the table/view name
-    cur.select(); // We'll retrieve every record
-    while ( cur.next() ) {
-        qDebug( cur.value( "id" ).toString() + ": " +
-        cur.value( "surname" ).toString() + " " +
-        cur.value( "salary" ).toString() );
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 0
 
     In the above example, a cursor is created specifying a table or
     view name in the database. Then, select() is called, which can be
@@ -176,17 +190,7 @@ QString qWhereClause(QSqlRecord* rec, const QString& prefix, const QString& sep,
 
     For example:
 
-    \code
-    Q3SqlCursor cur( "prices" );
-    cur.select( "id=202" );
-    if ( cur.next() ) {
-        QSqlRecord *buffer = cur.primeUpdate();
-        double price = buffer->value( "price" ).toDouble();
-        double newprice = price * 1.05;
-        buffer->setValue( "price", newprice );
-        cur.update();
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 1
 
     To edit an existing database record, first move to the record you
     wish to update. Call primeUpdate() to get the pointer to the
@@ -433,15 +437,15 @@ QString Q3SqlCursor::name() const
 QString Q3SqlCursor::toString(const QString& prefix, const QString& sep) const
 {
     QString pflist;
-    QString pfix =  prefix.isEmpty() ? prefix : prefix + ".";
+    QString pfix =  prefix.isEmpty() ? prefix : prefix + QLatin1Char('.');
     bool comma = false;
 
     for (int i = 0; i < count(); ++i) {
         const QString fname = fieldName(i);
         if (isGenerated(i)) {
             if(comma)
-                pflist += sep + " ";
-            pflist += pfix + fname;
+                pflist += sep + QLatin1Char(' ');
+            pflist += pfix + driver()->escapeIdentifier(fname, QSqlDriver::FieldName);
             comma = true;
         }
     }
@@ -621,50 +625,31 @@ QSqlIndex Q3SqlCursor::index(const QString& fieldName) const
     row, use seek(), first(), last(), previous() or next().
 
     Example:
-    \code
-    Q3SqlCursor cur("Employee"); // Use the Employee table or view
-    cur.select("deptno=10"); // select all records in department 10
-    while(cur.next()) {
-        ... // process data
-    }
-    ...
-    // select records in other departments, ordered by department number
-    cur.select("deptno>10", cur.index("deptno"));
-    ...
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 2
 
     The filter will apply to any subsequent select() calls that do not
     explicitly specify another filter. Similarly the sort will apply
     to any subsequent select() calls that do not explicitly specify
     another sort.
 
-    \code
-    Q3SqlCursor cur("Employee");
-    cur.select("deptno=10"); // select all records in department 10
-    while(cur.next()) {
-        ... // process data
-    }
-    ...
-    cur.select(); // re-selects all records in department 10
-    ...
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 3
 
 */
 
 bool Q3SqlCursor::select(const QString & filter, const QSqlIndex & sort)
 {
-    QString fieldList = toString(d->nm);
+    QString fieldList(toString(d->nm));
     if (fieldList.isEmpty())
         return false;
-    QString str= "select " + fieldList;
-    str += " from " + d->nm;
+    QString str(QLatin1String("select ") + fieldList);
+    str += QLatin1String(" from ") + d->nm;
     if (!filter.isEmpty()) {
         d->ftr = filter;
-        str += " where " + filter;
+        str += QLatin1String(" where ") + filter;
     } else
         d->ftr = QString();
     if (sort.count() > 0)
-        str += " order by " + sort.toString(d->nm);
+        str += QLatin1String(" order by ") + sort.toString(d->nm);
     d->srt = sort;
     return exec(str);
 }
@@ -721,13 +706,7 @@ bool Q3SqlCursor::select(const QSqlIndex& sort)
     last(), previous() or next(). This function is useful, for example,
     for retrieving data based upon a table's primary index:
 
-    \code
-    Q3SqlCursor cur("Employee");
-    QSqlIndex pk = cur.primaryIndex();
-    cur.setValue("id", 10);
-    cur.select(pk, pk); // generates "SELECT ... FROM Employee WHERE id=10 ORDER BY id"
-    ...
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 4
 
     In this example the QSqlIndex, pk, is used for two different
     purposes. When used as the filter (first) argument, the field
@@ -739,7 +718,7 @@ bool Q3SqlCursor::select(const QSqlIndex& sort)
 
 bool Q3SqlCursor::select(const QSqlIndex & filter, const QSqlIndex & sort)
 {
-    return select(toString(filter, this, d->nm, "=", "and"), sort);
+    return select(toString(filter, this, d->nm, QString(QLatin1Char('=')), QLatin1String("and")), sort);
 }
 
 /*!
@@ -747,15 +726,7 @@ bool Q3SqlCursor::select(const QSqlIndex & filter, const QSqlIndex & sort)
     combination of \l Q3SqlCursor::Mode values. The default mode for a
     cursor is Q3SqlCursor::Writable.
 
-    \code
-    Q3SqlCursor cur("Employee");
-    cur.setMode(Q3SqlCursor::Writable); // allow insert/update/delete
-    ...
-    cur.setMode(Q3SqlCursor::Insert | Q3SqlCursor::Update); // allow inserts and updates only
-    ...
-    cur.setMode(Q3SqlCursor::ReadOnly); // no inserts/updates/deletes allowed
-
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 5
 */
 
 void Q3SqlCursor::setMode(int mode)
@@ -910,10 +881,10 @@ QString Q3SqlCursor::toString(const QString& prefix, QSqlField* field, const QSt
 {
     QString f;
     if (field && driver()) {
-        f = (prefix.length() > 0 ? prefix + QString(".") : QString()) + field->name();
-        f += " " + fieldSep + " ";
+        f = (prefix.length() > 0 ? prefix + QLatin1Char('.') : QString()) + field->name();
+        f += QLatin1Char(' ') + fieldSep + QLatin1Char(' ');
         if (field->isNull()) {
-            f += "NULL";
+            f += QLatin1String("NULL");
         } else {
             f += driver()->formatValue(field);
         }
@@ -934,7 +905,7 @@ QString Q3SqlCursor::toString(const QString& prefix, QSqlField* field, const QSt
 QString Q3SqlCursor::toString(QSqlRecord* rec, const QString& prefix, const QString& fieldSep,
                               const QString& sep) const
 {
-    static QString blank(" ");
+    static QString blank(QLatin1Char(' '));
     QString filter;
     bool separator = false;
     for (int j = 0; j < count(); ++j) {
@@ -971,7 +942,7 @@ QString Q3SqlCursor::toString(const QSqlIndex& i, QSqlRecord* rec, const QString
     for(int j = 0; j < i.count(); ++j){
         if (rec->isGenerated(j)) {
             if(separator) {
-                filter += " " + sep + " " ;
+                filter += QLatin1Char(' ') + sep + QLatin1Char(' ') ;
             }
             QString fn = i.fieldName(j);
             QSqlField f = rec->field(fn);
@@ -993,14 +964,7 @@ QString Q3SqlCursor::toString(const QSqlIndex& i, QSqlRecord* rec, const QString
     new select() call must be made before navigating to a valid
     record.
 
-    \code
-    Q3SqlCursor cur("prices");
-    QSqlRecord *buffer = cur.primeInsert();
-    buffer->setValue("id",    53981);
-    buffer->setValue("name",  "Thingy");
-    buffer->setValue("price", 105.75);
-    cur.insert();
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 6
 
     In the above example, a cursor is created on the 'prices' table
     and a pointer to the insert buffer is acquired using primeInsert().
@@ -1032,11 +996,11 @@ int Q3SqlCursor::insert(bool invalidate)
             QSqlField f = d->editBuffer.field(j);
             if (d->editBuffer.isGenerated(j)) {
                 if (comma) {
-                    fList += ",";
-                    vList += ",";
+                    fList += QLatin1Char(',');
+                    vList += QLatin1Char(',');
                 }
-                fList += f.name();
-                vList += (oraStyle == true) ? ":f" + QString::number(cnt) : QString("?");
+                fList += driver()->escapeIdentifier(f.name(), QSqlDriver::FieldName);
+                vList += (oraStyle == true) ? QLatin1String(":f") + QString::number(cnt) : QString(QLatin1Char('?'));
                 cnt++;
                 comma = true;
             }
@@ -1045,17 +1009,20 @@ int Q3SqlCursor::insert(bool invalidate)
             return 0;
         }
         QString str;
-        str.append("insert into ").append(name()).append(" (").append(fList).append(") values (").append(vList). append (")");
+        str.append(QLatin1String("insert into ")).append(name())
+           .append(QLatin1String(" (")).append(fList)
+           .append(QLatin1String(") values (")).append(vList). append(QLatin1Char(')'));
+
         return applyPrepared(str, invalidate);
     } else {
         for(int j = 0; j < k; ++j) {
             QSqlField f = d->editBuffer.field(j);
             if (d->editBuffer.isGenerated(j)) {
                 if (comma) {
-                    fList += ",";
-                    vList += ",";
+                    fList += QLatin1Char(',');
+                    vList += QLatin1Char(',');
                 }
-                fList += f.name();
+                fList += driver()->escapeIdentifier(f.name(), QSqlDriver::FieldName);
                 vList += driver()->formatValue(&f);
                 comma = true;
             }
@@ -1066,7 +1033,8 @@ int Q3SqlCursor::insert(bool invalidate)
             return 0;
         }
         QString str;
-        str.append("insert into ").append(name()).append(" (").append(fList).append(") values (").append(vList). append (")");
+        str.append(QLatin1String("insert into ")).append(name()).append(QLatin1String(" ("))
+           .append(fList).append(QLatin1String(") values (")).append(vList). append (QLatin1String(")"));
         return apply(str, invalidate);
     }
 }
@@ -1113,9 +1081,9 @@ QSqlRecord* Q3SqlCursor::primeUpdate()
     QSqlRecord* buf = editBuffer(true);
     QSqlIndex idx = primaryIndex(false);
     if (!idx.isEmpty())
-        d->editIndex = toString(idx, buf, d->nm, "=", "and");
+        d->editIndex = toString(idx, buf, d->nm, QString(QLatin1Char('=')), QLatin1String("and"));
     else
-        d->editIndex = qWhereClause(buf, d->nm, "and", driver());
+        d->editIndex = qWhereClause(buf, d->nm, QLatin1String("and"), driver());
     return buf;
 }
 
@@ -1164,17 +1132,7 @@ QSqlRecord* Q3SqlCursor::primeInsert()
     longer be navigated. A new select() call must be made before you
     can move to a valid record. For example:
 
-    \code
-    Q3SqlCursor cur("prices");
-    cur.select("id=202");
-    if (cur.next()) {
-        QSqlRecord *buffer = cur.primeUpdate();
-        double price = buffer->value("price").toDouble();
-        double newprice = price * 1.05;
-        buffer->setValue("price", newprice);
-        cur.update();
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 7
 
     In the above example, a cursor is created on the 'prices' table
     and is positioned on the record to be updated. Then a pointer to
@@ -1238,9 +1196,9 @@ int Q3SqlCursor::update(const QString & filter, bool invalidate)
             QSqlField f = d->editBuffer.field(j);
             if (d->editBuffer.isGenerated(j)) {
                 if (comma) {
-                    fList += ",";
+                    fList += QLatin1Char(',');
                 }
-                fList += f.name() + " = " + (oraStyle == true ? ":f" + QString::number(cnt) : QString("?"));
+                fList += f.name() + QLatin1String(" = ") + (oraStyle == true ? QLatin1String(":f") + QString::number(cnt) : QString(QLatin1Char('?')));
                 cnt++;
                 comma = true;
             }
@@ -1248,16 +1206,16 @@ int Q3SqlCursor::update(const QString & filter, bool invalidate)
         if (!comma) {
             return 0;
         }
-        QString str = "update " + name() + " set " + fList;
+        QString str(QLatin1String("update ") + name() + QLatin1String(" set ") + fList);
         if (filter.length()) {
-            str+= " where " + filter;
+            str+= QLatin1String(" where ") + filter;
         }
         return applyPrepared(str, invalidate);
     } else {
-        QString str = "update " + name();
-        str += " set " + toString(&d->editBuffer, QString(), "=", ",");
+        QString str = QLatin1String("update ") + name();
+        str += QLatin1String(" set ") + toString(&d->editBuffer, QString(), QString(QLatin1Char('=')), QString(QLatin1Char(',')));
         if (filter.length()) {
-            str+= " where " + filter;
+            str+= QLatin1String(" where ") + filter;
         }
         return apply(str, invalidate);
     }
@@ -1276,14 +1234,7 @@ int Q3SqlCursor::update(const QString & filter, bool invalidate)
     be navigated. A new select() call must be made before you can move
     to a valid record. For example:
 
-    \code
-    Q3SqlCursor cur("prices");
-    cur.select("id=999");
-    if (cur.next()) {
-        cur.primeDelete();
-        cur.del();
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.sql.q3sqlcursor.cpp 8
 
     In the above example, a cursor is created on the 'prices' table
     and positioned to the record to be deleted. First primeDelete() is
@@ -1300,8 +1251,8 @@ int Q3SqlCursor::del(bool invalidate)
 {
     QSqlIndex idx = primaryIndex(false);
     if (idx.isEmpty())
-        return del(qWhereClause(&d->editBuffer, d->nm, "and", driver()), invalidate);
-    return del(toString(primaryIndex(), &d->editBuffer, d->nm, "=", "and"), invalidate);
+        return del(qWhereClause(&d->editBuffer, d->nm, QLatin1String("and"), driver()), invalidate);
+    return del(toString(primaryIndex(), &d->editBuffer, d->nm, QString(QLatin1Char('=')), QLatin1String("and")), invalidate);
 }
 
 /*!
@@ -1325,9 +1276,9 @@ int Q3SqlCursor::del(const QString & filter, bool invalidate)
         return 0;
     int k = count();
     if(k == 0) return 0;
-    QString str = "delete from " + name();
+    QString str = QLatin1String("delete from ") + name();
     if (filter.length())
-        str+= " where " + filter;
+        str+= QLatin1String(" where ") + filter;
     return apply(str, invalidate);
 }
 
@@ -1565,4 +1516,6 @@ bool Q3SqlCursor::last()
     return res;
 }
 
+QT_END_NAMESPACE
+
 #endif
