@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -53,6 +53,8 @@
 #include "qcombobox.h"
 #include "qdatetimeedit.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class Q3SqlEditorFactory qsqleditorfactory.h
     \brief The Q3SqlEditorFactory class is used to create the editors
@@ -224,4 +226,6 @@ QWidget * Q3SqlEditorFactory::createEditor(QWidget * parent,
     return w;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SQL
