@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,6 +56,8 @@
 #include "qmessagebox.h"
 #include "qbitarray.h"
 
+QT_BEGIN_NAMESPACE
+
 //#define QT_DEBUG_DATAMANAGER
 
 class Q3SqlCursorManagerPrivate
@@ -358,17 +360,7 @@ values for the record being sought.  This function is typically used
 to relocate a cursor to the correct position after an insert or
 update.  For example:
 
-\code
-    Q3SqlCursor* myCursor = myManager.cursor();
-    ...
-    QSqlRecord* buf = myCursor->primeUpdate();
-    buf->setValue("name", "Ola");
-    buf->setValue("city", "Oslo");
-    ...
-    myCursor->update();  // update current record
-    myCursor->select();  // refresh the cursor
-    myManager.findBuffer(myCursor->primaryIndex()); // go to the updated record
-\endcode
+\snippet doc/src/snippets/code/src.qt3support.sql.q3sqlmanager_p.cpp 0
 
 */
 
@@ -966,4 +958,6 @@ QSql::Confirm Q3DataManager::confirmCancel(QWidget* parent, QSql::Op)
 #endif // QT_NO_MESSAGEBOX
 }
 
+QT_END_NAMESPACE
+
 #endif
