@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 
 #ifndef QT_NO_RICHTEXT
 
+QT_BEGIN_NAMESPACE
+
 Q3TextCommand::~Q3TextCommand() {}
 Q3TextCommand::Commands Q3TextCommand::type() const { return Invalid; }
 
@@ -631,4 +633,6 @@ Q3TextParagraphPseudoDocument::Q3TextParagraphPseudoDocument():pFormatter(0),com
 Q3TextParagraphPseudoDocument::~Q3TextParagraphPseudoDocument(){ delete pFormatter; delete commandHistory; }
 
 
+QT_END_NAMESPACE
+
 #endif //QT_NO_RICHTEXT
