@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -75,6 +75,8 @@
 #include "Qt3Support/q3stylesheet.h"
 #include "Qt3Support/q3mimefactory.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_RICHTEXT
 
 class Q3TextDocument;
@@ -2096,4 +2098,6 @@ inline int Q3TextStringChar::descent() const
 
 #endif // QT_NO_RICHTEXT
 
+QT_END_NAMESPACE
+
 #endif // Q3RICHTEXT_P_H
