@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,7 +53,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
-#ifndef Q_OS_TEMP
+#ifndef Q_OS_WINCE
 #include <locale.h>
 #endif
 
@@ -41,6 +61,8 @@
 #include "qt_windows.h"
 #endif
 
+QT_BEGIN_NAMESPACE
+
 #ifndef QT_NO_TEXTCODEC
 static void resetCodecConverterState(QTextCodec::ConverterState *state) {
     state->flags = QTextCodec::DefaultConversion;
@@ -197,8 +219,8 @@ void Q3TextStream::init()
     owndev = FALSE;
     mapper = 0;
 #ifndef QT_NO_TEXTCODEC
-    ::resetCodecConverterState(&mapperReadState);
-    ::resetCodecConverterState(&mapperWriteState);
+    resetCodecConverterState(&mapperReadState);
+    resetCodecConverterState(&mapperWriteState);
 #endif
     d = new Q3TextStreamPrivate;
     doUnicodeHeader = TRUE; // autodetect
@@ -316,9 +338,9 @@ qint64 QStringBuffer::readData( char *p, qint64 len )
 	return qint64(-1);
     }
 #endif
-    if ( pos() + len > s->length()*sizeof(QChar) ) {
+    if ( pos() + len > qint64(s->length()*sizeof(QChar)) ) {
 	// overflow
-	if ( pos() >= s->length()*sizeof(QChar) ) {
+	if ( pos() >= qint64(s->length()*sizeof(QChar)) ) {
 	    return -1;
 	} else {
 	    len = s->length()*2 - pos();
@@ -365,20 +387,12 @@ qint64 QStringBuffer::writeData( const char *p, qint64 len )
     this setting is ignored for text streams that operate on QString.
 
     Example:
-    \code
-    QString str;
-    Q3TextStream ts( &str, IO_WriteOnly );
-    ts << "pi = " << 3.14; // str == "pi = 3.14"
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.text.q3textstream.cpp 0
 
     Writing data to the text stream will modify the contents of the
     string. The string will be expanded when data is written beyond
     the end of the string. Note that the string will not be truncated:
-    \code
-    QString str = "pi = 3.14";
-    Q3TextStream ts( &str, IO_WriteOnly );
-    ts <<  "2+2 = " << 2+2; // str == "2+2 = 414"
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.text.q3textstream.cpp 1
 
     Note that because QString is Unicode, you should not use
     readRawBytes() or writeRawBytes() on such a stream.
@@ -420,25 +434,14 @@ Q3TextStream::Q3TextStream( QString& str, int filemode )
     to the device's open() function; see \l{QIODevice::mode()}.
 
     Example:
-    \code
-    QByteArray array;
-    Q3TextStream ts( array, IO_WriteOnly );
-    ts << "pi = " << 3.14 << '\0'; // array == "pi = 3.14"
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.text.q3textstream.cpp 2
 
     Writing data to the text stream will modify the contents of the
     array. The array will be expanded when data is written beyond the
     end of the string.
 
     Same example, using a QBuffer:
-    \code
-    QByteArray array;
-    QBuffer buf( array );
-    buf.open( IO_WriteOnly );
-    Q3TextStream ts( &buf );
-    ts << "pi = " << 3.14 << '\0'; // array == "pi = 3.14"
-    buf.close();
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.text.q3textstream.cpp 3
 */
 
 Q3TextStream::Q3TextStream( QByteArray &a, int mode )
@@ -635,13 +638,13 @@ uint Q3TextStream::ts_getbuf( QChar* buf, uint len )
 	    // (arnt doubts whether it makes a difference, but lets it stand)
 	    int c = (ungetHack == EOF) ? dev->getch() : ungetHack;
 	    if ( c != EOF ) {
-		*buf = (char)c;
+		*buf = QLatin1Char((char)c);
 		buf++;
 		rnum++;
 	    }
 	} else {
 	    if ( ungetHack != EOF ) {
-		*buf = (char)ungetHack;
+		*buf = QLatin1Char((char)ungetHack);
 		buf++;
 		rnum++;
 		ungetHack = EOF;
@@ -653,7 +656,7 @@ uint Q3TextStream::ts_getbuf( QChar* buf, uint len )
 		char *it = cbuf;
 		char *end = cbuf + rlen;
 		while ( it < end ) {
-		    *buf = *it;
+		    *buf = QLatin1Char(*it);
 		    buf++;
 		    it++;
 		}
@@ -768,8 +771,8 @@ uint Q3TextStream::ts_getline( QChar* buf )
 		readBlock = FALSE;
 	    }
 	    if ( dev->atEnd()
-		    || s.at( s.length()-1 ) == '\n'
-		    || s.at( s.length()-1 ) == '\r'
+		    || s.at( s.length()-1 ) == QLatin1Char('\n')
+		    || s.at( s.length()-1 ) == QLatin1Char('\r')
 	       ) {
 		break;
 	    } else {
@@ -933,7 +936,7 @@ Q3TextStream &Q3TextStream::writeBlock( const char* p, uint len )
     } else if ( !mapper && internalOrder ) {
 	QChar *u = new QChar[len];
 	for ( uint i = 0; i < len; i++ )
-	    u[i] = p[i];
+	    u[i] = QLatin1Char(p[i]);
 	dev->writeBlock( (char*)u, len * sizeof(QChar) );
 	delete [] u;
     }
@@ -1149,7 +1152,7 @@ ulong Q3TextStream::input_hex()
     char c = ch.toLatin1();
     while ( isxdigit((uchar) c) ) {
 	val <<= 4;
-	if ( ts_isdigit(c) )
+	if ( ts_isdigit(QLatin1Char(c)) )
 	    val += c - '0';
 	else
 	    val += 10 + tolower( (uchar) c ) - 'a';
@@ -1624,10 +1627,10 @@ QString Q3TextStream::readLine()
 	if ( c[pos] == QEOF )
 	    return QString::null;
 
-	while ( c[pos] != QEOF && c[pos] != '\n' ) {
-	    if ( c[pos] == '\r' ) { // ( handle mac and dos )
+	while ( c[pos] != QEOF && c[pos] != QLatin1Char('\n') ) {
+	    if ( c[pos] == QLatin1Char('\r') ) { // ( handle mac and dos )
 		QChar nextc = ts_getc();
-		if ( nextc != '\n' )
+		if ( nextc != QLatin1Char('\n') )
 		    ts_ungetc( nextc );
 		break;
 	    }
@@ -1673,7 +1676,7 @@ QString Q3TextStream::read()
 	// convert dos (\r\n) and mac (\r) style eol to unix style (\n)
 	start = 0;
 	for ( i=0; i<num; i++ ) {
-	    if ( buf[i] == '\r' ) {
+	    if ( buf[i] == QLatin1Char('\r') ) {
 		// Only skip single cr's preceding lf's
 		if ( skipped_cr ) {
 		    result += buf[i];
@@ -1685,9 +1688,9 @@ QString Q3TextStream::read()
 		}
 	    } else {
 		if ( skipped_cr ) {
-		    if ( buf[i] != '\n' ) {
+		    if ( buf[i] != QLatin1Char('\n') ) {
 			// Should not have skipped it
-			result += '\n';
+			result += QLatin1Char('\n');
 		    }
 		    skipped_cr = FALSE;
 		}
@@ -1810,7 +1813,7 @@ Q3TextStream &Q3TextStream::output_int( int format, ulong n, bool neg )
 		*--p = '-';
 	    else if ( flags() & showpos )
 		*--p = '+';
-	    if ( (flags() & internal) && fwidth && !ts_isdigit(*p) ) {
+	    if ( (flags() & internal) && fwidth && !ts_isdigit(QLatin1Char(*p)) ) {
 		ts_putc( *p );			// special case for internal
 		++p;				//   padding
 		fwidth--;
@@ -2056,9 +2059,9 @@ Q3TextStream &Q3TextStream::operator<<( const QString& s )
     QString s1 = s;
     if ( fwidth ) {				// field width set
 	if ( !(flags() & left) ) {
-	    s1 = s.rightJustify(fwidth, (char)fillchar);
+	    s1 = s.rightJustify(fwidth, QLatin1Char((char)fillchar));
 	} else {
-	    s1 = s.leftJustify(fwidth, (char)fillchar);
+	    s1 = s.leftJustify(fwidth, QLatin1Char((char)fillchar));
 	}
 	fwidth = 0;				// reset width
     }
@@ -2309,8 +2312,8 @@ Q3TextStream &reset( Q3TextStream &s )
 
 void Q3TextStream::setEncoding( Encoding e )
 {
-    ::resetCodecConverterState(&mapperReadState);
-    ::resetCodecConverterState(&mapperWriteState);
+    resetCodecConverterState(&mapperReadState);
+    resetCodecConverterState(&mapperWriteState);
 
     if ( d->sourceType == Q3TextStreamPrivate::String )
 	return;
@@ -2430,4 +2433,6 @@ QTextCodec *Q3TextStream::codec()
 
 #endif
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_TEXTSTREAM
