@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,8 +46,7 @@
 #include "qmutex.h"
 #include "qwaitcondition.h"
 
-
-
+QT_BEGIN_NAMESPACE
 
 /*!
     \class Q3Semaphore q3semaphore.h
@@ -253,3 +252,5 @@ bool Q3Semaphore::tryAccess(int n)
 
     return true;
 }
+
+QT_END_NAMESPACE
