@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -52,6 +57,8 @@
 #include "qstatusbar.h"
 #include "qaction.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class Q3Action q3action.h
     \brief The Q3Action class provides an abstract user interface
@@ -208,16 +215,16 @@ Q3ActionPrivate::~Q3ActionPrivate()
         ++itci;
         QComboBox* combo = ci->combo;
         combo->clear();
-        Q3ActionGroup *group = ::qobject_cast<Q3ActionGroup*>(action->parent());
+        Q3ActionGroup *group = qobject_cast<Q3ActionGroup*>(action->parent());
         if (group) {
             QObjectList siblings = group->queryList("Q3Action");
 
             for (int i = 0; i < siblings.size(); ++i) {
-                Q3Action *sib = ::qobject_cast<Q3Action*>(siblings.at(i));
+                Q3Action *sib = qobject_cast<Q3Action*>(siblings.at(i));
                 sib->removeFrom(combo);
             }
             for (int i = 0; i < siblings.size(); ++i) {
-                Q3Action *sib = ::qobject_cast<Q3Action*>(siblings.at(i));
+                Q3Action *sib = qobject_cast<Q3Action*>(siblings.at(i));
                 if (sib == action)
                     continue;
                 sib->addTo(combo);
@@ -1014,7 +1021,7 @@ void Q3Action::toolButtonToggled(bool on)
 bool Q3Action::addTo(QWidget* w)
 {
 #ifndef QT_NO_TOOLBAR
-    if (::qobject_cast<Q3ToolBar*>(w)) {
+    if (qobject_cast<Q3ToolBar*>(w)) {
         if (objectName() == QLatin1String("qt_separator_action")) {
             ((Q3ToolBar*)w)->addSeparator();
         } else {
@@ -1198,7 +1205,7 @@ void Q3Action::clearStatusText()
 bool Q3Action::removeFrom(QWidget* w)
 {
 #ifndef QT_NO_TOOLBAR
-    if (::qobject_cast<Q3ToolBar*>(w)) {
+    if (qobject_cast<Q3ToolBar*>(w)) {
         QList<QToolButton*>::Iterator it(d->toolbuttons.begin());
         QToolButton* btn;
         while (it != d->toolbuttons.end()) {
@@ -1213,7 +1220,7 @@ bool Q3Action::removeFrom(QWidget* w)
         }
     } else
 #endif
-    if (::qobject_cast<Q3PopupMenu*>(w)) {
+    if (qobject_cast<Q3PopupMenu*>(w)) {
         QList<Q3ActionPrivate::MenuItem*>::Iterator it(d->menuitems.begin());
         Q3ActionPrivate::MenuItem* mi;
         while (it != d->menuitems.end()) {
@@ -1228,7 +1235,7 @@ bool Q3Action::removeFrom(QWidget* w)
                 delete mi;
             }
         }
-    } else if (::qobject_cast<QComboBox*>(w)) {
+    } else if (qobject_cast<QComboBox*>(w)) {
         QList<Q3ActionPrivate::ComboItem*>::Iterator it(d->comboitems.begin());
         Q3ActionPrivate::ComboItem *ci;
         while (it != d->comboitems.end()) {
@@ -1240,7 +1247,7 @@ bool Q3Action::removeFrom(QWidget* w)
                 delete ci;
             }
         }
-    } else if (::qobject_cast<QMenu*>(w)) {
+    } else if (qobject_cast<QMenu*>(w)) {
         QList<Q3ActionPrivate::Action4Item*>::Iterator it(d->action4items.begin());
         Q3ActionPrivate::Action4Item *a4i;
         while (it != d->action4items.end()) {
@@ -1375,7 +1382,7 @@ void Q3ActionGroupPrivate::update(const Q3ActionGroup* that)
     }
     for (QList<Q3ActionGroupPrivate::MenuItem*>::Iterator pu(menuitems.begin()); pu != menuitems.end(); ++pu) {
         QWidget* parent = (*pu)->popup->parentWidget();
-        if (::qobject_cast<Q3PopupMenu*>(parent)) {
+        if (qobject_cast<Q3PopupMenu*>(parent)) {
             Q3PopupMenu* ppopup = (Q3PopupMenu*)parent;
             ppopup->setItemEnabled((*pu)->id, that->isEnabled());
             ppopup->setItemVisible((*pu)->id, that->isVisible());
@@ -1385,7 +1392,7 @@ void Q3ActionGroupPrivate::update(const Q3ActionGroup* that)
     }
     for (QList<Q3PopupMenu*>::Iterator pm(popupmenus.begin()); pm != popupmenus.end(); ++pm) {
         Q3PopupMenu *popup = *pm;
-        Q3PopupMenu *parent = ::qobject_cast<Q3PopupMenu*>(popup->parentWidget());
+        Q3PopupMenu *parent = qobject_cast<Q3PopupMenu*>(popup->parentWidget());
         if (!parent)
             continue;
 
@@ -1666,7 +1673,7 @@ bool Q3ActionGroup::addTo(QWidget *w)
         if (d->dropdown) {
             if (!d->exclusive) {
                 QList<Q3Action*>::Iterator it(d->actions.begin());
-                if (!(*it))
+                if (it == d->actions.end() || !(*it))
                     return true;
 
                 Q3Action *defAction = *it;
@@ -1745,7 +1752,7 @@ bool Q3ActionGroup::addTo(QWidget *w)
         }
     } else
 #endif
-    if (::qobject_cast<Q3PopupMenu*>(w)) {
+    if (qobject_cast<Q3PopupMenu*>(w)) {
         Q3PopupMenu *popup;
         if (d->dropdown) {
             Q3PopupMenu *menu = (Q3PopupMenu*)w;
@@ -1781,7 +1788,7 @@ bool Q3ActionGroup::addTo(QWidget *w)
         }
         return true;
     }
-    if (::qobject_cast<QMenu*>(w)) {
+    if (qobject_cast<QMenu*>(w)) {
         QMenu *menu = (QMenu*)w;
         if (d->dropdown) {
             Q3ActionGroupPrivate::Action4Item *ai = new Q3ActionGroupPrivate::Action4Item;
@@ -1819,7 +1826,7 @@ bool Q3ActionGroup::removeFrom(QWidget* w)
         (*it)->removeFrom(w);
 
 #ifndef QT_NO_TOOLBAR
-    if (::qobject_cast<Q3ToolBar*>(w)) {
+    if (qobject_cast<Q3ToolBar*>(w)) {
         QList<QComboBox*>::Iterator cb(d->comboboxes.begin());
         while (cb != d->comboboxes.end()) {
             QComboBox *box = *cb;
@@ -1836,7 +1843,7 @@ bool Q3ActionGroup::removeFrom(QWidget* w)
         }
     } else
 #endif
-    if (::qobject_cast<Q3PopupMenu*>(w)) {
+    if (qobject_cast<Q3PopupMenu*>(w)) {
         QList<Q3ActionGroupPrivate::MenuItem*>::Iterator pu(d->menuitems.begin());
         while (pu != d->menuitems.end()) {
             Q3ActionGroupPrivate::MenuItem *mi = *pu;
@@ -1846,7 +1853,7 @@ bool Q3ActionGroup::removeFrom(QWidget* w)
             delete mi->popup;
         }
     }
-    if (::qobject_cast<QMenu*>(w)) {
+    if (qobject_cast<QMenu*>(w)) {
         QList<Q3ActionGroupPrivate::Action4Item*>::Iterator it(d->action4items.begin());
         Q3ActionGroupPrivate::Action4Item *a4i;
         while (it != d->action4items.end()) {
@@ -2225,4 +2232,6 @@ void Q3ActionGroup::addedTo(int index, Q3PopupMenu *menu)
     Use add(\a action) instead.
 */
 
+QT_END_NAMESPACE
+
 #endif
