@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,6 +53,8 @@
 #include "q3mainwindow.h"
 #include "q3toolbar.h"
 
+QT_BEGIN_NAMESPACE
+
 //#define QDOCKAREA_DEBUG
 
 struct Q3DockData
@@ -151,11 +173,10 @@ void Q3DockAreaLayout::init()
 
 void Q3DockAreaLayout::invalidate()
 {
-    if (!dirty)
-        QApplication::postEvent(parent(), new QEvent(QEvent::LayoutHint));
     dirty = true;
     cached_width = 0;
     cached_height = 0;
+    QLayout::invalidate();
 }
 
 static int start_pos(const QRect &r, Qt::Orientation o)
@@ -281,7 +302,6 @@ static void place_line(QList<Q3DockData> &lastLine, Qt::Orientation o, int lines
     }
 }
 
-
 QSize Q3DockAreaLayout::minimumSize() const
 {
     if (dockWindows->isEmpty())
@@ -367,7 +387,7 @@ int Q3DockAreaLayout::layoutItems(const QRect &rect, bool testonly)
             tbstrut = -1;
         }
 
-        // remeber first widget of a line
+        // remember first widget of a line
         if (lastLine.isEmpty()) {
             ls.append(dw);
             // try to make the best position
@@ -496,14 +516,10 @@ int Q3DockAreaLayout::widthForHeight(int h) const
     read back later to restore the saved positions.
 
     Save the positions to a QTextStream:
-    \code
-    ts << *myDockArea;
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3dockarea.cpp 0
 
     Restore the positions from a QTextStream:
-    \code
-    ts >> *myDockArea;
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3dockarea.cpp 1
 */
 
 /*!
@@ -1249,9 +1265,9 @@ QTextStream &operator<<(QTextStream &ts, const Q3DockArea &dockArea)
 
     for (int i = 0; i < l.size(); ++i) {
         Q3DockWindow *dw = l.at(i);
-        str += "[" + QString(dw->windowTitle()) + "," + QString::number((int)dw->offset()) +
-               "," + QString::number((int)dw->newLine()) + "," + QString::number(dw->fixedExtent().width()) +
-               "," + QString::number(dw->fixedExtent().height()) + "," + QString::number((int)!dw->isHidden()) + "]";
+        str += QLatin1Char('[') + QString(dw->windowTitle()) + QLatin1Char(',') + QString::number((int)dw->offset()) +
+               QLatin1Char(',') + QString::number((int)dw->newLine()) + QLatin1Char(',') + QString::number(dw->fixedExtent().width()) +
+               QLatin1Char(',') + QString::number(dw->fixedExtent().height()) + QLatin1Char(',') + QString::number((int)!dw->isHidden()) + QLatin1Char(']');
     }
     ts << str << endl;
 
@@ -1280,16 +1296,16 @@ QTextStream &operator>>(QTextStream &ts, Q3DockArea &dockArea)
 
     for (int i = 0; i < s.length(); ++i) {
         c = s[i];
-        if (state == Pre && c == '[') {
+        if (state == Pre && c == QLatin1Char('[')) {
             state++;
             continue;
         }
-        if (c == ',' &&
+        if (c == QLatin1Char(',') &&
              (state == Name || state == Offset || state == NewLine || state == Width || state == Height)) {
             state++;
             continue;
         }
-        if (state == Visible && c == ']') {
+        if (state == Visible && c == QLatin1Char(']')) {
             for (int j = 0; j < l.size(); ++j) {
                 Q3DockWindow *dw = l.at(j);
                 if (QString(dw->windowTitle()) == name) {
@@ -1305,7 +1321,7 @@ QTextStream &operator>>(QTextStream &ts, Q3DockArea &dockArea)
                 }
             }
 
-            name = offset = newLine = width = height = visible = "";
+            name = offset = newLine = width = height = visible = QLatin1String("");
 
             state = Pre;
             continue;
@@ -1330,4 +1346,6 @@ QTextStream &operator>>(QTextStream &ts, Q3DockArea &dockArea)
 }
 #endif
 
+QT_END_NAMESPACE
+
 #endif //QT_NO_MAINWINDOW
