@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 #include "qstyleoption.h"
 #include "qvector.h"
 
+QT_BEGIN_NAMESPACE
+
 class Q3HeaderData
 {
 public:
@@ -1412,12 +1414,7 @@ void Q3Header::setCellSize(int section, int s)
     is set for all existing sections and will be applied to any new
     sections that are added.
     Example:
-    \code
-    // Allow resizing of all current and future sections
-    header->setResizeEnabled(true);
-    // Disable resizing of section 3, (the fourth section added)
-    header->setResizeEnabled(false, 3);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3header.cpp 0
 
     If the user resizes a section, a sizeChange() signal is emitted.
 
@@ -2042,4 +2039,6 @@ void Q3Header::changeEvent(QEvent *ev)
     QWidget::changeEvent(ev);
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_HEADER
