@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -42,6 +42,8 @@
 ****************************************************************************/
 #include "q3popupmenu.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \fn void Q3PopupMenu::setFrameRect(QRect)
     \internal
@@ -149,3 +151,5 @@
     \fn int Q3PopupMenu::midLineWidth() const
     \internal
 */
+
+QT_END_NAMESPACE
