@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,6 +59,8 @@
 # include "private/qt_mac_p.h"
 #endif
 
+QT_BEGIN_NAMESPACE
+
 using namespace Qt;
 
 static const int coord_limit = 4000;
@@ -105,7 +127,9 @@ public:
         : QWidget (parent,name,f) {}
 };
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "q3scrollview.moc"
+QT_END_INCLUDE_NAMESPACE
 
 class Q3ScrollViewData {
 public:
@@ -198,7 +222,7 @@ public:
     uint signal_choke : 1;
 
     // This variables indicates in updateScrollBars() that we are
-    // in a resizeEvent() and thus don't want to flash scrollbars
+    // in a resizeEvent() and thus don't want to flash scroll bars
     uint inresize : 1;
     uint use_cached_size_hint : 1;
     QSize cachedSizeHint;
@@ -364,25 +388,16 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     reliable size on X11 servers). In this usage, you just make one
     large child in the Q3ScrollView. The child should be a child of the
     viewport() of the scrollview and be added with addChild():
-    \code
-        Q3ScrollView* sv = new Q3ScrollView(...);
-        QVBoxWidget* big_box = new QVBoxWidget(sv->viewport());
-        sv->addChild(big_box);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3scrollview.cpp 0
     You can go on to add arbitrary child widgets to the single child
     in the scrollview as you would with any widget:
-    \code
-        QLabel* child1 = new QLabel("CHILD", big_box);
-        QLabel* child2 = new QLabel("CHILD", big_box);
-        QLabel* child3 = new QLabel("CHILD", big_box);
-        ...
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3scrollview.cpp 1
 
     Here the Q3ScrollView has four children: the viewport(), the
     verticalScrollBar(), the horizontalScrollBar() and a small
-    cornerWidget(). The viewport() has one child: the big QVBoxWidget. The
-    QVBoxWidget has the three QLabel objects as child widgets. When the view
-    is scrolled, the QVBoxWidget is moved; its children move with it as
+    cornerWidget(). The viewport() has one child: the QWidget. The
+    QWidget has the three QLabel objects as child widgets. When the view
+    is scrolled, the QWidget is moved; its children move with it as
     child widgets normally do.
 
     \section1 Using a Very Big View with Some Widgets
@@ -397,15 +412,7 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     add some widgets by making them children of the viewport() and
     adding them with addChild() (this is the same as the process for
     the single large widget in the previous example):
-    \code
-        Q3ScrollView* sv = new Q3ScrollView(...);
-        QLabel* child1 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child1);
-        QLabel* child2 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child2);
-        QLabel* child3 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child3);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3scrollview.cpp 2
     Here, the Q3ScrollView has the same four children: the viewport(),
     the verticalScrollBar(), the horizontalScrollBar() and a small
     cornerWidget(). The viewport() has the three QLabel objects as
@@ -423,16 +430,7 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     reimplement drawContents() to paint the contents. You then call
     enableClipper(true) and add widgets, again by making them children
     of the viewport(), and adding them with addChild():
-    \code
-        Q3ScrollView* sv = new Q3ScrollView(...);
-        sv->enableClipper(true);
-        QLabel* child1 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child1);
-        QLabel* child2 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child2);
-        QLabel* child3 = new QLabel("CHILD", sv->viewport());
-        sv->addChild(child3);
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3scrollview.cpp 3
 
     Here, the Q3ScrollView has four children:  the clipper() (not the
     viewport() this time), the verticalScrollBar(), the
@@ -497,8 +495,8 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     childX() and childY() to get the position of a child widget.
 
     A widget may be placed in the corner between the vertical and
-    horizontal scrollbars with setCornerWidget(). You can get access
-    to the scrollbars using horizontalScrollBar() and
+    horizontal scroll bars with setCornerWidget(). You can get access
+    to the scroll bars using horizontalScrollBar() and
     verticalScrollBar(), and to the viewport with viewport(). The
     scroll view can be scrolled using scrollBy(), ensureVisible(),
     setContentsPos() or center().
@@ -517,7 +515,7 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     \warning Q3ScrollView currently does not erase the background when
     resized, i.e. you must always clear the background manually in
     scrollview subclasses. This will change in a future version of Qt
-    and we recommend specifying the WNoAutoErase flag explicitly.
+    and we recommend specifying the \c WNoAutoErase flag explicitly.
 */
 
 
@@ -774,11 +772,11 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
     int vsbExt = verticalScrollBar()->sizeHint().width();
 
     if (d->policy != AutoOne || d->anyVisibleChildren()) {
-        // Do we definitely need the scrollbar?
+        // Do we definitely need the scroll bar?
         needh = w-lmarg-rmarg < x;
         needv = h-tmarg-bmarg < y;
 
-        // Do we intend to show the scrollbar?
+        // Do we intend to show the scroll bar?
         if (d->hMode == AlwaysOn)
             showh = true;
         else if (d->hMode == AlwaysOff)
@@ -793,7 +791,7 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
         else
             showv = needv;
 
-        // Given other scrollbar will be shown, NOW do we need one?
+        // Given other scroll bar will be shown, NOW do we need one?
         if (showh && h-vsbExt-tmarg-bmarg < y) {
             if (d->vMode == Auto)
                 showv=true;
@@ -803,7 +801,7 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
                 showh=true;
         }
     } else {
-        // Scrollbars not needed, only show scrollbar that are always on.
+        // Scroll bars not needed, only show scroll bar that are always on.
         showh = d->hMode == AlwaysOn;
         showv = d->vMode == AlwaysOn;
     }
@@ -847,13 +845,13 @@ void Q3ScrollView::updateScrollBars()
     QSize oldVisibleSize(visibleWidth(), visibleHeight());
 
     if (d->policy != AutoOne || d->anyVisibleChildren()) {
-        // Do we definitely need the scrollbar?
+        // Do we definitely need the scroll bar?
         needh = w-lmarg-rmarg < d->contentsWidth();
         if (d->inresize)
             needh  = !horizontalScrollBar()->isHidden();
         needv = h-tmarg-bmarg < contentsHeight();
 
-        // Do we intend to show the scrollbar?
+        // Do we intend to show the scroll bar?
         if (d->hMode == AlwaysOn)
             showh = true;
         else if (d->hMode == AlwaysOff)
@@ -893,7 +891,7 @@ void Q3ScrollView::updateScrollBars()
         }
 #endif
 
-        // Given other scrollbar will be shown, NOW do we need one?
+        // Given other scroll bar will be shown, NOW do we need one?
         if (showh && h-vsbExt-tmarg-bmarg < contentsHeight()) {
             needv=true;
             if (d->vMode == Auto)
@@ -905,7 +903,7 @@ void Q3ScrollView::updateScrollBars()
                 showh=true;
         }
     } else {
-        // Scrollbars not needed, only show scrollbar that are always on.
+        // Scrollbars not needed, only show scroll bar that are always on.
         needh = needv = false;
         showh = d->hMode == AlwaysOn;
         showv = d->vMode == AlwaysOn;
@@ -914,7 +912,7 @@ void Q3ScrollView::updateScrollBars()
     bool sc = d->signal_choke;
     d->signal_choke=true;
 
-    // Hide unneeded scrollbar, calculate viewport size
+    // Hide unneeded scroll bar, calculate viewport size
     if (showh) {
         porth=h-hsbExt-tmarg-bmarg;
     } else {
@@ -932,7 +930,7 @@ void Q3ScrollView::updateScrollBars()
         portw=w-lmarg-rmarg;
     }
 
-    // Configure scrollbars that we will show
+    // Configure scroll bars that we will show
     if (needv) {
         d->vbar->setRange(0, contentsHeight()-porth);
         d->vbar->setSteps(Q3ScrollView::d->vbar->lineStep(), porth);
@@ -946,7 +944,7 @@ void Q3ScrollView::updateScrollBars()
         d->hbar->setRange(0, 0);
     }
 
-    // Position the scrollbars, viewport and corner widget.
+    // Position the scroll bars, viewport and corner widget.
     int bottom;
     bool reverse = QApplication::reverseLayout();
     int xoffset = (reverse && (showv || cornerWidget())) ? vsbExt : 0;
@@ -1455,6 +1453,7 @@ int Q3ScrollView::childY(QWidget* child)
 
 bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
 {
+    bool disabled = !(qobject_cast<QWidget*>(obj)->isEnabled());
     if (!d)
         return false; // we are destructing
     if (obj == d->viewport || obj == d->clipped_viewport) {
@@ -1468,30 +1467,42 @@ bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
                 viewportResizeEvent((QResizeEvent *)e);
             break;
         case QEvent::MouseButtonPress:
+            if (disabled)
+                return false;
             viewportMousePressEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseButtonRelease:
+            if (disabled)
+                return false;
             viewportMouseReleaseEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseButtonDblClick:
+            if (disabled)
+                return false;
             viewportMouseDoubleClickEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseMove:
+            if (disabled)
+                return false;
             viewportMouseMoveEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
 #ifndef QT_NO_DRAGANDDROP
         case QEvent::DragEnter:
+            if (disabled)
+                return false;
             viewportDragEnterEvent((QDragEnterEvent*)e);
             break;
         case QEvent::DragMove: {
+            if (disabled)
+                return false;
             if (d->drag_autoscroll) {
                 QPoint vp = ((QDragMoveEvent*) e)->pos();
                 QRect inside_margin(autoscroll_margin, autoscroll_margin,
@@ -1506,15 +1517,27 @@ bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
             viewportDragMoveEvent((QDragMoveEvent*)e);
         } break;
         case QEvent::DragLeave:
+            if (disabled)
+                return false;
             stopDragAutoScroll();
             viewportDragLeaveEvent((QDragLeaveEvent*)e);
             break;
         case QEvent::Drop:
+            if (disabled)
+                return false;
             stopDragAutoScroll();
             viewportDropEvent((QDropEvent*)e);
             break;
 #endif // QT_NO_DRAGANDDROP
+#ifndef QT_NO_WHEELEVENT
+        case QEvent::Wheel:
+            if (disabled)
+                return false;
+            break;
+#endif
         case QEvent::ContextMenu:
+            if (disabled)
+                return false;
             viewportContextMenuEvent((QContextMenuEvent*)e);
             if (((QContextMenuEvent*)e)->isAccepted())
                 return true;
@@ -2326,25 +2349,7 @@ void Q3ScrollView::drawContentsOffset(QPainter* p, int offsetx, int offsety, int
     rectangle is in the scrollview's coordinates.
 
     For example:
-    \code
-    {
-        // Fill a 40000 by 50000 rectangle at (100000,150000)
-
-        // Calculate the coordinates...
-        int x1 = 100000, y1 = 150000;
-        int x2 = x1+40000-1, y2 = y1+50000-1;
-
-        // Clip the coordinates so X/Windows will not have problems...
-        if (x1 < clipx) x1=clipx;
-        if (y1 < clipy) y1=clipy;
-        if (x2 > clipx+clipw-1) x2=clipx+clipw-1;
-        if (y2 > clipy+cliph-1) y2=clipy+cliph-1;
-
-        // Paint using the small coordinates...
-        if (x2 >= x1 && y2 >= y1)
-            p->fillRect(x1, y1, x2-x1+1, y2-y1+1, red);
-    }
-    \endcode
+    \snippet doc/src/snippets/code/src.qt3support.widgets.q3scrollview.cpp 4
 
     The clip rectangle and translation of the painter \a p is already
     set appropriately.
@@ -2795,4 +2800,6 @@ QSize Q3ScrollView::cachedSizeHint() const
     return d->use_cached_size_hint ? d->cachedSizeHint : QSize();
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SCROLLVIEW
