@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,7 +46,7 @@
 
 #include "q3mainwindow.h"
 #include "qapplication.h"
-#include "qcombobox.h"
+#include "q3combobox.h"
 #include "qcursor.h"
 #include "qdesktopwidget.h"
 #include "qdrawutil.h"
@@ -57,6 +62,8 @@
 #include "qtoolbutton.h"
 #include "qtooltip.h"
 
+QT_BEGIN_NAMESPACE
+
 static const char * const arrow_v_xpm[] = {
     "7 9 3 1",
     "            c None",
@@ -219,7 +226,9 @@ void Q3ToolBarSeparator::paintEvent(QPaintEvent *)
     style()->drawPrimitive(QStyle::PE_Q3DockWindowSeparator, &opt, &p, this);
 }
 
+QT_BEGIN_INCLUDE_NAMESPACE
 #include "q3toolbar.moc"
+QT_END_INCLUDE_NAMESPACE
 
 
 /*!
@@ -489,7 +498,8 @@ bool Q3ToolBar::event(QEvent * e)
         if (child && child->isWidgetType() && !((QWidget*)child)->isWindow()
              && child->parent() == this
             && QLatin1String("qt_dockwidget_internal") != child->objectName()) {
-            QWidgetItem *item = new QWidgetItem((QWidget*)child);
+            boxLayout()->addWidget((QWidget*)child);
+            QLayoutItem *item = boxLayout()->itemAt(boxLayout()->indexOf((QWidget*)child));
             if (QToolButton *button = qobject_cast<QToolButton*>(child)) {
                 item->setAlignment(Qt::AlignHCenter);
                 button->setFocusPolicy(Qt::NoFocus);
@@ -503,7 +513,6 @@ bool Q3ToolBar::event(QEvent * e)
                 }
                 button->setAutoRaise(true);
             }
-            boxLayout()->addItem(item);
             if (isVisible()) {
                 // toolbar compatibility: we auto show widgets that
                 // are not explicitly hidden
@@ -551,6 +560,8 @@ QString Q3ToolBar::label() const
 void Q3ToolBar::clear()
 {
     QObjectList childList = children();
+    d->extension = 0;
+    d->extensionPopup = 0; //they will both be destroyed by the following code
     for (int i = 0; i < childList.size(); ++i) {
         QObject *obj = childList.at(i);
         if (obj->isWidgetType() && QLatin1String("qt_dockwidget_internal") != obj->objectName())
@@ -613,7 +624,7 @@ void Q3ToolBar::createPopup()
         }
         int j = 2;
         QWidget *w = (QWidget*)obj;
-        if (qobject_cast<QComboBox*>(w))
+        if (qobject_cast<Q3ComboBox*>(w))
             j = 1;
         hide = false;
 
@@ -657,8 +668,8 @@ void Q3ToolBar::createPopup()
                 if (!b->isEnabled())
                     d->extensionPopup->setItemEnabled(id, false);
 #ifndef QT_NO_COMBOBOX
-            } else if (qobject_cast<QComboBox*>(w)) {
-                QComboBox *c = (QComboBox*)w;
+            } else if (qobject_cast<Q3ComboBox*>(w)) {
+                Q3ComboBox *c = (Q3ComboBox*)w;
                 if (c->count() != 0) {
                     QString s = c->windowTitle();
                     if (s.isEmpty())
@@ -825,4 +836,7 @@ more predictable.
 11.  Keep a single tool bar to fewer than 20 items divided into 4-7 groups of
 items.
 */
+
+QT_END_NAMESPACE
+
 #endif
