@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,6 +44,8 @@
 
 #include "q3vbox.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class Q3VBox qvbox.h
     \brief The Q3VBox widget provides vertical geometry management of
@@ -68,3 +70,5 @@ Q3VBox::Q3VBox( QWidget *parent, const char *name, Qt::WindowFlags f )
     :Q3HBox( false, parent, name, f )
 {
 }
+
+QT_END_NAMESPACE
