@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -66,9 +66,10 @@
 } while (0)
 
 
-
 #include "qscriptparser_p.h"
 
+QT_BEGIN_NAMESPACE
+
 inline static bool automatic(QScriptEnginePrivate *driver, int token)
 {
     return token == QScriptGrammar::T_RBRACE
@@ -1070,4 +1071,6 @@ case 228: {
     return false;
 }
 
+QT_END_NAMESPACE
+
 #endif // QT_NO_SCRIPT
