@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -65,6 +65,8 @@ typedef struct sqlite_vm sqlite_vm;
 Q_DECLARE_METATYPE(sqlite_vm*)
 Q_DECLARE_METATYPE(sqlite*)
 
+QT_BEGIN_NAMESPACE
+
 static QVariant::Type nameToType(const QString& typeName)
 {
     QString tName = typeName.toUpper();
@@ -532,3 +534,4 @@ QVariant QSQLite2Driver::handle() const
     return qVariantFromValue(d->access);
 }
 
+QT_END_NAMESPACE
