@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,6 +47,8 @@
 #include <qdatetime.h>
 #include <qvector.h>
 
+QT_BEGIN_NAMESPACE
+
 /*
    QSqlCachedResult is a convenience class for databases that only allow
    forward only fetching. It will cache all the results so we can iterate
@@ -183,10 +185,12 @@ bool QSqlCachedResult::fetch(int i)
     }
     if (d->rowCacheEnd > 0)
         setAt(d->cacheCount());
-    while (at() < i) {
+    while (at() < i + 1) {
         if (!cacheNext())
             return false;
     }
+    setAt(i);
+
     return true;
 }
 
@@ -292,3 +296,4 @@ QSqlCachedResult::ValueCache &QSqlCachedResult::cache()
     return d->cache;
 }
 
+QT_END_NAMESPACE
