@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -52,6 +52,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 QT_MODULE(Sql)
 
 class QSqlError;
@@ -119,6 +121,7 @@ public:
     QString driverName() const;
     int port() const;
     QString connectOptions() const;
+    QString connectionName() const;
 
     QSqlDriver* driver() const;
 
@@ -151,6 +154,8 @@ private:
 Q_SQL_EXPORT QDebug operator<<(QDebug, const QSqlDatabase &);
 #endif
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // QSQLDATABASE_H
