@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,11 +49,14 @@
 #include "qsqlindex.h"
 #include "qsqlquery.h"
 #include "qsqlrecord.h"
+#include "qsqlresult.h"
 
 #include "qsqltablemodel_p.h"
 
 #include <qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 /*! \internal
     Populates our record with values.
 */
@@ -116,6 +139,11 @@ void QSqlTableModelPrivate::clearEditBuffer()
     editBuffer = rec;
 }
 
+void QSqlTableModelPrivate::clearCache()
+{
+    cache.clear();
+}
+
 void QSqlTableModelPrivate::revertCachedRow(int row)
 {
     Q_Q(QSqlTableModel);
@@ -158,6 +186,11 @@ bool QSqlTableModelPrivate::exec(const QString &stmt, bool prepStatement,
     if (editQuery.driver() != db.driver())
         editQuery = QSqlQuery(db);
 
+    // workaround for In-Process databases - remove all read locks
+    // from the table to make sure the editQuery succeeds
+    if (db.driver()->hasFeature(QSqlDriver::SimpleLocking))
+        const_cast<QSqlResult *>(query.result())->detachFromResultSet();
+
     if (prepStatement) {
         if (editQuery.lastQuery() != stmt) {
             if (!editQuery.prepare(stmt)) {
@@ -220,10 +253,7 @@ QSqlRecord QSqlTableModelPrivate::primaryValues(int row)
     lower-level QSqlQuery and can be used to provide data to view
     classes such as QTableView. For example:
 
-    \quotefromfile snippets/sqldatabase/sqldatabase.cpp
-    \skipto QSqlTableModel_snippets
-    \skipto QSqlTableModel *model
-    \printuntil show()
+    \snippet doc/src/snippets/sqldatabase/sqldatabase.cpp 24
 
     We set the SQL table's name and the edit strategy, then we set up
     the labels displayed in the view header. The edit strategy
@@ -234,8 +264,7 @@ QSqlRecord QSqlTableModelPrivate::primaryValues(int row)
     QSqlTableModel can also be used to access a database
     programmatically, without binding it to a view:
 
-    \skipto QSqlTableModel model
-    \printuntil QString name =
+    \snippet doc/src/snippets/sqldatabase/sqldatabase.cpp 25
 
     The code snippet above extracts the \c salary field from record 4 in
     the result set of the query \c{SELECT * from employee}.
@@ -328,7 +357,9 @@ QSqlTableModel::~QSqlTableModel()
 
     To populate the model with the table's data, call select().
 
-    \sa select(), setFilter()
+    Error information can be retrieved with \l lastError().
+
+    \sa select(), setFilter(), lastError()
 */
 void QSqlTableModel::setTable(const QString &tableName)
 {
@@ -336,6 +367,11 @@ void QSqlTableModel::setTable(const QString &tableName)
     clear();
     d->tableName = tableName;
     d->initRecordAndPrimaryIndex();
+    d->initColOffsets(d->rec.count());
+
+    if (d->rec.count() == 0)
+        d->error = QSqlError(QLatin1String("Unable to find table ") + d->tableName, QString(),
+                             QSqlError::StatementError);
 }
 
 /*!
@@ -538,8 +574,9 @@ void QSqlTableModel::setQuery(const QSqlQuery &query)
 }
 
 /*!
-    Updates the row \a row in the currently active database table
-    with the values from \a values.
+    Updates the given \a row in the currently active database table
+    with the specified \a values. Returns true if successful; otherwise
+    returns false.
 
     This is a low-level method that operates directly on the database
     and should not be called directly. Use setData() to update values.
@@ -599,6 +636,12 @@ bool QSqlTableModel::insertRowIntoTable(const QSqlRecord &values)
     QString stmt = d->db.driver()->sqlStatement(QSqlDriver::InsertStatement, d->tableName,
                                                 rec, prepStatement);
 
+    if (stmt.isEmpty()) {
+        d->error = QSqlError(QLatin1String("No Fields to update"), QString(),
+                                 QSqlError::StatementError);
+        return false;
+    }
+
     return d->exec(stmt, prepStatement, rec);
 }
 
@@ -641,6 +684,9 @@ bool QSqlTableModel::deleteRowFromTable(int row)
     Returns false on error, detailed error information can be
     obtained with lastError().
 
+    On success the model will be repopulated. Any views 
+    presenting it will lose their selections.
+
     Note: In OnManualSubmit mode, already submitted changes won't
     be cleared from the cache when submitAll() fails. This allows
     transactions to be rolled back and resubmitted again without
@@ -663,6 +709,7 @@ bool QSqlTableModel::submitAll()
         if (d->insertIndex != -1) {
             if (!insertRowIntoTable(d->editBuffer))
                 return false;
+            d->bottom = d->bottom.sibling(d->bottom.row() + 1, d->bottom.column());
         } else {
             if (!updateRowInTable(d->editIndex, d->editBuffer))
                 return false;
@@ -678,6 +725,7 @@ bool QSqlTableModel::submitAll()
             case QSqlTableModelPrivate::Insert:
                 if (!insertRowIntoTable(it.value().rec))
                     return false;
+                d->bottom = d->bottom.sibling(d->bottom.row() + 1, d->bottom.column());
                 break;
             case QSqlTableModelPrivate::Update:
                 if (!updateRowInTable(it.key(), it.value().rec))
@@ -692,7 +740,7 @@ bool QSqlTableModel::submitAll()
                 break;
             }
         }
-        d->cache.clear();
+        d->clearCache();
         return select();
     }
     return false;
@@ -712,6 +760,9 @@ bool QSqlTableModel::submitAll()
     Returns true on success; otherwise returns false. Use lastError()
     to query detailed error information.
 
+    On success the model will be repopulated. Any views 
+    presenting it will lose their selections.
+
     \sa revert(), revertRow(), submitAll(), revertAll(), lastError()
 */
 bool QSqlTableModel::submit()
@@ -888,7 +939,7 @@ void QSqlTableModel::sort(int column, Qt::SortOrder order)
 }
 
 /*!
-    Sets the sort oder for \a column to \a order. This does not
+    Sets the sort order for \a column to \a order. This does not
     affect the current data, to refresh the data using the new
     sort order, call select().
 
@@ -914,8 +965,12 @@ QString QSqlTableModel::orderByClause() const
     QSqlField f = d->rec.field(d->sortColumn);
     if (!f.isValid())
         return s;
-    s.append(QLatin1String("ORDER BY ")).append(d->tableName).append(QLatin1Char('.')).append(f.name());
+        
+    QString table = d->db.driver()->escapeIdentifier(d->tableName, QSqlDriver::TableName);
+    QString field = d->db.driver()->escapeIdentifier(f.name(), QSqlDriver::FieldName);
+    s.append(QLatin1String("ORDER BY ")).append(table).append(QLatin1Char('.')).append(field);
     s += d->sortOrder == Qt::AscendingOrder ? QLatin1String(" ASC") : QLatin1String(" DESC");
+
     return s;
 }
 
@@ -992,7 +1047,9 @@ bool QSqlTableModel::removeColumns(int column, int count, const QModelIndex &par
     does not support hierarchical structures, \a parent must be
     an invalid model index.
 
-    Emits the beforeDelete() signal before a row is deleted.
+    Emits the beforeDelete() signal before a row is deleted. When
+    the edit strategy is OnManualSubmit signal emission is delayed
+    until submitAll() is called.
 
     Returns true if all rows could be removed; otherwise returns
     false. Detailed error information can be retrieved using
@@ -1025,8 +1082,10 @@ bool QSqlTableModel::removeRows(int row, int count, const QModelIndex &parent)
                 return false;
             if (d->cache.value(idx).op == QSqlTableModelPrivate::Insert)
                 revertRow(idx);
-            else
+            else {
                 d->cache[idx].op = QSqlTableModelPrivate::Delete;
+                emit headerDataChanged(Qt::Vertical, idx, idx);
+            }
         }
         break;
     }
@@ -1255,3 +1314,5 @@ bool QSqlTableModel::setRecord(int row, const QSqlRecord &record)
     }
     return false;
 }
+
+QT_END_NAMESPACE
