@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,8 +44,12 @@
 #include "qsvgnode_p.h"
 #include "qsvgtinydocument_p.h"
 
+#ifndef QT_NO_SVG
+
 #include "qdebug.h"
 
+QT_BEGIN_NAMESPACE
+
 QSvgNode::QSvgNode(QSvgNode *parent)
     : m_parent(parent),
       m_visible(true),
@@ -318,7 +322,11 @@ qreal QSvgNode::strokeWidth() const
 {
     QSvgStrokeStyle *stroke = static_cast<QSvgStrokeStyle*>(
         styleProperty(QSvgStyleProperty::STROKE));
-    if (!stroke)
+    if (!stroke || stroke->qpen().style() == Qt::NoPen)
         return 0;
     return stroke->qpen().widthF();
 }
+
+QT_END_NAMESPACE
+
+#endif // QT_NO_SVG
