@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -57,9 +57,13 @@
 
 #include "qsvgstyle_p.h"
 
+#ifndef QT_NO_SVG
+
 #include "QtCore/qstring.h"
 #include "QtCore/qhash.h"
 
+QT_BEGIN_NAMESPACE
+
 class QPainter;
 class QSvgTinyDocument;
 
@@ -195,4 +199,7 @@ inline QString QSvgNode::xmlClass() const
     return m_class;
 }
 
+QT_END_NAMESPACE
+
+#endif // QT_NO_SVG
 #endif // QSVGNODE_P_H
