@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,6 +47,8 @@
 #include <QtCore/qmetatype.h>
 #include <stdio.h>
 
+QT_BEGIN_NAMESPACE
+
 // if the flags change, you MUST to change it in qmetaobject.cpp too
 enum PropertyFlags  {
     Invalid = 0x00000000,
@@ -87,12 +109,36 @@ bool isVariantType(const char* type)
     return qvariant_nameToType(type) != 0;
 }
 
-Generator::Generator(FILE *outfile, ClassDef *classDef, const QList<QByteArray> &metaTypes) : out(outfile), cdef(classDef), metaTypes(metaTypes)
+Generator::Generator(ClassDef *classDef, const QList<QByteArray> &metaTypes, FILE *outfile)
+    : out(outfile), cdef(classDef), metaTypes(metaTypes)
 {
     if (cdef->superclassList.size())
         purestSuperClass = cdef->superclassList.first().first;
 }
 
+static inline int lengthOfEscapeSequence(const QByteArray &s, int i)
+{
+    if (s.at(i) != '\\' || i >= s.length() - 1)
+        return 1;
+    const int startPos = i;
+    ++i;
+    char ch = s.at(i);
+    if (ch == 'x') {
+        ++i;
+        while (i < s.length() && is_hex_char(s.at(i)))
+            ++i;
+    } else if (is_octal_char(ch)) {
+        while (i < startPos + 4
+               && i < s.length()
+               && is_octal_char(s.at(i))) {
+            ++i;
+        }
+    } else { // single character escape sequence
+        i = qMin(i + 1, s.length());
+    }
+    return i - startPos;
+}
+
 int Generator::strreg(const char *s)
 {
     int idx = 0;
@@ -103,7 +149,13 @@ int Generator::strreg(const char *s)
         if (str == s)
             return idx;
         idx += str.length() + 1;
-        idx -= str.count('\\');
+        for (int i = 0; i < str.length(); ++i) {
+            if (str.at(i) == '\\') {
+                int cnt = lengthOfEscapeSequence(str, i) - 1;
+                idx -= cnt;
+                i += cnt;
+            }
+        }
     }
     strings.append(s);
     return idx;
@@ -210,7 +262,25 @@ void Generator::generateCode()
             fprintf(out, "\"\"");
             len += 2;
         }
-        fprintf(out, "%s\\0", s.constData());
+        int idx = 0;
+        while (idx < s.length()) {
+            if (idx > 0) {
+                col = 0;
+                fprintf(out, "\"\n    \"");
+            }
+            int spanLen = qMin(70, s.length() - idx);
+            // don't cut escape sequences at the end of a line
+            int backSlashPos = s.lastIndexOf('\\', idx + spanLen - 1);
+            if (backSlashPos >= idx) {
+                int escapeLen = lengthOfEscapeSequence(s, backSlashPos);
+                spanLen = qBound(spanLen, backSlashPos + escapeLen - idx, s.length() - idx);
+            }
+            fwrite(s.constData() + idx, 1, spanLen, out);
+            idx += spanLen;
+            col += spanLen;
+        }
+
+        fputs("\\0", out);
         col += len + 2;
     }
     fprintf(out, "\"\n};\n\n");
@@ -276,13 +346,13 @@ void Generator::generateCode()
     fprintf(out, "\nvoid *%s::qt_metacast(const char *_clname)\n{\n", cdef->qualified.constData());
     fprintf(out, "    if (!_clname) return 0;\n");
     fprintf(out, "    if (!strcmp(_clname, qt_meta_stringdata_%s))\n"
-                  "\treturn static_cast<void*>(const_cast<%s*>(this));\n",
+                  "\treturn static_cast<void*>(const_cast< %s*>(this));\n",
             qualifiedClassNameIdentifier.constData(), cdef->classname.constData());
     for (int i = 1; i < cdef->superclassList.size(); ++i) { // for all superclasses but the first one
         if (cdef->superclassList.at(i).second == FunctionDef::Private)
             continue;
         const char *cname = cdef->superclassList.at(i).first;
-        fprintf(out, "    if (!strcmp(_clname, \"%s\"))\n\treturn static_cast<%s*>(const_cast<%s*>(this));\n",
+        fprintf(out, "    if (!strcmp(_clname, \"%s\"))\n\treturn static_cast< %s*>(const_cast< %s*>(this));\n",
                 cname, cname, cdef->classname.constData());
     }
     for (int i = 0; i < cdef->interfaceList.size(); ++i) {
@@ -290,8 +360,8 @@ void Generator::generateCode()
         for (int j = 0; j < iface.size(); ++j) {
             fprintf(out, "    if (!strcmp(_clname, %s))\n\treturn ", iface.at(j).interfaceId.constData());
             for (int k = j; k >= 0; --k)
-                fprintf(out, "static_cast<%s*>(", iface.at(k).className.constData());
-            fprintf(out, "const_cast<%s*>(this)%s;\n",
+                fprintf(out, "static_cast< %s*>(", iface.at(k).className.constData());
+            fprintf(out, "const_cast< %s*>(this)%s;\n",
                     cdef->classname.constData(), QByteArray(j+1, ')').constData());
         }
     }
@@ -789,7 +859,7 @@ void Generator::generateSignal(FunctionDef *def,int index)
     const char *constQualifier = "";
 
     if (def->isConst) {
-        thisPtr = "const_cast<";
+        thisPtr = "const_cast< ";
         thisPtr += cdef->qualified;
         thisPtr += " *>(this)";
         constQualifier = "const";
@@ -841,3 +911,306 @@ void Generator::generateSignal(FunctionDef *def,int index)
         fprintf(out, "    return _t0;\n");
     fprintf(out, "}\n");
 }
+
+//
+// Functions used when generating QMetaObject directly
+//
+// Much of this code is copied from the corresponding
+// C++ code-generating functions; we can change the
+// two generators so that more of the code is shared.
+// The key difference from the C++ code generator is
+// that instead of calling fprintf(), we append bytes
+// to a buffer.
+//
+
+QMetaObject *Generator::generateMetaObject(bool ignoreProperties)
+{
+//
+// build the data array
+//
+
+    // filter out undeclared enumerators and sets
+    {
+        QList<EnumDef> enumList;
+        for (int i = 0; i < cdef->enumList.count(); ++i) {
+            EnumDef def = cdef->enumList.at(i);
+            if (cdef->enumDeclarations.contains(def.name)) {
+                enumList += def;
+            }
+            QByteArray alias = cdef->flagAliases.value(def.name);
+            if (cdef->enumDeclarations.contains(alias)) {
+                def.name = alias;
+                enumList += def;
+            }
+        }
+        cdef->enumList = enumList;
+    }
+
+    int index = 10;
+    meta_data
+        << 1 // revision
+        << strreg(cdef->qualified) // classname
+        << cdef->classInfoList.count() << (cdef->classInfoList.count() ? index : 0) // classinfo
+        ;
+    index += cdef->classInfoList.count() * 2;
+
+    int methodCount = cdef->signalList.count() + cdef->slotList.count() + cdef->methodList.count();
+    meta_data << methodCount << (methodCount ? index : 0); // methods
+    index += methodCount * 5;
+    if (!ignoreProperties) {
+        meta_data << cdef->propertyList.count() << (cdef->propertyList.count() ? index : 0); // properties
+        index += cdef->propertyList.count() * 3;
+    } else {
+        meta_data << 0 << 0; // properties
+    }
+    meta_data << cdef->enumList.count() << (cdef->enumList.count() ? index : 0); // enums/sets
+
+//
+// Build classinfo array
+//
+    _generateClassInfos();
+
+//
+// Build signals array first, otherwise the signal indices would be wrong
+//
+    _generateFunctions(cdef->signalList, MethodSignal);
+
+//
+// Build slots array
+//
+    _generateFunctions(cdef->slotList, MethodSlot);
+
+//
+// Build method array
+//
+    _generateFunctions(cdef->methodList, MethodMethod);
+
+
+//
+// Build property array
+//
+    if (!ignoreProperties)
+        _generateProperties();
+
+//
+// Build enums array
+//
+    _generateEnums(index);
+
+//
+// Terminate data array
+//
+    meta_data << 0;
+
+//
+// Build stringdata array
+//
+    QVector<char> string_data;
+    for (int i = 0; i < strings.size(); ++i) {
+        const char *s = strings.at(i).constData();
+        char c;
+        do {
+            c = *(s++);
+            string_data << c;
+        } while (c != '\0');
+    }
+
+//
+// Finally create and initialize the static meta object
+//
+    const int meta_object_offset = 0;
+    const int meta_object_size = sizeof(QMetaObject);
+    const int meta_data_offset = meta_object_offset + meta_object_size;
+    const int meta_data_size = meta_data.count() * sizeof(uint);
+    const int string_data_offset = meta_data_offset + meta_data_size;
+    const int string_data_size = string_data.count();
+    const int total_size = string_data_offset + string_data_size;
+
+    char *blob = new char[total_size];
+
+    char *string_data_output = blob + string_data_offset;
+    const char *string_data_src = string_data.constData();
+    for (int i = 0; i < string_data.count(); ++i)
+        string_data_output[i] = string_data_src[i];
+
+    uint *meta_data_output = reinterpret_cast<uint *>(blob + meta_data_offset);
+    const uint *meta_data_src = meta_data.constData();
+    for (int i = 0; i < meta_data.count(); ++i)
+        meta_data_output[i] = meta_data_src[i];
+
+    QMetaObject *meta_object = new (blob + meta_object_offset)QMetaObject;
+    meta_object->d.superdata = 0;
+    meta_object->d.stringdata = string_data_output;
+    meta_object->d.data = meta_data_output;
+    meta_object->d.extradata = 0;
+    return meta_object;
+}
+
+void Generator::_generateClassInfos()
+{
+    for (int i = 0; i < cdef->classInfoList.size(); ++i) {
+        const ClassInfoDef &c = cdef->classInfoList.at(i);
+        meta_data << strreg(c.name) << strreg(c.value);
+    }
+}
+
+void Generator::_generateFunctions(QList<FunctionDef> &list, int type)
+{
+    for (int i = 0; i < list.count(); ++i) {
+        const FunctionDef &f = list.at(i);
+
+        QByteArray sig = f.name + '(';
+        QByteArray arguments;
+
+        for (int j = 0; j < f.arguments.count(); ++j) {
+            const ArgumentDef &a = f.arguments.at(j);
+            if (j) {
+                sig += ",";
+                arguments += ",";
+            }
+            sig += a.normalizedType;
+            arguments += a.name;
+        }
+        sig += ')';
+
+        char flags = type;
+        if (f.access == FunctionDef::Private)
+            flags |= AccessPrivate;
+        else if (f.access == FunctionDef::Public)
+            flags |= AccessPublic;
+        else if (f.access == FunctionDef::Protected)
+            flags |= AccessProtected;
+        if (f.access == FunctionDef::Private)
+            flags |= AccessPrivate;
+        else if (f.access == FunctionDef::Public)
+            flags |= AccessPublic;
+        else if (f.access == FunctionDef::Protected)
+            flags |= AccessProtected;
+        if (f.isCompat)
+            flags |= MethodCompatibility;
+        if (f.wasCloned)
+            flags |= MethodCloned;
+        if (f.isScriptable)
+            flags |= MethodScriptable;
+
+        meta_data << strreg(sig)
+                  << strreg(arguments)
+                  << strreg(f.normalizedType)
+                  << strreg(f.tag)
+                  << flags;
+    }
+}
+
+void Generator::_generateEnums(int index)
+{
+    index += 4 * cdef->enumList.count();
+    int i;
+    for (i = 0; i < cdef->enumList.count(); ++i) {
+        const EnumDef &e = cdef->enumList.at(i);
+        meta_data << strreg(e.name) << (cdef->enumDeclarations.value(e.name) ? 1 : 0)
+                  << e.values.count() << index;
+        index += e.values.count() * 2;
+    }
+
+    for (i = 0; i < cdef->enumList.count(); ++i) {
+        const EnumDef &e = cdef->enumList.at(i);
+        for (int j = 0; j < e.values.count(); ++j) {
+            const QByteArray &val = e.values.at(j);
+            meta_data << strreg(val) << 0; // we don't know the value itself
+        }
+    }
+}
+
+void Generator::_generateProperties()
+{
+    //
+    // specify get function, for compatibiliy we accept functions
+    // returning pointers, or const char * for QByteArray.
+    //
+    for (int i = 0; i < cdef->propertyList.count(); ++i) {
+        PropertyDef &p = cdef->propertyList[i];
+        if (p.read.isEmpty())
+            continue;
+        for (int j = 0; j < cdef->publicList.count(); ++j) {
+            const FunctionDef &f = cdef->publicList.at(j);
+            if (f.name != p.read)
+                continue;
+            if (!f.isConst) // get  functions must be const
+                continue;
+            if (f.arguments.size()) // and must not take any arguments
+                continue;
+            PropertyDef::Specification spec = PropertyDef::ValueSpec;
+            QByteArray tmp = f.normalizedType;
+            if (p.type == "QByteArray" && tmp == "const char *")
+                    tmp = "QByteArray";
+            if (tmp.left(6) == "const ")
+                tmp = tmp.mid(6);
+            if (p.type != tmp && tmp.endsWith('*')) {
+                tmp.chop(1);
+                spec = PropertyDef::PointerSpec;
+            } else if (f.type.name.endsWith('&')) { // raw type, not normalized type
+                spec = PropertyDef::ReferenceSpec;
+            }
+            if (p.type != tmp)
+                continue;
+            p.gspec = spec;
+            break;
+        }
+    }
+
+
+    //
+    // Create meta data
+    //
+
+    for (int i = 0; i < cdef->propertyList.count(); ++i) {
+        const PropertyDef &p = cdef->propertyList.at(i);
+        uint flags = Invalid;
+        if (!isVariantType(p.type)) {
+            flags |= EnumOrFlag;
+        } else {
+            flags |= qvariant_nameToType(p.type) << 24;
+        }
+        if (!p.read.isEmpty())
+            flags |= Readable;
+        if (!p.write.isEmpty()) {
+            flags |= Writable;
+            if (p.stdCppSet())
+                flags |= StdCppSet;
+        }
+        if (!p.reset.isEmpty())
+            flags |= Resettable;
+
+//         if (p.override)
+//             flags |= Override;
+
+        if (p.designable.isEmpty())
+            flags |= ResolveDesignable;
+        else if (p.designable != "false")
+            flags |= Designable;
+
+        if (p.scriptable.isEmpty())
+            flags |= ResolveScriptable;
+        else if (p.scriptable != "false")
+            flags |= Scriptable;
+
+        if (p.stored.isEmpty())
+            flags |= ResolveStored;
+        else if (p.stored != "false")
+            flags |= Stored;
+
+        if (p.editable.isEmpty())
+            flags |= ResolveEditable;
+        else if (p.editable != "false")
+            flags |= Editable;
+
+        if (p.user.isEmpty())
+            flags |= ResolveUser;
+        else if (p.user != "false")
+            flags |= User;
+
+        meta_data << strreg(p.name) << strreg(p.type) << flags;
+    }
+}
+
+QT_END_NAMESPACE
