@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -43,6 +43,8 @@
 
 #include "token.h"
 
+QT_BEGIN_NAMESPACE
+
 #if defined(DEBUG_MOC)
 const char *tokenTypeName(Token t)
 {
@@ -207,8 +209,14 @@ const char *tokenTypeName(Token t)
         case PP_INCOMPLETE: return "PP_INCOMPLETE";
         case PP_MOC_TRUE: return "PP_MOC_TRUE";
         case PP_MOC_FALSE: return "PP_MOC_FALSE";
+        case Q_DECLARE_METATYPE_TOKEN: return "Q_DECLARE_METATYPE_TOKEN";
+        case Q_MOC_COMPAT_TOKEN: return "Q_MOC_COMPAT_TOKEN";
+        case Q_QT3_SUPPORT_TOKEN: return "Q_QT3_SUPPORT_TOKEN";
+        case Q_INVOKABLE_TOKEN: return "Q_INVOKABLE_TOKEN";
+        case Q_SCRIPTABLE_TOKEN: return "Q_SCRIPTABLE_TOKEN";
     }
     return "";
 }
 #endif
 
+QT_END_NAMESPACE
