@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,6 +44,10 @@
 #ifndef UTILS_H
 #define UTILS_H
 
+#include <QtCore/qglobal.h>
+
+QT_BEGIN_NAMESPACE
+
 inline bool is_whitespace(char s)
 {
     return (s == ' ' || s == '\t' || s == '\n');
@@ -104,4 +108,6 @@ inline const char *skipQuote(const char *data)
     return data; 
 }
 
+QT_END_NAMESPACE
+
 #endif // UTILS_H
