@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -41,71 +41,20 @@
 **
 ****************************************************************************/
 
-#include "rcc.h"
+#include <rcc.h>
 #include "../../corelib/kernel/qcorecmdlineargs_p.h"
 
-#include <QFile>
+#include <QDebug>
 #include <QDir>
+#include <QFile>
 
-// Some static globals
-static bool writeBinary = false;
-static QString initName;
-static bool verbose = false;
-static int compressLevel = CONSTANT_COMPRESSLEVEL_DEFAULT;
-static int compressThreshold = CONSTANT_COMPRESSTHRESHOLD_DEFAULT;
-static QString resourceRoot;
-
-bool processResourceFile(const QStringList &filenamesIn, const QString &filenameOut, bool list)
-{
-    if (verbose)
-        fprintf(stderr, "Qt resource compiler\n");
-
-    //setup
-    RCCResourceLibrary library;
-    library.setFormat(writeBinary ? RCCResourceLibrary::Binary : RCCResourceLibrary::C_Code);
-    library.setInputFiles(filenamesIn);
-    library.setInitName(initName);
-    library.setVerbose(verbose);
-    library.setCompressLevel(compressLevel);
-    library.setCompressThreshold(compressThreshold);
-    library.setResourceRoot(resourceRoot);
-    if(!library.readFiles(list))
-        return false;
-
-    //open output
-    FILE *out_fd = stdout;
-    if (!filenameOut.isEmpty() && filenameOut != QLatin1String("-")) {
-#if defined(_MSC_VER) && _MSC_VER >= 1400
-        if (fopen_s(&out_fd, filenameOut.toLocal8Bit().constData(), writeBinary ? "wb": "w")) {
-#else
-        if(!(out_fd = fopen(filenameOut.toLocal8Bit().constData(), writeBinary ? "wb": "w"))) {
-#endif
-            fprintf(stderr, "Unable to open %s for writing\n", qPrintable(filenameOut));
-            return false;
-        }
-    }
-
-    //do the task
-    bool ret = true;
-    if(list) {
-        const QStringList data = library.dataFiles();
-        for(int i = 0; i < data.size(); ++i)
-            fprintf(out_fd, "%s\n", qPrintable(QDir::cleanPath(data.at(i))));
-    } else {
-        ret = library.output(out_fd);
-    }
-    if(out_fd != stdout)
-        fclose(out_fd);
-
-    //done
-    return ret;
-}
+QT_BEGIN_NAMESPACE
 
 int showHelp(const QString &argv0, const QString &error)
 {
     fprintf(stderr, "Qt resource compiler\n");
     if (!error.isEmpty())
-        fprintf(stderr, "%s: %s\n", argv0.toLocal8Bit().constData(), error.toLocal8Bit().constData());
+        fprintf(stderr, "%s: %s\n", qPrintable(argv0), qPrintable(error));
     fprintf(stderr, "Usage: %s  [options] <inputs>\n\n"
             "Options:\n"
             "  -o file           write output to file rather than stdout\n"
@@ -115,27 +64,31 @@ int showHelp(const QString &argv0, const QString &error)
             "  -root path        prefix resource access path with root path\n"
             "  -no-compress      disable all compression\n"
             "  -binary           output a binary file for use as a dynamic resource\n"
+            "  -namespace        turn off namespace macros\n"
             "  -version          display version\n"
             "  -help             display this information\n",
-            argv0.toLocal8Bit().constData());
+            qPrintable(argv0));
     return 1;
 }
 
 
-int main(int argc, char *argv[])
+int runRcc(int argc, char *argv[])
 {
     QString outFilename;
-    bool helpRequested = false, list = false;
+    bool helpRequested = false;
+    bool list = false;
     QStringList files;
 
     QStringList args = qCmdLineArgs(argc, argv);
 
+    RCCBuilder builder;
+
     //parse options
     QString errorMsg;
     for (int i = 1; i < args.count() && errorMsg.isEmpty(); i++) {
         if (args[i].isEmpty())
             continue;
-        if (args[i][0] == '-') {   // option
+        if (args[i][0] == QLatin1Char('-')) {   // option
             QString opt = args[i];
             if (opt == QLatin1String("-o")) {
                 if (!(i < argc-1)) {
@@ -148,31 +101,34 @@ int main(int argc, char *argv[])
                     errorMsg = QLatin1String("Missing target name");
                     break;
                 }
-                initName = args[++i];
+                builder.initName = args[++i];
             } else if (opt == QLatin1String("-root")) {
                 if (!(i < argc-1)) {
                     errorMsg = QLatin1String("Missing root path");
                     break;
                 }
-                resourceRoot = QDir::cleanPath(args[++i]);
-                if(resourceRoot.isEmpty() || resourceRoot.at(0) != QLatin1Char('/'))
+                builder.resourceRoot = QDir::cleanPath(args[++i]);
+                if (builder.resourceRoot.isEmpty()
+                        || builder.resourceRoot.at(0) != QLatin1Char('/'))
                     errorMsg = QLatin1String("Root must start with a /");
             } else if (opt == QLatin1String("-compress")) {
                 if (!(i < argc-1)) {
                     errorMsg = QLatin1String("Missing compression level");
                     break;
                 }
-                compressLevel = args[++i].toInt();
+                builder.compressLevel = args[++i].toInt();
             } else if (opt == QLatin1String("-threshold")) {
                 if (!(i < argc-1)) {
                     errorMsg = QLatin1String("Missing compression threshold");
                     break;
                 }
-                compressThreshold = args[++i].toInt();
+                builder.compressThreshold = args[++i].toInt();
             } else if (opt == QLatin1String("-binary")) {
-                writeBinary = true;
+                builder.writeBinary = true;
+            } else if (opt == QLatin1String("-namespace")) {
+                builder.useNameSpace = !builder.useNameSpace;
             } else if (opt == QLatin1String("-verbose")) {
-                verbose = true;
+                builder.verbose = true;
             } else if (opt == QLatin1String("-list")) {
                 list = true;
             } else if (opt == QLatin1String("-version") || opt == QLatin1String("-v")) {
@@ -181,13 +137,14 @@ int main(int argc, char *argv[])
             } else if (opt == QLatin1String("-help") || opt == QLatin1String("-h")) {
                 helpRequested = true;
             } else if (opt == QLatin1String("-no-compress")) {
-                compressLevel = -2;
+                builder.compressLevel = -2;
             } else {
                 errorMsg = QString::fromLatin1("Unknown option: '%1'").arg(args[i]);
             }
         } else {
             if (!QFile::exists(args[i])) {
-                qWarning("%s: File does not exist '%s'", qPrintable(args[0]), qPrintable(args[i]));
+                qWarning("%s: File does not exist '%s'",
+                    qPrintable(args[0]), qPrintable(args[i]));
                 return 1;
             }
             files.append(args[i]);
@@ -196,5 +153,12 @@ int main(int argc, char *argv[])
 
     if (!files.size() || !errorMsg.isEmpty() || helpRequested)
         return showHelp(args[0], errorMsg);
-    return int(!processResourceFile(files, outFilename, list));
+    return int(!builder.processResourceFile(files, outFilename, list));
+}
+
+QT_END_NAMESPACE
+
+int main(int argc, char *argv[])
+{
+    return QT_PREPEND_NAMESPACE(runRcc)(argc, argv);
 }
