@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,9 +45,11 @@
 #include "uic.h"
 #include "ui4.h"
 
-#include <QRegExp>
-#include <QFileInfo>
-#include <QtDebug>
+#include <QtCore/QRegExp>
+#include <QtCore/QFileInfo>
+#include <QtCore/QDebug>
+
+QT_BEGIN_NAMESPACE
 
 Driver::Driver()
     : m_stdout(stdout, QFile::WriteOnly | QFile::Text)
@@ -64,16 +71,20 @@ QString Driver::findOrInsertWidget(DomWidget *ui_widget)
 
 QString Driver::findOrInsertSpacer(DomSpacer *ui_spacer)
 {
-    if (!m_spacers.contains(ui_spacer))
-        m_spacers.insert(ui_spacer, unique(QString(), QLatin1String("QSpacerItem")));
+    if (!m_spacers.contains(ui_spacer)) {
+        const QString name = ui_spacer->hasAttributeName() ? ui_spacer->attributeName() : QString();
+        m_spacers.insert(ui_spacer, unique(name, QLatin1String("QSpacerItem")));
+    }
 
     return m_spacers.value(ui_spacer);
 }
 
 QString Driver::findOrInsertLayout(DomLayout *ui_layout)
 {
-    if (!m_layouts.contains(ui_layout))
-        m_layouts.insert(ui_layout, unique(QString(), ui_layout->attributeClass()));
+    if (!m_layouts.contains(ui_layout)) {
+        const QString name = ui_layout->hasAttributeName() ? ui_layout->attributeName() : QString();
+        m_layouts.insert(ui_layout, unique(name, ui_layout->attributeClass()));
+    }
 
     return m_layouts.value(ui_layout);
 }
@@ -342,3 +353,5 @@ DomAction *Driver::actionByName(const QString &name) const
 {
     return m_actions.key(name);
 }
+
+QT_END_NAMESPACE
