@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -58,6 +63,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 enum { warnHeaderGeneration = 0 };
 
 #define CONVERT_PROPERTY(o, n) \
@@ -437,16 +444,17 @@ DomUI *Ui3Reader::generateUi4(const QDomElement &widget, bool implicitIncludes)
         ui->setElementPixmapFunction(pixmapFunction);
 
     for (int i=0; i<ui_customwidget_list.size(); ++i) {
-        QString name = ui_customwidget_list.at(i)->elementClass();
+        const QString name = ui_customwidget_list.at(i)->elementClass();
         if (candidateCustomWidgets.contains(name))
             candidateCustomWidgets.remove(name);
     }
 
+
     QMapIterator<QString, bool> it(candidateCustomWidgets);
     while (it.hasNext()) {
         it.next();
 
-        QString customClass = it.key();
+        const QString customClass = it.key();
         QString baseClass;
 
         if (customClass.endsWith(QLatin1String("ListView")))
@@ -1061,12 +1069,15 @@ void Ui3Reader::createProperties(const QDomElement &n, QList<DomProperty*> *prop
             CONVERT_PROPERTY(QLatin1String("textLabel"), QLatin1String("text"));
 
             CONVERT_PROPERTY(QLatin1String("toggleButton"), QLatin1String("checkable"));
-            CONVERT_PROPERTY(QLatin1String("isOn"), QLatin1String("checked"));
+            CONVERT_PROPERTY(QLatin1String("on"), QLatin1String("checked"));
 
             CONVERT_PROPERTY(QLatin1String("maxValue"), QLatin1String("maximum"));
             CONVERT_PROPERTY(QLatin1String("minValue"), QLatin1String("minimum"));
             CONVERT_PROPERTY(QLatin1String("lineStep"), QLatin1String("singleStep"));
 
+            // QSlider
+            CONVERT_PROPERTY(QLatin1String("tickmarks"), QLatin1String("tickPosition"));
+
             name = prop->attributeName(); // sync the name
 
             if (className == QLatin1String("QLabel") && name == QLatin1String("alignment")) {
@@ -1256,3 +1267,5 @@ QString Ui3Reader::fixMethod(const QString &method) const
     result.append(normalized.mid(index));
     return QLatin1String(result);
 }
+
+QT_END_NAMESPACE
