@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 #include <QDomElement>
 #include <QFile>
 
+QT_BEGIN_NAMESPACE
+
 void Ui3Reader::computeDeps(const QDomElement &e,
         QStringList &globalIncludes,
         QStringList &localIncludes, bool impl)
@@ -128,3 +130,5 @@ void Ui3Reader::computeDeps(const QDomElement &e,
         }
     }
 }
+
+QT_END_NAMESPACE
