@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,6 +59,8 @@
 #include <QRegExp>
 #include <QtDebug>
 
+QT_BEGIN_NAMESPACE
+
 QByteArray combinePath(const char *infile, const char *outfile)
 {
     QFileInfo inFileInfo(QDir::current(), QFile::decodeName(infile));
@@ -80,7 +102,7 @@ QByteArray combinePath(const char *infile, const char *outfile)
 
   \sa createFormImpl()
 */
-void Ui3Reader::createFormDecl(const QDomElement &e)
+void Ui3Reader::createFormDecl(const QDomElement &e, bool implicitIncludes)
 {
     QDomElement body = e;
 
@@ -107,7 +129,6 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     QMap<QString, int> customWidgets;
     QStringList forwardDecl;
     QStringList forwardDecl2;
-    QString exportMacro;
     for (n = e; !n.isNull(); n = n.nextSibling().toElement()) {
         if (n.tagName().toLower() == QLatin1String("customwidgets")) {
             QDomElement n2 = n.firstChild().toElement();
@@ -160,17 +181,17 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         }
     }
 
-    QStringList::Iterator it;
+    QStringList::ConstIterator it;
 
     globalIncludes = unique(globalIncludes);
-    for (it = globalIncludes.begin(); it != globalIncludes.end(); ++it) {
+    for (it = globalIncludes.constBegin(); it != globalIncludes.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             QString header = fixHeaderName(*it);
             out << "#include <" << header << ">" << endl;
         }
     }
     localIncludes = unique(localIncludes);
-    for (it = localIncludes.begin(); it != localIncludes.end(); ++it) {
+    for (it = localIncludes.constBegin(); it != localIncludes.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             QString header = fixHeaderName(*it);
             out << "#include \"" << header << "\"" << endl;
@@ -184,7 +205,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     if (dbForms[QLatin1String("(default)")].count())
         dbForm = true;
     bool subDbForms = false;
-    for (it = dbConnections.begin(); it != dbConnections.end(); ++it) {
+    for (it = dbConnections.constBegin(); it != dbConnections.constEnd(); ++it) {
         if (!(*it).isEmpty() && (*it) != QLatin1String("(default)")) {
             if (dbForms[(*it)].count()) {
                 subDbForms = true;
@@ -195,7 +216,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
 
     // some typedefs, maybe
     typeDefs = unique(typeDefs);
-    for (it = typeDefs.begin(); it != typeDefs.end(); ++it) {
+    for (it = typeDefs.constBegin(); it != typeDefs.constEnd(); ++it) {
         if (!(*it).isEmpty())
             out << "typedef " << *it << ";" << endl;
     }
@@ -204,20 +225,16 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     for (i = 0; i < (int) nl.length(); i++)
         forwardDecl2 << fixDeclaration(nl.item(i).toElement().firstChild().toText().data());
 
-    nl = e.parentNode().toElement().elementsByTagName(QLatin1String("exportmacro"));
-    if (nl.length() == 1)
-        exportMacro = nl.item(0).firstChild().toText().data();
-
     forwardDecl = unique(forwardDecl);
-    for (it = forwardDecl.begin(); it != forwardDecl.end(); ++it) {
+    for (it = forwardDecl.constBegin(); it != forwardDecl.constEnd(); ++it) {
         if (!(*it).isEmpty() && (*it) != objClass) {
             QString forwardName = *it;
             QStringList forwardNamespaces = forwardName.split(QLatin1String("::"));
             forwardName = forwardNamespaces.last();
             forwardNamespaces.removeAt(forwardNamespaces.size()-1);
 
-            QStringList::ConstIterator ns = forwardNamespaces.begin();
-            while (ns != forwardNamespaces.end()) {
+            QStringList::ConstIterator ns = forwardNamespaces.constBegin();
+            while (ns != forwardNamespaces.constEnd()) {
                 out << "namespace " << *ns << " {" << endl;
                 ++ns;
             }
@@ -227,7 +244,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         }
     }
 
-    for (it = forwardDecl2.begin(); it != forwardDecl2.end(); ++it) {
+    for (it = forwardDecl2.constBegin(); it != forwardDecl2.constEnd(); ++it) {
         QString fd = *it;
         fd = fd.trimmed();
         if (!fd.endsWith(QLatin1String(";")))
@@ -240,14 +257,54 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     Driver d;
     d.option().headerProtection = false;
     d.option().copyrightHeader = false;
+    d.option().extractImages = m_extractImages;
+    d.option().qrcOutputFile = m_qrcOutputFile;
+    d.option().implicitIncludes = implicitIncludes;
     if (trmacro.size())
         d.option().translateFunction = trmacro;
-    DomUI *ui = generateUi4(e);
+    DomUI *ui = generateUi4(e, implicitIncludes);
     d.uic(fileName, ui, &out);
     delete ui;
 
-    QStringList::ConstIterator ns = namespaces.begin();
-    while (ns != namespaces.end()) {
+    createWrapperDeclContents(e);
+
+    out << "#endif // " << protector << endl;
+}
+
+void Ui3Reader::createWrapperDecl(const QDomElement &e, const QString &convertedUiFile)
+{
+    QString objName = getObjectName(e);
+
+    objName = registerObject(objName);
+    QString protector = objName.toUpper() + QLatin1String("_H");
+    protector.replace(QLatin1String("::"), QLatin1String("_"));
+    out << "#ifndef " << protector << endl;
+    out << "#define " << protector << endl;
+    out << endl;
+    out << "#include \"" << convertedUiFile << "\"" << endl;
+
+    createWrapperDeclContents(e);
+    out << endl;
+    out << "#endif // " << protector << endl;
+}
+
+void Ui3Reader::createWrapperDeclContents(const QDomElement &e)
+{
+    QString objClass = getClassName(e);
+    if (objClass.isEmpty())
+        return;
+
+    QDomNodeList nl;
+    QString exportMacro;
+    int i;
+    QDomElement n;
+    QStringList::ConstIterator it;
+    nl = e.parentNode().toElement().elementsByTagName(QLatin1String("exportmacro"));
+    if (nl.length() == 1)
+        exportMacro = nl.item(0).firstChild().toText().data();
+
+    QStringList::ConstIterator ns = namespaces.constBegin();
+    while (ns != namespaces.constEnd()) {
         out << "namespace " << *ns << " {" << endl;
         ++ns;
     }
@@ -264,11 +321,11 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
 
     // constructor
     if (objClass == QLatin1String("QDialog") || objClass == QLatin1String("QWizard")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);" << endl;
     } else if (objClass == QLatin1String("QWidget")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = 0);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = 0);" << endl;
     } else if (objClass == QLatin1String("QMainWindow") || objClass == QLatin1String("Q3MainWindow")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::WType_TopLevel);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::WType_TopLevel);" << endl;
         isMainWindow = true;
     } else {
         out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0);" << endl;
@@ -281,7 +338,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     // database connections
     dbConnections = unique(dbConnections);
     bool hadOutput = false;
-    for (it = dbConnections.begin(); it != dbConnections.end(); ++it) {
+    for (it = dbConnections.constBegin(); it != dbConnections.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             // only need pointers to non-default connections
             if ((*it) != QLatin1String("(default)") && !(*it).isEmpty()) {
@@ -379,7 +436,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     }
 
     if (!publicVars.isEmpty()) {
-        for (it = publicVars.begin(); it != publicVars.end(); ++it)
+        for (it = publicVars.constBegin(); it != publicVars.constEnd(); ++it)
             out << indent << *it << endl;
         out << endl;
     }
@@ -411,14 +468,14 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     // create signals
     if (!extraSignals.isEmpty()) {
         out << "signals:" << endl;
-        for (it = extraSignals.begin(); it != extraSignals.end(); ++it)
+        for (it = extraSignals.constBegin(); it != extraSignals.constEnd(); ++it)
             out << "    void " << (*it) << ";" << endl;
         out << endl;
     }
 
     if (!protectedVars.isEmpty()) {
         out << "protected:" << endl;
-        for (it = protectedVars.begin(); it != protectedVars.end(); ++it)
+        for (it = protectedVars.constBegin(); it != protectedVars.constEnd(); ++it)
             out << indent << *it << endl;
         out << endl;
     }
@@ -443,7 +500,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     if (!privateFuncts.isEmpty() || !privateVars.isEmpty()) {
         out << "private:" << endl;
         if (!privateVars.isEmpty()) {
-            for (it = privateVars.begin(); it != privateVars.end(); ++it)
+            for (it = privateVars.constBegin(); it != privateVars.constEnd(); ++it)
                 out << indent << *it << endl;
             out << endl;
         }
@@ -461,7 +518,6 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         out << "}" << endl;
 
     out << endl;
-    out << "#endif // " << protector << endl;
 }
 
 void Ui3Reader::writeFunctionsDecl(const QStringList &fuLst, const QStringList &typLst, const QStringList &specLst)
@@ -650,14 +706,14 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         out << " *  The " << objClass.mid(1).toLower() << " will by default be modeless, unless you set 'modal' to" << endl;
         out << " *  true to construct a modal " << objClass.mid(1).toLower() << "." << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, modal, fl)";
     } else if (objClass == QLatin1String("QWidget"))  {
         out << "/*" << endl;
         out << " *  Constructs a " << nameOfClass << " as a child of 'parent', with the" << endl;
         out << " *  name 'name' and widget flags set to 'f'." << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, fl)";
     } else if (objClass == QLatin1String("QMainWindow") || objClass == QLatin1String("Q3MainWindow")) {
         out << "/*" << endl;
@@ -665,7 +721,7 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         out << " *  name 'name' and widget flags set to 'f'." << endl;
         out << " *" << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, fl)";
         isMainWindow = true;
     } else {
@@ -717,49 +773,6 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         }
     }
 
-    for (n = e; !n.isNull(); n = n.nextSibling().toElement()) {
-        if (n.tagName() == QLatin1String("connections")) {
-            // setup signals and slots connections
-            out << endl << indent << "// signals and slots connections" << endl;
-            nl = n.elementsByTagName(QLatin1String("connection"));
-            for (i = 0; i < (int) nl.length(); i++) {
-                QString sender, receiver, signal, slot;
-                for (QDomElement n2 = nl.item(i).firstChild().toElement(); !n2.isNull(); n2 = n2.nextSibling().toElement()) {
-                    if (n2.tagName() == QLatin1String("sender"))
-                        sender = n2.firstChild().toText().data();
-                    else if (n2.tagName() == QLatin1String("receiver"))
-                        receiver = n2.firstChild().toText().data();
-                    else if (n2.tagName() == QLatin1String("signal"))
-                        signal = fixDeclaration(n2.firstChild().toText().data());
-                    else if (n2.tagName() == QLatin1String("slot"))
-                        slot = fixDeclaration(n2.firstChild().toText().data());
-                }
-                if (sender.isEmpty() ||
-                     receiver.isEmpty() ||
-                     signal.isEmpty() ||
-                     slot.isEmpty())
-                    continue;
-                if (sender[0] == QLatin1Char('<') ||
-                     receiver[0] == QLatin1Char('<') ||
-                     signal[0] == QLatin1Char('<') ||
-                     slot[0] == QLatin1Char('<'))
-                    continue;
-
-                sender = registeredName(sender);
-                receiver = registeredName(receiver);
-
-                 // translate formwindow name to "this"
-                if (sender == objName)
-                    sender = QLatin1String("this");
-                if (receiver == objName)
-                    receiver = QLatin1String("this");
-
-                out << indent << "connect(" << sender << ", SIGNAL(" << signal << "), "
-                    << receiver << ", SLOT(" << slot << "));" << endl;
-            }
-        }
-    }
-
     if (extraFuncts.contains(QLatin1String("init()")))
         out << indent << "init();" << endl;
 
@@ -906,3 +919,5 @@ void Ui3Reader::createFormImpl(const QDomElement& e, const QString& form, const
         createFormImpl(n, form, connection, table);
     }
 }
+
+QT_END_NAMESPACE
