@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -58,7 +58,9 @@
 #include <qt_windows.h>
 #endif
 
-int main(int argc, char * argv[])
+QT_BEGIN_NAMESPACE
+
+int runUic3(int argc, char * argv[])
 {
     bool impl = false;
     bool wrap = false;
@@ -405,3 +407,10 @@ int main(int argc, char * argv[])
 
     return 0;
 }
+
+QT_END_NAMESPACE
+
+int main(int argc, char * argv[])
+{
+    return QT_PREPEND_NAMESPACE(runUic3(argc, argv));
+}
