@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,6 +56,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
 
 bool Ui3Reader::isMainWindow = false;
 
@@ -632,3 +633,5 @@ void Ui3Reader::setExtractImages(bool extract, const QString &qrcOutputFile)
     m_extractImages = extract;
     m_qrcOutputFile = qrcOutputFile;
 }
+
+QT_END_NAMESPACE
