@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -78,6 +78,8 @@
 #include <Q3IconView>
 #include <Q3ProgressBar>
 
+QT_BEGIN_NAMESPACE
+
 WidgetInfo::WidgetInfo()
 {
 }
@@ -139,7 +141,8 @@ const QMetaObject *WidgetInfo::metaObject(const QString &widgetName)
         return &QWidget::staticMetaObject;
     else if (widgetName == QLatin1String("QDialog"))
         return &QDialog::staticMetaObject;
-    else if (widgetName == QLatin1String("QWizard"))
+    else if (widgetName == QLatin1String("QWizard") || 
+             widgetName == QLatin1String("Q3Wizard"))
         return &Q3Wizard::staticMetaObject;
     else if (widgetName == QLatin1String("QLCDNumber"))
         return &QLCDNumber::staticMetaObject;
@@ -277,3 +280,5 @@ QString WidgetInfo::resolveEnumerator(const QMetaEnum &metaEnum, const QString &
 
     return QString();
 }
+
+QT_END_NAMESPACE
