@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -51,6 +51,8 @@
 #include <QtCore/QPointer>
 #include <QtCore/QHash>
 
+QT_BEGIN_NAMESPACE
+
 class QDesignerFormWindowInterface;
 class QAction;
 
@@ -88,4 +90,6 @@ private:
 
 }  // namespace qdesigner_internal
 
+QT_END_NAMESPACE
+
 #endif // BUDDYEDITOR_PLUGIN_H
