@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,10 +45,7 @@
 #define FORMWINDOW_H
 
 #include "formeditor_global.h"
-#include "formeditor.h"
-
-// sdk
-#include <QtDesigner/QtDesigner>
+#include <formwindowbase_p.h>
 
 // Qt
 #include <QtCore/QHash>
@@ -36,11 +53,13 @@
 #include <QtCore/QMap>
 #include <QtCore/QSet>
 
-#include <QtGui/QWidget>
-#include <QtGui/QPixmap>
+QT_BEGIN_NAMESPACE
 
+class QDesignerDnDItemInterface;
+class QDesignerTaskMenuExtension;
 class DomConnections;
 
+class QWidget;
 class QLabel;
 class QTimer;
 class QAction;
@@ -52,27 +71,20 @@ namespace qdesigner_internal {
 
 class FormEditor;
 class FormWindowCursor;
-class WidgetSelection;
 class WidgetEditorTool;
 class FormWindowWidgetStack;
 class FormWindowManager;
 class FormWindowDnDItem;
 class SetPropertyCommand;
 class BreakLayoutCommand;
-class Connection;
 
-class QT_FORMEDITOR_EXPORT FormWindow: public QDesignerFormWindowInterface
+
+class QT_FORMEDITOR_EXPORT FormWindow: public FormWindowBase
 {
     Q_OBJECT
-public:
-    enum HighlightMode
-    {
-        Restore,
-        Highlight
-    };
 
 public:
-    FormWindow(FormEditor *core, QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    explicit FormWindow(FormEditor *core, QWidget *parent = 0, Qt::WindowFlags flags = 0);
     virtual ~FormWindow();
 
     virtual QDesignerFormEditorInterface *core() const;
@@ -85,10 +97,6 @@ public:
     virtual QDesignerFormWindowToolInterface *tool(int index) const;
     virtual void registerTool(QDesignerFormWindowToolInterface *tool);
 
-    virtual bool hasFeature(Feature f) const;
-    virtual Feature features() const;
-    virtual void setFeatures(Feature f);
-
     virtual QString author() const;
     virtual void setAuthor(const QString &author);
 
@@ -119,10 +127,7 @@ public:
 
     virtual QDir absoluteDir() const;
 
-    virtual QPoint grid() const { return m_grid; }
-    virtual void setGrid(const QPoint &grid) { m_grid = grid; }
-
-    virtual void simplifySelection(QList<QWidget*> *sel) const;
+    virtual void simplifySelection(QWidgetList *sel) const;
 
     virtual void ensureUniqueObjectName(QObject *object);
 
@@ -131,9 +136,6 @@ public:
     bool isMainContainer(const QWidget *w) const;
 
     QWidget *currentWidget() const;
-    void setCurrentWidget(QWidget *currentWidget);
-
-    virtual QSize sizeHint() const;
 
     bool hasInsertedChildren(QWidget *w) const;
 
@@ -144,21 +146,18 @@ public:
 
     void selectWidgets();
     void repaintSelection();
-    void repaintSelection(QWidget *w);
     void updateSelection(QWidget *w);
     void updateChildSelections(QWidget *w);
     void raiseChildSelections(QWidget *w);
     void raiseSelection(QWidget *w);
-    void hideSelection(QWidget *w);
 
-    inline QList<QWidget *> widgets() const { return m_widgets; }
+    inline const QList<QWidget *>& widgets() const { return m_widgets; }
     inline int widgetCount() const { return m_widgets.count(); }
     inline QWidget *widgetAt(int index) const { return m_widgets.at(index); }
 
     QList<QWidget *> widgets(QWidget *widget) const;
 
     QWidget *createWidget(DomUI *ui, const QRect &rect, QWidget *target);
-    void deleteWidgets(const QList<QWidget*> &widget_list);
 
     bool isManaged(QWidget *w) const;
 
@@ -171,8 +170,8 @@ public:
     void beginCommand(const QString &description);
     void endCommand();
 
-    bool blockSelectionChanged(bool blocked);
-    void emitSelectionChanged();
+    virtual bool blockSelectionChanged(bool blocked);
+    virtual void emitSelectionChanged();
 
     bool unify(QObject *w, QString &s, bool changeIt);
 
@@ -196,10 +195,23 @@ public:
 
     void resizeWidget(QWidget *widget, const QRect &geometry);
 
-    void dropWidgets(QList<QDesignerDnDItemInterface*> &item_list, QWidget *target,
+    bool dropWidgets(const QList<QDesignerDnDItemInterface*> &item_list, QWidget *target,
                         const QPoint &global_mouse_pos);
 
-    QWidget *findContainer(QWidget *w, bool excludeLayout) const;
+    virtual QWidget *findContainer(QWidget *w, bool excludeLayout) const;
+    // for WidgetSelection only.
+    QWidget *designerWidget(QWidget *w) const;
+
+    //  Initialize and return a popup menu for a managed widget
+    QMenu *initializePopupMenu(QWidget *managedWidget);
+
+    virtual void paste(PasteMode pasteMode);
+    virtual QEditorFormBuilder *createFormBuilder();
+
+    bool eventFilter(QObject *watched, QEvent *event);
+
+    QDesignerTaskMenuExtension *widgetTaskMenu(QWidget *w) const;
+
 signals:
     void contextMenuRequested(QMenu *menu, QWidget *widget);
 
@@ -215,14 +227,15 @@ public slots:
     void layoutHorizontal();
     void layoutVertical();
     void layoutGrid();
+    void layoutFormLayout();
     void layoutHorizontalSplit();
     void layoutVerticalSplit();
     void layoutHorizontalContainer(QWidget *w);
     void layoutVerticalContainer(QWidget *w);
+    void layoutFormLayoutContainer(QWidget *w);
     void layoutGridContainer(QWidget *w);
     void breakLayout(QWidget *w);
 
-    void breakLayout();
     void editContents();
 
 protected:
@@ -241,7 +254,11 @@ private:
     void init();
     void initializeCoreTools();
 
-    QPoint gridPoint(const QPoint &p) const;
+    int getValue(const QRect &rect, int key, bool size) const;
+    int calcValue(int val, bool forward, bool snap, int snapOffset) const;
+    QRect applyValue(const QRect &rect, int val, int key, bool size) const;
+
+    bool frameNeeded(QWidget *w) const;
 
     enum RectType { Insert, Rubber };
 
@@ -264,7 +281,11 @@ private:
     bool handleKeyReleaseEvent(QWidget *widget, QWidget *managedWidget, QKeyEvent *e);
 
     bool isCentralWidget(QWidget *w) const;
-    QWidget *designerWidget(QWidget *w) const;
+
+    bool setCurrentWidget(QWidget *currentWidget);
+    bool trySelectWidget(QWidget *w, bool select);
+
+    void dragWidgetWithinForm(QWidget *widget, const QRect &targetGeometry, QWidget *targetContainer);
 
     BreakLayoutCommand *breakLayoutCommand(QWidget *w);
 
@@ -275,60 +296,49 @@ private:
 
     QWidget *findTargetContainer(QWidget *widget) const;
 
-    bool isPageOfContainerWidget(QWidget *widget) const;
+    void clearMainContainer();
 
-    static int widgetDepth(QWidget *w);
-    static bool isChildOf(QWidget *c, const QWidget *p);
+    static int widgetDepth(const QWidget *w);
+    static bool isChildOf(const QWidget *c, const QWidget *p);
 
     void editWidgets();
 
     void updateWidgets();
 
-    void handleArrowKeyEvent(int key, bool modifier);
+    void handleArrowKeyEvent(int key, Qt::KeyboardModifiers modifiers);
+
+    void layoutSelection(int type);
+    void layoutContainer(QWidget *w, int type);
 
 private:
-    Feature m_feature;
+    QWidget *containerForPaste() const;
+
     FormEditor *m_core;
     FormWindowCursor *m_cursor;
     QWidget *m_mainContainer;
     QWidget *m_currentWidget;
-    QPoint m_grid;
 
-    uint m_blockSelectionChanged: 1;
-    uint drawRubber: 1;
-    uint oldRectValid: 1;
-    uint hadOwnPalette: 1;
-    uint pad[28];
+    bool m_blockSelectionChanged;
+    bool m_drawRubber;
 
-    QPoint rectAnchor;
-    QRect currRect;
+    QPoint m_rectAnchor;
+    QRect m_currRect;
 
-    QList<QWidget*> m_widgets;
+    QWidgetList m_widgets;
     QSet<QWidget*> m_insertedWidgets;
 
-    QList<WidgetSelection *> selections;
-    QHash<QWidget *, WidgetSelection *> usedSelections;
+    class Selection;
+    Selection *m_selection;
 
-    QPoint startPos;
-    QPoint currentPos;
-
-    QRect widgetGeom;
-    QPoint oldPressPos;
-    QPoint origPressPos;
-    QWidget *startWidget;
-    QWidget *endWidget;
-
-    QWidget *targetContainer;
-    QPalette restorePalette;
+    QPoint m_startPos;
 
     QUndoStack *m_commandHistory;
 
     QString m_fileName;
 
-    QList<QWidget*> orderedWidgets;
-    QList<QWidget*> stackedWidgets;
-
-    QMap<QWidget*, QPair<QPalette ,bool> > palettesBeforeHighlight;
+    typedef QPair<QPalette ,bool> PaletteAndFill;
+    typedef QMap<QWidget*, PaletteAndFill> WidgetPaletteMap;
+    WidgetPaletteMap m_palettesBeforeHighlight;
 
     QRubberBand *m_rubberBand;
 
@@ -354,15 +364,15 @@ private:
 
     QList<SetPropertyCommand*> m_moveSelection;
     int m_lastUndoIndex;
+    bool m_dblClicked;
+    QPoint m_contextMenuPosition;
 
 private:
-//    friend class FormWindowManager;
-    friend class WidgetHandle;
-    friend class WidgetSelection;
-    friend class QDesignerWidget;
     friend class WidgetEditorTool;
 };
 
 }  // namespace qdesigner_internal
 
+QT_END_NAMESPACE
+
 #endif // FORMWINDOW_H
