@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@ TRANSLATOR qdesigner_internal::SignalSlotEditorTool
 #include <QtGui/QAction>
 #include <QtCore/qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 using namespace qdesigner_internal;
 
 SignalSlotEditorTool::SignalSlotEditorTool(QDesignerFormWindowInterface *formWindow, QObject *parent)
@@ -123,3 +125,5 @@ void SignalSlotEditorTool::loadFromDom(DomUI *ui, QWidget *mainContainer)
 {
     m_editor->fromUi(ui->elementConnections(), mainContainer);
 }
+
+QT_END_NAMESPACE
