@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,27 +46,39 @@ TRANSLATOR qdesigner_internal::TreeWidgetEditor
 */
 
 #include "treewidgeteditor.h"
-#include <findicondialog_p.h>
+#include <formwindowbase_p.h>
 #include <iconloader_p.h>
 #include <qdesigner_command_p.h>
+#include <qdesigner_utils_p.h>
+#include <abstractformbuilder.h>
 
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerIconCacheInterface>
 #include <QtCore/QDir>
 #include <QtCore/QQueue>
-#include <QHeaderView>
+#include <QtGui/QHeaderView>
+#include <QtGui/QTreeWidgetItemIterator>
 
-using namespace qdesigner_internal;
+QT_BEGIN_NAMESPACE
+
+namespace qdesigner_internal {
 
 TreeWidgetEditor::TreeWidgetEditor(QDesignerFormWindowInterface *form, QWidget *parent)
     : QDialog(parent), m_updating(false)
 {
     ui.setupUi(this);
-    m_form = form;
-    QIcon resetIcon = createIconSet(QString::fromUtf8("editdelete.png"));
-    ui.deletePixmapItemButton->setIcon(resetIcon);
-    ui.deletePixmapColumnButton->setIcon(resetIcon);
-    ui.deletePixmapItemButton->setEnabled(false);
-    ui.deletePixmapColumnButton->setEnabled(false);
+    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
+    m_form = qobject_cast<FormWindowBase *>(form);
+    ui.itemIconSelector->setFormEditor(form->core());
+    ui.itemIconSelector->setEnabled(false);
+    ui.itemIconSelector->setPixmapCache(m_form->pixmapCache());
+    ui.itemIconSelector->setIconCache(m_form->iconCache());
+
+    ui.columnIconSelector->setFormEditor(form->core());
+    ui.columnIconSelector->setEnabled(false);
+    ui.columnIconSelector->setPixmapCache(m_form->pixmapCache());
+    ui.columnIconSelector->setIconCache(m_form->iconCache());
 
     QIcon upIcon = createIconSet(QString::fromUtf8("up.png"));
     QIcon downIcon = createIconSet(QString::fromUtf8("down.png"));
@@ -66,6 +98,8 @@ TreeWidgetEditor::TreeWidgetEditor(QDesignerFormWindowInterface *form, QWidget *
     ui.moveItemLeftButton->setIcon(backIcon);
 
     ui.treeWidget->header()->setMovable(false);
+
+    connect(m_form->iconCache(), SIGNAL(reloaded()), this, SLOT(cacheReloaded()));
 }
 
 TreeWidgetEditor::~TreeWidgetEditor()
@@ -86,6 +120,7 @@ void TreeWidgetEditor::fillContentsFromTreeWidget(QTreeWidget *treeWidget)
         item->setFlags(item->flags() | Qt::ItemIsEditable);
         item->setText(headerItem->text(col));
         item->setIcon(headerItem->icon(col));
+        item->setData(QAbstractFormBuilder::resourceRole(), headerItem->data(col, QAbstractFormBuilder::resourceRole()));
     }
 
     if (colCount > 0) {
@@ -110,45 +145,14 @@ void TreeWidgetEditor::copyContents(QTreeWidget *sourceWidget, QTreeWidget *dest
 
     int colCount = sourceWidget->columnCount();
     destWidget->setColumnCount(colCount);
-    QTreeWidgetItem *origHeaderItem = sourceWidget->headerItem();
-    QTreeWidgetItem *headerItem = destWidget->headerItem();
-    for (int col = 0; col < colCount; col++) {
-        headerItem->setText(col, origHeaderItem->text(col));
-        headerItem->setIcon(col, origHeaderItem->icon(col));
-    }
+    destWidget->setHeaderItem(sourceWidget->headerItem()->clone());
 
-    if (!colCount)
-        return;
+    const int topLevelItemCount = sourceWidget->topLevelItemCount();
+    for (int i = 0; i < topLevelItemCount; i++)
+        destWidget->addTopLevelItem(sourceWidget->topLevelItem(i)->clone());
 
-    QTreeWidgetItem *lastItem = 0;
-    for (int i = 0; i < sourceWidget->topLevelItemCount(); i++) {
-        QTreeWidgetItem *origItem = sourceWidget->topLevelItem(i);
-        lastItem = new QTreeWidgetItem(destWidget);
-        lastItem->setFlags(lastItem->flags() | Qt::ItemIsEditable);
-        for (int j = 0; j < colCount; j++) {
-            lastItem->setText(j, origItem->text(j));
-            lastItem->setIcon(j, origItem->icon(j));
-        }
-        destWidget->setItemExpanded(lastItem, true);
-        copyContents(origItem, lastItem);
-    }
-}
-
-void TreeWidgetEditor::copyContents(QTreeWidgetItem *sourceItem, QTreeWidgetItem *destItem)
-{
-    int colCount = sourceItem->treeWidget()->columnCount();
-
-    for (int i = 0; i < sourceItem->childCount(); i++) {
-        QTreeWidgetItem *origItem = sourceItem->child(i);
-        QTreeWidgetItem *item = new QTreeWidgetItem(destItem);
-        item->setFlags(item->flags() | Qt::ItemIsEditable);
-        for (int j = 0; j < colCount; j++) {
-            item->setText(j, origItem->text(j));
-            item->setIcon(j, origItem->icon(j));
-        }
-        item->treeWidget()->setItemExpanded(item, true);
-        copyContents(origItem, item);
-    }
+    for (QTreeWidgetItemIterator it(destWidget); *it ; ++it)
+        (*it)->setFlags((*it)->flags() | Qt::ItemIsEditable);
 }
 
 void TreeWidgetEditor::on_newItemButton_clicked()
@@ -444,11 +448,7 @@ void TreeWidgetEditor::updateEditor()
         }
     }
     ui.itemsBox->setEnabled(itemsEnabled);
-    ui.textLabel->setEnabled(currentItemEnabled);
-    ui.pixmapLabel->setEnabled(currentItemEnabled);
-    ui.deletePixmapItemButton->setEnabled(currentItemEnabled);
-    ui.previewPixmapItemButton->setEnabled(currentItemEnabled);
-    ui.itemTextLineEdit->setEnabled(currentItemEnabled);
+    ui.itemIconSelector->setEnabled(currentItemEnabled);
     ui.newSubItemButton->setEnabled(currentItemEnabled);
     ui.deleteItemButton->setEnabled(currentItemEnabled);
 
@@ -457,51 +457,29 @@ void TreeWidgetEditor::updateEditor()
     ui.moveItemRightButton->setEnabled(moveItemRightEnabled);
     ui.moveItemLeftButton->setEnabled(moveItemLeftEnabled);
 
-    ui.renameColumnButton->setEnabled(currentColumnEnabled);
     ui.deleteColumnButton->setEnabled(currentColumnEnabled);
-    ui.pixmapColumnLabel->setEnabled(currentColumnEnabled);
-    ui.previewPixmapColumnButton->setEnabled(currentColumnEnabled);
+    ui.columnIconSelector->setEnabled(currentColumnEnabled);
     ui.moveColumnUpButton->setEnabled(moveColumnUpEnabled);
     ui.moveColumnDownButton->setEnabled(moveColumnDownEnabled);
 
     QString itemText, columnText;
-    QIcon itemIcon, columnIcon;
+    PropertySheetIconValue itemIcon, columnIcon;
 
     if (currentColumn) {
         int idx = ui.listWidget->currentRow();
-        columnIcon = ui.treeWidget->headerItem()->icon(idx);
+        columnIcon = qVariantValue<PropertySheetIconValue>(ui.treeWidget->headerItem()->data(idx, QAbstractFormBuilder::resourceRole()));
 
         if (current) {
             itemText = current->text(idx);
-            itemIcon = current->icon(idx);
+            itemIcon = qVariantValue<PropertySheetIconValue>(current->data(idx, QAbstractFormBuilder::resourceRole()));
         }
     }
 
-    ui.itemTextLineEdit->setText(itemText);
-    ui.previewPixmapItemButton->setIcon(itemIcon);
-    ui.deletePixmapItemButton->setEnabled(!itemIcon.isNull());
-    ui.previewPixmapColumnButton->setIcon(columnIcon);
-    ui.deletePixmapColumnButton->setEnabled(!columnIcon.isNull());
-
+    ui.itemIconSelector->setIcon(itemIcon);
+    ui.columnIconSelector->setIcon(columnIcon);
 }
 
-void TreeWidgetEditor::on_itemTextLineEdit_textEdited(const QString &text)
-{
-    if (m_updating)
-        return;
-    QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
-    if (!curItem)
-        return;
-
-    if (!ui.listWidget->currentItem())
-        return;
-
-    m_updating = true;
-    curItem->setText(ui.listWidget->currentRow(), text);
-    m_updating = false;
-}
-
-void TreeWidgetEditor::on_deletePixmapItemButton_clicked()
+void TreeWidgetEditor::on_itemIconSelector_iconChanged(const PropertySheetIconValue &icon)
 {
     QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
     if (!curItem)
@@ -510,43 +488,8 @@ void TreeWidgetEditor::on_deletePixmapItemButton_clicked()
     if (!ui.listWidget->currentItem())
         return;
 
-    curItem->setIcon(ui.listWidget->currentRow(), QIcon());
-    ui.previewPixmapItemButton->setIcon(QIcon());
-    ui.deletePixmapItemButton->setEnabled(false);
-}
-
-void TreeWidgetEditor::on_previewPixmapItemButton_clicked()
-{
-    QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
-    if (!curItem)
-        return;
-
-    if (!ui.listWidget->currentItem())
-        return;
-
-    int currentRow = ui.listWidget->currentRow();
-
-    FindIconDialog dialog(m_form, this);
-    QString file_path;
-    QString qrc_path;
-
-    QIcon icon = curItem->icon(currentRow);
-    if (!icon.isNull()) {
-        file_path = m_form->core()->iconCache()->iconToFilePath(icon);
-        qrc_path = m_form->core()->iconCache()->iconToQrcPath(icon);
-    }
-
-    dialog.setPaths(qrc_path, file_path);
-    if (dialog.exec()) {
-        file_path = dialog.filePath();
-        qrc_path = dialog.qrcPath();
-        if (!file_path.isEmpty()) {
-            icon = m_form->core()->iconCache()->nameToIcon(file_path, qrc_path);
-            curItem->setIcon(currentRow, icon);
-            ui.previewPixmapItemButton->setIcon(icon);
-            ui.deletePixmapItemButton->setEnabled(!icon.isNull());
-        }
-    }
+    curItem->setData(ui.listWidget->currentRow(), QAbstractFormBuilder::resourceRole(), qVariantFromValue(icon));
+    curItem->setIcon(ui.listWidget->currentRow(), m_form->iconCache()->icon(icon));
 }
 
 void TreeWidgetEditor::moveColumnsLeft(int fromColumn, int toColumn)
@@ -555,14 +498,17 @@ void TreeWidgetEditor::moveColumnsLeft(int fromColumn, int toColumn)
         return;
 
     QTreeWidgetItem *headerItem = ui.treeWidget->headerItem();
-    QString text = headerItem->text(toColumn);
-    QIcon icon = headerItem->icon(toColumn);
+    const QString text = headerItem->text(toColumn);
+    const QIcon icon = headerItem->icon(toColumn);
+    const QVariant data = headerItem->data(toColumn, QAbstractFormBuilder::resourceRole());
     for (int i = toColumn; i > fromColumn; i--) {
         headerItem->setText(i, headerItem->text(i - 1));
         headerItem->setIcon(i, headerItem->icon(i - 1));
+        headerItem->setData(i, QAbstractFormBuilder::resourceRole(), headerItem->data(i - 1, QAbstractFormBuilder::resourceRole()));
     }
     headerItem->setText(fromColumn, text);
     headerItem->setIcon(fromColumn, icon);
+    headerItem->setData(fromColumn, QAbstractFormBuilder::resourceRole(), data);
 
     QQueue<QTreeWidgetItem *> pendingQueue;
     for (int i = 0; i < ui.treeWidget->topLevelItemCount(); i++)
@@ -573,14 +519,17 @@ void TreeWidgetEditor::moveColumnsLeft(int fromColumn, int toColumn)
         for (int i = 0; i < item->childCount(); i++)
             pendingQueue.enqueue(item->child(i));
 
-        QString text = item->text(toColumn);
-        QIcon icon = item->icon(toColumn);
+        const QString text = item->text(toColumn);
+        const QIcon icon = item->icon(toColumn);
+        const QVariant data = item->data(toColumn, QAbstractFormBuilder::resourceRole());
         for (int i = toColumn; i > fromColumn; i--) {
             item->setText(i, item->text(i - 1));
             item->setIcon(i, item->icon(i - 1));
+            item->setData(i, QAbstractFormBuilder::resourceRole(), headerItem->data(i - 1, QAbstractFormBuilder::resourceRole()));
         }
         item->setText(fromColumn, text);
         item->setIcon(fromColumn, icon);
+        item->setData(fromColumn, QAbstractFormBuilder::resourceRole(), data);
     }
 }
 
@@ -590,14 +539,17 @@ void TreeWidgetEditor::moveColumnsRight(int fromColumn, int toColumn)
         return;
 
     QTreeWidgetItem *headerItem = ui.treeWidget->headerItem();
-    QString text = headerItem->text(fromColumn);
-    QIcon icon = headerItem->icon(fromColumn);
+    const QString text = headerItem->text(fromColumn);
+    const QIcon icon = headerItem->icon(fromColumn);
+    const QVariant data = headerItem->data(fromColumn, QAbstractFormBuilder::resourceRole());
     for (int i = fromColumn; i < toColumn; i++) {
         headerItem->setText(i, headerItem->text(i + 1));
         headerItem->setIcon(i, headerItem->icon(i + 1));
+        headerItem->setData(i, QAbstractFormBuilder::resourceRole(), headerItem->data(i + 1, QAbstractFormBuilder::resourceRole()));
     }
     headerItem->setText(toColumn, text);
     headerItem->setIcon(toColumn, icon);
+    headerItem->setData(toColumn, QAbstractFormBuilder::resourceRole(), data);
 
     QQueue<QTreeWidgetItem *> pendingQueue;
     for (int i = 0; i < ui.treeWidget->topLevelItemCount(); i++)
@@ -608,14 +560,17 @@ void TreeWidgetEditor::moveColumnsRight(int fromColumn, int toColumn)
         for (int i = 0; i < item->childCount(); i++)
             pendingQueue.enqueue(item->child(i));
 
-        QString text = item->text(fromColumn);
-        QIcon icon = item->icon(fromColumn);
+        const QString text = item->text(fromColumn);
+        const QIcon icon = item->icon(fromColumn);
+        const QVariant data = item->data(fromColumn, QAbstractFormBuilder::resourceRole());
         for (int i = fromColumn; i < toColumn; i++) {
             item->setText(i, item->text(i + 1));
             item->setIcon(i, item->icon(i + 1));
+            item->setData(i, QAbstractFormBuilder::resourceRole(), item->data(i + 1, QAbstractFormBuilder::resourceRole()));
         }
         item->setText(toColumn, text);
         item->setIcon(toColumn, icon);
+        item->setData(toColumn, QAbstractFormBuilder::resourceRole(), icon);
     }
 }
 
@@ -649,15 +604,6 @@ void TreeWidgetEditor::on_newColumnButton_clicked()
     ui.listWidget->editItem(item);
 }
 
-void TreeWidgetEditor::on_renameColumnButton_clicked()
-{
-    QListWidgetItem *currentColumn = ui.listWidget->currentItem();
-    if (!currentColumn)
-        return;
-
-    ui.listWidget->editItem(currentColumn);
-}
-
 void TreeWidgetEditor::on_deleteColumnButton_clicked()
 {
     QListWidgetItem *currentColumn = ui.listWidget->currentItem();
@@ -732,59 +678,37 @@ void TreeWidgetEditor::on_moveColumnDownButton_clicked()
     updateEditor();
 }
 
-void TreeWidgetEditor::on_previewPixmapColumnButton_clicked()
-{
-    QListWidgetItem *currentColumn = ui.listWidget->currentItem();
-    if (!currentColumn)
-        return;
-
-    int currentRow = ui.listWidget->currentRow();
-
-    FindIconDialog dialog(m_form, this);
-    QString file_path;
-    QString qrc_path;
-
-    QIcon icon = currentColumn->icon();
-    if (icon.isNull()) {
-        file_path = m_form->absoluteDir().absolutePath();
-    } else {
-        file_path = m_form->core()->iconCache()->iconToFilePath(icon);
-        qrc_path = m_form->core()->iconCache()->iconToQrcPath(icon);
-    }
-
-    dialog.setPaths(qrc_path, file_path);
-    if (dialog.exec()) {
-        file_path = dialog.filePath();
-        qrc_path = dialog.qrcPath();
-        if (!file_path.isEmpty()) {
-            icon = m_form->core()->iconCache()->nameToIcon(file_path, qrc_path);
-            currentColumn->setIcon(icon);
-            ui.treeWidget->headerItem()->setIcon(currentRow, icon);
-            ui.previewPixmapColumnButton->setIcon(icon);
-            ui.deletePixmapColumnButton->setEnabled(!icon.isNull());
-        }
-    }
-
-}
-
-void TreeWidgetEditor::on_deletePixmapColumnButton_clicked()
+void TreeWidgetEditor::on_columnIconSelector_iconChanged(const PropertySheetIconValue &icon)
 {
     QListWidgetItem *curItem = ui.listWidget->currentItem();
     if (!curItem)
         return;
 
-    curItem->setIcon(QIcon());
-    ui.treeWidget->headerItem()->setIcon(ui.listWidget->currentRow(), QIcon());
-    ui.previewPixmapColumnButton->setIcon(QIcon());
-    ui.deletePixmapColumnButton->setEnabled(false);
+    m_updating = true;
+    const QVariant data = qVariantFromValue(icon);
+    const QIcon itemIcon = m_form->iconCache()->icon(icon);
+    curItem->setData(QAbstractFormBuilder::resourceRole(), data);
+    curItem->setIcon(itemIcon);
+    ui.treeWidget->headerItem()->setData(ui.listWidget->currentRow(), QAbstractFormBuilder::resourceRole(), data);
+    ui.treeWidget->headerItem()->setIcon(ui.listWidget->currentRow(), itemIcon);
+    m_updating = false;
+    updateEditor();
 }
 
-
 void TreeWidgetEditor::closeEditors()
 {
     if (QTreeWidgetItem *cur = ui.treeWidget->currentItem() ) {
         const int numCols = cur->columnCount ();
-        for (int i = 0; i < numCols; i++) 
+        for (int i = 0; i < numCols; i++)
             ui.treeWidget->closePersistentEditor (cur, i);
     }
 }
+
+void TreeWidgetEditor::cacheReloaded()
+{
+    reloadIconResources(m_form->iconCache(), ui.treeWidget);
+    reloadIconResources(m_form->iconCache(), ui.listWidget);
+}
+
+}
+QT_END_NAMESPACE
