@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -59,6 +59,8 @@
 #include "oublietteplan.h"
 #include <stdlib.h>
 
+QT_BEGIN_NAMESPACE
+
 double FOV::slope(double x1, double y1, double x2, double y2)
 {
 	double xDiff=x1-x2;
@@ -1128,3 +1130,5 @@ void SIMPLEFOV::applyCell(OublietteLevel *map, int x, int y)
 {
 	map->updateTileFlags(QPoint(x, y), Tile::Explored);
 }
+
+QT_END_NAMESPACE
