@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 #include <oublietteplan.h>
 #include "cursor.h"
 
+QT_BEGIN_NAMESPACE
+
 class QPaintEvent;
 class QKeyEvent;
 class QListWidgetItem;
@@ -95,4 +97,6 @@ private:
     int m_timerID;
 };
 
+QT_END_NAMESPACE
+
 #endif
