@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 #include "oubliette.h"
 #include "oublietteview.h"
 
+QT_BEGIN_NAMESPACE
+
 OublietteView::OublietteView()
 {
     m_oubliette = new Oubliette;
@@ -67,3 +69,5 @@ void OublietteView::scrollToCharacter(const QPoint &pt)
     horizontalScrollBar()->setValue(pt.x() - width() / 2);
     verticalScrollBar()->setValue(pt.y() - height() / 2);
 }
+
+QT_END_NAMESPACE
