@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -53,6 +53,8 @@
 
 #include <qevent.h>
 
+QT_BEGIN_NAMESPACE
+
 // ### review
 
 QDesignerServer::QDesignerServer(QObject *parent)
@@ -154,3 +156,5 @@ void QDesignerClient::readFromSocket()
         }
     }
 }
+
+QT_END_NAMESPACE
