@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,22 +44,38 @@
 #include "qdesigner.h"
 #include "preferences.h"
 #include "qdesigner_settings.h"
-#include "qdesigner_widgetbox.h"
 #include "qdesigner_workbench.h"
 #include "qdesigner_propertyeditor.h"
 #include "qdesigner_objectinspector.h"
 
 #include <qdesigner_utils_p.h>
+#include <previewconfigurationwidget_p.h>
+#include <previewmanager_p.h>
 
 #include <QtCore/QVariant>
 #include <QtCore/QDir>
 
 #include <QtGui/QDesktopWidget>
 #include <QtGui/QStyle>
+#include <QtGui/QListView>
 
 #include <QtCore/qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 static const char *designerPath = "/.designer";
+static const char *newFormShowKey = "newFormDialog/ShowOnStartup";
+static const char *mainWindowStateKey = "MainWindowState";
+static const char *toolBoxStateKey = "ToolBoxState";
+static const char *toolBarsStateKey = "ToolBarsState";
+static const char *backupOrgListKey = "backup/fileListOrg";
+static const char *backupBakListKey = "backup/fileListBak";
+static const char *previewKey = "Preview";
+static const char *defaultGridKey = "defaultGrid";
+static const char *formTemplatePathsKey = "FormTemplatePaths";
+static const char *recentFilesListKey = "recentFilesList";
+static const char *actionEditorViewModeKey = "ActionEditorViewMode";
+static const char *formTemplateKey = "FormTemplate";
 
 static bool checkTemplatePath(const QString &path, bool create)
 {
@@ -105,20 +121,22 @@ const QStringList &QDesignerSettings::defaultFormTemplatePaths()
 
 QStringList QDesignerSettings::formTemplatePaths() const
 {
-    return value(QLatin1String("FormTemplatePaths"),defaultFormTemplatePaths()).toStringList();
+    return value(QLatin1String(formTemplatePathsKey),defaultFormTemplatePaths()).toStringList();
 }
 
 void QDesignerSettings::setFormTemplatePaths(const QStringList &paths)
 {
-    setValue(QLatin1String("FormTemplatePaths"), paths);
+    setValue(QLatin1String(formTemplatePathsKey), paths);
 }
 
-QString QDesignerSettings::defaultUserWidgetBoxXml() const
+QString  QDesignerSettings::formTemplate() const
 {
-    QString rc = QDir::homePath();
-    rc += QLatin1String(designerPath);
-    rc += QLatin1String("/widgetbox.xml");
-    return rc;
+    return value(QLatin1String(formTemplateKey)).toString();
+}
+
+void QDesignerSettings::setFormTemplate(const QString &t)
+{
+    setValue(QLatin1String(formTemplateKey), t);
 }
 
 void QDesignerSettings::saveGeometryFor(const QWidget *w)
@@ -138,7 +156,7 @@ void QDesignerSettings::saveGeometryHelper(const QWidget *w, const QString &key)
 {
     beginGroup(key);
     setValue(QLatin1String("visible"), w->isVisible());
-    setValue(QLatin1String("geometry"), w->saveGeometry());
+QDesignerSettings::    setValue(QLatin1String("geometry"), w->saveGeometry());
     endGroup();
 }
 
@@ -160,48 +178,58 @@ void QDesignerSettings::setGeometryHelper(QWidget *w, const QString &key,
 
 QStringList QDesignerSettings::recentFilesList() const
 {
-    return value(QLatin1String("recentFilesList")).toStringList();
+    return value(QLatin1String(recentFilesListKey)).toStringList();
 }
 
 void QDesignerSettings::setRecentFilesList(const QStringList &sl)
 {
-    setValue(QLatin1String("recentFilesList"), sl);
+    setValue(QLatin1String(recentFilesListKey), sl);
 }
 
 void QDesignerSettings::setShowNewFormOnStartup(bool showIt)
 {
-    setValue(QLatin1String("newFormDialog/ShowOnStartup"), showIt);
+    setValue(QLatin1String(newFormShowKey), showIt);
 }
 
 bool QDesignerSettings::showNewFormOnStartup() const
 {
-    return value(QLatin1String("newFormDialog/ShowOnStartup"), true).toBool();
+    return value(QLatin1String(newFormShowKey), true).toBool();
 }
 
 QByteArray QDesignerSettings::mainWindowState() const
 {
-    return value(QLatin1String("MainWindowState")).toByteArray();
+    return value(QLatin1String(mainWindowStateKey)).toByteArray();
 }
 
 void QDesignerSettings::setMainWindowState(const QByteArray &mainWindowState)
 {
-    setValue(QLatin1String("MainWindowState"), mainWindowState);
+    setValue(QLatin1String(mainWindowStateKey), mainWindowState);
 }
 
 QByteArray QDesignerSettings::toolBoxState() const
 {
-    return value(QLatin1String("ToolBoxState")).toByteArray();
+    return value(QLatin1String(toolBoxStateKey)).toByteArray();
 }
 
 void QDesignerSettings::setToolBoxState(const QByteArray &state)
 {
-    setValue(QLatin1String("ToolBoxState"), state);
+    setValue(QLatin1String(toolBoxStateKey), state);
+}
+
+QByteArray QDesignerSettings::toolBarsState() const
+{
+    return value(QLatin1String(toolBarsStateKey)).toByteArray();
+}
+
+void QDesignerSettings::setToolBarsState(const QByteArray &toolBarsState)
+{
+    setValue(QLatin1String(toolBarsStateKey), toolBarsState);
 }
 
 void QDesignerSettings::clearBackup()
 {
-    remove(QLatin1String("backup/fileListOrg"));
-    remove(QLatin1String("backup/fileListBak"));
+    remove(QLatin1String(backupOrgListKey));
+    remove(QLatin1String(backupBakListKey));
 }
 
 void QDesignerSettings::setBackup(const QMap<QString, QString> &map)
@@ -209,22 +237,47 @@ void QDesignerSettings::setBackup(const QMap<QString, QString> &map)
     const QStringList org = map.keys();
     const QStringList bak = map.values();
 
-    setValue(QLatin1String("backup/fileListOrg"), org);
-    setValue(QLatin1String("backup/fileListBak"), bak);
+    setValue(QLatin1String(backupOrgListKey), org);
+    setValue(QLatin1String(backupBakListKey), bak);
 }
 
 QMap<QString, QString> QDesignerSettings::backup() const
 {
-    const QStringList org = value(QLatin1String("backup/fileListOrg"), QStringList()).toStringList();
-    const QStringList bak = value(QLatin1String("backup/fileListBak"), QStringList()).toStringList();
+    const QStringList org = value(QLatin1String(backupOrgListKey), QStringList()).toStringList();
+    const QStringList bak = value(QLatin1String(backupBakListKey), QStringList()).toStringList();
 
     QMap<QString, QString> map;
-    for (int i = 0; i < org.count(); ++i)
+    const int orgCount = org.count();
+    for (int i = 0; i < orgCount; ++i)
         map.insert(org.at(i), bak.at(i));
 
     return map;
 }
 
+void QDesignerSettings::setPreviewConfiguration(const qdesigner_internal::PreviewConfiguration &pc)
+{
+    pc.toSettings(QLatin1String(previewKey), *this);
+}
+
+qdesigner_internal::PreviewConfiguration QDesignerSettings::previewConfiguration() const
+{
+    qdesigner_internal::PreviewConfiguration rc;
+    rc.fromSettings(QLatin1String(previewKey), *this);
+    return rc;
+}
+
+qdesigner_internal::PreviewConfigurationWidgetState QDesignerSettings::previewConfigurationWidgetState() const
+{
+    qdesigner_internal::PreviewConfigurationWidgetState rc;
+    rc.fromSettings(QLatin1String(previewKey), *this);
+    return rc;
+}
+
+void QDesignerSettings::setPreviewConfigurationWidgetState(const qdesigner_internal::PreviewConfigurationWidgetState &pc)
+{
+    pc.toSettings(QLatin1String(previewKey), *this);
+}
+
 void QDesignerSettings::setPreferences(const Preferences& p)
 {
     beginGroup(QLatin1String("UI"));
@@ -234,7 +287,9 @@ void QDesignerSettings::setPreferences(const Preferences& p)
     setValue(QLatin1String("writingSystem"), p.m_writingSystem);
     endGroup();
     // grid
-    setValue(QLatin1String("defaultGrid"), p.m_defaultGrid.toVariantMap());
+    setValue(QLatin1String(defaultGridKey), p.m_defaultGrid.toVariantMap());
+    setPreviewConfigurationWidgetState(p.m_previewConfigurationWidgetState);
+    setPreviewConfiguration(p.m_previewConfiguration);
     // merge template paths
     QStringList templatePaths = defaultFormTemplatePaths();
     templatePaths += p.m_additionalTemplatePaths;
@@ -253,10 +308,12 @@ Preferences QDesignerSettings::preferences() const
     rc.m_writingSystem = static_cast<QFontDatabase::WritingSystem>(value(QLatin1String("UI/writingSystem"), QFontDatabase::Any).toInt());
     rc.m_font = qVariantValue<QFont>(value(QLatin1String("UI/font")));
     rc.m_useFont = value(QLatin1String("UI/useFont"), QVariant(false)).toBool();
-    const QVariantMap defaultGridMap = value(QLatin1String("defaultGrid"), QVariantMap()).toMap();
+    const QVariantMap defaultGridMap = value(QLatin1String(defaultGridKey), QVariantMap()).toMap();
     if (!defaultGridMap.empty())
         rc.m_defaultGrid.fromVariantMap(defaultGridMap);
     rc.m_additionalTemplatePaths = additionalFormTemplatePaths();
+    rc.m_previewConfigurationWidgetState = previewConfigurationWidgetState();
+    rc.m_previewConfiguration = previewConfiguration();
     return rc;
 }
 
@@ -271,3 +328,15 @@ QStringList QDesignerSettings::additionalFormTemplatePaths() const
     }
     return rc;
 }
+
+int QDesignerSettings::actionEditorViewMode() const
+{
+    return value(QLatin1String(actionEditorViewModeKey), 0).toInt();
+}
+
+void QDesignerSettings::setActionEditorViewMode(int vm)
+{
+    setValue(QLatin1String(actionEditorViewModeKey), vm);
+}
+
+QT_END_NAMESPACE
