@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -52,6 +52,8 @@
 #include <QtCore/QList>
 #include <QtCore/QRect>
 
+QT_BEGIN_NAMESPACE
+
 class QDesignerActions;
 class QDesignerToolWindow;
 class QDesignerFormWindow;
@@ -62,11 +64,13 @@ class QActionGroup;
 class QDockWidget;
 class QMenu;
 class QMenuBar;
+class QMainWindow;
 class QToolBar;
 class QMdiArea;
 class QMdiSubWindow;
 class QCloseEvent;
 class QFont;
+class QtToolBarManager;
 
 class QDesignerFormEditorInterface;
 class QDesignerFormWindowInterface;
@@ -91,8 +95,6 @@ public:
     QDesignerToolWindow *findToolWindow(QWidget *widget) const;
     QDesignerFormWindow *findFormWindow(QWidget *widget) const;
 
-    QDesignerFormWindow *createFormWindow();
-
     QDesignerFormWindow *openForm(const QString &fileName, QString *errorMessage);
     QDesignerFormWindow *openTemplate(const QString &templateFileName,
                                       const QString &editorFileName,
@@ -134,6 +136,7 @@ public slots:
     void setUIMode(UIMode mode);
     void bringAllToFront();
     void toggleFormMinimizationState();
+    void configureToolBars();
 
 // ### private slots:
     void switchToNeutralMode();
@@ -168,6 +171,10 @@ private:
     void setFormWindowMinimized(QDesignerFormWindow *fw, bool minimized);
     void setDesignerUIFont(const QFont &);
 
+    void createToolBarManager(QMainWindow *mw);
+    void removeToolBarManager();
+    void updateToolBarMenu();
+
     QDesignerFormEditorInterface *m_core;
     qdesigner_internal::QDesignerIntegration *m_integration;
 
@@ -188,12 +195,16 @@ private:
     QToolBar *m_editToolBar;
     QToolBar *m_fileToolBar;
 
+    QtToolBarManager *m_toolBarManager;
+
     UIMode m_mode;
 
     QList<QDesignerToolWindow*> m_toolWindows;
     QList<QDesignerFormWindow*> m_formWindows;
 
     QMdiArea *m_mdiArea;
+    QMenu *m_toolbarMenu;
+    QAction *m_configureToolBars;
 
     // Helper class to remember the position of a window while switching user interface modes.
     class Position {
@@ -222,4 +233,6 @@ private:
     State m_state;
 };
 
+QT_END_NAMESPACE
+
 #endif // QDESIGNER_WORKBENCH_H
