@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,13 +46,15 @@
 
 #include <QtDesigner/sdk_global.h>
 
+QT_BEGIN_HEADER
+
+QT_BEGIN_NAMESPACE
+
 class QDesignerFormWindowInterface;
 class QWidget;
 class QVariant;
 class QString;
 
-QT_BEGIN_HEADER
-
 class QDESIGNER_SDK_EXPORT QDesignerFormWindowCursorInterface
 {
 public:
@@ -102,6 +104,8 @@ public:
     bool isWidgetSelected(QWidget *widget) const;
 };
 
+QT_END_NAMESPACE
+
 QT_END_HEADER
 
 #endif // ABSTRACTFORMWINDOWCURSOR_H
