@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -43,6 +43,8 @@
 
 #include "abstractformwindowtool.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QDesignerFormWindowToolInterface
 
@@ -102,3 +104,5 @@ QDesignerFormWindowToolInterface::~QDesignerFormWindowToolInterface()
 /*!
     \fn virtual bool QDesignerFormWindowToolInterface::handleEvent(QWidget *widget, QWidget *managedWidget, QEvent *event) = 0
 */
+
+QT_END_NAMESPACE
