@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +45,12 @@
 #include <QtCore/qdebug.h>
 #include <qalgorithms.h>
 
+QT_BEGIN_NAMESPACE
+
+namespace {
+    enum { debugWidgetDataBase =  0 };
+}
+
 /*!
     \class QDesignerWidgetDataBaseInterface
     \brief The QDesignerWidgetDataBaseInterface class provides an interface that is used to
@@ -75,6 +101,9 @@ int QDesignerWidgetDataBaseInterface::indexOf(QDesignerWidgetDataBaseItemInterfa
 */
 void QDesignerWidgetDataBaseInterface::insert(int index, QDesignerWidgetDataBaseItemInterface *item)
 {
+    if (debugWidgetDataBase) 
+        qDebug() << "insert at " << index << ' ' << item->name() << " derived from " << item->extends(); 
+ 
     m_items.insert(index, item);
 }
 
@@ -82,6 +111,8 @@ void QDesignerWidgetDataBaseInterface::insert(int index, QDesignerWidgetDataBase
 */
 void QDesignerWidgetDataBaseInterface::append(QDesignerWidgetDataBaseItemInterface *item)
 {
+    if (debugWidgetDataBase) 
+        qDebug() << "append " << item->name() << " derived from " << item->extends();
     m_items.append(item);
 }
 
@@ -96,8 +127,9 @@ QDesignerFormEditorInterface *QDesignerWidgetDataBaseInterface::core() const
 */
 int QDesignerWidgetDataBaseInterface::indexOfClassName(const QString &name, bool) const
 {
-    for (int i=0; i<count(); ++i) {
-        QDesignerWidgetDataBaseItemInterface *entry = item(i);
+    const int itemCount = count();
+    for (int i=0; i<itemCount; ++i) {
+        const QDesignerWidgetDataBaseItemInterface *entry = item(i);
         if (entry->name() == name)
             return i;
     }
@@ -112,7 +144,7 @@ int QDesignerWidgetDataBaseInterface::indexOfObject(QObject *object, bool) const
     if (!object)
         return -1;
 
-    QString className = QString::fromUtf8(object->metaObject()->className());
+    const QString className = QString::fromUtf8(object->metaObject()->className());
     return indexOfClassName(className);
 }
 
@@ -120,7 +152,7 @@ int QDesignerWidgetDataBaseInterface::indexOfObject(QObject *object, bool) const
 */
 bool QDesignerWidgetDataBaseInterface::isContainer(QObject *object, bool resolveName) const
 {
-    if (QDesignerWidgetDataBaseItemInterface *i = item(indexOfObject(object, resolveName)))
+    if (const QDesignerWidgetDataBaseItemInterface *i = item(indexOfObject(object, resolveName)))
         return i->isContainer();
     return false;
 }
@@ -129,7 +161,7 @@ bool QDesignerWidgetDataBaseInterface::isContainer(QObject *object, bool resolve
 */
 bool QDesignerWidgetDataBaseInterface::isCustom(QObject *object, bool resolveName) const
 {
-    if (QDesignerWidgetDataBaseItemInterface *i = item(indexOfObject(object, resolveName)))
+    if (const QDesignerWidgetDataBaseItemInterface *i = item(indexOfObject(object, resolveName)))
         return i->isCustom();
     return false;
 }
@@ -326,3 +358,5 @@ bool QDesignerWidgetDataBaseInterface::isCustom(QObject *object, bool resolveNam
 
     Returns a list of default values to be used as properties for the item.
 */
+
+QT_END_NAMESPACE
