@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 #include "abstractformeditor.h"
 #include "abstractwidgetdatabase.h"
 
+QT_BEGIN_NAMESPACE
+
 /*!
     \class QDesignerWidgetFactoryInterface
     \brief The QDesignerWidgetFactoryInterface class provides an interface that is used to control
@@ -108,3 +110,5 @@ QDesignerWidgetFactoryInterface::~QDesignerWidgetFactoryInterface()
 /*!
     \fn virtual void QDesignerWidgetFactoryInterface::initialize(QObject *object) const = 0
 */
+
+QT_END_NAMESPACE
