@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 
 QT_BEGIN_HEADER
 
+QT_BEGIN_NAMESPACE
+
 class QVariant;
 
 class QDesignerPropertySheetExtension
@@ -79,7 +81,11 @@ public:
     virtual void setChanged(int index, bool changed) = 0;
 
 };
-Q_DECLARE_EXTENSION_INTERFACE(QDesignerPropertySheetExtension, "com.trolltech.Qt.Designer.PropertySheet")
+
+Q_DECLARE_EXTENSION_INTERFACE(QDesignerPropertySheetExtension,
+    "com.trolltech.Qt.Designer.PropertySheet")
+
+QT_END_NAMESPACE
 
 QT_END_HEADER
 
