@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -52,39 +52,45 @@ TRANSLATOR qdesigner_internal::ActionEditor
 #include "qdesigner_menu_p.h"
 #include "qdesigner_command_p.h"
 #include "qdesigner_propertycommand_p.h"
-#include "resourcemimedata_p.h"
 #include "qdesigner_objectinspector_p.h"
 #include "qdesigner_utils_p.h"
+#include "qsimpleresource_p.h"
+#include "formwindowbase_p.h"
 
 #include <QtDesigner/QDesignerFormEditorInterface>
 #include <QtDesigner/QDesignerPropertyEditorInterface>
 #include <QtDesigner/QDesignerPropertySheetExtension>
 #include <QtDesigner/QExtensionManager>
 #include <QtDesigner/QDesignerMetaDataBaseInterface>
-#include <QtDesigner/QDesignerFormWindowInterface>
 #include <QtDesigner/QDesignerIconCacheInterface>
 
 #include <QtGui/QMenu>
-#include <QtGui/QMessageBox>
-#include <QtGui/QListWidget>
 #include <QtGui/QToolBar>
 #include <QtGui/QSplitter>
 #include <QtGui/QAction>
+#include <QtGui/QApplication>
+#include <QtGui/QClipboard>
 #include <QtGui/QItemDelegate>
 #include <QtGui/QPainter>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QLineEdit>
 #include <QtGui/QLabel>
 #include <QtGui/QPushButton>
+#include <QtGui/QToolButton>
+#include <QtGui/QContextMenuEvent>
+#include <QtGui/QItemSelection>
 
 #include <QtCore/QRegExp>
-
-#include <qdebug.h>
+#include <QtCore/QDebug>
+#include <QtCore/QSignalMapper>
+#include <QtCore/QBuffer>
 
 Q_DECLARE_METATYPE(QAction*)
-Q_DECLARE_METATYPE(QListWidgetItem*)
+
+QT_BEGIN_NAMESPACE
 
 namespace qdesigner_internal {
+
 //-------- ActionFilterWidget
 class ActionFilterWidget: public QWidget
 {
@@ -92,8 +98,8 @@ class ActionFilterWidget: public QWidget
 public:
     ActionFilterWidget(ActionEditor *actionEditor, QToolBar *parent)
         : QWidget(parent),
-          m_button(new QPushButton(this)),
-          m_editor(new QLineEdit(this)),
+          m_button(new QPushButton),
+          m_editor(new QLineEdit),
           m_actionEditor(actionEditor)
     {
         QHBoxLayout *l = new QHBoxLayout(this);
@@ -102,7 +108,7 @@ public:
 
         l->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
-        QLabel *label = new QLabel(tr("Filter: "), this);
+        QLabel *label = new QLabel(tr("Filter: "));
         label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
         l->addWidget(label);
 
@@ -111,9 +117,10 @@ public:
         connect(m_editor, SIGNAL(textChanged(QString)), actionEditor, SLOT(setFilter(QString)));
 
         m_button->setIcon(createIconSet(QLatin1String("resetproperty.png")));
-        m_button->setIconSize(QSize(16, 16));
+        m_button->setIconSize(QSize(8, 8));
         m_button->setFlat(true);
         l->addWidget(m_button);
+
         connect(m_button, SIGNAL(clicked()), m_editor, SLOT(clear()));
         connect(m_editor, SIGNAL(textChanged(QString)), this, SLOT(checkButton(QString)));
     }
@@ -149,39 +156,99 @@ public:
 };
 
 //--------  ActionEditor
-ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent, Qt::WindowFlags flags)
-    : QDesignerActionEditorInterface(parent, flags),
-      m_core(core)
+ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent, Qt::WindowFlags flags) :
+    QDesignerActionEditorInterface(parent, flags),
+    m_core(core),
+    m_actionGroups(0),
+    m_actionView(new ActionView),
+    m_actionNew(new QAction(tr("New..."), this)),
+    m_actionEdit(new QAction(tr("Edit..."), this)),
+    m_actionCopy(new QAction(tr("Copy"), this)),
+    m_actionCut(new QAction(tr("Cut"), this)),
+    m_actionPaste(new QAction(tr("Paste"), this)),
+    m_actionSelectAll(new QAction(tr("Select all"), this)),
+    m_actionDelete(new QAction(tr("Delete"), this)),
+    m_viewModeGroup(new  QActionGroup(this)),
+    m_iconViewAction(0),
+    m_listViewAction(0),
+    m_filterWidget(0),
+    m_selectAssociatedWidgetsMapper(0)
 {
+    m_actionView->initialize(m_core);
+    m_actionView->setSelectionMode(QAbstractItemView::ExtendedSelection);
     setWindowTitle(tr("Actions"));
 
     QVBoxLayout *l = new QVBoxLayout(this);
     l->setMargin(0);
     l->setSpacing(0);
 
-    QToolBar *toolbar = new QToolBar(this);
-    toolbar->setIconSize(QSize(24, 24));
+    QToolBar *toolbar = new QToolBar;
+    toolbar->setIconSize(QSize(22, 22));
     toolbar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
     l->addWidget(toolbar);
-
-    m_actionNew = toolbar->addAction(tr("New..."));
+    // edit actions
     m_actionNew->setIcon(createIconSet(QLatin1String("filenew.png")));
     m_actionNew->setEnabled(false);
     connect(m_actionNew, SIGNAL(triggered()), this, SLOT(slotNewAction()));
+    toolbar->addAction(m_actionNew);
+
+    connect(m_actionSelectAll, SIGNAL(triggered()), m_actionView, SLOT(selectAll()));
+
+    m_actionCut->setEnabled(false);
+    connect(m_actionCut, SIGNAL(triggered()), this, SLOT(slotCut()));
+    m_actionCut->setIcon(createIconSet(QLatin1String("editcut.png")));
+
+    m_actionCopy->setEnabled(false);
+    connect(m_actionCopy, SIGNAL(triggered()), this, SLOT(slotCopy()));
+    m_actionCopy->setIcon(createIconSet(QLatin1String("editcopy.png")));
+    toolbar->addAction(m_actionCopy);
+
+    connect(m_actionPaste, SIGNAL(triggered()), this, SLOT(slotPaste()));
+    m_actionPaste->setIcon(createIconSet(QLatin1String("editpaste.png")));
+    toolbar->addAction(m_actionPaste);
+
+    m_actionEdit->setEnabled(false);
+    connect(m_actionEdit, SIGNAL(triggered()), this, SLOT(editCurrentAction()));
 
-    m_actionDelete = toolbar->addAction(tr("Delete"));
     m_actionDelete->setIcon(createIconSet(QLatin1String("editdelete.png")));
     m_actionDelete->setEnabled(false);
+    connect(m_actionDelete, SIGNAL(triggered()), this, SLOT(slotDelete()));
+    toolbar->addAction(m_actionDelete);
 
+    // Toolbutton with menu containing action group for detailed/icon view. Steal the icons from the file dialog.
+    QToolButton *configureButton = new QToolButton;
+    QAction *configureAction = new QAction(tr("Configure Action Editor"), this);
+    configureAction->setIcon(createIconSet(QLatin1String("configure.png")));
+    QMenu *configureMenu = new QMenu(this);
+    configureAction->setMenu(configureMenu);
+    configureButton->setDefaultAction(configureAction);
+    configureButton->setPopupMode(QToolButton::InstantPopup);
+    toolbar->addWidget(configureButton);
+
+    m_viewModeGroup->setExclusive(true);
+    connect(m_viewModeGroup, SIGNAL(triggered(QAction*)), this, SLOT(slotViewMode(QAction*)));
+
+    m_iconViewAction = m_viewModeGroup->addAction(tr("Icon View"));
+    m_iconViewAction->setData(QVariant(ActionView::IconView));
+    m_iconViewAction->setCheckable(true);
+    m_iconViewAction->setIcon(style()->standardIcon (QStyle::SP_FileDialogListView));
+    configureMenu->addAction(m_iconViewAction);
+
+    m_listViewAction = m_viewModeGroup->addAction(tr("Detailed View"));
+    m_listViewAction->setData(QVariant(ActionView::DetailedView));
+    m_listViewAction->setCheckable(true);
+    m_listViewAction->setIcon(style()->standardIcon (QStyle::SP_FileDialogDetailedView));
+    configureMenu->addAction(m_listViewAction);
+    // filter
     m_filterWidget = new ActionFilterWidget(this, toolbar);
     m_filterWidget->setEnabled(false);
     toolbar->addWidget(m_filterWidget);
 
-    connect(m_actionDelete, SIGNAL(triggered()), this, SLOT(slotDeleteAction()));
-
-    QSplitter *splitter = new QSplitter(Qt::Horizontal, this);
+    // main layout
+    QSplitter *splitter = new QSplitter(Qt::Horizontal);
     splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
+    splitter->addWidget(m_actionView);
     l->addWidget(splitter);
 
 #if 0 // ### implement me
@@ -196,20 +263,22 @@ ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent,
     m_actionGroups->setWrapping(false);
 #endif
 
-    m_actionRepository = new ActionRepository(splitter);
-    connect(m_actionRepository, SIGNAL(resourceImageDropped(ResourceMimeData,QAction*)),
-            this, SLOT(resourceImageDropped(ResourceMimeData,QAction*)));
-    splitter->addWidget(m_actionRepository);
+    connect(m_actionView, SIGNAL(resourceImageDropped(QString,QAction*)),
+            this, SLOT(resourceImageDropped(QString,QAction*)));
 
-    connect(m_actionRepository, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
-            this, SLOT(slotItemChanged(QListWidgetItem*)));
+    connect(m_actionView, SIGNAL(currentChanged(QAction*)),this, SLOT(slotCurrentItemChanged(QAction*)));
     // make it possible for vs integration to reimplement edit action dialog
-    connect(m_actionRepository, SIGNAL(itemActivated(QListWidgetItem*)),
-            this, SIGNAL(itemActivated(QListWidgetItem*)));
-    connect(m_actionRepository, SIGNAL(contextMenuRequested(QContextMenuEvent*, QListWidgetItem*)),
-            this, SIGNAL(contextMenuRequested(QContextMenuEvent*, QListWidgetItem*)));
-    connect(this, SIGNAL(itemActivated(QListWidgetItem*)),
-            this, SLOT(editAction(QListWidgetItem*)));
+    connect(m_actionView, SIGNAL(activated(QAction*)), this, SIGNAL(itemActivated(QAction*)));
+
+    connect(m_actionView,SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+            this, SLOT(slotSelectionChanged(QItemSelection,QItemSelection)));
+
+    connect(m_actionView, SIGNAL(contextMenuRequested(QContextMenuEvent*, QAction*)),
+            this, SLOT(slotContextMenuRequested(QContextMenuEvent*, QAction*)));
+
+    connect(this, SIGNAL(itemActivated(QAction*)), this, SLOT(editAction(QAction*)));
+
+    updateViewModeActions();
 }
 
 ActionEditor::~ActionEditor()
@@ -241,18 +310,22 @@ void ActionEditor::setFormWindow(QDesignerFormWindowInterface *formWindow)
         return;
 
     if (m_formWindow != 0) {
-        const QList<QAction*> actionList = qFindChildren<QAction*>(m_formWindow->mainContainer());
+        const ActionList actionList = qFindChildren<QAction*>(m_formWindow->mainContainer());
         foreach (QAction *action, actionList)
             disconnect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
     }
 
     m_formWindow = formWindow;
 
-    m_actionRepository->clear();
+    m_actionView->model()->clearActions();
+
+    m_actionEdit->setEnabled(false);
+    m_actionCopy->setEnabled(false);
+    m_actionCut->setEnabled(false);
+    m_actionDelete->setEnabled(false);
 
     if (!formWindow || !formWindow->mainContainer()) {
         m_actionNew->setEnabled(false);
-        m_actionDelete->setEnabled(false);
         m_filterWidget->setEnabled(false);
         return;
     }
@@ -260,74 +333,34 @@ void ActionEditor::setFormWindow(QDesignerFormWindowInterface *formWindow)
     m_actionNew->setEnabled(true);
     m_filterWidget->setEnabled(true);
 
-    const QList<QAction*> actionList = qFindChildren<QAction*>(formWindow->mainContainer());
-    foreach (QAction *action, actionList) {
-        if (!core()->metaDataBase()->item(action)
-            || action->isSeparator()
-            // ### || action->menu()
-            ) {
-            continue;
+    const ActionList actionList = qFindChildren<QAction*>(formWindow->mainContainer());
+    foreach (QAction *action, actionList)
+        if (!action->isSeparator() && core()->metaDataBase()->item(action) != 0) {
+            // Show unless it has a menu. However, listen for change on menu actions also as it might be removed
+            if (!action->menu())
+                m_actionView->model()->addAction(action);
+            connect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
         }
 
-        createListWidgetItem(action);
-        connect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
-    }
-
     setFilter(m_filter);
 }
 
-static QIcon fixActionIcon(QIcon icon)
+void  ActionEditor::slotSelectionChanged(const QItemSelection& selected, const QItemSelection& /*deselected*/)
 {
-    if (icon.isNull())
-        return emptyIcon();
-    return icon;
+    const bool hasSelection = !selected.indexes().empty();
+    m_actionCopy->setEnabled(hasSelection);
+    m_actionCut->setEnabled(hasSelection);
+    m_actionDelete->setEnabled(hasSelection);
 }
 
-// Set up list widget item, icon, tooltip
-static void setListWidgetItem(const QAction *action,  QListWidgetItem *item)
-{
-    item->setText(action->objectName());
-    item->setIcon(fixActionIcon(action->icon()));
-
-    QString tooltip = action->objectName();
-    const QString text = action->text();
-    if (!text.isEmpty()) {
-        tooltip += QLatin1Char('\n');
-        tooltip += text;
-    }
-    item->setToolTip(tooltip);
-    item->setWhatsThis(tooltip);
-}
-
-QListWidgetItem *ActionEditor::createListWidgetItem(QAction *action)
-{
-    if (action->menu())
-        return 0;
-
-    QListWidgetItem *item = new QListWidgetItem(m_actionRepository);
-    const QSize s = m_actionRepository->iconSize();
-    item->setSizeHint(QSize(s.width() * 3, s.height() * 2));
-
-    setListWidgetItem(action, item);
-
-    QVariant itemData;
-    qVariantSetValue(itemData, action);
-    item->setData(ActionRepository::ActionRole, itemData);
-    item->setFlags(item->flags() | Qt::ItemIsDropEnabled);
-    return item;
-}
-
-void ActionEditor::slotItemChanged(QListWidgetItem *item)
+void ActionEditor::slotCurrentItemChanged(QAction *action)
 {
     QDesignerFormWindowInterface *fw = formWindow();
     if (!fw)
         return;
 
-    QAction *action = 0;
-    if (item)
-        action = itemToAction(item);
-
-    m_actionDelete->setEnabled(action != 0);
+    const bool hasCurrentAction = action != 0;
+    m_actionEdit->setEnabled(hasCurrentAction);
 
     if (!action) {
         fw->clearSelection();
@@ -337,7 +370,7 @@ void ActionEditor::slotItemChanged(QListWidgetItem *item)
     QDesignerObjectInspector *oi = qobject_cast<QDesignerObjectInspector *>(core()->objectInspector());
 
     if (action->associatedWidgets().empty()) {
-        // Special case: action not in object tree. Unselect all and set in property editor
+        // Special case: action not in object tree. Deselect all and set in property editor
         fw->clearSelection(false);
         if (oi)
             oi->clearSelection();
@@ -348,36 +381,21 @@ void ActionEditor::slotItemChanged(QListWidgetItem *item)
     }
 }
 
-QListWidgetItem *ActionEditor::actionToItem(QAction *action) const
-{
-    const int cnt = m_actionRepository->count();
-    for (int i = 0; i < cnt; ++i) {
-        QListWidgetItem *item = m_actionRepository->item(i);
-        if (itemToAction(item) == action)
-            return item;
-    }
-    return 0;
-}
-
-QAction *ActionEditor::itemToAction(QListWidgetItem *item) const
-{
-    return qvariant_cast<QAction*>(item->data(ActionRepository::ActionRole));
-}
-
 void ActionEditor::slotActionChanged()
 {
     QAction *action = qobject_cast<QAction*>(sender());
     Q_ASSERT(action != 0);
 
-    QListWidgetItem *item = actionToItem(action);
-    if (item == 0) {
+    ActionModel *model = m_actionView->model();
+    const int row = model->findAction(action);
+    if (row == -1) {
         if (action->menu() == 0) // action got its menu deleted, create item
-            createListWidgetItem(action);
+            model->addAction(action);
     } else if (action->menu() != 0) { // action got its menu created, remove item
-        delete item;
+        model->removeRow(row);
     } else {
         // action text or icon changed, update item
-        setListWidgetItem(action, item);
+        model->update(row);
     }
 }
 
@@ -394,7 +412,7 @@ QString ActionEditor::filter() const
 void ActionEditor::setFilter(const QString &f)
 {
     m_filter = f;
-    m_actionRepository->filter(m_filter);
+    m_actionView->filter(m_filter);
 }
 
 // Set changed state of icon property,  reset when icon is cleared
@@ -416,9 +434,7 @@ void ActionEditor::manageAction(QAction *action)
     sheet->setChanged(sheet->indexOf(QLatin1String("text")), true);
     refreshIconPropertyChanged(action, sheet);
 
-    QListWidgetItem *item = createListWidgetItem(action);
-    m_actionRepository->setCurrentItem(item);
-
+    m_actionView->setCurrentIndex(m_actionView->model()->addAction(action));
     connect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
 }
 
@@ -429,11 +445,9 @@ void ActionEditor::unmanageAction(QAction *action)
 
     disconnect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
 
-    QListWidgetItem *item = actionToItem(action);
-    if (item == 0)
-        return;
-
-    delete item;
+    const int row = m_actionView->model()->findAction(action);
+    if (row != -1)
+        m_actionView->model()->remove(row);
 }
 
 void ActionEditor::slotNewAction()
@@ -442,11 +456,13 @@ void ActionEditor::slotNewAction()
     dlg.setWindowTitle(tr("New action"));
 
     if (dlg.exec() == QDialog::Accepted) {
+        m_actionView->clearSelection();
         QAction *action = new QAction(formWindow());
         action->setObjectName(dlg.actionName());
         formWindow()->ensureUniqueObjectName(action);
         action->setText(dlg.actionText());
-        action->setIcon(dlg.actionIcon());
+        QDesignerPropertySheetExtension *sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
+        sheet->setProperty(sheet->indexOf("icon"), qVariantFromValue(dlg.actionIcon()));
 
         AddActionCommand *cmd = new AddActionCommand(formWindow());
         cmd->init(action);
@@ -460,43 +476,29 @@ static inline bool isSameIcon(const QIcon &i1, const QIcon &i2)
 }
 
 // return a FormWindow command to apply an icon
-static QDesignerFormWindowCommand *setIconPropertyCommand(const QIcon &newIcon, QAction *action, QDesignerFormWindowInterface *fw)
+static QDesignerFormWindowCommand *setIconPropertyCommand(const PropertySheetIconValue &newIcon, QAction *action, QDesignerFormWindowInterface *fw)
 {
     const QString iconProperty = QLatin1String("icon");
-    if (newIcon.isNull()) {
+    if (newIcon.paths().isEmpty()) {
         ResetPropertyCommand *cmd = new ResetPropertyCommand(fw);
         cmd->init(action, iconProperty);
         return cmd;
     }
     SetPropertyCommand *cmd = new SetPropertyCommand(fw);
-    cmd->init(action, iconProperty, newIcon);
+    cmd->init(action, iconProperty, qVariantFromValue(newIcon));
     return cmd;
 }
 
-static QDesignerFormWindowCommand *setTextPropertyCommand(const QString &propertyName, const QString &text, QAction *action, QDesignerFormWindowInterface *fw)
+void ActionEditor::editAction(QAction *action)
 {
-    if (text.isEmpty()) {
-        ResetPropertyCommand *cmd = new ResetPropertyCommand(fw);
-        cmd->init(action, propertyName);
-        return cmd;
-    }
-    SetPropertyCommand *cmd = new SetPropertyCommand(fw);
-    cmd->init(action, propertyName, text);
-    return cmd;
-}
-
-void ActionEditor::editAction(QListWidgetItem *item)
-{
-    if (!item)
-        return;
-
-    QAction *action = itemToAction(item);
-    if (action == 0)
+    if (!action)
         return;
 
     NewActionDialog dlg(this);
     dlg.setWindowTitle(tr("Edit action"));
-    dlg.setActionData(action->text(), action->objectName(), action->icon());
+    QDesignerPropertySheetExtension *sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
+    const PropertySheetIconValue icon = qVariantValue<PropertySheetIconValue>(sheet->property(sheet->indexOf("icon")));
+    dlg.setActionData(action->text(), action->objectName(), icon);
 
     if (!dlg.exec())
         return;
@@ -505,7 +507,7 @@ void ActionEditor::editAction(QListWidgetItem *item)
     enum ChangedMask { NameChanged = 1, TextChanged = 2 , IconChanged = 4 };
     const QString newName = dlg.actionName();
     const QString newText = dlg.actionText();
-    const QIcon newIcon = dlg.actionIcon();
+    const PropertySheetIconValue newIcon = dlg.actionIcon();
 
     int changedMask = 0;
 
@@ -513,7 +515,7 @@ void ActionEditor::editAction(QListWidgetItem *item)
         changedMask |= NameChanged;
     if (newText != action->text())
         changedMask |= TextChanged;
-    if (!isSameIcon(newIcon, action->icon()))
+    if (newIcon.paths() != icon.paths())
         changedMask |= IconChanged;
 
     if (!changedMask)
@@ -524,10 +526,10 @@ void ActionEditor::editAction(QListWidgetItem *item)
         formWindow()->beginCommand(QLatin1String("Edit action"));
 
     if (changedMask & NameChanged)
-        formWindow()->commandHistory()->push(setTextPropertyCommand(QLatin1String("objectName"), newName, action, formWindow()));
+        formWindow()->commandHistory()->push(createTextPropertyCommand(QLatin1String("objectName"), newName, action, formWindow()));
 
     if (changedMask & TextChanged)
-        formWindow()->commandHistory()->push(setTextPropertyCommand(QLatin1String("text"), newText, action, formWindow()));
+        formWindow()->commandHistory()->push(createTextPropertyCommand(QLatin1String("text"), newText, action, formWindow()));
 
     if (changedMask & IconChanged)
         formWindow()->commandHistory()->push(setIconPropertyCommand(newIcon, action, formWindow()));
@@ -537,24 +539,60 @@ void ActionEditor::editAction(QListWidgetItem *item)
 
 }
 
-void ActionEditor::slotDeleteAction()
+void ActionEditor::editCurrentAction()
 {
-    QListWidgetItem *item = m_actionRepository->currentItem();
-    if (item == 0)
-        return;
-
-    QAction *action = itemToAction(item);
-    if (action == 0)
-        return;
-
-    RemoveActionCommand *cmd = new RemoveActionCommand(formWindow());
-    cmd->init(action);
-    formWindow()->commandHistory()->push(cmd);
+    if (QAction *a = m_actionView->currentAction())
+        editAction(a);
 }
 
-void ActionEditor::slotNotImplemented()
+void ActionEditor::deleteActions(QDesignerFormWindowInterface *fw, const ActionList &actions)
 {
-    QMessageBox::information(this, tr("Designer"), tr("Feature not implemented!"));
+    const bool hasMulti = actions.size() > 1;
+    if (hasMulti)
+        fw->beginCommand(tr("Remove actions"));
+
+    foreach(QAction *action, actions) {
+        RemoveActionCommand *cmd = new RemoveActionCommand(fw);
+        cmd->init(action);
+        fw->commandHistory()->push(cmd);
+    }
+    if (hasMulti)
+        fw->endCommand();
+}
+
+void ActionEditor::copyActions(QDesignerFormWindowInterface *fwi, const ActionList &actions)
+{
+    FormWindowBase *fw = qobject_cast<FormWindowBase *>(fwi);
+    if (!fw )
+        return;
+
+    FormBuilderClipboard clipboard;
+    clipboard.m_actions = actions;
+
+    if (clipboard.empty())
+        return;
+
+    QEditorFormBuilder *formBuilder = fw->createFormBuilder();
+    Q_ASSERT(formBuilder);
+
+    QBuffer buffer;
+    if (buffer.open(QIODevice::WriteOnly))
+        if (formBuilder->copy(&buffer, clipboard))
+            qApp->clipboard()->setText(QString::fromUtf8(buffer.buffer()), QClipboard::Clipboard);
+    delete formBuilder;
+}
+
+void ActionEditor::slotDelete()
+{
+    QDesignerFormWindowInterface *fw =  formWindow();
+    if (!fw)
+        return;
+
+    const ActionView::ActionList selection = m_actionView->selectedActions();
+    if (selection.empty())
+        return;
+
+    deleteActions(fw,  selection);
 }
 
 QString ActionEditor::actionTextToName(const QString &text, const QString &prefix)
@@ -574,18 +612,20 @@ QString ActionEditor::actionTextToName(const QString &text, const QString &prefi
     return name;
 }
 
-void  ActionEditor::resourceImageDropped(const ResourceMimeData &data, QAction *action)
+void  ActionEditor::resourceImageDropped(const QString &path, QAction *action)
 {
     QDesignerFormWindowInterface *fw =  formWindow();
     if (!fw)
         return;
 
-    const QIcon icon = resourceMimeDataToIcon(data, fw);
-
-    if (icon.isNull() || isSameIcon(icon, action->icon()))
+    QDesignerPropertySheetExtension *sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
+    const PropertySheetIconValue oldIcon = qVariantValue<PropertySheetIconValue>(sheet->property(sheet->indexOf("icon")));
+    PropertySheetIconValue newIcon;
+    newIcon.setPixmap(QIcon::Normal, QIcon::Off, PropertySheetPixmapValue(path));
+    if (newIcon.paths().isEmpty() || newIcon.paths() == oldIcon.paths())
         return;
 
-    fw->commandHistory()->push(setIconPropertyCommand(icon , action, fw));
+    fw->commandHistory()->push(setIconPropertyCommand(newIcon , action, fw));
 }
 
 void ActionEditor::mainContainerChanged()
@@ -594,6 +634,127 @@ void ActionEditor::mainContainerChanged()
     if (sender() == formWindow())
         setFormWindow(0);
 }
+
+int ActionEditor::viewMode() const
+{
+    return m_actionView->viewMode();
+}
+
+void ActionEditor::setViewMode(int lm)
+{
+    m_actionView->setViewMode(lm);
+    updateViewModeActions();
+}
+
+void ActionEditor::slotViewMode(QAction *a)
+{
+    setViewMode(a->data().toInt());
+}
+
+void ActionEditor::slotSelectAssociatedWidget(QWidget *w)
+{
+    QDesignerFormWindowInterface *fw = formWindow();
+    if (!fw )
+        return;
+
+    QDesignerObjectInspector *oi = qobject_cast<QDesignerObjectInspector *>(core()->objectInspector());
+    if (!oi)
+        return;
+
+    fw->clearSelection(); // Actually, there are no widgets selected due to focus in event handling. Just to be sure.
+    oi->selectObject(w);
+}
+
+void ActionEditor::updateViewModeActions()
+{
+    switch (viewMode()) {
+    case ActionView::IconView:
+        m_iconViewAction->setChecked(true);
+        break;
+    case ActionView::DetailedView:
+        m_listViewAction->setChecked(true);
+        break;
+    }
+}
+
+void ActionEditor::slotCopy()
+{
+    QDesignerFormWindowInterface *fw = formWindow();
+    if (!fw )
+        return;
+
+    const ActionView::ActionList selection = m_actionView->selectedActions();
+    if (selection.empty())
+        return;
+
+    copyActions(fw, selection);
+}
+
+void ActionEditor::slotCut()
+{
+    QDesignerFormWindowInterface *fw = formWindow();
+    if (!fw )
+        return;
+
+    const ActionView::ActionList selection = m_actionView->selectedActions();
+    if (selection.empty())
+        return;
+
+    copyActions(fw, selection);
+    deleteActions(fw, selection);
+}
+
+void ActionEditor::slotPaste()
+{
+    FormWindowBase *fw = qobject_cast<FormWindowBase *>(formWindow());
+    if (!fw)
+        return;
+    m_actionView->clearSelection();
+    fw->paste(FormWindowBase::PasteActionsOnly);
+}
+
+void ActionEditor::slotContextMenuRequested(QContextMenuEvent *e, QAction *item)
+{
+    // set up signal mapper
+    if (!m_selectAssociatedWidgetsMapper) {
+        m_selectAssociatedWidgetsMapper = new QSignalMapper(this);
+        connect(m_selectAssociatedWidgetsMapper, SIGNAL(mapped(QWidget*)), this, SLOT(slotSelectAssociatedWidget(QWidget*)));
+    }
+
+    QMenu menu(this);
+    menu.addAction(m_actionNew);
+    menu.addSeparator();
+    menu.addAction(m_actionEdit);
+    // Associated Widgets
+    if (QAction *action = m_actionView->currentAction()) {
+        const QWidgetList associatedWidgets = ActionModel::associatedWidgets(action);
+        if (!associatedWidgets.empty()) {
+            QMenu *associatedWidgetsSubMenu =  menu.addMenu(tr("Used In"));
+            foreach (QWidget *w, associatedWidgets) {
+                QAction *action = associatedWidgetsSubMenu->addAction(w->objectName());
+                m_selectAssociatedWidgetsMapper->setMapping(action, w);
+                connect(action, SIGNAL(triggered()), m_selectAssociatedWidgetsMapper, SLOT(map()));
+            }
+        }
+    }
+    menu.addSeparator();
+    menu.addAction(m_actionCut);
+    menu.addAction(m_actionCopy);
+    menu.addAction(m_actionPaste);
+    menu.addAction(m_actionSelectAll);
+    menu.addAction(m_actionDelete);
+    menu.addSeparator();
+    menu.addAction(m_iconViewAction);
+    menu.addAction(m_listViewAction);
+
+    emit contextMenuRequested(&menu, item);
+
+    menu.exec(e->globalPos());
+    e->accept();
+}
+
 } // namespace qdesigner_internal
 
+QT_END_NAMESPACE
+
 #include "actioneditor.moc"
