@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,65 +49,53 @@
 #include <QtDesigner/QExtensionManager>
 
 #include <QtGui/QHBoxLayout>
+#include <QtGui/QFormLayout>
 #include <QtGui/QSplitter>
+#include <QtCore/QDebug>
+#include <QtCore/QHash>
 
-#include <QtCore/QMap>
-#include <QtCore/qdebug.h>
+QT_BEGIN_NAMESPACE
 
 namespace qdesigner_internal {
-
-LayoutInfo::Type LayoutInfo::layoutType(QDesignerFormEditorInterface *core, QWidget *w, QLayout *&layout)
-{
-    layout = 0;
-
-    if (QDesignerContainerExtension *container = qt_extension<QDesignerContainerExtension*>(core->extensionManager(), w))
-        w = container->widget(container->currentIndex());
-
-    if (qobject_cast<QSplitter*>(w))
-        return static_cast<QSplitter*>(w)->orientation() == Qt::Horizontal ? HBox : VBox;
-
-    if (!w || !w->layout())
-        return NoLayout;
-
-    QLayout *lay = w->layout();
-
-    if (lay && core->metaDataBase()->item(lay) == 0) {
-        lay = qFindChild<QLayout*>(lay);
-    }
-    layout = lay;
-
-#ifdef QD_DEBUG
-    Q_ASSERT (lay == 0 || core->metaDataBase()->item(lay) != 0);
-#endif
-
-    return layoutType(core, lay);
-}
-
 /*!
   \overload
 */
-LayoutInfo::Type LayoutInfo::layoutType(QDesignerFormEditorInterface *core, QLayout *layout)
+LayoutInfo::Type LayoutInfo::layoutType(const QDesignerFormEditorInterface *core, const QLayout *layout)
 {
     Q_UNUSED(core)
-
-    if (qobject_cast<QHBoxLayout*>(layout))
+    if (qobject_cast<const QHBoxLayout*>(layout))
         return HBox;
-    else if (qobject_cast<QVBoxLayout*>(layout))
+    else if (qobject_cast<const QVBoxLayout*>(layout))
         return VBox;
-    else if (qobject_cast<QGridLayout*>(layout))
+    else if (qobject_cast<const QGridLayout*>(layout))
         return Grid;
+    else if (qobject_cast<const QFormLayout*>(layout))
+       return Form;
     return NoLayout;
 }
 
+LayoutInfo::Type LayoutInfo::layoutType(const QString &typeName)
+{
+    static QHash<QString, Type> nameTypeMap;
+    if (nameTypeMap.empty()) {
+        nameTypeMap.insert(QLatin1String("QVBoxLayout"), VBox);
+        nameTypeMap.insert(QLatin1String("QHBoxLayout"), HBox);
+        nameTypeMap.insert(QLatin1String("QGridLayout"), Grid);
+        nameTypeMap.insert(QLatin1String("QFormLayout"), Form);
+    }
+    return nameTypeMap.value(typeName, NoLayout);
+}
 /*!
   \overload
 */
-LayoutInfo::Type LayoutInfo::layoutType(QDesignerFormEditorInterface *core, QWidget *w)
+LayoutInfo::Type LayoutInfo::layoutType(const QDesignerFormEditorInterface *core, const QWidget *w)
 {
+    if (const QSplitter *splitter = qobject_cast<const QSplitter *>(w))
+        return  splitter->orientation() == Qt::Horizontal ? HSplitter : VSplitter;
     return layoutType(core, w->layout());
 }
 
-QWidget *LayoutInfo::layoutParent(QDesignerFormEditorInterface *core, QLayout *layout)
+QWidget *LayoutInfo::layoutParent(const QDesignerFormEditorInterface *core, QLayout *layout)
 {
     Q_UNUSED(core)
 
@@ -121,7 +109,7 @@ QWidget *LayoutInfo::layoutParent(QDesignerFormEditorInterface *core, QLayout *l
     return 0;
 }
 
-void LayoutInfo::deleteLayout(QDesignerFormEditorInterface *core, QWidget *widget)
+void LayoutInfo::deleteLayout(const QDesignerFormEditorInterface *core, QWidget *widget)
 {
     if (QDesignerContainerExtension *container = qt_extension<QDesignerContainerExtension*>(core->extensionManager(), widget))
         widget = container->widget(container->currentIndex());
@@ -132,58 +120,56 @@ void LayoutInfo::deleteLayout(QDesignerFormEditorInterface *core, QWidget *widge
 
     if (layout == 0 || core->metaDataBase()->item(layout) != 0) {
         delete layout;
+        widget->updateGeometry();
         return;
     }
 
     qDebug() << "trying to delete an unmanaged layout:" << "widget:" << widget << "layout:" << layout;
 }
 
-void LayoutInfo::cells(QLayout *layout, IntervalList *rows, IntervalList *columns)
+LayoutInfo::Type LayoutInfo::laidoutWidgetType(const QDesignerFormEditorInterface *core, QWidget *widget, bool *isManaged)
 {
-    QMap<Interval, int> rowDict;
-    QMap<Interval, int> columnDict;
-
-    int i = 0;
-    while (QLayoutItem *item = layout->itemAt(i)) {
-        ++i;
-
-        QRect g = item->geometry();
-        columnDict.insert(Interval(g.left(), g.right()), 1);
-        rowDict.insert(Interval(g.top(), g.bottom()), 1);
-    }
-
-    if (columns)
-        *columns = columnDict.keys();
-
-    if (rows)
-        *rows = rowDict.keys();
-}
-
-bool LayoutInfo::isWidgetLaidout(QDesignerFormEditorInterface *core, QWidget *widget)
-{
-    Q_UNUSED(core);
+    if (isManaged)
+        *isManaged = false;
 
     QWidget *parent = widget->parentWidget();
+    if (!parent)
+        return NoLayout;
 
-    if (qobject_cast<QSplitter*>(parent)) { // ### generalize
-        return true;
+    // 1) Splitter
+    if (QSplitter *splitter  = qobject_cast<QSplitter*>(parent)) {
+        if (isManaged)
+            *isManaged = core->metaDataBase()->item(splitter);
+        return  splitter->orientation() == Qt::Horizontal ? HSplitter : VSplitter;
     }
 
-    if (parent && parent->layout()) {
-        if (parent->layout()->indexOf(widget) != -1)
-            return true;
+    // 2) Layout of parent
+    QLayout *parentLayout = parent->layout();
+    if (!parentLayout)
+        return NoLayout;
 
-        QList<QLayout*> childLayouts = qFindChildren<QLayout*>(parent->layout());
-        foreach (QLayout *childLayout, childLayouts) {
-            if (childLayout->indexOf(widget) != -1)
-                return true;
+    if (parentLayout->indexOf(widget) != -1) {
+        if (isManaged)
+            *isManaged = core->metaDataBase()->item(parentLayout);
+        return layoutType(core, parentLayout);
+    }
+
+    // 3) Some child layout
+    const QList<QLayout*> childLayouts = qFindChildren<QLayout*>(parentLayout);
+    if (childLayouts.empty())
+        return NoLayout;
+    const QList<QLayout*>::const_iterator lcend = childLayouts.constEnd();
+    for (QList<QLayout*>::const_iterator it = childLayouts.constBegin(); it != lcend; ++it)
+        if ((*it)->indexOf(widget) != -1) {
+            if (isManaged)
+                *isManaged = core->metaDataBase()->item(*it);
+            return layoutType(core, parentLayout);
         }
-    }
 
-    return false;
+    return NoLayout;
 }
 
-QLayout *LayoutInfo::internalLayout(QWidget *widget)
+QLayout *LayoutInfo::internalLayout(const QWidget *widget)
 {
     QLayout *widgetLayout = widget->layout();
     if (widgetLayout && widget->inherits("Q3GroupBox")) {
@@ -197,25 +183,25 @@ QLayout *LayoutInfo::internalLayout(QWidget *widget)
 }
 
 
-QLayout *LayoutInfo::managedLayout(QDesignerFormEditorInterface *core, QWidget *widget)
+QLayout *LayoutInfo::managedLayout(const QDesignerFormEditorInterface *core, const QWidget *widget)
 {
     if (widget == 0)
         return 0;
-    
+
     QLayout *layout = widget->layout();
     if (!layout)
         return 0;
 
-    return managedLayout(core, layout);    
+    return managedLayout(core, layout);
 }
 
-QLayout *LayoutInfo::managedLayout(QDesignerFormEditorInterface *core, QLayout *layout)
+QLayout *LayoutInfo::managedLayout(const QDesignerFormEditorInterface *core, QLayout *layout)
 {
     QDesignerMetaDataBaseInterface *metaDataBase = core->metaDataBase();
 
     if (!metaDataBase)
         return layout;
-    
+
     const QDesignerMetaDataBaseItemInterface *item = metaDataBase->item(layout);
     if (item == 0) {
         layout = qFindChild<QLayout*>(layout);
@@ -226,5 +212,27 @@ QLayout *LayoutInfo::managedLayout(QDesignerFormEditorInterface *core, QLayout *
     return layout;
 }
 
+    static inline void getFormItemPosition(const QFormLayout *formLayout, int index, int *row, int *column)
+{
+    QFormLayout::ItemRole role;
+    formLayout->getItemPosition(index, row, &role);
+    *column = role == QFormLayout::LabelRole ? 0 : 1;
+}
 
+QDESIGNER_SHARED_EXPORT void getFormLayoutItemPosition(const QFormLayout *formLayout, int index, int *rowPtr, int *columnPtr, int *rowspanPtr, int *colspanPtr)
+{
+    int row;
+    QFormLayout::ItemRole role;
+    formLayout->getItemPosition(index, &row, &role);
+    if (rowPtr)
+        *rowPtr = row;
+    if (columnPtr)
+        *columnPtr = role == QFormLayout::LabelRole ? 0 : 1;
+    if (rowspanPtr)
+        *rowspanPtr = 1;
+    if (colspanPtr)
+        *colspanPtr = 1;
+}
 } // namespace qdesigner_internal
+
+QT_END_NAMESPACE
