@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -57,11 +57,12 @@
 
 #include "shared_global_p.h"
 
-#include <QtCore/QList>
+QT_BEGIN_NAMESPACE
 
 class QWidget;
 class QLayout;
 class QDesignerFormEditorInterface;
+class QFormLayout;
 
 namespace qdesigner_internal {
 
@@ -70,37 +71,34 @@ class QDESIGNER_SHARED_EXPORT LayoutInfo
 public:
     enum Type
     {
+        NoLayout,
+        HSplitter,
+        VSplitter,
         HBox,
         VBox,
         Grid,
-        Stacked,
-        NoLayout
+        Form
     };
 
-    static void deleteLayout(QDesignerFormEditorInterface *core, QWidget *widget);
-    static Type layoutType(QDesignerFormEditorInterface *core, QWidget *w, QLayout *&layout);
-    static Type layoutType(QDesignerFormEditorInterface *core, QLayout *layout);
-    static Type layoutType(QDesignerFormEditorInterface *core, QWidget *w);
-    static QWidget *layoutParent(QDesignerFormEditorInterface *core, QLayout *layout);
-    static bool isWidgetLaidout(QDesignerFormEditorInterface *core, QWidget *widget);
+    static void deleteLayout(const QDesignerFormEditorInterface *core, QWidget *widget);
 
-    static QLayout *managedLayout(QDesignerFormEditorInterface *core, QWidget *widget);
-    static QLayout *managedLayout(QDesignerFormEditorInterface *core, QLayout *layout);
-    static QLayout *internalLayout(QWidget *widget);
+    static Type layoutType(const QDesignerFormEditorInterface *core, const QWidget *w);
+    static Type layoutType(const QDesignerFormEditorInterface *core, const QLayout *layout);
+    static Type layoutType(const QString &typeName);
 
-    class Interval
-    {
-    public:
-        int v1, v2;
-        inline Interval(int _v1 = 0, int _v2 = 0)
-            : v1(_v1), v2(_v2) {}
-        bool operator < (const Interval &other) const
-            { return v1 < other.v1; }
-    };
-    typedef QList<Interval> IntervalList;
-    static void cells(QLayout *layout, IntervalList *rows, IntervalList *columns);
+    static QWidget *layoutParent(const QDesignerFormEditorInterface *core, QLayout *layout);
+
+    static Type laidoutWidgetType(const QDesignerFormEditorInterface *core, QWidget *widget, bool *isManaged = 0);
+    static bool inline isWidgetLaidout(const QDesignerFormEditorInterface *core, QWidget *widget) { return laidoutWidgetType(core, widget) != NoLayout; }
+
+    static QLayout *managedLayout(const QDesignerFormEditorInterface *core, const QWidget *widget);
+    static QLayout *managedLayout(const QDesignerFormEditorInterface *core, QLayout *layout);
+    static QLayout *internalLayout(const QWidget *widget);
 };
 
+QDESIGNER_SHARED_EXPORT void getFormLayoutItemPosition(const QFormLayout *formLayout, int index, int *rowPtr, int *columnPtr = 0, int *rowspanPtr = 0, int *colspanPtr = 0);
 } // namespace qdesigner_internal
 
+QT_END_NAMESPACE
+
 #endif // LAYOUTINFO_H
