@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -59,16 +59,18 @@
 
 #include <QtCore/QMap>
 #include <QtCore/QStringList>
-#include <QtCore/QSettings>
+
+QT_BEGIN_NAMESPACE
 
 class QDesignerFormEditorInterface;
 class QDesignerCustomWidgetInterface;
+class QDesignerPluginManagerPrivate;
 
 class QDESIGNER_SHARED_EXPORT QDesignerPluginManager: public QObject
 {
     Q_OBJECT
 public:
-    QDesignerPluginManager(QDesignerFormEditorInterface *core);
+    explicit QDesignerPluginManager(QDesignerFormEditorInterface *core);
     virtual ~QDesignerPluginManager();
 
     QDesignerFormEditorInterface *core() const;
@@ -103,18 +105,11 @@ private:
     void registerPlugin(const QString &plugin);
 
 private:
-    QDesignerFormEditorInterface *m_core;
-    QStringList m_pluginPaths;
-    QStringList m_registeredPlugins;
-    QStringList m_disabledPlugins;
-
-    typedef QMap<QString, QString> FailedPluginMap;
-    FailedPluginMap m_failedPlugins;
-
-    typedef QList<QDesignerCustomWidgetInterface*> CustomWidgetList;
-    CustomWidgetList m_customWidgets;
-
-    QStringList defaultPluginPaths() const;
+    static QStringList defaultPluginPaths();
+    
+    QDesignerPluginManagerPrivate *m_d;
 };
 
+QT_END_NAMESPACE
+
 #endif // PLUGINMANAGER_H
