@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,6 +50,8 @@
 #include "pluginmanager_p.h"
 #include "widgetfactory_p.h"
 #include "qdesigner_widgetbox_p.h"
+#include "qtgradientmanager.h"
+#include "qtgradientutils.h"
 
 // sdk
 #include <QtDesigner/QDesignerFormEditorInterface>
@@ -59,21 +61,45 @@
 #include <QtDesigner/QDesignerActionEditorInterface>
 #include <QtDesigner/QDesignerWidgetBoxInterface>
 #include <QtDesigner/QExtensionManager>
+#include <QtDesigner/QDesignerResourceBrowserInterface>
 
 #include <QtCore/QVariant>
+#include <QtCore/QFile>
+#include <QtCore/QDir>
 
 #include <QtCore/qdebug.h>
 
+QT_BEGIN_NAMESPACE
+
 namespace qdesigner_internal {
 
-QDesignerIntegration::QDesignerIntegration(QDesignerFormEditorInterface *core, QObject *parent)
-    : QDesignerIntegrationInterface(core, parent)
+// ---------------- DesignerIntegrationPrivate
+class QDesignerIntegrationPrivate {
+public:
+    QDesignerIntegrationPrivate() : m_gradientManager(0) {}
+
+    QString m_gradientsPath;
+    QtGradientManager *m_gradientManager;
+};
+
+// -------------- QDesignerIntegration
+// As of 4.4, the header will be distributed with the Eclipse plugin.
+
+QDesignerIntegration::QDesignerIntegration(QDesignerFormEditorInterface *core, QObject *parent) :
+    QDesignerIntegrationInterface(core, parent),
+    m_d(new QDesignerIntegrationPrivate)
 {
     initialize();
 }
 
 QDesignerIntegration::~QDesignerIntegration()
 {
+    QFile f(m_d->m_gradientsPath);
+    if (f.open(QIODevice::WriteOnly)) {
+        f.write(QtGradientUtils::saveState(m_d->m_gradientManager).toUtf8());
+        f.close();
+    }
+    delete m_d;
 }
 
 void QDesignerIntegration::initialize()
@@ -81,18 +107,19 @@ void QDesignerIntegration::initialize()
     //
     // integrate the `Form Editor component'
     //
-    connect(core()->propertyEditor(), SIGNAL(propertyChanged(QString,QVariant)),
-            this, SLOT(updateProperty(QString,QVariant)));
-
 
     // Extensions
     if (QDesignerPropertyEditor *designerPropertyEditor= qobject_cast<QDesignerPropertyEditor *>(core()->propertyEditor())) {
+        connect(designerPropertyEditor, SIGNAL(propertyValueChanged(QString, QVariant, bool)), this, SLOT(updateProperty(QString, QVariant, bool)));
         connect(designerPropertyEditor, SIGNAL(propertyCommentChanged(QString, QString)), this, SLOT(updatePropertyComment(QString, QString)));
         connect(designerPropertyEditor, SIGNAL(resetProperty(QString)), this, SLOT(resetProperty(QString)));
         connect(designerPropertyEditor, SIGNAL(addDynamicProperty(QString,QVariant)),
                 this, SLOT(addDynamicProperty(QString,QVariant)));
         connect(designerPropertyEditor, SIGNAL(removeDynamicProperty(QString)),
                 this, SLOT(removeDynamicProperty(QString)));
+    } else {
+        connect(core()->propertyEditor(), SIGNAL(propertyChanged(QString,QVariant)),
+                this, SLOT(updatePropertyPrivate(QString,QVariant)));
     }
 
     connect(core()->formWindowManager(), SIGNAL(formWindowAdded(QDesignerFormWindowInterface*)),
@@ -100,9 +127,32 @@ void QDesignerIntegration::initialize()
 
     connect(core()->formWindowManager(), SIGNAL(activeFormWindowChanged(QDesignerFormWindowInterface*)),
             this, SLOT(updateActiveFormWindow(QDesignerFormWindowInterface*)));
+
+    m_d->m_gradientManager = new QtGradientManager(this);
+    core()->setGradientManager(m_d->m_gradientManager);
+
+    QString designerFolder = QDir::homePath();
+    designerFolder += QDir::separator();
+    designerFolder += QLatin1String(".designer");
+    m_d->m_gradientsPath = designerFolder;
+    m_d->m_gradientsPath += QDir::separator();
+    m_d->m_gradientsPath += QLatin1String("gradients.xml");
+
+    QFile f(m_d->m_gradientsPath);
+    if (f.open(QIODevice::ReadOnly)) {
+        QtGradientUtils::restoreState(m_d->m_gradientManager, f.readAll());
+        f.close();
+    } else {
+        QFile defaultGradients(":/trolltech/designer/defaultgradients.xml");
+        if (defaultGradients.open(QIODevice::ReadOnly)) {
+            QtGradientUtils::restoreState(m_d->m_gradientManager, defaultGradients.readAll());
+            defaultGradients.close();
+        }
+    }
+
 }
 
-void QDesignerIntegration::updateProperty(const QString &name, const QVariant &value)
+void QDesignerIntegration::updateProperty(const QString &name, const QVariant &value, bool enableSubPropertyHandling)
 {
     QDesignerFormWindowInterface *formWindow = core()->formWindowManager()->activeFormWindow();
     if (!formWindow)
@@ -112,24 +162,24 @@ void QDesignerIntegration::updateProperty(const QString &name, const QVariant &v
     getSelection(selection);
     if (selection.empty())
         return;
-    // Legacy: set properties on widgets via cursor
-    if (!selection.m_cursorSelection.empty() && selection.m_selectedObjects.empty()) {
-        QDesignerFormWindowCursorInterface *cursor = formWindow->cursor();
-        cursor->setProperty(name, value);
+
+    SetPropertyCommand *cmd = new SetPropertyCommand(formWindow);
+    // find a reference object to compare to and to find the right group
+    if (cmd->init(selection.selection(), name, value, propertyEditorObject(), enableSubPropertyHandling)) {
+        formWindow->commandHistory()->push(cmd);
     } else {
-        SetPropertyCommand *cmd = new SetPropertyCommand(formWindow);
-        // find a reference object to compare to and to find the right group
-        if (cmd->init(selection.selection(), name, value, propertyEditorObject())) {
-            formWindow->commandHistory()->push(cmd);
-        } else {
-            delete cmd;
-            qDebug() << "Unable to set  property " << name << '.';
-        }
+        delete cmd;
+        qDebug() << "Unable to set  property " << name << '.';
     }
 
     emit propertyChanged(formWindow, name, value);
 }
 
+void QDesignerIntegration::updatePropertyPrivate(const QString &name, const QVariant &value)
+{
+    updateProperty(name, value, true);
+}
+
 void QDesignerIntegration::updatePropertyComment(const QString &name, const QString &value)
 {
     QDesignerFormWindowInterface *formWindow = core()->formWindowManager()->activeFormWindow();
@@ -282,7 +332,7 @@ void QDesignerIntegration::getSelection(Selection &s)
         if (s.empty())
             if (QObject *object = core()->propertyEditor()->object())
                 s.m_selectedObjects.push_back(object);
-            
+
     } else {
         s.clear();
         // get single selection
@@ -315,14 +365,18 @@ QObject *QDesignerIntegration::propertyEditorObject()
 void QDesignerIntegration::initializePlugins(QDesignerFormEditorInterface *formEditor)
 {
     // load the plugins
-    if (WidgetDataBase *widgetDatabase = qobject_cast<WidgetDataBase*>(formEditor->widgetDataBase())) {
-        widgetDatabase->loadPlugins();
-        widgetDatabase->grabDefaultPropertyValues();
+    WidgetDataBase *widgetDataBase = qobject_cast<WidgetDataBase*>(formEditor->widgetDataBase());
+    if (widgetDataBase) {
+        widgetDataBase->loadPlugins();
     }
 
     if (WidgetFactory *widgetFactory = qobject_cast<WidgetFactory*>(formEditor->widgetFactory())) {
         widgetFactory->loadPlugins();
     }
+
+    if (widgetDataBase) {
+        widgetDataBase->grabDefaultPropertyValues();
+    }
 }
 
 void QDesignerIntegration::updateCustomWidgetPlugins()
@@ -343,9 +397,22 @@ void QDesignerIntegration::updateCustomWidgetPlugins()
     }
 }
 
-void QDesignerIntegration::emitObjectNameChanged(QDesignerFormWindowInterface *formWindow, QObject *object, const QString &name)
+void QDesignerIntegration::emitObjectNameChanged(QDesignerFormWindowInterface *formWindow, QObject *object, const QString &newName, const QString &oldName)
 {
-    emit objectNameChanged(formWindow, object, name);
+    emit objectNameChanged(formWindow, object, newName, oldName);
+}
+
+void QDesignerIntegration::requestHelp(const QDesignerFormEditorInterface *core, const QString &manual, const QString &document)
+{
+    if (QDesignerIntegration *di = qobject_cast<QDesignerIntegration *>(core->integration()))
+        emit di->helpRequested(manual, document);
+}
+
+QDesignerResourceBrowserInterface *QDesignerIntegration::createResourceBrowser(QWidget *)
+{
+    return 0;
 }
 
 } // namespace qdesigner_internal
+
+QT_END_NAMESPACE
