@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -56,9 +56,12 @@
 #define QDESIGNER_STACKEDBOX_H
 
 #include "shared_global_p.h"
+#include "qdesigner_propertysheet_p.h"
 
-#include <QtGui/QStackedWidget>
+QT_BEGIN_NAMESPACE
 
+class QStackedWidget;
+class QWidget;
 class QAction;
 class QMenu;
 class QToolButton;
@@ -67,41 +70,69 @@ namespace qdesigner_internal {
     class PromotionTaskMenu;
 }
 
-class QDESIGNER_SHARED_EXPORT QDesignerStackedWidget : public QStackedWidget
+// Event filter to be installed on a QStackedWidget in preview mode.
+// Create two buttons to switch pages.
+
+class QDESIGNER_SHARED_EXPORT QStackedWidgetPreviewEventFilter : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(QString currentPageName READ currentPageName WRITE setCurrentPageName STORED false DESIGNABLE true)
 public:
-    QDesignerStackedWidget(QWidget *parent);
-    
-    // Add context menu and return page submenu or 0.
-    QMenu *addContextMenuActions(QMenu *popup);
+    explicit QStackedWidgetPreviewEventFilter(QStackedWidget *parent);
 
-    QString currentPageName() const;
-    void setCurrentPageName(const QString &pageName);
+    // Install helper on QStackedWidget
+    static void install(QStackedWidget *stackedWidget);
+    bool eventFilter(QObject *watched, QEvent *event);
+
+    void setButtonToolTipEnabled(bool v) { m_buttonToolTipEnabled = v; }
+    bool buttonToolTipEnabled() const    { return m_buttonToolTipEnabled; }
 
 public slots:
     void updateButtons();
-
-protected:
-    virtual void childEvent(QChildEvent *e);
-    virtual void resizeEvent(QResizeEvent *e);
-    virtual void showEvent(QShowEvent *e);
-    virtual bool event(QEvent *e);
-
-private slots:
     void prevPage();
     void nextPage();
+
+protected:
+    QStackedWidget *stackedWidget() const { return m_stackedWidget; }
+    virtual void gotoPage(int page);
+
+private:
+    void updateButtonToolTip(QObject *o);
+
+    bool m_buttonToolTipEnabled;
+    QStackedWidget *m_stackedWidget;
+    QToolButton *m_prev;
+    QToolButton *m_next;
+};
+
+// Event filter to be installed on a QStackedWidget in editing mode.
+//  In addition to the browse buttons, handles context menu and everything
+
+class QDESIGNER_SHARED_EXPORT QStackedWidgetEventFilter : public QStackedWidgetPreviewEventFilter
+{
+    Q_OBJECT
+public:
+    explicit QStackedWidgetEventFilter(QStackedWidget *parent);
+
+    // Install helper on QStackedWidget
+    static void install(QStackedWidget *stackedWidget);
+    static QStackedWidgetEventFilter *eventFilterOf(const QStackedWidget *stackedWidget);
+    // Convenience to add a menu on a tackedWidget
+    static QMenu *addStackedWidgetContextMenuActions(const QStackedWidget *stackedWidget, QMenu *popup);
+
+    // Add context menu and return page submenu or 0.
+    QMenu *addContextMenuActions(QMenu *popup);
+
+private slots:
     void removeCurrentPage();
     void addPage();
     void addPageAfter();
     void changeOrder();
     void slotCurrentChanged(int index);
 
+protected:
+    virtual void gotoPage(int page);
+
 private:
-    void gotoPage(int page);
-    QToolButton *m_prev;
-    QToolButton *m_next;
     QAction *m_actionPreviousPage;
     QAction *m_actionNextPage;
     QAction *m_actionDeletePage;
@@ -111,4 +142,26 @@ private:
     qdesigner_internal::PromotionTaskMenu* m_pagePromotionTaskMenu;
 };
 
+// PropertySheet to handle the "currentPageName" property
+class QDESIGNER_SHARED_EXPORT QStackedWidgetPropertySheet : public QDesignerPropertySheet {
+public:
+    explicit QStackedWidgetPropertySheet(QStackedWidget *object, QObject *parent = 0);
+
+    virtual void setProperty(int index, const QVariant &value);
+    virtual QVariant property(int index) const;
+    virtual bool reset(int index);
+    virtual bool isEnabled(int index) const;
+
+    // Check whether the property is to be saved. Returns false for the page
+    // properties (as the property sheet has no concept of 'stored')
+    static bool checkProperty(const QString &propertyName);
+
+private:
+    QStackedWidget *m_stackedWidget;
+};
+
+typedef QDesignerPropertySheetFactory<QStackedWidget, QStackedWidgetPropertySheet> QStackedWidgetPropertySheetFactory;
+
+QT_END_NAMESPACE
+
 #endif // QDESIGNER_STACKEDBOX_H
