@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,7 +50,6 @@
 
 #include <QtDesigner/QDesignerFormEditorInterface>
 #include <QtDesigner/QDesignerLanguageExtension>
-#include <QtDesigner/QDesignerIconCacheInterface>
 #include <script_p.h>
 #include <QtDesigner/QExtensionManager>
 #include <QtDesigner/QDesignerCustomWidgetInterface>
@@ -58,12 +57,15 @@
 
 #include <QtGui/QIcon>
 #include <QtGui/QWidget>
+#include <QtGui/QAction>
+#include <QtCore/QDebug>
+
+QT_BEGIN_NAMESPACE
 
 namespace {
     typedef QList<DomWidgetData*> DomWidgetDataList;
     typedef QList<DomProperty*> DomPropertyList;
     typedef QList<QDesignerCustomWidgetInterface *> CustomWidgetInterfaces;
-    typedef QHash<QString, QString> ClassNameScriptHash;
 }
 
 namespace qdesigner_internal {
@@ -99,78 +101,46 @@ DomBrush *QSimpleResource::saveBrush(const QBrush &brush)
 
 QIcon QSimpleResource::nameToIcon(const QString &filePath, const QString &qrcPath)
 {
-    QString file_path = filePath;
-    QString qrc_path = qrcPath;
-
-    if (qrc_path.isEmpty()) {
-        QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension*>(core()->extensionManager(), core());
-        if (!lang || !lang->isLanguageResource(file_path))
-            file_path = workingDirectory().absoluteFilePath(file_path);
-    } else {
-        qrc_path = workingDirectory().absoluteFilePath(qrc_path);
-    }
-
-    return core()->iconCache()->nameToIcon(file_path, qrc_path);
+    Q_UNUSED(filePath)
+    Q_UNUSED(qrcPath)
+    qWarning() << "QSimpleResource::nameToIcon() is obsoleted";
+    return QIcon();
 }
 
 QString QSimpleResource::iconToFilePath(const QIcon &pm) const
 {
-    QString file_path = core()->iconCache()->iconToFilePath(pm);
-    QString qrc_path = core()->iconCache()->iconToQrcPath(pm);
-    if (qrc_path.isEmpty()) {
-        QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension*>(core()->extensionManager(), core());
-        if (!lang || !lang->isLanguageResource(file_path))
-            return workingDirectory().relativeFilePath(file_path);
-    }
-
-    return file_path;
+    Q_UNUSED(pm)
+    qWarning() << "QSimpleResource::iconToFilePath() is obsoleted";
+    return QString();
 }
 
 QString QSimpleResource::iconToQrcPath(const QIcon &pm) const
 {
-    QString qrc_path = core()->iconCache()->iconToQrcPath(pm);
-    if (qrc_path.isEmpty())
-        return QString();
-
-    return workingDirectory().relativeFilePath(qrc_path);
+    Q_UNUSED(pm)
+    qWarning() << "QSimpleResource::iconToQrcPath() is obsoleted";
+    return QString();
 }
 
 QPixmap QSimpleResource::nameToPixmap(const QString &filePath, const QString &qrcPath)
 {
-    QString file_path = filePath;
-    QString qrc_path = qrcPath;
-
-    if (qrc_path.isEmpty()) {
-        QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension*>(core()->extensionManager(), core());
-        if (!lang || !lang->isLanguageResource(file_path))
-            file_path = workingDirectory().absoluteFilePath(file_path);
-    } else {
-        qrc_path = workingDirectory().absoluteFilePath(qrc_path);
-    }
-
-    return core()->iconCache()->nameToPixmap(file_path, qrc_path);
+    Q_UNUSED(filePath)
+    Q_UNUSED(qrcPath)
+    qWarning() << "QSimpleResource::nameToPixmap() is obsoleted";
+    return QPixmap();
 }
 
 QString QSimpleResource::pixmapToFilePath(const QPixmap &pm) const
 {
-    QString file_path = core()->iconCache()->pixmapToFilePath(pm);
-    QString qrc_path = core()->iconCache()->pixmapToQrcPath(pm);
-    if (qrc_path.isEmpty()) {
-        QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension*>(core()->extensionManager(), core());
-        if (!lang || !lang->isLanguageResource(file_path))
-            return workingDirectory().relativeFilePath(file_path);
-    }
-
-    return file_path;
+    Q_UNUSED(pm)
+    qWarning() << "QSimpleResource::pixmapToFilePath() is obsoleted";
+    return QString();
 }
 
 QString QSimpleResource::pixmapToQrcPath(const QPixmap &pm) const
 {
-    QString qrc_path = core()->iconCache()->pixmapToQrcPath(pm);
-    if (qrc_path.isEmpty())
-        return QString();
-
-    return workingDirectory().relativeFilePath(qrc_path);
+    Q_UNUSED(pm)
+    qWarning() << "QSimpleResource::pixmapToQrcPath() is obsoleted";
+    return QString();
 }
 
 DomScript *QSimpleResource::createScript(const QString &script, ScriptSource source)
@@ -203,7 +173,7 @@ void QSimpleResource::addScript(const QString &script, ScriptSource source, DomS
 }
 
 void QSimpleResource::addExtensionDataToDOM(QAbstractFormBuilder *afb,
-                                            QDesignerFormEditorInterface *core, 
+                                            QDesignerFormEditorInterface *core,
                                             DomWidget *ui_widget, QWidget *widget)
 {
     QExtensionManager *emgr = core->extensionManager();
@@ -243,7 +213,7 @@ void QSimpleResource::addExtensionDataToDOM(QAbstractFormBuilder *afb,
 }
 
 void QSimpleResource::applyExtensionDataFromDOM(QAbstractFormBuilder *afb,
-                                                QDesignerFormEditorInterface *core, 
+                                                QDesignerFormEditorInterface *core,
                                                 DomWidget *ui_widget, QWidget *widget, bool applyState)
 {
     QExtensionManager *emgr = core->extensionManager();
@@ -262,7 +232,7 @@ void QSimpleResource::applyExtensionDataFromDOM(QAbstractFormBuilder *afb,
                     const DomPropertyList properties = domData->elementProperty();
                     foreach(const DomProperty *prop, properties) {
                         const QVariant vprop = domPropertyToVariant(afb, widget->metaObject(), prop);
-                        if (vprop.type() != QVariant::Invalid) 
+                        if (vprop.type() != QVariant::Invalid)
                             data.insert(prop->attributeName(), vprop);
                     }
                 }
@@ -272,39 +242,18 @@ void QSimpleResource::applyExtensionDataFromDOM(QAbstractFormBuilder *afb,
     }
 }
 
-static const ClassNameScriptHash &customWidgetClassNameScriptHash(const QDesignerFormEditorInterface *core)
-{
-    static bool firstTime = true;
-    static ClassNameScriptHash classNameScriptHash;
-    if (firstTime) {
-        // Populate hash with scripts
-        firstTime = false;
-        foreach (const QDesignerCustomWidgetInterface *customWidget, core->pluginManager()->registeredCustomWidgets()) {
-            const QString codeTemplate = customWidget->codeTemplate();
-            if (!codeTemplate.isEmpty()) 
-                classNameScriptHash.insert(customWidget->name(), codeTemplate);
-        }
-    }
-    return classNameScriptHash;
-}
-
 QString QSimpleResource::customWidgetScript(QDesignerFormEditorInterface *core, QObject *object)
 {
     return customWidgetScript(core, qdesigner_internal::WidgetFactory::classNameOf(core, object));
 }
 
-bool QSimpleResource::hasCustomWidgetScript(QDesignerFormEditorInterface *core, QObject *object)
+bool QSimpleResource::hasCustomWidgetScript(QDesignerFormEditorInterface *, QObject *)
 {
-    return customWidgetClassNameScriptHash(core).contains(qdesigner_internal::WidgetFactory::classNameOf(core, object));
+    return false;
 }
 
-QString QSimpleResource::customWidgetScript(QDesignerFormEditorInterface *core, const QString &className)
+QString QSimpleResource::customWidgetScript(QDesignerFormEditorInterface *, const QString &)
 {
-    const ClassNameScriptHash &classNameScriptHash = customWidgetClassNameScriptHash(core);
-
-    const ClassNameScriptHash::const_iterator it = classNameScriptHash.constFind(className);
-    if (it != classNameScriptHash.constEnd())
-        return it.value();
     return QString();
 }
 
@@ -319,4 +268,18 @@ bool QSimpleResource::warningsEnabled()
 {
     return m_warningsEnabled;
 }
+
+// ------------ FormBuilderClipboard
+
+FormBuilderClipboard::FormBuilderClipboard(QWidget *w)
+{
+    m_widgets += w;
 }
+
+bool FormBuilderClipboard::empty() const
+{
+    return m_widgets.empty() && m_actions.empty();
+}
+}
+
+QT_END_NAMESPACE
