@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 #include <QtGui/QStyle>
 #include <QtGui/QPainter>
 
+QT_BEGIN_NAMESPACE
+
 namespace qdesigner_internal {
 
 SheetDelegate::SheetDelegate(QTreeView *view, QWidget *parent)
@@ -108,3 +110,5 @@ QSize SheetDelegate::sizeHint(const QStyleOptionViewItem &opt, const QModelIndex
 }
 
 } // namespace qdesigner_internal
+
+QT_END_NAMESPACE
