@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -64,6 +64,8 @@
 
 #include "formbuilderextra_p.h"
 
+QT_BEGIN_NAMESPACE
+
 #ifdef QFORMINTERNAL_NAMESPACE
 namespace QFormInternal
 {
@@ -72,7 +74,6 @@ namespace QFormInternal
 class QAbstractFormBuilder;
 class DomProperty;
 
-QDESIGNER_UILIB_EXPORT DomProperty *variantToDomProperty(const QString &propertyName, const QVariant &value, bool translateString);
 QDESIGNER_UILIB_EXPORT DomProperty *variantToDomProperty(QAbstractFormBuilder *abstractFormBuilder, QObject *object, const QString &propertyName, const QVariant &value);
 
 
@@ -150,4 +151,6 @@ inline EnumType enumKeyOfObjectToValue(const char *enumName, const char *key, co
 }
 #endif
 
+QT_END_NAMESPACE
+
 #endif // UILIBPROPERTIES_H
