@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,9 +47,12 @@
 #include <QtGui/QLayout>
 #include <QtCore/qplugin.h>
 
-Q3GroupBoxPlugin::Q3GroupBoxPlugin(QObject *parent)
+QT_BEGIN_NAMESPACE
+
+Q3GroupBoxPlugin::Q3GroupBoxPlugin(const QIcon &icon, QObject *parent)
     : QObject(parent),
-      m_initialized(false)
+      m_initialized(false),
+      m_icon(icon)
 {
 }
 
@@ -85,7 +88,7 @@ QString Q3GroupBoxPlugin::includeFile() const
 
 QIcon Q3GroupBoxPlugin::icon() const
 {
-    return QIcon();
+    return m_icon;
 }
 
 bool Q3GroupBoxPlugin::isContainer() const
@@ -113,3 +116,5 @@ void Q3GroupBoxPlugin::initialize(QDesignerFormEditorInterface *core)
     m_initialized = true;
 }
 
+
+QT_END_NAMESPACE
