@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -52,6 +52,8 @@
 
 #include <Qt3Support/Q3ToolBar>
 
+QT_BEGIN_NAMESPACE
+
 Q3MainWindowContainer::Q3MainWindowContainer(Q3MainWindow *widget, QObject *parent)
     : QObject(parent),
       m_mainWindow(widget)
@@ -126,3 +128,5 @@ QObject *Q3MainWindowContainerFactory::createExtension(QObject *object, const QS
     return 0;
 }
 
+
+QT_END_NAMESPACE
