@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -50,8 +50,10 @@
 #include <QtCore/qplugin.h>
 #include "qdesigner_q3widgetstack_p.h"
 
-Q3WidgetStackPlugin::Q3WidgetStackPlugin(QObject *parent)
-    : QObject(parent), m_initialized(false)
+QT_BEGIN_NAMESPACE
+
+Q3WidgetStackPlugin::Q3WidgetStackPlugin(const QIcon &icon, QObject *parent)
+    : QObject(parent), m_initialized(false), m_icon(icon)
 {}
 
 QString Q3WidgetStackPlugin::name() const
@@ -70,7 +72,7 @@ QString Q3WidgetStackPlugin::includeFile() const
 { return QLatin1String("q3widgetstack.h"); }
 
 QIcon Q3WidgetStackPlugin::icon() const
-{ return QIcon(); }
+{ return m_icon; }
 
 bool Q3WidgetStackPlugin::isContainer() const
 { return true; }
@@ -113,3 +115,5 @@ QString Q3WidgetStackPlugin::domXml() const
     </widget>\
     ");
 }
+
+QT_END_NAMESPACE
