@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,8 +51,10 @@
 #include <QtGui/QPushButton>
 #include <Qt3Support/Q3Wizard>
 
-Q3WizardPlugin::Q3WizardPlugin(QObject *parent)
-    : QObject(parent), m_initialized(false)
+QT_BEGIN_NAMESPACE
+
+Q3WizardPlugin::Q3WizardPlugin(const QIcon &icon, QObject *parent)
+    : QObject(parent), m_initialized(false), m_icon(icon)
 {}
 
 QString Q3WizardPlugin::name() const
@@ -51,7 +73,7 @@ QString Q3WizardPlugin::includeFile() const
 { return QLatin1String("q3wizard.h"); }
 
 QIcon Q3WizardPlugin::icon() const
-{ return QIcon(); }
+{ return m_icon; }
 
 bool Q3WizardPlugin::isContainer() const
 { return true; }
@@ -59,6 +81,7 @@ bool Q3WizardPlugin::isContainer() const
 QWidget *Q3WizardPlugin::createWidget(QWidget *parent)
 {
     Q3Wizard *wizard = new Q3Wizard(parent);
+    new Q3WizardHelper(wizard);
     wizard->backButton()->setObjectName(QLatin1String("__qt__passive_") + wizard->backButton()->objectName());
     wizard->nextButton()->setObjectName(QLatin1String("__qt__passive_") + wizard->nextButton()->objectName());
     return wizard;
@@ -76,7 +99,9 @@ void Q3WizardPlugin::initialize(QDesignerFormEditorInterface *core)
 
     m_initialized = true;
     QExtensionManager *mgr = core->extensionManager();
+    Q3WizardPropertySheetFactory::registerExtension(mgr);
     mgr->registerExtensions(new Q3WizardContainerFactory(mgr), Q_TYPEID(QDesignerContainerExtension));
+    mgr->registerExtensions(new Q3WizardExtraInfoFactory(core, mgr), Q_TYPEID(QDesignerExtraInfoExtension));
 }
 
 QString Q3WizardPlugin::codeTemplate() const
@@ -100,3 +125,5 @@ QString Q3WizardPlugin::domXml() const
     ");
 }
 
+
+QT_END_NAMESPACE
