@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,7 +48,9 @@
 #include "phrase.h"
 #include "messagemodel.h"
 
-CheckableListModel::CheckableListModel(QObject *parent) 
+QT_BEGIN_NAMESPACE
+
+CheckableListModel::CheckableListModel(QObject *parent)
 : QStandardItemModel(parent)
 {
 }
@@ -66,14 +68,14 @@ BatchTranslationDialog::BatchTranslationDialog(MessageModel *model, QWidget *w)
     connect(m_ui.cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
     connect(m_ui.moveUpButton, SIGNAL(clicked()), this, SLOT(movePhraseBookUp()));
     connect(m_ui.moveDownButton, SIGNAL(clicked()), this, SLOT(movePhraseBookDown()));
-    
+
     m_ui.phrasebookList->setModel(&m_model);
     m_ui.phrasebookList->setSelectionBehavior(QAbstractItemView::SelectItems);
     m_ui.phrasebookList->setSelectionMode(QAbstractItemView::SingleSelection);
 }
 
 
-void BatchTranslationDialog::setPhraseBooks(const QList<PhraseBook> &phrasebooks)
+void BatchTranslationDialog::setPhraseBooks(const QList<PhraseBook *> &phrasebooks)
 {
     m_model.clear();
     m_model.insertColumn(0);
@@ -81,18 +83,16 @@ void BatchTranslationDialog::setPhraseBooks(const QList<PhraseBook> &phrasebooks
     int count = phrasebooks.count();
     m_model.insertRows(0, count);
     for (int i = 0; i < count; ++i) {
-        QString name = phrasebooks[i].friendlyPhraseBookName();
-        //QString name = QLatin1String("language %1");
-        name = name.arg(i);
-        m_model.setData(m_model.index(i,0), name);
-        m_model.setData(m_model.index(i,0), Qt::Checked, Qt::CheckStateRole);
+        QString name = phrasebooks[i]->friendlyPhraseBookName();
+        m_model.setData(m_model.index(i, 0), name);
+        m_model.setData(m_model.index(i, 0), Qt::Checked, Qt::CheckStateRole);
     }
 }
 
 PhraseBook *BatchTranslationDialog::GetNamedPhraseBook(const QString &name)
 {
     for (int i = 0; i < m_phrasebooks.count(); ++i) {
-        if (m_phrasebooks[i].friendlyPhraseBookName() == name) return &m_phrasebooks[i];
+        if (m_phrasebooks[i]->friendlyPhraseBookName() == name) return m_phrasebooks[i];
     }
     return 0;
 }
@@ -122,10 +122,9 @@ void BatchTranslationDialog::startTranslation()
             if (checkState == Qt::Checked) {
                 QVariant pbname = m_model.data(m_model.index(b, 0));
                 PhraseBook *pb = GetNamedPhraseBook(pbname.toString());
-                for (int p = 0; p < pb->count(); ++p) {
-                    Phrase ph = pb->at(p);
-                    if (ph.source() == m->sourceText() && !m->finished()) {
-                        m->setTranslation(ph.target());
+                foreach (const Phrase *ph, pb->phrases()) {
+                    if (ph->source() == m->sourceText() && !m->finished()) {
+                        m->setTranslation(ph->target());
                         m->setFinished(m_ui.ckMarkFinished->isChecked());
                         ++translatedcount;
                     }
@@ -145,7 +144,7 @@ void BatchTranslationDialog::startTranslation()
     setCursor(oldCursor);
     m_messagemodel->updateAll();
     emit finished();
-    QMessageBox::information(this, tr("Linguist batch translator"), 
+    QMessageBox::information(this, tr("Linguist batch translator"),
         tr("Batch translated %n entries", "", translatedcount), QMessageBox::Ok);
 
     //### update stats
@@ -184,3 +183,4 @@ void BatchTranslationDialog::movePhraseBookDown()
     }
 }
 
+QT_END_NAMESPACE
