@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,6 +47,8 @@
 #include "ui_finddialog.h"
 #include <QDialog>
 
+QT_BEGIN_NAMESPACE
+
 class FindDialog : public QDialog, public Ui::FindDialog
 {
     Q_OBJECT
@@ -56,11 +58,14 @@ public:
     FindDialog(QWidget *parent = 0);
 
 signals:
-    void findNext(const QString& text, int where, bool matchCase);
+    void findNext(const QString& text, int where, bool matchCase, bool ignoreAccelerators);
 
 private slots:
     void emitFindNext();
     void verifyText(const QString &);
+    void find();
 };
 
+QT_END_NAMESPACE
+
 #endif
