@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 #include <QtCore/QByteArray>
 #include <QtCore/QList>
 
+QT_BEGIN_NAMESPACE
+
 struct AbstractProItemVisitor;
 
 class ProItem {
@@ -130,7 +132,7 @@ public:
     virtual bool Accept(AbstractProItemVisitor *visitor);
 private:
     VariableOperator m_variableKind;
-    QByteArray m_variable;    
+    QByteArray m_variable;
 };
 
 class ProValue : public ProItem {
@@ -210,7 +212,7 @@ public:
 
     void setModified(bool modified);
     bool isModified() const;
-    
+
     virtual bool Accept(AbstractProItemVisitor *visitor);
 
 private:
@@ -219,4 +221,6 @@ private:
     bool m_modified;
 };
 
+QT_END_NAMESPACE
+
 #endif //PROITEMS_H
