@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -49,6 +49,8 @@ const int textSimilarityThreshold = 190;
 #include <QString>
 #include <QList>
 
+QT_BEGIN_NAMESPACE
+
 class MetaTranslator;
 
 struct Candidate {
@@ -72,7 +74,7 @@ typedef QList<Candidate> CandidateList;
 struct CoMatrix;
 /**
  * This class is more efficient for searching through a large array of candidate strings, since we only
- * have to construct the CoMatrix for the \a stringToMatch once, 
+ * have to construct the CoMatrix for the \a stringToMatch once,
  * after that we just call getSimilarityScore(strCandidate).
  * \sa getSimilarityScore
  */
@@ -93,4 +95,6 @@ CandidateList similarTextHeuristicCandidates( const MetaTranslator *tor,
 					      const char *text,
 					      int maxCandidates );
 
+QT_END_NAMESPACE
+
 #endif
