@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -48,6 +48,8 @@
 #include "smallobject.h"
 #include "list.h"
 
+QT_BEGIN_NAMESPACE
+
 class AST;
 class NameAST;
 class TypeIdAST;
@@ -1593,4 +1595,6 @@ SNOC(List<InitDeclaratorAST *>, InitDeclaratorAST)
 SNOC(List<TemplateParameterAST *>, TemplateParameterAST)
 SNOC(List<StatementAST *>, StatementAST)
 
+QT_END_NAMESPACE
+
 #endif // AST_H
