@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 #include <QChar>
 #include <ctype.h>
 
+QT_BEGIN_NAMESPACE
+
 using namespace TokenEngine;
 
 CppLexer::CppLexer()
@@ -1292,3 +1294,5 @@ void CppLexer::scanOperator(int *kind)
 
     *kind = m_buffer[m_ptr++];
 }
+
+QT_END_NAMESPACE
