@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qt3to4 porting application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +45,8 @@
 #include <QFile>
 #include <QDateTime>
 
+QT_BEGIN_NAMESPACE
+
 LogEntry::LogEntry(QString type, QString location)
 :type(type), location(location)
 {}
@@ -39,10 +61,10 @@ SourcePointLogEntry::SourcePointLogEntry(QString type, QString location, QString
 
 QString SourcePointLogEntry::description() const
 {
-    return "In file "  + file +
-           " at line " + QString("%1").arg(line + 1) + //line count is zero based, adjust here.
-           " column "  + QString("%1").arg(column) +
-           ": " + text ;
+    return QLatin1String("In file ")  + file +
+           QLatin1String(" at line ") + QString(QLatin1String("%1")).arg(line + 1) + //line count is zero based, adjust here.
+           QLatin1String(" column ")  + QString(QLatin1String("%1")).arg(column) +
+           QLatin1String(": ") + text ;
 }
 
 void SourcePointLogEntry::updateLinePos(int threshold,  int delta)
@@ -106,7 +128,8 @@ QStringList Logger::fullReport()
 {
     commitSection();
     QStringList report;
-    report << "Log for qt3to4 on " + QDateTime::currentDateTime().toString() + ". Number of log entries: " + QString("%1").arg(logEntries.size());
+    report << QLatin1String("Log for qt3to4 on ") + QDateTime::currentDateTime().toString() +
+              QLatin1String(". Number of log entries: ") + QString(QLatin1String("%1")).arg(logEntries.size());
     foreach(LogEntry *logEntry, logEntries) {
         report << logEntry->description();
     }
@@ -123,3 +146,5 @@ void Logger::updateLineNumbers(int insertLine, int numLines)
         logEntry->updateLinePos(insertLine, numLines);
     }
 }
+
+QT_END_NAMESPACE
