@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -55,6 +55,8 @@
 #include <QFileInfo>
 #include <QBuffer>
 
+QT_BEGIN_NAMESPACE
+
 using namespace TokenEngine;
 
 ProjectPorter::ProjectPorter(QString basePath, QStringList includeDirectories, QStringList qt3HeadersFilenames)
@@ -410,3 +412,5 @@ void ProjectPorter::enableAttributes(const IncludeFiles &includeFiles, const QSt
     TokenAttributes *attributes = tokenContainer.tokenAttributes();
     attributes->addAttribute("CreateAttributes", "True");
 }
+
+QT_END_NAMESPACE
