@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qt3to4 porting application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,6 +60,8 @@
 #define QT_POPEN popen
 #endif
 
+QT_BEGIN_NAMESPACE
+
 QString loadFile( const QString &fileName )
 {
     QFile file( fileName );
@@ -67,32 +89,32 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
             still need to do it after we include to make sure the
             included file does not have comments
         */
-        t.replace( QRegExp(QString("#[^\n]*\n")), QString(" ") );
+        t.replace( QRegExp(QLatin1String("#[^\n]*\n")), QLatin1String(" ") );
       /*
             Strip comments, merge lines ending with backslash, add
             spaces around '=' and '+=', replace '\n' with ';', and
             simplify white spaces.
         */
-        t.replace( QRegExp(QString("#[^\n]*\n")), QString(" ") );
-        t.replace( QRegExp(QString("\\\\[^\n\\S]*\n")), QString(" ") );
-        t.replace( "=", QString(" = ") );
-        t.replace( "+ =", QString(" += ") );
-        t.replace( "\n", QString(";") );
+        t.replace( QRegExp(QLatin1String("#[^\n]*\n")), QLatin1String(" ") );
+        t.replace( QRegExp(QLatin1String("\\\\[^\n\\S]*\n")), QLatin1String(" ") );
+        t.replace( QLatin1String("="), QLatin1String(" = ") );
+        t.replace( QLatin1String("+ ="), QLatin1String(" += ") );
+        t.replace( QLatin1String("\n"), QLatin1String(";") );
         t = t.simplified();
 
         /*
             Populate tagMap with 'key = value' entries.
         */
-        QStringList lines = t.split(';');
+        QStringList lines = t.split(QLatin1Char(';'));
         QStringList::Iterator line;
         for ( line = lines.begin(); line != lines.end(); ++line ) {
-            QStringList toks = (*line).split(' ', QString::SkipEmptyParts);
+            QStringList toks = (*line).split(QLatin1Char(' '), QString::SkipEmptyParts);
 
             if ( toks.count() >= 3 &&
-                (toks[1] == QString("=") || toks[1] == QString("+=") ||
-                toks[1] == QString("*=")) ) {
+                (toks[1] == QLatin1String("=") || toks[1] == QLatin1String("+=") ||
+                toks[1] == QLatin1String("*=")) ) {
                 QString tag = toks.first();
-                int k = tag.lastIndexOf( QChar(':') ); // as in 'unix:'
+                int k = tag.lastIndexOf( QLatin1Char(':') ); // as in 'unix:'
                 if ( k != -1 )
                     tag = tag.mid( k + 1 );
                 toks.erase( toks.begin() );
@@ -101,12 +123,12 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
                 toks.erase( toks.begin() );
 
                 if ( tagMap.contains(tag) ) {
-                    if ( action == QString("=") )
-                        tagMap.insert( tag, toks.join(" ") );
+                    if ( action == QLatin1String("=") )
+                        tagMap.insert( tag, toks.join(QLatin1String(" ")) );
                     else
-                        tagMap[tag] += QChar( ' ' ) + toks.join( " " );
+                        tagMap[tag] += QLatin1Char( ' ' ) + toks.join( QLatin1String(" ") );
                 } else {
-                    tagMap[tag] = toks.join( " " );
+                    tagMap[tag] = toks.join( QLatin1String(" ") );
                 }
             }
         }
@@ -114,7 +136,7 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
             Expand $$variables within the 'value' part of a 'key = value'
             pair.
         */
-        QRegExp var( "\\$\\$[({]?([a-zA-Z0-9_]+)[)}]?" );
+        QRegExp var( QLatin1String("\\$\\$[({]?([a-zA-Z0-9_]+)[)}]?") );
         QMap<QString, QString>::Iterator it;
         for ( it = tagMap.begin(); it != tagMap.end(); ++it ) {
             int i = 0;
@@ -123,13 +145,13 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
                 QString invocation = var.cap(1);
                 QString after;
 
-                if ( invocation == "system" ) {
+                if ( invocation == QLatin1String("system") ) {
                     // skip system(); it will be handled in the next pass
                     ++i;
                 } else {
                     if ( tagMap.contains(invocation) )
                         after = tagMap[invocation];
-                    else if (invocation.toLower() == "pwd")
+                    else if (invocation.toLower() == QLatin1String("pwd"))
                         after = currentPath;
                     (*it).replace( i, len, after );
                     i += after.length();
@@ -140,7 +162,7 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
         /*
           Execute system() calls.
         */
-        QRegExp callToSystem( "\\$\\$system\\s*\\(([^()]*)\\)" );
+        QRegExp callToSystem( QLatin1String("\\$\\$system\\s*\\(([^()]*)\\)") );
         for ( it = tagMap.begin(); it != tagMap.end(); ++it ) {
             int i = 0;
             while ( (i = callToSystem.indexIn((*it), i)) != -1 ) {
@@ -161,7 +183,7 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
                             buff[i] = ' ';
                     }
                     buff[read_in] = '\0';
-                    after += buff;
+                    after += QLatin1String(buff);
                 }
                 (*it).replace( i, callToSystem.matchedLength(), after );
                 i += after.length();
@@ -169,3 +191,5 @@ QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath )
         }
     return tagMap;
 }
+
+QT_END_NAMESPACE
