@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -47,7 +47,11 @@
 #include <QMap>
 #include <QString>
 
+QT_BEGIN_NAMESPACE
+
 QMap<QString, QString> proFileTagMap( const QString& text, QString currentPath = QString() );
 QString loadFile( const QString &fileName );
 
+QT_END_NAMESPACE
+
 #endif
