@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qt3to4 porting application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,6 +44,8 @@
 #include "qtsimplexml.h"
 #include <QDomDocument>
 
+QT_BEGIN_NAMESPACE
+
 QtSimpleXml::QtSimpleXml(const QString &name)
 {
     valid = false;
@@ -44,7 +66,7 @@ QtSimpleXml &QtSimpleXml::operator [](int index)
     }
 
     if (index == children.size()) {
-        QtSimpleXml *item = new QtSimpleXml("item");
+        QtSimpleXml *item = new QtSimpleXml(QLatin1String("item"));
         item->parent = this;
         children.insert(item->name(), item);
         return *item;
@@ -83,8 +105,8 @@ QtSimpleXml &QtSimpleXml::operator =(const QString &text)
 QDomDocument QtSimpleXml::toDomDocument() const
 {
     QDomDocument doc;
-    QString data = "version=\"1.0\" encoding=\"UTF-8\"";
-    doc.appendChild(doc.createProcessingInstruction("xml", data));
+    QString data = QLatin1String("version=\"1.0\" encoding=\"UTF-8\"");
+    doc.appendChild(doc.createProcessingInstruction(QLatin1String("xml"), data));
 
     if (!valid)
 	return doc;
@@ -169,7 +191,7 @@ bool QtSimpleXml::setContent(const QString &content)
 
     if (!doc.setContent(content, false, &errorMsg, &errorLine, &errorColumn)) {
         errorStr = errorMsg;
-        errorStr += " at " + QString::number(errorLine) + ":" + QString::number(errorColumn);
+        errorStr += QLatin1String(" at ") + QString::number(errorLine) + QLatin1String(":") + QString::number(errorColumn);
         return false;
     }
 
@@ -185,7 +207,7 @@ bool QtSimpleXml::setContent(QIODevice *device)
     int errorColumn;
     if (!doc.setContent(device, false, &errorMsg, &errorLine, &errorColumn)) {
         errorStr = errorMsg;
-        errorStr += " at " + QString::number(errorLine) + ":" + QString::number(errorColumn);
+        errorStr += QLatin1String(" at ") + QString::number(errorLine) + QLatin1String(":") + QString::number(errorColumn);
         return false;
     }
 
@@ -254,3 +276,5 @@ QString QtSimpleXml::errorString() const
 {
     return errorStr;
 }
+
+QT_END_NAMESPACE
