@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 #include <QChar>
 #include <QtDebug>
 
+QT_BEGIN_NAMESPACE
+
 using namespace TokenEngine;
 namespace Rpp {
 
@@ -550,3 +552,5 @@ TokenSection MacroFunctionParser::argument(int argumentIndex)
 }
 
 } //namespace Rpp
+
+QT_END_NAMESPACE
