@@ -1,21 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 ** Copyright (C) 2001-2004 Roberto Raggi
 **
 ** This file is part of the qt3to4 porting application of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +49,8 @@
 #include <QString>
 #include <QRegExp>
 
+QT_BEGIN_NAMESPACE
+
 using namespace TokenStreamAdapter;
 using namespace TokenEngine;
 using namespace CodeModel;
@@ -1057,7 +1079,7 @@ QByteArray Semantic::typeSpecToString(TypeSpecifierAST* typeSpec)
         tp += "const ";
     }
 
-    tp += (QString(textOf(typeSpec)).replace(QRegExp(" :: "), QString::fromUtf8("::"))).toLatin1();
+    tp += (QString::fromLatin1(textOf(typeSpec)).replace(QRegExp(QLatin1String(" :: ")), QString::fromUtf8("::"))).toLatin1();
     return tp;
 }
 
@@ -1073,13 +1095,13 @@ QByteArray Semantic::declaratorToString(DeclaratorAST* declarator, const QByteAr
         foreach (AST *current, ptrOpList) {
             text += textOf(current);
         }
-        text += " ";
+        text += QByteArray(" ");
     }
 
     text += scope;
 
     if (declarator->subDeclarator())
-        text += "(" + declaratorToString(declarator->subDeclarator()) + ")";
+        text += QByteArray("(") + declaratorToString(declarator->subDeclarator()) + QByteArray(")");
 
     if (declarator->declaratorId())
         text += textOf(declarator->declaratorId());
@@ -1088,12 +1110,12 @@ QByteArray Semantic::declaratorToString(DeclaratorAST* declarator, const QByteAr
         List<AST*> arrays = *declarator->arrayDimensionList();
         foreach (AST *current, arrays) {
             current=current;    //silence unused symbol warning
-            text += "[]";
+            text += QByteArray("[]");
         }
     }
 
     if (declarator->parameterDeclarationClause()){
-        text += "(";
+        text += QByteArray("(");
 
         ParameterDeclarationListAST* l = declarator->parameterDeclarationClause()->parameterDeclarationList();
         if (l != 0){
@@ -1102,21 +1124,21 @@ QByteArray Semantic::declaratorToString(DeclaratorAST* declarator, const QByteAr
                 QByteArray type = typeSpecToString(current->typeSpec());
                 text += type;
                 if (!type.isEmpty())
-                    text += " ";
+                    text += QByteArray(" ");
                 text += declaratorToString(current->declarator());
 
                 // ### FIXME if (it.current())
-                    text += ", ";
+                    text += QByteArray(", ");
             }
         }
 
-        text += ")";
+        text += QByteArray(")");
 
         if (declarator->constant() != 0)
-            text += " const";
+            text += QByteArray(" const");
     }
 
-    return QString(text).replace(QRegExp(" :: "), "::").simplified().toLatin1();
+    return QString::fromLatin1(text).replace(QRegExp(QLatin1String(" :: ")), QLatin1String("::")).simplified().toLatin1();
 }
 
 QByteArray Semantic::textOf(const AST *node) const
@@ -1127,7 +1149,7 @@ QByteArray Semantic::textOf(const AST *node) const
     for (int i = node->startToken(); i < node->endToken(); ++i) {
         if (!m_tokenStream->isHidden(i)) {
             if (i != node->startToken())
-                text += " ";
+                text += QByteArray(" ");
             text += m_tokenStream->tokenText(i);
         }
     }
@@ -1203,3 +1225,5 @@ TokenEngine::TokenRef Semantic::tokenRefFromAST(AST *node)
     const int containerIndex = m_tokenStream->containerIndex(startTokenIndex);
     return TokenEngine::TokenRef(tokenContainer, containerIndex);
 }
+
+QT_END_NAMESPACE
