@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -54,6 +54,8 @@
 #include "semantic.h"
 #include <QSharedData>
 
+QT_BEGIN_NAMESPACE
+
 class TranslationUnitData : public QSharedData
 {
 public:
@@ -88,4 +90,6 @@ private:
     Parser parser;
 };
 
+QT_END_NAMESPACE
+
 #endif
