@@ -14,7 +14,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -44,6 +44,8 @@
 
 #include "treewalker.h"
 
+QT_BEGIN_NAMESPACE
+
 /*
 template <class T>
 inline void parseAll(TreeWalker *w, const List<T *> *l)
@@ -453,3 +455,5 @@ void TreeWalker::parseReturnStatement(ReturnStatementAST *node)
 {
     parseNode(node->expression());
 }
+
+QT_END_NAMESPACE
