@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 
 #include <QAbstractButton>
 
+QT_BEGIN_NAMESPACE
+
 class ColorButton : public QAbstractButton
 {
     Q_OBJECT
@@ -85,4 +87,6 @@ private:
     bool mousepressed;
 };
 
+QT_END_NAMESPACE
+
 #endif // COLORBUTTON_H
