@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,8 @@
 
 #include <QWidget>
 
+QT_BEGIN_NAMESPACE
+
 class GammaView: public QWidget
 {
     Q_OBJECT
@@ -54,4 +56,6 @@ public:
         : QWidget(parent,f) { }
 };
 
+QT_END_NAMESPACE
+
 #endif
