@@ -1,20 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
+**
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -23,11 +43,15 @@
 
 #include "qvfb.h"
 #include "qvfbview.h"
+#ifdef Q_WS_X11
+#include "qvfbx11view.h"
+#endif
 #include "qvfbratedlg.h"
 #include "ui_config.h"
-#include "skin.h"
 #include "qanimationwriter.h"
 
+#include <deviceskin.h>
+
 #include <QMenuBar>
 #include <QMenu>
 #include <QApplication>
@@ -49,11 +73,15 @@
 #include <QPushButton>
 #include <QTextStream>
 #include <QFile>
+#include <QFileInfo>
+#include <QDebug>
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
+QT_BEGIN_NAMESPACE
+
 // =====================================================================
 
 static const char *red_on_led_xpm[] = {
@@ -106,12 +134,26 @@ static const char *red_off_led_xpm[] = {
 "   ;==&$   ",
 "           "};
 
+static bool copyButtonConfiguration(const QString &prefix, int displayId)
+{
+    const QString destDir = QString(QLatin1String("/tmp/qtembedded-%1/")).arg(displayId);
+    const QFileInfo src(prefix + QLatin1String("defaultbuttons.conf"));
+    const QFileInfo dst(destDir + QLatin1String("defaultbuttons.conf"));
+    unlink(dst.absoluteFilePath().toLatin1().constData());
+    if (!src.exists())
+        return false;
+    const bool rc = QFile::copy(src.absoluteFilePath(), dst.absoluteFilePath());
+    if (!rc)
+        qWarning() << "Failed to copy the button configuration file " << src.absoluteFilePath() << " to " <<  dst.absoluteFilePath() << '.';
+    return rc;
+}
+
 // =====================================================================
 
 class AnimationSaveWidget : public QWidget {
     Q_OBJECT
 public:
-    AnimationSaveWidget(QVFbView *v);
+    AnimationSaveWidget(QVFbAbstractView *v);
     ~AnimationSaveWidget();
     bool detectPpmtoMpegCommand();
     void timerEvent(QTimerEvent *te);
@@ -122,7 +164,7 @@ protected slots:
     void reset();
     void save();
 private:
-    QVFbView *view;
+    QVFbAbstractView *view;
     QProgressBar *progressBar;
     QLabel *statusText;
     bool haveMpeg, savingAsMpeg, recording;
@@ -162,10 +204,12 @@ void Zoomer::zoom(int z)
 
 // =====================================================================
 
-QVFb::QVFb( int display_id, int w, int h, int d, int r, const QString &skin, QWidget *parent, Qt::WFlags flags )
+QVFb::QVFb( int display_id, int w, int h, int d, int r, const QString &skin, DisplayType displayType, QWidget *parent, Qt::WindowFlags flags )
     : QMainWindow( parent, flags )
 {
+    this->displayType = displayType;
     view = 0;
+    secondaryView = 0;
     scroller = 0;
     this->skin = 0;
     currentSkinIndex = -1;
@@ -202,6 +246,8 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
 {
     delete view;
     view = 0;
+    delete secondaryView;
+    secondaryView = 0;
     delete scroller;
     scroller = 0;
     delete skin;
@@ -213,19 +259,40 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
 			     ((r == 270) ? QVFbView::Rot270 :
 					   QVFbView::Rot0 )));
     if ( !skin_name.isEmpty() ) {
-	bool vis = isVisible();
-	int sw, sh;
-	skin = new Skin( this, skin_name, sw, sh );
-	if (skin && skin->isValid()){
+	const bool vis = isVisible();
+	DeviceSkinParameters parameters;
+	QString readError;
+	if (parameters.read(skin_name,DeviceSkinParameters::ReadAll, &readError)) {
+	    skin = new DeviceSkin(parameters, this);
+	    connect(skin, SIGNAL(popupMenu()), this, SLOT(popupMenu()));
+	    const int sw = parameters.screenSize().width();
+	    const int sh = parameters.screenSize().height();
+	    const int sd = parameters.screenDepth;
             if ( !pw ) pw = sw;
             if ( !ph ) ph = sh;
+            if ( d < 0 )
+                if ( sd )
+                    d = sd;
+                else
+                    d = -d;
     	    if ( vis ) hide();
     	    menuBar()->hide();
 	    scroller = 0;
-	    view = new QVFbView( display_id, pw, ph, d, rot, skin );
+#ifdef Q_WS_X11
+	    if (displayType == X11)
+		view = new QVFbX11View( display_id, pw, ph, d, rot, skin );
+	    else
+#endif
+		view = new QVFbView( display_id, pw, ph, d, rot, skin );
 	    skin->setView( view );
 	    view->setContentsMargins( 0, 0, 0, 0 );
-	    view->setFixedSize( sw, sh );
+	    view->setFixedSize( sw, sh);
+            view->setTouchscreenEmulation(!parameters.hasMouseHover);
+	    connect(skin, SIGNAL(skinKeyPressEvent(int,QString,bool)), view, SLOT(skinKeyPressEvent(int,QString,bool)));
+	    connect(skin, SIGNAL(skinKeyReleaseEvent(int,QString,bool)), view, SLOT(skinKeyReleaseEvent(int,QString,bool)));
+
+	    copyButtonConfiguration(skin->prefix(), view->displayId());
+
 	    setCentralWidget( skin );
 	    adjustSize();
 	    skinscaleH = (double)sw/pw;
@@ -233,10 +300,23 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
 	    if ( skinscaleH != 1.0 || skinscaleH != 1.0 )
 		setZoom(skinscaleH);
 	    view->show();
+
+            if (parameters.hasSecondaryScreen()) {
+                const QSize ssize = parameters.secondaryScreenSize();
+                // assumes same depth and rotation
+#ifdef Q_WS_X11
+		if (displayType == X11)
+		    secondaryView = new QVFbX11View( display_id+1, ssize.width(), ssize.height(), d, rot, skin );
+		else
+#endif
+		    secondaryView = new QVFbView( display_id+1, ssize.width(), ssize.height(), d, rot, skin );
+                skin->setSecondaryView(secondaryView);
+                secondaryView->show();
+            }
+
 	    if ( vis ) show();
 	} else {
-	    delete skin;
-	    skin = 0;
+	    qWarning(readError.toUtf8().constData());
 	}
     }
 
@@ -244,11 +324,17 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
     //	    with one then fallback to a framebuffer without
     //	    a skin
     if (!skin){
-	// Default size
-	if ( !pw ) pw = 240;
-	if ( !ph ) ph = 320;
+	// Default values
+	if (!pw)
+            pw = 240;
+	if (!ph)
+            ph = 320;
+        if (!d)
+            d = 32;
+        else if (d < 0)
+            d = -d;
 
-     	if ( currentSkinIndex!=-1 ) {
+     	if (currentSkinIndex != -1) {
 	    clearMask();
             setParent( 0, 0 );
             move( pos() );
@@ -259,7 +345,12 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
 	}
 	menuBar()->show();
 	scroller = new QScrollArea(this);
-	view = new QVFbView( display_id, pw, ph, d, rot, scroller );
+#ifdef Q_WS_X11
+	if (displayType == X11)
+	    view = new QVFbX11View( display_id, pw, ph, d, rot, scroller );
+	else
+#endif
+	    view = new QVFbView( display_id, pw, ph, d, rot, scroller );
 	scroller->setWidget(view);
 	view->setContentsMargins( 0, 0, 0, 0 );
 	setCentralWidget(scroller);
@@ -269,8 +360,13 @@ void QVFb::init( int display_id, int pw, int ph, int d, int r, const QString& sk
 	scroller->show();
 	// delete defaultbuttons.conf if it was left behind...
 	unlink(QFileInfo(QString("/tmp/qtembedded-%1/defaultbuttons.conf").arg(view->displayId())).absoluteFilePath().toLatin1().constData());
+        if (secondaryView)
+            unlink(QFileInfo(QString("/tmp/qtembedded-%1/defaultbuttons.conf").arg(view->displayId()+1)).absoluteFilePath().toLatin1().constData());
     }
     view->setRate(refreshRate);
+    if (secondaryView) {
+        secondaryView->setRate(refreshRate);
+    }
     // Resize QVFb to the new size
     QSize newSize = view->sizeHint();
 
@@ -288,8 +384,12 @@ void QVFb::enableCursor( bool e )
 {
     if ( skin && skin->hasCursor() ) {
 	view->setCursor( Qt::BlankCursor );
+        if (secondaryView)
+            secondaryView->setCursor( Qt::BlankCursor );
     } else {
 	view->setCursor( e ? Qt::ArrowCursor : Qt::BlankCursor );
+        if (secondaryView)
+            secondaryView->setCursor( e ? Qt::ArrowCursor : Qt::BlankCursor );
     }
     cursorAction->setChecked( e );
 }
@@ -305,11 +405,11 @@ void QVFb::createMenu(T *menu)
 
 QMenu* QVFb::createFileMenu()
 {
-    QMenu *file = new QMenu( "&File", this );
-    file->addAction( "&Configure...", this, SLOT(configure()), Qt::ALT+Qt::CTRL+Qt::Key_C );
+    QMenu *file = new QMenu( "File", this );
+    file->addAction( "Configure...", this, SLOT(configure()), 0 );
     file->addSeparator();
-    file->addAction( "&Save image...", this, SLOT(saveImage()), Qt::ALT+Qt::CTRL+Qt::Key_S );
-    file->addAction( "&Animation...", this, SLOT(toggleAnimation()), Qt::ALT+Qt::CTRL+Qt::Key_A );
+    file->addAction( "&Save image...", this, SLOT(saveImage()), 0 );
+    file->addAction( "&Animation...", this, SLOT(toggleAnimation()), 0 );
     file->addSeparator();
     file->addAction( "&Quit", qApp, SLOT(quit()) );
     return file;
@@ -317,7 +417,7 @@ QMenu* QVFb::createFileMenu()
 
 QMenu* QVFb::createViewMenu()
 {
-    viewMenu = new QMenu( "&View", this );
+    viewMenu = new QMenu( "View", this );
     cursorAction = viewMenu->addAction( "Show &Cursor", this,
                                         SLOT(toggleCursor()) );
     cursorAction->setCheckable(true);
@@ -339,7 +439,7 @@ QMenu* QVFb::createViewMenu()
 
 QMenu* QVFb::createHelpMenu()
 {
-    QMenu *help = new QMenu( "&Help", this );
+    QMenu *help = new QMenu( "Help", this );
     help->addAction("About...", this, SLOT(about()));
     return help;
 }
@@ -347,11 +447,18 @@ QMenu* QVFb::createHelpMenu()
 void QVFb::setZoom(double z)
 {
     view->setZoom(z,z*skinscaleV/skinscaleH);
+    if (secondaryView)
+        secondaryView->setZoom(z,z*skinscaleV/skinscaleH);
+
     if (skin) {
 	skin->setZoom(z/skinscaleH);
 	view->setFixedSize(
 	    int(view->displayWidth()*z),
 	    int(view->displayHeight()*z*skinscaleV/skinscaleH));
+        if (secondaryView)
+            secondaryView->setFixedSize(
+                    int(secondaryView->displayWidth()*z),
+                    int(secondaryView->displayHeight()*z*skinscaleV/skinscaleH));
     }
 }
 
@@ -395,9 +502,15 @@ void QVFb::setZoom4()
 void QVFb::saveImage()
 {
     QImage img = view->image();
-    QString filename = QFileDialog::getSaveFileName(this, "Save image", "snapshot.png", "Portable Network Graphics (*.png)");
+    QString filename = QFileDialog::getSaveFileName(this, "Save Main Screen image", "snapshot.png", "Portable Network Graphics (*.png)");
     if (!filename.isEmpty())
         img.save(filename,"PNG");
+    if (secondaryView) {
+        QImage img = view->image();
+        QString filename = QFileDialog::getSaveFileName(this, "Save Second Screen image", "snapshot.png", "Portable Network Graphics (*.png)");
+        if (!filename.isEmpty())
+            img.save(filename,"PNG");
+    }
 }
 
 void QVFb::toggleAnimation()
@@ -430,6 +543,8 @@ void QVFb::setRate(int i)
 {
     refreshRate = i;
     view->setRate(i);
+    if (secondaryView)
+        secondaryView->setRate(i);
 }
 
 
@@ -451,7 +566,7 @@ void QVFb::findSkins(const QString &currentSkin)
 {
     skinnames.clear();
     skinfiles.clear();
-    QDir dir(".","*.skin");
+    QDir dir(":/skins/","*.skin");
     const QFileInfoList l = dir.entryInfoList();
     int i = 1; // "None" is already in list at index 0
     for (QFileInfoList::const_iterator it = l.begin(); it != l.end(); ++it) {
@@ -490,14 +605,10 @@ void QVFb::configure()
     config->skin->insertItems(config->skin->count(), skinnames);
     if (currentSkinIndex > 0)
 	config->skin->setCurrentIndex(currentSkinIndex);
+    config->skin->addItem(tr("Browse..."));
     config->touchScreen->setChecked(view->touchScreenEmulation());
     config->lcdScreen->setChecked(view->lcdScreenEmulation());
-    config->depth_1->setChecked(view->displayDepth()==1);
-    config->depth_4gray->setChecked(view->displayDepth()==4);
-    config->depth_8->setChecked(view->displayDepth()==8);
-    config->depth_12->setChecked(view->displayDepth()==12);
-    config->depth_16->setChecked(view->displayDepth()==16);
-    config->depth_32->setChecked(view->displayDepth()==32);
+    chooseDepth(view->displayDepth(), view->displayFormat());
     connect(config->skin, SIGNAL(activated(int)), this, SLOT(skinConfigChosen(int)));
     if ( view->gammaRed() == view->gammaGreen() && view->gammaGreen() == view->gammaBlue() ) {
 	config->gammaslider->setValue(int(view->gammaRed()*400));
@@ -530,6 +641,10 @@ void QVFb::configure()
 	    w=320; h=240;
 	} else if ( config->size_640_480->isChecked() ) {
 	    w=640; h=480;
+	} else if ( config->size_800_600->isChecked() ) {
+	    w=800; h=600;
+	} else if ( config->size_1024_768->isChecked() ) {
+	    w=1024; h=768;
 	} else {
 	    w=config->size_width->value();
 	    h=config->size_height->value();
@@ -543,10 +658,18 @@ void QVFb::configure()
 	    d=8;
 	else if ( config->depth_12->isChecked() )
 	    d=12;
+	else if ( config->depth_15->isChecked() )
+	    d = 15;
 	else if ( config->depth_16->isChecked() )
 	    d=16;
+	else if ( config->depth_18->isChecked() )
+	    d=18;
+	else if ( config->depth_24->isChecked() )
+	    d=24;
 	else
 	    d=32;
+	QVFbView::PixelFormat displayFormat = config->depth_32_argb->isChecked()
+					      ? QVFbView::ARGBFormat : QVFbView::DefaultFormat;
 	int skinIndex = config->skin->currentIndex();
 	if ( w != view->displayWidth() || h != view->displayHeight()
 		|| d != view->displayDepth() || skinIndex != currentSkinIndex ) {
@@ -557,6 +680,7 @@ void QVFb::configure()
 	    currentSkinIndex = skinIndex;
 	    init( id, w, h, d, r, skinIndex > 0 ? skinfiles[skinIndex-1] : QString::null );
 	}
+	view->setViewFormat(displayFormat);
 	view->setTouchscreenEmulation( config->touchScreen->isChecked() );
 	bool lcdEmulation = config->lcdScreen->isChecked();
 	view->setLcdScreenEmulation( lcdEmulation );
@@ -584,12 +708,52 @@ void QVFb::chooseSize(const QSize& sz)
     config->size_240_320->setChecked(sz == QSize(240,320));
     config->size_320_240->setChecked(sz == QSize(320,240));
     config->size_640_480->setChecked(sz == QSize(640,480));
+    config->size_800_600->setChecked(sz == QSize(800,600));
+    config->size_1024_768->setChecked(sz == QSize(1024,768));
+}
+
+void QVFb::chooseDepth(int depth, QVFbView::PixelFormat displayFormat)
+{
+    config->depth_1->setChecked(depth==1);
+    config->depth_4gray->setChecked(depth==4);
+    config->depth_8->setChecked(depth==8);
+    config->depth_12->setChecked(depth==12);
+    config->depth_15->setChecked(depth==15);
+    config->depth_16->setChecked(depth==16);
+    config->depth_18->setChecked(depth==18);
+    config->depth_24->setChecked(depth==24);
+    config->depth_32->setChecked(depth==32 && displayFormat != QVFbView::ARGBFormat);
+    config->depth_32_argb->setChecked(depth==32 && displayFormat == QVFbView::ARGBFormat);
 }
 
 void QVFb::skinConfigChosen(int i)
 {
+    if (i == config->skin->count() - 1) { // Browse... ?
+        QFileDialog dlg(this);
+        dlg.setFileMode(QFileDialog::DirectoryOnly);
+        dlg.setWindowTitle(tr("Load Custom Skin..."));
+        dlg.setFilter(tr("All QVFB Skins (*.skin)"));
+        dlg.setDirectory(QDir::current());
+        if (dlg.exec() && dlg.selectedFiles().count() == 1) {
+            skinfiles.append(dlg.selectedFiles().first());
+            i = skinfiles.count();
+            config->skin->insertItem(i, QFileInfo(skinfiles.last()).baseName());
+            config->skin->setCurrentIndex(i);
+        } else {
+            i = 0;
+        }
+    }
     if ( i ) {
-	chooseSize(Skin::screenSize(skinfiles[i-1]));
+	DeviceSkinParameters parameters;
+	QString readError;
+	if (parameters.read(skinfiles[i-1], DeviceSkinParameters::ReadSizeOnly, &readError)) {
+	    chooseSize(parameters.screenSize());
+            if (parameters.screenDepth)
+                chooseDepth(parameters.screenDepth,QVFbView::ARGBFormat);
+            config->touchScreen->setChecked(!parameters.hasMouseHover);
+	} else {
+	    qWarning(readError.toUtf8().constData());
+	}
     }
 }
 
@@ -644,7 +808,7 @@ QSize QVFb::sizeHint() const
 
 // =====================================================================
 
-AnimationSaveWidget::AnimationSaveWidget(QVFbView *v) :
+AnimationSaveWidget::AnimationSaveWidget(QVFbAbstractView *v) :
 	QWidget((QWidget*)0,0),
 	view(v), recording(false), animation(0),
 	timerId(-1), progressTimerId(-1),
@@ -829,6 +993,7 @@ void AnimationSaveWidget::convertToMpeg(QString filename)
     file.close();
 
     // ### can't use QProcess, not in Qt 2.3
+    // ### but it's certainly in Qt 4! use it?
     // Execute the ppmtompeg command as a seperate process to do the encoding
     pid_t pid = ::fork();
     if ( !pid ) {
@@ -917,15 +1082,22 @@ void AnimationSaveWidget::save()
 	    convertToMpeg(filename);
     } else {
 	filename = QFileDialog::getSaveFileName(this, tr("Save animation..."), "", "*.mng");
-	if ( !filename.isNull() ) {
-	    // This might be the simplest posix way to move a file.
-	    // For windows, might need to do something else
-	    ::link("/tmp/qvfb_tmp_animation.mng", filename.toLatin1().constData());
-	    ::unlink("/tmp/qvfb_tmp_animation.mng");
-	    statusText->setText( tr("Finished saving."));
-	    reset();
+	if (filename.isNull()) {
+            statusText->setText(tr("Save canceled."));
+        } else {
+            QFile::remove(filename);
+            bool success = QFile::rename(QLatin1String("/tmp/qvfb_tmp_animation.mng"),
+                                         filename);
+            if (success) {
+                statusText->setText(tr("Finished saving."));
+                reset();
+            } else {
+                statusText->setText(tr("Save failed!"));
+            }
 	}
     }
 }
 
+QT_END_NAMESPACE
+
 #include "qvfb.moc"
