@@ -1,35 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
-** License version 2.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of
-** this file.  Please review the following information to ensure GNU
-** General Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/
+** License versions 2.0 or 3.0 as published by the Free Software
+** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file.  Alternatively you may (at
+** your option) use any later version of the GNU General Public
+** License if such license has been publicly approved by Trolltech ASA
+** (or its successors, if any) and the KDE Free Qt Foundation. In
+** addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.2, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
 **
-** If you are unsure which license is appropriate for your use, please
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+** you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 ** or contact the sales department at sales@trolltech.com.
 **
-** In addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.0, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** In addition, as a special exception, Trolltech, as the sole
+** copyright holder for Qt Designer, grants users of the Qt/Eclipse
+** Integration plug-in the right for the Qt/Eclipse Integration to
+** link to functionality provided by Qt Designer and its related
+** libraries.
 **
-** In addition, as a special exception, Trolltech, as the sole copyright
-** holder for Qt Designer, grants users of the Qt/Eclipse Integration
-** plug-in the right for the Qt/Eclipse Integration to link to
-** functionality provided by Qt Designer and its related libraries.
-**
-** Trolltech reserves all rights not expressly granted herein.
-** 
-** Trolltech ASA (c) 2007
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+** granted herein.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,14 +46,17 @@
 
 #include <QMainWindow>
 #include <QStringList>
+#include "qvfbview.h"
 
-class QVFbView;
+QT_BEGIN_NAMESPACE
+
+class QVFbAbstractView;
 class QVFbRateDialog;
 class QPopupMenu;
 class QMenuData;
 class QAction;
 class Config;
-class Skin;
+class DeviceSkin;
 class QVFb;
 class QLabel;
 class QMenu;
@@ -71,14 +79,18 @@ class QVFb: public QMainWindow
 {
     Q_OBJECT
 public:
-    QVFb(int display_id, int w, int h, int d, int r, const QString &skin, QWidget *parent = 0, Qt::WindowFlags wflags = 0);
+    enum DisplayType { QWS, X11 };
+
+    QVFb( int display_id, int w, int h, int d, int r, const QString &skin, DisplayType displayType, QWidget *parent = 0, Qt::WindowFlags wflags = 0 );
     ~QVFb();
 
     void enableCursor( bool e );
-    void popupMenu();
 
     QSize sizeHint() const;
 
+public slots:
+     void popupMenu();
+
 protected slots:
     void saveImage();
     void toggleAnimation();
@@ -90,6 +102,7 @@ protected slots:
     void configure();
     void skinConfigChosen(int i);
     void chooseSize(const QSize& sz);
+    void chooseDepth(int depth, QVFbView::PixelFormat displayFormat);
 
     void setZoom1();
     void setZoom2();
@@ -112,11 +125,11 @@ protected:
 
 private:
     void findSkins(const QString &currentSkin);
-    void init(int display_id, int w, int h, int d, int r, const QString& skin);
-    Skin *skin;
+    void init( int display_id, int w, int h, int d, int r, const QString& skin );
+    DeviceSkin *skin;
     double skinscaleH,skinscaleV;
-    QVFbView *view;
-    QVFbView *secondaryView;
+    QVFbAbstractView *view;
+    QVFbAbstractView *secondaryView;
     QVFbRateDialog *rateDlg;
     QMenu *viewMenu;
     QAction *cursorAction;
@@ -126,6 +139,7 @@ private:
     int currentSkinIndex;
     Zoomer *zoomer;
     QScrollArea* scroller;
+    DisplayType displayType;
 
     int refreshRate;
 private slots:
@@ -136,4 +150,6 @@ private slots:
     void updateGammaLabels();
 };
 
+QT_END_NAMESPACE
+
 #endif
