@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -46,6 +46,9 @@
 #include <QImage>
 #include <QVector>
 #include <QColor>
+
+QT_BEGIN_NAMESPACE
+
 class QVFbKeyProtocol;
 class QVFbMouseProtocol;
 class QVFbViewProtocol : public QObject
@@ -69,6 +72,7 @@ public:
     virtual int  numcols() const = 0;
     virtual QVector<QRgb> clut() const = 0;
     virtual unsigned char *data() const = 0;
+    virtual int brightness() const = 0;
 
     virtual void setRate(int) {}
 public slots:
@@ -164,4 +168,7 @@ protected:
     QPoint lastPos;
     QTimer *repeater;
 };
+
+QT_END_NAMESPACE
+
 #endif
