@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -45,6 +45,8 @@
 
 #include "qvfbprotocol.h"
 
+QT_BEGIN_NAMESPACE
+
 class QVFbHeader;
 class QTimer;
 class QShMemViewProtocol : public QVFbViewProtocol
@@ -60,6 +62,7 @@ public:
     int  numcols() const;
     QVector<QRgb> clut() const;
     unsigned char *data() const;
+    int brightness() const;
 
     void setRate(int);
     int rate() const;
@@ -82,4 +85,6 @@ private:
     QTimer *mRefreshTimer;
 };
 
+QT_END_NAMESPACE
+
 #endif
