@@ -13,7 +13,7 @@
 ** (or its successors, if any) and the KDE Free Qt Foundation. In
 ** addition, as a special exception, Trolltech gives you certain
 ** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.1, which can be found at
+** Exception version 1.2, which can be found at
 ** http://www.trolltech.com/products/qt/gplexception/ and in the file
 ** GPL_EXCEPTION.txt in this package.
 **
@@ -51,13 +51,15 @@
 //#include <QGLWidget>
 //#endif
 
+QT_BEGIN_NAMESPACE
+
 class QImage;
 class QTimer;
 class QAnimationWriter;
 struct QVFbHeader;
 class QVFbViewProtocol;
 
-class QVFbView :
+class QVFbAbstractView :
 #ifdef QVFB_USE_GLWIDGET
     public QGLWidget
 #else
@@ -67,6 +69,48 @@ class QVFbView :
     Q_OBJECT
 public:
     enum Rotation { Rot0, Rot90, Rot180, Rot270 };
+    enum PixelFormat { DefaultFormat, GrayscaleFormat, RGBFormat, ARGBFormat };
+    QVFbAbstractView( QWidget *parent = 0);
+    virtual ~QVFbAbstractView();
+
+    virtual int displayId() const = 0;
+    virtual int displayWidth() const = 0;
+    virtual int displayHeight() const = 0;
+    virtual int displayDepth() const = 0;
+    virtual PixelFormat displayFormat() const { return DefaultFormat; }
+    virtual Rotation displayRotation() const = 0;
+
+    virtual void setGamma(double gr, double gg, double gb) = 0;
+    virtual double gammaRed() const = 0;
+    virtual double gammaGreen() const = 0;
+    virtual double gammaBlue() const = 0;
+    virtual void getGamma(int i, QRgb& rgb) = 0;
+
+    virtual bool touchScreenEmulation() const = 0;
+    virtual bool lcdScreenEmulation() const = 0;
+    virtual int rate() = 0;
+    virtual bool animating() const = 0;
+    virtual QImage image() const = 0;
+    virtual void setRate(int) = 0;
+
+    virtual double zoomH() const = 0;
+    virtual double zoomV() const = 0;
+
+public slots:
+    virtual void setTouchscreenEmulation( bool ) = 0;
+    virtual void setLcdScreenEmulation( bool ) = 0;
+    virtual void setZoom( double, double ) = 0;
+    virtual void startAnimation( const QString& ) = 0;
+    virtual void stopAnimation() = 0;
+    virtual void skinKeyPressEvent( int code, const QString& text, bool autorep=FALSE ) = 0;
+    virtual void skinKeyReleaseEvent( int code, const QString& text, bool autorep=FALSE ) = 0;
+    virtual void setViewFormat(PixelFormat) {}
+};
+
+class QVFbView : public QVFbAbstractView
+{
+    Q_OBJECT
+public:
     QVFbView(int id, int w, int h, int d, Rotation r, QWidget *parent = 0);
     virtual ~QVFbView();
 
@@ -74,6 +118,7 @@ public:
     int displayWidth() const;
     int displayHeight() const;
     int displayDepth() const;
+    PixelFormat displayFormat() const;
     Rotation displayRotation() const;
 
     bool touchScreenEmulation() const { return emulateTouchscreen; }
@@ -87,8 +132,6 @@ public:
     double gammaGreen() const { return ggreen; }
     double gammaBlue() const { return gblue; }
     void getGamma(int i, QRgb& rgb);
-    void skinKeyPressEvent(int code, const QString& text, bool autorep=FALSE);
-    void skinKeyReleaseEvent(int code, const QString& text, bool autorep=FALSE);
     void skinMouseEvent(QMouseEvent *e);
 
     double zoomH() const { return hzm; }
@@ -103,6 +146,9 @@ public slots:
     void setZoom(double, double);
     void startAnimation(const QString&);
     void stopAnimation();
+    void skinKeyPressEvent(int code, const QString& text, bool autorep=FALSE);
+    void skinKeyReleaseEvent(int code, const QString& text, bool autorep=FALSE);
+    void setViewFormat(PixelFormat);
 
 protected slots:
     void refreshDisplay(const QRect &);
@@ -123,10 +169,12 @@ protected:
     virtual void wheelEvent(QWheelEvent *e);
     virtual void keyPressEvent(QKeyEvent *e);
     virtual void keyReleaseEvent(QKeyEvent *e);
+    virtual bool event(QEvent *event);
 
 private:
     void setDirty(const QRect&);
     int viewdepth; // "faked" depth
+    PixelFormat viewFormat;
     int rsh;
     int gsh;
     int bsh;
@@ -147,4 +195,6 @@ private:
     Rotation rotation;
 };
 
+QT_END_NAMESPACE
+
 #endif
